SystemOrganization addCategory: #'EToys-Buttons'!
SystemOrganization addCategory: #'EToys-CustomEvents'!
SystemOrganization addCategory: #'EToys-Experimental'!
SystemOrganization addCategory: #'EToys-Protocols'!
SystemOrganization addCategory: #'EToys-Protocols-Type Vocabularies'!
SystemOrganization addCategory: #'EToys-Scripting'!
SystemOrganization addCategory: #'EToys-Scripting Support'!
SystemOrganization addCategory: #'EToys-Scripting Tiles'!
SystemOrganization addCategory: #'EToys-Stacks'!
SystemOrganization addCategory: #'EToys-StarSqueak'!
SystemOrganization addCategory: #'EToys-Tile Scriptors'!
SystemOrganization addCategory: #'EToys-Widgets'!

RectangleMorph subclass: #TileMorph
	instanceVariableNames: 'type slotName literal operatorOrExpression actualObject downArrow upArrow suffixArrow typeColor lastArrowTick nArrowTicks operatorReadoutString possessive retractArrow vocabulary vocabularySymbol'
	classVariableNames: 'DownPicture RetractPicture SuffixArrowAllowance SuffixPicture UpArrowAllowance UpPicture UpdatingOperators'
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!TileMorph commentStamp: '<historical>' prior: 0!
A tile with up, down and suffix arrows.

To install new Forms for the arrows, just nil out UpPicture, DownPicture,
or SuffixPicture.
Create actors with the picture you want and write it out with these file names:
'tile inc arrow.morph' 'tile dec arrow.morph' 'tile suffix
arrow.morph'.  Make sure that file is in the same directory as the image.
Open an EToy.!

TileMorph subclass: #AssignmentTileMorph
	instanceVariableNames: 'assignmentRoot assignmentSuffix dataType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!AssignmentTileMorph methodsFor: 'arrow' stamp: 'nk 10/8/2004 14:27'!
addArrowsIfAppropriate
	"If the receiver's slot is of an appropriate type, add arrows to the tile."

	(Vocabulary vocabularyForType: dataType)
		ifNotNilDo:
			[:aVocab | aVocab wantsAssignmentTileVariants ifTrue:
				[self addArrows]].
	(assignmentSuffix = ':') ifTrue:
		[ self addMorphBack: (ImageMorph new image: (ScriptingSystem formAtKey: #NewGets)).
		(self findA: StringMorph) ifNotNilDo: [ :sm |
			(sm contents endsWith: ' :') ifTrue: [ sm contents: (sm contents allButLast: 2) ]]]! !

!AssignmentTileMorph methodsFor: 'code generation' stamp: 'sw 2/6/2002 01:17'!
assignmentReceiverTile
	"Answer the TilePadMorph that should be sent storeCodeOn:indent: to get the receiver of the assignment properly stored on the code stream"

	^ owner submorphs first! !

!AssignmentTileMorph methodsFor: 'player viewer' stamp: 'yo 1/1/2004 19:51'!
assignmentRoot
	"Answer the assignment root"

	^ assignmentRoot! !

!AssignmentTileMorph methodsFor: 'initialization' stamp: 'yo 1/30/2005 11:11'!
computeOperatorOrExpression
	"Compute the operator or expression to use, and set the wording correectly on the tile face"

	| aSuffix wording anInterface getter doc |
	operatorOrExpression _ (assignmentRoot, assignmentSuffix) asSymbol.
	aSuffix _ self currentVocabulary translatedWordingFor:  assignmentSuffix.
	getter _ Utilities getterSelectorFor: assignmentRoot.
	anInterface _ self currentVocabulary methodInterfaceAt: getter ifAbsent: [Vocabulary eToyVocabulary methodInterfaceAt: getter ifAbsent: [nil]].
	wording _ anInterface ifNotNil: [anInterface wording] ifNil: [assignmentRoot copyWithout: $:].
	(anInterface notNil and: [(doc _ anInterface documentation) notNil])
		ifTrue:
			[self setBalloonText: doc].
	operatorReadoutString _ wording translated, ' ', aSuffix.
 	self line1: operatorReadoutString.
	self addArrowsIfAppropriate! !

!AssignmentTileMorph methodsFor: 'as yet unclassified' stamp: ''!
fixLayoutOfSubmorphsNotIn: aCollection 
	super fixLayoutOfSubmorphsNotIn: aCollection.
	self updateLiteralLabel; updateWordingToMatchVocabulary; layoutChanged; fullBounds! !

!AssignmentTileMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:44'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	type _ #operator.
	assignmentSuffix _ ':'! !

!AssignmentTileMorph methodsFor: 'code generation' stamp: 'sw 2/6/2002 01:25'!
operatorForAssignmentSuffix: aString
	"Answer the operator associated with the receiver, assumed to be one of the compound assignments"

	| toTest |
	toTest _ aString asString.
	#(	('Incr:'				'+')
		('Decr:'				'-')
		('Mult:'				'*'))
	do:
		[:pair | toTest = pair first ifTrue: [^ pair second]].
	^ toTest

	"AssignmentTileMorph new operatorForAssignmentSuffix: 'Incr:'"! !

!AssignmentTileMorph methodsFor: 'accessing' stamp: 'tak 12/5/2004 14:04'!
options
	^ {#(#: #Incr: #Decr: #Mult: ). {nil. nil. nil. nil}}! !

!AssignmentTileMorph methodsFor: 'initialization' stamp: 'sw 1/17/1999 21:10'!
setAssignmentSuffix: aString
	assignmentSuffix _ aString.
	self computeOperatorOrExpression.
	type _ #operator.
 	self line1: (ScriptingSystem wordingForOperator: operatorOrExpression).
	self addArrowsIfAppropriate; updateLiteralLabel! !

!AssignmentTileMorph methodsFor: 'initialization' stamp: 'yo 1/1/2004 19:50'!
setRoot: aString
	"Establish the assignment root, and update the label on the tile"

	assignmentRoot _ aString.
	self updateLiteralLabel! !

!AssignmentTileMorph methodsFor: 'initialization' stamp: 'sw 2/16/98 01:12'!
setRoot: aString dataType: aSymbol
	assignmentRoot _ aString.
	assignmentSuffix _ ':'.
	dataType _ aSymbol.
	self updateLiteralLabel! !

!AssignmentTileMorph methodsFor: 'code generation' stamp: 'aoy 2/15/2003 21:09'!
storeCodeOn: aStream indent: tabCount 
	"Generate code for an assignment statement.  The code generated looks presentable in the case of simple assignment, though the code generated for the increment/decrement/multiply cases is still the same old assignGetter... sort for now"
aStream nextPutAll: (Utilities setterSelectorFor: assignmentRoot).
			aStream space."Simple assignment, don't need existing value"
	assignmentSuffix = ':' 

		ifFalse: 
			["Assignments that require that old values be retrieved"

			
			self assignmentReceiverTile storeCodeOn: aStream indent: tabCount.
			aStream space.
			aStream nextPutAll: (Utilities getterSelectorFor: assignmentRoot).
			aStream space.
			aStream nextPutAll: (self operatorForAssignmentSuffix: assignmentSuffix).
			aStream space]! !

!AssignmentTileMorph methodsFor: 'player viewer' stamp: 'sw 1/31/98 00:42'!
updateLiteralLabel
	self computeOperatorOrExpression.
	super updateLiteralLabel! !

!AssignmentTileMorph methodsFor: 'initialization' stamp: 'sw 9/12/2001 22:52'!
updateWordingToMatchVocabulary
	"The current vocabulary has changed; change the wording on my face, if appropriate"

	self computeOperatorOrExpression! !

!AssignmentTileMorph methodsFor: 'accessing' stamp: 'tak 12/5/2004 14:09'!
value
	^ assignmentSuffix! !

!AssignmentTileMorph methodsFor: 'accessing' stamp: 'tak 12/5/2004 14:06'!
value: anObject 
	self setAssignmentSuffix: anObject.
	self acceptNewLiteral! !

AssignmentTileMorph subclass: #KedamaSetColorComponentTile
	instanceVariableNames: 'patchTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaSetColorComponentTile commentStamp: '<historical>' prior: 0!
I provide the red, green or blue component setter tile.
!

!KedamaSetColorComponentTile methodsFor: 'initalization support' stamp: 'yo 1/30/2005 11:20'!
addPatchTile

	| m1 desiredW m2 label |
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: (operatorOrExpression asString upTo: $:) translated font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Patch.
	m1 addMorph: m2.

	desiredW _ m1 width + 6.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).

	self addMorphBack: m1.
	self addMorphFront: label.
	patchTile _ m1.
! !

!KedamaSetColorComponentTile methodsFor: 'initalization support' stamp: 'yo 10/12/2004 22:38'!
componentName: aSymbol

	aSymbol = #blue ifTrue: [
		operatorOrExpression _ #setBlueComponentIn:to:.
	].
	aSymbol = #green ifTrue: [
		operatorOrExpression _ #setGreenComponentIn:to:.
	].
	aSymbol = #red ifTrue: [
		operatorOrExpression _ #setRedComponentIn:to:.
	].
	self addPatchTile.

! !

!KedamaSetColorComponentTile methodsFor: 'initialization' stamp: 'yo 10/12/2004 22:36'!
initialize

	super initialize.
	type _ #Patch.
	operatorOrExpression _ #setBlueComponentIn:to:.
	self addPatchTile.
	self line1: 'setBlueComponent:'.
! !

!KedamaSetColorComponentTile methodsFor: 'initialization' stamp: 'yo 1/30/2005 11:20'!
line1: line1

	| chopped label label2 tail headIndex |
	(line1 select: [:e | e = $:]) size >= 2 ifTrue: [^ self].
	self removeAllMorphs.
	headIndex _ line1 indexOf: $: ifAbsent: [line1 indexOf: Character space ifAbsent: [0]].

	chopped _ line1 copyFrom: 1 to: headIndex - 1.
	tail _ line1 copyFrom: chopped size + 2 to: line1 size.
	label _ 	StringMorph contents: (chopped) translated font: ScriptingSystem fontForTiles.
	label2 _ StringMorph contents: tail translated font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: patchTile.
	self addMorphBack: label2.
! !

!KedamaSetColorComponentTile methodsFor: 'tile protocol' stamp: 'yo 10/12/2004 22:39'!
setAssignmentSuffix: aString
	assignmentSuffix _ aString.
	self computeOperatorOrExpression.
	type _ #operator.
 	self addArrowsIfAppropriate; updateLiteralLabel.
! !

!KedamaSetColorComponentTile methodsFor: 'initalization support' stamp: 'yo 10/12/2004 22:39'!
setPatchDefaultTo: aPlayer

	patchTile submorphs first setToReferTo: aPlayer.
! !

!KedamaSetColorComponentTile methodsFor: 'initialization' stamp: 'yo 10/12/2004 23:33'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| firstKeyword |
	(#('redComponentIn:' 'setRedComponentIn:') includes: assignmentRoot) ifTrue: [
		firstKeyword _ 'setRedComponentIn'.
	].
	(#('greenComponentIn:' 'setGreenComponentIn:') includes: assignmentRoot) ifTrue: [
		firstKeyword _ 'setGreenComponentIn'
	].
	(#('blueComponentIn:' 'setBlueComponentIn:') includes: assignmentRoot) ifTrue: [
		firstKeyword _ 'setBlueComponentIn'
	].

	aStream nextPutAll: firstKeyword.
	aStream nextPut: $:.
			aStream space."Simple assignment, don't need existing value"
	patchTile submorphs first storeCodeOn: aStream indent: tabCount.
	aStream nextPutAll: ' to: '.

	assignmentSuffix = ':' 
		ifFalse: 
			["Assignments that require that old values be retrieved"

			aStream nextPutAll: '( '.
			self assignmentReceiverTile storeCodeOn: aStream indent: tabCount.
			aStream space.
			aStream nextPutAll: 'getPatchValueIn:'.
			patchTile submorphs first storeCodeOn: aStream indent: tabCount.
			aStream nextPutAll: ')'.
			aStream space.
			aStream nextPutAll: (self operatorForAssignmentSuffix: assignmentSuffix).
			aStream space]! !

!KedamaSetColorComponentTile methodsFor: 'initalization support' stamp: 'yo 10/12/2004 22:40'!
updateLiteralLabel
	self computeOperatorOrExpression.
	"super basicUpdateLiteralLabel"
! !

AssignmentTileMorph subclass: #KedamaSetPixelValueTile
	instanceVariableNames: 'patchTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaSetPixelValueTile commentStamp: 'yo 6/18/2004 18:32' prior: 0!
I provide the special tile of 'patchValueIn' setter.
!

!KedamaSetPixelValueTile methodsFor: 'initialization-support' stamp: 'yo 10/9/2004 17:59'!
addPatchTile

	| m1 desiredW m2 label |
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: 'setValueIn' font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Patch.
	m1 addMorph: m2.

	desiredW _ m1 width + 6.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).

	self addMorphBack: m1.
	self addMorphFront: label.
	patchTile _ m1.
! !

!KedamaSetPixelValueTile methodsFor: 'initialization' stamp: 'yo 10/8/2004 09:25'!
initialize

	super initialize.
	type _ #Patch.
	operatorOrExpression _ #setPatchValueIn:to:.
	self addPatchTile.
	self line1: 'setPatchValue:'.
! !

!KedamaSetPixelValueTile methodsFor: 'tile protocol' stamp: 'yo 10/8/2004 09:28'!
line1: line1

	| chopped label label2 tail headIndex |
	(line1 select: [:e | e = $:]) size >= 2 ifTrue: [^ self].
	self removeAllMorphs.
	headIndex _ line1 indexOf: $: ifAbsent: [line1 indexOf: Character space ifAbsent: [0]].

	chopped _ line1 copyFrom: 1 to: headIndex - 1.
	tail _ line1 copyFrom: chopped size + 2 to: line1 size.
	label _ 	StringMorph contents: (chopped) font: ScriptingSystem fontForTiles.
	label2 _ StringMorph contents: tail font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: patchTile.
	self addMorphBack: label2.
! !

!KedamaSetPixelValueTile methodsFor: 'tile protocol' stamp: 'yo 6/7/2004 21:01'!
setAssignmentSuffix: aString
	assignmentSuffix _ aString.
	self computeOperatorOrExpression.
	type _ #operator.
 	self addArrowsIfAppropriate; updateLiteralLabel! !

!KedamaSetPixelValueTile methodsFor: 'initialization-support' stamp: 'yo 6/7/2004 20:19'!
setPatchDefaultTo: aPlayer

	patchTile submorphs first setToReferTo: aPlayer.
! !

!KedamaSetPixelValueTile methodsFor: 'tile protocol' stamp: 'yo 6/7/2004 21:15'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

"	| parts |

	parts := operatorOrExpression keywords.
	aStream nextPutAll: parts first.
	aStream space.
	patchTile submorphs first storeCodeOn: aStream indent: tabCount.
	aStream space.
	aStream nextPutAll: ' to: '.
"

	aStream nextPutAll: 'setPatchValueIn:'.
			aStream space."Simple assignment, don't need existing value"
	patchTile submorphs first storeCodeOn: aStream indent: tabCount.
	aStream nextPutAll: ' to: '.

	assignmentSuffix = ':' 
		ifFalse: 
			["Assignments that require that old values be retrieved"

			aStream nextPutAll: '( '.
			self assignmentReceiverTile storeCodeOn: aStream indent: tabCount.
			aStream space.
			aStream nextPutAll: 'getPatchValueIn:'.
			patchTile submorphs first storeCodeOn: aStream indent: tabCount.
			aStream nextPutAll: ')'.
			aStream space.
			aStream nextPutAll: (self operatorForAssignmentSuffix: assignmentSuffix).
			aStream space]! !

!KedamaSetPixelValueTile methodsFor: 'tile protocol' stamp: 'yo 6/9/2004 12:56'!
updateLiteralLabel
	self computeOperatorOrExpression.
	"super basicUpdateLiteralLabel"
! !

TileMorph subclass: #BooleanTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!BooleanTile commentStamp: '<historical>' prior: 0!
A tile whose result type is boolean.!

!BooleanTile methodsFor: 'accessing' stamp: 'sw 9/27/2001 17:19'!
resultType
	"Answer the result type of the receiver"

	^ #Boolean! !

TileMorph subclass: #ColorTileMorph
	instanceVariableNames: 'colorSwatch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

ColorTileMorph subclass: #ColorSeerTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!ColorSeerTile methodsFor: 'initialization' stamp: 'mir 7/12/2004 20:23'!
initialize
"initialize the state of the receiver"
	| m1 m2 desiredW wording |
	super initialize.
""
	self removeAllMorphs.
	"get rid of the parts of a regular Color tile"
	type _ #operator.
	operatorOrExpression _ #color:sees:.
	wording _ (Vocabulary eToyVocabulary
				methodInterfaceAt: operatorOrExpression
				ifAbsent: []) wording.
	m1 _ StringMorph contents: wording font: ScriptingSystem fontForTiles.
	m2 _ Morph new extent: 12 @ 8;
				
				color: (Color
						r: 0.8
						g: 0
						b: 0).
	desiredW _ m1 width + 6.
	self extent: (desiredW max: self basicWidth)
			@ self class defaultH.
	m1 position: bounds center x - (m1 width // 2) @ (bounds top + 5).
	m2 position: bounds center x - (m2 width // 2) + 3 @ (bounds top + 8).
	self addMorph: m1;
		 addMorphFront: m2.
	colorSwatch _ m2! !

!ColorSeerTile methodsFor: 'code generation' stamp: 'dgd 2/22/2003 14:25'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| parts |
	parts := operatorOrExpression keywords.	"color:sees:"
	^aStream
		nextPutAll: (parts first);
		space;
		nextPutAll: colorSwatch color printString;
		space;
		nextPutAll: (parts second)! !

!ColorSeerTile methodsFor: 'initialization' stamp: 'mir 7/15/2004 15:20'!
updateWordingToMatchVocabulary
	"The current vocabulary has changed; change the wording on my face, if appropriate"

	| aMethodInterface |
	aMethodInterface _ self currentVocabulary methodInterfaceAt: operatorOrExpression ifAbsent: [Vocabulary eToyVocabulary methodInterfaceAt: operatorOrExpression ifAbsent: [^ self]].
	self labelMorph contents: aMethodInterface wording.
	self setBalloonText: aMethodInterface helpMessage.! !

!ColorTileMorph methodsFor: 'other' stamp: 'yo 7/2/2004 17:33'!
addColorSwatch

	| m1 m2 desiredW |
	m1 _ StringMorph contents: 'color' translated font: ScriptingSystem fontForTiles.
	m2 _ Morph new extent: 12@8; color: (Color r: 0.8 g: 0 b: 0).
	desiredW _ m1 width + 6.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	m2 position: (bounds center x - (m2 width // 2)) @ (m1 bottom - 1).
	self addMorph: m1; addMorph: m2.
	colorSwatch _ m2! !

!ColorTileMorph methodsFor: 'other' stamp: 'sw 10/25/1998 00:25'!
colorSwatch
	^ colorSwatch! !

!ColorTileMorph methodsFor: 'other' stamp: 'yo 3/24/2005 12:16'!
colorSwatchColor: aColor
	colorSwatch color: aColor.
! !

!ColorTileMorph methodsFor: 'event handling' stamp: ''!
handlesMouseDown: evt

	(colorSwatch containsPoint: evt cursorPoint)
		ifTrue: [^ true]
		ifFalse: [^ super handlesMouseDown: evt].
! !

!ColorTileMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:44'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	type _ #literal.
	self addColorSwatch! !

!ColorTileMorph methodsFor: 'other' stamp: 'yo 3/24/2005 12:17'!
kedamaColorSwatchColor: aColor

	colorSwatch userSelectedColor: aColor.
! !

!ColorTileMorph methodsFor: 'code generation' stamp: 'yo 4/5/2005 11:19'!
kedamaStoreCodeAsPixelValueOn: aStream indent: tabCount

	aStream nextPutAll: ((colorSwatch color pixelValueForDepth: 32) bitAnd: 16rFFFFFF) printString.
! !

!ColorTileMorph methodsFor: 'event handling' stamp: ''!
mouseDown: evt

	(colorSwatch containsPoint: evt cursorPoint)
		ifFalse: [super mouseDown: evt].
! !

!ColorTileMorph methodsFor: 'event handling' stamp: 'yo 9/29/2004 10:50'!
mouseUp: evt
	self changeColorTarget: self selector: #kedamaColorSwatchColor: originalColor: colorSwatch color hand: evt hand
! !

!ColorTileMorph methodsFor: 'accessing' stamp: 'sw 9/27/2001 17:27'!
resultType
	"Answer the result type of the receiver"

	^ #Color! !

!ColorTileMorph methodsFor: 'initialization' stamp: 'sw 10/13/2004 23:28'!
setLiteral: aLiteral
	"Set the receiver's literal"

	self colorSwatchColor: aLiteral! !

!ColorTileMorph methodsFor: 'code generation' stamp: 'jm 5/28/1998 19:02'!
storeCodeOn: aStream indent: tabCount

	aStream nextPutAll: colorSwatch color printString.
! !

!ColorTileMorph methodsFor: 'player viewer' stamp: 'sw 1/6/1999 10:43'!
updateLiteralLabel
	"Do nothing"! !

!ColorTileMorph methodsFor: 'initialization' stamp: 'yo 7/2/2004 17:33'!
updateWordingToMatchVocabulary

	| stringMorph |
	stringMorph _ submorphs detect: [:morph | morph class == StringMorph] ifNone: [^ self].
	stringMorph contents: 'color' translated.
! !

TileMorph subclass: #GraphicTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!GraphicTile commentStamp: '<historical>' prior: 0!
A tile representing a graphic image.!

!GraphicTile methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:45'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	type _ #literal.
	self
		useForm: (ScriptingSystem formAtKey: #Menu)! !

!GraphicTile methodsFor: 'accessing' stamp: 'sw 9/26/2001 04:05'!
resultType
	"Answer the result type of the argument represented by the receiver"

	^ #Graphic! !

!GraphicTile methodsFor: 'initialization' stamp: 'sw 4/3/2001 15:52'!
setLiteral: anObject
	"Set the receiver's literal to be anObject. No readout morph here."

	type _ #literal.
	self setLiteralInitially: anObject.
! !

!GraphicTile methodsFor: 'code generation' stamp: 'sw 4/2/2001 23:09'!
storeCodeOn: aStream indent: tabCount
	"Write code that will reconstitute the receiver"

	aStream nextPutAll: literal uniqueNameForReference! !

!GraphicTile methodsFor: 'initialization' stamp: 'sw 4/3/2001 15:40'!
useForm: aForm
	"Set the receiver to represent the given form"

	| thumbnail |
	self removeAllMorphs.
	literal _ aForm.
	thumbnail _ ThumbnailMorph  new objectToView: self viewSelector: #literal.
	self addMorphBack: thumbnail.
	thumbnail extent: 16 @ 16.! !

TileMorph subclass: #KedamaAngleToTile
	instanceVariableNames: 'turtleTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaAngleToTile commentStamp: 'yo 6/18/2004 18:29' prior: 0!
I provide the special tile of 'angle to'.
!

!KedamaAngleToTile methodsFor: 'initialization support' stamp: 'yo 1/30/2005 10:57'!
addTile

	| m1 desiredW m2 label |
	self removeAllMorphs.
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: 'thetaTo' translated font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Player.
	m1 addMorph: m2.
	desiredW _ m1 width.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	self addMorphBack: m1.
	self addMorphFront: label.
	turtleTile _ m1.
! !

!KedamaAngleToTile methodsFor: 'initialization' stamp: 'yo 6/17/2004 10:03'!
initialize

	super initialize.
	type _ #Player.
	operatorOrExpression _ #getAngleTo:.
	self addTile.
! !

!KedamaAngleToTile methodsFor: 'tile protocol' stamp: 'yo 1/30/2005 11:19'!
line1: line1

	| label |
	self removeAllMorphs.

	label _ 	StringMorph contents: 'thetaTo' translated font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: turtleTile.
! !

!KedamaAngleToTile methodsFor: 'initialization support' stamp: 'yo 6/15/2004 18:43'!
setTurtleDefaultTo: aPlayer

	turtleTile submorphs first setToReferTo: aPlayer.
! !

!KedamaAngleToTile methodsFor: 'tile protocol' stamp: 'yo 10/9/2004 10:01'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| parts |
	parts := operatorOrExpression keywords.	"thetaTo"
	aStream nextPutAll: parts first.
	aStream space.
	turtleTile storeCodeOn: aStream indent: tabCount.
! !

TileMorph subclass: #KedamaBounceOnColorTile
	instanceVariableNames: 'playerTile colorTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaBounceOnColorTile methodsFor: 'initalization support' stamp: 'yo 1/30/2005 10:56'!
addPlayerTile

	| m1 desiredW m2 label |
	self removeAllMorphs.
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: 'bounce on' translated font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Player.
	m1 addMorph: m2.
	desiredW _ m1 width + 6.

	m2 _ ColorTileMorph new.

	"m3 _ Morph new extent: 12@8; color: (Color r: 0.8 g: 0.0 b: 0.0)."
	desiredW _ desiredW + m2 width.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	"m2 addMorph: m3."
	m2 position: ((bounds center x - (m2 width // 2)) + 3)@ (bounds top + 1).
	self addMorphBack: m2.
	self addMorphFront: m1.
	self addMorphFront: label.
	playerTile _ m1.
	colorTile _ m2.
! !

!KedamaBounceOnColorTile methodsFor: 'initialization' stamp: 'yo 9/28/2004 18:43'!
initialize

	super initialize.
	type _ #Player.
	operatorOrExpression _ #bounceOn:color:.
	self addPlayerTile.
! !

!KedamaBounceOnColorTile methodsFor: 'tile protocol' stamp: 'yo 1/30/2005 10:56'!
line1: line1

	| label |
	self removeAllMorphs.

	label _ 	StringMorph contents: 'bounce on' translated font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: playerTile.
! !

!KedamaBounceOnColorTile methodsFor: 'tile protocol' stamp: 'yo 10/8/2004 11:42'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	aStream nextPutAll: 'bounceOn:'.
	aStream space.
	playerTile storeCodeOn: aStream indent: tabCount.
	aStream space.
	aStream nextPutAll: 'color:'.
	aStream space.
	colorTile kedamaStoreCodeAsPixelValueOn: aStream indent: tabCount.

	"aStream nextPutAll: submorphs third color printString."
! !

!KedamaBounceOnColorTile methodsFor: 'initalization support' stamp: 'yo 9/28/2004 18:23'!
updateLiteralLabel
! !

TileMorph subclass: #KedamaBounceOnTile
	instanceVariableNames: 'playerTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaBounceOnTile methodsFor: 'initialization support' stamp: 'yo 1/30/2005 10:58'!
addPlayerTile

	| m1 desiredW m2 label |
	self removeAllMorphs.
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: 'bounce on' translated font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Player.
	m1 addMorph: m2.
	desiredW _ m1 width + 6.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	self addMorphBack: m1.
	self addMorphFront: label.
	playerTile _ m1.
! !

!KedamaBounceOnTile methodsFor: 'initialization' stamp: 'yo 9/27/2004 14:53'!
initialize

	super initialize.
	type _ #Player.
	operatorOrExpression _ #bounceOn:.
	self addPlayerTile.
! !

!KedamaBounceOnTile methodsFor: 'tile protocol' stamp: 'yo 1/30/2005 10:56'!
line1: line1

	| label |
	self removeAllMorphs.

	label _ 	StringMorph contents: 'bounce on' translated font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: playerTile.
! !

!KedamaBounceOnTile methodsFor: 'tile protocol' stamp: 'yo 10/9/2004 10:02'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| parts |
	parts := operatorOrExpression keywords.	"bounceOn:"
	aStream nextPutAll: parts first.
	aStream space.
	playerTile storeCodeOn: aStream indent: tabCount.
! !

!KedamaBounceOnTile methodsFor: 'tile protocol' stamp: 'yo 9/27/2004 14:54'!
updateLiteralLabel
! !

TileMorph subclass: #KedamaDistanceToTile
	instanceVariableNames: 'turtleTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaDistanceToTile commentStamp: 'yo 6/18/2004 18:30' prior: 0!
I provide the special tile of 'distance to'.
!

!KedamaDistanceToTile methodsFor: 'initalization support' stamp: 'yo 1/30/2005 10:59'!
addTile

	| m1 desiredW m2 label |
	self removeAllMorphs.
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: 'distanceTo' translated font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Player.
	m1 addMorph: m2.
	desiredW _ m1 width.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	self addMorphBack: m1.
	self addMorphFront: label.
	turtleTile _ m1.
! !

!KedamaDistanceToTile methodsFor: 'initialization' stamp: 'yo 6/14/2004 15:58'!
initialize

	super initialize.
	type _ #Player.
	operatorOrExpression _ #getDistanceTo:.
	self addTile.
! !

!KedamaDistanceToTile methodsFor: 'tile protocol' stamp: 'yo 1/30/2005 10:59'!
line1: line1

	| label |
	self removeAllMorphs.

	label _ 	StringMorph contents: 'distanceTo' translated font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: turtleTile.
! !

!KedamaDistanceToTile methodsFor: 'initalization support' stamp: 'yo 6/15/2004 18:43'!
setTurtleDefaultTo: aPlayer

	turtleTile submorphs first setToReferTo: aPlayer.
! !

!KedamaDistanceToTile methodsFor: 'tile protocol' stamp: 'yo 10/9/2004 10:00'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| parts |
	parts := operatorOrExpression keywords.	"distanceTo:"
	aStream nextPutAll: parts first.
	aStream space.
	turtleTile storeCodeOn: aStream indent: tabCount.
! !

TileMorph subclass: #KedamaGetColorComponentTile
	instanceVariableNames: 'patchTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaGetColorComponentTile commentStamp: '<historical>' prior: 0!
I provide the red, green or blue component getter tile.
!

!KedamaGetColorComponentTile methodsFor: 'initalization support' stamp: 'yo 10/12/2004 22:32'!
addPatchTile

	| m1 desiredW m2 label |
	self removeAllMorphs.
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: (operatorOrExpression asString copyWithout: $:) font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Patch.
	m1 addMorph: m2.
	desiredW _ m1 width.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	self addMorphBack: m1.
	self addMorphFront: label.
	patchTile _ m1.
! !

!KedamaGetColorComponentTile methodsFor: 'initalization support' stamp: 'yo 10/12/2004 22:31'!
componentName: aSymbol

	aSymbol = #blue ifTrue: [
		operatorOrExpression _ #getBlueComponentIn:.
	].
	aSymbol = #green ifTrue: [
		operatorOrExpression _ #getGreenComponentIn:.
	].
	aSymbol = #red ifTrue: [
		operatorOrExpression _ #getRedComponentIn:.
	].
	self addPatchTile.

! !

!KedamaGetColorComponentTile methodsFor: 'initialization' stamp: 'yo 10/12/2004 22:29'!
initialize

	super initialize.
	type _ #Patch.
	operatorOrExpression _ #getBlueComponentIn:.
	self addPatchTile.
	self line1: 'getBlueComponentIn:'.
! !

!KedamaGetColorComponentTile methodsFor: 'tile protocol' stamp: 'yo 10/12/2004 22:32'!
line1: line1

	| label |
	self removeAllMorphs.

	label _ 	StringMorph contents: (operatorOrExpression asString copyWithout: $:) font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: patchTile.
! !

!KedamaGetColorComponentTile methodsFor: 'initalization support' stamp: 'yo 10/12/2004 22:32'!
setPatchDefaultTo: aPlayer

	patchTile submorphs first setToReferTo: aPlayer.
! !

!KedamaGetColorComponentTile methodsFor: 'tile protocol' stamp: 'yo 10/12/2004 22:33'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| parts |
	parts := operatorOrExpression keywords.	"getPatchValueIn:"
	aStream nextPutAll: parts first.
	aStream space.
	patchTile storeCodeOn: aStream indent: tabCount.
! !

TileMorph subclass: #KedamaGetPixelValueTile
	instanceVariableNames: 'patchTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaGetPixelValueTile commentStamp: '<historical>' prior: 0!
I provide the patchValueIn getter tile.
!

!KedamaGetPixelValueTile methodsFor: 'initialization support' stamp: 'yo 1/30/2005 11:03'!
addPatchTile

	| m1 desiredW m2 label |
	self removeAllMorphs.
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: 'getValueIn' translated font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Patch.
	m1 addMorph: m2.
	desiredW _ m1 width.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	self addMorphBack: m1.
	self addMorphFront: label.
	patchTile _ m1.
! !

!KedamaGetPixelValueTile methodsFor: 'initialization' stamp: 'yo 10/8/2004 07:45'!
initialize

	super initialize.
	type _ #Patch.
	operatorOrExpression _ #getPatchValueIn:.
	self addPatchTile.
	self line1: 'getPatchValueIn:'.
! !

!KedamaGetPixelValueTile methodsFor: 'initialization support' stamp: 'yo 1/30/2005 11:04'!
line1: line1

	| label |
	self removeAllMorphs.

	label _ 	StringMorph contents: 'patchValueIn' translated font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: patchTile.
! !

!KedamaGetPixelValueTile methodsFor: 'tile protocol' stamp: 'yo 6/7/2004 21:19'!
setPatchDefaultTo: aPlayer

	patchTile submorphs first setToReferTo: aPlayer.
! !

!KedamaGetPixelValueTile methodsFor: 'tile protocol' stamp: 'yo 10/7/2004 21:58'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| parts |
	parts := operatorOrExpression keywords.	"getPatchValueIn:"
	aStream nextPutAll: parts first.
	aStream space.
	patchTile storeCodeOn: aStream indent: tabCount.
! !

TileMorph subclass: #KedamaPatchTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaPatchTile commentStamp: 'yo 6/18/2004 18:32' prior: 0!
A special Tile that represents the Patch.
!

!KedamaPatchTile methodsFor: 'initialization' stamp: 'yo 6/7/2004 13:56'!
initialize

	super initialize.
	type _ #literal.
	self extent: 16@16.
! !

!KedamaPatchTile methodsFor: 'tile protocol' stamp: 'yo 6/5/2004 16:12'!
resultType
	"Answer the result type of the argument represented by the receiver"

	^ #Patch.
! !

!KedamaPatchTile methodsFor: 'tile protocol' stamp: 'yo 6/5/2004 16:11'!
setLiteral: anObject
	"Set the receiver's literal to be anObject. No readout morph here."

	type _ #literal.
	self setLiteralInitially: anObject.
! !

!KedamaPatchTile methodsFor: 'tile protocol' stamp: 'yo 6/5/2004 16:11'!
storeCodeOn: aStream indent: tabCount
	"Write code that will reconstitute the receiver"

	aStream nextPutAll: literal uniqueNameForReference! !

!KedamaPatchTile methodsFor: 'initialization support' stamp: 'yo 6/18/2004 16:02'!
usePatch: aPatch

	| aTile displayer |
	self removeAllMorphs.
	literal _ aPatch.


	aTile _ KedamaPatchType basicNew newReadoutTile.

	displayer _ UpdatingStringMorph new
		getSelector: #externalName;
		target: aPatch;
		growable: true;
		minimumWidth: 24;
		putSelector: nil.
	displayer stepTime: 1000.
	"Note that when typeSymbol = #number, the #target: call above will have dealt with floatPrecision details"

	displayer useStringFormat.
	aTile addMorphBack: displayer.
	aTile setLiteralInitially: (aPatch perform: #externalName).
	self addMorphBack: aTile.
! !

TileMorph subclass: #KedamaTurtleAtTile
	instanceVariableNames: 'turtleTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaTurtleAtTile methodsFor: 'initialization-support' stamp: 'yo 10/11/2004 15:49'!
addTile

	| m1 desiredW m2 label |
	self removeAllMorphs.
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: 'turtle at' font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Player.
	m1 addMorph: m2.
	desiredW _ m1 width.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	self addMorphBack: m1.
	self addMorphFront: label.
	turtleTile _ m1.
! !

!KedamaTurtleAtTile methodsFor: 'initialization' stamp: 'yo 10/9/2004 10:35'!
initialize

	super initialize.
	type _ #Player.
	operatorOrExpression _ #getTurtleAt:.
	self addTile.
! !

!KedamaTurtleAtTile methodsFor: 'tile protocol' stamp: 'yo 10/9/2004 10:35'!
line1: line1

	| label |
	self removeAllMorphs.

	label _ 	StringMorph contents: 'turtleAt' font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: turtleTile.
! !

!KedamaTurtleAtTile methodsFor: 'initialization-support' stamp: 'yo 10/9/2004 10:35'!
setTurtleDefaultTo: aPlayer

	turtleTile submorphs first setToReferTo: aPlayer.
! !

!KedamaTurtleAtTile methodsFor: 'tile protocol' stamp: 'yo 10/9/2004 15:37'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| parts |
	parts := operatorOrExpression keywords.	"turtleAt:"
	aStream nextPutAll: parts first.
	aStream space.
	turtleTile storeCodeOn: aStream indent: tabCount.
! !

TileMorph subclass: #KedamaTurtleOfTile
	instanceVariableNames: 'turtleTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaTurtleOfTile methodsFor: 'initalization support' stamp: 'yo 3/7/2005 08:38'!
addTile

	| m1 desiredW m2 label |
	self removeAllMorphs.
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: 'turtle of' font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Player.
	m1 addMorph: m2.
	desiredW _ m1 width.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	self addMorphBack: m1.
	self addMorphFront: label.
	turtleTile _ m1.
! !

!KedamaTurtleOfTile methodsFor: 'initialization' stamp: 'yo 3/7/2005 08:36'!
initialize

	super initialize.
	type _ #Player.
	operatorOrExpression _ #getTurtleOf:.
	self addTile.
! !

!KedamaTurtleOfTile methodsFor: 'tile protocol' stamp: 'yo 3/7/2005 08:38'!
line1: line1

	| label |
	self removeAllMorphs.

	label _ 	StringMorph contents: 'turtleOf' font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: turtleTile.
! !

!KedamaTurtleOfTile methodsFor: 'initalization support' stamp: 'yo 3/7/2005 08:38'!
setTurtleDefaultTo: aPlayer

	turtleTile submorphs first setToReferTo: aPlayer.
! !

!KedamaTurtleOfTile methodsFor: 'tile protocol' stamp: 'yo 3/7/2005 08:38'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| parts |
	parts := operatorOrExpression keywords.	"getTurtleOf:"
	aStream nextPutAll: parts first.
	aStream space.
	turtleTile storeCodeOn: aStream indent: tabCount.
! !

TileMorph subclass: #KedamaUpHillTile
	instanceVariableNames: 'patchTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaUpHillTile commentStamp: 'yo 6/18/2004 18:34' prior: 0!
I provide the special tile of 'upHill' setter.
!

!KedamaUpHillTile methodsFor: 'initialization-support' stamp: 'yo 4/5/2005 14:53'!
addPatchTile

	| m1 desiredW m2 label |
	self removeAllMorphs.
	m1 _ TilePadMorph new.
	label _ 	StringMorph contents: 'upHillIn' translated font: ScriptingSystem fontForTiles.

	m2 _ TileMorph new.
	m2 extent: 20@22.
	m2 minWidth: 20.
	m1 extent: (m2 extent + (2@2)).
	m1 setType: #Patch.
	m1 addMorph: m2.
	desiredW _ m1 width + 6.
	self extent: (desiredW max: self basicWidth) @ self class defaultH.
	m1 position: (bounds center x - (m1 width // 2)) @ (bounds top + 1).
	self addMorphBack: m1.
	self addMorphFront: label.
	patchTile _ m1.
! !

!KedamaUpHillTile methodsFor: 'initialization' stamp: 'yo 6/7/2004 18:34'!
initialize

	super initialize.
	type _ #Patch.
	operatorOrExpression _ #getUphillIn:.
	self addPatchTile.
! !

!KedamaUpHillTile methodsFor: 'tile protocol' stamp: 'yo 4/5/2005 14:53'!
line1: line1

	| label |
	self removeAllMorphs.

	label _ 	StringMorph contents: 'upHillIn' translated font: ScriptingSystem fontForTiles.

	self addMorphBack: label.
	self addMorphBack: patchTile.
! !

!KedamaUpHillTile methodsFor: 'initialization-support' stamp: 'yo 6/7/2004 21:56'!
setPatchDefaultTo: aPlayer

	patchTile submorphs first setToReferTo: aPlayer.
! !

!KedamaUpHillTile methodsFor: 'tile protocol' stamp: 'yo 10/9/2004 10:03'!
storeCodeOn: aStream indent: tabCount 
	"We have a hidden arg. Output two keywords with interspersed arguments."

	| parts |
	parts := operatorOrExpression keywords.	"getUphillIn:"
	aStream nextPutAll: parts first.
	aStream space.
	patchTile storeCodeOn: aStream indent: tabCount.
! !

!KedamaUpHillTile methodsFor: 'tile protocol' stamp: 'yo 6/18/2004 18:44'!
updateLiteralLabel
! !

TileMorph subclass: #MenuTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!MenuTile commentStamp: '<historical>' prior: 0!
A tile representing a menu item!

!MenuTile methodsFor: 'arrows' stamp: 'sw 11/22/1999 11:54'!
arrowAction: delta
	| phrase aPlayer newItem |
	(phrase _ self ownerThatIsA: PhraseTileMorph) ifNil: [^ self].
	aPlayer _ phrase associatedPlayer.
	newItem _ delta > 0
		ifTrue:
			[aPlayer menuItemAfter: literal]
		ifFalse:
			[aPlayer menuItemBefore: literal].
	self literal: newItem.
	self layoutChanged! !

!MenuTile methodsFor: 'events-processing' stamp: 'ar 9/12/2000 23:05'!
handlerForMouseDown: anEvent
	"Don't give anybody over me a chance"
	^self! !

!MenuTile methodsFor: 'event handling' stamp: 'sw 11/22/1999 12:15'!
handlesMouseDown: evt
	^ true! !

!MenuTile methodsFor: 'initialization' stamp: 'dgd 9/6/2003 17:36'!
initialize
	"Initialize the menu tile"

	super initialize.
	self addArrows; setLiteral: 'send to back' translated.
	self labelMorph useStringFormat; putSelector: nil! !

!MenuTile methodsFor: 'event handling' stamp: 'sw 10/3/2002 21:16'!
mouseDown: evt
	| aPoint aMenu reply |
	aPoint _ evt cursorPoint.
	nArrowTicks _ 0.
	((upArrow bounds containsPoint: aPoint) or: [downArrow bounds containsPoint: aPoint]) ifTrue: [^ self mouseStillDown: evt].
	aMenu _ SelectionMenu selections: (((self ownerThatIsA: PhraseTileMorph) associatedPlayer costume allMenuWordings) copyWithout: '').
	reply _ aMenu startUp.
	reply ifNotNil: [self literal: reply; layoutChanged]! !

!MenuTile methodsFor: 'events-processing' stamp: 'ar 10/9/2000 21:16'!
mouseDownPriority
	^100! !

!MenuTile methodsFor: 'accessing' stamp: 'sw 9/27/2001 17:28'!
resultType
	"Answer the result type of the receiver"

	^ #Menu! !

TileMorph subclass: #NumericReadoutTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!NumericReadoutTile classSide methodsFor: 'scripting' stamp: 'tk 12/14/2001 19:30'!
authoringPrototype
	"Enclose my prototype in a SyntaxMorph."

	| aWatcher aTile aLine aColor ms slotMsg |

	aColor _ Color r: 0.387 g: 0.581 b: 1.0.
	aTile _ self new typeColor: aColor.
	aWatcher _ UpdatingStringMorph new.
	aWatcher growable: true;
		setToAllowTextEdit;
		getSelector: nil;
		putSelector: nil.
	aWatcher target: nil.
	aTile addMorphBack: aWatcher.
	aTile addArrows.
	aTile setLiteralTo: 5 width: 30.

	"This is the long way around to do this..."
	ms _ MessageSend receiver: nil selector: #aNumber arguments: #().
	slotMsg _ ms asTilesIn: Player globalNames: false.
		"For CardPlayers, use 'aPlayer'.  For others, name it, and use its name."
	ms _ MessageSend receiver: 3 selector: #= asSymbol arguments: #(5).
	aLine _ ms asTilesIn: Player globalNames: false.
	aLine firstSubmorph delete.	
	aLine addMorphFront: (slotMsg submorphs second) firstSubmorph.
	aLine firstSubmorph setNameTo: 'label'.
	aLine addMorphFront: (Morph new transparentSpacerOfSize: 3@3).
	aLine lastSubmorph delete.
	aLine lastSubmorph delete.
	aLine color: aColor; setNameTo: 'Number (fancy)'.
	aLine addMorphBack: (Morph new transparentSpacerOfSize: 3@3).
	aLine addMorphBack: aTile.
	aLine readOut setNameTo: 'value'.
	aLine cellPositioning: #leftCenter.
	aWatcher step; fitContents.
	^ aLine markAsPartsDonor.! !

!NumericReadoutTile classSide methodsFor: 'instance creation' stamp: 'tk 12/14/2001 19:32'!
borderedPrototype
	"Just number and up/down arrows"

	| aWatcher aTile |

	aTile _ self new typeColor: (Color r: 0.387 g: 0.581 b: 1.0).
	aWatcher _ UpdatingStringMorph new.
	aWatcher growable: true; setNameTo: 'value'.
	aTile addMorphBack: aWatcher.
	aTile addArrows; setNameTo: 'Number (mid)'.
	aTile setLiteralTo: 5 width: 30.
	aWatcher step; fitContents; setToAllowTextEdit.
	^ aTile extent: 30@24; markAsPartsDonor! !

!NumericReadoutTile classSide methodsFor: 'instance creation' stamp: 'tk 12/14/2001 19:29'!
simplePrototype
	"Bare number readout.  Will keep up to data with a number once it has target, getterSelector, setterSelector."

	^ (UpdatingStringMorph new) contents: '5'; growable: true; setToAllowTextEdit; 
		step; fitContents; setNameTo: 'Number (bare)'; markAsPartsDonor! !

!NumericReadoutTile classSide methodsFor: 'instance creation' stamp: 'nk 8/23/2004 18:11'!
supplementaryPartsDescriptions
	"Answer additional items for the parts bin"

	Preferences universalTiles ifFalse: [^ #()].

	^ {DescriptionForPartsBin
		formalName: 'Number (fancy)'
		categoryList: #('Basic')
		documentation: 'A number readout for a Stack.  Shows current value.  Click and type the value.  Shift-click on title to edit.'
		globalReceiverSymbol: #NumericReadoutTile
		nativitySelector: #authoringPrototype.

	   DescriptionForPartsBin
		formalName: 'Number (bare)'
		categoryList: #('Basic')
		documentation: 'A number readout for a Stack.  Shows current value.  Click and type the value.'
		globalReceiverSymbol: #NumericReadoutTile
		nativitySelector: #simplePrototype.

	   DescriptionForPartsBin
		formalName: 'Number (mid)'
		categoryList: #('Basic')
		documentation: 'A number readout for a Stack.  Shows current value.  Click and type the value.'
		globalReceiverSymbol: #NumericReadoutTile
		nativitySelector: #borderedPrototype}! !

!NumericReadoutTile methodsFor: 'accessing' stamp: 'ar 9/15/2000 23:27'!
abandonLabelFocus
	| aLabel |
	"If the receiver's label has editing focus, abandon it"
	self flag: #arNote. "Probably unnecessary"
	(aLabel _ self labelMorph) ifNotNil:
		[aLabel hasFocus ifTrue:
			[aLabel contents: aLabel readFromTarget.
			aLabel handsWithMeForKeyboardFocus do:
				[:aHand | aHand releaseKeyboardFocus]]]! !

!NumericReadoutTile methodsFor: 'testing' stamp: 'tk 11/1/2001 12:41'!
basicType
	"Answer a symbol representing the inherent type I hold"

	"Number String Boolean player collection sound color etc"
	^ #Number! !

!NumericReadoutTile methodsFor: 'misc' stamp: 'sw 9/17/1999 08:01'!
basicWidth
	^ 26! !

!NumericReadoutTile methodsFor: 'mouse' stamp: 'ar 10/25/2000 18:07'!
handlesMouseMove: evt
	^true! !

!NumericReadoutTile methodsFor: 'parts bin' stamp: 'sw 11/15/2001 20:22'!
initializeToStandAlone
	"Enclose my prototype in a SyntaxMorph.  For the ObjectTool"

	| aWatcher aTile aLine aColor ms slotMsg |

	super initializeToStandAlone.
	aColor _ Color r: 0.387 g: 0.581 b: 1.0.
	aTile _ self typeColor: aColor.
	aWatcher _ UpdatingStringMorph new.
	aWatcher growable: true;
		getSelector: nil;
		putSelector: nil;
		setToAllowTextEdit.
	aWatcher target: nil.
	aTile addMorphBack: aWatcher.
	aTile addArrows.
	aTile setLiteralTo: 5 width: 30.

	ms _ MessageSend receiver: nil selector: #aNumber arguments: #().
	slotMsg _ ms asTilesIn: Player globalNames: false.
		"For CardPlayers, use 'aPlayer'.  For others, name it, and use its name."
	ms _ MessageSend receiver: 3 selector: #= asSymbol arguments: #(5).
	aLine _ ms asTilesIn: Player globalNames: false.
	aLine firstSubmorph delete.		"A little over-complicated?  Yes?"
	aLine addMorphFront: (slotMsg submorphs second) firstSubmorph.
	aLine addMorphFront: (Morph new transparentSpacerOfSize: 3@3).
	aLine lastSubmorph delete.
	aLine lastSubmorph delete.
	aLine color: aColor.
	aLine addMorphBack: (Morph new transparentSpacerOfSize: 3@3).
	aLine addMorphBack: aTile.
	aLine cellPositioning: #leftCenter.
	aWatcher step; fitContents.
	^ aLine markAsPartsDonor.! !

!NumericReadoutTile methodsFor: 'accessing' stamp: 'tak 4/6/2005 14:10'!
literal: anObject 
	literal := anObject.
	self updateLiteralLabel.
	self labelMorph
		ifNotNilDo: [:label | label informTarget]! !

!NumericReadoutTile methodsFor: 'accessing' stamp: 'tak 12/6/2004 01:53'!
literalFromContents
	| label |
	label _ self labelMorph
				ifNil: [^ super literal].
	label step.
	^ literal _ label valueFromContents! !

!NumericReadoutTile methodsFor: 'misc' stamp: 'sw 9/17/1999 08:18'!
minimumWidth
	^ 40! !

!NumericReadoutTile methodsFor: 'event handling' stamp: 'tak 8/2/2005 23:36'!
mouseStillDown: evt 
	(self labelMorph notNil
			and: [self labelMorph containsPoint: evt cursorPoint])
		ifTrue: [^ self labelMorph mouseDown: evt].
	^ super mouseStillDown: evt! !

!NumericReadoutTile methodsFor: 'literal' stamp: 'sw 9/15/1999 15:14'!
setLiteralTo: anObject width: w
	"like literal:width: but does not inform the target"
	literal _ anObject.
	self updateLiteralLabel.
	submorphs last setWidth: w.
	self updateLiteralLabel! !

TileMorph subclass: #ParameterTile
	instanceVariableNames: 'scriptEditor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!ParameterTile commentStamp: '<historical>' prior: 0!
Represents a parameter in a user-defined script in "classic" tile-scripting.  The type of a script's parameter is declared in the ScriptEditor for the script, and a parameter tile gets its type from the script.  But because the user can change the parameter type *after* having created parameter tiles, we can later have type mismatches.  Which however we at least deal with reasonably cleverly.!

!ParameterTile methodsFor: 'miscellaneous' stamp: 'sw 3/15/2005 21:55'!
associatedPlayer
	"Answer the player with which the receiver is associated"

	^ self scriptEditor playerScripted! !

!ParameterTile methodsFor: 'type' stamp: 'sw 7/22/2002 17:48'!
assureTypeStillValid
	"Consider the possibility that the parameter type of my surrounding method has changed and that hence I no longer represent a possible value for the parameter of the script.  If this condition obtains, then banish me in favor of a default literal tile of the correct type"

	(self ownerThatIsA: TilePadMorph) ifNotNilDo:
		[:aPad | aPad type = self scriptEditor typeForParameter ifFalse:
			[aPad setToBearDefaultLiteral]]! !

!ParameterTile methodsFor: 'miscellaneous' stamp: 'sw 3/15/2005 22:37'!
booleanComparatorPhrase
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase rel retrieverType |
	retrieverType _ self resultType.

	rel _ (Vocabulary vocabularyForType: retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	outerPhrase firstSubmorph addMorph: self.
	outerPhrase submorphs last addMorph: (ScriptingSystem tileForArgType: retrieverType).

	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!ParameterTile methodsFor: 'initialization' stamp: 'yo 3/14/2005 08:01'!
forScriptEditor: aScriptEditor
	"Make the receiver be associated with the given script editor"

	scriptEditor _ aScriptEditor.
	self line1: aScriptEditor typeForParameter translated.! !

!ParameterTile methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:45'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	self typeColor: Color red! !

!ParameterTile methodsFor: 'access' stamp: 'sw 3/15/2005 21:45'!
isBoolean 
	"Answer whether the receiver's type is inherently boolean"

	^ self scriptEditor typeForParameter == #Boolean! !

!ParameterTile methodsFor: 'accessing' stamp: 'sw 7/18/2002 02:45'!
resultType
	"Answer the result type of the receiver"

	^ self scriptEditor typeForParameter! !

!ParameterTile methodsFor: 'access' stamp: 'sw 3/15/2005 22:36'!
rowOfRightTypeFor: aLayoutMorph forActor: aPlayer
	"Answer a phrase of the right type for the putative container"

	| aTemporaryViewer aPhrase |
	aLayoutMorph demandsBoolean ifTrue:
		[aTemporaryViewer _ CategoryViewer new invisiblySetPlayer: aPlayer.
		aPhrase _ aTemporaryViewer booleanPhraseFromPhrase: self.
		aPhrase justGrabbedFromViewer: false.
		^ aPhrase].
	^ self! !

!ParameterTile methodsFor: 'access' stamp: 'sw 1/18/2004 22:12'!
scriptEditor
	"Answer the receiver's script editor.  The slightly strange code here is in order to contend with the unusual situation where a parameter tile obtained from one script editor is later dropped into a different script editor.  As long as the parameter tile is *in* a script editor, that containing scriptEditor is the one; if it is *not*, then we use the last known one"

	| aScriptEditor |
	^ (aScriptEditor _ self outermostMorphThat: [:m | m isKindOf: ScriptEditorMorph])
		ifNotNil:
			[scriptEditor _ aScriptEditor]
		ifNil:
			[scriptEditor]! !

!ParameterTile methodsFor: 'code generation' stamp: 'yo 12/20/2003 02:49'!
storeCodeOn: aStream indent: tabCount
	"Store code on the stream"
 
	| myTypeString |
	myTypeString _ self resultType.
	(self scriptEditor hasParameter and: [self scriptEditor typeForParameter = myTypeString])
		ifTrue:
			[aStream nextPutAll: 'parameter']
		ifFalse:
			["This script no longer bears a parameter, yet there's an orphaned Parameter tile in it"
			aStream nextPutAll: '(self defaultValueOfType: #', myTypeString, ')']! !

!ParameterTile methodsFor: 'miscellaneous' stamp: 'sw 3/15/2005 22:41'!
tileRows
	"Answer a list of tile rows -- in this case exactly one row -- representing the receiver."

	^ Array with: (Array with: self)! !

TileMorph subclass: #RandomNumberTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!RandomNumberTile classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 10:50'!
initialize

	self registerInFlapsRegistry.	! !

!RandomNumberTile classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 10:51'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: #(RandomNumberTile		new		'Random'		'A random-number tile for use with tile scripting')
						forFlapNamed: 'PlugIn Supplies'.
						cl registerQuad: #(RandomNumberTile	new	'Random'		'A tile that will produce a random number in a given range')
						forFlapNamed: 'Scripting'.]! !

!RandomNumberTile classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 12:39'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!RandomNumberTile methodsFor: 'code generation' stamp: 'yo 9/3/2004 00:45'!
basicStoreCodeOn: aStream indent: tabCount
	aStream
			nextPut: $(;
			nextPutAll: literal printString;
			nextPutAll: ' atRandom)'.! !

!RandomNumberTile methodsFor: 'event handling' stamp: 'RAA 7/31/2001 12:02'!
handlesMouseDown: evt

	| aPoint |

	self inPartsBin ifTrue: [^false].
	aPoint _ evt cursorPoint.

	"This might actually be a suitable test for the superclass, but I'll do it here to minimize the downside"

	{upArrow. downArrow. suffixArrow. retractArrow} do: [ :each |
		(each notNil and: [each bounds containsPoint: aPoint]) ifTrue: [
			^true
		]
	].

	^false		"super handlesMouseDown: evt"! !

!RandomNumberTile methodsFor: 'initialization' stamp: 'dgd 9/20/2003 19:11'!
initialize
	"Initialize the receiver fully, including adding all its relevant submorphs"

	| m1 m2 |
	super initialize.
	self vResizing: #shrinkWrap.
	self typeColor: (ScriptingSystem colorForType: #Number).
	self addArrows.
	m1 _ StringMorph contents: 'random' translated font: ScriptingSystem fontForTiles.
	self addMorph: m1.
	m2 _ UpdatingStringMorph contents: '180' font: ScriptingSystem fontForTiles.
	m2 target: self; getSelector: #literal; putSelector: #literal:.
	m2 position: m1 topRight.
	self addMorphBack: m2.
	literal _ 180.
	self updateLiteralLabel.
	self makeAllTilesGreen! !

!RandomNumberTile methodsFor: 'code generation' stamp: 'yo 9/3/2004 00:54'!
kedamaStoreCodeOn: aStream indent: tabCount actualObject: obj
	aStream
			nextPut: $(;
			nextPutAll: obj uniqueNameForReference;
			nextPutAll: ' random: ';
			nextPutAll: literal printString;
			nextPut: $).
! !

!RandomNumberTile methodsFor: 'accessing' stamp: 'sw 3/24/2000 00:43'!
labelMorph
	^ submorphs detect: [:m | m isKindOf: UpdatingStringMorph] ifNone: [nil].! !

!RandomNumberTile methodsFor: 'misc' stamp: 'sw 6/25/1998 17:44'!
numericValue
	^ super numericValue atRandom! !

!RandomNumberTile methodsFor: 'accessing' stamp: 'yo 9/16/2004 18:50'!
storeCodeOn: aStream indent: tabCount

	| phrase player costume |
	phrase _ self outermostMorphThat: [:m| m isKindOf: PhraseTileMorph].
	phrase ifNil: [^ self basicStoreCodeOn: aStream indent: tabCount].

	player _ phrase associatedPlayer.
	player ifNil: [^ self basicStoreCodeOn: aStream indent: tabCount].

	costume _ player costume.
	costume ifNil: [^ self basicStoreCodeOn: aStream indent: tabCount].

	(player isKindOf: KedamaExamplerPlayer) ifTrue: [
		^ self kedamaStoreCodeOn: aStream indent: tabCount actualObject: player costume renderedMorph kedamaWorld].

	(costume renderedMorph isMemberOf: KedamaMorph) ifTrue: [
		^ self kedamaStoreCodeOn: aStream indent: tabCount actualObject: costume renderedMorph].

	^ self basicStoreCodeOn: aStream indent: tabCount.! !

!RandomNumberTile methodsFor: 'player viewer' stamp: 'tk 8/14/2000 23:23'!
updateLiteralLabel
	| myReadout |
	(myReadout _ self labelMorph) ifNil: [^ self].
	myReadout contents: literal stringForReadout.
! !

!RandomNumberTile methodsFor: 'initialization' stamp: 'yo 7/2/2004 20:59'!
updateWordingToMatchVocabulary

	| stringMorph |
	stringMorph _ submorphs
				detect: [:morph | morph class == StringMorph]
				ifNone: [^ self].
	stringMorph contents: 'random' translated.

! !

TileMorph subclass: #SoundTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!SoundTile commentStamp: 'sw 1/28/2005 01:42' prior: 0!
A scripting tile representing a 'sound' constant.  Sounds are represented by their names, which are strings; the actual sounds live in SampleSound's SoundLibrary.!

!SoundTile methodsFor: 'mouse handling' stamp: 'sw 11/24/2003 14:44'!
handlerForMouseDown: anEvent
	"Return the (prospective) handler for a mouse down event. The handler is temporarily installed and can be used for morphs further down the hierarchy to negotiate whether the inner or the outer morph should finally handle the event"

	^ ((self findA: UpdatingStringMorph) bounds containsPoint: anEvent cursorPoint)
		ifTrue:
			[self]
		ifFalse:
			[super handlerForMouseDown: anEvent]! !

!SoundTile methodsFor: 'initialization' stamp: 'yo 7/22/2005 15:51'!
initialize
	"Initialize the state of the receiver. Pick the croak sound
	if available, otherwise any sound."
	
	| soundChoices startingSoundName |
	super initialize.
	soundChoices _ self soundChoices.
	startingSoundName _ (soundChoices includes: 'croak')
							ifTrue: ['croak']
							ifFalse: [[soundChoices anyOne] ifError: ['silence']].
	self addArrows; setLiteral: startingSoundName.
	self labelMorph useSymbolFormat! !

!SoundTile methodsFor: 'event handling' stamp: 'sw 1/28/2005 01:49'!
options
	"Answer the options of the tile for an arrow"

	| soundChoices |
	soundChoices _ self soundChoices.
	^ {soundChoices. soundChoices}! !

!SoundTile methodsFor: 'accessing' stamp: 'sw 9/27/2001 17:28'!
resultType
	"Answer the result type of the receiver"

	^ #Sound! !

!SoundTile methodsFor: 'initialization' stamp: 'yo 4/6/2005 16:34'!
setLiteral: anObject

	super setLiteral: anObject.
	self labelMorph useSymbolFormat
! !

!SoundTile methodsFor: 'access' stamp: 'gk 2/23/2004 21:08'!
sound

	^ SoundService default soundNamed: literal
! !

!SoundTile methodsFor: 'event handling' stamp: 'tak 12/5/2004 02:09'!
value: anObject 
	super value: anObject.
	self playSoundNamed: anObject! !

!SoundTile methodsFor: 'event handling' stamp: 'sw 11/3/97 02:11'!
wantsKeyboardFocusFor: aSubmorph
	^ false! !

TileMorph subclass: #StringReadoutTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

StringReadoutTile subclass: #SoundReadoutTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!SoundReadoutTile commentStamp: 'sw 11/24/2003 15:25' prior: 0!
A tile comprising a readout for a sound-valued instance variable in a Viewer.  It sports up/down  arrows, and a click on the sound name results in a pop-up menu, offering the user the opportunity to choose a new one.!

!SoundReadoutTile methodsFor: 'arrows' stamp: 'sw 1/28/2005 00:57'!
arrowAction: delta
	"Do what is appropriate when an arrow on the tile is pressed; delta will be +1 or -1"

	| soundChoices index |
	soundChoices _ self soundChoices.
	index _ soundChoices indexOf: literal.
	self literal: (soundChoices atWrap: (index + delta)).
	self playSoundNamed: literal! !

!SoundReadoutTile methodsFor: 'arrows' stamp: 'sw 11/24/2003 14:54'!
handlerForMouseDown: anEvent
	"Return the (prospective) handler for a mouse down event. The handler is temporarily installed and can be used for morphs further down the hierarchy to negotiate whether the inner or the outer morph should finally handle the event"

	^ ((self findA: UpdatingStringMorph) bounds containsPoint: anEvent cursorPoint)
		ifTrue:
			[self]
		ifFalse:
			[super handlerForMouseDown: anEvent]! !

!SoundReadoutTile methodsFor: 'arrows' stamp: 'sw 1/28/2005 00:56'!
mouseDown: evt
	"Handle a mouse down event"

	| aPoint index isUp soundChoices adjustment |
	upArrow ifNotNil: [((isUp _ upArrow containsPoint: (aPoint _ evt cursorPoint)) or:  [downArrow containsPoint: aPoint])
		ifTrue:
			[soundChoices _ self soundChoices.
			index _ soundChoices indexOf: literal ifAbsent: [1].
			index > 0 ifTrue:
				[adjustment _ isUp ifTrue: [1] ifFalse: [-1].
				self literal: (soundChoices atWrap: (index + adjustment))].
			self playSoundNamed: literal.
			^ self]].
	self soundNameFromUser ifNotNilDo:
		[:aSoundName |
			self literal: aSoundName.
			self playSoundNamed: literal]! !

!SoundReadoutTile methodsFor: 'arrows' stamp: 'yo 2/11/2005 16:12'!
setLiteral: aLiteral
	super  setLiteral: aLiteral.
	(self findA: UpdatingStringMorph) useSymbolFormat; lock! !

!SoundReadoutTile methodsFor: 'literal' stamp: 'sw 1/28/2005 00:57'!
setLiteralTo: anObject width: w
	"Set the literal and width of the tile as indicated"

	| soundChoices index |
	soundChoices _ self soundChoices.
	index _ soundChoices indexOf: anObject.
	self setLiteral: (soundChoices atWrap: index)! !

!SoundReadoutTile methodsFor: 'arrows' stamp: 'yo 2/11/2005 16:08'!
soundNameFromUser
	"Obtain a sound from the user.  Exclude the items designated as being discouraged, except that if the current selection is one of those, show it anyway"

	| choices |
	choices _ self soundChoices.
	^ (SelectionMenu labels: (choices collect: [:t | t translated]) selections: self soundChoices) startUpWithCaption: 'Sounds' translated! !

!SoundReadoutTile methodsFor: 'private' stamp: 'yo 2/11/2005 16:13'!
updateLiteralLabel
	"Update the wording emblazoned on the tile, if needed"

	super updateLiteralLabel.
	(self findA: UpdatingStringMorph) useSymbolFormat; lock! !

!StringReadoutTile methodsFor: 'misc' stamp: 'sw 9/17/1999 08:01'!
basicWidth
	^ 26! !

!StringReadoutTile methodsFor: 'event handling' stamp: 'tak 8/2/2005 18:20'!
handlesMouseDown: evt 
	^ true! !

!StringReadoutTile methodsFor: 'accessing' stamp: 'sw 11/1/97 13:15'!
literal: anObject
	literal _ anObject.
	self updateLiteralLabel.
	submorphs last informTarget! !

!StringReadoutTile methodsFor: 'event handling' stamp: 'tak 8/2/2005 23:38'!
mouseStillDown: evt 
	(self labelMorph notNil
			and: [self labelMorph containsPoint: evt cursorPoint])
		ifTrue: [^ self labelMorph mouseDown: evt].
	^ super mouseStillDown: evt! !

!StringReadoutTile methodsFor: 'literal' stamp: 'sw 9/15/1999 15:14'!
setLiteralTo: anObject width: w
	"like literal:width: but does not inform the target"
	literal _ anObject.
	self updateLiteralLabel.
	submorphs last setWidth: w.
	self updateLiteralLabel! !

TileMorph subclass: #SymbolListTile
	instanceVariableNames: 'choices dataType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!SymbolListTile commentStamp: '<historical>' prior: 0!
Instances of SymbolListTile are literal tiles whose literals are choosable from a finite list.!

SymbolListTile subclass: #ScriptNameTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!ScriptNameTile commentStamp: '<historical>' prior: 0!
A tile which refers to a script name.  The choices available to the user, via the arrows and via the pop-up she gets when she clicks on the current script-name, are the names of all the user scripts in any Players in the active World.!

!ScriptNameTile methodsFor: 'user interface' stamp: 'sw 3/9/2004 18:46'!
adjustHelpMessage
	"Adjust the help message to reflect the new literal"

	self labelMorph setBalloonText: 'The name of the script to which this operation applies' translated! !

!ScriptNameTile methodsFor: 'initialization' stamp: 'sw 12/19/2003 23:30'!
choices
	"Answer the current list of choices"

	^ ActiveWorld presenter allKnownUnaryScriptSelectors! !

!ScriptNameTile methodsFor: 'initialization' stamp: 'sw 12/19/2003 23:09'!
dataType: aDataType
	"Initialize the receiver with the given data type"

	dataType _ aDataType.
	literal _ #emptyScript! !

!SymbolListTile classSide methodsFor: 'customevents-updating' stamp: 'nk 7/21/2003 22:16'!
updateAllTilesForVocabularyNamed: aVocabularyName
	"The choices in the Vocabulary named aVocabularyName may have changed.
	Update my subinstances if necessary to reflect the changes."

	 (self allSubInstances select: [ :ea | ea dataType = aVocabularyName ])
		do: [ :ea | ea updateChoices ] ! !

!SymbolListTile methodsFor: 'user interface' stamp: 'yo 1/12/2005 14:38'!
acceptNewLiteral: aLiteral
	"Accept the new literal"

	self labelMorph useSymbolFormat.
	self literal: aLiteral.
	self adjustHelpMessage.
	self acceptNewLiteral.  "so tile scriptor can recompile if necessary"
	self labelMorph informTarget
! !

!SymbolListTile methodsFor: 'user interface' stamp: 'sw 3/10/2004 23:24'!
adjustHelpMessage
	"Adjust the help message to reflect the new literal"

	(ScriptingSystem helpStringOrNilForOperator: literal) ifNotNilDo:
		[:aString |
			self labelMorph setBalloonText: aString]! !

!SymbolListTile methodsFor: 'user interface' stamp: 'sw 12/21/2003 00:07'!
choices
	"Answer the list of current choices for the receiver's symbol"

	dataType == #ScriptName ifTrue: "Backward compatibility with old tiles"
		[^ ActiveWorld presenter allKnownUnaryScriptSelectors].
	^ choices! !

!SymbolListTile methodsFor: 'initialization' stamp: 'sw 10/30/2000 09:04'!
choices: choiceList dataType: aDataType
	"Initialize the receiver with the given choice-list and data type"

	choices _ choiceList.
	dataType _ aDataType.
	literal _ choiceList first! !

!SymbolListTile methodsFor: 'customevents-accessing' stamp: 'nk 7/21/2003 22:02'!
dataType
	^dataType! !

!SymbolListTile methodsFor: 'event handling' stamp: 'sw 12/3/2001 21:30'!
handlesMouseDown: evt
	"Answer whether the receiver handles mouse-down"

	^ true! !

!SymbolListTile methodsFor: 'initialization' stamp: 'tak 12/6/2004 01:38'!
initialize
	super initialize.
	literal _ #nothing! !

!SymbolListTile methodsFor: 'private' stamp: 'yo 1/12/2005 14:28'!
line1: line1
	"Emblazon the receiver with the requested label.  If the receiver already has a label, make the new label be of the same class"

	super line1: line1.
	self labelMorph useSymbolFormat! !

!SymbolListTile methodsFor: 'accessing' stamp: 'yo 7/2/2004 21:27'!
literal: anObject
	"Set the receiver's literal as indicated"
	self flag: #yo.

	literal _ anObject asSymbol.
	self updateLiteralLabel.
"
	key _ Vocabulary eToyVocabulary translationKeyFor: literal.
	key isNil ifFalse: [literal _ key].
"
	self flag: #deferred.  "The below formerly was necessary but now is problematical, leading to low-space condition etc.  May need to revisit, since as I comment this out now I am uncertain what if anything this may break"
	"self labelMorph informTarget"

! !

!SymbolListTile methodsFor: 'events-processing' stamp: 'sw 12/3/2001 20:45'!
mouseDownPriority
	"Higher-priority than parts donor, so that the tile can offer a popup even when it is in a larger structure, such as a PhraseTileMorph, that itself behaves as a parts donor"

	^ 75! !

!SymbolListTile methodsFor: 'user interface' stamp: 'sw 1/4/2005 00:16'!
offerAllChoicesInAPopUp
	"Retained in deference to pre-existing content that may have event handlers that send this message."

	! !

!SymbolListTile methodsFor: 'accessing' stamp: 'tak 12/6/2004 01:58'!
options
	^ {self choices. self choices
		collect: [:each | ScriptingSystem helpStringForOperator: literal]}! !

!SymbolListTile methodsFor: 'misc' stamp: 'sw 11/6/2001 13:30'!
setLiteralInitially: anObject
	"Establish the initial literal.  Get the label correct, but do *not* send the value back to the target via the setter (unlike #literal:)"

	literal _ anObject ifNotNil: [anObject asSymbol].
	self updateLiteralLabel! !

!SymbolListTile methodsFor: 'customevents-initialization' stamp: 'nk 7/21/2003 22:14'!
updateChoices
	choices _ (Vocabulary vocabularyNamed: dataType) choices.
	(choices includes: literal) ifFalse: [ literal _ choices first. self changed ]! !

!SymbolListTile methodsFor: 'player viewer' stamp: 'yo 1/12/2005 14:28'!
updateLiteralLabel
	"Update the wording emblazoned on the tile, if needed.  Copied down, for jimmying, unfortunately"

	| myLabel |
	(myLabel _ self labelMorph) ifNil: [^ self].
	myLabel useSymbolFormat.
	myLabel acceptValue: literal asString.
	self changed.! !

!SymbolListTile methodsFor: 'accessing' stamp: 'tak 12/7/2004 14:42'!
value: anObject 
	self acceptNewLiteral: anObject! !

!SymbolListTile methodsFor: 'event handling' stamp: 'sw 11/16/2001 07:31'!
wantsKeyboardFocusFor: aSubmorph
	"Answer whether a plain mouse click on aSubmorph, a text-edit-capable thing, should result in a text selection there"

	^ false! !

SymbolListTile subclass: #TypeListTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!TypeListTile commentStamp: '<historical>' prior: 0!
A tile that offers a list of supported data types.!

!TypeListTile methodsFor: 'user interface' stamp: 'sw 7/19/2002 14:48'!
acceptNewLiteral: aLiteral
	"Accept the new literal"

	| scriptEditor |
	super acceptNewLiteral: aLiteral.
	(scriptEditor _ self ownerThatIsA: ScriptEditorMorph) ifNotNil:
			[scriptEditor setParameterType: aLiteral asSymbol]! !

!TypeListTile methodsFor: 'menus' stamp: 'yo 3/14/2005 12:08'!
addCustomMenuItems: aCustomMenu hand: aHandMorph
	"Add morph-specific items to the given menu"

	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	aCustomMenu add: 'choose type...' translated action: #showSuffixChoices! !

!TypeListTile methodsFor: 'arrows' stamp: 'em 3/29/2005 15:01'!
addMenuIcon
	"Add a little menu icon; store it in my suffixArrow slot"

	suffixArrow _ ImageMorph new image: (ScriptingSystem formAtKey: #MenuTriangle).
	suffixArrow setBalloonText: 'click here to choose a new type for this parameter' translated.
	self addMorphBack: suffixArrow! !

!TypeListTile methodsFor: 'user interface' stamp: 'sw 7/19/2002 14:58'!
adjustHelpMessage
	"In this case, don't"! !

!TypeListTile methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:45'!
initialize
	"Initialize the receiver. Cheesily, we use the extension arrow 
	graphic for a menu icon temporarily"
	super initialize
"".
	self addMenuIcon! !

!TypeListTile methodsFor: 'mouse handling' stamp: 'tak 12/6/2004 02:43'!
showOptions
	| topScript |
	suffixArrow
		ifNotNil: [(suffixArrow bounds containsPoint: ActiveHand cursorPoint)
				ifTrue: [^ super showOptions]].
	topScript _ self
				outermostMorphThat: [:m | m isKindOf: ScriptEditorMorph].
	topScript
		ifNotNil: [topScript handUserParameterTile]! !

!TypeListTile methodsFor: 'arrows' stamp: 'tak 12/7/2004 14:28'!
showSuffixChoices
	"When the user clicks on the suffix arrow, put up the type-choices menu"
	"A bit dirty hack, avoided testing whether the down arrow includes mouse point"
	 super showOptions! !

!TypeListTile methodsFor: 'accessing' stamp: 'tak 12/6/2004 02:50'!
value: anObject 
	| scriptEditor |
	super value: anObject.
	(scriptEditor _ self ownerThatIsA: ScriptEditorMorph)
		ifNotNil: [scriptEditor setParameterType: anObject]! !

!TileMorph classSide methodsFor: 'constants' stamp: 'tk 9/17/97 17:24'!
defaultH

	^ 22! !

!TileMorph classSide methodsFor: 'scripting' stamp: 'dgd 8/26/2004 12:12'!
defaultNameStemForInstances
	^ 'Tile'! !

!TileMorph classSide methodsFor: 'class initialization' stamp: 'sw 1/21/98 16:40'!
downPicture
	^ DownPicture! !

!TileMorph classSide methodsFor: 'class initialization' stamp: 'sw 1/21/98 17:30'!
fixCaretForms
	"TileMorph fixCaretForms"
	"UpPicture storeString"
	"DownPicture storeString"

	UpPicture _  Form
	extent: 9@8
	depth: 16
	fromArray: #( 0 0 60817408 0 0 0 0 60818336 0 0 0 928 60818336 60817408 0 0 928 60818336 60817408 0 0 60818336 60818336 60818336 0 928 60818336 60818336 60818336 0 928 60818336 60818336 60818336 60817408 60818336 60818336 60818336 60818336 60817408)
	offset: 0@0.

	DownPicture _ Form
	extent: 9@8
	depth: 16
	fromArray: #( 60818336 60818336 60818336 60818336 60817408 928 60818336 60818336 60818336 60817408 928 60818336 60818336 60818336 0 0 60818336 60818336 60818336 0 0 928 60818336 60817408 0 0 928 60818336 60817408 0 0 0 60818336 0 0 0 0 60817408 0 0)
	offset: 0@8.

	SuffixPicture _  Form
	extent: 10@8
	depth: 16
	fromArray: #( 928 0 0 0 0 60818336 60818336 0 0 0 60818336 60818336 60818336 60817408 0 60818336 60818336 60818336 60818336 0 60818336 60818336 60818336 60818336 60818336 60818336 60818336 60818336 60818336 60818336 60818336 60818336 60818336 60818336 0 60818336 60818336 60818336 0 0)
	offset: 0@0! !

!TileMorph classSide methodsFor: 'class initialization' stamp: 'tk 8/15/2000 11:00'!
initialize
	"TileMorph readInArrowGraphics    -- call manually if necessary to bring graphics forward"
	"TileMorph initialize"

	UpdatingOperators _ Dictionary new.
	UpdatingOperators at: #incr: put: #+.
	UpdatingOperators at: #decr: put: #-.
	UpdatingOperators at: #set: put: ''.

	RetractPicture ifNil: [
		RetractPicture _ (SuffixPicture flipBy: #horizontal centerAt: (SuffixPicture center))].
	SuffixArrowAllowance _ 5 + SuffixPicture width + RetractPicture width.
	UpArrowAllowance _ 10.
! !

!TileMorph classSide methodsFor: 'class initialization' stamp: 'mdr 9/4/2000 11:06'!
readInArrowGraphics
	"TileMorph readInArrowGraphics"

	| obj |
	obj _ (FileStream readOnlyFileNamed: 'tile inc arrow.morph') fileInObjectAndCode.
	UpPicture _ obj form.

	obj _ (FileStream readOnlyFileNamed: 'tile dec arrow.morph') fileInObjectAndCode.
	DownPicture _ obj form.

	obj _ (FileStream readOnlyFileNamed: 'tile suffix arrow.morph')fileInObjectAndCode.
	SuffixPicture _ obj form.! !

!TileMorph classSide methodsFor: 'class initialization' stamp: 'sw 1/21/98 16:29'!
upPicture
	^ UpPicture! !

!TileMorph methodsFor: 'accessing' stamp: 'ar 9/15/2000 23:31'!
abandonLabelFocus
	"If the receiver's label has editing focus, abandon it"
	self flag: #arNote. "Probably unnecessary"
	self currentHand releaseKeyboardFocus: self labelMorph.! !

!TileMorph methodsFor: 'code generation' stamp: 'sw 8/30/2004 16:34'!
acceptNewLiteral
	"Tell the scriptEditor who I belong to that I have a new literal value."

	| topScript |
	topScript _ self outermostMorphThat:
		[:m | m isKindOf: ScriptEditorMorph].
	topScript ifNotNil: [topScript installWithNewLiteral].
	(self ownerThatIsA: ViewerLine) ifNotNilDo:
		[:aLine |
			(self ownerThatIsA: PhraseTileMorph) ifNotNil:
				[aLine removeHighlightFeedback.
				self layoutChanged.
				ActiveWorld doOneSubCycle.
				aLine addCommandFeedback]]! !

!TileMorph methodsFor: 'initialization' stamp: 'sw 10/3/97 18:00'!
actualObject
	^ actualObject! !

!TileMorph methodsFor: 'arrows' stamp: 'ar 11/2/2000 22:29'!
addArrows
	| frame |
	downArrow _ ImageMorph new image: DownPicture.
	upArrow _ ImageMorph new image: UpPicture.
	frame _ Morph new color: Color transparent.
	frame 
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #shrinkWrap; 
		vResizing: #shrinkWrap;
		cellInset: 0@1;
		layoutInset: 0@1.
	frame addMorphBack: upArrow; addMorphBack: downArrow.
	self addMorphFront: frame.
! !

!TileMorph methodsFor: 'misc' stamp: 'dgd 8/30/2003 22:19'!
addCustomMenuItems:  aMenu hand: aHandMorph
	"Add custom halo menu items to a menu"

	| aPlayer |
	super addCustomMenuItems: aMenu hand: aHandMorph.
	((aPlayer _ self associatedPlayer) notNil and:
		[aPlayer costume isMorph]) ifTrue:
			[aMenu addLine.
			aMenu add: 'hand me this object' translated target: self action: #handReferentMorph.
			aMenu balloonTextForLastItem: 'This tile refers to an actual graphical object; use this menu item to grab that object.  Caution!!  This may remove the object from a place it really ought to stay.' translated.
			aMenu addLine ]! !

!TileMorph methodsFor: 'arrows' stamp: 'tk 8/16/2000 09:48'!
addRetractArrow
	"Must be situated in a script"

	self couldRetract ifNil: [^ self].
	retractArrow _ ImageMorph new image: RetractPicture.
	suffixArrow ifNotNil: [
		self addMorph: retractArrow inFrontOf: suffixArrow].
	fullBounds _ nil.
	self extent: self fullBounds extent! !

!TileMorph methodsFor: 'arrows' stamp: 'ar 11/2/2000 22:30'!
addSuffixArrow

	suffixArrow _ ImageMorph new image: SuffixPicture.
	self addMorphBack: suffixArrow.! !

!TileMorph methodsFor: 'arrows' stamp: 'tk 10/12/97 15:05'!
addSuffixIfCan
	"Should this tile have a suffix arrow?"

	self addSuffixArrow.! !

!TileMorph methodsFor: 'e-toy support' stamp: 'sw 6/4/2001 16:48'!
adoptVocabulary: aVocabulary
	"Set the receiver's vocabulary"

	vocabularySymbol _ aVocabulary vocabularyName.
	self updateWordingToMatchVocabulary.
	super adoptVocabulary: aVocabulary! !

!TileMorph methodsFor: 'arrows' stamp: 'tak 12/5/2004 14:36'!
arrowAction: delta 
	"Do what is appropriate when an arrow on the tile is pressed; delta will  
	be +1 or -1"
	| index options |
	(type == #literal
			and: [literal isNumber])
		ifTrue: [self value: literal + delta]
		ifFalse: [options _ self options
						ifNil: [^ self].
			index _ (options first indexOf: self value)
						+ delta.
			self
				value: (options first atWrap: index).
			submorphs last
				setBalloonText: (options second atWrap: index)]! !

!TileMorph methodsFor: 'mouse handling' stamp: 'sw 11/29/2001 13:14'!
arrowDelta
	"Answer the amount by which a number I display should increase at a time"

	^  self valueOfProperty: #arrowDelta ifAbsent: [1]! !

!TileMorph methodsFor: 'accessing' stamp: 'sw 11/6/1998 11:02'!
associatedPlayer
	^ actualObject! !

!TileMorph methodsFor: 'misc' stamp: 'sw 9/17/1999 08:00'!
basicWidth
	"Provide a nominal minimum, exclusive of arrows and independent of label width"

	^ operatorOrExpression
		ifNotNil:
			[3]
		ifNil:
			[18]! !

!TileMorph methodsFor: 'initialization' stamp: 'dgd 9/1/2003 13:34'!
bePossessive
	possessive := true.
	self line1: actualObject externalName , '''s' translated! !

!TileMorph methodsFor: 'initialization' stamp: 'dgd 9/1/2003 13:35'!
bringUpToDate
	"Make certain, if the receiver is an object-reference tile, that it shows the current external name of the object, which may just have changed.  This only applies to the Player regime." 

	| newLabel |
		(type == #objRef and: [actualObject isPlayerLike]) ifTrue:
		[newLabel _ actualObject externalName.
		self isPossessive ifTrue:
			[newLabel _ newLabel, '''s' translated].
		self line1: newLabel]! !

!TileMorph methodsFor: 'code generation' stamp: 'sw 9/2/1999 15:01'!
codeString
	^ String streamContents: [:aStream | self storeCodeOn: aStream indent: 1]
! !

!TileMorph methodsFor: 'change reporting' stamp: 'tk 2/21/2001 20:07'!
colorChangedForSubmorph: aSubmorph
	"Invoked when the user selects a new color on a colorTile or a color-seer-tile; need to recompile the script."
	self acceptNewLiteral
	owner ifNil: [^ self].
	owner isTileLike ifFalse: [owner colorChangedForSubmorph: aSubmorph].! !

!TileMorph methodsFor: 'private' stamp: 'ar 11/2/2000 19:21'!
convertAlignment
	"Convert the receiver's alignment rules"
	| where frame |
	owner ifNotNil:[
		owner class == TilePadMorph ifTrue:[
			owner layoutPolicy: TableLayout new.
			owner hResizing: #shrinkWrap.
			owner vResizing: #spaceFill.
		].
	].
	self layoutPolicy: TableLayout new.
	self cellInset: 2@0.
	self layoutInset: 1@0.
	self listDirection: #leftToRight.
	self wrapCentering: #center.
	self hResizing: #shrinkWrap.
	self vResizing: #spaceFill.
	"Now convert up and down arrow"
	(upArrow notNil and:[upArrow owner == self "e.g., not converted"
		and:[downArrow notNil and:[downArrow owner == self]]]) ifTrue:[
			"where to insert the frame"
			where _ (submorphs indexOf: upArrow) min: (submorphs indexOf: downArrow).
			frame _ Morph new color: Color transparent.
			frame 
				layoutPolicy: TableLayout new;
				listDirection: #topToBottom;
				hResizing: #shrinkWrap; 
				vResizing: #shrinkWrap;
				cellInset: 0@1;
				layoutInset: 0@1.
			self privateAddMorph: frame atIndex: where.
			frame addMorphBack: upArrow; addMorphBack: downArrow.
		].
! !

!TileMorph methodsFor: 'arrows' stamp: 'tk 8/16/2000 08:59'!
couldRetract
	"See if it makes sense to retract this tile and the op before it.  Return the phrase that gets retracted, or nil if not allowed."
	| phrase pad |
	(phrase _ self ownerThatIsA: PhraseTileMorph) ifNil: [^ nil].
	(pad _ phrase ownerThatIsA: TilePadMorph) ifNil: [^ nil].
	(phrase firstSubmorph "goodPad") type == pad type ifFalse: [
		phrase submorphs size < 3 ifFalse: [^ nil].	"types should have matched"
		"Go up a level"
		(phrase _ pad ownerThatIsA: PhraseTileMorph) ifNil: [^ nil].
		(pad _ phrase ownerThatIsA: TilePadMorph) ifNil: [^ nil].
		(phrase firstSubmorph "goodPad") type == pad type ifFalse: [^ nil].
		].
	^ phrase
! !

!TileMorph methodsFor: 'misc' stamp: 'nk 8/29/2004 17:22'!
currentEToyVocabulary
	"Answer the etoy vocabulary that pertains"
	| aVocab |
	^ (aVocab := self currentVocabulary) isEToyVocabulary
		ifTrue: [aVocab]
		ifFalse: [Vocabulary eToyVocabulary]! !

!TileMorph methodsFor: 'macpal' stamp: 'sw 6/4/2001 19:33'!
currentVocabulary
	"Answer the receiver's current vocabulary"

	| outer aVocab |
	vocabulary ifNotNil:  "old structures -- bring up to date"
		[vocabularySymbol _ vocabulary vocabularyName.
		vocabulary _ nil].
	^ vocabularySymbol
		ifNotNil:
			[Vocabulary vocabularyNamed: vocabularySymbol]
		ifNil:
			[(outer _ self ownerThatIsA: StandardViewer orA: ScriptEditorMorph) 
				ifNotNil:
					[aVocab _ outer currentVocabulary.
					vocabularySymbol _ aVocab vocabularyName.
					aVocab]
				ifNil:
					[super currentVocabulary]]! !

!TileMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:44'!
defaultBorderWidth
"answer the default border width for the receiver"
	^ 1! !

!TileMorph methodsFor: 'arrows' stamp: 'ar 9/24/2000 19:11'!
deleteLastTwoTiles
	"Remove the current suffix (last two tiles) in this line of tiles"
	| phrase pad goodPad |
	(phrase _ self couldRetract) ifNil: [^ self].
	pad _ phrase ownerThatIsA: TilePadMorph.
	goodPad _ phrase firstSubmorph.
	pad owner addMorphBack: goodPad.
	pad delete.
	(goodPad lastSubmorph respondsTo: #addSuffixArrow) 
		ifTrue: [goodPad lastSubmorph addSuffixArrow; addRetractArrow]
		ifFalse: [goodPad lastSubmorph lastSubmorph addSuffixArrow; addRetractArrow].
	goodPad topEditor install. "recompile"! !

!TileMorph methodsFor: 'arrows' stamp: 'tk 8/10/2000 14:34'!
deleteSuffixArrow

	suffixArrow delete.
	suffixArrow _ nil.
	retractArrow ifNotNil: ["backward compat"
		retractArrow delete.
		retractArrow _ nil].
	self updateLiteralLabel! !

!TileMorph methodsFor: 'as yet unclassified' stamp: ''!
fixLayoutOfSubmorphsNotIn: aCollection 
	self
		allMorphsDo: [:m | (aCollection includes: m)
				ifFalse: [(m respondsTo: #fixLayoutOfSubmorphsNotIn:)
						ifTrue: [m ~~ self
								ifTrue: [m fixLayoutOfSubmorphsNotIn: aCollection]]
						ifFalse: [m layoutChanged].
					aCollection add: m]].
	self layoutChanged; fullBounds! !

!TileMorph methodsFor: 'misc' stamp: 'sw 4/9/2001 12:55'!
handReferentMorph
	"Hand the user the actual morph referred to"

	| aMorph surrogate |
	((aMorph _ actualObject costume) isMorph and:
		[aMorph isWorldMorph not])
			ifTrue:
				[surrogate _ CollapsedMorph collapsedMorphOrNilFor: aMorph.
				surrogate
					ifNotNil:
						[surrogate uncollapseToHand]
					ifNil:
						[ActiveHand attachMorph: aMorph]]! !

!TileMorph methodsFor: 'events-processing' stamp: 'ar 9/18/2000 18:03'!
handlerForMouseDown: anEvent
	"Return the (prospective) handler for a mouse down event. The handler is temporarily installed and can be used for morphs further down the hierarchy to negotiate whether the inner or the outer morph should finally handle the event"
	| aPoint |
	upArrow ifNotNil:
		[(upArrow bounds containsPoint: (aPoint _ anEvent cursorPoint))
			ifTrue: [^self].
		(downArrow bounds containsPoint: aPoint)
			ifTrue: [^self]].

	^super handlerForMouseDown: anEvent! !

!TileMorph methodsFor: 'event handling' stamp: 'ar 2/8/2001 20:07'!
handlesMouseDown: evt
	"Answer whether the receiver would handle the mouseDown represented by evt"

	| aPoint |
	aPoint _ evt cursorPoint.
	(operatorOrExpression notNil and: [upArrow notNil]) ifTrue: [^ true].
		"Click on the operator presents list of alternatives"

	upArrow ifNotNil: [^true].
	suffixArrow ifNotNil: [^true].
	retractArrow ifNotNil: [^true].
	^ super handlesMouseDown: evt! !

!TileMorph methodsFor: 'event handling' stamp: 'ar 10/22/2000 17:42'!
handlesMouseStillDown: evt
	^true! !

!TileMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:44'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	self extent: 1 @ 1.
	self
		typeColor: (Color
				r: 0.8
				g: 1.0
				b: 0.6).

	type _ #literal.
	"#literal, #slotRef, #objRef, #operator, #expression"
	slotName _ ''.
	literal _ 1.
	self layoutPolicy: TableLayout new.
	self cellInset: 2 @ 0.
	self layoutInset: 1 @ 0.
	self listDirection: #leftToRight.
	self wrapCentering: #center.
	self hResizing: #shrinkWrap.
	self vResizing: #spaceFill! !

!TileMorph methodsFor: 'e-toy support' stamp: 'sw 7/30/1999 13:08'!
isCandidateForAutomaticViewing
	^ false! !

!TileMorph methodsFor: 'initialization' stamp: 'dgd 2/21/2003 22:56'!
isPossessive
	possessive isNil ifTrue: [^false].
	^possessive! !

!TileMorph methodsFor: 'scripting' stamp: 'tk 10/1/97 18:24'!
isTileLike
	"Can be dropped into a script"
	^ true! !

!TileMorph methodsFor: 'scripting' stamp: 'sw 4/21/1998 21:34'!
isTileScriptingElement
	^ true! !

!TileMorph methodsFor: 'dropping/grabbing' stamp: 'sw 1/7/2004 02:45'!
justDroppedInto: aMorph event: anEvent
	"This message is sent to a dropped morph after it has been dropped on -- and been accepted by -- a drop-sensitive morph"

	super justDroppedInto: aMorph event: anEvent.
	aMorph isPlayfieldLike
		ifTrue:
			[self vResizing: #shrinkWrap]
		ifFalse:
			[aMorph isTileScriptingElement ifTrue:
				[self vResizing: #spaceFill]]
! !

!TileMorph methodsFor: 'dropping/grabbing' stamp: 'ar 2/9/2001 23:57'!
justGrabbedFrom: formerOwner
	| editor |
	formerOwner ifNil:[^self].
	editor _ formerOwner topEditor.
	editor ifNotNil:[editor scriptEdited].! !

!TileMorph methodsFor: 'accessing' stamp: 'sw 4/27/1998 16:10'!
labelMorph
	^ submorphs detect: [:m | m isKindOf: StringMorph] ifNone: [nil].! !

!TileMorph methodsFor: 'accessing' stamp: 'tk 8/16/2000 10:01'!
lastTile
	"The tile that might get an extension arrow"

	^ self! !

!TileMorph methodsFor: 'private' stamp: 'sw 10/30/2000 09:00'!
line1: line1
	"Emblazon the receiver with the requested label.  If the receiver already has a label, make the new label be of the same class"

	| m desiredW classToUse lab |
	classToUse _ (lab _ self labelMorph) ifNotNil: [lab class] ifNil: [StringMorph].
	self removeAllMorphs.
	m _ classToUse contents: line1 font: ScriptingSystem fontForTiles.
	desiredW _ m width + 6.
	self extent: (desiredW max: self minimumWidth) @ self class defaultH.
	m position: self center - (m extent // 2).
	self addMorph: m.
! !

!TileMorph methodsFor: 'accessing' stamp: ''!
literal

	^ literal
! !

!TileMorph methodsFor: 'accessing' stamp: 'di 9/18/97 12:02'!
literal: anObject

	literal _ anObject.
	self updateLiteralLabel.
	self acceptNewLiteral.		"Show that we are out of date, install is needed"
! !

!TileMorph methodsFor: 'accessing' stamp: 'tak 12/6/2004 01:55'!
literalFromContents
	"Get value from StringMorph if it is needed. (See subclass)"
	^ literal! !

!TileMorph methodsFor: 'e-toy support' stamp: 'mir 7/15/2004 15:20'!
localeChanged
	"Update myself to reflect the change in locale"

	self updateWordingToMatchVocabulary! !

!TileMorph methodsFor: 'misc' stamp: 'sw 9/17/1999 08:02'!
minimumWidth
	| aWidth |
	aWidth _ self basicWidth.
	upArrow ifNotNil: [aWidth _ aWidth + UpArrowAllowance].
	suffixArrow ifNotNil: [aWidth _ aWidth + SuffixArrowAllowance].
	^ aWidth
	! !

!TileMorph methodsFor: 'event handling' stamp: 'tak 8/2/2005 18:28'!
mouseDown: evt 
	self setProperty: #previousLiteral toValue: self literalFromContents.
	(upArrow notNil
			and: [(upArrow containsPoint: evt position)
					or: [downArrow containsPoint: evt position]])
		ifTrue: [self setProperty: #previousPoint toValue: evt position].
	self currentHand releaseKeyboardFocus.
	evt hand
		waitForClicksOrDrag: self
		event: evt
		selectors: {#mouseStillDown:. nil. nil. #startDrag:}
		threshold: 5.
	^ super mouseDown: evt! !

!TileMorph methodsFor: 'event handling' stamp: 'tak 8/2/2005 18:11'!
mouseMove: evt 
	self options
		ifNotNil: [^ self showOptions].
	(self hasProperty: #previousLiteral)
		ifFalse: [^ self].
	(self hasProperty: #previousPoint)
		ifFalse: [^ evt hand grabMorph: self rootInPasteUp].
	self currentHand releaseKeyboardFocus.
	"Once reviving the value at drag start"
	literal := self valueOfProperty: #previousLiteral.
	"Then applying delta"
	self arrowAction: (self valueOfProperty: #previousPoint) y - evt position y * self arrowDelta abs.
	^ super mouseMove: evt! !

!TileMorph methodsFor: 'event handling' stamp: 'tak 12/6/2004 01:00'!
mouseStillDown: evt 
	"See if arrows are being pressed and call arrowAction:..."
	| aPoint |
	upArrow
		ifNil: [^ super mouseStillDown: evt].
	aPoint _ evt cursorPoint.
	(upArrow containsPoint: aPoint)
		ifTrue: [^ self
				variableDelay: [self arrowAction: self arrowDelta]].
	(downArrow containsPoint: aPoint)
		ifTrue: [^ self
				variableDelay: [self arrowAction: self arrowDelta negated]].
	self options
		ifNotNil: [^ self showOptions]! !

!TileMorph methodsFor: 'event handling' stamp: 'tak 12/5/2004 13:54'!
mouseUp: evt 
	self removeProperty: #previousLiteral.
	self removeProperty: #previousPoint.
	suffixArrow
		ifNotNil: [(suffixArrow bounds containsPoint: evt cursorPoint)
				ifTrue: [self showSuffixChoices.
					^ self]].
	retractArrow
		ifNotNil: [(retractArrow bounds containsPoint: evt cursorPoint)
				ifTrue: [self deleteLastTwoTiles.
					^ self]].
	^ super mouseUp: evt! !

!TileMorph methodsFor: 'misc' stamp: 'sw 1/29/98 00:45'!
numericValue
	literal isNumber ifFalse: [^ 0].
	^ literal! !

!TileMorph methodsFor: 'accessing' stamp: ''!
operatorOrExpression

	^ operatorOrExpression
! !

!TileMorph methodsFor: 'accessing' stamp: 'tak 12/5/2004 01:07'!
options
	"Answer the options of the tile for an arrow"
	(type == #literal
			and: [literal isKindOf: Boolean])
		ifTrue: [^ {{true. false}. #('true' 'false' )}].
	operatorOrExpression
		ifNil: [^ nil].
	(ScriptingSystem arithmeticalOperatorsAndHelpStrings first includes: operatorOrExpression)
		ifTrue: [^ ScriptingSystem arithmeticalOperatorsAndHelpStrings].
	(ScriptingSystem numericComparitorsAndHelpStrings first includes: operatorOrExpression)
		ifTrue: [self receiverType = #Number
				ifTrue: [^ ScriptingSystem numericComparitorsAndHelpStrings]
				ifFalse: [^ #(#(#= #~=) #('equal' 'not equal') )]].
	^ nil! !

!TileMorph methodsFor: 'change reporting' stamp: 'ar 11/8/2000 23:46'!
ownerChanged
	super ownerChanged.
	(owner class == TilePadMorph and:[owner layoutPolicy isNil]) ifTrue:[
		owner layoutPolicy: TableLayout new.
		owner hResizing: #shrinkWrap.
		owner vResizing: #spaceFill.
	].! !

!TileMorph methodsFor: 'arrows' stamp: 'sw 1/4/2005 02:09'!
phraseForOp: op arg: arg resultType: resultType
	"Answer a numeric-valued phrase derived from the receiver, whose extension arrow has just been hit.  Pass along my float-precision."

	| phrase srcLabel distLabel |
	phrase _ self presenter
				phraseForReceiver: literal
				op: op
				arg: 1
				resultType: #Number.
	srcLabel _ self findA: UpdatingStringMorph.
	distLabel _ phrase submorphs first submorphs first findA: UpdatingStringMorph.
	distLabel floatPrecision: srcLabel floatPrecision.
	^ phrase! !

!TileMorph methodsFor: 'accessing' stamp: 'sw 2/15/2002 01:47'!
playerBearingCode
	"Answer the actual Player object who will be the 'self' when the receiver is being asked to generate code"

	self topEditor ifNotNilDo:
		[:anEditor | ^ anEditor playerScripted].
	(self nearestOwnerThat: [:m | m isAViewer]) 
		ifNotNilDo:
			[:aViewer | ^ aViewer scriptedPlayer].
	^ actualObject! !

!TileMorph methodsFor: 'commands' stamp: 'nk 9/9/2004 13:17'!
playerRepresented
	"Answer the player represented by the receiver. Morphs that serve as 
	references to other morphs reimplement this; by default a morph 
	represents its own player."
	^ type == #objRef
		ifTrue: [actualObject]
		ifFalse: [super playerRepresented]! !

!TileMorph methodsFor: 'code generation' stamp: ''!
precedingTileType
	"Return the slot reference type of the preceding TileMorph in my owner."

	| row i tile |
	row _ owner submorphs.
	i _ row indexOf: self.
	((i > 1) and: [(tile _ row at: i - 1) isKindOf: TileMorph])
		ifTrue: [^ tile type]
		ifFalse: [^ #unknown].
! !

!TileMorph methodsFor: 'mouse handling' stamp: 'sw 11/22/2000 10:50'!
presentOperatorAlternatives: evt
	"The receiver is a tile that represents an operator; a click on the receiver's label will pop up a menu of alternative operator choices"

	| result ops |

	((ops _ ScriptingSystem arithmeticalOperatorsAndHelpStrings first) includes: operatorOrExpression) ifFalse:
		[((ops _ ScriptingSystem numericComparitorsAndHelpStrings first) includes: operatorOrExpression)
			ifFalse: [^ self]].
		
	(result _ (SelectionMenu selections: ops) startUp) ifNotNil:
		[self setOperatorAndUseArrows: result asString.
		self scriptEdited]! !

!TileMorph methodsFor: 'printing' stamp: 'sw 4/28/1998 00:14'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' - ', type printString.
	operatorOrExpression ifNotNil: [aStream nextPutAll: ' op= ', operatorOrExpression printString].
	slotName ifNotNil: [aStream nextPutAll: ' op= ', slotName printString].! !

!TileMorph methodsFor: 'initialization' stamp: 'gm 2/24/2003 18:08'!
rawVocabulary: aVocabulary 
	"Set the receiver's vocabulary, without side effects."

	vocabularySymbol := (aVocabulary isKindOf: Symbol) 
				ifTrue: [aVocabulary]
				ifFalse: [aVocabulary vocabularyName]! !

!TileMorph methodsFor: 'accessing' stamp: 'tak 1/4/2005 08:53'!
receiverType
	owner
		ifNil: [^ nil].
	owner submorphs size > 0
		ifFalse: [^ nil].
	^ (owner submorphs first respondsTo: #type)
		ifTrue: [owner submorphs first type]! !

!TileMorph methodsFor: 'initialization' stamp: 'sw 9/28/2000 12:27'!
referTo: anObject
	"Make the receiver be an object-reference tile whose referent is anObject"

	type _ #objRef.
	typeColor _ ScriptingSystem colorForType: anObject basicType.
	actualObject _ anObject.
	self line1: anObject uniqueNameForReference! !

!TileMorph methodsFor: 'initialization' stamp: 'yo 4/4/2005 14:31'!
referToSimilarObject: anObject
	"Make the receiver be an object-reference tile whose referent is anObject"
	actualObject _ anObject.
! !

!TileMorph methodsFor: 'scripting' stamp: 'sw 1/21/98 17:44'!
restoreTypeColor
	self borderColor: Color black.
	typeColor ifNotNil: [self color: typeColor]! !

!TileMorph methodsFor: 'accessing' stamp: 'yo 6/18/2004 14:54'!
resultType
	"Answer the result type of the receiver"

	type == #literal 
		ifTrue: 
			[(literal isNumber) ifTrue: [^#Number].
			(literal isString) ifTrue: [^#String].
			(literal isKindOf: Boolean) ifTrue: [^#Boolean]].
	type == #expression ifTrue: [^#Number].
	type == #objRef ifTrue: [(actualObject costume renderedMorph isMemberOf: KedamaPatchMorph) ifTrue: [^ #Patch] ifFalse: [^#Player]].
	^#unknown! !

!TileMorph methodsFor: 'initialization' stamp: 'sw 3/3/2004 16:28'!
retargetFrom: oldPlayer to: newPlayer
	"Change the receiver so that if formerly it referred to oldPlayer, it refers to newPlayer instead"

	| newLabel |
	(type == #objRef  and: [actualObject == oldPlayer]) ifTrue:
		[actualObject _ newPlayer.
		newLabel _ actualObject externalName.
		self isPossessive ifTrue:
			[newLabel _ newLabel, '''s' translated].
		self line1: newLabel]! !

!TileMorph methodsFor: 'code generation' stamp: 'sw 9/26/97 10:51'!
scriptEdited
	"Tell the scriptEditor who I belong to that I have changed.  "
	| him |
	(him _ self outermostMorphThat: [:m| m isKindOf: ScriptEditorMorph])
		ifNotNil: [him scriptEdited]! !

!TileMorph methodsFor: 'macpal' stamp: 'sw 5/25/2000 15:52'!
scriptPerformer
	"Guard against obscure circumstance in which the tile itself has an associated Player, which then might be asked to interact in inappropriate ways with, for example, an UpdatingStringMorph to provide the literal for a RandomNumberTile.  This is at best a finger in the dike.  Still very unsatisfactory!!"
	^ self! !

!TileMorph methodsFor: 'tiles from method' stamp: 'tk 8/6/1999 16:00'!
selectorTile: msgNode in: aScriptor
	| sel selTile |
	"Make a selector (operator) tile"

	sel _ msgNode selector key.
	sel == #color:sees: ifTrue: [
		selTile _ (Viewer new) colorSeesPhrase submorphs second.	"ColorSeer tile"
		selTile colorSwatchColor: msgNode arguments first eval.
		^ selTile].
	^ self setOperator: sel! !

!TileMorph methodsFor: 'initialization' stamp: ''!
setExpression: exprString label: labelString

	type _ #expression.
	operatorOrExpression _ exprString.
	self line1: labelString.
! !

!TileMorph methodsFor: 'initialization' stamp: 'nk 8/30/2004 08:02'!
setLiteral: anObject
	"Set the receiver's literal to be anObject.  Create a readout morph and add it to the receiver, deleting any existing one that may be there."

	| m already |
	already _ submorphs detect: [:aSubMorph  | aSubMorph isKindOf: UpdatingStringMorph] ifNone: [nil].
	already ifNotNil: [already delete].
	type _ #literal.
	m _ UpdatingStringMorph contents: ' ' font: ScriptingSystem fontForTiles.
	m target: self; getSelector: #literal; putSelector: #literal:.
	(anObject isString or: [ anObject isText]) ifTrue: [m useStringFormat].
	self addMorphBack: m.
	self setLiteralInitially: anObject.
! !

!TileMorph methodsFor: 'misc' stamp: 'sw 9/15/2000 06:00'!
setLiteralInitially: anObject
	"Establish the initial literal.  Get the label correct, but do *not* send the value back to the target via the setter (unlike #literal:)"

	literal _ anObject.
	self updateLiteralLabel! !

!TileMorph methodsFor: 'initialization' stamp: 'sw 9/27/2001 04:16'!
setOperator: aString
	"Set the operator symbol from the string provided"

	self setOperator: aString andUseWording:  (self currentVocabulary tileWordingForSelector: aString)! !

!TileMorph methodsFor: 'initialization' stamp: 'sw 9/27/2001 04:09'!
setOperator: aString andUseWording: wording
	"Set the operator symbol from the string provided"

	type _ #operator.
	operatorOrExpression _ aString asSymbol.
 	self line1: wording.
	(ScriptingSystem doesOperatorWantArrows: operatorOrExpression)
		ifTrue: [self addArrows].
	self updateLiteralLabel

	"operatorOrExpression == #heading ifTrue: [self halt]."! !

!TileMorph methodsFor: 'initialization' stamp: 'sw 5/17/2001 12:17'!
setOperatorAndUseArrows: aString
	"Set the operator as per aString, and add up/down arrows"

	type _ #operator.
	operatorOrExpression _ aString asSymbol.
 	self line1: (self currentVocabulary tileWordingForSelector: operatorOrExpression).
	self addArrows; updateLiteralLabel.
	submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: operatorOrExpression)! !

!TileMorph methodsFor: 'initialization' stamp: 'sw 9/13/2001 21:58'!
setSlotRefOperator: getter
	"getter represents the name of a slot that the receiver is to represent; configure the receiver to serve thi duty, and set upthe wording on the tile appropriately"

	type _ #operator.
	operatorOrExpression _ getter asSymbol.
	self line1:  (self currentEToyVocabulary tileWordingForSelector: operatorOrExpression).
	self updateLiteralLabel! !

!TileMorph methodsFor: 'initialization' stamp: 'dgd 8/26/2004 12:04'!
setToReferTo: anObject
	"Set the receiver to bear an object reference to the given object."
	self flag: #yo.
	type _ #objRef.
	actualObject _ anObject.
	self line1: anObject nameForViewer.
	self typeColor: (ScriptingSystem colorForType: #Player).
	self enforceTileColorPolicy
! !

!TileMorph methodsFor: 'mouse handling' stamp: 'yo 3/14/2005 10:27'!
showOptions
	"The receiver is a tile that represents an operator; a click on the 
	receiver's label will pop up a menu of alternative operator choices"
	| result menuChoices word |
	menuChoices _ (self options first collect: [:each | each asString translated]) collect: [:each | 
							word := self currentVocabulary translatedWordingFor: each asSymbol.
							word isEmpty
								ifTrue: ['<-']
								ifFalse: [word]].
	result _ (SelectionMenu labelList: menuChoices lines: nil selections: self options first) startUp.
	result 
		ifNotNil: [self value: result.
			self scriptEdited]! !

!TileMorph methodsFor: 'arrows' stamp: 'tak 12/5/2004 15:33'!
showSuffixChoices
	"The suffix arrow has been hit, so respond appropriately"

	| plusPhrase phrase pad outer num |
	(phrase _ self ownerThatIsA: PhraseTileMorph) ifNil: [^ self].

	(type == #literal) & (literal isNumber) ifTrue: ["Tile is a constant number"
		phrase lastSubmorph == owner "pad"
			ifTrue: ["we are adding the first time (at end of our phrase)"
				plusPhrase _ self phraseForOp: #+ arg: 1 resultType: #Number.
				plusPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: #+).
				owner acceptDroppingMorph: plusPhrase event: self primaryHand lastEvent.
				num _ plusPhrase firstSubmorph firstSubmorph.
				num deleteSuffixArrow]].

	type == #operator ifTrue: ["Tile is accessor of an expression"
		phrase resultType == #Number ifTrue:
			[outer _ phrase ownerThatIsA: PhraseTileMorph.
			pad _ self ownerThatIsA: TilePadMorph.
			outer ifNotNil:
				[outer lastSubmorph == pad ifTrue: [ "first time"
					plusPhrase _ self presenter phraseForReceiver: 1 
							op: #+ arg: 1 resultType: #Number.
					plusPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: #+).
					pad acceptDroppingMorph: plusPhrase event: self primaryHand lastEvent.
					plusPhrase firstSubmorph removeAllMorphs; addMorph: phrase.	"car's heading"
					self deleteSuffixArrow.
					pad topEditor install "recompile"]]]].

	(phrase topEditor ifNil: [phrase]) enforceTileColorPolicy! !

!TileMorph methodsFor: 'accessing' stamp: 'sw 11/9/1998 14:18'!
slotName
	"Caution: the slotName is not in use!!"
	^ slotName
! !

!TileMorph methodsFor: 'misc' stamp: 'sw 1/28/2005 00:54'!
soundChoices
	"Answer a list of sound choices.  This applies only to tiles that have sound-names as their literals, viz. SoundTiles and SoundReadoutTiles."

	| aList |
	aList _ SoundService default sampledSoundChoices asOrderedCollection.
	aList removeAllFoundIn: (ScriptingSystem soundNamesToSuppress copyWithout: literal).
	^ aList asSortedArray! !

!TileMorph methodsFor: 'code generation' stamp: 'yo 4/4/2005 16:14'!
storeCodeOn: aStream indent: tabCount 
	"Store code representing the receiver onto the stream, with the given amount of indentation"

	| op playerBearingCode |
	playerBearingCode := self playerBearingCode.	"Must determine whom is scripted for what follows to work; if it's ever nil, we've got trouble"
	type = #expression 
		ifTrue: 
			[^aStream
				nextPut: $(;
				nextPutAll: operatorOrExpression;
				nextPut: $)].
	type = #literal 
		ifTrue: 
			[^aStream
				nextPut: $(;
				nextPutAll: literal printString;
				nextPut: $)].
	type == #objRef 
		ifTrue: 
			[^playerBearingCode == actualObject 
				ifTrue: 
					["If the object is the method's own 'self' then we MUST, rather than just MAY, put out 'self' rather than the referencer call, though the latter will temporarily work if only one instance of the uniclass exists."

					aStream nextPutAll: 'self']
				ifFalse: 
					[(actualObject isPlayerLike and: [actualObject isSequentialStub]) ifTrue: [
						actualObject storeCodeOn: aStream indent: tabCount.
					] ifFalse: [
						 Preferences capitalizedReferences 
						ifTrue: 
							["Global dictionary References"

							self flag: #deferred.	"Start deploying the meesage-receiver hints soon"
							aStream nextPutAll: actualObject uniqueNameForReference]
						ifFalse: 
							["old class-inst-var-based scheme used  Feb 1998 to Oct 2000, and indeed
						ongoing in school year 2000-01 at the open school"

							aStream nextPutAll: 'self class '.
							aStream 
								nextPutAll: (playerBearingCode class referenceSelectorFor: actualObject)]]]].
	type = #operator 
		ifTrue: 
			[op := ((UpdatingOperators includesKey: operatorOrExpression) 
				and: [self precedingTileType = #slotRef]) 
					ifTrue: [UpdatingOperators at: operatorOrExpression]
					ifFalse: [operatorOrExpression].
			^op isEmpty 
				ifTrue: [aStream position: aStream position - 1]
				ifFalse: [aStream nextPutAll: op]]

	"The following branch has long been disused
	type = #slotRef ifTrue:
		[self isThisEverCalled.
		refType _ self slotRefType.
		refType = #get ifTrue:
			[^ aStream
				nextPutAll: targetName;
				space;
				nextPutAll: (Utilities getterSelectorFor: slotName)].
		refType = #set ifTrue:
			[^ aStream
				nextPutAll: targetName;
				space;
				nextPutAll: (Utilities setterSelectorFor: slotName);
				nextPut: $:].
		refType = #update ifTrue:
			[^ aStream
				nextPutAll: targetName;
				space;
				nextPutAll: slotName;
				nextPutAll: ': ';
				nextPutAll: targetName;
				space;
				nextPutAll: slotName]]"! !

!TileMorph methodsFor: 'private' stamp: 'tk 8/14/2000 13:32'!
test
	| pos hh |
	"Set the position of all my submorphs.  Compute my bounds.  Caller must call layoutChanged or set fullBounds to nil."

	fullBounds ifNil: [
		pos _ self topLeft.
		self submorphsDo: [:sub | 
			hh _ (self class defaultH - sub height) // 2.	"center in Y"
			sub privateBounds: (pos + (2@hh) extent: sub extent).
			pos x: (sub right min: 1200)].	"2 pixels spacing on left"
		bounds _ bounds topLeft corner: pos + (2 @ self class defaultH).
		fullBounds _ bounds.
		].
	owner class == TilePadMorph ifTrue: [owner bounds: bounds].
	^ fullBounds! !

!TileMorph methodsFor: 'accessing' stamp: ''!
type

	^ type
! !

!TileMorph methodsFor: 'misc' stamp: 'sw 8/28/2004 15:20'!
typeColor: aColor
	"Set the receiver's typeColor"

	borderColor _ ScriptingSystem standardTileBorderColor.
	typeColor _ aColor.
	color _ ScriptingSystem uniformTileInteriorColor ! !

!TileMorph methodsFor: 'player viewer' stamp: 'sw 9/13/2001 21:58'!
updateLiteralLabel
	"Update the wording emblazoned on the tile, if needed"

	| myLabel |
	(myLabel _ self labelMorph) ifNil: [^ self].

	myLabel acceptValue:
		(type == #literal
			ifTrue:
				[literal] 
			ifFalse: [operatorReadoutString 
				ifNil:		[self currentEToyVocabulary tileWordingForSelector: operatorOrExpression]
				ifNotNil:  	[operatorReadoutString]]).
	self changed.! !

!TileMorph methodsFor: 'initialization' stamp: 'yo 7/2/2004 21:44'!
updateWordingToMatchVocabulary
	"The current vocabulary has changed; change the wording on my face, if appropriate"

	| aMethodInterface |
	type == #operator ifTrue:
		[self line1: (self currentVocabulary tileWordingForSelector: operatorOrExpression).
		(ScriptingSystem doesOperatorWantArrows: operatorOrExpression)
			ifTrue: [self addArrows].
		self updateLiteralLabel.

		aMethodInterface _ self currentVocabulary methodInterfaceAt: operatorOrExpression
			ifAbsent: [
				Vocabulary eToyVocabulary
					methodInterfaceAt: operatorOrExpression ifAbsent: [^ self]].
		self setBalloonText: aMethodInterface documentation.
	].

	type == #objRef ifTrue: [
		self isPossessive
			ifTrue: [self bePossessive]
			ifFalse: [self labelMorph contents: self actualObject nameForViewer asSymbol translated]].

		"submorphs last setBalloonText: aMethodInterface documentation"! !

!TileMorph methodsFor: 'accessing' stamp: 'sw 7/4/2004 01:09'!
updatingStringMorph
	"If the receiver has an updatingStringMorph as a submorph, answer it, else answer nil"
	
	^ submorphs detect: [:m | m isKindOf: UpdatingStringMorph] ifNone: [nil]! !

!TileMorph methodsFor: 'scripting' stamp: 'sw 5/2/1998 15:01'!
useUniformTileColor
	self color: ScriptingSystem uniformTileInteriorColor! !

!TileMorph methodsFor: 'accessing' stamp: 'tak 12/5/2004 00:30'!
value
	^ type == #literal
		ifTrue: [literal]
		ifFalse: [type == #objRef
				ifTrue: [actualObject]
				ifFalse: [operatorOrExpression]]! !

!TileMorph methodsFor: 'accessing' stamp: 'sw 1/12/2005 12:35'!
value: anObject
	"Set the receiver's 'value'.  For a literal tile, this is the literal itself; for operator tiles it is the operator.  Recompile any enclosing script."

	type == #literal
		ifTrue: [self literal: anObject]
		ifFalse: [self setOperatorAndUseArrows: anObject asString].
	self scriptEdited.
	self layoutChanged! !

!TileMorph methodsFor: 'arrows' stamp: 'ar 2/12/2001 13:55'!
variableDelay: aBlock

	| now delay dt |
	(self hasProperty: #inVariableDelay) ifTrue:[^self].
	nArrowTicks ifNil: [nArrowTicks _ 1].
	now _ Time millisecondClockValue.
	aBlock value.
	delay _ nArrowTicks > 5 ifTrue: [100] ifFalse: [300].
	nArrowTicks _ nArrowTicks + 1.
	dt _ Time millisecondClockValue - now max: 0.  "Time it took to do."
	dt < delay ifTrue: [
		self setProperty: #inVariableDelay toValue: true.
		self addAlarm: #removeProperty: withArguments: #(inVariableDelay) after: (delay - dt)].
! !

!TileMorph methodsFor: 'copying' stamp: 'tk 1/7/1999 17:15'!
veryDeepFixupWith: deepCopier
	"If target and arguments fields were weakly copied, fix them here.  If they were in the tree being copied, fix them up, otherwise point to the originals!!!!"

super veryDeepFixupWith: deepCopier.
actualObject _ deepCopier references at: actualObject ifAbsent: [actualObject].! !

!TileMorph methodsFor: 'copying' stamp: 'sw 6/4/2001 16:45'!
veryDeepInner: deepCopier
	"Copy all of my instance variables.  Some need to be not copied at all, but shared.  	Warning!!!!  Every instance variable defined in this class must be handled.  We must also implement veryDeepFixupWith:.  See DeepCopier class comment."

super veryDeepInner: deepCopier.
type _ type veryDeepCopyWith: deepCopier.
slotName _ slotName veryDeepCopyWith: deepCopier.
literal _ literal veryDeepCopyWith: deepCopier.
operatorOrExpression _ operatorOrExpression veryDeepCopyWith: deepCopier.
"actualObject _ actualObject.		Weakly copied"
downArrow _ downArrow veryDeepCopyWith: deepCopier.
upArrow _ upArrow veryDeepCopyWith: deepCopier.
suffixArrow _ suffixArrow veryDeepCopyWith: deepCopier.
typeColor _ typeColor veryDeepCopyWith: deepCopier.
lastArrowTick _ lastArrowTick veryDeepCopyWith: deepCopier.
nArrowTicks _ nArrowTicks veryDeepCopyWith: deepCopier.
operatorReadoutString _ operatorReadoutString veryDeepCopyWith: deepCopier.
possessive _ possessive veryDeepCopyWith: deepCopier.
retractArrow _ retractArrow veryDeepCopyWith: deepCopier.
vocabularySymbol _ vocabularySymbol.  "Weakly copied"
vocabulary _ nil.   "obsolete - clobbered"! !

!TileMorph methodsFor: 'event handling' stamp: 'tak 12/5/2004 01:30'!
wantsKeyboardFocusFor: aSubmorph 
	^ type == #literal
		and: [(literal isKindOf: Boolean) not]! !

TileMorph subclass: #UndescribedTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!UndescribedTile methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:45'!
initialize
	"Initialize the receiver. This is in its infancy -- just a place-holder 
	at present"
	super initialize.
""
	self extent: 20 @ 22.
	self setLiteral: 'arg'! !

RectangleMorph subclass: #UpdatingRectangleMorph
	instanceVariableNames: 'target lastValue getSelector putSelector contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Support'!

!UpdatingRectangleMorph commentStamp: '<historical>' prior: 0!
Intended for use as a color swatch coupled to a color obtained from the target, but made just slightly more general than that.!

UpdatingRectangleMorph subclass: #ColorSwatch
	instanceVariableNames: 'argument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Support'!

!ColorSwatch methodsFor: 'as yet unclassified' stamp: 'sw 7/13/1999 13:39'!
argument: arg
	argument _ arg! !

!ColorSwatch methodsFor: 'target access' stamp: 'dgd 2/22/2003 13:32'!
readFromTarget
	"Obtain a value from the target and set it into my lastValue"

	| v |
	(target isNil or: [getSelector isNil]) ifTrue: [^contents].
	v := target perform: getSelector with: argument.
	lastValue := v.
	^v! !

!ColorSwatch methodsFor: 'setting' stamp: 'sw 3/23/2001 12:12'!
setTargetColor: aColor
	"Set the target color as indicated"

	putSelector ifNotNil:
		[self color: aColor.
		contents _ aColor.
		target perform: self putSelector withArguments: (Array with: argument with: aColor)]
! !

!ColorSwatch methodsFor: 'testing' stamp: 'sw 7/13/1999 18:39'!
stepTime
	^ 1000! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: 'sw 9/4/97 21:43'!
contents
	^ contents! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: 'sw 9/4/97 21:43'!
contents: c
	contents _ c! !

!UpdatingRectangleMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:35'!
defaultBorderColor
	"answer the default border color/fill style for the receiver"
	^ Color lightGray lighter! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: 'sw 1/6/2005 16:31'!
getSelector
	"Answer the getSelector"

	^ getSelector! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: ''!
getSelector: aSymbol

	getSelector _ aSymbol.
! !

!UpdatingRectangleMorph methodsFor: 'event handling' stamp: 'dgd 2/22/2003 18:45'!
handlesMouseDown: evt 
	^putSelector notNil! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: 'sw 1/6/2005 01:28'!
isEtoyReadout
	"Answer whether the receiver can serve as an etoy readout"

	^ true! !

!UpdatingRectangleMorph methodsFor: 'event handling' stamp: 'ar 10/5/2000 18:54'!
mouseUp: evt

	self changeColorTarget: self selector: #setTargetColor: originalColor: color hand: evt hand.! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: 'sw 10/30/97 00:55'!
putSelector
	^ putSelector! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: 'sw 10/30/97 00:55'!
putSelector: aSymbol
	putSelector _ aSymbol! !

!UpdatingRectangleMorph methodsFor: 'target access' stamp: 'dgd 2/22/2003 14:40'!
readFromTarget
	"Read the color value from my target"

	| v |
	(target isNil or: [getSelector isNil]) ifTrue: [^contents].
	target isMorph ifTrue: [target isInWorld ifFalse: [^contents]].
	v := self valueProvider perform: getSelector.
	lastValue := v.
	^v! !

!UpdatingRectangleMorph methodsFor: 'setting' stamp: 'sw 3/23/2001 23:26'!
setTargetColor: aColor
	"Set my target's color as indicated"

	putSelector ifNotNil:
		[self color: aColor.
		contents _ aColor.
		self valueProvider perform: self putSelector withArguments: (Array with: aColor)]
! !

!UpdatingRectangleMorph methodsFor: 'stepping and presenter' stamp: 'sw 7/15/1999 07:27'!
step
	| s |
	super step.
	s _ self readFromTarget.
	s = contents ifFalse:
		[self contents: s.
		self color: s]
! !

!UpdatingRectangleMorph methodsFor: 'testing' stamp: ''!
stepTime

	^ 50! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: ''!
target

	^ target
! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: ''!
target: anObject

	target _ anObject.
! !

!UpdatingRectangleMorph methodsFor: 'accessing' stamp: 'sw 11/15/2001 16:22'!
userEditsAllowed
	"Answer whether it is suitable for a user to change the value represented by this readout"

	^ putSelector notNil! !

!UpdatingRectangleMorph methodsFor: 'setting' stamp: 'sw 3/23/2001 13:24'!
valueProvider
	"Answer the object to which my get/set messages should be sent.  This is inefficient and contorted in order to support grandfathered content for an earlier design"

	^ target isMorph
		ifTrue:
			[target topRendererOrSelf player ifNil: [target]]
		ifFalse:
			[target]! !

!UpdatingRectangleMorph methodsFor: 'copying' stamp: 'tk 1/7/1999 17:17'!
veryDeepFixupWith: deepCopier
	"If target and arguments fields were weakly copied, fix them here.  If they were in the tree being copied, fix them up, otherwise point to the originals!!!!"

super veryDeepFixupWith: deepCopier.
target _ deepCopier references at: target ifAbsent: [target].! !

!UpdatingRectangleMorph methodsFor: 'copying' stamp: 'tk 1/7/1999 17:17'!
veryDeepInner: deepCopier
	"Copy all of my instance variables.  Some need to be not copied at all, but shared.  	Warning!!!!  Every instance variable defined in this class must be handled.  We must also implement veryDeepFixupWith:.  See DeepCopier class comment."

super veryDeepInner: deepCopier.
"target _ target.		Weakly copied"
lastValue _ lastValue veryDeepCopyWith: deepCopier.
"getSelector _ getSelector.		a Symbol"
"putSelector _ putSelector.		a Symbol"
contents _ contents veryDeepCopyWith: deepCopier.! !

!BraceNode methodsFor: '*eToys-tiles' stamp: 'di 11/13/2000 21:17'!
asMorphicSyntaxIn: parent

	| row |

	row _ (parent addRow: #brace on: self) layoutInset: 1.
	row addMorphBack: (StringMorph new contents: 
		(String streamContents: [:aStream | self printOn: aStream indent: 0])).
	^row
! !

!TabbedPalette methodsFor: '*eToys-e-toy support' stamp: 'sw 7/5/1998 15:40'!
succeededInRevealing: aPlayer
	| result |
	result _ super succeededInRevealing: aPlayer.
	result ifTrue:
		["BookMorph code will have called goToPageNumber:; here, we just need to get the tab selection right here"
		self selectTabOfBook: self currentPalette].
	^ result! !

!TabbedPalette methodsFor: '*eToys-viewer tab' stamp: 'sw 3/3/1999 13:17'!
viewMorph: aMorph
	"The receiver is expected to have a viewer tab; select it, and target it to aMorph"
	| aPlayer aViewer oldOwner |
	((currentPage isKindOf: Viewer) and: [currentPage scriptedPlayer == aMorph player])
		ifTrue:
			[^ self].
	oldOwner _ owner.
	self delete.
	self visible: false.
	aPlayer _ aMorph assuredPlayer.
	self showNoPalette.
	aViewer _  StandardViewer new initializeFor: aPlayer barHeight: 0.
	aViewer enforceTileColorPolicy.
	self showNoPalette.
	currentPage ifNotNil: [currentPage delete].
	self addMorphBack: (currentPage _ aViewer beSticky).
	self snapToEdgeIfAppropriate.
	tabsMorph highlightTab: nil.
	self visible: true.
	oldOwner addMorphFront: self.
	self world startSteppingSubmorphsOf: aViewer.
	self layoutChanged! !

!String methodsFor: '*eToys-*Morphic' stamp: 'ar 4/10/2005 17:06'!
newTileMorphRepresentative
	^ TileMorph new setLiteral: self;addSuffixIfCan! !

!SketchMorph classSide methodsFor: '*eToys-scripting' stamp: 'sw 7/14/2004 21:11'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((graphics (
(slot graphic 	'The picture currently being worn' Graphic	 readWrite Player getGraphic Player setGraphic:)
(command wearCostumeOf: 'wear the costume of...' Player)
(slot baseGraphic 	'The picture originally painted for this object, but can subsequently be changed via menu or script' Graphic	 readWrite Player getBaseGraphic Player setBaseGraphic:)
(command restoreBaseGraphic 'Make my picture be the one I remember in my baseGraphic')

(slot rotationStyle 'How the picture should change when the heading is modified' RotationStyle readWrite Player getRotationStyle Player setRotationStyle:)
)))


! !

!SketchMorph classSide methodsFor: '*eToys-instance creation' stamp: 'sw 6/13/2001 22:50'!
exampleBackgroundSketch
	"Answer an instance suitable for serving as a prototype for a background-field incarnation of a sketch"


	| aSketch |
	aSketch _ self newSticky form: (ScriptingSystem formAtKey: #squeakyMouse).
	aSketch setProperty: #shared toValue: true.
	aSketch setProperty: #holdsSeparateDataForEachInstance toValue: true.
	^ aSketch
! !

!SketchMorph methodsFor: '*eToys-e-toy support' stamp: 'nk 6/12/2004 10:04'!
acquirePlayerSimilarTo: aSketchMorphsPlayer
	"Retrofit into the receiver a player derived from the existing scripted player of a different morph.  Works only between SketchMorphs. Maddeningly complicated by potential for transformations or native sketch-morph scaling in donor or receiver or both"

	| myName myTop itsTop newTop newSketch |
	myTop _ self topRendererOrSelf.
	aSketchMorphsPlayer belongsToUniClass ifFalse: [^ Beeper beep].
	itsTop _ aSketchMorphsPlayer costume.
	(itsTop renderedMorph isSketchMorph)
		ifFalse:	[^ Beeper beep].

	newTop _ itsTop veryDeepCopy.  "May be a sketch or a tranformation"
	myName _ myTop externalName.  "Snag before the replacement is added to the world, because otherwise that could affect this"

	newSketch _ newTop renderedMorph.
	newSketch form: self form.
	newSketch scalePoint: self scalePoint.
	newSketch bounds: self bounds.
	myTop owner addMorph: newTop after: myTop.

	newTop heading ~= myTop heading ifTrue:
		"avoids annoying round-off error in what follows"
			[newTop player setHeading: myTop heading]. 
	(newTop isFlexMorph and: [myTop == self])
		ifTrue:
			[newTop removeFlexShell].
	newTop _ newSketch topRendererOrSelf.
	newTop bounds: self bounds.
	(newTop isFlexMorph and:[myTop isFlexMorph]) ifTrue:[
		"Note: This completely dumps the above #bounds: information.
		We need to recompute the bounds based on the transform."
		newTop transform: myTop transform copy.
		newTop computeBounds].
	newTop setNameTo: myName.
	newTop player class bringScriptsUpToDate.
	myTop delete! !

!SketchMorph methodsFor: '*eToys-e-toy support' stamp: 'RAA 1/13/2001 11:38'!
appearsToBeSameCostumeAs: aMorph

	(aMorph isKindOf: self class) ifFalse: [^false].
	^originalForm == aMorph form or: [
		originalForm appearsToBeSameCostumeAs: aMorph form
	]! !

!SketchMorph methodsFor: '*eToys-e-toy support' stamp: 'nk 6/12/2004 10:04'!
asWearableCostume
	"Return a wearable costume for some player"
	^(World drawingClass withForm: originalForm) copyCostumeStateFrom: self! !

!SketchMorph methodsFor: '*eToys-card in a stack' stamp: 'sw 10/30/2000 10:29'!
couldHoldSeparateDataForEachInstance
	"Answer whether this type of morph is inherently capable of holding separate data for each instance ('card data')  SketchMorphs answer true, because they can serve as 'picture-holding' fields"

	^ true! !

!SketchMorph methodsFor: '*eToys-player' stamp: 'sw 10/30/2000 10:28'!
currentDataValue
	"Answer the object which bears the current datum for the receiver"

	^ originalForm! !

!SketchMorph methodsFor: '*eToys-geometry eToy' stamp: 'ar 9/22/2000 21:19'!
heading: newHeading
	"If not rotating normally, change forward direction rather than heading"
	rotationStyle == #normal ifTrue:[^super heading: newHeading].
	self isFlexed
		ifTrue:[self forwardDirection: newHeading - owner rotationDegrees]
		ifFalse:[self forwardDirection: newHeading].
	self layoutChanged! !

!SketchMorph methodsFor: '*eToys-player' stamp: 'sw 10/25/2000 07:01'!
variableDocks
	"Answer a list of VariableDock objects for docking up my data with an instance held in my containing playfield"

	^ Array with: (VariableDock new variableName: self defaultVariableName  type: #form definingMorph: self morphGetSelector: #form morphPutSelector: #setNewFormFrom:)! !

!SketchMorph methodsFor: '*eToys-accessing' stamp: 'RAA 3/15/2001 09:11'!
wearCostume: anotherMorph

	self form: anotherMorph form.
! !

SketchMorph subclass: #StickySketchMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Support'!

!StickySketchMorph methodsFor: 'e-toy support' stamp: 'sw 4/16/1998 13:44'!
mustBeBackmost
	^ true! !

!StickySketchMorph methodsFor: 'thumbnail' stamp: 'sw 6/16/1999 11:32'!
permitsThumbnailing
	^ false! !

!StickySketchMorph methodsFor: 'halos and balloon help' stamp: 'sw 9/18/97 15:37'!
wantsHalo
	^ false! !

!LiteralNode methodsFor: '*eToys-tiles' stamp: 'tk 8/24/2001 15:43'!
asMorphicSyntaxIn: parent

	| row |

	row _ parent addRow: #literal on: self.
	(key isVariableBinding) ifFalse: [
		row layoutInset: 1.
		^ row addMorphBack: (row addString: key storeString special: false)].
	key key isNil ifTrue: [
		^ row addTextRow: ('###',key value soleInstance name)
	] ifFalse: [
		^ row addTextRow: ('##', key key)
	].	! !

!LiteralNode methodsFor: '*eToys-tiles' stamp: 'ar 8/16/2001 13:27'!
explanation

	(key isVariableBinding) ifFalse: [
		^'Literal ', key storeString
	].
	key key isNil ifTrue: [
		^'Literal ', ('###',key value soleInstance name) 
	] ifFalse: [
		^'Literal ', ('##', key key) 
	].	! !

MethodInterface subclass: #MethodWithInterface
	instanceVariableNames: 'playerClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!MethodWithInterface commentStamp: '<historical>' prior: 0!
A MethodInterface bound to an actual class.

	selector					A symbol - the selector being described
	argumentSpecifications	A list of specifications for the formal arguments of the method
	resultSpecification 		A characterization of the return value of the method
	userLevel				
	attributeKeywords		A list of symbols, comprising keywords that the user wishes to
								associate with this method
	defaultStatus			The status to apply to new instances of the class by default
	defaultFiresPerTick		How many fires per tick, by default, should be allowed if ticking.
	playerClass				The actual class with which this script is associated!

!MethodWithInterface methodsFor: 'script editor' stamp: 'sw 3/10/2003 23:58'!
allScriptActivationButtons
	"Answer all the script-activation buttons that exist for this interface"

	^ ScriptActivationButton allInstances select: 
		[:aButton | aButton uniclassScript == self]! !

!MethodWithInterface methodsFor: 'script editor' stamp: 'sw 3/28/2001 16:26'!
allScriptEditors
	"Answer all the script editors that exist for the class and selector of this interface"

	^ ScriptEditorMorph allInstances select: 
		[:aScriptEditor | aScriptEditor playerScripted class == self playerClass and:
			[aScriptEditor scriptName == selector]]! !

!MethodWithInterface methodsFor: 'updating' stamp: 'sw 3/28/2001 16:26'!
bringUpToDate
	"Bring all scriptors related to this method up to date.  Note that this will not change the senders of this method if the selector changed -- that's something still ahead."

	(ScriptEditorMorph allInstances select:
		[:m | (m playerScripted isMemberOf: self playerClass) and: [m scriptName == selector]])
			do:
				[:m | m bringUpToDate]! !

!MethodWithInterface methodsFor: 'initialization' stamp: 'sw 1/30/2001 11:37'!
convertFromUserScript: aUserScript
	"The argument represents an old UserScript object.  convert it over"

	defaultStatus _ aUserScript status.! !

!MethodWithInterface methodsFor: 'script editor' stamp: 'sw 2/17/2001 03:28'!
currentScriptEditor: anEditor
	"Set the receiver's currentScriptEditor as indicated, if I care.  MethodWithInterface does not care, since it does not hold on to a ScriptEditor.  A subclass of mine, however does, or did, care"! !

!MethodWithInterface methodsFor: 'initialization' stamp: 'sw 1/26/2001 16:44'!
initialize
	"Initialize the receiver by setting its inst vars to default values"

	super initialize.
	defaultStatus _ #normal! !

!MethodWithInterface methodsFor: 'script editor' stamp: 'sw 3/28/2001 16:26'!
instantiatedScriptEditorForPlayer: aPlayer
	"Return a new script editor for the player and selector"

	| aScriptEditor |
	aScriptEditor _ (self playerClass includesSelector: selector) 
			ifTrue: [ScriptEditorMorph new 
				fromExistingMethod: selector 
				forPlayer: aPlayer]
			ifFalse: [ScriptEditorMorph new
				setMorph: aPlayer costume
				scriptName: selector].
		defaultStatus == #ticking ifTrue:
			[aPlayer costume arrangeToStartStepping].
	
	^ aScriptEditor! !

!MethodWithInterface methodsFor: 'initialization' stamp: 'sw 2/20/2001 03:29'!
isTextuallyCoded
	"Answer whether the receiver is in a textually-coded state.  A leftover from much earlier times, this is a vacuous backstop"

	^ false! !

!MethodWithInterface methodsFor: 'rename' stamp: 'sw 2/17/2001 04:10'!
okayToRename
	"Answer whether the receiver is in a state to be renamed."

	^ true! !

!MethodWithInterface methodsFor: 'access' stamp: 'sw 3/28/2001 16:25'!
playerClass
	"Answer the playerClass associated with the receiver.  Note: fixes up cases where the playerClass slot was a Playerxxx object because of an earlier bug"

	^ (playerClass isKindOf: Class)
		ifTrue:
			[playerClass]
		ifFalse:
			[playerClass _ playerClass class]! !

!MethodWithInterface methodsFor: 'initialization' stamp: 'sw 9/12/2001 11:59'!
playerClass: aPlayerClass selector: aSelector
	"Set the playerClass and selector of the receiver"

	playerClass _ aPlayerClass.
	selector _ aSelector.! !

!MethodWithInterface methodsFor: 'script editor' stamp: 'sw 7/28/2001 01:00'!
recompileScriptFromTilesUnlessTextuallyCoded
	"Recompile Script From Tiles Unless Textually Coded.  For the universal-tiles MethodWithInterface case, this is moot.  Used only in support of a reintegration of Open-school forked projects from Sept 2000 in 7/01"! !

!MethodWithInterface methodsFor: 'rename' stamp: 'sw 3/11/2003 00:01'!
renameScript: newSelector fromPlayer: aPlayer
	"The receiver's selector has changed to the new selector.  Get various things right, including the physical appearance of any Scriptor open on this method"

	self allScriptEditors do:
		[:aScriptEditor | aScriptEditor renameScriptTo: newSelector].

	(selector numArgs = 0 and: [newSelector numArgs = 1])
		ifTrue:
			[self argumentVariables: (OrderedCollection with:
				(Variable new name: #parameter type: #Number))].
	(selector numArgs = 1 and: [newSelector numArgs = 0])
		ifTrue:
			[self argumentVariables: OrderedCollection new].

	selector _ newSelector asSymbol.
	self bringUpToDate.
	self playerClass atSelector: selector putScript: self.
	self allScriptActivationButtons do:
		[:aButton | aButton bringUpToDate].

! !

!MethodWithInterface methodsFor: 'updating' stamp: 'sw 2/20/2001 03:43'!
revertToLastSavedTileVersionFor: anEditor
	"revert to the last saved tile version.  Only for universal tiles."

	anEditor removeAllButFirstSubmorph.
	anEditor insertUniversalTiles.
	anEditor showingMethodPane: false! !

!MethodWithInterface methodsFor: 'updating' stamp: 'sw 2/20/2001 03:41'!
saveScriptVersion: timeStamp
	"Save the tile script version if I do that sort of thing"! !

!MethodWithInterface methodsFor: 'initialization' stamp: 'nk 7/2/2004 07:18'!
status
	^defaultStatus
! !

MethodWithInterface subclass: #UniclassScript
	instanceVariableNames: 'currentScriptEditor formerScriptingTiles isTextuallyCoded lastSourceString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!UniclassScript commentStamp: '<historical>' prior: 0!
Represents a tile script of uniclass.  Holds the ScriptEditorMorph structures for the current version of a user-defined tile script, as well as previous versions thereof.

In addition to the instance variables of my superclass, my instance variables are:

currentScriptEditor		The current version of the ScriptEditorMorph for the script
formerScriptingTiles		A collection of pairs, (<timeStamp>  (list of morphs)) 
							each pair characterizing a prior tile version
isTextuallyCoded			A boolean.  If true, then a hand-crafted user coding supersedes
							the tale of the tiles.  This architecture is in transition, perhaps.!

!UniclassScript methodsFor: 'textually coded' stamp: 'sw 3/28/2002 00:39'!
becomeTextuallyCoded
	"Transform the receiver into one which is textually coded"

	isTextuallyCoded _ true.
	lastSourceString _ (playerClass compiledMethodAt: selector) decompileString 		"Save this to compare when going back to tiles"! !

!UniclassScript methodsFor: 'updating' stamp: 'sw 10/17/2001 09:46'!
bringUpToDate
	"Bring all versions of the receiver's tile-script source up to date"

	currentScriptEditor ifNotNil:
		[currentScriptEditor bringTileScriptingElementsUpToDate].
	formerScriptingTiles isEmptyOrNil ifFalse:
		[formerScriptingTiles do:
			[:aPair | aPair second do:
				[:aMorph | aMorph bringTileScriptingElementsUpToDate]]]! !

!UniclassScript methodsFor: 'initialization' stamp: 'sw 1/30/2001 11:37'!
convertFromUserScript: aUserScript
	"The argument represents an old UserScript object.  convert it over"

	defaultStatus _ aUserScript status.
	isTextuallyCoded _ aUserScript isTextuallyCoded.
	currentScriptEditor _ aUserScript currentScriptEditor.
	formerScriptingTiles _  aUserScript formerScriptEditors ifNotNil:
		[aUserScript formerScriptEditors collect:
			[:aScriptEditor |
				Array with: aScriptEditor timeStamp with: aScriptEditor submorphs allButFirst]]! !

!UniclassScript methodsFor: 'script editor' stamp: 'sw 7/28/2004 18:32'!
currentScriptEditor
	"Answer the currentScriptEditor"

	^ currentScriptEditor! !

!UniclassScript methodsFor: 'script editor' stamp: 'sw 1/23/2001 17:12'!
currentScriptEditor: anEditor
	"Set the receiver's currentScriptEditor as indicated"

	currentScriptEditor _ anEditor! !

!UniclassScript methodsFor: 'script editor' stamp: 'sw 7/22/2002 17:44'!
currentScriptEditorDo: aBlock
	"Evaluate a block on behalf of my current script editor, if any"

	currentScriptEditor ifNotNil:
		[aBlock value: currentScriptEditor]! !

!UniclassScript methodsFor: 'initialization' stamp: 'sw 1/26/2001 16:44'!
initialize
	"Initialize the receiver by setting its inst vars to default values"

	super initialize.
	isTextuallyCoded _ false

! !

!UniclassScript methodsFor: 'script editor' stamp: 'sw 7/20/2002 14:27'!
instantiatedScriptEditorForPlayer: aPlayer
	"Return the current script editor, creating it if necessary"

	currentScriptEditor ifNil:
		[currentScriptEditor _ (self playerClass includesSelector: selector) 
			ifTrue:
				[Preferences universalTiles
					ifFalse:
						[self error: 'duplicate selector'].
				ScriptEditorMorph new fromExistingMethod: selector forPlayer: aPlayer]
			ifFalse:
				[ScriptEditorMorph new setMorph: aPlayer costume scriptName: selector].

		(defaultStatus == #ticking and: [selector numArgs == 0]) ifTrue:
			[aPlayer costume arrangeToStartStepping]].
	
	^ currentScriptEditor! !

!UniclassScript methodsFor: 'initialization' stamp: 'sw 1/23/2001 17:14'!
isTextuallyCoded
	"Answer whether the receiver is textually coded"

	^ isTextuallyCoded! !

!UniclassScript methodsFor: 'textually coded' stamp: 'sw 3/27/2002 22:43'!
lastSourceString
	"Answer the most recent source string"

	^ lastSourceString! !

!UniclassScript methodsFor: 'access' stamp: 'sw 12/15/2004 20:46'!
playerClass
	"Answer the playerClass associated with the receiver"

	^ playerClass ifNil:
		[playerClass _ currentScriptEditor playerScripted ifNotNil: [currentScriptEditor playerScripted class]]! !

!UniclassScript methodsFor: 'initialization' stamp: 'sw 7/2/2002 14:13'!
playerClass: aPlayerClass selector: aSelector
	"Set the playerClass and selector of the receiver"

	super playerClass: aPlayerClass selector: aSelector.
	aSelector numArgs = 1 ifTrue:
		[argumentVariables _ {Variable new name: 'parameter' type: #Number}]! !

!UniclassScript methodsFor: 'access' stamp: 'sw 12/15/2004 16:42'!
playerClassPerSe
	"Answer the current value of the playerClass inst var."

	^ playerClass! !

!UniclassScript methodsFor: 'initialization' stamp: 'sw 12/15/2004 15:15'!
printOn: aStream
	aStream nextPutAll: 'A UniclassScript - selector: ', selector printString, ' scriptEditor: ', currentScriptEditor printString! !

!UniclassScript methodsFor: 'script editor' stamp: 'sw 7/28/2001 01:11'!
recompileScriptFromTilesUnlessTextuallyCoded
	"recompile Script From Tiles Unless Textually Coded"

	self isTextuallyCoded ifFalse:
		[currentScriptEditor ifNotNil: [currentScriptEditor recompileScript]]! !

!UniclassScript methodsFor: 'versions' stamp: 'di 2/19/2001 10:09'!
recreateScriptFrom: anEditor
	"Used to revert to old tiles"

	formerScriptingTiles isEmptyOrNil ifTrue: [^ self].
	anEditor reinsertSavedTiles: formerScriptingTiles last second.
	isTextuallyCoded _ false! !

!UniclassScript methodsFor: 'initialize-release' stamp: 'RAA 5/18/2001 13:25'!
releaseCachedState
	"release all non-showing scriptors.  What do we do about versions????"
	"18 May 2001 - get more aggressive in dropping stuff"

	formerScriptingTiles _ OrderedCollection new.

	currentScriptEditor ifNil: [^ self].

	true ifTrue: [^ self].	"<<< to test the reconstruction of scripts, change to false"
	currentScriptEditor world ifNil: ["not showing"
		currentScriptEditor _ nil]! !

!UniclassScript methodsFor: 'versions' stamp: 'md 10/22/2003 16:16'!
revertScriptVersionFrom: anEditor 
	"Let user choose which prior tile version to revert to, and revert to it"

	| aMenu chosenStampAndTileList |
	formerScriptingTiles isEmptyOrNil ifTrue: [^Beeper beep].
	chosenStampAndTileList := formerScriptingTiles size == 1 
		ifTrue: [ formerScriptingTiles first]
		ifFalse: 
			[aMenu := SelectionMenu 
						labelList: (formerScriptingTiles collect: [:e | e first])
						selections: formerScriptingTiles.
			aMenu startUp].
	chosenStampAndTileList ifNotNil: 
			[anEditor reinsertSavedTiles: chosenStampAndTileList second.
			isTextuallyCoded := false]! !

!UniclassScript methodsFor: 'updating' stamp: 'sw 2/16/2001 00:46'!
revertToLastSavedTileVersionFor: anEditor
	"revert to the last saved tile version"

	Preferences universalTiles
		ifFalse:
			[formerScriptingTiles isEmptyOrNil ifFalse:
				[anEditor reinsertSavedTiles: formerScriptingTiles last second]]
		ifTrue:
			[anEditor removeAllButFirstSubmorph.
			anEditor insertUniversalTiles].
	anEditor showingMethodPane: false.
	isTextuallyCoded _ false! !

!UniclassScript methodsFor: 'updating' stamp: 'sw 2/18/2001 18:27'!
saveScriptVersion: timeStamp
	"Save the tile script version by appending a pair of the form

		<time stamp>     <morph list>

to my list of former scripting tiles.  The morph-list will get copied back into the Scriptor following restoration.  Only applies to classic tiles."

	Preferences universalTiles ifFalse:  "the following only applies to Classic tiles"
		[(currentScriptEditor notNil and: [currentScriptEditor showingMethodPane not]) ifTrue:
				[formerScriptingTiles ifNil: [formerScriptingTiles _ OrderedCollection new].
				formerScriptingTiles add:
					(Array with: timeStamp
						with: (currentScriptEditor submorphs allButFirst collect: [:m | m veryDeepCopy])).
				formerScriptingTiles size > 100 ifTrue: [^ self halt: 'apparent runaway versions, proceed at your own risk.']]]! !

!UniclassScript methodsFor: 'versions' stamp: 'sw 1/30/2001 11:31'!
savedTileVersionsCount
	"Answer the number of saved tile versions of the script"

	^ formerScriptingTiles ifNil: [0] ifNotNil: [formerScriptingTiles size]! !

PluggableTextMorph subclass: #MethodMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!MethodMorph classSide methodsFor: 'as yet unclassified' stamp: 'dgd 8/26/2004 12:11'!
defaultNameStemForInstances
	^ 'Method'! !

!MethodMorph classSide methodsFor: 'new-morph participation' stamp: 'kfr 5/3/2000 12:52'!
includeInNewMorphMenu
	"Not to be instantiated from the menu"
	^ false! !

!MethodMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 18:25'!
initialize
	"initialize the state of the receiver"
	super initialize.
	self useRoundedCorners! !

!MethodMorph methodsFor: 'as yet unclassified' stamp: 'tk 9/7/2000 22:07'!
installRollBackButtons: target
	| mine |
	"If I don't already have such a button, put one in at the upper right.  Set its target to the furtherest enclosing book.  Send chooseAndRevertToVersion when clicked.  Stay in place via scrollBar install."

	mine _ self submorphNamed: #chooseAndRevertToVersion ifNone: [nil].
	mine ifNil: [mine _ SimpleButtonMorph new.
		"mine height: mine height - 2."
		mine label: 'Roll Back'; cornerStyle: #square.
		mine color: Color white; borderColor: Color black.
		mine actionSelector: #chooseAndRevertToVersion.
		mine align: mine topRight with: (self findA: ScrollBar) topLeft +(1@1).
		self addMorphFront: mine.
		mine height: mine height - 5 "14"].
	mine target: target.! !

!MethodMorph methodsFor: 'scrolling' stamp: 'nk 4/28/2004 10:23'!
showScrollBar
	"Copied down and modified to get rid of the ruinous comeToFront of the inherited version."

	| scriptor |
	(submorphs includes: scrollBar)
		ifTrue: [^ self].
	self vResizeScrollBar.
	self privateAddMorph: scrollBar atIndex: 1.
	retractableScrollBar
		ifTrue:
			["Bring the pane to the front so that it is fully visible"
			"self comeToFront. -- thanks but no thanks"
			(scriptor _ self ownerThatIsA: ScriptEditorMorph)
				ifNotNil:
					[scriptor comeToFront]]
		ifFalse: [self resetExtent]! !

!MethodMorph methodsFor: 'stepping and presenter' stamp: 'sw 10/21/1999 09:41'!
step
	model updateCodePaneIfNeeded! !

!MethodMorph methodsFor: 'testing' stamp: 'sw 10/23/1999 23:03'!
stepTime
	^ 3000! !

!PluggableTextMorph methodsFor: '*eToys-transcript' stamp: 'RAA 5/1/2002 18:17'!
appendTextEtoy: moreText
	"Append the text in the model's writeStream to the editable text. "

	self handleEdit: [
		self 
			selectInvisiblyFrom: textMorph asText size + 1 to: textMorph asText size;
			replaceSelectionWith: moreText;
			selectFrom: textMorph asText size + 1 to: textMorph asText size;
			hasUnacceptedEdits: false;
			scrollSelectionIntoView;
			changed
	]! !

!PluggableTextMorph methodsFor: '*eToys-model access' stamp: 'RAA 11/5/2000 14:10'!
eToyGetMainFont

	^ textMorph textStyle! !

!ImageMorph methodsFor: '*eToys-card in a stack' stamp: 'sw 10/23/2000 18:22'!
couldHoldSeparateDataForEachInstance
	"Answer whether the receiver can potentially hold separate data for each instance"

	^ true! !

!ImageMorph methodsFor: '*eToys-player' stamp: 'sw 10/23/2000 18:22'!
currentDataValue
	"Answer the current data value of the receiver, to be stored in each card instance if appropriate"

	^ image! !

!ImageMorph methodsFor: '*eToys-player' stamp: 'sw 10/25/2000 06:58'!
variableDocks
	"Answer a list of VariableDock objects for docking up my data with an instance held in my containing playfield"

	^ Array with: (VariableDock new variableName: self defaultVariableName type: #form definingMorph: self morphGetSelector: #image morphPutSelector: #setNewImageFrom:)! !

!ImageMorph methodsFor: '*eToys-other' stamp: ''!
wearCostume: anotherMorph

	self image: anotherMorph form.
! !

!UnknownType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 13:33'!
affordsCoercionToBoolean
	"Answer true if a tile of this data type, when dropped into a pane that demands a boolean, could plausibly be expanded into a comparison (of the form  frog < toad   or frog = toad) to provide a boolean expression"

	^ false! !

!UnknownType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:33'!
wantsArrowsOnTiles
	"Answer whether this data type wants up/down arrows on tiles representing its values"

	^ false! !

!WeakMessageSend methodsFor: '*eToys-tiles' stamp: 'nk 4/25/2002 09:34'!
asTilesIn: playerClass
	^self asMessageSend asTilesIn: playerClass
! !

!WeakMessageSend methodsFor: '*eToys-tiles' stamp: 'nk 4/25/2002 09:34'!
asTilesIn: playerClass globalNames: makeSelfGlobal
	^self asMessageSend asTilesIn: playerClass globalNames: makeSelfGlobal
! !

!WeakMessageSend methodsFor: '*eToys-tiles' stamp: 'nk 4/25/2002 09:35'!
stringFor: anObject
	^self asMessageSend stringFor: anObject
! !

Model subclass: #Player
	instanceVariableNames: 'costume costumes'
	classVariableNames: 'BiggestSubclassNumber TimeOfError'
	poolDictionaries: 'References'
	category: 'EToys-Scripting'!

Player class
	instanceVariableNames: 'scripts slotInfo'!

!Player commentStamp: '<historical>' prior: 0!
The fundamental user-scriptable entity.  Always represented by a user-specific subclass of Player; instance vars and methods relate to user-defined structures.

costume  is a Morph, the primary morph I am currently wearing for graphical display.

Scripts are defined in subclasses of Player.  These are UniClasses.

Messages in scripts are sent to Players.  A Player may delegate to its costume, or to an object the costume suggests.  Or, a Player may designate some other object to receive the script messages it does not understand. (see doesNotUnderstand:) !

Player class
	instanceVariableNames: 'scripts slotInfo'!

Player subclass: #CardPlayer
	instanceVariableNames: 'privateMorphs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Stacks'!

CardPlayer class
	instanceVariableNames: 'variableDocks'!

!CardPlayer commentStamp: '<historical>' prior: 0!
CardPlayer
	Instance variables of the Uniclass represent the data in the "fields" of each card in the stack.
	Each Instance variable is some kind of value holder.

	The code for the *buttons* on the background resides in the CardPlayer uniclass.

privateMorphs -- OrderedCollection of objects specific to this card.

Individual CardPlayer classes need to store the search results of any instances that are templates.  As a hack, we use a class variable TemplateMatches in each individual class (CardPlayer21).  It is initialized in #matchIndex:.
TemplateMatches   an IndentityDictionary of 
		(aCardPlayer -> (list of matching cards, index in that list))
!

CardPlayer class
	instanceVariableNames: 'variableDocks'!

!CardPlayer classSide methodsFor: 'compiling' stamp: 'sw 10/13/2000 13:02'!
acceptsLoggingOfCompilation
	"Answer whether methods of the receiver should be logged when submitted."

	^ #(CardPlayer) includes: self class theNonMetaClass name! !

!CardPlayer classSide methodsFor: 'slots' stamp: 'NS 1/28/2004 14:41'!
compileAccessorsFor: varName
	"Compile instance-variable accessor methods for the given variable name"

	| nameString |
	nameString _ varName asString capitalized.
	self compileSilently: ('get', nameString, '
	^ ', varName)
		classified: 'access'.
	self compileSilently: ('set', nameString, ': val
	', varName, ' _ val')
		classified: 'access'! !

!CardPlayer classSide methodsFor: 'instance creation' stamp: 'sw 10/13/2000 13:05'!
isUniClass
	"Answer, for the purpose of providing annotation in a method holder, whether the receiver is a uniClass."

	^ self ~~ CardPlayer! !

!CardPlayer classSide methodsFor: 'variable docks' stamp: 'sw 10/13/2000 16:36'!
newVariableDocks: dockList
	"Set the receiver's variableDocks to be the list provided in dockList.  Assimilate this new information into the receiver's slotInfo, which contains both automatically-generated variables such as the variable docks and also explicitly-user-specified variables"

	self variableDocks: dockList.
	self setSlotInfoFromVariableDocks! !

!CardPlayer classSide methodsFor: 'testing' stamp: 'sw 10/13/2000 13:07'!
officialClass
	"Answer (for the purpose of copying mechanisms) the system class underlying the receiver."

	^ CardPlayer! !

!CardPlayer classSide methodsFor: 'slots' stamp: 'NS 1/30/2004 13:11'!
removeAccessorsFor: varName
	"Remove the instance-variable accessor methods associated with varName"

	| nameString |
	nameString _ varName asString capitalized.
	self removeSelectorSilently: ('get', nameString) asSymbol.
	self removeSelectorSilently: ('set', nameString, ':') asSymbol! !

!CardPlayer classSide methodsFor: 'user-defined inst vars' stamp: 'sw 12/6/2001 20:36'!
resortInstanceVariables: newList
	"Accept a new ordering for instance variables"

	variableDocks _ newList collect: [:aName | variableDocks detect: [:d | d variableName = aName]].
	self setNewInstVarNames: newList asOrderedCollection.
	self newVariableDocks: variableDocks.
! !

!CardPlayer classSide methodsFor: 'user-defined inst vars' stamp: 'tk 8/26/2001 16:58'!
setNewInstVarNames: listOfStrings
	"Make listOfStrings be the new list of instance variable names for the receiver"

	| disappearing firstAppearing instVarString instVarList |
	instVarList _ self instVarNames asOrderedCollection.
	disappearing _ instVarList copy.
	disappearing removeAllFoundIn: listOfStrings.
	disappearing do:
		[:oldName | 	self removeAccessorsFor: oldName].
	firstAppearing _ listOfStrings copy.
	firstAppearing removeAllFoundIn: instVarList.
	instVarString _ String streamContents:
		[:aStream | listOfStrings do: [:aString | aStream nextPutAll: aString; nextPut: $ ]].

	superclass subclass: self name instanceVariableNames: instVarString 
		classVariableNames: '' poolDictionaries: '' category: self categoryForUniclasses.
	firstAppearing do:
		[:newName | self compileAccessorsFor: newName].
! !

!CardPlayer classSide methodsFor: 'variable docks' stamp: 'md 3/1/2006 09:17'!
setSlotInfoFromVariableDocks
	"Get the slotInfo fixed up after a change in background shape.  Those instance variables that are proactively added by the user will persist, whereas those that are automatically generated will be updated"

	| aDock newInfo |
	
	self slotInfo copy do:  "Remove old automatically-created slots"
		[:aSlotInfo | (aDock _ aSlotInfo variableDock) ifNotNil:
			[slotInfo removeKey: aDock variableName]].

	self variableDocks do:  [:dock | "Generate fresh slots from variable docks"
			newInfo _ SlotInformation new type: dock variableType.
			newInfo variableDock: dock.
			slotInfo at: dock variableName asSymbol put: newInfo]! !

!CardPlayer classSide methodsFor: 'variable docks' stamp: 'sw 10/13/2000 16:39'!
variableDocks
	"Answer the list of variable docks in the receiver.  Initialize the variable-dock list if not already done."

	variableDocks ifNil: [variableDocks _ OrderedCollection new].
	^ variableDocks! !

!CardPlayer classSide methodsFor: 'variable docks' stamp: 'sw 10/13/2000 16:39'!
variableDocks: dockList
	"Set the variable-dock list as indicated"

	variableDocks _ dockList! !

!CardPlayer classSide methodsFor: 'compiling' stamp: 'tk 9/28/2001 11:42'!
wantsChangeSetLogging
	"Log changes for CardPlayer itself, but not for automatically-created subclasses like CardPlayer1, CardPlayer2, but *do* log it for uniclasses that have been manually renamed."

	^ (self == CardPlayer or:
		[(self name beginsWith: 'CardPlayer') not]) or:
			[Preferences universalTiles]! !

!CardPlayer methodsFor: 'card data' stamp: 'dgd 2/22/2003 14:43'!
allStringsAfter: aText 
	"return an OrderedCollection of strings of text in my instance vars.  If aText is non-nil, begin with that object."

	| list ok instVarValue string |
	list := OrderedCollection new.
	ok := aText isNil.
	self class variableDocks do: 
			[:vdock | 
			instVarValue := self perform: vdock playerGetSelector.
			ok ifFalse: [ok := instVarValue == aText].	"and do this one too"
			ok 
				ifTrue: 
					[string := nil.
					instVarValue isString ifTrue: [string := instVarValue].
					instVarValue isText ifTrue: [string := instVarValue string].
					instVarValue isNumber ifTrue: [string := instVarValue printString].
					instVarValue isMorph ifTrue: [string := instVarValue userString].	"not used"
					string ifNotNil: 
							[string isString ifTrue: [list add: string] ifFalse: [list addAll: string]]]].
	privateMorphs 
		ifNotNil: [privateMorphs do: [:mm | list addAll: (mm allStringsAfter: nil)]].
	^list! !

!CardPlayer methodsFor: 'card data' stamp: 'tk 5/25/2001 17:42'!
asKeys
	| keys kk vd gotData |
	"Take my fields, tokenize the text, and return as an array in the same order as variableDocks.  Simple background fields on the top level.  If no data, return nil."

	keys _ self class variableDocks copy.
	gotData _ false.
	1 to: keys size do: [:ind |
		kk _ nil.
		vd _ self class variableDocks at: ind.
		vd type == #text ifTrue: [
			kk _ (self perform: vd playerGetSelector) string
					findTokens: Character separators.
			kk isEmpty ifTrue: [kk _ nil] ifFalse: [gotData _ true]].
		keys at: ind put: kk].
	^ gotData ifTrue: [keys] ifFalse: [nil]! !

!CardPlayer methodsFor: 'card data' stamp: 'sw 10/13/2000 16:46'!
commitCardPlayerData
	"Transport data back from the morphs that may be holding it into the instance variables that must hold it when the receiver is not being viewed"

	| prior |
	self class variableDocks do:
		[:aDock | aDock storeMorphDataInInstance: self].
	prior _ nil.
	privateMorphs _ OrderedCollection new.
	self costume ifNotNil:
		[self costume submorphs do:
			[:aMorph | aMorph renderedMorph isShared
				ifFalse:
					[aMorph setProperty: #priorMorph toValue: prior.
					privateMorphs add: aMorph.
					aMorph delete].
			prior _ aMorph]]! !

!CardPlayer methodsFor: 'card data' stamp: 'sw 11/14/2000 11:21'!
commitCardPlayerDataFrom: aPlayfield
	"Transport data back from the morphs that may be holding it into the instance variables that must hold it when the receiver is not being viewed"

	| prior itsOrigin |
	itsOrigin _ aPlayfield topLeft.
	self class variableDocks do:
		[:aDock | aDock storeMorphDataInInstance: self].
	prior _ nil.
	privateMorphs _ OrderedCollection new.
	self costume ifNotNil:
		[self costume submorphs do:
			[:aMorph | aMorph renderedMorph isShared
				ifFalse:
					[aMorph setProperty: #priorMorph toValue: prior.
					privateMorphs add: aMorph.
					aMorph delete.
					aMorph position: (aMorph position - itsOrigin)].
			prior _ aMorph]]! !

!CardPlayer methodsFor: 'card data' stamp: 'tk 1/16/2001 16:12'!
installPrivateMorphsInto: aBackground
	"The receiver is being installed as the current card in a given pasteup morph being used as a background.  Install the receiver's private morphs into that playfield"

	| prior originToUse |
	self flag: #deferred.  "not robust if the background is showing a list view"
	privateMorphs ifNotNil: [privateMorphs do:
		[:aMorph |
			originToUse _ aBackground topLeft.
			prior _ aMorph valueOfProperty: #priorMorph ifAbsent: [nil].
			aMorph position: (aMorph position + originToUse).
			(prior notNil and: [aBackground submorphs includes: prior])
				ifTrue:
					[aBackground addMorph: aMorph after: prior]
				ifFalse:
					[aBackground addMorphFront: aMorph].
		aMorph removeProperty: #priorMorph]]! !

!CardPlayer methodsFor: 'card data' stamp: 'tk 5/25/2001 17:02'!
match: keys fields: docks
	| longString |
	"see if each key occurs in my corresponding text instance."

	keys withIndexDo: [:kk :ind |
		kk ifNotNil: [
			longString _ (self perform: (docks at: ind) playerGetSelector) string.
			kk do: [:aKey |
				((longString findString: aKey startingAt: 1 caseSensitive: false) > 0)
					ifFalse: [^ false]]]]. 	"all keys must match"
	^ true! !

!CardPlayer methodsFor: 'as template' stamp: 'tk 5/29/2001 22:44'!
matchIndex
	| tms |
	"Index of one we are looking at, in the cards that matched the last search with this template."

	tms _ self class classPool at: #TemplateMatches ifAbsent: [^ 0].
	^ (tms at: self ifAbsent: [#(0 0)]) second.
! !

!CardPlayer methodsFor: 'as template' stamp: 'tk 5/29/2001 22:47'!
matchIndex: newPlace
	| tms pair |
	"One we are looking at, in cards that matched the last template search."

	tms _ self class classPool at: #TemplateMatches ifAbsent: [
		self class addClassVarName: 'TemplateMatches'.
		self class classPool at: #TemplateMatches put: IdentityDictionary new].
	pair _ tms at: self ifAbsent: [tms at: self put: (Array new: 2)].
	pair at: 2 put: newPlace.
	newPlace = 0 ifTrue: [^ self].
	pair first ifNil: [^ self].
	(costume valueOfProperty: #myStack ifAbsent: [^ self]) goToCard: 
		((pair first "list") at: newPlace).
	self changed: #matchIndex.	"update my selection"
! !

!CardPlayer methodsFor: 'as template' stamp: 'tk 5/31/2001 16:46'!
matchNames
	| list str ll tms stk crds |
	"List of names of cards that matched the last template search."

	tms _ self class classPool at: #TemplateMatches ifAbsent: [^ #()].
	list _ (tms at: self ifAbsent: [#(#() 0)]) first.
	stk _ costume valueOfProperty: #myStack ifAbsent: [nil].
	crds _ stk ifNil: [#()] ifNotNil: [stk cards].
	^ list collect: [:cd | 
		str _ ''.
		(ll _ cd allStringsAfter: nil) ifNotNil: [
			str _ ll inject: '' into: [:strr :this | strr, this]]. 
		(str copyFrom: 1 to: (30 min: str size)), '...  (' , (crds indexOf: cd) printString, ')'].
		"Maybe include a card title?"! !

!CardPlayer methodsFor: 'printing' stamp: 'sw 10/23/2000 17:58'!
printOn: aStream
	"Print out a human-readable representation of the receiver onto aStream"

	super printOn: aStream.
	self class instVarNames do:
		[:aName | aStream nextPutAll: ', ', aName, ' = ', (self instVarNamed: aName) printString]! !

!CardPlayer methodsFor: 'card data' stamp: 'tk 1/30/2001 23:42'!
privateMorphs

	^ privateMorphs! !

!CardPlayer methodsFor: 'scripts-kernel' stamp: 'svp 10/15/2001 14:44'!
renameScript: oldSelector newSelector: newSelector
	"Find all buttons that fire this script and tell them the new name"

	| stack |
	super renameScript: oldSelector newSelector: newSelector.
	costume allMorphsDo: [:mm |
		self retargetButton: mm oldSelector: oldSelector newSelector: newSelector].

	stack _ costume valueOfProperty: #myStack.
	stack ifNotNil:
		[stack cards do: [:cc |
			cc privateMorphs do: [:pp | 
				pp allMorphsDo: [:mm |
					self retargetButton: mm oldSelector: oldSelector newSelector: newSelector]]]]! !

!CardPlayer methodsFor: 'as template' stamp: 'tk 5/29/2001 22:49'!
results
	"Return my (cardlist index) pair from the last search"

	^ (self class classPool at: #TemplateMatches ifAbsent: [^ Array new: 2]) at: self
! !

!CardPlayer methodsFor: 'scripts-kernel' stamp: 'tk 9/29/2001 10:27'!
retargetButton: mm oldSelector: oldSelector newSelector: newSelector
	"changing the name of a script -- tell any buttons that fire it"

	(mm respondsTo: #scriptSelector) ifTrue: [
		mm scriptSelector == oldSelector ifTrue: [
			mm scriptSelector: newSelector.
			mm setNameTo: newSelector]].
	(mm respondsTo: #actionSelector) ifTrue: [
		mm actionSelector == oldSelector ifTrue: [
			mm target class == self class ifTrue: [
				mm actionSelector: newSelector.
				mm setNameTo: newSelector]]].
! !

!CardPlayer methodsFor: 'slots-kernel' stamp: 'sw 7/28/2004 21:03'!
tileReferringToSelf
	"Answer a tile that refers to the receiver.  For CardPlayer, want 'self', not the specific name of this card.  Script needs to work for any card of the background."

	Preferences universalTiles ifTrue:
		[^ self universalTileReferringToSelf].

	^ TileMorph new setToReferTo: self! !

!CardPlayer methodsFor: 'card data' stamp: 'tk 5/7/2001 15:51'!
url
	"For now, don't know we could be on a server"

	^ nil! !

Player subclass: #KedamaExamplerPlayer
	instanceVariableNames: 'kedamaWorld turtles sequentialStub'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaExamplerPlayer classSide methodsFor: 'compiling' stamp: 'yo 8/23/2004 11:30'!
wantsChangeSetLogging
	"Log changes for Player itself, but not for automatically-created subclasses like Player1, Player2, but *do* log it for uniclasses that have been manually renamed."

	^ (self == KedamaExamplerPlayer or:
		[(self name beginsWith: 'KedamaExamplerPlayer') not]) or:
			[Preferences universalTiles]! !

!KedamaExamplerPlayer methodsFor: 'player protocol' stamp: 'yo 9/27/2004 16:24'!
addInstanceVarNamed: aName withValue: aValue

	self basicAddInstanceVarNamed: aName withValue: aValue.
	"turtles _ kedamaWorld turtlesOf: self."
	turtles addInstanceVarVectorNamed: aName withValue: aValue.
! !

!KedamaExamplerPlayer methodsFor: 'player protocol' stamp: 'yo 3/25/2005 11:45'!
chooseSlotTypeFor: aGetter
	"Let the user designate a type for the slot associated with the given getter"

	| typeChoices typeChosen slotName |
	slotName _ Utilities inherentSelectorForGetter: aGetter.
	typeChoices _ Vocabulary typeChoices.

	typeChosen _ (SelectionMenu labelList: (typeChoices collect: [:t | t translated]) lines: #() selections: typeChoices) startUpWithCaption: 
		('Choose the TYPE
for ' translated, slotName, '
(currently ' translated, (self slotInfoAt: slotName) type translated, ')').
	typeChosen isEmptyOrNil ifTrue: [^ self].
	(self typeForSlot: slotName) capitalized = typeChosen ifTrue: [^ self].

	(self slotInfoAt: slotName) type: typeChosen.
	self class allInstancesDo:   "allSubInstancesDo:"
		[:anInst | anInst instVarNamed: slotName asString put: 
			(anInst valueOfType: typeChosen from: (anInst instVarNamed: slotName))].
	turtles setVectorSlotTypeFor: slotName typeChosen: typeChosen.
	sequentialStub ifNotNil: [sequentialStub setScalarSlotTypeFor: slotName typeChosen: typeChosen].
	self updateAllViewers.	"does siblings too"
! !

!KedamaExamplerPlayer methodsFor: 'subclass players management' stamp: 'yo 9/26/2004 10:49'!
clonedSequentialStub

	^ sequentialStub clone.
! !

!KedamaExamplerPlayer methodsFor: 'player commands' stamp: 'yo 10/12/2004 22:25'!
colorFromPatch: aPatch
! !

!KedamaExamplerPlayer methodsFor: 'player commands' stamp: 'yo 10/12/2004 22:24'!
colorToPatch: aPatch
! !

!KedamaExamplerPlayer methodsFor: 'method management' stamp: 'yo 9/27/2004 16:25'!
compileInstVarAccessorsFor: varName

	self basicCompileInstVarAccessorsFor: varName.
	"turtles _ kedamaWorld turtlesOf: self."
	turtles compileVectorInstVarAccessorsFor: varName.
	sequentialStub compileScalarInstVarAccessorsFor: varName.
! !

!KedamaExamplerPlayer methodsFor: 'debug support' stamp: 'yo 10/22/2004 07:26'!
copyAllMethodsAgain

	| c result |
	c _ turtles class.
	result _ (ClassBuilder new)
		name: c name
		inEnvironment: c environment
		subclassOf: c superclass
		type: c typeOfClass
		instanceVariableNames: KedamaTurtleVectorPlayer instanceVariablesString
		classVariableNames: KedamaTurtleVectorPlayer classVariablesString
		poolDictionaries: KedamaTurtleVectorPlayer sharedPoolsString
		category: c category.
	turtles class copyAllCategoriesUnobtrusivelyFrom: KedamaTurtleVectorPlayer.
	sequentialStub ifNotNil: [sequentialStub class copyAllCategoriesUnobtrusivelyFrom: KedamaSequenceExecutionStub].
! !

!KedamaExamplerPlayer methodsFor: 'subclass players management' stamp: 'yo 9/27/2004 16:34'!
createSequenceStub

	| class |
	class _ self class createSequencialStubSubclass.
	sequentialStub _ class new.
	sequentialStub kedamaWorld: kedamaWorld.
	sequentialStub examplerPlayer: self.
	sequentialStub turtles: turtles.
! !

!KedamaExamplerPlayer methodsFor: 'subclass players management' stamp: 'yo 9/28/2004 14:03'!
createTurtles

	turtles _ self class createTurtleSubclass new.
	turtles kedamaWorld: kedamaWorld.
	turtles exampler: self.
	^ turtles.
! !

!KedamaExamplerPlayer methodsFor: 'deleting' stamp: 'yo 10/8/2004 11:06'!
delete

	| anInstance |
	turtles delete.
	sequentialStub delete.

	self class removeFromSystem: false.

	anInstance _ UnscriptedPlayer new.
	self become: anInstance.
! !

!KedamaExamplerPlayer methodsFor: 'command execution' stamp: 'yo 9/27/2004 16:26'!
doCommand: aBlock

	| ret |
	ret _ self doExamplerCommand: aBlock.
	turtles ifNotNil: [aBlock value: turtles].

	^ ret.
! !

!KedamaExamplerPlayer methodsFor: 'command execution' stamp: 'yo 9/27/2004 16:26'!
doDieCommand: aBlock

	| ret origSize |
	ret _ self doExamplerCommand: aBlock.
	turtles initializeDeletingIndex.
	origSize _ turtles size.
	1 to: origSize do: [:i |
		i > origSize ifTrue: [^ ret].
		sequentialStub index: (turtles nextDeletingIndex).
		aBlock value: sequentialStub.
	].

	^ ret.

! !

!KedamaExamplerPlayer methodsFor: 'command execution' stamp: 'yo 9/30/2004 14:51'!
doExamplerCommand: aBlock

	 "^ aBlock value: self."! !

!KedamaExamplerPlayer methodsFor: 'command execution' stamp: 'yo 9/28/2004 10:21'!
doSequentialCommand: aBlock

	| ret |
	ret _ self doExamplerCommand: aBlock.
	self getGrouped ifFalse: [
		1 to: turtles size do: [:i |
			sequentialStub index: i.
			aBlock value: sequentialStub.
		].
	] ifTrue: [
		aBlock value: turtles.
	].
	turtles invalidateTurtleMap.
	^ ret.

! !

!KedamaExamplerPlayer methodsFor: 'player commands' stamp: 'yo 10/12/2004 15:24'!
getBlueComponentIn: aPatch

	^ 0.
! !

!KedamaExamplerPlayer methodsFor: 'slot-accessing' stamp: 'yo 4/5/2005 11:06'!
getColorOpaque

	^ self getColor.
! !

!KedamaExamplerPlayer methodsFor: 'player commands' stamp: 'yo 10/12/2004 15:24'!
getGreenComponentIn: aPatch

	^ 0.
! !

!KedamaExamplerPlayer methodsFor: 'slot-accessing' stamp: 'yo 9/29/2004 09:07'!
getNormal

	^ 0.0.
! !

!KedamaExamplerPlayer methodsFor: 'player commands' stamp: 'yo 10/12/2004 15:24'!
getRedComponentIn: aPatch

	^ 0.
! !

!KedamaExamplerPlayer methodsFor: 'player commands' stamp: 'yo 10/9/2004 15:41'!
getTurtleAt: aPlayer

	^ turtles getTurtleAt: aPlayer.

! !

!KedamaExamplerPlayer methodsFor: 'player commands' stamp: 'yo 3/7/2005 08:36'!
getTurtleOf: aBreedPlayer

	^ self.

! !

!KedamaExamplerPlayer methodsFor: 'slot-accessing' stamp: 'yo 9/14/2004 20:36'!
getVisible

	^ self costume renderedMorph visible
! !

!KedamaExamplerPlayer methodsFor: 'slot-accessing' stamp: 'yo 9/14/2004 20:35'!
getWho

	^ 1.
! !

!KedamaExamplerPlayer methodsFor: 'testing' stamp: 'yo 8/12/2004 11:21'!
isPrototypeTurtlePlayer

	^ true.
! !

!KedamaExamplerPlayer methodsFor: 'accessing' stamp: 'yo 9/23/2004 17:15'!
kedamaWorld

	^ kedamaWorld.
! !

!KedamaExamplerPlayer methodsFor: 'accessing' stamp: 'yo 9/23/2004 17:15'!
kedamaWorld: k

	kedamaWorld _ k.
! !

!KedamaExamplerPlayer methodsFor: 'accessing' stamp: 'yo 4/3/2005 17:56'!
nearestIDFor: aNumber

	| whoArray last next smaller bigger |
	whoArray _ turtles arrays first.
	whoArray isEmpty ifTrue: [^ nil].
	aNumber < whoArray first ifTrue: [^ whoArray first].

	last _ whoArray at: 1.
	2 to: whoArray size do: [:i |
		(last <= aNumber and: [aNumber < (next _ whoArray at: i)]) ifTrue: [
			smaller _ last.
			bigger _ next.
			(smaller - aNumber) abs <= (bigger - aNumber) abs ifTrue: [^ smaller] ifFalse: [^ bigger].
		].
		last _ next.
	].
	^ last.
! !

!KedamaExamplerPlayer methodsFor: 'method management' stamp: 'yo 9/27/2004 16:26'!
recompileAccessorsOf: oldSlotName to: newSlotName inPlayer: aPlayer
	"Note that aPlayer has renamed a slot formerly known as oldSlotName to be newSlotName"

	self isPrototypeTurtlePlayer ifTrue: [
		sequentialStub ifNotNil: [sequentialStub noteRenameOf: oldSlotName to: newSlotName inPlayer: aPlayer].
		turtles noteRenameOf: oldSlotName to: newSlotName inPlayer: aPlayer.
	].
! !

!KedamaExamplerPlayer methodsFor: 'player protocol' stamp: 'yo 9/27/2004 16:26'!
removeSlotNamed: aSlotName

	self basicRemoveSlotNamed: aSlotName.
	turtles removeVectorSlotNamed: aSlotName.
! !

!KedamaExamplerPlayer methodsFor: 'player protocol' stamp: 'yo 10/27/2004 20:19'!
renameSlot: oldSlotName newSlotName: newSlotName

	self basicRenameSlot: oldSlotName newSlotName: newSlotName.
	turtles renameVectorSlot: oldSlotName newSlotName: newSlotName.
	self recompileAccessorsOf: oldSlotName to: newSlotName inPlayer: self.
! !

!KedamaExamplerPlayer methodsFor: 'slot-accessing' stamp: 'yo 9/24/2004 14:33'!
setHeading: degree

! !

!KedamaExamplerPlayer methodsFor: 'slot-accessing' stamp: 'yo 9/29/2004 09:07'!
setNormal: degree

! !

!KedamaExamplerPlayer methodsFor: 'slot-accessing' stamp: 'yo 7/24/2005 19:53'!
setTurtleCount: aNumber

	"self allOpenViewers do: [:v | v resetWhoIfNecessary]."
	super setCostumeSlot: #turtleCount: toValue: aNumber.
! !

!KedamaExamplerPlayer methodsFor: 'slot-accessing' stamp: 'yo 9/14/2004 16:08'!
turn: degree

! !

!KedamaExamplerPlayer methodsFor: 'subclass players management' stamp: 'yo 9/27/2004 16:28'!
turtles

	^ turtles.
! !

!KedamaExamplerPlayer methodsFor: 'accessing' stamp: 'yo 4/4/2005 13:59'!
whoAt: index

	| whoArray |
	whoArray _ turtles arrays first.
	index < 1 ifTrue: [^ 0].
	index > whoArray size ifTrue: [^ 0].

	^ whoArray at: index.
! !

KedamaExamplerPlayer subclass: #KedamaSequenceExecutionStub
	instanceVariableNames: 'who index arrays exampler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/23/2004 18:51'!
basicGetAngleTo: player

	| ret i |
	i _ self index.
	ret _ ((player getX - ((turtles arrays at: 2) at: i))@(player getY - ((turtles arrays at: 3) at: i))) theta radiansToDegrees + 90.0.
	ret > 360.0 ifTrue: [^ ret - 360.0].
	^ ret.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/23/2004 18:07'!
basicGetDistanceTo: aPlayer

	| i |
	i _ self index.
	^ ((aPlayer getX - ((turtles arrays at: 2) at: i)) squared + (aPlayer getY - ((turtles arrays at: 3) at: i)) squared) sqrt.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/23/2004 23:47'!
bounceOn: aPlayer

	| aTurtle xy |
	aPlayer = exampler ifTrue: [^ false].
	xy _ self getXAndY.
	aTurtle _ aPlayer turtles aTurtleAtX: xy x y: xy y.
	aTurtle ifNil: [^ false].
	self doBounceOn: aTurtle of: aPlayer.
	^ true.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/23/2004 23:48'!
bounceOn: aPlayer color: c

	| aTurtle xy |
	aPlayer = exampler ifTrue: [^ false].
	xy _ self getXAndY.
	aTurtle _ aPlayer turtles aTurtleAtX: xy x y: xy y.
	aTurtle ifNil: [^ false].
	((aPlayer turtles arrays at: 5) at: aTurtle index) = c ifFalse: [^ false].
	self doBounceOn: aTurtle of: aPlayer.
	^ true.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/28/2004 18:40'!
bounceOnColor: aPlayer

	^ false.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:33'!
color: cPixel

	(turtles arrays at: 5) at: self index put: cPixel.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 4/5/2005 11:04'!
colorFromPatch: aPatch

	| i |
	i _ self index.
	(turtles arrays at: 5) at: i put: ((aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i)) bitAnd: 16rFFFFFF).
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/12/2004 22:01'!
colorToPatch: aPatch

	| i |
	i _ self index.
	aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i) put: ((turtles arrays at: 5) at: i).
! !

!KedamaSequenceExecutionStub methodsFor: 'method management' stamp: 'yo 9/29/2004 09:02'!
compileAllAccessors

	turtles info keys asArray do: [:k |
		(#(who x y heading color visible normal) includes: k) ifFalse: [
			self compileScalarInstVarAccessorsFor: k.
		].
	].
! !

!KedamaSequenceExecutionStub methodsFor: 'method management' stamp: 'yo 9/30/2004 13:13'!
compileScalarInstVarAccessorsFor: varName

	| nameString type setPhrase arrayIndex getPhrase |
	nameString _ varName asString capitalized.
	arrayIndex _ turtles info at: varName asSymbol.

	type _ turtles types at: arrayIndex.
	type = #Number ifTrue: [
		setPhrase _ 'setNumberVarAt:'.
		getPhrase _ 'getNumberVarOf:'.
	].
	type = #Boolean ifTrue: [
		setPhrase _ 'setBooleanVarAt:'.
		getPhrase _ 'getBooleanVarOf:'
	].
	type = #Color ifTrue: [
		setPhrase _ 'setColorVarAt:'.
		getPhrase _ 'getColorVarOf:'
	].
	setPhrase ifNil: [setPhrase _ 'setObjectVarAt:'].
	getPhrase ifNil: [getPhrase _ 'getObjectVarOf:'].

	self class compileSilently: ('get', nameString, '
	^ self ', getPhrase, '((turtles arrays at: ', arrayIndex printString, ') at: self index)')
		classified: 'access'.

	self class compileSilently: ('set', nameString, ': xxxArg
		self ', setPhrase, arrayIndex printString, ' at: self index put: xxxArg' )
		classified: 'access'! !

!KedamaSequenceExecutionStub methodsFor: 'deleting' stamp: 'yo 10/8/2004 11:03'!
delete

	| anInstance |
	arrays _ nil.
	exampler _ nil.
	self class removeFromSystem: false.
	anInstance := UnscriptedPlayer new.
	self become: anInstance.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 4/4/2005 17:43'!
die

	kedamaWorld deleteTurtleID: self getWho of: exampler.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/23/2004 23:48'!
doBounceOn: aTurtle of: aPlayer

	| bounceDir myHeading neg newHeading xy |
	aPlayer = exampler ifTrue: [^ false].
	bounceDir _ aTurtle getHeading.
	myHeading _ self getHeading.
	neg _ (myHeading + 180.0) \\ 360.0.

	newHeading _ (bounceDir - neg + bounceDir).
	self setHeading: newHeading.
	[xy _ self getXAndY. (aPlayer turtles aTurtleAtX: xy x y: xy y) ~= nil] whileTrue: [
		self forward: 0.5.
	].
! !

!KedamaSequenceExecutionStub methodsFor: 'command execution' stamp: 'yo 9/30/2004 14:28'!
doCommand: aBlock

	^ aBlock value: self.
! !

!KedamaSequenceExecutionStub methodsFor: 'command execution' stamp: 'yo 9/29/2004 09:02'!
doDieCommand: aBlock

	| ret |
	ret _ self doExamplerCommand: aBlock.
	sequentialStub index: index.
	aBlock value: sequentialStub.

	^ ret.

! !

!KedamaSequenceExecutionStub methodsFor: 'command execution' stamp: 'yo 9/17/2004 09:25'!
doExamplerCommand: aBlock
! !

!KedamaSequenceExecutionStub methodsFor: 'command execution' stamp: 'yo 9/30/2004 14:29'!
doSequentialCommand: aBlock

	^ aBlock value: self.
! !

!KedamaSequenceExecutionStub methodsFor: 'player protocol' stamp: 'yo 9/17/2004 17:15'!
examplerPlayer: e

	exampler _ e.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:35'!
forward: v

	^ self primForwardAt: self index xArray: (turtles arrays at: 2) yArray: (turtles arrays at: 3) headingArray: (turtles arrays at: 4) value: v asFloat destWidth: kedamaWorld wrapX destHeight: kedamaWorld wrapY leftEdgeMode: kedamaWorld leftEdgeModeMnemonic rightEdgeMode: kedamaWorld rightEdgeModeMnemonic topEdgeMode: kedamaWorld topEdgeModeMnemonic bottomEdgeMode: kedamaWorld bottomEdgeModeMnemonic.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/23/2004 23:49'!
getAngleTo: aPlayer

	| i xy |
	i _ self index.
	xy _ aPlayer getXAndY.
	^ self primGetAngleToX: xy x toY: xy y fromX: ((turtles arrays at: 2) at: i) fromY: ((turtles arrays at: 3) at: i).
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/12/2004 15:24'!
getBlueComponentIn: aPatch

	| i pix |
	i _ self index.
	pix _ aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i).
	^ pix bitAnd: 16rFF.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 9/24/2004 10:25'!
getBooleanVarOf: val

	^ val ~= 0.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:33'!
getColor

	^ Color colorFromPixelValue: ((turtles arrays at: 5) at: self index) depth: 32.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 4/5/2005 11:07'!
getColorOpaque

	^ Color colorFromPixelValue: (((turtles arrays at: 5) at: self index) bitOr: 16rFF000000) depth: 32.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 9/24/2004 10:25'!
getColorVarOf: val

	^ Color colorFromPixelValue: val depth: 32.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/23/2004 23:49'!
getDistanceTo: aPlayer

	| i xy |
	i _ self index.
	xy _ aPlayer getXAndY.
	^ self primGetDistanceToX: xy x toY: xy y fromX: ((turtles arrays at: 2) at: i) fromY: ((turtles arrays at: 3) at: i).
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/12/2004 15:24'!
getGreenComponentIn: aPatch

	| i pix |
	i _ self index.
	pix _ aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i).
	^ (pix bitShift: -8) bitAnd: 16rFF.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/29/2004 15:11'!
getGrouped

	^ exampler getGrouped.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/11/2004 12:46'!
getHeading

	^ self primGetHeadingAt: self index headingArray: (turtles arrays at: 4).
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/29/2004 09:04'!
getNormal

	| headingRadians degrees |
	headingRadians _ (turtles arrays at: 7) at: self index.
	degrees _ 90.0 - headingRadians radiansToDegrees.
	^ degrees >= 0.0 ifTrue: [degrees] ifFalse: [degrees + 360.0].
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 9/24/2004 10:25'!
getNumberVarOf: val

	^ val.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 9/28/2004 11:25'!
getObjectVarOf: var

	^ var.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 18:16'!
getPatchValueIn: aPatch

	| i |
	i _ self index.
	^ aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i).
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/12/2004 15:24'!
getRedComponentIn: aPatch

	| i pix |
	i _ self index.
	pix _ aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i).
	^ (pix bitShift: -16) bitAnd: 16rFF.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:35'!
getReplicated

	| proto newWho n |
	proto _ (1 to: turtles arrays size) collect: [:i | (turtles arrays at: i) at: self index].
	kedamaWorld makeReplicatedTurtles: 1 examplerPlayer: exampler color: nil ofPrototype: proto randomize: false.
	exampler costume renderedMorph privateTurtleCount: (kedamaWorld turtlesCountOf: exampler).
	newWho _ (kedamaWorld lastWhoOf: exampler).
	n _ self clone.
	n who: newWho.
	^ n.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/23/2004 23:50'!
getTurtleAt: aPlayer

	| ret xy |
	xy _ aPlayer getXAndY.
	ret _ exampler turtles aTurtleAtX: xy x y: xy y.
	^ ret ifNil: [self].
! !

!KedamaSequenceExecutionStub methodsFor: 'player protocol' stamp: 'yo 4/4/2005 08:09'!
getTurtleCount

	^ exampler getTurtleCount.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 3/7/2005 11:33'!
getTurtleOf: aBreedPlayer

	| ret xy |
	xy _ self getXAndY.
	ret _ aBreedPlayer turtles aTurtleAtX: xy x y: xy y.
	^ ret ifNil: [self].
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 3/7/2005 07:30'!
getTurtleVisible

	^ ((turtles arrays at: 6) at: self index) = 1.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/30/2004 11:42'!
getUphillIn: aPatch

	| i p |
	i _ self index.
	p _ aPatch costume renderedMorph.
	^ p
			uphillForTurtleX: ((turtles arrays at: 2) at: i)
			turtleY: ((turtles arrays at: 3) at: i)
			turtleHeading: ((turtles arrays at: 4) at: i).
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:33'!
getWho

	^ (turtles arrays at: 1) at: self index.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:34'!
getX

	^ (turtles arrays at: 2) at: self index.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 10/23/2004 23:44'!
getXAndY

	| i |
	i _ self index.
	^ ((turtles arrays at: 2) at: i)@((turtles arrays at: 3) at: i).
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:32'!
getY

	^ (turtles arrays at: 3) at: self index.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:35'!
hide

	(turtles arrays at: 6) at: self index put: 0.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing' stamp: 'yo 9/27/2004 16:32'!
index

	who ifNil: [^ index].
	^ turtles indexOf: who.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing' stamp: 'yo 9/16/2004 10:41'!
index: i

	index _ i.
! !

!KedamaSequenceExecutionStub methodsFor: 'testing' stamp: 'yo 9/24/2004 06:08'!
isSequentialStub

	^ true.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 4/2/2005 17:41'!
noteDecimalPlaces: aNumber forGetter: aGetter
! !

!KedamaSequenceExecutionStub methodsFor: 'player protocol' stamp: 'yo 9/24/2004 10:03'!
noteRenameOf: oldSlotName to: newSlotName inPlayer: aPlayer

	self compileAllAccessors
! !

!KedamaSequenceExecutionStub methodsFor: 'primitives' stamp: 'yo 10/10/2004 09:14'!
primForwardAt: i xArray: xArray yArray: yArray headingArray: headingArray value: value destWidth: destWidth destHeight: destHeight leftEdgeMode: leftEdgeMode rightEdgeMode: rightEdgeMode topEdgeMode: topEdgeMode bottomEdgeMode: bottomEdgeMode

	| dist newX newY |
	<primitive: 'primScalarForward' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primScalarForward."

	dist _ value.
	newX _ (xArray at: i) + (dist asFloat * (headingArray at: i) cos).
	newY _ (yArray at: i) - (dist asFloat * (headingArray at: i) sin).
	KedamaMorph scalarXAt: i xArray: xArray headingArray: headingArray value: newX destWidth: destWidth leftEdgeMode: leftEdgeMode rightEdgeMode: rightEdgeMode.
	KedamaMorph scalarYAt: i yArray: yArray headingArray: headingArray value: newY destHeight: destHeight topEdgeMode: topEdgeMode bottomEdgeMode: bottomEdgeMode.
! !

!KedamaSequenceExecutionStub methodsFor: 'primitives' stamp: 'yo 10/23/2004 18:55'!
primGetAngleToX: toX toY: toY fromX: fromX fromY: fromY

	| ret |
	<primitive: 'scalarGetAngleTo' module:'KedamaPlugin'>
	ret _ ((toX - fromX)@(toY - fromY)) theta radiansToDegrees + 90.0.
	ret > 360.0 ifTrue: [^ ret - 360.0].
	^ ret.
! !

!KedamaSequenceExecutionStub methodsFor: 'primitives' stamp: 'yo 10/22/2004 15:36'!
primGetDistanceToX: toX toY: toY fromX: fromX fromY: fromY

	<primitive: 'scalarGetDistanceTo' module:'KedamaPlugin'>
	^ ((fromX - toX) squared + (fromY - toY)) squared sqrt.
! !

!KedamaSequenceExecutionStub methodsFor: 'primitives' stamp: 'yo 10/11/2004 14:33'!
primGetHeadingAt: i headingArray: headingArray

	| heading |
	<primitive: 'getScalarHeading' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #getScalarHeading."

	heading _ headingArray at: i.
	^ heading _ KedamaMorph radiansToDegrees: heading.
! !

!KedamaSequenceExecutionStub methodsFor: 'primitives' stamp: 'yo 10/11/2004 14:07'!
primSetHeadingAt: i headingArray: headingArray value: heading

	| rad |
	<primitive: 'setScalarHeading' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #setScalarHeading."

	rad _ KedamaMorph degreesToRadians: heading.
	headingArray at: index put: rad.
! !

!KedamaSequenceExecutionStub methodsFor: 'primitives' stamp: 'yo 10/11/2004 11:15'!
primSetX: xArray xIndex: xIndex headingArray: headingArray value: value destWidth: destWidth leftEdgeMode: leftEdgeMode rightEdgeMode: rightEdgeMode

	<primitive: 'turtleScalarSetX' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #turtleScalarSetX."
	KedamaMorph scalarXAt: xIndex xArray: xArray headingArray: headingArray value: value destWidth: destWidth leftEdgeMode: leftEdgeMode rightEdgeMode: rightEdgeMode.
! !

!KedamaSequenceExecutionStub methodsFor: 'primitives' stamp: 'yo 10/11/2004 11:16'!
primSetY: yArray yIndex: yIndex headingArray: headingArray value: value destHeight: destHeight topEdgeMode: topEdgeMode bottomEdgeMode: bottomEdgeMode

	<primitive: 'turtleScalarSetY' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #turtleScalarSetY."
	KedamaMorph scalarYAt: yIndex yArray: yArray headingArray: headingArray value: value destHeight: destHeight topEdgeMode: topEdgeMode bottomEdgeMode: bottomEdgeMode.
! !

!KedamaSequenceExecutionStub methodsFor: 'printing' stamp: 'yo 3/31/2005 14:12'!
printOn: aStream

	aStream nextPutAll: 'KedamaExecutionStub for ('.
	super printOn: aStream.
	aStream nextPut: $).
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/13/2004 06:42'!
setBlueComponentIn: aPatch to: value

	| i pix |
	i _ self index.
	pix _ aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i).
	pix _ (pix bitAnd: 16rFFFF00) bitOr: ((value asInteger bitAnd: 16rFF) bitShift: 16).
	aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i) put: pix.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 9/27/2004 16:35'!
setBooleanVarAt: arrayIndex at: i put: v

	(turtles arrays at: arrayIndex) at: i put: ((v == true or: [v isNumber and: [v ~= 0]]) ifTrue: [1] ifFalse: [0]).
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 4/5/2005 10:59'!
setColor: aColor

	(turtles arrays at: 5) at: self index put: ((aColor pixelValueForDepth: 32) bitAnd: 16rFFFFFF).
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 9/27/2004 16:34'!
setColorVarAt: arrayIndex at: i put: v

	| val |
	val _ v isColor ifTrue: [v pixelValueForDepth: 32] ifFalse: [v].
	(turtles arrays at: arrayIndex) at: i put: val.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/13/2004 06:42'!
setGreenComponentIn: aPatch to: value

	| i pix |
	i _ self index.
	pix _ aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i).
	pix _ (pix bitAnd: 16rFF00FF) bitOr: ((value asInteger bitAnd: 16rFF) bitShift: 8).
	aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i) put: pix.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/11/2004 14:12'!
setHeading: degrees

	^ self primSetHeadingAt: self index headingArray: (turtles arrays at: 4) value: degrees asFloat.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/29/2004 09:05'!
setNormal: degrees

	| headingRadians |
	headingRadians _ ((90.0 - degrees) \\ 360.0) degreesToRadians.

	(turtles arrays at: 7) at: self index put: headingRadians.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 9/27/2004 16:32'!
setNumberVarAt: arrayIndex at: i put: v

	(turtles arrays at: arrayIndex) at: i put: v.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing - private' stamp: 'yo 9/27/2004 16:32'!
setObjectVarAt: arrayIndex at: i put: v

	(turtles arrays at: arrayIndex) at: i put: v.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 18:19'!
setPatchValueIn: aPatch to: value

	| i |
	i _ self index.
	aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i) put: value.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 10/13/2004 06:42'!
setRedComponentIn: aPatch to: value

	| i pix |
	i _ self index.
	pix _ aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i).
	pix _ (pix bitAnd: 16rFFFF) bitOr: ((value asInteger bitAnd: 16rFF) bitShift: 16).
	aPatch costume renderedMorph pixelAtX: ((turtles arrays at: 2) at: i) y: ((turtles arrays at: 3) at: i) put: pix.
! !

!KedamaSequenceExecutionStub methodsFor: 'player protocol' stamp: 'yo 9/24/2004 09:47'!
setScalarSlotTypeFor: slotName typeChosen: typeChosen

	self compileAllAccessors.
! !

!KedamaSequenceExecutionStub methodsFor: 'player protocol' stamp: 'yo 4/4/2005 20:12'!
setTurtleCount: aNumber

	^ exampler setTurtleCount: aNumber.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 3/7/2005 07:32'!
setTurtleVisible: aBoolean

	^ (turtles arrays at: 6) at: self index put: (aBoolean ifTrue: [1] ifFalse: [0])
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:33'!
setX: val

	^ self primSetX: (turtles arrays at: 2) xIndex: self index headingArray: (turtles arrays at: 4) value: val asFloat destWidth: kedamaWorld wrapX leftEdgeMode: kedamaWorld leftEdgeModeMnemonic rightEdgeMode: kedamaWorld rightEdgeModeMnemonic.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:33'!
setY: val

	^ self primSetY: (turtles arrays at: 3) yIndex: self index headingArray: (turtles arrays at: 4) value: val asFloat destHeight: kedamaWorld wrapY topEdgeMode: kedamaWorld topEdgeModeMnemonic bottomEdgeMode: kedamaWorld bottomEdgeModeMnemonic.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/27/2004 16:34'!
show

	(turtles arrays at: 6) at: self index put: 1.
! !

!KedamaSequenceExecutionStub methodsFor: 'printing' stamp: 'yo 4/4/2005 16:14'!
storeCodeOn: aStream indent: indent

	aStream nextPutAll: '(('.
	aStream nextPutAll: exampler uniqueNameForReference.
	aStream nextPutAll: ') clonedSequentialStub who: '.
	aStream nextPutAll: who printString.
	aStream nextPutAll: ')'.
! !

!KedamaSequenceExecutionStub methodsFor: 'player commands' stamp: 'yo 9/16/2004 23:16'!
turn: degrees
	"Turn by the given number of degrees."

	self setHeading: (self getHeading + degrees).
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing' stamp: 'yo 9/28/2004 11:15'!
turtles: t

	turtles _ t.
! !

!KedamaSequenceExecutionStub methodsFor: 'accessing' stamp: 'yo 9/18/2004 15:34'!
who: w

	who _ w.
	index _ nil.
! !

KedamaExamplerPlayer subclass: #KedamaTurtleVectorPlayer
	instanceVariableNames: 'exampler info types arrays deletingIndex whoTable whoTableBase whoTableValid turtlesMap turtleMapValid lastWho lastWhoStub'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaTurtleVectorPlayer methodsFor: 'turtle map' stamp: 'yo 10/22/2004 07:10'!
aTurtleAtX: xPos y: yPos

	| w x y index who stub |
	turtleMapValid ifFalse: [
		self makeTurtlesMap.
	].

	w _ kedamaWorld dimensions x.
	x _ xPos truncated.
	y _ yPos truncated.
	x < 0 ifTrue: [^ nil].
	x >= w ifTrue: [^ nil].
	y < 0 ifTrue: [^ nil].
	y >= kedamaWorld dimensions y ifTrue: [^ nil].
	index _ (w * y) + x + 1.
	who _ turtlesMap at: index.
	who = 0 ifTrue: [^ nil].
	who = lastWho ifTrue: [^ lastWhoStub].
	stub _ exampler clonedSequentialStub.
	stub who: who.
	lastWho _ who.
	^ lastWhoStub _ stub.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player protocol' stamp: 'yo 10/24/2004 18:45'!
addInstanceVarNamed: aName withValue: aValue

	| newArray |
	(aValue isKindOf: Number) ifTrue: [
		newArray _ KedamaFloatArray new: self size.
	].
	(aValue isKindOf: Color) ifTrue: [
		newArray _ WordArray new: self size.
	].
	(aValue isKindOf: Player) ifTrue: [
		newArray _ Array new: self size.
	].

	arrays _ arrays, newArray.
	info at: aName asSymbol put: arrays size.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player protocol' stamp: 'yo 10/24/2004 18:45'!
addInstanceVarVectorNamed: aName withValue: aValue

	| newArray |
	newArray _ KedamaFloatArray new: self size.
	arrays _ arrays, (Array with: newArray).
	newArray atAllPut: aValue.
	info at: aName asSymbol put: arrays size.
	types at: arrays size put: #Number.
! !

!KedamaTurtleVectorPlayer methodsFor: 'add turtles' stamp: 'yo 10/30/2004 18:56'!
addTurtlesCount: count ofPrototype: prototype for: aKedamaWorld positionAndColorArray: positionAndColorArray

	| index array defaultValue newArray oldCount |
	oldCount _ self size.
	info associationsDo: [:assoc |
		index _ info at: assoc key.
		array _ arrays at: (info at: assoc key).
		defaultValue _ prototype at: index.
		newArray _ array class new: count.
		(#(who x y heading color normal) includes: assoc key) ifFalse: [
			newArray atAllPut: defaultValue.
		].
		assoc key = #x ifTrue: [newArray replaceFrom: 1 to: newArray size with: positionAndColorArray first startingAt: 1].
		assoc key = #y ifTrue: [newArray replaceFrom: 1 to: newArray size with: positionAndColorArray second startingAt: 1].
		assoc key = #color ifTrue: [newArray replaceFrom: 1 to: newArray size with: positionAndColorArray third startingAt: 1].
		assoc key = #heading ifTrue: [newArray atAllPut: 1.57079631 "Float pi / 2.0"].
		assoc key = #normal ifTrue: [newArray atAllPut: 1.57079631 "Float pi / 2.0"].

		arrays at: (assoc value) put: array, newArray.
	].

	#(who) do: [:name |
		self setInitialValueOf: name from: oldCount + 1 to: self size for: aKedamaWorld.
	].
	whoTableValid _ false.
	turtleMapValid _ false.

	
! !

!KedamaTurtleVectorPlayer methodsFor: 'add turtles' stamp: 'yo 9/18/2004 15:27'!
addTurtlesCount: count ofPrototype: prototype for: aKedamaWorld randomize: randomizeFlag

	| index array defaultValue newArray oldCount |
	oldCount _ self size.
	info associationsDo: [:assoc |
		index _ info at: assoc key.
		array _ arrays at: index.
		defaultValue _ prototype at: index.
		newArray _ array class new: count.
		newArray atAllPut: defaultValue.
		arrays at: index put: (array, newArray).
	].

	self setInitialValueOf: #who from: oldCount + 1 to: self size for: aKedamaWorld.

	randomizeFlag ifTrue: [
		#(x y heading) do: [:name |
			self setInitialValueOf: name from: oldCount + 1 to: self size for: aKedamaWorld.
		].
	].
	whoTableValid _ false.
! !

!KedamaTurtleVectorPlayer methodsFor: 'private' stamp: 'yo 10/24/2004 18:45'!
arrayForType: typeName

	| newArray |
	(typeName = #Number) ifTrue: [
		newArray _ KedamaFloatArray new: self size.
	].
	(typeName = #Color) ifTrue: [
		newArray _ WordArray new: self size.
	].
	(typeName = #Boolean) ifTrue: [
		newArray _ ByteArray new: self size.
	].

	newArray ifNil: [
		newArray _ Array new: self size.
	].

	^ newArray.
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing' stamp: 'yo 9/16/2004 17:43'!
arrays

	^ arrays.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/24/2004 18:45'!
basicGetAngleTo: players

	| ret p xArray yArray result |
	players isCollection ifFalse: [
		p _ players
	].
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	result _ KedamaFloatArray new: self size.
	1 to: self size do: [:index |
		players isCollection ifTrue: [
			p _ players at: index.
		].
		ret _ ((p getX - (xArray at: index))@(p getY - (yArray at: index))) theta radiansToDegrees + 90.0.
		ret > 360.0 ifTrue: [ret _ ret - 360.0].
		result at: index put: ret.
	].
	^ result.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/24/2004 18:45'!
basicGetDistanceTo: players

	| p xArray yArray result |
	players isCollection ifFalse: [
		p _ players
	].
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	result _ KedamaFloatArray new: self size.
	1 to: self size do: [:index |
		players isCollection ifTrue: [
			p _ players at: index.
		].
		result at: index put: ((p getX - (xArray at: index)) squared + (p getY - (yArray at: index)) squared) sqrt.

	].
	^ result.
! !

!KedamaTurtleVectorPlayer methodsFor: 'turtle map' stamp: 'yo 9/29/2004 17:24'!
basicMakeTurtlesMap

	| x y xArray yArray width height mapIndex whoArray |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	whoArray _ arrays at: 1.
	width _ kedamaWorld dimensions x.
	height _ kedamaWorld dimensions y.
	turtlesMap atAllPut: 0.

	1 to: self size do: [:index |
		x _ (xArray at: index) truncated.
		y _ (yArray at: index) truncated.
		mapIndex _ (width * y) + x + 1.
		(0 < mapIndex and: [mapIndex <= turtlesMap size]) ifTrue: [
			turtlesMap at: mapIndex put: (whoArray at: index).
		].
	].

	turtleMapValid _ true.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 4/5/2005 10:58'!
color: c

	c isColor ifTrue: [
		self setColorVarAt: 5 put: ((c pixelValueForDepth: 32) bitAnd: 16rFFFFFF).
	] ifFalse: [
		self setColorVarAt: 5 put: c.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 4/5/2005 11:04'!
colorFromPatch: aPatch

	| xArray yArray cArray patch |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	cArray _ arrays at: 5.
	patch _ aPatch costume renderedMorph.
	1 to: self size do: [:i |
		cArray at: i put: ((patch pixelAtX: (xArray at: i) y: (yArray at: i)) bitAnd: 16rFFFFFF).
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/12/2004 23:14'!
colorToPatch: aPatch

	| xArray yArray cArray patch |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	cArray _ arrays at: 5.
	patch _ aPatch costume renderedMorph.
	1 to: self size do: [:i |
		patch pixelAtX: (xArray at: i) y: (yArray at: i) put: (cArray at: i).
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player protocol' stamp: 'yo 9/20/2004 12:10'!
compileAllAccessors

	info keys asArray do: [:k |
		(#(who x y heading color visible) includes: k) ifFalse: [
			self compileVectorInstVarAccessorsFor: k.
		].
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player protocol' stamp: 'yo 9/20/2004 12:00'!
compileVectorInstVarAccessorsFor: varName

	| nameString index type setPhrase |
	nameString _ varName asString capitalized.
	index _ info at: varName asSymbol.
	self class compileSilently: ('get', nameString, '
	^ ', '(arrays at: ', index printString, ')')
		classified: 'access'.

	type _ types at: index.
	type = #Number ifTrue: [
		setPhrase _ 'setNumberVarAt:'.
	].
	type = #Boolean ifTrue: [
		setPhrase _ 'setBooleanVarAt:'.
	].
	type = #Color ifTrue: [
		setPhrase _ 'setColorVarAt:'.
	].
	setPhrase ifNil: [setPhrase _ 'setObjectVarAt:'].

	self class compileSilently: ('set', nameString, ': xxxArg
	self ', setPhrase, index printString, ' put: xxxArg')
		classified: 'access'! !

!KedamaTurtleVectorPlayer methodsFor: 'deleting' stamp: 'yo 10/8/2004 11:04'!
delete

	| anInstance |
	exampler _ nil.
	arrays _ nil.
	whoTable _ nil.
	turtlesMap _ nil.
	self class removeFromSystem: false.
	anInstance := UnscriptedPlayer new.
	self become: anInstance.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/26/2004 11:34'!
deleteTurtleID: who

	| whoArray whoIndex newArray |
	whoArray _ arrays at: 1.
	whoIndex _ whoArray indexOf: who ifAbsent: [^ self].
	deletingIndex _ whoIndex - 1.
	arrays withIndexDo: [:array :index |
		newArray _ (array copyFrom: 1 to: whoIndex - 1), (array copyFrom: whoIndex + 1 to: array size).
		arrays at: index put: newArray.
	].
	whoTableValid _ false.
	turtleMapValid _ false.
! !

!KedamaTurtleVectorPlayer methodsFor: 'command execution' stamp: 'yo 10/13/2004 06:49'!
doCommand: aBlock

	^ aBlock value: self.
! !

!KedamaTurtleVectorPlayer methodsFor: 'command execution' stamp: 'yo 9/28/2004 14:04'!
doDieCommand: aBlock

	^ exampler doDieCommand: aBlock.
! !

!KedamaTurtleVectorPlayer methodsFor: 'command execution' stamp: 'yo 9/17/2004 09:25'!
doExamplerCommand: aBlock
! !

!KedamaTurtleVectorPlayer methodsFor: 'command execution' stamp: 'yo 9/29/2004 15:15'!
doSequentialCommand: aBlock

	^ exampler doSequentialCommand: aBlock.
! !

!KedamaTurtleVectorPlayer methodsFor: 'displaying' stamp: 'yo 9/16/2004 21:40'!
drawOn: aForm

	| xArray yArray colorArray visibleArray bits x y visible bitsIndex dimX dimY ret |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	colorArray _ arrays at: 5.
	visibleArray _ arrays at: 6.
	bits _ aForm bits.
	dimX _ aForm width.
	dimY _ aForm height.

	ret _ self primDrawOn: bits destWidth: dimX destHeight: dimY xArray: xArray yArray: yArray colorArray: colorArray visibleArray: visibleArray.
	ret ifNotNil: [^ self].

	1 to: self size do: [:index |
		x _ (xArray at: index) truncated.
		y _ (yArray at: index) truncated.
		visible _ (visibleArray at: index) ~= 0.
		(visible and: [((x >= 0) and: [y >= 0]) and: [(x < dimX) and: [y < dimY]]]) ifTrue: [
			bitsIndex _ ((y * dimX) + x) + 1.
			bits at: bitsIndex put: (colorArray at: index).
		].
	].

! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing' stamp: 'yo 9/28/2004 14:02'!
exampler: e

	exampler _ e.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/30/2004 11:13'!
forward: val

	exampler getGrouped ifFalse: [
		self primForwardXArray: (arrays at: 2) yArray: (arrays at: 3) headingArray: (arrays at: 4) value: (val isNumber ifTrue: [val asFloat] ifFalse: [val]) destWidth: kedamaWorld wrapX asFloat destHeight: kedamaWorld wrapY asFloat leftEdgeMode: kedamaWorld leftEdgeModeMnemonic rightEdgeMode: kedamaWorld rightEdgeModeMnemonic topEdgeMode: kedamaWorld topEdgeModeMnemonic bottomEdgeMode: kedamaWorld bottomEdgeModeMnemonic.
	] ifTrue: [
		self groupForward: val
	].
	turtleMapValid _ false.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/24/2004 18:45'!
getAngleTo: players

	| p xArray yArray result pX pY xy |
	players isCollection ifFalse: [
		p _ players
	].
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	result _ KedamaFloatArray new: self size.
	players isCollection ifTrue: [
		pX _ KedamaFloatArray new: players size.
		pY _ KedamaFloatArray new: players size.
		1 to: players size do: [:i |
			xy _ (players at: i) getXAndY.
			pX at: i put: xy x.
			pY at: i put: xy y.
		].
	] ifFalse: [
		xy _ p getXAndY.
		pX _ xy x.
		pY _ xy y.
	].
	^ self primGetAngleToX: pX toY: pY xArray: xArray yArray: yArray resultInto: result.

! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/25/2004 11:34'!
getBlueComponentIn: aPatch

	| pix xArray yArray patch w |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	patch _ aPatch costume renderedMorph.
	w _ WordArray new: self size.
	1 to: self size do: [:i |
		pix _ patch pixelAtX: (xArray at: i) y: (yArray at: i).
		w at: i put: (pix bitAnd: 16rFF).
	].
	^ w.
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing - private' stamp: 'yo 9/20/2004 11:06'!
getBooleanVarAt: index

	^ arrays at: index.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/20/2004 11:12'!
getColor

	^ self getVarAt: 5.
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing - private' stamp: 'yo 9/20/2004 11:04'!
getColorVarAt: index

	^ (arrays at: index) collect: [:c | Color colorFromPixelValue: c depth: 32].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/24/2004 18:45'!
getDistanceTo: players

	| p xArray yArray result pX pY xy |
	players isCollection ifFalse: [
		p _ players
	].
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	result _ KedamaFloatArray new: self size.
	players isCollection ifTrue: [
		pX _ KedamaFloatArray new: players size.
		pY _ KedamaFloatArray new: players size.
		1 to: players size do: [:i |
			xy _ (players at: i) getXAndY.
			pX at: i put: xy x.
			pY at: i put: xy y.
		].
	] ifFalse: [
		xy _ p getXAndY.
		pX _ xy x.
		pY _ xy y.
	].
	^ self primGetDistanceToX: pX toY: pY xArray: xArray yArray: yArray resultInto: result.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/25/2004 11:34'!
getGreenComponentIn: aPatch

	| pix xArray yArray patch w |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	patch _ aPatch costume renderedMorph.
	w _ WordArray new: self size.
	1 to: self size do: [:i |
		pix _ patch pixelAtX: (xArray at: i) y: (yArray at: i).
		w at: i put: ((pix bitShift: -8) bitAnd: 16rFF).
	].
	^ w.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/29/2004 15:11'!
getGrouped

	^ exampler getGrouped.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/24/2004 18:45'!
getHeading

	| heading result ret |
	heading _ (arrays at: 4).
	result _ KedamaFloatArray new: heading size.
	ret _ self primGetHeading: heading into: result.
	ret ifNotNil: [^ result].
	! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/24/2004 18:46'!
getNormal

	| result ret normal |
	normal _ (arrays at: 7).
	result _ KedamaFloatArray new: normal size.
	ret _ self primGetHeading: normal into: result.
	ret ifNotNil: [^ result].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/25/2004 16:44'!
getPatchValueIn: aPatch

	| w patch xArray yArray |
	w _ WordArray new: self size.
	patch _ aPatch costume renderedMorph.
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	patch pixelsAtXArray: xArray yArray: yArray into: w.
	^ w.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/25/2004 11:35'!
getRedComponentIn: aPatch

	| pix xArray yArray patch w |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	patch _ aPatch costume renderedMorph.
	w _ WordArray new: self size.
	1 to: self size do: [:i |
		pix _ patch pixelAtX: (xArray at: i) y: (yArray at: i).
		w at: i put: ((pix bitShift: -16) bitAnd: 16rFF).
	].
	^ w.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 3/7/2005 08:22'!
getTurtleAt: aPlayer

	| xy |
	aPlayer isCollection ifTrue: [
		self error: 'should not happen'.
	].
	xy _ aPlayer getXAndY.
	^ (self aTurtleAtX: xy x y: xy y) ifNil: [^ aPlayer].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 3/7/2005 08:37'!
getTurtleOf: aBreedPlayer

	| xy |
	aBreedPlayer isCollection ifTrue: [
		"self error: 'should not happen'."
		^ aBreedPlayer.
	].
	xy _ aBreedPlayer getXAndY.
	^ (self aTurtleAtX: xy x y: xy y) ifNil: [^ aBreedPlayer].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 3/7/2005 07:32'!
getTurtleVisible

	^ (turtles arrays at: 6) collect: [:t | t = 1].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/24/2004 18:46'!
getUphillIn: aPatch

	| xArray yArray headingArray result patch |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	headingArray _ arrays at: 4.
	result _ KedamaFloatArray new: self size.
	patch _ aPatch costume renderedMorph.
	1 to: self size do: [:index |
		result at: index put: (patch
			uphillForTurtleX: (xArray at: index)
			turtleY: (yArray at: index)
			turtleHeading: (headingArray at: index)).
	].
	^ result.
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing - private' stamp: 'yo 9/20/2004 11:06'!
getVarAt: index

	^ arrays at: index.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/29/2004 14:21'!
getX

	| xArray |
	exampler getGrouped ifFalse: [
		^ arrays at: 2.
	] ifTrue: [
		xArray _ arrays at: 2.
		xArray size = 0 ifTrue: [^ exampler getX].
		^ xArray first.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/29/2004 14:22'!
getY

	| yArray |
	exampler getGrouped ifFalse: [
		^ arrays at: 3.
	] ifTrue: [
		yArray _ arrays at: 3.
		yArray size = 0 ifTrue: [^ exampler getY].
		^ yArray first.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'private' stamp: 'yo 9/28/2004 17:16'!
groupForward: dist

	| x y headingRadians |
	self size = 0 ifTrue: [^ self].

	x _ (arrays at: 2) first.
	y _ (arrays at: 3) first.
	headingRadians _ (arrays at: 4) first.
	self groupSetX: (x + (dist asFloat * headingRadians cos)).
	self groupSetY: (y - (dist asFloat * headingRadians sin)).
! !

!KedamaTurtleVectorPlayer methodsFor: 'private' stamp: 'yo 9/29/2004 08:35'!
groupSetX: val

	| xArray headingArray origX origHeading leftEdgeMode rightEdgeMode newArray wrapX minX maxX |
	self size = 0 ifTrue: [^ self].
	xArray _ arrays at: 2.
	headingArray _ arrays at: 4.
	
	origX _ xArray first.
	origHeading _ headingArray first.

	leftEdgeMode _ kedamaWorld leftEdgeModeMnemonic.
	rightEdgeMode _ kedamaWorld rightEdgeModeMnemonic.

	newArray _ xArray collect: [:e | e + val - origX].
	wrapX _ kedamaWorld wrapX.
	minX _ newArray min.
	maxX _ newArray max.
	((minX < 0.0) not and: [(maxX >= wrapX) not]) ifTrue: [
		arrays at: 2 put: newArray.
		^ self.
	].

	minX < 0.0 ifTrue: [
		leftEdgeMode = 1 ifTrue: [
			newArray withIndexDo: [:e :i |
				e < 0.0 ifTrue: [newArray at: i put: e + wrapX].
			].
		].
		leftEdgeMode = 2 ifTrue: [
			newArray withIndexDo: [:e :i |
				newArray at: i put: e - minX.
			].
		].
		leftEdgeMode = 3 ifTrue: [
			newArray withIndexDo: [:e :i |
				newArray at: i put: e + (minX * -2.0).
			].
		].		
	].

	maxX >= wrapX ifTrue: [
		rightEdgeMode = 1 ifTrue: [
			newArray withIndexDo: [:e :i |
				e >= wrapX ifTrue: [newArray at: i put: e - wrapX].
			].
		].
		rightEdgeMode = 2 ifTrue: [
			newArray withIndexDo: [:e :i |
				newArray at: i put: e - (maxX - wrapX) - 2.35099e-038.
			].
		].
		rightEdgeMode = 3 ifTrue: [
			newArray withIndexDo: [:e :i |
				newArray at: i put: e - ((maxX - wrapX) * 2.0) - 2.35099e-038.
			].
		].
	].

	arrays at: 2 put: newArray.
! !

!KedamaTurtleVectorPlayer methodsFor: 'private' stamp: 'yo 9/28/2004 17:58'!
groupSetY: val

	| yArray headingArray origY origHeading topEdgeMode bottomEdgeMode newArray wrapY minY maxY |
	self size = 0 ifTrue: [^ self].
	yArray _ arrays at: 3.
	headingArray _ arrays at: 4.
	
	origY _ yArray first.
	origHeading _ headingArray first.

	topEdgeMode _ kedamaWorld topEdgeModeMnemonic.
	bottomEdgeMode _ kedamaWorld bottomEdgeModeMnemonic.

	newArray _ yArray collect: [:e | e + val - origY].
	wrapY _ kedamaWorld wrapY.
	minY _ newArray min.
	maxY _ newArray max.
	((minY < 0.0) not and: [(maxY >= wrapY) not]) ifTrue: [
		arrays at: 3 put: newArray.
		^ self.
	].

	minY < 0.0 ifTrue: [
		topEdgeMode = 1 ifTrue: [
			newArray withIndexDo: [:e :i |
				e < 0.0 ifTrue: [newArray at: i put: e + wrapY].
			].
		].
		topEdgeMode = 2 ifTrue: [
			newArray withIndexDo: [:e :i |
				newArray at: i put: e - minY.
			].
		].
		topEdgeMode = 3 ifTrue: [
			newArray withIndexDo: [:e :i |
				newArray at: i put: e + (minY * -2.0).
			].
		].		
	].

	maxY >= wrapY ifTrue: [
		bottomEdgeMode = 1 ifTrue: [
			newArray withIndexDo: [:e :i |
				e >= wrapY ifTrue: [newArray at: i put: e - wrapY].
			].
		].
		bottomEdgeMode = 2 ifTrue: [
			newArray withIndexDo: [:e :i |
				newArray at: i put: e - (maxY - wrapY) - 2.35099e-038.
			].
		].
		bottomEdgeMode = 3 ifTrue: [
			newArray withIndexDo: [:e :i |
				newArray at: i put: e - ((maxY - wrapY) * 2.0) - 2.35099e-038.
			].
		].
	].

	arrays at: 3 put: newArray.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/16/2004 22:59'!
heading

	^ self getHeading.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/16/2004 23:02'!
heading: degrees

	| deg |
	deg _ degrees isNumber ifTrue: [degrees asFloat] ifFalse: [degrees].
	self primSetHeading: (arrays at: 4) from: deg.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/17/2004 15:46'!
hide

	(arrays at: 6) atAllPut: 0.
! !

!KedamaTurtleVectorPlayer methodsFor: 'enumeration' stamp: 'yo 9/27/2004 15:16'!
indexOf: who

	| whoArray |
	whoTableValid ifTrue: [^ whoTable at: (who - whoTableBase)].

	whoArray _ arrays at: 1.

	whoArray size = 0 ifTrue: [^ 0].

	whoTableBase _ whoArray first - 1.
	whoTable _ WordArray new: whoArray last - whoTableBase.
	1 to: whoArray size do: [:w |
		whoTable at: (whoArray at: w) - whoTableBase put: w.
	].
	whoTableValid _ true.

	^ whoTable at: (who - whoTableBase).

! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing' stamp: 'yo 9/14/2004 18:13'!
info

	^ info.
! !

!KedamaTurtleVectorPlayer methodsFor: 'initialization' stamp: 'yo 10/24/2004 18:46'!
initialize

	super initialize.
	info _ IdentityDictionary new.
	info at: #who put: 1.
	info at: #x put: 2.
	info at: #y put: 3.
	info at: #heading put: 4.
	info at: #color put: 5.
	info at: #visible put: 6.
	info at: #normal put: 7.

	arrays _ Array new: 7.
	arrays at: (info at: #who) put: (WordArray new: 0).
	arrays at: (info at: #x) put: (KedamaFloatArray new: 0).
	arrays at: (info at: #y) put: (KedamaFloatArray new: 0).
	arrays at: (info at: #heading) put: (KedamaFloatArray new: 0).
	arrays at: (info at: #color) put: (WordArray new: 0).
	arrays at: (info at: #visible) put: (ByteArray new: 0).
	arrays at: (info at: #normal) put: (KedamaFloatArray new: 0).

	types _ Array new: 64.

	types at: 1 put: #Integer.
	types at: 2 put: #Number.
	types at: 3 put: #Number.
	types at: 4 put: #Number.
	types at: 5 put: #Color.
	types at: 6 put: #Boolean.
	types at: 7 put: #Number.

	whoTableValid _ false.
	turtleMapValid _ false.


! !

!KedamaTurtleVectorPlayer methodsFor: 'enumeration' stamp: 'yo 9/18/2004 12:02'!
initializeDeletingIndex

	deletingIndex _ 0.
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing' stamp: 'yo 9/14/2004 18:13'!
instSize

	^ arrays size.
! !

!KedamaTurtleVectorPlayer methodsFor: 'turtle map' stamp: 'yo 9/28/2004 10:21'!
invalidateTurtleMap

	turtleMapValid _ false.
! !

!KedamaTurtleVectorPlayer methodsFor: 'add turtles' stamp: 'yo 9/14/2004 18:10'!
makePrototypeFromFirstInstance

	^ (1 to: arrays size) collect: [:index |
		(arrays at: index) first
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'turtle map' stamp: 'yo 9/29/2004 17:31'!
makeTurtlesMap

	| xArray yArray width height whoArray ret |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	whoArray _ arrays at: 1.
	width _ kedamaWorld dimensions x.
	height _ kedamaWorld dimensions y.
	turtlesMap ifNil: [turtlesMap _ WordArray new: width * height].

	ret _ self primMakeTurtlesMap: turtlesMap whoArray: whoArray xArray: xArray yArray: yArray width: width height: height.

	ret ifNil: [self basicMakeTurtlesMap].

	turtleMapValid _ true.
! !

!KedamaTurtleVectorPlayer methodsFor: 'enumeration' stamp: 'yo 9/18/2004 12:02'!
nextDeletingIndex

	^ deletingIndex _ deletingIndex + 1.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/29/2004 09:06'!
normal: degrees

	| deg |
	deg _ degrees isNumber ifTrue: [degrees asFloat] ifFalse: [degrees].
	self primSetHeading: (arrays at: 7) from: deg.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player protocol' stamp: 'yo 9/24/2004 10:03'!
noteRenameOf: oldSlotName to: newSlotName inPlayer: aPlayer

	self compileAllAccessors
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 10/11/2004 11:49'!
primDrawOn: destBits destWidth: destWidth destHeight: destHeight xArray: xArray yArray: yArray colorArray: colorArray visibleArray: visibleArray

	| x y visible bitsIndex |
	<primitive: 'drawTurtlesInArray' module: 'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #drawTurtlesInArray."

	1 to: xArray size do: [:i |
		x _ (xArray at: i) asInteger.
		y _ (yArray at: i) asInteger.
		visible _ (visibleArray at: i).
		(visible ~= 0 and: [((x >= 0) and: [y >= 0]) and: [(x < destWidth) and: [y < destHeight]]]) ifTrue: [
			bitsIndex _ ((y * destWidth) + x) + 1.
			destBits at: bitsIndex put: (colorArray at: i).
		]
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 10/11/2004 11:04'!
primForwardXArray: xArray yArray: yArray headingArray: headingArray value: v destWidth: destWidth destHeight: destHeight leftEdgeMode: leftEdgeMode rightEdgeMode: rightEdgeMode topEdgeMode: topEdgeMode bottomEdgeMode: bottomEdgeMode

	| dist newX newY |
	<primitive: 'primTurtlesForward' module: 'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primTurtlesForward."

	1 to: xArray size do: [:i |
		v isCollection ifTrue: [
			dist _ (v at: i) asFloat.
		] ifFalse: [
			dist _ v asFloat.
		].
		newX _ (xArray at: i) + (dist * (headingArray at: i) cos).
		newY _ (yArray at: i) - (dist * (headingArray at: i) sin).
		KedamaMorph scalarXAt: i xArray: xArray headingArray: headingArray value: newX destWidth: destWidth leftEdgeMode: leftEdgeMode rightEdgeMode: rightEdgeMode.
		KedamaMorph scalarYAt: i yArray: yArray headingArray: headingArray value: newY destHeight: destHeight topEdgeMode: topEdgeMode bottomEdgeMode: bottomEdgeMode.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 10/23/2004 22:19'!
primGetAngleToX: pX toY: pY xArray: xArray yArray: yArray resultInto: result

	| ppx ppy x y ret |
	<primitive: 'vectorGetAngleTo' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #vectorGetAngleTo."

	ppx _ pX.
	ppy _ pY.
	1 to: result size do: [:index |
		pX isCollection ifTrue: [
			ppx _ pX at: index.
			ppy _ pY at: index.
		].
		x _ ppx - (xArray at: index).
		y _ ppy - (yArray at: index).
		ret _ (x@y) theta radiansToDegrees + 90.0.
		ret > 360.0 ifTrue: [ret _ ret - 360.0].
		result at: index put: ret.
	].
	^ result.
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 10/23/2004 18:02'!
primGetDistanceToX: pX toY: pY xArray: xArray yArray: yArray resultInto: result

	| ppx ppy |
	<primitive: 'vectorGetDistanceTo' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #vectorGetDistanceTo."

	ppx _ pX.
	ppy _ pY.
	1 to: result size do: [:index |
		pX isCollection ifTrue: [
			ppx _ pX at: index.
			ppy _ pY at: index.
		].
		result at: index put: ((ppx - (xArray at: index)) squared + (ppy - (yArray at: index)) squared) sqrt.

	].
	^ result.
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 10/9/2004 17:29'!
primGetHeading: headingArray into: resultArray

	| heading |
	<primitive: 'getHeadingArrayInto' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #getHeadingArrayInto."

	1 to: headingArray size do: [:i |
		heading _ headingArray at: i.
		heading _ heading / 0.0174532925199433.
		heading _ 90.0 - heading.
		heading > 0.0 ifFalse: [heading _ heading + 360.0].
		resultArray at: i put: heading.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 9/29/2004 17:20'!
primMakeTurtlesMap: map whoArray: whoArray xArray: xArray yArray: yArray width: w height: h

	<primitive: 'makeTurtlesMap' module: 'KedamaPlugin'>
	^ nil.
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 3/7/2005 13:40'!
primRandomRange: range from: from to: to intoFloatArray: aFloatArray factor: factor

	<primitive: 'randomIntoFloatArray' module: 'KedamaPlugin'>

	from to: to do: [:index |
		aFloatArray at: index put: (kedamaWorld random: range) asFloat * factor.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 3/7/2005 13:30'!
primRandomRange: range from: from to: to intoIntegerArray: anIntegerArray factor: factor

	<primitive: 'randomIntoIntegerArray' module: 'KedamaPlugin'>

	from to: to do: [:index |
		anIntegerArray at: index put: ((kedamaWorld random: range) asFloat * factor) asInteger.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 10/11/2004 11:40'!
primSetHeading: headingArray from: val

	| heading |
	<primitive: 'setHeadingArrayFrom' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #setHeadingArrayFrom."

	val isCollection ifFalse: [
		heading _ val asFloat.
		heading _ KedamaMorph degreesToRadians: heading.
	].

	1 to: headingArray size do: [:i |
		val isCollection ifTrue: [
			heading _ val at: i.
			heading _ KedamaMorph degreesToRadians: heading.
		].
		headingArray at: i put: heading.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 10/11/2004 11:19'!
primSetX: xArray headingArray: headingArray value: v destWidth: destWidth leftEdgeMode: leftEdgeMode rightEdgeMode: rightEdgeMode

	| val newX |
	<primitive: 'turtlesSetX' module: 'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #turtlesSetX."

	v isCollection ifFalse: [
		val _ v asFloat.
	].

	1 to: xArray size do: [:i |
		v isCollection ifTrue: [
			newX _ v at: i.
		] ifFalse: [
			newX _ val.
		].
		KedamaMorph scalarXAt: i xArray: xArray headingArray: headingArray value: newX destWidth: destWidth leftEdgeMode: leftEdgeMode rightEdgeMode: rightEdgeMode.
	].

! !

!KedamaTurtleVectorPlayer methodsFor: 'primitives' stamp: 'yo 10/11/2004 11:21'!
primSetY: yArray headingArray: headingArray value: v destHeight: destHeight topEdgeMode: topEdgeMode bottomEdgeMode: bottomEdgeMode

	| val newY |
	<primitive: 'turtlesSetY' module: 'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #turtlesSetY."

	v isCollection ifFalse: [
		val _ v asFloat.
	].

	1 to: yArray size do: [:i |
		v isCollection ifTrue: [
			newY _ v at: i.
		] ifFalse: [
			newY _ val.
		].
		KedamaMorph scalarYAt: i yArray: yArray headingArray: headingArray value: newY destHeight: destHeight topEdgeMode: topEdgeMode bottomEdgeMode: bottomEdgeMode.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player protocol' stamp: 'yo 9/20/2004 12:20'!
removeSlotNamed: aSlotName

	| index newArrays |
	self basicRemoveSlotNamed: aSlotName.
	index _ info at: aSlotName asSymbol ifAbsent: [^ self].
	newArrays _ (arrays copyFrom: 1 to: index - 1), (arrays copyFrom: index + 1 to: arrays size).
	types replaceFrom: index to: types size with: types startingAt: index + 1.

	info removeKey: aSlotName asSymbol.
	arrays _ newArrays.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player protocol' stamp: 'yo 9/20/2004 16:34'!
removeVectorSlotNamed: aSlotName

	| index newArrays |
	index _ info at: aSlotName asSymbol ifAbsent: [^ self].
	newArrays _ (arrays copyFrom: 1 to: index - 1), (arrays copyFrom: index + 1 to: arrays size).
	types replaceFrom: index to: types size - 1 with: types startingAt: index + 1.

	info removeKey: aSlotName asSymbol.
	arrays _ newArrays.
	self compileAllAccessors.

! !

!KedamaTurtleVectorPlayer methodsFor: 'player protocol' stamp: 'yo 10/27/2004 16:29'!
renameVectorSlot: oldSlotName newSlotName: newSlotName

	| index |
	index _ info at: oldSlotName asSymbol ifAbsent: [^ self].
	info removeKey: oldSlotName asSymbol.
	info at: newSlotName put: index.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/13/2004 06:41'!
setBlueComponentIn: aPatch to: value

	| pix xArray yArray patch component |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	patch _ aPatch costume renderedMorph.
	value isCollection ifFalse: [
		component _ value asInteger bitAnd: 16rFF.
	].
	(1 to: self size) do: [:i |
		value isCollection ifTrue: [
			component _ (value at: i) asInteger bitAnd: 16rFF.
		].
		pix _ patch pixelAtX: (xArray at: i) y: (yArray at: i).
		pix _ (pix bitAnd: 16rFFFF00) bitOr: component.
		patch pixelAtX: (xArray at: i) y: (yArray at: i) put: pix.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing - private' stamp: 'yo 9/20/2004 11:20'!
setBooleanVarAt: index put: v

	(v == true or: [v == false]) ifTrue: [
		(arrays at: index) atAllPut: (v == true ifTrue: [1] ifFalse: [0]).
		^ self.
	].
	(arrays at: index) replaceFrom: 1 to: v size with: v startingAt: 1.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/20/2004 16:55'!
setColor: color

	^ self color: color.
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing - private' stamp: 'yo 4/5/2005 10:52'!
setColorVarAt: index put: cPixel

	cPixel isNumber ifTrue: [
		(arrays at: index) atAllPut: cPixel.
		^ self.
	].

	(arrays at: index) replaceFrom: 1 to: cPixel size with: cPixel startingAt: 1.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/13/2004 06:40'!
setGreenComponentIn: aPatch to: value

	| pix xArray yArray patch component |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	patch _ aPatch costume renderedMorph.
	value isCollection ifFalse: [
		component _ (value asInteger bitAnd: 16rFF) bitShift: 8.
	].
	(1 to: self size) do: [:i |
		value isCollection ifTrue: [
			component _ ((value at: i) asInteger bitAnd: 16rFF) bitShift: 8.
		].
		pix _ patch pixelAtX: (xArray at: i) y: (yArray at: i).
		pix _ (pix bitAnd: 16rFF00FF) bitOr: component.
		patch pixelAtX: (xArray at: i) y: (yArray at: i) put: pix.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/16/2004 22:59'!
setHeading: degrees

	^ self heading: degrees.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 3/7/2005 13:41'!
setInitialValueOf: name from: from to: to for: aKedamaWorld

	| array max |
	array _ arrays at: (info at: name).
	name = #who ifTrue: [
		from to: to do: [:index |
			array at: index put: (aKedamaWorld nextTurtleID).
		].
		^ self.
	].
	name = #x ifTrue: [
		max _ aKedamaWorld dimensions x * 100.
		self primRandomRange: max from: from to: to intoFloatArray: array factor: 0.01.
		^ self.
	].
	name = #y ifTrue: [
		max _ aKedamaWorld dimensions y * 100.
		self primRandomRange: max from: from to: to intoFloatArray: array factor: 0.01.
		^ self.
	].
	name = #heading ifTrue: [
		self primRandomRange: 36000 from: from to: to intoFloatArray: array factor: (0.01 *  0.0174532925199433).
		^ self.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/29/2004 09:06'!
setNormal: degrees

	^ self normal: degrees.
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing - private' stamp: 'yo 10/29/2004 19:35'!
setNumberVarAt: index put: v

	v isNumber ifTrue: [
		(arrays at: index) atAllPut: v.
		^ self.
	].
	(v isMemberOf: KedamaFloatArray) ifTrue: [
		(arrays at: index) replaceFrom: 1 to: v size with: v startingAt: 1.
		^ self.
	].
	(v isMemberOf: WordArray) ifTrue: [
		v primAddScalar: v and: 0.0 into: (arrays at: index).
		^ self.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing - private' stamp: 'yo 9/20/2004 11:21'!
setObjectVarAt: index put: v

	(v isKindOf: Array) ifFalse: [
		(arrays at: index) atAllPut: v.
		^ self.
	].
	(arrays at: index) replaceFrom: 1 to: v size with: v startingAt: 1.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/25/2004 16:57'!
setPatchValueIn: aPatch to: value

	| xArray yArray patchMorph |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	patchMorph _ aPatch costume renderedMorph.
	patchMorph setPixelsAtXArray: xArray yArray: yArray value: value.

! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 10/13/2004 06:40'!
setRedComponentIn: aPatch to: value

	| pix xArray yArray patch component |
	xArray _ arrays at: 2.
	yArray _ arrays at: 3.
	patch _ aPatch costume renderedMorph.
	value isCollection ifFalse: [
		component _ (value asInteger bitAnd: 16rFF) bitShift: 16.
	].
	(1 to: self size) do: [:i |
		value isCollection ifTrue: [
			component _ ((value at: i) asInteger bitAnd: 16rFF) bitShift: 16.
		].
		pix _ patch pixelAtX: (xArray at: i) y: (yArray at: i).
		pix _ (pix bitAnd: 16r00FFFF) bitOr: component.
		patch pixelAtX: (xArray at: i) y: (yArray at: i) put: pix.
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 3/7/2005 07:34'!
setTurtleVisible: aBooleanArray

	1 to: self size do: [:i |
		(turtles arrays at: 6) at: i put: ((aBooleanArray at: i) ifTrue: [1] ifFalse: [0]).
	].
! !

!KedamaTurtleVectorPlayer methodsFor: 'add turtles' stamp: 'yo 9/27/2004 16:14'!
setTurtlesCount: count prototype: prototype for: aKedamaWorld randomize: rondomizeFlag

	| anInteger array |
	anInteger _ count.
	count < 0 ifTrue: [anInteger _ 0].

	self size > anInteger ifTrue: [
		info associationsDo: [:assoc |
			array _ (arrays at: assoc value).
			array _ array copyFrom: 1 to: anInteger.
			arrays at: assoc value put: array.
		].
		turtleMapValid _ false.
		whoTableValid _ false.
	].

	self size < anInteger ifTrue: [
		self addTurtlesCount: (anInteger - self size) ofPrototype: prototype for: aKedamaWorld randomize: rondomizeFlag.
		turtleMapValid _ false.
		whoTableValid _ false.

	].


! !

!KedamaTurtleVectorPlayer methodsFor: 'player protocol' stamp: 'yo 9/20/2004 17:59'!
setVectorSlotTypeFor: slotName typeChosen: typeChosen

	| index initVar |
	index _ info at: slotName asSymbol.
	index = 0 ifTrue: [^ self].

	initVar _ self initialValueForSlotOfType: typeChosen.

	types at: index put: typeChosen.

	self compileAllAccessors.
	arrays at: index put: (self arrayForType: typeChosen).
	self perform: ('set', slotName capitalized, ':') asSymbol with: initVar.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/29/2004 14:13'!
setX: v

	exampler getGrouped ifFalse: [
		self
			primSetX: (arrays at: 2)
			headingArray: (arrays at: 4)
			value: (v isNumber ifTrue: [v asFloat] ifFalse: [v])
			destWidth: kedamaWorld wrapX
			leftEdgeMode: kedamaWorld leftEdgeModeMnemonic
			rightEdgeMode: kedamaWorld rightEdgeModeMnemonic.
	] ifTrue: [
		self groupSetX: v
	].
	turtleMapValid _ false.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/29/2004 14:13'!
setY: v

	exampler getGrouped ifFalse: [
		self
			primSetY: (arrays at: 3)
			headingArray: (arrays at: 4)
			value: (v isNumber ifTrue: [v asFloat] ifFalse: [v])
			destHeight: kedamaWorld wrapY
			topEdgeMode: kedamaWorld topEdgeModeMnemonic
			bottomEdgeMode: kedamaWorld bottomEdgeModeMnemonic.
	] ifTrue: [
		self groupSetY: v.
	].
	turtleMapValid _ false.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/17/2004 15:46'!
show

	(arrays at: 6) atAllPut: 1.
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing' stamp: 'yo 9/14/2004 17:43'!
size

	^ arrays first size.
! !

!KedamaTurtleVectorPlayer methodsFor: 'player commands' stamp: 'yo 9/16/2004 22:03'!
turn: degrees
	"Turn by the given number of degrees."

	self heading: (self heading + degrees).
! !

!KedamaTurtleVectorPlayer methodsFor: 'accessing' stamp: 'yo 9/23/2004 19:03'!
types

	^ types.
! !

!KedamaTurtleVectorPlayer methodsFor: 'private' stamp: 'yo 10/25/2004 16:23'!
useKedamaFloatArray

	| newArray |
	arrays withIndexDo: [:array :index |
		(array isMemberOf: FloatArray) ifTrue: [
			newArray _ KedamaFloatArray new: array size.
			newArray replaceFrom: 1 to: array size with: array startingAt: 1.
			arrays at: index put: newArray.
		].
	].
! !

Player subclass: #KedamaTurtlePlayer
	instanceVariableNames: 'world who x y headingRadians color visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaTurtlePlayer commentStamp: '<historical>' prior: 0!
A defunct class that represents the turtles in the KedamaWorld.  In this version, each turtle was represented as a full-fledged player.  Kept here to remember the alternative implementation.!

!KedamaTurtlePlayer methodsFor: 'accessing' stamp: 'yo 6/2/2004 22:36'!
asPoint

	^ x truncated @ y truncated
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/2/2004 22:39'!
color

	^ color.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/17/2004 07:26'!
color: aColor

	color _ aColor.


! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 9/14/2004 19:22'!
die
	"Delete this turtle at the end of the current cycle. The turtle will finish running all demons for the current cycle before it dies."

	world deleteTurtleID: self getScalarWho of: self class.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/17/2004 09:57'!
forward: dist

	self setX: (x + (dist asFloat * headingRadians cos)).
	self setY: (y - (dist asFloat * headingRadians sin)).
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/17/2004 09:59'!
getAngleTo: aPlayer

	| ret |
	ret _ ((aPlayer x - self x)@(aPlayer y - self y)) theta radiansToDegrees + 90.0.
	ret > 360.0 ifTrue: [^ ret - 360].
	^ ret.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/14/2004 16:09'!
getDistanceTo: aPlayer

	| dist |
	dist _ ((aPlayer x - self x) squared + (aPlayer y - self y) squared) sqrt.
	^ dist.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/6/2004 01:45'!
getHeading
	"Answer my heading in degrees."

	| degrees |
	degrees _ 90.0 - headingRadians radiansToDegrees.
	^ degrees >= 0.0 ifTrue: [degrees] ifFalse: [degrees + 360.0].
! !

!KedamaTurtlePlayer methodsFor: 'patches' stamp: 'yo 6/7/2004 17:56'!
getPatchValueIn: aPatch

	^ aPatch costume renderedMorph pixelAtX: x y: y.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 9/14/2004 19:56'!
getReplicated

	| newGuys theNewGuy |
	newGuys _ world makeTurtles: 1 turtlePlayerClass: self class color: self color ofPrototype: self.
	theNewGuy _ newGuys first.
	theNewGuy x: self x.
	theNewGuy y: self y.
	theNewGuy heading: self heading.
	(world prototypeOf: self class) privateTurtleCount: (world turtlesCountOf: self class).
	^ theNewGuy
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/7/2004 18:35'!
getUphillIn: aPatch

	^ aPatch costume renderedMorph uphillForTurtle: self.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/3/2004 02:19'!
getX

	^ x.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/3/2004 02:16'!
getY

	^ y.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/2/2004 22:36'!
goto: aPoint
	"Jump to the given location."

	x _ aPoint x.
	y _ aPoint y.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/2/2004 22:36'!
heading
	"Answer my heading in degrees."

	| degrees |
	degrees _ 90.0 - headingRadians radiansToDegrees.
	^ degrees >= 0.0 ifTrue: [degrees] ifFalse: [degrees + 360.0].
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/2/2004 22:36'!
heading: angleInDegrees
	"Set my heading in degrees. Like a compass, up or north is 0 degrees and right or east is 90 degrees."

	headingRadians _ ((90.0 - angleInDegrees) \\ 360.0) degreesToRadians.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 9/9/2004 14:45'!
hide

	visible _ false
! !

!KedamaTurtlePlayer methodsFor: 'initialization' stamp: 'yo 9/16/2004 17:41'!
initializeWorld: aStarSqueakWorld who: anInteger color: cPixel

	| dims |
	dims _ aStarSqueakWorld dimensions.
	world _ aStarSqueakWorld.
	who _ anInteger.
	x _ world random: dims x - 1.
	y _ world random: dims y - 1.
	headingRadians _ ((self random: 36000) / 100.0) degreesToRadians.
	color _ cPixel.
	self isGroup: false.
	self show.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 9/14/2004 15:43'!
isGroup: aBoolean
! !

!KedamaTurtlePlayer methodsFor: 'accessing' stamp: 'yo 6/2/2004 22:36'!
isInWorld

	^ world isInWorld.
! !

!KedamaTurtlePlayer methodsFor: 'accessing' stamp: 'yo 6/2/2004 22:36'!
owner

	^ world.
! !

!KedamaTurtlePlayer methodsFor: 'patches' stamp: 'yo 6/2/2004 22:36'!
patchBrightness: percent
	"Set the brightness of the patch below this turtle to the given value, where 0 is black and 100 is full brightness."

	world setPatchBrightnessAtX: x y: y to: percent.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/2/2004 22:36'!
random: range
	"Answer a random integer between 0 and range."

	^ world random: range
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/17/2004 07:26'!
setColor: aColor

	self color: (aColor pixelValueForDepth: 32).
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/4/2004 02:15'!
setHeading: newHeading

	^ self heading: newHeading.
! !

!KedamaTurtlePlayer methodsFor: 'patches' stamp: 'yo 6/7/2004 18:37'!
setPatchValueIn: aPatch to: value

	aPatch costume renderedMorph pixelAtX: x y: y put: value.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/9/2004 13:14'!
setTurtleCount: aNumber
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 9/9/2004 15:09'!
setX: val

	| leftEdgeMode rightEdgeMode |
	x _ val.
	x < 0.0 ifTrue: [
		leftEdgeMode _ world leftEdgeMode.
		leftEdgeMode == #wrap ifTrue: [
			x _ x + world wrapX.
		].
		leftEdgeMode == #stick ifTrue: [
			x _ 0.0.
		].
		leftEdgeMode == #bounce ifTrue: [
			x _ val negated.
			headingRadians < Float pi
				ifTrue: [headingRadians _ Float pi - headingRadians]
				ifFalse: [headingRadians _ Float threePi - headingRadians]
		].
	].

	x >= world wrapX ifTrue: [
		rightEdgeMode _ world rightEdgeMode.
		rightEdgeMode == #wrap ifTrue: [
			x _ x - world wrapX.
		].
		rightEdgeMode == #stick ifTrue: [
			x _ world wrapX - 0.0000001.
		].
		rightEdgeMode == #bounce ifTrue: [
			x _ world wrapX - 0.0000001 - (x - world wrapX).
			headingRadians < Float pi
				ifTrue: [headingRadians _ Float pi - headingRadians]
				ifFalse: [headingRadians _ Float threePi - headingRadians]
		].
	].
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 9/9/2004 15:09'!
setY: val

	| topEdgeMode bottomEdgeMode |
	y _ val.
	y < 0.0 ifTrue: [
		topEdgeMode _ world topEdgeMode.
		topEdgeMode == #wrap ifTrue: [
			y _ y + world wrapY.
		].
		topEdgeMode == #stick ifTrue: [
			y _ 0.0.
		].
		topEdgeMode == #bounce ifTrue: [
			y _ val negated.
			headingRadians _ Float twoPi - headingRadians.
		].
	].

	y >= world wrapY ifTrue: [
		bottomEdgeMode _ world bottomEdgeMode.
		bottomEdgeMode == #wrap ifTrue: [
			y _ y - world wrapY.
		].
		bottomEdgeMode == #stick ifTrue: [
			y _ world wrapY - 0.0000001.
		].
		bottomEdgeMode == #bounce ifTrue: [
			y _ world wrapY - 0.0000001 - (y - world wrapY).
			headingRadians _ Float twoPi - headingRadians.
		].
	].
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 9/9/2004 14:44'!
show

	visible _ true.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 7/27/2004 17:04'!
triggerScript: aSymbol
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/2/2004 23:43'!
turn: degrees

	self turnRight: degrees.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/2/2004 22:36'!
turnLeft: degrees
	"Turn left by the given number of degrees."

	self heading: (self heading - degrees).
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 6/2/2004 22:36'!
turnRight: degrees
	"Turn right by the given number of degrees."

	self heading: (self heading + degrees).
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 9/9/2004 14:56'!
visible

	^ visible.
! !

!KedamaTurtlePlayer methodsFor: 'commands and slots' stamp: 'yo 9/9/2004 14:44'!
visible: aBoolean

	visible _ aBoolean.
! !

!KedamaTurtlePlayer methodsFor: 'accessing' stamp: 'yo 6/2/2004 22:36'!
who

	^ who
! !

!KedamaTurtlePlayer methodsFor: 'accessing' stamp: 'yo 6/2/2004 22:36'!
who: anInteger

	who _ anInteger.
! !

!KedamaTurtlePlayer methodsFor: 'accessing' stamp: 'yo 6/2/2004 22:36'!
x

	^ x
! !

!KedamaTurtlePlayer methodsFor: 'accessing' stamp: 'yo 6/2/2004 22:36'!
x: aNumber

	x _ aNumber.
! !

!KedamaTurtlePlayer methodsFor: 'accessing' stamp: 'yo 6/2/2004 22:36'!
y

	^ y
! !

!KedamaTurtlePlayer methodsFor: 'accessing' stamp: 'yo 6/2/2004 22:36'!
y: aNumber

	y _ aNumber.
! !

!Player classSide methodsFor: 'other' stamp: 'sw 7/27/2001 13:45'!
abandonOldReferenceScheme
	"Abandon the old reference scheme"

	"(ActiveWorld presenter allExtantPlayers collect:
		[:aPlayer | aPlayer class]) asSet do:
			[:aPlayerClass | aPlayerClass abandonOldReferenceScheme]"

	self isUniClass ifTrue:
		[self userScriptsDo:
			[:aScript | aScript recompileScriptFromTilesUnlessTextuallyCoded].
		self class selectors do:
			[:sel | self class removeSelector: sel].
		self class instVarNames do:
			[:aName | self class removeInstVarName: aName].
		self organization removeEmptyCategories.
		self class organization removeEmptyCategories]! !

!Player classSide methodsFor: 'housekeeping' stamp: 'ls 10/10/1999 13:42'!
abandonUnnecessaryUniclasses
	"Player abandonUnnecessaryUniclasses"
	| oldCount oldFree newFree newCount report |
	oldCount _ self subclasses size - 1.
	oldFree _ Smalltalk garbageCollect.
	self allSubInstances do:
		[:aPlayer | aPlayer revertToUnscriptedPlayerIfAppropriate.  
		"encourage last one to get garbage-collected"
		aPlayer _ nil ].

	ScriptingSystem spaceReclaimed.
	newFree _ Smalltalk garbageCollect.
	newCount _ self subclasses size - 1.
	report _ 'Before: ', oldCount printString, ' uniclasses, ', oldFree
printString, ' bytes free
After:  ', newCount printString, ' uniclasses, ', newFree printString, '
bytes free'.
	Transcript cr; show: 'abandonUnnecessaryUniclasses:'; cr; show: report.
	^ report
	! !

!Player classSide methodsFor: 'user-scripted subclasses' stamp: 'sw 6/4/2004 13:59'!
addDocumentationForScriptsTo: aStream
	"Add documentation for every script in the receiver to the stream"

	self scripts do:
		[:aScript |
			aScript selector ifNotNil:
				[aStream cr; cr.
				aStream nextPutAll: self typicalInstanceName, '.'.
				self printMethodChunk: aScript selector withPreamble: false on: aStream moveSource: false toFile: nil.
				aStream position: (aStream position - 2)]].
	self scripts size == 0 ifTrue:
		[aStream cr; tab; nextPutAll: 'has no scripts']! !

!Player classSide methodsFor: 'user-scripted subclasses' stamp: 'sw 6/4/2004 15:04'!
addMethodReferencesTo: aCollection
	"For each extant script in the receiver, add a MethodReference object"

	| sel |
	self scripts do:
		[:aScript |
			(sel _ aScript selector) ifNotNil:
				[aCollection add: (MethodReference new setStandardClass: self methodSymbol: sel)]]! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 2/17/2001 03:44'!
assuredMethodInterfaceFor: aSelector
	"Answer the method interface object for aSelector, creating it if it does not already exist."

	| selSym  aMethodInterface |
	selSym _ aSelector asSymbol.
	aMethodInterface _ self scripts at: selSym ifAbsent: 
		[scripts at: selSym put: (self nascentUserScriptInstance playerClass: self selector: selSym)].
	
	^ aMethodInterface! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 2/17/2001 02:50'!
atSelector: aSelector putScript: aMethodWithInterface
	"Place the given method interface in my directory of scripts, at the given selector"

	self scripts at: aSelector asSymbol put: aMethodWithInterface! !

!Player classSide methodsFor: 'user-scripted subclasses' stamp: 'sw 10/30/2000 07:04'!
baseUniclass
	"Answer the uniclass that new instances should be instances of; this protocol allows for individual cards of a background to have their own class"

	| curr |
	curr _ self.
	[curr theNonMetaClass superclass name endsWithDigit]
		whileTrue:
			[curr _ curr superclass].
	^ curr

"CardPlayer100 baseUniclass 
CardPlayer100X baseUniclass
"! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 10/17/2001 09:08'!
bringScriptsUpToDate
	"Bring all the receiver's scripts up to date, after, for example, a name change"

	self scripts do:
		[:aUniclassScript |
			aUniclassScript bringUpToDate]! !

!Player classSide methodsFor: 'other' stamp: 'tk 6/26/1998 18:21'!
chooseUniqueClassName
	| ii className |
	ii _ BiggestSubclassNumber ifNil: [1] ifNotNil: [BiggestSubclassNumber+1].
	[className _ (self name , ii printString) asSymbol.
	 Smalltalk includesKey: className]
		whileTrue: [ii _ ii + 1].
	BiggestSubclassNumber _ ii.
	^ className	

! !

!Player classSide methodsFor: 'other' stamp: 'yo 6/2/2004 23:34'!
chooseUniqueTurtleClassName
	| ii className |
	className _ self name.
	[className last isDigit] whileTrue: [className _ className copyFrom: 1 to: className size - 1].

	ii _ BiggestSubclassNumber ifNil: [1] ifNotNil: [BiggestSubclassNumber+1].
	[className _ (className , ii printString) asSymbol.
	 Smalltalk includesKey: className]
		whileTrue: [ii _ ii + 1].
	BiggestSubclassNumber _ ii.
	^ className	

! !

!Player classSide methodsFor: 'housekeeping' stamp: 'sw 12/15/2004 20:43'!
cleanseScripts
	"Fix up various known structure errors in the uniclass relating to the scripts dctionary.  Answer the number of fixes made."

	| errs ed |
	scripts ifNil: [scripts _ IdentityDictionary new].
	errs _ 0.
	(scripts includesKey: nil) ifTrue: [errs _ errs + 1.  scripts removeKey: nil].
	scripts keysAndValuesDo: 
		[:sel :uniclassScript |
			uniclassScript
				ifNil:
					[errs _ errs + 1.
					Transcript cr; show: ' fix type 1, nil scripts key'.
					scripts removeKey: sel]
				ifNotNil:
					[(ed _ uniclassScript currentScriptEditor)
						ifNil:
							[errs _ errs + 1.
							Transcript cr; show: ' fix type 2, sel = ', sel.
							self someInstance removeScriptWithSelector: uniclassScript selector.]
						ifNotNil:
							[uniclassScript playerClassPerSe
								ifNil:
									[errs _ errs + 1.
									Transcript cr; show: ' fix type 3, sel = ', sel.
									uniclassScript playerClass: self selector:  sel]
								ifNotNil:
									[(ed scriptName ~= uniclassScript selector) ifTrue:
										[errs _ errs + 1.
										ed restoreScriptName: sel.
										Transcript cr; show: ' fix type 4, sel = ', sel.]]]]].
	^ errs! !

!Player classSide methodsFor: 'housekeeping' stamp: 'sw 4/10/2001 20:03'!
cleanseScriptsOfNilKeys
	"If, owing to an earlier bug, the receiver's scripts dictionary has a nil key, remove that offender before he causes more trouble"

	scripts ifNotNil:
		[scripts removeKey: nil ifAbsent: []]! !

!Player classSide methodsFor: 'housekeeping' stamp: 'sw 5/19/2000 16:26'!
cleanseSlotInfo
	| newInfo |
	slotInfo ifNotNil:
		[newInfo _ IdentityDictionary new.
		slotInfo associationsDo:
			[:assoc | newInfo at: assoc key asSymbol put: assoc value].
		slotInfo _ newInfo]
! !

!Player classSide methodsFor: 'slots' stamp: 'sw 12/18/1998 21:08'!
compileAccessorsFor: varName
	^ self compileInstVarAccessorsFor: varName! !

!Player classSide methodsFor: 'slots' stamp: 'NS 1/28/2004 14:41'!
compileInstVarAccessorsFor: varName
	"Compile getters and setteres for the given instance variable name"

	| nameString |
	nameString _ varName asString capitalized.
	self compileSilently: ('get', nameString, '
	^ ', varName)
		classified: 'access'.
	self compileSilently: ('set', nameString, ': val
	', varName, ' _ val')
		classified: 'access'! !

!Player classSide methodsFor: 'namespace' stamp: 'NS 1/28/2004 14:41'!
compileReferenceAccessorFor: varName
	"Compile reference accessors for the given variable.  If the #capitalizedReferences preference is true, then nothing is done here"

	Preferences capitalizedReferences ifTrue: [^ self].

	self class compileSilently: ((self referenceAccessorSelectorFor: varName), '
	^ ', varName)
		classified: 'reference'! !

!Player classSide methodsFor: 'turtles' stamp: 'yo 9/17/2004 08:06'!
createSequencialStubSubclass

	| instVarString classInstVarString aName aClass |
	instVarString _ KedamaSequenceExecutionStub instVarNames.
	classInstVarString _ ''.
	aName _ self chooseUniqueTurtleClassName.
	aClass _ self subclass: aName instanceVariableNames: instVarString 
		classVariableNames: '' poolDictionaries: '' category: self categoryForUniclasses.
	classInstVarString size > 0 ifTrue:
		[aClass class instanceVariableNames: classInstVarString].
	aClass copyAllCategoriesUnobtrusivelyFrom: KedamaSequenceExecutionStub.
	^ aClass
! !

!Player classSide methodsFor: 'turtles' stamp: 'yo 9/14/2004 21:36'!
createTurtleSubclass

	| instVarString classInstVarString aName aClass |
	instVarString _ KedamaTurtleVectorPlayer instVarNames.
	classInstVarString _ ''.
	aName _ self chooseUniqueTurtleClassName.
	aClass _ self subclass: aName instanceVariableNames: instVarString 
		classVariableNames: '' poolDictionaries: '' category: self categoryForUniclasses.
	classInstVarString size > 0 ifTrue:
		[aClass class instanceVariableNames: classInstVarString].
	aClass copyAllCategoriesUnobtrusivelyFrom: KedamaTurtleVectorPlayer.
	^ aClass
! !

!Player classSide methodsFor: 'housekeeping' stamp: 'ar 4/25/2005 13:36'!
freeUnreferencedSubclasses
	"Player classes may hold in their class instance variables references
to instances of themselves that are housekeepingwise unreachable. This
method allows such loops to be garbage collected. This is done in three
steps:
	1. Remove user-created subclasses from the 'subclasses' set and from
Smalltalk. Only remove classes whose name begins with 'Player' and which
have no references.
	2. Do a full garbage collection.
	3. Enumerate all Metaclasses and find those whose soleInstance's
superclass is this class. Reset the subclasses set to this set of
classes, and add back to Smalltalk."
	"Player freeUnreferencedSubclasses"

	| oldFree candidatesForRemoval class |
	oldFree _ Smalltalk garbageCollect.
	candidatesForRemoval _ self subclasses asOrderedCollection select:
		[:aClass | (aClass name beginsWith: 'Player') and: [aClass name
endsWithDigit]].

	"Break all system links and then perform garbage collection."
	candidatesForRemoval do:
		[:c | self removeSubclass: c.  "Break downward subclass pointers."
		Smalltalk removeKey: c name ifAbsent: [].  "Break binding of global
name"].
	candidatesForRemoval _ nil.
	Smalltalk garbageCollect.  "Now this should reclaim all unused
subclasses"

	"Now reconstruct system links to subclasses with valid references."
	"First restore any global references via associations"
	(Association allSubInstances select:
			[:assn | (assn key isSymbol)
					and: [(assn key beginsWith: 'Player')
					and: [assn key endsWithDigit]]])
		do: [:assn | class _ assn value.
			(class isKindOf: self class) ifTrue:
				[self addSubclass: class.
				Smalltalk add: assn]].
	"Then restore any further direct references, creating new
associations."
	(Metaclass allInstances select:
			[:m | (m soleInstance name beginsWith: 'Player')
					and: [m soleInstance name endsWithDigit]])
		do: [:m | class _ m soleInstance.
			((class isKindOf: self class) and: [(Smalltalk includesKey: class
name) not]) ifTrue:
				[self addSubclass: class.
				Smalltalk at: class name put: class]].
	SystemOrganization removeMissingClasses.
	^ Smalltalk garbageCollect - oldFree
! !

!Player classSide methodsFor: 'user-scripted subclasses' stamp: 'sw 5/6/1998 12:16'!
initialInstance
	self == Player ifTrue: [self error: 'must not instantiate naked Player'].
	^ super initialInstance! !

!Player classSide methodsFor: 'other' stamp: 'sw 8/17/1998 07:33'!
isSystemDefined
	^ name endsWithDigit not! !

!Player classSide methodsFor: 'housekeeping' stamp: 'sw 12/18/2000 15:45'!
isUniClass
	"UnscriptedPlayer reimplements to false"

	^ self ~~ Player! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 4/9/98 21:55'!
jettisonScripts
	scripts _ IdentityDictionary new! !

!Player classSide methodsFor: 'namespace' stamp: 'sw 4/27/1998 23:42'!
makeReferenceFor: anObject

	| stem otherNames i partName |
	stem _ anObject class name.
	(stem size > 5 and: [stem endsWith: 'Morph'])
		ifTrue: [stem _ stem copyFrom: 1 to: stem size - 5].
	stem _ stem first asLowercase asString, stem allButFirst.
	otherNames _ self class allInstVarNames.
	i _ 1.
	[otherNames includes: (partName _ stem, i printString)]
		whileTrue: [i _ i + 1].
	self class addInstVarName: partName.
	self instVarAt: self class instSize put: anObject.  "assumes added as last field"

	self compileReferenceAccessorFor: partName.
	^ self referenceAccessorSelectorFor: partName! !

!Player classSide methodsFor: 'other' stamp: 'sw 1/6/2001 06:27'!
nameForViewer
	"Answer the name by which the receiver is to be referred in a viewer"

	^ self isUniClass
		ifTrue:
			[self someInstance getName]
		ifFalse:
			[super nameForViewer]! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 1/25/2001 16:24'!
namedTileScriptSelectors
	"Answer a list of all the selectors of named tile scripts"

	scripts ifNil: [^ OrderedCollection new].
	^ scripts select: [:aScript | aScript selector ~~ nil] 
		thenCollect: [:aScript | aScript selector]! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 12/19/2003 23:28'!
namedUnaryTileScriptSelectors
	"Answer a list of all the selectors of named unary tile scripts"

	| sel |
	scripts ifNil: [^ OrderedCollection new].
	^ scripts select: [:aScript | ((sel _ aScript selector) ~~ nil) and: [sel numArgs == 0]] 
		thenCollect: [:aScript | aScript selector]! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 2/18/2001 18:42'!
nascentUserScriptInstance
	"Answer a new script object of the appropriate class"

	| classToUse |
	classToUse _ Preferences universalTiles
		ifTrue:	[MethodWithInterface]
		ifFalse:	[UniclassScript].
	^ classToUse new! !

!Player classSide methodsFor: 'other' stamp: 'tk 3/15/98 20:23'!
officialClass
	"We want to make a new instance of the receiver, which is a subclass of Player.  Answer who to make a new subclass of.  Also used to tell if a given class is a UniClass, existing only for its single instance."
	^ Player! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 2/17/2001 00:59'!
permanentUserScriptFor: aSelector player: aPlayer
	"Create and answer a suitable script  object for the given player (who will be an instance of the receiver) and selector.  Save that script-interface object in my (i.e. the class's) directory of scripts"

	|  entry |
	scripts ifNil: [scripts _ IdentityDictionary new].
	entry _ self nascentUserScriptInstance playerClass: aPlayer class selector: aSelector.
	scripts at: aSelector put: entry.
	^ entry! !

!Player classSide methodsFor: 'housekeeping' stamp: 'tk 8/12/1999 15:45'!
playersWithUnnecessarySubclasses
	"Return a list of all players whose scripts dictionaries contain entries with nil selectors"
	"Player playersWithUnnecessarySubclasses size"
	^ self class allSubInstances select:
		[:p | p class isSystemDefined not and: [p scripts size == 0 and: [p instVarNames size == 0]]] ! !

!Player classSide methodsFor: 'scripts' stamp: 'tk 8/31/1998 15:42'!
privateScripts
	"for copying"
	^ scripts! !

!Player classSide methodsFor: 'slots' stamp: 'tk 8/31/1998 15:45'!
privateSlotInfo
	"for copying"
	^ slotInfo! !

!Player classSide methodsFor: 'namespace' stamp: 'sw 4/27/1998 23:38'!
referenceAccessorSelectorFor: varName
	^ 'ref', (varName asString capitalized)! !

!Player classSide methodsFor: 'namespace' stamp: 'sw 4/28/1998 00:03'!
referenceSelectorFor: anObject
	self class instVarNames do:  "Just those added in the unique subclass"
		[:aName | (self instVarNamed: aName) == anObject
			ifTrue:
				[^ self referenceAccessorSelectorFor: aName]].
	^ self makeReferenceFor: anObject! !

!Player classSide methodsFor: 'slots' stamp: 'sw 12/17/1998 20:30'!
removeAccessorsFor: varName
	^ self removeInstVarAccessorsFor: varName! !

!Player classSide methodsFor: 'slots' stamp: 'sw 12/17/1998 20:31'!
removeInstVarAccessorsFor: varName
	| nameString |

	nameString _ varName asString capitalized.
	self removeSelector: ('get', nameString) asSymbol.
	self removeSelector: ('set', nameString, ':') asSymbol! !

!Player classSide methodsFor: 'slots' stamp: 'di 1/19/2000 17:18'!
removeInstVarName: aName
	self removeInstVarAccessorsFor: aName.
	super removeInstVarName: aName.
	self slotInfo removeKey: aName asSymbol ifAbsent: []! !

!Player classSide methodsFor: 'scripts' stamp: 'NS 1/30/2004 13:11'!
removeScriptNamed: aScriptName
	aScriptName ifNotNil:
		[scripts removeKey: aScriptName.
		self removeSelectorSilently: aScriptName]! !

!Player classSide methodsFor: 'housekeeping' stamp: 'yo 6/3/2004 13:36'!
removeUninstantiatedSubclassesSilently
	"Remove the classes of any subclasses that have neither instances nor subclasses.  Answer the number of bytes reclaimed"
	"Player removeUninstantiatedSubclassesSilently"

	| candidatesForRemoval  oldFree |

	oldFree _ Smalltalk garbageCollect.
	candidatesForRemoval _ OrderedCollection new.
	self allSubclassesWithLevelDo: [:e :l | candidatesForRemoval add: e] startingLevel: 2.
	candidatesForRemoval _ candidatesForRemoval reverse.
	candidatesForRemoval _
		candidatesForRemoval select: [:c |
			(c instanceCount = 0) and: [c subclasses size = 0]].
	candidatesForRemoval _ candidatesForRemoval select:
		[:aClass | aClass isSystemDefined not].
	candidatesForRemoval do: [:c | c removeFromSystemUnlogged].
	^ Smalltalk garbageCollect - oldFree! !

!Player classSide methodsFor: 'slots' stamp: 'sw 5/19/2000 16:37'!
renameSilentlyInstVar: oldName to: newName
	(instanceVariables includes: oldName asString) ifFalse:
		[self error: oldName , ' is not defined in ', self name].
	self removeAccessorsFor: oldName.

	super renameSilentlyInstVar: oldName to: newName.
	self compileAccessorsFor: newName.
	slotInfo at: newName asSymbol put: (slotInfo at: oldName).
	slotInfo removeKey: oldName! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 4/20/2001 20:11'!
scripts
	"Answer the receiver's scripts -- an IdentityDictionary"

	scripts
		ifNil:
			[scripts _ IdentityDictionary new]
		ifNotNil:
			[self cleanseScriptsOfNilKeys].
	^ scripts! !

!Player classSide methodsFor: 'scripts' stamp: 'tk 8/31/1998 15:41'!
scripts: aDict
	"for copying"
	scripts _ aDict! !

!Player classSide methodsFor: 'slots' stamp: 'sw 5/25/2001 10:26'!
slotGettersOfType: aType
	"Answer a list of gettter selectors for slots of mine of the given type"

	| aList |
	aList _ OrderedCollection new.
	self slotInfo associationsDo:
		[:assoc |
			(assoc value type = aType) ifTrue:
				[aList add: (Utilities getterSelectorFor: assoc key)]].
	^ aList! !

!Player classSide methodsFor: 'slots' stamp: 'sw 4/8/98 11:58'!
slotInfo
	slotInfo ifNil: [slotInfo _ IdentityDictionary new].
	^ slotInfo! !

!Player classSide methodsFor: 'slots' stamp: 'tk 8/31/1998 15:44'!
slotInfo: aDict
	"for copying"
	slotInfo _ aDict! !

!Player classSide methodsFor: 'user-scripted subclasses' stamp: 'sw 8/2/1998 22:52'!
tileScriptNames
	scripts ifNil: [^ OrderedCollection new].
	"The following is an emergency workaround for damaged script dictionaries occurring in Alan's demo image 8/2/98; no selector should be nil but somehow some is, so here we filter such damaging things out"
	^ scripts collect: [:aScript | aScript selector] thenSelect: [:n | n ~~ nil]! !

!Player classSide methodsFor: 'scripts' stamp: 'tk 1/22/2001 14:54'!
tilesFrom: aString
	| code tree syn block phrase |
	"Construct SyntaxMorph tiles for the String."

	"This is really cheating!!  Make a true parse tree later. -tk"
	code _ String streamContents: [:strm | 
		strm nextPutAll: 'doIt'; cr; tab.
		strm nextPutAll: aString].
	"decompile to tiles"
	tree _ Compiler new 
		parse: code 
		in: self
		notifying: nil.
	syn _ tree asMorphicSyntaxUsing: SyntaxMorph.
	block _ syn submorphs detect: [:mm | 
		(mm respondsTo: #parseNode) ifTrue: [
			mm parseNode class == BlockNode] ifFalse: [false]].
	phrase _ block submorphs detect: [:mm | 
		(mm respondsTo: #parseNode) ifTrue: [
			mm parseNode class == MessageNode] ifFalse: [false]].
	^ phrase

! !

!Player classSide methodsFor: 'other' stamp: 'sw 6/4/2004 13:56'!
typicalInstanceName
	"For the purpose of documentation, answer the name of a named instance of the receiver, if possible, else answer the class name"

	| known |
	known _ (self allInstances collect: [:i | i knownName]) detect: [:n | n isEmptyOrNil not] ifNone: [nil].
	^ known ifNil: [self name]! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 3/28/2001 16:18'!
userScriptForPlayer: aPlayer selector: aSelector
	"Answer the user script for the player (one copy for all instances of the uniclass) and selector"

	|  newEntry existingEntry |
	scripts ifNil: [scripts _ IdentityDictionary new].
	existingEntry _ scripts at: aSelector ifAbsent: [nil].

	"Sorry for all the distasteful isKindOf: and isMemberOf: stuff here, folks; it arises out of concern for preexisting content saved on disk from earlier stages of this architecture.  Someday much of it could be cut loose"
	Preferences universalTiles
		ifTrue:
			[(existingEntry isMemberOf: MethodWithInterface) ifTrue: [^ existingEntry].
			newEntry _ (existingEntry isKindOf: UniclassScript)
				ifTrue:
					[existingEntry as: MethodWithInterface] "let go of extra stuff if it was UniclassScript"
				ifFalse:
					[MethodWithInterface new playerClass: aPlayer class selector: aSelector].
			scripts at: aSelector put: newEntry.
			^ newEntry]
		ifFalse:
			[(existingEntry isKindOf: UniclassScript)
				ifTrue:
					[^ existingEntry]
				ifFalse:
					[newEntry _ UniclassScript new playerClass: self selector: aSelector.
					scripts at: aSelector put: newEntry.
					existingEntry ifNotNil: "means it is a grandfathered UserScript that needs conversion"
						[newEntry convertFromUserScript: existingEntry].
					^ newEntry]]! !

!Player classSide methodsFor: 'scripts' stamp: 'sw 4/14/1998 01:16'!
userScriptsDo: aBlock
	self scripts do:
		[:aUserScript | aBlock value: aUserScript]! !

!Player classSide methodsFor: 'other' stamp: 'tk 9/28/2001 11:43'!
wantsChangeSetLogging
	"Log changes for Player itself, but not for automatically-created subclasses like Player1, Player2, but *do* log it for uniclasses that have been manually renamed."

	^ (self == Player or:
		[(self name beginsWith: 'Player') not]) or:
			[Preferences universalTiles]! !

!Player classSide methodsFor: 'other' stamp: 'sw 7/31/2000 12:57'!
wantsRecompilationProgressReported
	"Report progress for Player itself, but not for automatically-created subclasses like Player1, Player2"

	^ self == Player or:
		[(self class name beginsWith: 'Player') not]! !

!Player methodsFor: 'scripts-vector' stamp: 'nk 9/25/2003 11:46'!
* aNumber
	"Treating Players like vectors, return a new Player that is myself scaled by the number"

	| new |
	new _ costume usableSiblingInstance player.
	new setX: self getX * aNumber asPoint x.
	new setY: self getY * aNumber asPoint y.
	^ new
! !

!Player methodsFor: 'scripts-vector' stamp: 'nk 9/25/2003 11:46'!
+ aPlayer
	"Treating Players like vectors, add aPlayer to me and return a new Player"

	| new |
	new _ costume usableSiblingInstance player.
	new setX: self getX + aPlayer asPoint x.
	new setY: self getY + aPlayer asPoint y.
	^ new! !

!Player methodsFor: 'scripts-vector' stamp: 'nk 9/25/2003 11:46'!
- aPlayer
	"Treating Players like vectors, subtract aPlayer from me and return a new Player"

	| new |
	new _ costume usableSiblingInstance player.
	new setX: self getX - aPlayer asPoint x.
	new setY: self getY - aPlayer asPoint y.
	^ new! !

!Player methodsFor: 'scripts-vector' stamp: 'nk 9/25/2003 11:45'!
/ aNumber
	"Treating Players like vectors, return a new Player that is myself divided by the number"

	| new |
	new _ costume usableSiblingInstance player.
	new setX: self getX / aNumber asPoint x.
	new setY: self getY / aNumber asPoint y.
	^ new
! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 12/6/2001 21:57'!
absorbBackgroundDataFrom: aLine forInstanceVariables: slotNames
	"Fill my background fields from the substrings in a tab-delimited line of data.  At the moment this only really cateres to string-valued items"

	slotNames doWithIndex:
		[:aSlotName :anIndex |
			aLine do:
				[:aValue |
					self instVarNamed: aSlotName put: aValue] toFieldNumber: anIndex]! !

!Player methodsFor: 'scripts-kernel' stamp: 'NS 1/28/2004 14:41'!
acceptScript: aScriptEditorMorph for: aSelector
	"Accept the tile code in the script editor as the code for the given selector.  This branch is only for the classic-tile system, 1997-2001"

	| aUniclassScript |
	self class compileSilently: aScriptEditorMorph methodString
		classified: 'scripts'.
	aUniclassScript _ self class assuredMethodInterfaceFor: aSelector asSymbol.
	aUniclassScript currentScriptEditor: aScriptEditorMorph! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 1/4/2005 02:18'!
acceptableScriptNameFrom: originalString forScriptCurrentlyNamed: currentName
	"Produce an acceptable script name, derived from the current name, for the receiver.  This method will always return a valid script name that will be suitable for use in the given situation, though you might not like its beauty sometimes."

	| aString stemAndSuffix proscribed stem suffix withoutColon currentNumArgs withColon |
	withoutColon _ originalString copyWithoutAll: {$:. $ }.
	(currentName notNil and: [(currentName copyWithout: $:) = withoutColon])
		ifTrue:
			[^ currentName].  "viz. no change; otherwise, the #respondsTo: check gets in the way"

	currentNumArgs _ currentName ifNil: [0] ifNotNil: [currentName numArgs].
	aString _ withoutColon asIdentifier: false.  "get an identifier starting with a lowercase letter"
	stemAndSuffix _ aString stemAndNumericSuffix.
	proscribed _ #(self super thisContext costume costumes dependents #true #false size).

	stem _ stemAndSuffix first.
	suffix _ stemAndSuffix last.
	withoutColon _ aString asSymbol.
	withColon _ (withoutColon, ':') asSymbol.

	[(proscribed includes: withoutColon)
		or: [self respondsTo: withoutColon]
		or: [self respondsTo: withColon]
		or:	[Smalltalk includesKey: withoutColon]
		or: [Smalltalk includesKey: withColon]]
	whileTrue:
		[suffix _ suffix + 1.
		withoutColon _ (stem, suffix printString) asSymbol.
		withColon _ (withoutColon, ':') asSymbol].

	^ currentNumArgs = 0
		ifTrue:
			[withoutColon]
		ifFalse:
			[withColon]! !

!Player methodsFor: 'misc' stamp: 'sw 10/7/2004 17:20'!
actorState
	"Answer the receiver's actorState.  If the receiver has no costume, answer nil, else answer the costume's current actor state, creating it at this time if necessary."

	^ self costume ifNotNil: [costume actorState]
! !

!Player methodsFor: 'scripts-kernel' stamp: 'yo 2/12/2005 19:53'!
addIdiosyncraticMenuItemsTo: aMenu forSlotSymol: slotSym
	"The menu provided has the receiver as its argument, and is used as the menu for the given slot-symbol in a line of a Viewer.  Add special-case items"

	(#(copy getNewClone newClone) includes: slotSym) ifTrue:
		[aMenu add: 'give me a copy now' translated action: #handTheUserACopy].

"	(slotSym == #dropShadow) ifTrue:
		[aMenu add: 'set shadow offset' translated action: #setShadowOffset].

	(slotSym == #useGradientFill) ifTrue:
		[aMenu add: 'set gradient origin...' translated action: #setGradientOffset.
		aMenu add: 'set gradient direction...' translated action: #setGradientDirection]."
! !

!Player methodsFor: 'slots-user' stamp: 'yo 9/24/2004 09:09'!
addInstanceVariable
	"Offer the user the opportunity to add an instance variable, and if he goes through with it, actually add it."

	| itsName initialValue typeChosen usedNames initialAnswer setterSelector originalString |
	usedNames _ self class instVarNames.

	initialAnswer _ Utilities keyLike: ('var' translated, (usedNames size + 1) asString)  satisfying: [:aKey | (usedNames includes: aKey) not].

	originalString _ FillInTheBlank request: 'name for new variable: ' translated initialAnswer: initialAnswer.
	originalString isEmptyOrNil ifTrue: [^ self].
	itsName _ ScriptingSystem acceptableSlotNameFrom: originalString forSlotCurrentlyNamed: nil asSlotNameIn: self world: self costume world.

 	itsName size == 0 ifTrue: [^ self].	
	self assureUniClass.
	typeChosen _ self initialTypeForSlotNamed: itsName.
	self slotInfo at: itsName put: (SlotInformation new initialize type: typeChosen).
	initialValue _ self initialValueForSlotOfType: typeChosen.
	self addInstanceVarNamed: itsName withValue: initialValue.
	self compileInstVarAccessorsFor: itsName.
	setterSelector _ Utilities setterSelectorFor: itsName.
	((self class allSubInstances copyWithout: self) reject: [:e | e isSequentialStub]) do:
		[:anInstance | anInstance perform: setterSelector with: initialValue].
	self updateAllViewersAndForceToShow: ScriptingSystem nameForInstanceVariablesCategory! !

!Player methodsFor: 'slots-user' stamp: 'yo 6/5/2004 18:49'!
addInstanceVariable2Named: nameSymbol type: typeChosen value: aValue
	"Add an instance variable of the given name and type, and initialize it to have the given value"

	| initialValue setterSelector |
	self assureUniClass.
	self slotInfo at: nameSymbol put: (SlotInformation new initialize type: typeChosen).
	initialValue _ aValue.
	self addInstanceVarNamed: nameSymbol withValue: aValue.
	self class compileAccessorsFor: nameSymbol.
	setterSelector _ Utilities setterSelectorFor: nameSymbol.
	(self class allSubInstances copyWithout: self) do:
		[:anInstance | anInstance perform: setterSelector with: initialValue].
	self updateAllViewersAndForceToShow: #'instance variables'
! !

!Player methodsFor: 'slots-user' stamp: 'md 3/1/2006 09:15'!
addInstanceVariableNamed: nameSymbol type: typeChosen value: aValue
	"Add an instance variable of the given name and type, and initialize it to have the given value"

	| initialValue setterSelector |
	self assureUniClass.
	self slotInfo at: nameSymbol put: (SlotInformation new type: typeChosen).
	initialValue _ self initialValueForSlotOfType: typeChosen.
	self addInstanceVarNamed: nameSymbol withValue: aValue.
	self class compileAccessorsFor: nameSymbol.
	setterSelector _ Utilities setterSelectorFor: nameSymbol.
	(self class allSubInstances copyWithout: self) do:
		[:anInstance | anInstance perform: setterSelector with: initialValue].
	self updateAllViewersAndForceToShow: ScriptingSystem nameForInstanceVariablesCategory
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 2/2/2005 17:47'!
addPatchVarNamed: nameSymbol

	| f |
	f _ KedamaPatchMorph newExtent: self costume dimensions.
	f assuredPlayer assureUniClass.
	f setNameTo: (ActiveWorld unusedMorphNameLike: f innocuousName).
	self addInstanceVariable2Named: nameSymbol type: #Patch value: f player.
	^ f.
! !

!Player methodsFor: 'pen' stamp: 'nk 6/12/2004 10:00'!
addPlayerMenuItemsTo: aMenu hand: aHandMorph
	"Note that these items are primarily available in another way in an object's Viewer"

	| subMenu |
	subMenu _ MenuMorph new defaultTarget: self.
	self getPenDown
		ifTrue: [subMenu add: 'lift pen' action: #liftPen]
		ifFalse: [subMenu add: 'lower pen' action: #lowerPen].
	subMenu add: 'choose pen size...' action: #choosePenSize.
	subMenu add: 'choose pen color...' action: #choosePenColor:.
	aMenu add: 'pen...' subMenu: subMenu.

	(costume renderedMorph isSketchMorph) ifTrue:
		[self belongsToUniClass
			ifFalse: 
				[aMenu add: 'adopt scripts from...' target: self action: #adoptScriptsFrom]
			ifTrue:
				[aMenu add: 'impart scripts to...' target: self action: #impartSketchScripts]]! !

!Player methodsFor: 'slots-user' stamp: 'NS 1/28/2004 14:47'!
addSpecialSetter: selector
	| instVar code |
	"For the special setters, fooIncreaseBy:, fooDecreaseBy:, fooMultiplyBy:, add a method that does them."

 	self assureUniClass.
	instVar _ (selector allButLast: 11) asLowercase.  "all three are 11 long!!"
	(self respondsTo: ('set', instVar capitalized, ':') asSymbol) ifFalse: [^ false].
	code _ String streamContents: [:strm |
		strm nextPutAll: selector, ' amount'; crtab.
		strm nextPutAll: 'self set', instVar capitalized, ': (self get', instVar capitalized; space.
		(selector endsWith: 'IncreaseBy:') ifTrue: [strm nextPut: $+].
		(selector endsWith: 'DecreaseBy:') ifTrue: [strm nextPut: $-].
		(selector endsWith: 'MultiplyBy:') ifTrue: [strm nextPut: $*].
		strm nextPutAll: ' amount)'].

	self class compileSilently: code classified: 'access' notifying: nil.
	^ true
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/9/2004 15:14'!
addToPatchDisplayList: aPlayer

	costume renderedMorph addToPatchDisplayList: aPlayer costume renderedMorph.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/9/2004 15:28'!
addToTurtleDisplayList: aPlayer

	costume renderedMorph addToTurtleDisplayList: aPlayer "costume renderedMorph".
! !

!Player methodsFor: 'misc' stamp: 'nk 8/18/2004 16:43'!
adoptScriptsFrom
	"Let the user click on another object form which the receiver should obtain scripts and code"

	| aMorph |
	Sensor waitNoButton.
	aMorph _ ActiveWorld chooseClickTarget.
	aMorph ifNil: [^ Beeper beep].

	(((aMorph isSketchMorph) and: [aMorph player belongsToUniClass]) and: [self belongsToUniClass not])
		ifTrue:
			[costume acquirePlayerSimilarTo: aMorph player]
		ifFalse:
			[Beeper beep]! !

!Player methodsFor: 'misc' stamp: 'gm 2/22/2003 14:53'!
allOpenViewers
	"Answer a list of all the viewers open on the receiver.  Include viewers in closed flaps"

	| aWorld all |
	(aWorld := self costume world) ifNil: [^#()].
	all := aWorld allMorphs.
	aWorld closedViewerFlapTabs 
		do: [:aTab | all addAll: aTab referent allMorphs].
	^all 
		select: [:m | (m isStandardViewer) and: [m scriptedPlayer == self]]! !

!Player methodsFor: 'misc' stamp: 'gm 2/22/2003 14:54'!
allOpenViewersOnReceiverAndSiblings
	"Answer a list of all the viewers open on the receiver and any of its sibling instances.  Include viewers in closed flaps"

	| aWorld all |
	(aWorld := self costume world) ifNil: [^#()].
	all := aWorld allMorphs.
	aWorld closedViewerFlapTabs 
		do: [:aTab | all addAll: aTab referent allMorphs].
	^all select: 
			[:m | 
			(m isStandardViewer) and: [m scriptedPlayer class == self class]]! !

!Player methodsFor: 'slots-user' stamp: 'sw 1/6/2005 01:32'!
allPossibleWatchersFromWorld
	"Answer a list of all UpdatingStringMorphs, PlayerReferenceReadouts, ThumbnailMorphs, and  UpdatingReferenceMorphs in the Active world and its hidden book pages, etc., which have me or any of my siblings as targets"

	| a |
	a _ IdentitySet new: 400.
	ActiveWorld allMorphsAndBookPagesInto: a.
	^ a select: [:e | e isEtoyReadout and: [e target class == self class]]! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 4/9/98 21:56'!
allScriptEditors
	"Used presently only an one-shot efforts to update all tile scripts to new styles"

	^ self class tileScriptNames collect: [:n | self scriptEditorFor: n]! !

!Player methodsFor: 'scripts-standard' stamp: 'ar 4/10/2005 18:50'!
append: aPlayer 
	"Add aPlayer to the list of objects logically 'within' me.  This is visually represented by its morph becoming my costume's last submorph.   Also allow text to be appended."

	| aCostume |
	(aPlayer isNil or: [aPlayer == self]) ifTrue: [^self].
	(aPlayer isText or: [aPlayer isString]) 
		ifTrue: 
			[self costume class == TextFieldMorph 
				ifTrue: [^self costume append: aPlayer]
				ifFalse: [^self]].
	(aCostume := self costume topRendererOrSelf) 
		addMorphNearBack: aPlayer costume.
	aPlayer costume goHome.	"assure it's in view"
	(aCostume isKindOf: PasteUpMorph) 
		ifTrue: [self setCursor: (aCostume submorphs indexOf: aPlayer costume)]! !

!Player methodsFor: 'misc' stamp: 'sw 7/4/2004 00:20'!
arrowDeltaFor: aGetSelector
	"Answer the arrowDelta to use in conjunction with a readout for aGetSelector, which will be of the form 'getXXX'"

	costume ifNotNil:
		[^ costume renderedMorph arrowDeltaFor: aGetSelector].
	^ 1
	
	"For the future, possibly:  If we want the SlotInformation for a user-defined slot to be able to specify a standard arrowDelta for that slot, we'd include something like the following... 
	| aSlotName slotInfo |
	aSlotName _ Utilities inherentSelectorForGetter: aGetSelector.
	(slotInfo _ self slotInfoAt: aSlotName ifAbsent: [nil]) ifNotNil:
		[^ slotInfo arrowDelta]."
! !

!Player methodsFor: 'pen' stamp: 'tk 10/4/2001 18:16'!
arrowheadsOnAllPens
	"Only for the Player of a World"

	self costume arrowheadsOnAllPens! !

!Player methodsFor: 'scripts-vector' stamp: 'tk 8/18/2001 22:41'!
asPoint

	^ self getX @ self getY! !

!Player methodsFor: 'slots-assignment' stamp: 'sw 1/31/98 00:06'!
assignDecrGetter: getterSelector setter: setterSelector amt: aDecrement
	self perform: setterSelector with:
		((self perform: getterSelector) - aDecrement)! !

!Player methodsFor: 'slots-assignment' stamp: 'sw 1/31/98 00:07'!
assignGetter: getterSelector setter: setterSelector amt: amt
	self perform: setterSelector with: amt! !

!Player methodsFor: 'slots-assignment' stamp: 'sw 1/31/98 00:07'!
assignIncrGetter: getterSelector setter: setterSelector amt: anIncrement
	self perform: setterSelector with:
		((self perform: getterSelector) + anIncrement)! !

!Player methodsFor: 'slots-assignment' stamp: 'sw 1/31/98 00:06'!
assignMultGetter: getterSelector setter: setterSelector amt: aMultiplier
	self perform: setterSelector with:
		((self perform: getterSelector) * aMultiplier)! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/4/2001 07:42'!
assignStatus: newStatus toAllFor: scriptName
	"Change the status of my script of the given name to be as specified in me and all of my siblings."

	| aWorld |
	(self existingScriptInstantiationForSelector: scriptName) ifNotNilDo:
		[:scriptInstantiation |
				scriptInstantiation status: newStatus.
				scriptInstantiation assignStatusToAllSiblings.
				^ (aWorld _ self costume world) ifNotNil:
					[aWorld updateStatusForAllScriptEditors]]! !

!Player methodsFor: 'scripts-execution' stamp: 'sw 5/13/1998 13:10'!
assureEventHandlerRepresentsStatus
	self instantiatedUserScriptsDo:
			[:aScriptInst | aScriptInst assureEventHandlerRepresentsStatus]! !

!Player methodsFor: 'scripts-execution' stamp: 'sw 6/13/2002 10:57'!
assureNoScriptOtherThan: aScriptInstantiation hasStatus: aStatus
	self instantiatedUserScriptsDo:
		[:aScriptInst | aScriptInst == aScriptInstantiation  ifFalse: [aScriptInst resetToNormalIfCurrently:  aStatus]]! !

!Player methodsFor: 'viewer' stamp: 'sw 9/15/1998 13:19'!
assureUniClass
	"If I am not currently a member of a UniClass, become one now"! !

!Player methodsFor: 'costume' stamp: 'sw 5/13/1998 14:07'!
availableCostumeNames
	"Answer a list of class names (minus trailing ...Morph) to be offered in the thumbnail 'new-Costume' menu in the Viewer"

 	^ #(Book BouncingAtoms Curve Ellipse  Flasher Image Joystick  PasteUp Polygon Rectangle Ruler Sketch Star String Text  )

"  ScrollBar  SimpleButton SimpleSlider SimpleSwitch  UserFrame  "! !

!Player methodsFor: 'costume' stamp: 'sw 4/5/1999 13:48'!
availableCostumesForArrows
	^ costumes ifNil: [Array new]! !

!Player methodsFor: 'slots-user' stamp: 'yo 9/20/2004 10:36'!
basicCompileInstVarAccessorsFor: varName
	self class compileInstVarAccessorsFor: varName! !

!Player methodsFor: 'slots-user' stamp: 'yo 9/20/2004 12:13'!
basicRemoveSlotNamed: aSlotName
	"The user has requested that an instance variable be removed..."

	| aSetter aGetter |
	(self okayToRemoveSlotNamed: aSlotName) ifFalse:
		[^ self inform: 'Sorry, ', aSlotName, ' is in
use in a script.'].

	aSetter _ Utilities setterSelectorFor: aSlotName.
	aGetter _ Utilities getterSelectorFor: aSlotName.
	((self systemNavigation allCallsOn: aSetter) size > 0 or: [(self systemNavigation allCallsOn: aGetter) size > 0]) ifTrue:
		[self inform: 
'Caution!!  There may be scripts belonging to
other objects that may rely on the presence of
this variable.  If there are, they may now be broken.
You may need to fix them up manually.'].

	self class removeInstVarName: aSlotName asString.

	self updateAllViewers! !

!Player methodsFor: 'slots-user' stamp: 'yo 9/24/2004 10:06'!
basicRenameSlot: oldSlotName newSlotName: newSlotName
	"Give an existing instance variable a new name"

	self class renameSilentlyInstVar: oldSlotName to: newSlotName.
	self renameSlotInWatchersOld: oldSlotName new: newSlotName.

	self updateAllViewers.

	self presenter allExtantPlayers do:
		[:aPlayer | (aPlayer hasScriptReferencing: oldSlotName ofPlayer: self)
			ifTrue:
				[aPlayer noteRenameOf: oldSlotName to: newSlotName inPlayer: self]].

	self presenter hasAnyTextuallyCodedScripts
		ifTrue:
			[self inform: 
'Caution!!  References in texutally coded scripts won''t be renamed.'].

	^ true! !

!Player methodsFor: 'testing' stamp: 'sw 9/27/2001 17:26'!
basicType
	"Answer a symbol representing the inherent type of the receiver"

	^ #Player! !

!Player methodsFor: 'misc' stamp: 'sw 3/17/2005 00:47'!
beNotZero: aNumber
	"This is a runtime check if the arg to divide in a script is zero.  If it is, put up a warning message.  Return 0.001 instead of 0.  Note the time.  If fails again within 1 min., don't tell the user again."

	aNumber = 0 ifFalse: [^ aNumber].	"normal case"
	"We have a problem"
	TimeOfError 
		ifNil: [TimeOfError _ Time totalSeconds]
		ifNotNil: [(Time totalSeconds - TimeOfError) > 45 ifTrue: [
			TimeOfError _ Time totalSeconds.	"in case user interrupt and reenter"
			self inform: 
'Dividing by zero makes a number too
large for even a Sorcerer to handle.
Please change your script.' translated.
			TimeOfError _ Time totalSeconds]].
	^ 0.001! !

!Player methodsFor: 'misc' stamp: 'sw 7/28/2004 20:51'!
beRevealedInActiveWorld
	"Reveal my corresponding morph in the active world"

	self revealPlayerIn: ActiveWorld! !

!Player methodsFor: 'misc' stamp: 'gk 2/23/2004 20:51'!
beep: soundName
	"Play given sound or at least beep."

	SoundService default playSoundNamedOrBeep: soundName
! !

!Player methodsFor: 'viewer' stamp: 'sw 1/12/1999 16:00'!
belongsToUniClass
	"UnscriptedPlayer reimplements to false"
	^ true! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/13/2004 00:57'!
blueComponentFrom: another

	costume renderedMorph blueComponentFrom: another costume renderedMorph.

! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/13/2004 00:18'!
blueComponentInto: another

	costume renderedMorph blueComponentInto: another costume renderedMorph.

! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/7/2002 13:18'!
bookEmbodied
	"Answer the book embodied by the receiver's costume; usually this is directly the receiver's costume, but in case it is not, we look up the owner chain for one.  This allows page-number messages to be sent to a *page* of the stack, as Alan is wont to do, and have them still find their way to the right place"

	| aMorph |
	^ ((aMorph _ self costume renderedMorph) isKindOf: BookMorph)
		ifTrue:
			[aMorph]
		ifFalse:
			[aMorph ownerThatIsA: BookMorph]! !

!Player methodsFor: 'scripts-standard' stamp: 'dgd 2/22/2003 13:42'!
bounce: soundName 
	"If the receiver's current bounds obtrude beyond the bounds of its container, then 'bounce' it back within the container, and make the indicated sound while doing so"

	| box bounced aCostume |
	(aCostume := self costume) ifNil: [^self].
	(aCostume owner isNil or: [aCostume owner isHandMorph]) ifTrue: [^self].
	box := aCostume owner bounds.
	bounced := false.
	aCostume left < box left 
		ifTrue: 
			[self headRight.
			bounced := true].
	aCostume right > box right 
		ifTrue: 
			[self headLeft.
			bounced := true].
	aCostume top < box top 
		ifTrue: 
			[self headDown.
			bounced := true].
	aCostume bottom > box bottom 
		ifTrue: 
			[self headUp.
			bounced := true].
	bounced ifTrue: [^self makeBounceSound: soundName]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/27/2004 11:43'!
bounceOn: aPlayer

	^ false.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/28/2004 18:25'!
bounceOn: aPlayer color: c

	^ false.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/28/2004 18:39'!
bounceOnColor: aPlayer

	^ false.
! !

!Player methodsFor: 'misc' stamp: 'sw 1/12/1999 16:00'!
browsePlayerClass
	Browser newOnClass: self class label: 'Compiled scripts for ', self externalName! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 12/2/2000 14:48'!
cameraPoint

	^ self costume cameraPoint! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 12/2/2000 14:49'!
cameraPoint: pt

	self costume cameraPoint: pt
! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 3/3/2004 23:58'!
categories
	"Answer a list of categories appropriate to the the receiver and its costumes"

	| aList |
	(self hasCostumeThatIsAWorld)
		ifTrue:	[^ self categoriesForWorld].

	aList _ OrderedCollection new.
	self slotNames notEmpty ifTrue:
		[aList add: ScriptingSystem nameForInstanceVariablesCategory].
	aList addAll: costume categoriesForViewer.
	aList remove: ScriptingSystem nameForScriptsCategory ifAbsent: [].
	aList add: ScriptingSystem nameForScriptsCategory after: aList first.
	^ aList! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 3/3/2004 00:00'!
categoriesForVocabulary: aVocabulary
	"Answer a list of categories appropriate to the receiver and its costumes, in the given Vocabulary"

	| aList |
	self hasCostumeThatIsAWorld
		ifTrue:
			[aList _ self categoriesForWorld]
		ifFalse:
			[aList _ OrderedCollection new.
			self slotNames ifNotEmpty:
				[aList add: ScriptingSystem nameForInstanceVariablesCategory].
			aList addAll: costume categoriesForViewer].
	aVocabulary addCustomCategoriesTo: aList.
	aList remove: ScriptingSystem nameForScriptsCategory ifAbsent: [].
	aList add: ScriptingSystem nameForScriptsCategory after: aList first.
	^ aList! !

!Player methodsFor: 'slots-kernel' stamp: 'dgd 4/4/2006 16:21'!
categoriesForWorld
	"Answer the list of categories given that the receiver is the Player representing a World"

	| aList |
	aList _ #(#'color & border' #geometry #'pen trails' playfield collections #'stack navigation') asOrderedCollection.
	aList addFirst: ScriptingSystem nameForScriptsCategory.
	aList addFirst: ScriptingSystem nameForInstanceVariablesCategory.
	aList add: #input.
	Preferences eToyFriendly ifFalse:
		[aList add: #preferences].

	^ aList! !

!Player methodsFor: 'costume' stamp: 'sw 9/30/2004 04:29'!
ceaseHavingAParameterFor: aSelector
	"Make the script represented by aSelector cease bearing a parameter"

	| newSel |
	self renameScript: aSelector newSelector: (newSel _ (aSelector copyWithout: $:) asSymbol).

	(self scriptEditorFor: newSel) assureParameterTilesValid; install! !

!Player methodsFor: 'costume' stamp: 'sw 7/18/2002 11:26'!
changeParameterTypeFor: aSelector
	"Change the parameter type for the given selector.  Not currently sent, since types are now set by direct manipulation in the Scriptor header.  If this were reinstated someday, there would probably be an issue about getting correct-looking Parameter tile(s) into the Scriptor header(s)"

	| current typeChoices typeChosen |
	current _ self typeforParameterFor: aSelector.
	typeChoices _ Vocabulary typeChoices.
	typeChosen _ (SelectionMenu selections: typeChoices lines: #()) startUpWithCaption: 
		('Choose the TYPE
for the parameter (currently ', current, ')').
	self setParameterFor: aSelector toType: typeChosen

! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/11/2001 20:47'!
changeScript: scriptName toStatus: statusSymbol
	"Change the script of the given name to have the given status, and get all relevant script-status controls updated"

	scriptName ifNil: [^ self].
	Symbol hasInterned: scriptName ifTrue:
		[:sym | self instantiatedUserScriptsDo:
			[:aScriptInstantiation | aScriptInstantiation selector == sym
				ifTrue:
					[aScriptInstantiation status: statusSymbol.
					aScriptInstantiation updateAllStatusMorphs]]]! !

!Player methodsFor: 'translation' stamp: 'sw 7/25/2005 05:04'!
changeTypesInWatchersOf: slotName
	"The type of a variable has changed; adjust watchers to that fact."

	| aGetter newWatcher |
	aGetter _ Utilities getterSelectorFor: slotName.
	self allPossibleWatchersFromWorld do: [:aWatcher |
		(aWatcher getSelector = aGetter) ifTrue:
			[(aWatcher ownerThatIsA: WatcherWrapper) ifNotNilDo:
				[:aWrapper |
					newWatcher _ (aWrapper submorphs size = 1)
						ifTrue:
							[WatcherWrapper new unlabeledForPlayer: self getter: aGetter]
						ifFalse:
							[WatcherWrapper new fancyForPlayer: self getter: aGetter].
					newWatcher position: aWatcher position.
					aWrapper owner replaceSubmorph: aWrapper by: newWatcher]]]
! !

!Player methodsFor: 'pen' stamp: 'ar 10/5/2000 18:53'!
choosePenColor: evt
	self costume changeColorTarget:  self costume  selector: #penColor: originalColor: self getPenColor hand: evt hand! !

!Player methodsFor: 'pen' stamp: 'sw 4/22/1998 13:25'!
choosePenSize
	^ self actorState choosePenSize! !

!Player methodsFor: 'slots-user' stamp: 'sw 3/24/2005 01:33'!
chooseSlotTypeFor: aGetter
	"Let the user designate a type for the slot associated with the given getter"

	| typeChoices typeChosen slotName |
	slotName _ Utilities inherentSelectorForGetter: aGetter.
	typeChoices _ Vocabulary typeChoices.
	typeChosen _ (SelectionMenu labelList: (typeChoices collect: [:t | t translated]) lines: #() selections: typeChoices) startUpWithCaption: 
		('Choose the TYPE
for ' translated, slotName, '
(currently ' translated, (self slotInfoAt: slotName) type translated, ')').
	typeChosen isEmptyOrNil ifTrue: [^ self].
	(self typeForSlot: slotName) capitalized = typeChosen ifTrue: [^ self].

	(self slotInfoAt: slotName) type: typeChosen.
	self class allInstancesDo:   "allSubInstancesDo:"
		[:anInst | anInst instVarNamed: slotName asString put: 
			(anInst valueOfType: typeChosen from: (anInst instVarNamed: slotName))].
	self updateAllViewers.	"does siblings too"
	self changeTypesInWatchersOf: slotName  "does siblings too"
! !

!Player methodsFor: 'slots-user' stamp: 'sw 4/5/1999 13:55'!
chooseUserSlot
	| names aMenu result |
	(names _ self slotNames) size == 1
		ifTrue: [^ names first].
	aMenu _ SelectionMenu selections: names.
	result _ aMenu startUpWithCaption: 'Please choose a variable'.
	result isEmptyOrNil ifTrue: [^ nil].
	^ result! !

!Player methodsFor: 'scripts-standard' stamp: 'jm 4/22/1999 15:52'!
clear

	self sendMessageToCostume: #clear.
! !

!Player methodsFor: 'costume' stamp: 'sw 2/27/2001 18:28'!
clearOwnersPenTrails
	"Clear the pen trails of the containing playfield"

	self costume referencePlayfield clearTurtleTrails! !

!Player methodsFor: 'costume' stamp: 'sw 3/4/2001 13:57'!
clearPenTrails
	"Allow old code invoking the short-lived deviant clearPenTrails command to continue to work when imported from a bj image into a mainstream image.  For backward compatibility only!!"

	self costume referencePlayfield clearTurtleTrails! !

!Player methodsFor: 'pen' stamp: 'sw 10/4/2002 13:14'!
clearTurtleTrails
	"Clear all turtle trails within my costume, presumed to be a playfield"

	self costume renderedMorph clearTurtleTrails! !

!Player methodsFor: 'misc' stamp: 'sw 2/1/98 02:36'!
color
	^ self costume color! !

!Player methodsFor: 'misc' stamp: 'sw 12/13/2001 14:34'!
color: myColor sees: externalColor
	"Answer whether any pixel of one color on my costume is coincident with any pixel of a second color in its surround.  Returns false if the costume is not currently in the world"

	self costume isInWorld ifFalse: [^ false].
	^ self costume color: myColor sees: externalColor! !

!Player methodsFor: 'pen' stamp: 'sw 8/17/1998 17:12'!
colorUnder
	^ self costume colorUnder! !

!Player methodsFor: 'slots-user' stamp: 'sw 1/12/1999 16:00'!
compileInstVarAccessorsFor: varName
	self class compileInstVarAccessorsFor: varName! !

!Player methodsFor: 'scripts-standard' stamp: 'tk 1/14/2001 06:27'!
contents
	^ costume contents! !

!Player methodsFor: 'scripts-standard' stamp: 'tk 1/14/2001 06:27'!
contents: stuff
	^ costume contents: stuff! !

!Player methodsFor: 'copying' stamp: 'tk 1/8/2001 10:38'!
copyUniClassWith: deepCopier
	"my class is a subclass of Player.  Return another class just like my class.  Share the costume list."
	| newCls |
	newCls _ self class officialClass 
		newUniqueClassInstVars: self class instanceVariablesString 
		classInstVars: self class class instanceVariablesString.
	newCls copyMethodDictionaryFrom: self class.
	newCls class copyMethodDictionaryFrom: self class class.
	newCls scripts: self class privateScripts.	"duplicate this in mapUniClasses"
	newCls slotInfo: (self class privateSlotInfo veryDeepCopyWith: deepCopier).
	newCls copyAddedStateFrom: self class.  "All class inst vars for inter Player refs"
	^ newCls
! !

!Player methodsFor: 'costume' stamp: 'sw 12/16/97 11:00'!
costume
	^ costume! !

!Player methodsFor: 'customevents-costume' stamp: 'nk 9/24/2003 17:33'!
costume: aMorph
	"Make aMorph be the receiver's current costume"
	| itsBounds |
	costume == aMorph ifTrue: [^ self].
	costume ifNotNil:
		[self rememberCostume: costume renderedMorph.
		itsBounds _ costume bounds.
		(costume ownerThatIsA: HandMorph orA: PasteUpMorph) replaceSubmorph: costume topRendererOrSelf by: aMorph.
		aMorph position: itsBounds origin.
		aMorph actorState: costume actorStateOrNil.
		aMorph setNameTo: costume externalName].
	aMorph player: self.
	costume _ aMorph.
	aMorph arrangeToStartStepping! !

!Player methodsFor: 'costume' stamp: 'sw 4/7/1999 21:20'!
costumeRespondingTo: aSelector
	"Answer a costume that responds to the given selector, or nil if none"
	| aMorph |
	((aMorph _ self costume renderedMorph) respondsTo: aSelector) ifTrue: [^ aMorph].
	costumes isEmptyOrNil ifFalse:
		[costumes do: [:aCostume | (aCostume respondsTo: aSelector) ifTrue: [^ aCostume]]].
	^ nil "usually an error will result"! !

!Player methodsFor: 'testing' stamp: 'sw 9/15/1998 13:12'!
costumes
	^ costumes! !

!Player methodsFor: 'costume' stamp: 'nk 9/4/2004 17:05'!
costumesDo: aBlock 
	"Evaluate aBlock against every real (not flex) costume known to the receiver,
	starting with the current costume."

	costume ifNotNil: [ aBlock value: costume renderedMorph ].
	costumes
		ifNil: [^ self].
	costumes
		do: [:aCostume | aCostume ~~ costume
				ifTrue: [aBlock value: aCostume renderedMorph]]! !

!Player methodsFor: 'slot-kedama' stamp: 'KR 4/14/2006 15:40'!
createSlotForPatch: aPatchMorph
	"anEmbeddedMorph has just been added to my costume because of explicit user action.  
	Create an instance variable and accessors for it as a Player-valued slot."
	| itsName openViewers |
	self costume isInWorld ifFalse: [ ^self ].

	itsName _ aPatchMorph externalName asSymbol.

	self slotInfo
		at: itsName
		put: (SlotInformation new initialize type: #Patch).
	self addInstanceVarNamed: itsName withValue: (aPatchMorph assuredPlayer).
	self class compileAccessorsFor: itsName.
	openViewers := self allOpenViewersOnReceiverAndSiblings.
	openViewers isEmpty
		ifTrue: [^ self]
		ifFalse: [| aPresenter | 
			(aPresenter := self costume presenter)
				ifNil: [^ self].
			openViewers
				do: [:aViewer | aPresenter updateViewer: aViewer forceToShow: ScriptingSystem nameForInstanceVariablesCategory]]
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/3/2004 23:28'!
decayPatchVariable

	self costume renderedMorph evaporate.
! !

!Player methodsFor: 'misc' stamp: 'sw 9/13/2002 17:52'!
decimalPlacesForGetter: aGetter
	"Answer the number of decimal places wanted when displaying the getter's value.  Answer nil if this object does not have a personal preference regarding this getter."

	^ costume decimalPlacesForGetter: aGetter! !

!Player methodsFor: 'scripts-vector' stamp: 'tk 8/18/2001 22:46'!
decr: aPlayer
	"Treating Players like vectors, subtract aPlayer from me"

	self setX: self getX - aPlayer asPoint x.
	self setY: self getY - aPlayer asPoint y.! !

!Player methodsFor: 'misc' stamp: 'sw 7/4/2004 00:29'!
defaultFloatPrecisionFor: aGetSelector
	"Answer the float position to use in conjunction with a readout for aGetSelector, which will be of the form 'getXXX'"

	| aSlotName slotInfo |
	aSlotName _ Utilities inherentSelectorForGetter: aGetSelector.
	(slotInfo _ self slotInfoAt: aSlotName ifAbsent: [nil]) ifNotNil:
		[^ slotInfo floatPrecision].

	self costume ifNotNil:
		[^ self costume renderedMorph defaultFloatPrecisionFor: aGetSelector].
	^ 1! !

!Player methodsFor: 'user interface' stamp: 'tk 1/3/2001 09:37'!
defaultLabelForInspector
	"Answer the default label to be used for an Inspector window on the receiver."
	^ self knownName ifNil: ['An unaffiliated Player'] ifNotNil: [self knownName]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/16/2004 18:50'!
defaultPatchPlayer

	^ costume renderedMorph kedamaWorld player getPatch.
! !

!Player methodsFor: 'pen' stamp: 'sw 2/4/98 15:16'!
defaultPenColor
	^ Color blue! !

!Player methodsFor: 'pen' stamp: 'sw 2/4/98 15:03'!
defaultPenSize
	^ 1! !

!Player methodsFor: 'slots-user' stamp: 'sw 7/18/2002 11:15'!
defaultValueOfType: aSymbol
	"Answer a default value for the given type -- invoked in compiled user scripts when a parameter tile of the wrong type is present"

	^ self initialValueForSlotOfType: aSymbol
	"Not really intended for that purpose but seemingly serves adequately"! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/23/2000 16:07'!
deleteCard
	"Tell the receiver's stack to delete the current card"

	self costume stackDo: [:aStack | aStack deleteCard]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/3/2004 00:29'!
die
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/3/2004 23:28'!
diffusePatchVariable
	"Diffuse the patch variable of the given name."

	self costume renderedMorph diffuse.
! !

!Player methodsFor: 'scripts-vector' stamp: 'tk 8/18/2001 22:51'!
dividedBy: aNumber
	"Treating Players like vectors, divide myself by aNumber"

	self setX: self getX / aNumber asPoint x.
	self setY: self getY / aNumber asPoint y.
! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 2/5/2001 11:25'!
doButtonAction
	"Do the button action of my costume"

	self costume renderedMorph doButtonAction! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/14/2004 15:56'!
doCommand: aBlock

	^ aBlock value: self.
! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/28/2000 10:58'!
doMenuItem: menuString
	"Do the menu item whose wording is provided"

	self costume doMenuItem: menuString! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/2/2001 11:59'!
doScript: scriptNameString
	"On the next tick of the clock, run the given script once"

	Symbol hasInterned: scriptNameString ifTrue:
		[:sym | (self class includesSelector: sym) ifTrue:
			[costume addAlarm: #triggerScript: with: sym after: 1]]! !

!Player methodsFor: 'error handling' stamp: 'tk 11/21/2000 16:17'!
doesNotUnderstand: aMessage 
	 | ours |
"See it the message is a special setter that has not been defined.  Define it and try again."

	ours _ false.
	(aMessage selector endsWith: 'IncreaseBy:') ifTrue: [ours _ true].
	(aMessage selector endsWith: 'DecreaseBy:') ifTrue: [ours _ true].
	(aMessage selector endsWith: 'MultiplyBy:') ifTrue: [ours _ true].
	ours ifFalse: [^ super doesNotUnderstand: aMessage].
	(self addSpecialSetter: aMessage selector) ifFalse: ["not our inst var"
		^ super doesNotUnderstand: aMessage].
	^ aMessage sentTo: self! !

!Player methodsFor: 'misc' stamp: 'sw 11/28/2000 07:25'!
dummy
	"this space for rent"! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 2/5/2001 14:04'!
editDescriptionForSelector:  aSelector
	"Allow the user to edit the balloon-help description for the given selector"

	(self class userScriptForPlayer: self selector: aSelector) editDescription.
	self updateAllViewers! !

!Player methodsFor: 'viewer' stamp: 'sw 5/22/2001 14:56'!
elementTypeFor: aStringOrSymbol vocabulary: aVocabulary
	"Answer whether aStringOrSymbol is best characterized as a #systemSlot, #systemScript, #userSlot, or #userScript.  This is ancient and odious but too tedious to rip out at this point."

	| aSymbol anInterface aSlotName |
	aSymbol _ aStringOrSymbol asSymbol.
	aSlotName _ Utilities inherentSelectorForGetter: aSymbol.
	(self slotInfo includesKey: aSlotName) ifTrue: [^ #userSlot].
	(self class isUniClass and: [self class scripts includesKey: aSymbol]) ifTrue: [^ #userScript].
	
	anInterface _ aVocabulary methodInterfaceAt: aSymbol ifAbsent: [nil].
	^ anInterface
		ifNotNil:
			[(anInterface resultType == #unknown)
				ifTrue:
					[#systemScript]
				ifFalse:
					[#systemSlot]]
		ifNil:
			[#systemScript]! !

!Player methodsFor: 'scripts-kernel' stamp: 'ar 1/10/2001 12:10'!
emptyScript
	"The empty script"! !

!Player methodsFor: 'misc' stamp: 'sw 7/28/2004 20:52'!
entryForPlayersTool: aPlayersTool
	"Answer an entry for the receiver in the All Players tool"

	^ PlayerSurrogate newRow playerRepresented: self! !

!Player methodsFor: 'misc' stamp: 'sw 7/8/2004 01:29'!
erase
	"Dismiss the receiver from the screen.  It can subsequently be found in the trash if need be, provided the preserveTrash preference is set to true"

	self costume topRendererOrSelf dismissViaHalo! !

!Player methodsFor: 'customevents-scripts-kernel' stamp: 'nk 9/25/2003 11:53'!
existingScriptInstantiationForSelector: scriptName
	"Answer the existing script instantiation for the given selector, or nil if none"

	scriptName ifNil: [^ nil].
	Symbol hasInterned: scriptName
		ifTrue: [ :sym |
			self costume actorStateOrNil ifNotNilDo: [ :actorState |
				^actorState instantiatedUserScriptsDictionary at: sym ifAbsent: [nil]]].
	^ nil! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 17:03'!
expungeEmptyScripts
	"Track down and destroy -- and destroy screen artifacts relating to -- all scripts belonging to the receiver that have no lines of code in them"

	| any |
	any _ false.
	self class namedTileScriptSelectors do:
		[:aSel |
			(self isEmptyTileScript: aSel)
				ifTrue:
					[any _ true.
					self removeScriptWithoutUpdatingViewers: aSel]].
	any ifTrue: [self updateAllViewersAndForceToShow: #scripts]
			
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 17:04'!
expungeEmptyUnRenamedScripts
	"Track down and destroy -- and destroy screen artifacts relating to -- all scripts belonging to the receiver that have not been named and that have no lines of code in them"

	| any |
	any _ false.
	self class namedTileScriptSelectors do:
		[:aSel |
			(self isExpendableScript: aSel)
				ifTrue:
					[any _ true.
					self removeScriptWithoutUpdatingViewers: aSel]].
	any ifTrue:
		[self updateAllViewersAndForceToShow: #scripts]
			
! !

!Player methodsFor: 'viewer' stamp: 'sw 1/28/2001 20:17'!
externalName
	"Answer an external name for the receiver.  If it has none, supply a backstop name"

	| aCostume |
	^ (aCostume _ self costume) ifNotNil: [aCostume externalName] ifNil: ['an orphaned Player']! !

!Player methodsFor: 'slots-user' stamp: 'yo 2/12/2005 18:58'!
fancyWatcherFor: aGetter
	"Anser a labeled readout for viewing a value textuallyi"

	| aWatcher aColor aLine itsName aSelector aLabel |
	aWatcher _ self unlabeledWatcherFor: aGetter.
	aColor _ Color r: 0.387 g: 0.581 b: 1.0.
	aLine _ WatcherWrapper newRow.
	aLine player: self variableName: (aSelector _ Utilities inherentSelectorForGetter: aGetter).
	itsName _ aWatcher externalName.
	aWatcher setNameTo: 'readout'.
	aLine addMorphFront: (self tileReferringToSelf
				borderWidth: 0; layoutInset: 4@0;
				typeColor: aColor; 
				color: aColor; bePossessive).
	aLabel _ StringMorph contents: aSelector translated, ' = ' font: ScriptingSystem fontForTiles.
	aLabel setProperty: #watcherLabel toValue: true.
	aLine addMorphBack: aLabel.
	aLine addMorphBack: aWatcher.
	aLine setNameTo: itsName.

	^ aLine! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 2/6/2001 21:13'!
fire
	"Do the button action of my costume"

	self costume renderedMorph fire! !

!Player methodsFor: 'scripts-execution' stamp: 'sw 2/6/2001 23:21'!
fireOnce
	"If the receiver has any script armed to be triggered on mouse down and/or mouse-up, run those scripts now -- first the mouseDown ones, then the mouseUp ones."

	self instantiatedUserScriptsDo:
		[:aScriptInst |
			aScriptInst status == #mouseDown ifTrue: [aScriptInst fireOnce]].
	self instantiatedUserScriptsDo:
		[:aScriptInst |
			aScriptInst status == #mouseUp ifTrue: [aScriptInst fireOnce]].
! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/26/1998 15:56'!
firstPage
	self sendMessageToCostume: #firstPage! !

!Player methodsFor: 'costume' stamp: 'mga 11/18/2003 11:23'!
flipHorizontal
	self costume flipHorizontal! !

!Player methodsFor: 'costume' stamp: 'mga 11/18/2003 11:23'!
flipVertical
	self costume flipVertical! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/28/2000 10:59'!
followPath
	"If there is a path defined for this object, follow it now"

	self costume followPath! !

!Player methodsFor: 'costume' stamp: 'sw 8/3/2000 10:16'!
forgetOtherCostumes
	self resetCostumeList.
	self updateAllViewers! !

!Player methodsFor: 'scripts-standard' stamp: 'mir 6/7/2002 17:08'!
forward: dist 
	"Move forward (viz. in the direction of my heading) by the given amount"

	| rho radians delta didStray p aCostume aPlayfield |
	(aCostume _ self costume) isInWorld ifFalse: [^ self].
	aCostume isWorldOrHandMorph ifTrue: [^ self].
	aCostume owner isHandMorph ifTrue: [^ self].

	rho _ (aCostume asNumber: dist) asFloat.
	radians _ (self getHeadingUnrounded asFloat - 90.0) degreesToRadians.
	delta _ (radians cos @ radians sin) * rho.

	(aPlayfield _ aCostume pasteUpMorph) fenceEnabled ifTrue:
		[(aPlayfield bounds containsRect: aCostume bounds) ifFalse:
			["If I stray out of the bounds of my playfield, pull me back, but
			 without changing my heading as bounce would. Do nothing if
			 bounce has already corrected the direction."
			didStray _ false.
			((aCostume left < aPlayfield left and: [delta x < 0]) or:
			 [aCostume right > aPlayfield right and: [delta x > 0]]) ifTrue:
				[delta _ delta x negated @ delta y.
				didStray _ true].
			((aCostume top < aPlayfield top and: [delta y < 0]) or:
			 [aCostume bottom > aPlayfield bottom and: [delta y > 0]]) ifTrue:
				[delta _ delta x @ delta y negated.
				didStray _ true].
			(didStray and: [Preferences fenceSoundEnabled]) ifTrue: [aCostume makeFenceSound]]].

	"use and record the fractional position"
	p _ aCostume referencePosition + delta.
	aCostume referencePosition: p! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 10/30/2000 07:08'!
getActWhen
	"Answer the #actWhen status of my costume, which is expected to be a button with an #actWhen protocol"

	^ costume renderedMorph actWhen! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/15/2000 06:11'!
getAllButFirstCharacter 
	"Answer a string consisting of all but the first character in its string"

	^ costume renderedMorph getAllButFirstCharacter! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:25'!
getAllowEtoyUserCustomEvents
	"Answer whether to use the vector vocabulary."

	^ Preferences allowEtoyUserCustomEvents! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 7/19/2005 01:47'!
getAlpha
	"Answer the alpha of the color of my costume."

	^ self getColor alpha! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 10/24/1998 23:15'!
getAmount
	^ self getValueFromCostume: #amount! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/17/1998 15:40'!
getAngle
	^ self getValueFromCostume: #angle! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 09:59'!
getAngleTo: aPlayer

	^ 0.0.
! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 10/3/2004 01:45'!
getBackgroundColor
	"Answer the background color; the costume is presumed to be a TextMorph"

	^  self costume renderedMorph backgroundColor ifNil: [Color transparent]! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 6/12/2004 10:00'!
getBaseGraphic
	"Answer a form representing the receiver's base graphic"

	| aMorph |
	^ ((aMorph _ costume renderedMorph) isSketchMorph)
		ifTrue:
			[aMorph baseGraphic]
		ifFalse:
			[aMorph imageForm]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:26'!
getBatchPenTrails
	"Answer whether to batch pen trails."

	^ Preferences batchPenTrails! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/12/2004 22:55'!
getBlueComponentIn: aPatch! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 7/5/2004 22:47'!
getBorderColor
	"Answer the border color of my costume"

	^ costume renderedMorph borderStyle color ifNil: [costume renderedMorph borderStyle baseColor]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/26/2001 10:02'!
getBorderStyle
	"Answer the border style"

	^ costume renderedMorph borderStyle style! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/26/2001 11:33'!
getBorderWidth
	"Answer the border width of my costume"

	^ costume renderedMorph borderStyle width! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/28/2000 11:06'!
getBottom
	"Answer the bottom coordinate, in the cartesian sense (decreases towards bottom of screen)"

	^ self costume cartesianBoundsTopLeft y - self costume height! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 16:34'!
getBottomEdgeMode

	^ self getValueFromCostume: #bottomEdgeMode.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'ar 1/15/2001 17:45'!
getBrightnessUnder
	^ self costume colorUnder brightness * 100! !

!Player methodsFor: 'slot getters/setters' stamp: 'mir 9/4/2005 16:45'!
getButton1
	^ self getValueFromCostume: #button1! !

!Player methodsFor: 'slot getters/setters' stamp: 'mir 9/4/2005 16:46'!
getButton2
	^ self getValueFromCostume: #button2! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 12/2/2000 15:58'!
getCameraPoint

	^ self costume cameraPoint! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/4/2002 19:31'!
getCardNumber
	"Answer the current card number"

	| aStack |
	^ (aStack _ self stackEmbodied) cardNumberOf: aStack currentCard! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:31'!
getCellInset
	"Getter for costume's cellInset"

	^ costume cellInset! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 2/17/2003 18:09'!
getCharacterAtCursor
	"Answer the value of the text cursor"

	| aLoc aTextMorph aString |
	aLoc _ (aTextMorph _ self costume renderedMorph) cursor.
	aString _ aTextMorph text string.
	^ (aString at: aLoc ifAbsent: ['']) asString! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/14/2000 11:26'!
getCharacters
	"Answer the characters in my costume, likely a TextMorph"

	^ costume renderedMorph getCharacters! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 10:22'!
getClipSubmorphs
	"Getter for costume's clipSubmorphs"

	^ costume renderedMorph clipSubmorphs! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 12/10/2001 00:34'!
getColor
	"Answer the color of my costume.  If it uses a gradient fill, answer the first color."

	| aFillStyle aMorph |
	^ (aFillStyle _ (aMorph _ self costume renderedMorph) fillStyle) isGradientFill
		ifTrue:
			[aFillStyle colorRamp first value]
		ifFalse:
			[aMorph color]! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 4/5/2005 11:08'!
getColorOpaque

	^ self getColor.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 12/15/2000 13:30'!
getColorUnder
	"Answer the color beneath the center of the costume"

	^ self costume colorUnder! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 6/12/2004 10:00'!
getCostume
	"Answer a form representing the receiver's primary graphic.  An earlier wording, disused but may persist in preexisting scripts."

	| aMorph |
	^ ((aMorph _ costume renderedMorph) isSketchMorph)
		ifTrue:
			[aMorph form]
		ifFalse:
			[aMorph imageForm]! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 6/12/2004 10:00'!
getCostumeAtCursor
	"Answer the form representing the object at the current cursor.  An earlier wording, disused but may persist in preexisting scripts"

	| anObject aMorph |
	
	anObject _ self getValueFromCostume: #valueAtCursor.
	^ anObject == 0  "weird return from GraphMorph"
		ifTrue:
			[ScriptingSystem formAtKey: #Paint]
		ifFalse:
			[((aMorph _ anObject renderedMorph) isSketchMorph)
				ifTrue:
					[aMorph form]
				ifFalse:
					[anObject imageForm]]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 2/17/2003 11:46'!
getCount
	"Answer the number of elements"

	^ self costume renderedMorph elementCount! !

!Player methodsFor: 'playing commands' stamp: 'jm 9/28/2004 17:09'!
getCurrentFrameForm

	^ self sendMessageToCostume: #getCurrentFrameForm
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/7/2000 09:25'!
getCursor
	"Obtain the cursor setting from the receiver's costume"

	^ costume renderedMorph cursor
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/7/2000 09:25'!
getCursorWrapped
	"maintained for backward compatibility only, for preexisting etoys"

	^ self getCursor
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/10/2000 13:50'!
getDescending
	^  self getValueFromCostume: #descending! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 5/30/2004 17:09'!
getDiffusionRate

	^ self getValueFromCostume: #diffusionRate.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/15/2004 05:13'!
getDisplayScaleMax

	^ self getValueFromCostume: #displayMax.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/4/2004 17:44'!
getDisplayShiftAmount

	^ self getValueFromCostume: #displayShiftAmount.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 3/7/2005 12:00'!
getDisplayType

	^ self getValueFromCostume: #displayType.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/17/2001 15:11'!
getDistance
	"Answer distance from the origin to the objet's position"

	^ (self getX @ self getY) r! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/14/2004 15:53'!
getDistanceTo: aPlayer

	^ 0.0.
! !

!Player methodsFor: 'pen' stamp: 'sw 4/17/2003 12:26'!
getDotSize
	"Answer the receiver's dotSize"

	^ self costume renderedMorph valueOfProperty: #trailDotSize ifAbsentPut: [6]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:29'!
getDragEnabled
	"Getter for costume's dragEnabled"

	^ costume dragEnabled! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:32'!
getDropEnabled
	"Getter for costume's dropEnabled"

	^ costume dropEnabled! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 7/10/2005 04:22'!
getDropProducesWatcher
	"Answer the value of the dropProducesWatcher preference."

	^ Preferences dropProducesWatcher! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/12/2005 14:43'!
getDropShadow
	"Getter for costume's hasDropShadow"

	^ costume renderedMorph hasDropShadow! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:27'!
getEToyFriendly
	"Answer the value of the eToyFriendly preference."

	^ Preferences eToyFriendly! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 5/30/2004 17:23'!
getEvaporationRate

	^ self getValueFromCostume: #evaporationRate.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:28'!
getFenceEnabled
	"Answer the value of the fenceEnabled preference."

	^ Preferences fenceEnabled! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/14/2000 11:26'!
getFirstCharacter
	"Answer a string consisting of the first character in my costume, likely itself a TextMorph"

	^ costume renderedMorph getFirstCharacter! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 2/22/2003 13:43'!
getFirstElement
	"Answer a player representing the receiver's costume's first submorph"

	| itsMorphs |
	^(itsMorphs := costume submorphs) notEmpty 
		ifFalse: [costume presenter standardPlayer]
		ifTrue: [itsMorphs first assuredPlayer]! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:11'!
getFogColor
	^self costume renderedMorph fogColor! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:22'!
getFogDensity
	^self costume renderedMorph fogDensity * 100! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:23'!
getFogRangeEnd
	^self costume renderedMorph fogRangeEnd * 100! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:23'!
getFogRangeStart
	^self costume renderedMorph fogRangeStart * 100! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:11'!
getFogType
	^self costume renderedMorph fogType! !

!Player methodsFor: 'playing commands' stamp: 'sw 10/13/2004 06:41'!
getFrameGraphic
	"Answer a form representing the receiver's costume's current graphic"

	^ self sendMessageToCostume: #getCurrentFrameForm
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 10/30/2000 07:09'!
getGetListSelector
	"Answer the selector used by my costume (a list) to obtain a fresh copy of its list"

	^ self costume renderedMorph getListSelector! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 6/12/2004 10:00'!
getGraphic
	"Answer a form representing the receiver's primary graphic"

	| aMorph |
	^ ((aMorph _ costume renderedMorph) isSketchMorph)
		ifTrue:
			[aMorph form]
		ifFalse:
			[aMorph isPlayfieldLike
				ifTrue:
					[aMorph backgroundForm]
				ifFalse:
					[aMorph imageForm]]! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 6/12/2004 10:00'!
getGraphicAtCursor
	"Answer the form representing the object at the current cursor"

	| anObject aMorph |
	
	anObject _ self getValueFromCostume: #valueAtCursor.
	^ anObject == 0  "weird return from GraphMorph"
		ifTrue:
			[ScriptingSystem formAtKey: #Paint]
		ifFalse:
			[((aMorph _ anObject renderedMorph) isSketchMorph)
				ifTrue:
					[aMorph form]
				ifFalse:
					[aMorph isPlayfieldLike
						ifTrue:
							[aMorph backgroundForm]
						ifFalse:
							[aMorph imageForm]]]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/12/2004 22:55'!
getGreenComponentIn: aPatch! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/14/2004 15:46'!
getGrouped

	^ self getValueFromCostume: #isGroup.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:29'!
getHResizing
	"Getter for costume's hResizing"

	^ costume hResizing! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/15/2002 12:03'!
getHeading
	"Answer the heading of the object, *formerly* ;-) always given as a whole number"

	^ self getHeadingUnrounded "rounded"! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/17/2002 10:50'!
getHeadingTheta
	"Answer the angle, in degrees, between the positive x-axis and the receiver's heading vector"

	| aHeading excess normalized |
	aHeading _ self getHeadingUnrounded.
	excess _ aHeading - (aHeading rounded).

	normalized _ (450 - aHeading) \\ 360.
	^ normalized + excess! !

!Player methodsFor: 'slot getters/setters' stamp: 'ar 9/22/2000 13:42'!
getHeadingUnrounded
	^ self costume heading asSmallAngleDegrees! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/28/2000 11:08'!
getHeight
	"Answer the height of the object"

	^ self costume height! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/6/2001 19:38'!
getHolder
	"Answer the player belonging to my costume's container"

	^ costume topRendererOrSelf owner topRendererOrSelf assuredPlayer! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:35'!
getIndexInOwner
	"Answer my costume's index in its owner"

	^ costume getIndexInOwner! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 12/10/2001 00:39'!
getIsLocked
	"Answer whether the receiver's costume is locked"

	^ costume isLocked! !

!Player methodsFor: 'misc' stamp: 'sw 2/1/1999 14:04'!
getIsOverColor: aColor
	^ self seesColor: aColor! !

!Player methodsFor: 'playing commands' stamp: 'dgd 2/15/2004 22:03'!
getIsRunning
	^ self sendMessageToCostume: #getIsRunning! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/28/2000 07:24'!
getIsUnderMouse
	"Answer true or false, depending on whether the object currently is or is not under the mouse"

	costume isInWorld ifFalse: [^ false].
	^ costume containsPoint: (costume pointFromWorld: costume primaryHand lastEvent cursorPoint)! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:29'!
getKeepTickingWhilePainting
	"Answer the value of the keepTickingWhilePainting preference."

	^ Preferences keepTickingWhilePainting! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/7/2000 17:36'!
getKnobColor
	^ self getValueFromCostume: #sliderColor! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 10/30/2000 07:10'!
getLabel
	"Answer the label of my costume"

	^ self costume renderedMorph label! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/10/2004 12:15'!
getLastCharacter
	"Answer my costume's last character."

	^ costume renderedMorph getLastCharacter! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 10/13/2004 11:30'!
getLastKeystroke
	"Answer the last keystroke fielded"

	^ self getValueFromCostume: #lastKeystroke! !

!Player methodsFor: 'slot getters/setters' stamp: 'jm 4/21/1999 11:46'!
getLastValue

	^ self getValueFromCostume: #lastValue
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:29'!
getLayoutInset
	"Getter for costume's layoutInset"

	^ costume layoutInset! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/28/2000 11:09'!
getLeft
	"answer the left coordinate"

	^ self costume cartesianBoundsTopLeft x! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 16:33'!
getLeftEdgeMode

	^ self getValueFromCostume: #leftEdgeMode.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/17/1998 15:41'!
getLeftRight
	^ self getValueFromCostume: #leftRight! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 3/10/2004 12:15'!
getLength
	"Answer the length of the object"

	| aLength cost |
	((cost  _ self costume) isLineMorph) "annoying special case"
		ifTrue:
			[^ cost unrotatedLength].
	aLength _ cost renderedMorph height.  "facing upward when unrotated"
	cost isRenderer
		ifTrue:
			[aLength _ aLength * cost scaleFactor].
	^ aLength! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:31'!
getListCentering
	"Getter for costume's listCentering"

	^ costume listCentering! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:30'!
getListDirection
	"Getter for costume's listDirection"

	^ costume listDirection! !

!Player methodsFor: 'slot getters/setters' stamp: 'ar 1/15/2001 17:45'!
getLuminanceUnder
	^ self costume colorUnder luminance * 100! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/6/2000 17:11'!
getMaxVal
	^  self getValueFromCostume: #maxVal! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/6/2000 17:11'!
getMinVal
	^  self getValueFromCostume: #minVal! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 2/12/2005 20:18'!
getMouseX
	^ self costume renderedMorph mouseX! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 2/12/2005 20:26'!
getMouseY
	^ self costume renderedMorph mouseY.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 2/3/98 23:36'!
getName
	^ self externalName! !

!Player methodsFor: 'slot getters/setters' stamp: 'tak 1/26/2005 14:58'!
getNewClone
	"Answer a new player of the same class as the receiver, with a costume much like mine"

	| clone |
	clone _  costume usableSiblingInstance.
	costume pasteUpMorph ifNotNilDo: [:parent | parent addMorph: clone].
	^ clone player
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:36'!
getNumberAtCursor
	"Answer the number borne by the object at my costume's current cursor position"

	| renderedMorph aCostume |
	aCostume _ self costume.
	((renderedMorph _ aCostume renderedMorph) respondsTo: #valueAtCursor:) ifTrue: [^ renderedMorph valueAtCursor getNumericValue]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/14/2000 11:39'!
getNumericValue
	"Answer the numeric value contained in my costume"

	^ costume renderedMorph getNumericValue! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/7/2000 12:52'!
getObtrudes
	"Answer whether the receiver's costume obtrudes beyond the bounds of its container"

	| aCostume |
	(aCostume _ self costume) ifNil: [^ false].
	^ aCostume obtrudesBeyondContainer
! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 11/22/2000 08:43'!
getOffsetX

	^ self costume offsetX! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 11/22/2000 08:43'!
getOffsetY

	^ self costume offsetY! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 13:53'!
getOliveHandleForScriptedObjects

	"Answer the value of the oliveHandleForScriptedObjects preference."

	^ Preferences oliveHandleForScriptedObjects! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/7/2002 13:31'!
getPageNumber
	"Answer the current page number of my book"

	| aBook |
	^ (aBook _ self bookEmbodied) pageNumberOf: aBook currentPage! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 19:46'!
getPatchName

	^ (self perform: #getPatch) externalName.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/19/2004 00:05'!
getPatchValueIn: aPatch

	^ 0.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/5/2004 16:41'!
getPatchesList

	^ self getValueFromCostume: #patchesToDisplayAsString.
! !

!Player methodsFor: 'pen' stamp: 'tk 10/4/2001 16:47'!
getPenArrowheads
	"Answer a boolean indicating whether the receiver's pen will draw an arrowhead at the end of a stroke"
	^ self actorState getPenArrowheads! !

!Player methodsFor: 'pen' stamp: 'sw 11/28/2000 07:25'!
getPenColor
	"Answer the current pen color"

	^ self actorState getPenColor! !

!Player methodsFor: 'pen' stamp: 'sw 11/28/2000 07:26'!
getPenDown
	"Answer a boolean indicating whether the receiver's pen is currently down (true) or up (false)"
	^ self actorState getPenDown! !

!Player methodsFor: 'pen' stamp: 'sw 11/28/2000 07:26'!
getPenSize
	"Answer a number indicating the current pen size"

	^ self actorState getPenSize! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/11/2004 01:16'!
getPixelsPerPatch

	^ self getValueFromCostume: #pixelsPerPatch.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/15/2000 10:10'!
getPlayerAtCursor
	^ self getValueAtCursor! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 2/15/2004 21:11'!
getPosition
	"Answer the numeric value contained in my costume"
	^ costume renderedMorph getPosition! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 08:32'!
getRadialGradientFill
	"Geter for costume's useGradientFill"

	| aStyle |
	^ (aStyle _ costume renderedMorph fillStyle) isGradientFill and:
		[aStyle isRadialFill]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/12/2004 22:55'!
getRedComponentIn: aPatch! !

!Player methodsFor: 'playing commands' stamp: 'dgd 2/15/2004 22:06'!
getRepeat
	^ self sendMessageToCostume: #getRepeat! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/2/2004 23:17'!
getReplicated
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/15/2001 16:34'!
getResistsRemoval
	"Answer whether the receiver is marked to resist removal"

	^ costume resistsRemoval! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/28/2000 11:09'!
getRight
	"Answer the coordinate of the right edge of the object"

	^ self costume cartesianBoundsTopLeft x + self costume width! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 16:32'!
getRightEdgeMode

	^ self getValueFromCostume: #rightEdgeMode.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 7/14/2004 21:37'!
getRotationStyle
	"Answer the symbol representing the rotation style"

	^ (#(rotate #'do not rotate' #'flip left right' #'flip up down') at:
		(#(normal none leftRight upDown ) indexOf: costume renderedMorph rotationStyle))! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/30/2000 07:12'!
getRoundedCorners
	"Anwer the rounded-corners attribute of my costume"

	^ costume renderedMorph cornerStyle == #rounded! !

!Player methodsFor: 'slot getters/setters' stamp: 'jm 4/22/1999 15:45'!
getSampleAtCursor
	"Note: Performance hacked to allow real-time sound. Assumes costume is a GraphMorph."

	^ costume renderedMorph interpolatedValueAtCursor
! !

!Player methodsFor: 'slot getters/setters' stamp: 'ar 1/15/2001 17:45'!
getSaturationUnder
	^ self costume colorUnder saturation * 100! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 11/22/2000 08:35'!
getScale

	^ self costume scale! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 2/15/2002 02:37'!
getScaleFactor
	"Answer the scale factor of the object"

	^ self costume scaleFactor! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 12/10/2001 00:12'!
getSecondColor
	"Getter for costume's second color, if it's using gradient fill; sonst answers white."

	| aFillStyle |
	^ (aFillStyle _ costume renderedMorph fillStyle) isGradientFill
		ifTrue:
			[aFillStyle  colorRamp last value]
		ifFalse:
			[Color white]! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/14/2005 10:38'!
getShadowColor
	"Getter for costume's shadowColor"

	^ costume renderedMorph shadowColor! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 5/30/2004 17:07'!
getSniffRange

	^ self getValueFromCostume: #sniffRange.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 2/21/98 23:23'!
getSpeed
	"If user defines a speed slot, it will override this.  This provides a backstop value for use with moveToward:, etc"

	^ 5 ! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:30'!
getSticky
	"Getter for costume's isSticky"

	^ costume isSticky! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 2/18/2003 02:20'!
getStringContents
	"Answer the String contents"

	^ self costume renderedMorph getCharacters! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 3/8/2004 21:40'!
getSubtitlesFileName
	"Answer the subtitlesFileName in my costume"
	^ costume renderedMorph getSubtitlesFileName! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/17/2001 15:10'!
getTheta
	"Answer the angle between the positive x-axis and the line connecting the origin and the object's position"

	^ (self getX @ self getY) degrees! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/28/2000 11:10'!
getTop
	"Answer the coordinate of the topmost point of the object, using cartesian sense"

	^ self costume cartesianBoundsTopLeft y! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 16:33'!
getTopEdgeMode

	^ self getValueFromCostume: #topEdgeMode.
! !

!Player methodsFor: 'playing commands' stamp: 'sw 10/13/2004 06:38'!
getTotalFrames
	"Answer the receiver's costume's totalFrames.  Applies to MPEGMoviePlayerMorphs"

	^ self sendMessageToCostume: #totalFrames
! !

!Player methodsFor: 'playing commands' stamp: 'sw 10/13/2004 06:37'!
getTotalSeconds
	"Answer the total number of seconds in the receiver's costume, typically a movie"

	^ self sendMessageToCostume: #totalSeconds
! !

!Player methodsFor: 'pen' stamp: 'sw 3/11/2003 11:28'!
getTrailStyle
	"Answer the receiver's trailStyle"

	^ self actorState trailStyle! !

!Player methodsFor: 'customevents-custom events' stamp: 'nk 11/1/2004 11:26'!
getTriggeringObject
	"Answer the Player that is triggering the current script,
	or the default UnscriptedPlayer if none."

	| rcvr |
	rcvr := GetTriggeringObjectNotification signal.

	^rcvr ifNil: [ self costume presenter standardPlayer ]
		ifNotNil: [ rcvr isMorph
				ifTrue: [ rcvr assuredPlayer ]
				ifFalse: [ rcvr ]]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/12/2000 11:51'!
getTruncate
	^  self getValueFromCostume: #truncate! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/2/2004 16:36'!
getTurtleCount

	^ self getValueFromCostume: #turtleCount.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 3/7/2005 07:32'!
getTurtleVisible

	^ self costume visible.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/9/2004 15:21'!
getTurtlesList

	^ self getValueFromCostume: #turtlesToDisplayAsString.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/17/1998 15:42'!
getUpDown
	^ self getValueFromCostume: #upDown! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/7/2004 18:04'!
getUphillIn: aPatch

	^ self costume heading.

! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 08:33'!
getUseGradientFill
	"Geter for costume's useGradientFill"

	^ costume renderedMorph fillStyle isGradientFill! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/15/2004 05:15'!
getUseLogDisplay

	^ self getValueFromCostume: #useLogDisplay.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 7/10/2005 03:36'!
getUseVectorVocabulary
	"Answer whether to use the vector vocabulary."

	^ Preferences useVectorVocabulary! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:29'!
getVResizing
	"Getter for costume's vResizing"

	^ costume vResizing! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/17/1998 15:43'!
getValueAtCursor
	| anObject |
	anObject _ self getValueFromCostume: #valueAtCursor.
	^ anObject == 0  "weird return from GraphMorph"
		ifTrue:
			[nil]
		ifFalse:
			[anObject assuredPlayer]! !

!Player methodsFor: 'costume' stamp: 'sw 8/17/1998 15:39'!
getValueFromCostume: aSelector
	| aCostume |
	(aCostume _ self costumeRespondingTo: aSelector) ifNotNil:
		[^ aCostume perform: aSelector].
	^ nil! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 3/8/2004 18:26'!
getVideoFileName
	"Answer the videoFileName in my costume"
	^ costume renderedMorph getVideoFileName! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 10/30/2000 07:12'!
getViewingByIcon
	"Answer whether my costume is currently viewing by icon"

	^ costume renderedMorph viewingNormally! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 2/15/2004 21:13'!
getVolume
	"Answer the numeric value contained in my costume"
	^ costume renderedMorph getVolume! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 3/10/2004 12:15'!
getWidth
	"Answer the width of the object"

	| aWidth cost |
	((cost _ self costume) isLineMorph) "annoying special case"
		ifTrue:
			[^ cost unrotatedWidth].
	aWidth _ cost renderedMorph width.  "facing upward when unrotated"

	cost isRenderer
		ifTrue:
			[aWidth _ aWidth * cost scaleFactor].
	^ aWidth! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:27'!
getWrapDirection
	"Getter for costume's wrapDirection"

	^ costume wrapDirection! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/17/1998 17:23'!
getX
		"emergency patch; unclear why not needed in getY; in any case, have
		 removed the getX/getY retrievals from the viewer in 2.0-final anyway"
	| aCostume |
	(aCostume _ self costume) isInWorld ifFalse: [^ 100].
	^ aCostume x! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/28/2000 07:22'!
getY
	"Answer the y coordinate of the object, relative to its container"

	^ self costume y! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/23/2000 16:12'!
goToCard: aCard
	"Install aCard as the new current card of the stack"

	self stackDo: [:aStack | aStack goToCard: aCard]! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/23/2000 16:19'!
goToFirstCardInBackground
	"Send the stack to the first card of the current background"

	costume stackDo: [:aStack | aStack goToFirstCardInBackground]! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/23/2000 16:19'!
goToFirstCardOfStack
	"Send the stack to the first card of the entire stack"

	costume stackDo: [:aStack | aStack goToFirstCardOfStack]! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/23/2000 16:17'!
goToLastCardInBackground
	"Send the stack to the final card of the current background"

	costume stackDo: [:aStack | aStack goToLastCardInBackground]! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/23/2000 16:20'!
goToLastCardOfStack
	"Send the stack to the last card of the entire stack"

	costume stackDo: [:aStack | aStack goToLastCardOfStack]! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/23/2000 14:54'!
goToNextCardInStack
	"Install the next card into the receiver's stack"

	self costume goToNextCardInStack! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/23/2000 16:13'!
goToPreviousCardInStack
	"Install the previous card into the receiver's stack"

	self costume goToPreviousCardInStack! !

!Player methodsFor: 'scripts-standard' stamp: 'yo 3/16/2005 15:43'!
goToRightOf: aPlayer 
	"Place the object so that it lies directly to the right of the given object"

	| hisCostume aCostume |
	(aPlayer isNil or: [aPlayer == self]) ifTrue: [^self].
	(hisCostume := aPlayer costume) isInWorld ifFalse: [^self].
	aCostume := self costume.
	aCostume isWorldMorph ifTrue: [^ self].
	aCostume owner == hisCostume owner 
		ifFalse: [hisCostume owner addMorphFront: aCostume].
	aCostume 
		position: hisCostume bounds rightCenter - (0 @ (aCostume height // 2))! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/26/1998 15:56'!
goto: aPlayer
	self sendMessageToCostume: #goto: with: aPlayer! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/5/2004 19:03'!
grabPatchMorph

	World primaryHand attachMorph: costume renderedMorph.
! !

!Player methodsFor: 'misc' stamp: 'ka 3/25/2004 05:25'!
grabPlayerIn: aWorld
	"Invoked from a Viewer: rip my morph out of its container, wherever that may be, and place it in the hand, being careful to set things up so that if the subsequent drop is rejected, the morph will end up in a visible location on the screen"

	| aMorph newPosition |
	self costume == aWorld ifTrue: [^ self].
	ActiveHand releaseMouseFocus.
	(aMorph _ self costume) visible: true.
	newPosition _ ActiveHand position - (aMorph extent // 2).
	aMorph isInWorld
		ifTrue:
			[aMorph goHome.
			aMorph formerPosition: aMorph positionInWorld]
		ifFalse:
			[aMorph formerPosition: aWorld center].
	aMorph formerOwner: ActiveWorld.
	aMorph position: newPosition.

	ActiveHand targetOffset: aMorph position - ActiveHand position.
	ActiveHand addMorphBack: aMorph.! !

!Player methodsFor: 'misc' stamp: 'sw 7/19/2004 16:41'!
grabPlayerInActiveWorld
	"Invoked from a Viewer: rip my morph out of its container, wherever that may be, and place it in the hand, being careful to set things up so that if the subsequent drop is rejected, the morph will end up in a visible location on the screen"

	self grabPlayerIn: ActiveWorld! !

!Player methodsFor: 'misc' stamp: 'sw 1/29/2001 21:57'!
grabScriptorForSelector: itsSelector in: aWorld
	"Grab the scriptor for the given selector and place it in the hand"

	aWorld currentHand attachMorph: (self scriptEditorFor: itsSelector) ! !

!Player methodsFor: 'viewer' stamp: 'sw 5/2/2001 23:46'!
graphicForViewerTab
	"Answer the graphic to show in the tab of a Viewer looking at me"

	^ self costume renderedMorph! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/13/2004 00:57'!
greenComponentFrom: another

	costume renderedMorph greenComponentFrom: another costume renderedMorph.

! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/13/2004 00:18'!
greenComponentInto: another

	costume renderedMorph greenComponentInto: another costume renderedMorph.

! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/8/2000 12:21'!
handTheUserACopy
	"Called from the user-interface: hand the user a copy"

	^ costume currentHand attachMorph: self getNewClone costume! !

!Player methodsFor: 'costume' stamp: 'nk 9/4/2004 17:05'!
hasCostumeThatIsAWorld

	self costumesDo: [ :aCostume | (aCostume isWorldMorph) ifTrue: [^ true]].
	^ false! !

!Player methodsFor: 'costume' stamp: 'nk 9/4/2004 17:04'!
hasOnlySketchCostumes
	"Answer true if the only costumes assocaited with this Player are SketchMorph costumes"

	self costumesDo: [ :aCostume | aCostume isSketchMorph ifFalse: [^ false]].
	^ true! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 2/21/98 23:32'!
hasScriptInvoking: scriptName ofPlayer: aPlayer
	"Answer whether the receiver bears any script that invokes a script of the given name for  the given player"
	self allScriptEditors do:
		[:anEditor | (anEditor hasScriptInvoking: scriptName ofPlayer: aPlayer) ifTrue: [^ true]].
	^ false! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 1/16/1999 16:27'!
hasScriptReferencing: slotName ofPlayer: aPlayer
	"Answer whether the receiver bears any script that references a slot of the given name for  the given player"
	self allScriptEditors do:
		[:anEditor | (anEditor hasScriptReferencing: slotName ofPlayer: aPlayer) ifTrue: [^ true]].
	^ false! !

!Player methodsFor: 'slots-user' stamp: 'dgd 2/22/2003 13:43'!
hasUserDefinedScripts
	^self class scripts notEmpty! !

!Player methodsFor: 'viewer' stamp: 'dgd 2/22/2003 13:43'!
hasUserDefinedSlots
	^self class slotInfo notEmpty! !

!Player methodsFor: 'heading' stamp: 'jm 4/24/1998 22:30'!
headDown

	| radians |
	radians _ (self getHeadingUnrounded - 90.0) degreesToRadians.
	self setHeading:
		((radians cos @ radians sin abs) theta radiansToDegrees
			roundTo: 0.001) + 90.0.
! !

!Player methodsFor: 'heading' stamp: 'jm 4/24/1998 22:30'!
headLeft

	| radians |
	radians _ (self getHeadingUnrounded - 90.0) degreesToRadians.
	self setHeading:
		((radians cos abs negated @ radians sin) theta radiansToDegrees
			roundTo: 0.001) + 90.0.
! !

!Player methodsFor: 'heading' stamp: 'jm 4/24/1998 22:30'!
headRight

	| radians |
	radians _ (self getHeadingUnrounded - 90.0) degreesToRadians.
	self setHeading:
		((radians cos abs @ radians sin) theta radiansToDegrees
			roundTo: 0.001) + 90.0.
! !

!Player methodsFor: 'heading' stamp: 'jm 4/24/1998 22:30'!
headUp

	| radians |
	radians _ (self getHeadingUnrounded - 90.0) degreesToRadians.
	self setHeading:
		((radians cos @ radians sin abs negated) theta radiansToDegrees
			roundTo: 0.001) + 90.0.
! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/28/2000 10:57'!
hide
	"Make the object be hidden, as opposed to visible"

	self costume hide! !

!Player methodsFor: 'misc' stamp: 'nk 6/12/2004 10:01'!
impartSketchScripts
	"Let the user designate another object to which my scripts and code should be imparted"

	| aMorph |
	Sensor waitNoButton.
	aMorph _ ActiveWorld chooseClickTarget.
	aMorph ifNil: [^ self].
	(aMorph renderedMorph isSketchMorph) ifTrue:
		[aMorph acquirePlayerSimilarTo: self]! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 2/3/2002 23:09'!
include: anObject
	"Add the object to my content"

	^ self append: anObject! !

!Player methodsFor: 'scripts-standard' stamp: 'ar 4/10/2005 18:50'!
includeAtCursor: aPlayer 
	"Add aPlayer to the list of objects logically 'within' me, at my current cursor position. ."

	| aCostume |
	(aPlayer isNil or: [aPlayer == self]) ifTrue: [^self].
	(aPlayer isText or: [aPlayer isString]) 
		ifTrue: 
			[^ self costume class == TextFieldMorph 
				ifTrue: [self costume append: aPlayer]
				ifFalse: [self]].
	aCostume := self costume topRendererOrSelf.
	aPlayer costume goHome.	"assure it's in view"
	(aCostume isKindOf: PasteUpMorph) 
		ifTrue:
			[aCostume addMorph: aPlayer costume asElementNumber: self getCursor.
			aCostume invalidRect: aCostume bounds]
		ifFalse:
			[aCostume addMorphBack: aPlayer.
			self setCursor: aCostume submorphs size]! !

!Player methodsFor: 'scripts-vector' stamp: 'tk 8/18/2001 22:49'!
incr: aPlayer
	"Treating Players like vectors, add aPlayer to me"

	self setX: self getX + aPlayer asPoint x.
	self setY: self getY + aPlayer asPoint y.
! !

!Player methodsFor: 'misc' stamp: 'sw 7/28/2001 01:03'!
indicateLocationOnScreen
	"Give momentary feedback on screen until mouse button is clicked"

	| bds |
	bds _ self costume boundsInWorld.
	5 timesRepeat:
		[Display reverse: bds.
		(Delay forMilliseconds: 80) wait.
		Display reverse: bds.
		(Delay forMilliseconds: 200) wait.].
	costume changed! !

!Player methodsFor: 'viewer' stamp: 'yo 2/11/2005 15:48'!
infoFor: anElement inViewer: aViewer 
	"The user made a gesture asking for info/menu relating"

	| aMenu elementType aSelector |
	elementType := self elementTypeFor: anElement
				vocabulary: aViewer currentVocabulary.
	elementType = #systemSlot | (elementType == #userSlot) 
		ifTrue: [^self slotInfoButtonHitFor: anElement inViewer: aViewer].
	aMenu := MenuMorph new defaultTarget: self.
	aMenu defaultTarget: self.
	aSelector := anElement asSymbol.
	elementType == #userScript 
		ifTrue: 
			[aMenu 
				add: 'destroy "' translated , anElement , '"'
				selector: #removeScriptWithSelector:
				argument: aSelector.
			aMenu 
				add: 'rename  "' translated, anElement , '"'
				selector: #renameScript:
				argument: aSelector.
			aMenu 
				add: 'textual scripting pane' translated
				selector: #makeIsolatedCodePaneForSelector:
				argument: aSelector.
			aSelector numArgs > 0 
				ifTrue: 
					[aMenu 
						add: 'remove parameter' translated
						selector: #ceaseHavingAParameterFor:
						argument: aSelector]
				ifFalse: 
					[aMenu 
						add: 'add parameter' translated
						selector: #startHavingParameterFor:
						argument: aSelector.
					aMenu 
						add: 'button to fire this script' translated
						selector: #tearOffButtonToFireScriptForSelector:
						argument: aSelector].
			aMenu 
				add: 'edit balloon help' translated
				selector: #editDescriptionForSelector:
				argument: aSelector].
	aMenu 
		add: 'show categories....' translated
		target: aViewer
		selector: #showCategoriesFor:
		argument: aSelector.
	aMenu items isEmpty 
		ifTrue: 
			["Never 0 at the moment because of show categories addition"

			aMenu add: 'ok' translated action: nil].
	aMenu addTitle: anElement asString , ' (' , elementType translated , ')'.
	aMenu popUpInWorld: aViewer world! !

!Player methodsFor: 'viewer' stamp: 'sw 9/27/2001 17:41'!
initialTypeForSlotNamed: aName
	"Answer the initial type to be ascribed to the given instance variable"

	^ #Number! !

!Player methodsFor: 'slots-user' stamp: 'sw 9/25/2001 22:28'!
initialValueForSlotOfType: aType
	"Answer the default initial value to ascribe to a slot of the given type"

	^ (Vocabulary vocabularyForType: aType)
		initialValueForASlotFor: self! !

!Player methodsFor: 'copying' stamp: 'sw 9/15/1998 13:12'!
initializeCostumesFrom: aPlayer
	"Used to copy costume info over from an UnscriptedPlayer to a Player UniClass"
	costume _ aPlayer costume.
	costumes _ aPlayer costumes! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 8/17/1998 17:18'!
initiatePainting
	(self costume isKindOf: PasteUpMorph) ifTrue:
		[self costume makeNewDrawingWithin]! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/23/2000 16:14'!
insertCard
	"Insert a new card into the stack"

	self costume stackDo: [:aStack | aStack insertCard]! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 2/18/2003 02:57'!
insertCharacters: aString
	"Insert the given characters at my current cursor position"

	self costume renderedMorph insertCharacters: aString! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 2/18/2003 02:57'!
insertContentsOf: aPlayer
	"Insert the string contents of the given player at my given cursor position"

	self costume renderedMorph insertContentsOf: aPlayer! !

!Player methodsFor: 'customevents-scripts-kernel' stamp: 'nk 9/24/2003 17:36'!
instantiatedUserScriptsDo: aBlock
	"Evaluate aBlock on behalf of all the instantiated user scripts in the receiver"

	| aState aCostume |
	((aCostume _ self costume) notNil and: [(aState _ aCostume actorStateOrNil) notNil]) ifTrue:
		[aState instantiatedUserScriptsDictionary do: aBlock]! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 2/20/2001 02:28'!
isEmptyTileScript: aScriptName
	"Answer whether the script of the given name is an empty classic tile script.  Presently disused -- formerly it was all too easy to propagate many empty tile scripts but this difficulty has receded considerably with recent changes, so this has no senders other than from an unusual menu item, and will perhaps die soon"

	| aUserScript |
	Preferences universalTiles ifTrue: [^ false].
	aUserScript _ self class userScriptForPlayer: self selector: aScriptName.
	^ (aUserScript instantiatedScriptEditorForPlayer: self) isEmpty
! !

!Player methodsFor: 'scripts-kernel' stamp: 'dgd 9/1/2003 14:17'!
isExpendableScript: aScriptName
	^ (self isEmptyTileScript: aScriptName) and:
		[aScriptName beginsWith: 'script' translated]
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/14/2004 15:44'!
isGroup: aBoolean

	self setCostumeSlot: #isGroup: toValue: aBoolean.
! !

!Player methodsFor: 'viewer' stamp: 'ar 5/26/2001 16:14'!
isPlayerLike
	"Return true if the receiver is a player-like object"
	^true! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/14/2004 17:18'!
isPrototypeTurtlePlayer

	^ false.
"
	costume ifNil: [^ false].
	^ costume renderedMorph isMemberOf: KedamaTurtleMorph
"! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/24/2004 06:08'!
isSequentialStub

	^ false.
! !

!Player methodsFor: 'macpal' stamp: 'sw 3/20/2001 13:28'!
isUniversalTiles
	"Return true if I (my world) uses universal tiles.  This message can be called in places where the current World is not known, such as when writing out a project.  For information about the writingUniversalTiles thing, contact Ted Kaehler."

	^ costume world
		ifNil:
			[ScriptEditorMorph writingUniversalTiles == true  "only valid during a project write"]
		ifNotNil:
			[Preferences universalTiles]! !

!Player methodsFor: 'testing' stamp: 'sw 10/24/2000 07:28'!
knownName
	"Answer a name by which the receiver is known, or nil if none"

	^ costume knownName! !

!Player methodsFor: 'costume' stamp: 'nk 6/12/2004 10:01'!
knownSketchCostumeWithSameFormAs: aSketchMorph 
	| itsForm |
	itsForm := aSketchMorph form.
	^ costumes
		ifNotNil: [costumes
				detect: [:c | c isSketchMorph
						and: [c form == itsForm]]
				ifNone: []]! !

!Player methodsFor: 'translation' stamp: 'yo 1/18/2004 10:59'!
labelFromWatcher: w
	"Answer the morph holding the label of the given watcher"

	^ w owner owner submorphs third submorphs second! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/26/1998 15:56'!
lastPage
	self sendMessageToCostume: #lastPage! !

!Player methodsFor: 'pen' stamp: 'sw 8/3/1998 16:10'!
liftAllPens
	self costume liftAllPens! !

!Player methodsFor: 'pen' stamp: 'sw 4/13/1998 22:38'!
liftPen
	self actorState liftPen! !

!Player methodsFor: 'scripts-standard' stamp: 'jm 4/22/1999 15:53'!
loadSineWave

	self sendMessageToCostume: #loadSineWave.
! !

!Player methodsFor: 'scripts-standard' stamp: 'gk 2/23/2004 20:51'!
loadSound: soundName

	| snd |
	snd _ SoundService default soundNamed: soundName.
	snd ifNotNil: [self sendMessageToCostume: #loadSound: with: snd].
! !

!Player methodsFor: 'pen' stamp: 'sw 8/3/1998 16:10'!
lowerAllPens
	self costume lowerAllPens! !

!Player methodsFor: 'pen' stamp: 'sw 4/13/1998 22:38'!
lowerPen
	self actorState lowerPen! !

!Player methodsFor: 'misc' stamp: 'sw 1/10/2005 00:08'!
makeBounceSound: soundName
	"Having bounced off an edge, produce the given sound"

	Preferences soundsEnabled
		ifTrue: [self costume playSoundNamed: soundName]! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 2/5/2001 14:03'!
makeIsolatedCodePaneForSelector: aSelector
	"make an isolated code pane for the given selector"

	MethodHolder makeIsolatedCodePaneForClass: self class selector: aSelector
! !

!Player methodsFor: 'scripts-standard' stamp: 'dgd 2/22/2003 13:43'!
makeNewDrawingIn: paintPlacePlayer 
	| paintPlace |
	((paintPlacePlayer isNil 
		or: [((paintPlace := paintPlacePlayer costume) isKindOf: PasteUpMorph) not]) 
			or: [paintPlace isInWorld not]) 
			ifTrue: 
				[^self 
					inform: 'Error: not a plausible
place in which to make
a new drawing'].
	paintPlace makeNewDrawingWithin! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/22/1999 11:50'!
menuItemAfter: menuString
	^ self costume menuItemAfter: menuString! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/22/1999 11:55'!
menuItemBefore: menuString
	^ self costume menuItemBefore: menuString! !

!Player methodsFor: 'scripts-kernel' stamp: 'mir 7/12/2004 19:41'!
methodInterfaceForEmptyScript
	"Answer a MethodInterface representing Andreas's 'emptyScript' feature"

	| anInterface |
	anInterface _ MethodInterface new.
	anInterface receiverType: #Player.
	anInterface flagAttribute: #scripts.
	anInterface
		wording: (ScriptingSystem wordingForOperator: #emptyScript);
		helpMessage: 'an empty script; drop on desktop to get a new empty script for this object'.

	anInterface selector: #emptyScript type: nil setter: nil.
	^ anInterface! !

!Player methodsFor: 'slots-kernel' stamp: 'yo 8/1/2004 02:04'!
methodInterfacesForInstanceVariablesCategoryIn: aVocabulary
	"Return a collection of methodInterfaces for the instance-variables category.  The vocabulary parameter, at present anyway, is not used."

	| aList anInterface itsSlotName |
	aList _ OrderedCollection new.
	self slotInfo associationsDo:
		[:assoc |
			anInterface _ MethodInterface new.
			itsSlotName _ assoc key.
			anInterface
				wording: itsSlotName;
				helpMessage: 'a variable defined by this object' translated.

			anInterface selector: (Utilities getterSelectorFor: itsSlotName) type: assoc value type setter: (Utilities setterSelectorFor: itsSlotName).
			anInterface setToRefetch.
			aList add: anInterface].
	^ aList! !

!Player methodsFor: 'scripting' stamp: 'tk 8/13/2001 09:27'!
methodInterfacesForScriptsCategoryIn: aVocabulary
	"Answer a list of method interfaces for the category #scripts, as seen in a viewer or other tool.  The vocabulary argument is not presently used."

	| myScripts |
	myScripts _ self class scripts values collect: [:us |
		(us isKindOf: UserScript)
			ifTrue: [us as: MethodWithInterface]
			ifFalse: [us]].
	^ {self methodInterfaceForEmptyScript}, myScripts! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 7/29/2005 17:32'!
moveToward: aPlayer
	"Move a standard amount in the direction of the given player.  If the object has an instance variable named 'speed', the speed of the motion will be governed by that value"

	| myPosition itsPosition |
	((aPlayer ~~ self) and: [(self overlaps: aPlayer) not]) ifTrue:
		[((myPosition _ self costume referencePosition) = (itsPosition _ aPlayer costume referencePosition))
			ifFalse:
				[self setHeading: (myPosition bearingToPoint: itsPosition).
				self forward: self getSpeed]]! !

!Player methodsFor: 'scripts-vector' stamp: 'tk 8/18/2001 22:51'!
multBy: aNumber
	"Treating Players like vectors, scale myself by aNumber"

	self setX: self getX * aNumber asPoint x.
	self setY: self getY * aNumber asPoint y.
! !

!Player methodsFor: 'testing' stamp: 'sw 10/25/2000 23:16'!
nameForViewer
	"Answer the name to be used for the receiver in its Viewer"

	^ self getName! !

!Player methodsFor: 'costume' stamp: 'sw 10/24/1998 21:48'!
newCostume

	| aMenu reply |
	aMenu _ SelectionMenu selections: self availableCostumeNames.
	(reply _ aMenu startUpWithCaption: 'choose a costume') ifNil: [^ self].
	self wearCostumeOfName: reply.
	self updateAllViewers! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/13/2005 22:03'!
newPatch

	| f |
	f _ KedamaPatchMorph newExtent: self costume renderedMorph dimensions.
	f assuredPlayer assureUniClass.
	f setNameTo: (ActiveWorld unusedMorphNameLike: f innocuousName).
	self createSlotForPatch: f.
	self addToPatchDisplayList: f assuredPlayer.
	self costume world primaryHand attachMorph: f.
	^ f.
! !

!Player methodsFor: 'viewer' stamp: 'tk 8/3/2001 11:08'!
newScriptorAround: aPhrase
	"Sprout a scriptor around aPhrase, thus making a new script.  aPhrase may either be a PhraseTileMorph (classic tiles 1997-2001) or a SyntaxMorph (2001 onward)"

	| aScriptEditor aUniclassScript tw blk |
	aUniclassScript _ self class permanentUserScriptFor: self unusedScriptName player: self.
	aScriptEditor _ aUniclassScript instantiatedScriptEditorForPlayer: self.

	Preferences universalTiles ifTrue: [
		aScriptEditor install.
		"aScriptEditor hResizing: #shrinkWrap;
			vResizing: #shrinkWrap;
			cellPositioning: #topLeft;
			setProperty: #autoFitContents toValue: true."
		aScriptEditor insertUniversalTiles.  "Gets an empty SyntaxMorph for a MethodNode"
		tw _ aScriptEditor findA: TwoWayScrollPane.
		aPhrase ifNotNil:
			[blk _ (tw scroller findA: SyntaxMorph "MethodNode") findA: BlockNode.
			blk addMorphFront: aPhrase.
			aPhrase accept.
		].
		SyntaxMorph setSize: nil andMakeResizable: aScriptEditor.
	] ifFalse: [
		aPhrase 
				ifNotNil: [aScriptEditor phrase: aPhrase]	"does an install"
				ifNil: [aScriptEditor install]
	].
	self class allSubInstancesDo: [:anInst | anInst scriptInstantiationForSelector: aUniclassScript selector].
		"The above assures the presence of a ScriptInstantiation for the new selector in all siblings"
	self updateAllViewersAndForceToShow: #scripts.
	^ aScriptEditor! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 7/17/2002 16:41'!
newTextualScriptorFor: aSelector
	"Sprout a scriptor for aSelector, opening up in textual mode.  Rather special-purpose, consult my lone sender"

	| aMethodWithInterface aScriptEditor |
	(self class selectors includes: aSelector) ifTrue: [self error: 'selector already exists'].

	aMethodWithInterface _ self class permanentUserScriptFor: aSelector player: self.
	aScriptEditor _ aMethodWithInterface instantiatedScriptEditorForPlayer: self.
	aScriptEditor install.
	aScriptEditor showSourceInScriptor.
	aMethodWithInterface selector numArgs == 0 ifTrue:
		[self class allSubInstancesDo: [:anInst | anInst scriptInstantiationForSelector: aMethodWithInterface selector]].
		"The above assures the presence of a ScriptInstantiation for the new selector in all siblings"

	self updateAllViewersAndForceToShow: #scripts.
	^ aScriptEditor! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 8/18/2005 15:33'!
newTurtle

	| m |
	m _ KedamaTurtleMorph new openInWorld.
	self costume hasNoTurtleBreed ifTrue: [m color: Color red].
	self useTurtle: m player.
	m setNameTo: (ActiveWorld unusedMorphNameLike: m innocuousName).
	self costume world primaryHand attachMorph: m.
	^ m.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 8/10/2004 12:37'!
newTurtleSilently

	| m |
	m _ KedamaTurtleMorph new openInWorld.
	self useTurtle: m player.
	m turtleCount: 0.
	m setNameTo: (ActiveWorld unusedMorphNameLike: m innocuousName).
	^ m.
! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/26/1998 15:55'!
nextPage
	self sendMessageToCostume: #nextPage! !

!Player methodsFor: 'pen' stamp: 'tk 10/4/2001 18:14'!
noArrowheadsOnAllPens
	"Only for the Player of a Playfield"

	self costume noArrowheadsOnAllPens! !

!Player methodsFor: 'misc' stamp: 'sw 9/13/2002 17:53'!
noteDecimalPlaces: aNumber forGetter: aGetter
	"Note the given preference of decimal places for the given getter"

	costume noteDecimalPlaces: aNumber forGetter: aGetter! !

!Player methodsFor: 'customevents-misc' stamp: 'nk 9/24/2003 18:26'!
noteDeletionOf: aMorph fromWorld: aWorld
	"aMorph, while pointing to me as its costumee, has been deleted"
	"This may be too aggressive because deletion of a morph may not really mean deletion of its associated player -- in light of hoped-for multiple viewing"

	| viewers scriptors viewerFlaps |
	viewers _ OrderedCollection new.
	viewerFlaps _ OrderedCollection new.
	scriptors _ OrderedCollection new.
	aWorld allMorphs do:
		[:m | m isAViewer ifTrue: [viewers add: m].
			((m isKindOf: ViewerFlapTab) and: [m scriptedPlayer == self])
				ifTrue:
					[viewerFlaps add: m].
			((m isKindOf: ScriptEditorMorph) and: [m myMorph == aMorph])
				ifTrue: [scriptors add: m]].

	aMorph  removeAllEventTriggersFor: self.
	aWorld removeAllEventTriggersFor: self.
	viewers do: [:v |  v noteDeletionOf: aMorph].
	viewerFlaps do: [:v  | v dismissViaHalo].
	scriptors do: [:s | s privateDelete] ! !

!Player methodsFor: 'scripts-kernel' stamp: 'yo 1/2/2004 06:40'!
noteRenameOf: oldSlotName to: newSlotName inPlayer: aPlayer
	"Note that aPlayer has renamed a slot formerly known as oldSlotName to be newSlotName"

	self allScriptEditors do:
		[:anEditor | (anEditor showingMethodPane not and: [anEditor hasScriptReferencing: oldSlotName ofPlayer: aPlayer]) ifTrue: 
			[anEditor replaceReferencesToSlot: oldSlotName inPlayer: aPlayer with: newSlotName]]! !

!Player methodsFor: 'misc' stamp: 'sd 5/10/2008 17:50'!
offerAlternateViewerMenuFor: aViewer event: evt
	"Put up an alternate Viewer menu on behalf of the receiver."

	| aMenu aWorld  |
	aWorld _ aViewer world.
	aMenu _ MenuMorph new defaultTarget: self.
	costumes ifNotNil:
		[(costumes size > 1 or: [costumes size == 1 and: [costumes first ~~ costume renderedMorph]])
			ifTrue:
				[aMenu add: 'forget other costumes' translated target: self selector: #forgetOtherCostumes]].

	aMenu add: 'expunge empty scripts' translated target: self action: #expungeEmptyScripts.
	aMenu addLine.
	aMenu add: 'choose vocabulary...' translated target: aViewer action: #chooseVocabulary.
	aMenu balloonTextForLastItem: 'Choose a different vocabulary for this Viewer.' translated.
	aMenu add: 'choose limit class...' translated target: aViewer action: #chooseLimitClass.
	aMenu balloonTextForLastItem: 'Specify what the limitClass should be for this Viewer -- i.e., the most generic class whose methods and categories should be considered here.' translated.

	
	aMenu addLine.
	aMenu add: 'inspect morph' translated target: costume selector: #inspect.
	aMenu add: 'inspect player' translated target: self selector: #inspect.
	self belongsToUniClass ifTrue:
		[aMenu add: 'browse class' translated target: self action: #browsePlayerClass.
		aMenu add: 'inspect class' translated target: self class action: #inspect].
	aMenu add: 'inspect this Viewer' translated target: aViewer selector: #inspect.
	aMenu add: 'inspect this Vocabulary' translated target: aViewer currentVocabulary selector: #inspect.

	aMenu addLine.
	aMenu add: 'relaunch this Viewer' translated target: aViewer action: #relaunchViewer.
	aMenu add: 'attempt repairs' translated target: ActiveWorld action: #attemptCleanup.
	aMenu add: 'view morph directly' translated target: aViewer action: #viewMorphDirectly.
	aMenu balloonTextForLastItem: 'opens a Viewer directly on the rendered morph.' translated.
	(costume renderedMorph isSketchMorph) ifTrue:
		[aMenu addLine.
		aMenu add: 'impart scripts to...' translated target: self action: #impartSketchScripts].

	aMenu popUpEvent: evt in: aWorld! !

!Player methodsFor: 'slots-user' stamp: 'yo 2/12/2005 20:09'!
offerGetterTiles: slotName 
	"For a player-type slot, offer to build convenient compound tiles that otherwise would be hard to get"

	| typeChoices typeChosen thePlayerThereNow slotChoices slotChosen getterTiles aCategoryViewer playerGetter |
	typeChoices := Vocabulary typeChoices.
	typeChosen := (SelectionMenu labelList: (typeChoices collect: [:t | t translated]) lines: #() selections: typeChoices) 
				startUpWithCaption: ('Choose the TYPE
of data to get from
{1}''s {2}' translated format: {self externalName. slotName translated}).
	typeChosen isEmptyOrNil ifTrue: [^self].
	thePlayerThereNow := self perform: (Utilities getterSelectorFor: slotName).
	thePlayerThereNow 
		ifNil: [thePlayerThereNow := self presenter standardPlayer].
	slotChoices := thePlayerThereNow slotNamesOfType: typeChosen.
	slotChoices isEmpty 
		ifTrue: [^self inform: 'sorry -- no slots of that type' translated].
	slotChoices _ slotChoices asSortedArray.
	slotChosen := (SelectionMenu labelList: (slotChoices collect: [:t | t translated]) selections: slotChoices) 
				startUpWithCaption: ('Choose the datum
you want to extract from {1}''s {2}' translated format: {self externalName. slotName translated}).
	slotChosen isEmptyOrNil ifTrue: [^self].
	"Now we want to tear off tiles of the form
		holder's valueAtCursor's foo"
	getterTiles := nil.
	aCategoryViewer := CategoryViewer new initializeFor: thePlayerThereNow
				categoryChoice: 'basic'.
	getterTiles := aCategoryViewer 
				getterTilesFor: (Utilities getterSelectorFor: slotChosen)
				type: typeChosen.
	aCategoryViewer := CategoryViewer new initializeFor: self
				categoryChoice: 'basic'.
	playerGetter := aCategoryViewer 
				getterTilesFor: (Utilities getterSelectorFor: slotName)
				type: #Player.
	getterTiles submorphs first acceptDroppingMorph: playerGetter event: nil.	"the pad"	"simulate a drop"
	getterTiles makeAllTilesGreen.
	getterTiles justGrabbedFromViewer: false.
	(getterTiles firstSubmorph)
		changeTableLayout;
		hResizing: #shrinkWrap;
		vResizing: #spaceFill.
	ActiveHand attachMorph: getterTiles! !

!Player methodsFor: 'misc' stamp: 'dgd 4/4/2006 16:25'!
offerViewerMenuFor: aViewer event: evt
	"Put up the Viewer menu on behalf of the receiver.  If the shift key is held down, put up the alternate menu. The menu omits the 'add a new variable' item when in eToyFriendly mode, as per request from teachers using Squeakland in 2003 once the button for adding a new variable was added to the viewer"

	| aMenu aWorld  |
	(evt notNil and: [evt shiftPressed] and: [Preferences eToyFriendly not]) ifTrue:[
		^ self offerAlternateViewerMenuFor: aViewer event: evt
	].

	aWorld _ aViewer world.
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu title: self externalName.
	aMenu addStayUpItem.
	self costume renderedMorph offerCostumeViewerMenu: aMenu.

	Preferences eToyFriendly ifFalse: "exclude this from squeakland-like UI "
		[aMenu add: 'add a new variable' translated target: self action: #addInstanceVariable.
		aMenu balloonTextForLastItem: 'Add a new variable to this object and all of its siblings.  You will be asked to supply a name for it.' translated].

	aMenu add: 'add a new script' translated target: aViewer action: #newPermanentScript.
	aMenu balloonTextForLastItem: 'Add a new script that will work for this object and all of its siblings' translated.
	aMenu addLine.
	self hasCostumeThatIsAWorld ifFalse:[
	
	aMenu add: 'grab me' translated target: self selector: #grabPlayerIn: argument: aWorld.
	
	aMenu balloonTextForLastItem: 'This will actually pick up the object this Viewer is looking at, and hand it to you.  Click the (left) button to drop it' translated.
	].

	aMenu add: 'reveal me' translated target: self selector: #revealPlayerIn: argument: aWorld.
	aMenu balloonTextForLastItem: 'If you have misplaced the object that this Viewer is looking at, use this item to (try to) make it visible' translated.

	aMenu addLine.
	aMenu add: 'tile representing me' translated action: #tearOffTileForSelf.
	aMenu add: 'add search pane' translated target: aViewer action: #addSearchPane.
	Preferences eToyFriendly ifFalse:[
	
	aMenu addLine.
	
	aMenu add: 'more...' translated target: self selector: #offerAlternateViewerMenuFor:event: argumentList: {aViewer. evt}.
	].

	aMenu popUpEvent: evt in: aWorld
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 8/17/1998 17:13'!
okayToDestroyScriptNamed: scriptName
	self costume world presenter allExtantPlayers do:
		[:aPlayer | (aPlayer hasScriptInvoking: scriptName ofPlayer: self)
			ifTrue:
				[^ false]].
	^ true! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 8/17/1998 17:21'!
okayToRemoveSlotNamed: aSlotName
	self costume world presenter allExtantPlayers do:
		[:aPlayer | (aPlayer hasScriptReferencing: aSlotName ofPlayer: self)
			ifTrue:
				[^ false]].
	^ true! !

!Player methodsFor: 'misc' stamp: 'sw 1/19/2001 17:44'!
openUnderlyingScriptorFor: aSelector
	"Open the underlying scriptor for the given selector"

	self grabScriptorForSelector: aSelector in: self currentWorld! !

!Player methodsFor: 'misc' stamp: 'sw 10/27/2000 06:38'!
ordinalNumber
	"Answer a number indicating the relative position of the receiver in its stack, if any, else 1"

	| aStack |
	^ (aStack _ self stack) ifNotNil: [aStack cardIndexOf: self] ifNil: [nil]! !

!Player methodsFor: 'misc' stamp: 'tak 1/21/2005 11:59'!
overlaps: aPlayer 
	"Answer whether my costume overlaps that of another player"

	| goalCostume intersection myShadow goalShadow bb myRect goalRect |
	aPlayer ifNil: [^false].
	goalCostume := aPlayer costume.
	costume world == goalCostume world ifFalse: [^false].

	"check if the 2 player costumes intersect"
	intersection := costume bounds intersect: goalCostume bounds.
	(intersection width = 0 or: [intersection height = 0]) 
		ifTrue: [^false]
		ifFalse: 
			["check if the overlapping region is non-transparent"

			"compute 1-bit, black and white versions (stencils) of the intersecting  
			part of each morph's costume"

			myRect := intersection translateBy: 0 @ 0 - costume topLeft.
			myShadow := (costume imageForm contentsOfArea: myRect) stencil.
			goalRect := intersection translateBy: 0 @ 0 - goalCostume topLeft.
			goalShadow := (goalCostume imageForm contentsOfArea: goalRect) stencil.

			"compute a pixel-by-pixel AND of the two stencils.  Result will be black 
			(pixel value = 1) where black parts of the stencils overlap"
			bb := BitBlt toForm: myShadow.
			bb 
				copyForm: goalShadow
				to: 0 @ 0
				rule: Form and.

			"return TRUE if resulting form contains any black pixels"
			^(bb destForm tallyPixelValues second) > 0]! !

!Player methodsFor: 'scripts-standard' stamp: 'tak 1/21/2005 12:08'!
overlapsAny: aPlayer 
	"Answer true if my costume overlaps that of aPlayer, or any of its  
	siblings (if aPlayer is a scripted player)  
	or if my costume overlaps any morphs of the same class (if aPlayer is  
	unscripted)."
	| possibleCostumes itsCostume itsCostumeClass myShadow |
	(self ~= aPlayer
			and: [self overlaps: aPlayer])
		ifTrue: [^ true].
	possibleCostumes := IdentitySet new.
	aPlayer belongsToUniClass
		ifTrue: [aPlayer class
				allSubInstancesDo: [:anInstance | (anInstance ~~ aPlayer
							and: [itsCostume := anInstance costume.
								(itsCostume bounds intersects: costume bounds)
									and: [itsCostume world == costume world]])
						ifTrue: [possibleCostumes add: itsCostume]]]
		ifFalse: [itsCostumeClass := aPlayer costume class.
			self costume world presenter allExtantPlayers
				do: [:ep | ep costume
						ifNotNilDo: [:ea | (ea class == itsCostumeClass
									and: [ea bounds intersects: costume bounds])
								ifTrue: [possibleCostumes add: ea]]]].
	possibleCostumes isEmpty
		ifTrue: [^ false].
	myShadow := costume shadowForm.
	^ possibleCostumes
		anySatisfy: [:m | m overlapsShadowForm: myShadow bounds: costume fullBounds]! !

!Player methodsFor: 'customevents-scripts-kernel' stamp: 'nk 8/18/2004 17:40'!
pacifyScript: aSymbol
	"Make sure the script represented by the symbol doesn't do damage by lingering in related structures on the morph side"

	| aHandler aUserScript |
	aUserScript _ self class userScriptForPlayer: self selector: aSymbol.
	aUserScript ifNil: [self flag: #deferred.  ^ Beeper beep].  
	"Maddeningly, without this line here the thing IS nil and the debugger is in a bad state
	(the above note dates from 1/12/99 ?!!"

	self class allInstancesDo:
		[:aPlayer | | itsCostume |
		aPlayer actorState instantiatedUserScriptsDictionary removeKey: aSymbol ifAbsent: [].
		itsCostume _ aPlayer costume renderedMorph.
		(aHandler _ itsCostume eventHandler) ifNotNil:
			[aHandler forgetDispatchesTo: aSymbol].
		itsCostume removeEventTrigger: aSymbol ]! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/4/2001 07:23'!
pauseAll: scriptName
	"Change the status of my script of the given name to be #paused in me and all of my siblings."

	self assignStatus: #paused toAllFor: scriptName! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/28/2000 10:57'!
pauseScript: scriptName
	"Change the status of my script of the given name to be #paused"

	self changeScript: scriptName toStatus: #paused! !

!Player methodsFor: 'pen' stamp: 'sw 4/13/1998 22:38'!
penColor: aColor
	self actorState penColor: aColor! !

!Player methodsFor: 'scripts-standard' stamp: 'nk 8/21/2004 12:15'!
performScriptIfCan: scriptNameString 
	"If I understand the given script name, perform it now"
	^Symbol
		hasInterned: scriptNameString
		ifTrue: [:sym | (self class includesSelector: sym)
				ifTrue: [self triggerScript: sym]]! !

!Player methodsFor: 'playing commands' stamp: 'dgd 2/15/2004 21:29'!
play
	self sendMessageToCostume: #play! !

!Player methodsFor: 'playing commands' stamp: 'jm 10/12/2004 11:17'!
playUntilPosition: aNumber

	 self sendMessageToCostume: #playUntilPosition: with: aNumber! !

!Player methodsFor: 'scripts-execution' stamp: 'ar 2/12/2001 18:57'!
prepareToBeRunning
	self instantiatedUserScriptsDo:
		[:aScriptInstantiation | aScriptInstantiation prepareToBeRunning].! !

!Player methodsFor: 'scripts-standard' stamp: 'ar 4/10/2005 18:51'!
prepend: aPlayer 
	"Add aPlayer to the list of objects logically 'within' me.  This is visually represented by its morph becoming my costume's first submorph.   Also allow text to be prepended."

	| aCostume |
	(aPlayer isNil or: [aPlayer == self]) ifTrue: [^self].
	(aPlayer isText or: [aPlayer isString]) 
		ifTrue: 
			[^ self costume class == TextFieldMorph 
				ifTrue: [self costume prepend: aPlayer]
				ifFalse: [self]].
	(aCostume := self costume topRendererOrSelf) 
		addMorphFront: aPlayer costume.
	aPlayer costume goHome.	"assure it's in view"
	(aCostume isKindOf: PasteUpMorph) 
		ifTrue: [self setCursor: (aCostume submorphs indexOf: aPlayer costume)]! !

!Player methodsFor: 'accessing' stamp: 'sw 8/17/1998 17:21'!
presenter
	"Convenience for calling from user scripts"
	^ self costume presenter! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/26/1998 15:55'!
previousPage
	self sendMessageToCostume: #previousPage! !

!Player methodsFor: 'printing' stamp: 'sw 8/17/1998 17:14'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: (' (', self asOop printString, ')').
	self costume ifNil: [aStream nextPutAll: ' (with nil costume)'.  ^ self].
	aStream nextPutAll: ' named ', self externalName! !

!Player methodsFor: 'costume' stamp: 'sw 8/20/1998 17:35'!
rawCostume: aMorph
	costume _ aMorph! !

!Player methodsFor: 'accessing' stamp: 'yo 3/31/2005 15:00'!
readoutPlayer

	^ self.
! !

!Player methodsFor: 'costume' stamp: 'nk 6/12/2004 10:01'!
recaptureUniqueCostumes
	"Recapture all unique sketch-like costumes. Debugging only."
	| unique |
	costumes ifNil:[^self].
	unique := PluggableSet new 
				equalBlock:[:s1 :s2| s1 form == s2 form];
				hashBlock:[:s| s form identityHash].
	unique addAll: (costumes select:[:c| c isSketchMorph]).
	unique := unique asIdentitySet.
	costumes := costumes select:[:c|
		(c isSketchMorph) not or:[unique includes: c]].
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/13/2004 00:57'!
redComponentFrom: another

	costume renderedMorph redComponentFrom: another costume renderedMorph.

! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/13/2004 00:18'!
redComponentInto: another

	costume renderedMorph redComponentInto: another costume renderedMorph.

! !

!Player methodsFor: 'fileIn/Out' stamp: 'tk 8/6/1999 17:18'!
releaseCachedState
	"release all non-showing scriptors"

	self class userScriptsDo: [:userScript | userScript releaseCachedState].! !

!Player methodsFor: 'costume' stamp: 'nk 6/12/2004 10:02'!
rememberCostume: aCostume
	"Put aCostume in my remembered-costumes list, as the final element"
	| costumeToRemember existing |
	costumeToRemember _ aCostume renderedMorph.
		"Remember real morphs, not their transformations"
	costumes ifNil: [costumes _ OrderedCollection new].
	existing _ (costumeToRemember isSketchMorph)
		ifTrue:
			[self knownSketchCostumeWithSameFormAs: costumeToRemember]
		ifFalse:
			[costumes detect: [:c | c == costumeToRemember] ifNone: [nil]].
	costumes _ costumes copyWithout: existing.
	costumes addLast: costumeToRemember! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/8/2000 17:05'!
removeAll
	"Remove all the elements from my companion morph's collection.  This is destructive!!"

	costume ifNotNil: [costume removeAllMorphs]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/9/2004 15:16'!
removeAllFromPatchDisplayList

	costume renderedMorph removeAllFromPatchDisplayList.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/9/2004 15:19'!
removeAllFromTurtleDisplayList

	costume renderedMorph removeAllFromTurtleDisplayList.
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 9/8/2000 17:06'!
removeScript: aSymbol fromWorld: aWorld
	"Remove the given script, and get the display right in aWorld"

	self removeScriptWithoutUpdatingViewers: aSymbol fromWorld: aWorld.
	self updateAllViewersAndForceToShow: #scripts
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 2/5/2001 14:01'!
removeScriptWithSelector: aSelector
	"Remove the given script, and get the display right"

	self removeScript: aSelector fromWorld:  self currentWorld
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 7/16/1999 17:06'!
removeScriptWithoutUpdatingViewers: aSymbol
	self pacifyScript: aSymbol.
	self class removeScriptNamed: aSymbol.

	(self scriptorsForSelector: aSymbol inWorld: costume world) do:
		[:s | s privateDelete].
! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 8/15/2000 11:57'!
removeScriptWithoutUpdatingViewers: aSymbol fromWorld: aWorld
	self pacifyScript: aSymbol.
	self class removeScriptNamed: aSymbol.

	(self scriptorsForSelector: aSymbol inWorld: aWorld) do:
		[:s | s privateDelete].
! !

!Player methodsFor: 'slots-user' stamp: 'yo 2/11/2005 15:44'!
removeSlotNamed: aSlotName
	"The user has requested that an instance variable be removed..."

	| aSetter aGetter |
	(self okayToRemoveSlotNamed: aSlotName) ifFalse:
		[^ self inform: ('Sorry, {1} is in
use in a script.' translated format: {aSlotName})].

	aSetter _ Utilities setterSelectorFor: aSlotName.
	aGetter _ Utilities getterSelectorFor: aSlotName.
	((self systemNavigation allCallsOn: aSetter) size > 0 or: [(self systemNavigation allCallsOn: aGetter) size > 0]) ifTrue:
		[self inform: 
'Caution!!  There may be scripts belonging to
other objects that may rely on the presence of
this variable.  If there are, they may now be broken.
You may need to fix them up manually.' translated].

	self class removeInstVarName: aSlotName asString.

	self updateAllViewers! !

!Player methodsFor: 'testing' stamp: 'tk 7/28/2005 04:52'!
renameInternal: aName

	^ self costume topRendererOrSelf renameInternal: aName! !

!Player methodsFor: 'scripts-kernel' stamp: 'yo 2/11/2005 15:37'!
renameScript: oldSelector 
	"The user has asked to rename the script formerly known by oldSelector; obtain a new selector from the user, check it out, and if all is well, ascribe the new name as appropriate"

	| reply newSelector aUserScript |
	self flag: #deferred.
	"Relax the restriction below, before too long"
	aUserScript := self class userScriptForPlayer: self selector: oldSelector.
	aUserScript okayToRename 
		ifFalse: 
			[self 
				inform: 'Sorry, we do not permit you to rename
classic-tiled scripts that are currently
textually coded.  Go back to tile scripts
and try again.  Humble apologies.' translated.
			^self].
	reply := FillInTheBlank request: 'Script Name' translated initialAnswer: oldSelector.
	reply isEmpty ifTrue: [^self].
	reply = oldSelector ifTrue: [^Beeper beep].
	newSelector := self acceptableScriptNameFrom: reply
				forScriptCurrentlyNamed: oldSelector.
	Preferences universalTiles 
		ifTrue: 
			["allow colons"

			(reply copyWithout: $:) = newSelector 
				ifTrue: [newSelector := reply asSymbol]
				ifFalse: [self inform: 'name will be modified']].
	self renameScript: oldSelector newSelector: newSelector! !

!Player methodsFor: 'scripts-kernel' stamp: 'yo 5/6/2005 09:49'!
renameScript: oldSelector newSelector: newSelector
	"Rename the given script to have the new selector"

	|  aUserScript anInstantiation aDict |
	oldSelector = newSelector ifTrue: [^ self].

	oldSelector numArgs == 0
		ifTrue:
			[self class allSubInstancesDo:
				[:aPlayer | | itsCostume |
					anInstantiation _ aPlayer scriptInstantiationForSelector: oldSelector.
					anInstantiation ifNotNil: [
						newSelector numArgs == 0
							ifTrue:
								[anInstantiation changeSelectorTo: newSelector].
						aDict _ aPlayer costume actorState instantiatedUserScriptsDictionary.
						itsCostume _ aPlayer costume renderedMorph.
						itsCostume renameScriptActionsFor: aPlayer from: oldSelector to: newSelector.
						self currentWorld renameScriptActionsFor: aPlayer from: oldSelector to: newSelector.
						aDict removeKey: oldSelector.

						newSelector numArgs  == 0 ifTrue:
							[aDict at: newSelector put: anInstantiation.
							anInstantiation assureEventHandlerRepresentsStatus]]]]
		ifFalse:
			[newSelector numArgs == 0 ifTrue:
				[self class allSubInstancesDo:
					[:aPlayer |
						anInstantiation _ aPlayer scriptInstantiationForSelector: newSelector.
						anInstantiation ifNotNil: [anInstantiation assureEventHandlerRepresentsStatus]]]].

	aUserScript _ self class userScriptForPlayer: self selector: oldSelector.

	aUserScript renameScript: newSelector fromPlayer: self.
		"updates all script editors, and inserts the new script in my scripts directory"

	self class removeScriptNamed: oldSelector.
	((self existingScriptInstantiationForSelector: newSelector) notNil and:
		[newSelector numArgs > 0]) ifTrue: [self error: 'ouch'].
		
	self updateAllViewersAndForceToShow: 'scripts'! !

!Player methodsFor: 'slots-user' stamp: 'yo 7/2/2004 19:36'!
renameSlot: oldSlotName 
	| reply newSlotName |
	reply := FillInTheBlank request: 'New name for "' translated , oldSlotName , '":'
				initialAnswer: oldSlotName.
	reply isEmpty ifTrue: [^self].
	newSlotName := ScriptingSystem 
				acceptableSlotNameFrom: reply
				forSlotCurrentlyNamed: oldSlotName
				asSlotNameIn: self
				world: self costume currentWorld.
	self renameSlot: oldSlotName newSlotName: newSlotName! !

!Player methodsFor: 'slots-user' stamp: 'sw 3/8/2004 22:14'!
renameSlot: oldSlotName newSlotName: newSlotName
	"Give an existing instance variable a new name"

	self class renameSilentlyInstVar: oldSlotName to: newSlotName.
	self renameSlotInWatchersOld: oldSlotName new: newSlotName.

	self updateAllViewers.

	self presenter allExtantPlayers do:
		[:aPlayer | (aPlayer hasScriptReferencing: oldSlotName ofPlayer: self)
			ifTrue:
				[aPlayer noteRenameOf: oldSlotName to: newSlotName inPlayer: self]].

	self presenter hasAnyTextuallyCodedScripts
		ifTrue:
			[self inform: 
'Caution!!  References in texutally coded scripts won''t be renamed.'].

	^ true! !

!Player methodsFor: 'translation' stamp: 'sw 1/6/2005 16:34'!
renameSlotInWatchersOld: oldName new: newName
	"A variable has been renamed; get all relevant extant watchers updated.  All this assumed to be happening in the ActiveWorld"

	| wasStepping oldGetter |
	oldGetter _ Utilities getterSelectorFor: oldName.
	self allPossibleWatchersFromWorld do: [:aWatcher |
		(aWatcher getSelector = oldGetter) ifTrue:
			[(wasStepping _ aWatcher isStepping) ifTrue: [aWatcher stopStepping].
			aWatcher getSelector: (Utilities getterSelectorFor: newName).
			aWatcher putSelector ifNotNil:
				[aWatcher putSelector: (Utilities setterSelectorFor: newName)].
			((aWatcher isKindOf: UpdatingStringMorph) and: [aWatcher hasStructureOfComplexWatcher]) ifTrue:  "Old style fancy watcher"
				[aWatcher owner owner traverseRowTranslateSlotOld: oldName to: newName.
				(aWatcher target labelFromWatcher: aWatcher) contents: newName, ' = '].
			(aWatcher ownerThatIsA: WatcherWrapper) ifNotNilDo:
				[:wrapper | wrapper player: self variableName: newName].
			wasStepping ifTrue: [aWatcher startStepping]]]! !

!Player methodsFor: 'testing' stamp: 'tk 7/28/2005 04:55'!
renameTo: aName

	^ self costume topRendererOrSelf renameTo: aName! !

!Player methodsFor: 'costume' stamp: 'mir 6/13/2001 15:28'!
renderedCostume: aMorph
	"Make aMorph be the receiver's rendered costume; if flexing is currently in effect, make the new morph be flexed correspondingly"
	self renderedCostume: aMorph remember: true! !

!Player methodsFor: 'costume' stamp: 'sw 12/12/2001 14:13'!
renderedCostume: aMorph remember: rememberCostume
	"Make aMorph be the receiver's rendered costume; if flexing is currently in effect, make the new morph be flexed correspondingly"

	| renderedMorph known anEventHandler w baseGraphic |
	renderedMorph _ costume renderedMorph.
	renderedMorph == aMorph ifTrue: [^ self].
	baseGraphic _ costume renderedMorph valueOfProperty: #baseGraphic.
	rememberCostume
		ifTrue: [self rememberCostume: renderedMorph].
	renderedMorph changed.
	w _ renderedMorph world.
	"Copy 'player state' (e.g., state which should be associated with the player but is stored in the morph itself these days) from the old rendered morph the new morph."
	aMorph rotationStyle: renderedMorph rotationStyle.
	aMorph forwardDirection: renderedMorph forwardDirection.
	"Note: referencePosition is *not* state but #moveTo: behavior"
	aMorph referencePosition: renderedMorph referencePosition.
	anEventHandler _ renderedMorph eventHandler.
	costume isFlexMorph
		ifTrue:
			[costume adjustAfter:
				[costume replaceSubmorph: renderedMorph by: aMorph]]
		ifFalse:
			[costume owner ifNotNil: [costume owner replaceSubmorph: costume by: aMorph].
			aMorph player: self.
			aMorph actorState: costume actorState.
			(known _ costume knownName) ifNotNil:
				[aMorph setNameTo: known].
			costume _ aMorph.
			w ifNotNil:
				[w stopStepping: renderedMorph.
				w startStepping: aMorph]].

	baseGraphic ifNotNil: [self setBaseGraphic: baseGraphic].
	aMorph eventHandler: anEventHandler.
	aMorph changed! !

!Player methodsFor: 'costume' stamp: 'sw 8/3/2000 10:16'!
resetCostumeList
	costumes _ nil! !

!Player methodsFor: 'costume' stamp: 'nk 6/12/2004 10:02'!
restoreBaseGraphic
	"Restore my base graphic"

	| cos |
	((cos _ self costume renderedMorph) isSketchMorph)
		ifTrue:
			[cos restoreBaseGraphic]! !

!Player methodsFor: 'misc' stamp: 'sw 7/28/2001 01:05'!
revealPlayerIn: aWorld
	"Reveal the receiver if at all possible in the world; once it's visible, flash its image for a bit, and leave it with its halo showing"

	| aMorph |
	(aMorph _ self costume) isInWorld ifTrue:
		[aMorph goHome.
		self indicateLocationOnScreen.
		aMorph addHalo.
		^ self].

	"It's hidden somewhere; search for it"
	aWorld submorphs do:
		[:m | (m succeededInRevealing: self) ifTrue:  "will have obtained halo already"
			[aWorld doOneCycle.
			self indicateLocationOnScreen.
			^ self]].

	"The morph is truly unreachable in this world at present.  So extract it from hyperspace, and place it at center of screen, wearing a halo."
	aMorph isWorldMorph ifFalse:
		[aWorld addMorphFront: aMorph.
		aMorph position: aWorld bounds center.
		aMorph addHalo]
	
	! !

!Player methodsFor: 'misc' stamp: 'tak 1/18/2005 13:12'!
revealPlayerNamed: aSymbol in: aWorld 
	| getter |
	getter := Utilities getterSelectorFor: aSymbol.
	^ (self perform: getter)
		revealPlayerIn: aWorld! !

!Player methodsFor: 'scripts-standard' stamp: 'jm 4/22/1999 15:52'!
reverse

	self sendMessageToCostume: #reverse.
! !

!Player methodsFor: 'misc' stamp: 'dgd 2/22/2003 13:45'!
revertToUnscriptedPlayerIfAppropriate
	| anInstance |
	(self class selectors notEmpty or: [self class instVarNames notEmpty]) 
		ifTrue: [^self].
	anInstance := UnscriptedPlayer new.
	anInstance initializeCostumesFrom: self.
	self become: anInstance! !

!Player methodsFor: 'playing commands' stamp: 'dgd 2/15/2004 21:32'!
rewind
	self sendMessageToCostume: #rewind! !

!Player methodsFor: 'misc' stamp: 'sw 1/1/1999 22:28'!
roundUpStrays
	self sendMessageToCostume: #roundUpStrays! !

!Player methodsFor: 'scripts-execution' stamp: 'sw 7/3/1998 20:22'!
runAllClosingScripts
	"Run all the receiver's scripts marked as #closing.  Return a boolean indicating whether any such scripts were encountered and run"

	| any |
	any _ false.
	self instantiatedUserScriptsDo:
		[:aScriptInstantiation | aScriptInstantiation runIfClosing ifTrue: [any _ true]].
	^ any! !

!Player methodsFor: 'scripts-execution' stamp: 'sw 7/3/1998 20:21'!
runAllOpeningScripts
	"Run all the receiver's scripts marked as #opening.  Return a boolean indicating whether any such scripts were encountered and run"

	| any |
	any _ false.
	self instantiatedUserScriptsDo:
		[:aScriptInstantiation | aScriptInstantiation runIfOpening ifTrue: [any _ true]].
	^ any! !

!Player methodsFor: 'scripts-execution' stamp: 'ar 2/12/2001 18:04'!
runAllTickingScripts: nowTick

	self instantiatedUserScriptsDo: [:aScriptInstantiation | aScriptInstantiation runIfTicking: nowTick]! !

!Player methodsFor: 'scripts-kernel' stamp: 'yo 7/27/2004 15:59'!
runScript: aSelector
	"Called from script-activation buttons.  Provides a safe way to run a script that may have changed its name"
	(self respondsTo: aSelector) ifTrue:
		[^ self triggerScript: aSelector].
	self inform: 
'Oops, object "', self externalName, '" no longer has
a script named "', aSelector, '".
It must have been deleted or renamed.'! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 2/17/2001 01:05'!
scriptEditorFor: aSelector
	"Answer the receiver's script editor for aSelector"

	| aScriptEditor |
	aScriptEditor _ (self class userScriptForPlayer: self selector: aSelector) instantiatedScriptEditorForPlayer: self.
	aScriptEditor updateToPlayer: self.
	aScriptEditor bringUpToDate.
	^ aScriptEditor! !

!Player methodsFor: 'scripts-kernel' stamp: 'yo 5/6/2005 09:47'!
scriptInstantiationForSelector: aSelector
	"Answer a script instantiation for the given selector, creating it at this time if necessary"

	|  entry scriptDict classEntry actorState |
	actorState _ self actorState.
	actorState ifNil: [^ nil].
	scriptDict _ actorState instantiatedUserScriptsDictionary.
	entry _ scriptDict at: aSelector ifAbsent: [nil].
	entry ifNil:
		[classEntry _ self class userScriptForPlayer: self selector: aSelector.
		entry _ ScriptInstantiation new player: self selector: aSelector status: classEntry defaultStatus.
		scriptDict at: aSelector put: entry].
	^ entry! !

!Player methodsFor: 'scripts-execution' stamp: 'sw 12/1/1999 13:12'!
scriptingError: aMessage
	"An error arose, characterized by aMessage, when a script was being run.  For the moment, we report it to the transcript only"
	Transcript cr; show: 'Scripting error for ', self externalName, ': ', aMessage! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 1/19/2001 15:15'!
scriptorsForSelector: aSelector inWorld: aWorld
	"Answer, for the purpose of deletion, a list of all scriptor objects for the given selector that are associated with any member of the receiver's uniclass"

	| scriptors |
	aWorld ifNil: [^ OrderedCollection new].
	scriptors _ (aWorld allMorphs select:
		[:m | (((m isKindOf: ScriptEditorMorph) and: [m playerScripted class == self class]) and: [m scriptName == aSelector])] thenCollect: [:m | m topEditor]) asSet.
	^ scriptors asArray! !

!Player methodsFor: 'misc' stamp: 'sw 10/26/1998 15:45'!
seesColor: aColor
	costume isInWorld ifFalse: [^ false].
	^ costume touchesColor: aColor! !

!Player methodsFor: 'costume' stamp: 'sw 10/26/1998 15:55'!
sendMessageToCostume: aSelector
	| aCostume |
	(aCostume _ self costumeRespondingTo: aSelector) ifNotNil:
		[^ aCostume perform: aSelector].
	^ nil! !

!Player methodsFor: 'costume' stamp: 'sw 10/26/1998 15:54'!
sendMessageToCostume: aSelector with: arg
	| aCostume |
	(aCostume _ self costumeRespondingTo: aSelector) ifNotNil:
		[^ aCostume perform: aSelector with: arg].
	^ nil! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/25/2001 03:38'!
setActWhen: val
	"Tell the receiver's costume (hopefully a button!!) to set its actWhen parameter as indicated"

	costume renderedMorph actWhen: val! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/14/2000 11:29'!
setAllButFirstCharacter: aString
	"Set my costume's all-but-first characters to be aString"

	costume renderedMorph setAllButFirstCharacter: aString! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:25'!
setAllowEtoyUserCustomEvents: aValue
	"Set the allowEtoyUserCustomEvents preferences"

	Preferences setPreference: #allowEtoyUserCustomEvents toValue: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 7/19/2005 01:49'!
setAlpha: alpha
	"Set the alpha of the color of my costume."

	| adjusted |
	adjusted _ (alpha max: 0.0) min: 1.0.
	^ self setColor: (self getColor alpha: adjusted)! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 10/3/2004 01:32'!
setBackgroundColor: aColor
	"Set the background color; the costume is presumed to be a text morph."

	self costume renderedMorph backgroundColor: aColor! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 6/12/2004 10:02'!
setBaseGraphic: aGraphic
	"Set the base graphic"

	| aMorph |
	^ ((aMorph _ costume renderedMorph) isSketchMorph)
		ifTrue:
			[aMorph baseGraphic: aGraphic]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:26'!
setBatchPenTrails: aValue
	"Set the batchPenTrails preference."

	Preferences setPreference: #batchPenTrails toValue: aValue! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/12/2004 22:55'!
setBlueComponentIn: aPatch to: value
! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 4/13/2004 18:55'!
setBorderColor: aColor 
	"Set the border color as requested"

	costume renderedMorph borderColor: aColor! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/26/2001 16:20'!
setBorderStyle: aSymbol
	"Set the border style of my costume"

	costume renderedMorph setBorderStyle: aSymbol! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 4/13/2004 18:55'!
setBorderWidth: aWidth 
	"Set the border width to the given number"
	costume renderedMorph borderWidth: aWidth! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/16/2005 15:32'!
setBottom: w
	"Set the bottom coordinate (cartesian sense) of the object as requested"

	| topLeftNow cost |
	cost _ self costume.
	cost isWorldMorph ifTrue: [^ self].
	topLeftNow _ cost cartesianBoundsTopLeft.
	^ cost bottom: cost top + topLeftNow y - w
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 16:35'!
setBottomEdgeMode: aSymbol

	self setCostumeSlot: #bottomEdgeMode: toValue: aSymbol.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 12/2/2000 15:58'!
setCameraPoint: pt

	self costume cameraPoint: pt
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/4/2002 19:32'!
setCardNumber: aNumber
	"Go to the given card number"

	self stackEmbodied goToCardNumber: aNumber! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:34'!
setCellInset: aValue
	"Setter for costume's cellInset"

	costume cellInset: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 2/17/2003 17:35'!
setCharacterAtCursor: aCharOrString
	"Insert the given character at my cursor position"

	| aLoc aTextMorph aString charToUse |
	aLoc _ (aTextMorph _ self costume renderedMorph) cursor.
	charToUse _ (aString _ aCharOrString asString) size > 0
		ifTrue:
			[aString first]
		ifFalse:
			[''].
	aTextMorph paragraph replaceFrom: aLoc to: aLoc with: charToUse asString asText displaying: true.
	aTextMorph updateFromParagraph  ! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/14/2000 11:28'!
setCharacters: amt
	"Set my costume's characters as indicated"

	costume renderedMorph setCharacters: amt! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 10:23'!
setClipSubmorphs: aBoolean
	"Setter for costume's clipSubmorphs"

	costume renderedMorph clipSubmorphs: aBoolean.
	costume renderedMorph changed! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 12/10/2001 00:27'!
setColor: aColor
	"Set the color of the graphic as requested"

	| aFillStyle aMorph |
	(aFillStyle _ (aMorph _ self costume renderedMorph) fillStyle) isGradientFill
		ifTrue:
			[aFillStyle firstColor: aColor forMorph: aMorph hand: nil]
		ifFalse:
			[aMorph color: aColor]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 5/4/2001 06:06'!
setColorUnder: aValue
	"Provide a soft landing for old readouts that may try to send this"! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 6/12/2004 10:02'!
setCostume: aForm
	"Set the receiver's graphic as indicated.  An earlier wording, disused but may persist in preexisting scripts."

	| aMorph |
	^ ((aMorph _ costume renderedMorph) isSketchMorph)
		ifTrue:
			[aMorph form: aForm]
		ifFalse:
			["what to do?"]! !

!Player methodsFor: 'costume' stamp: 'sw 8/17/1998 15:45'!
setCostumeSlot: setterSelector toValue: aValue
	| aCostume |
	(aCostume _ self costumeRespondingTo: setterSelector) ifNotNil:
		[aCostume perform: setterSelector with: aValue]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/8/2000 11:16'!
setCursor: aNumber
	"Set my costume's cursor to the given number"

	costume renderedMorph cursorWrapped: aNumber! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/7/2000 09:29'!
setCursorWrapped: aNumber
	"maintained for backward compatibility with existing etoys"

	^ self setCursor: aNumber
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/10/2000 13:52'!
setDescending: aBoolean
	self setCostumeSlot: #descending: toValue: aBoolean! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 5/30/2004 17:09'!
setDiffusionRate: aNumber

	^ self setCostumeSlot: #diffusionRate: toValue: aNumber.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/15/2004 05:11'!
setDisplayScaleMax: aNumber

	self setCostumeSlot: #displayMax: toValue: aNumber.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/4/2004 17:44'!
setDisplayShiftAmount: aNumber

	self setCostumeSlot: #displayShiftAmount: toValue: aNumber.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 3/7/2005 12:04'!
setDisplayType: aSymbol

	^ self setCostumeSlot: #displayType: toValue: aSymbol.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/16/2005 15:35'!
setDistance: aDistance
	"Set the object's distance from the origin to be as indicated, preserving its angle."

	| cost |
	cost _ self costume.
	cost isWorldMorph ifTrue: [^ self].
	cost cartesianXY: (Point r: aDistance degrees:  self getTheta)! !

!Player methodsFor: 'pen' stamp: 'sw 4/17/2003 11:56'!
setDotSize: aNumber
	"Set the trail dot size as indicated, but confine matters to a reasonable range"

	self costume renderedMorph setProperty: #trailDotSize toValue: ((aNumber max: 1) min: 100)! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:32'!
setDragEnabled: aValue
	"Setter for costume's dragEnabled"

	costume dragEnabled: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:34'!
setDropEnabled: aValue
	"Setter for costume's dropEnabled"

	costume dropEnabled: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 7/10/2005 04:22'!
setDropProducesWatcher: aValue
	"Set the dropProducesWatcher preferences"

	Preferences setPreference: #dropProducesWatcher toValue: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/15/2005 18:23'!
setDropShadow: aValue
	"Setter for costume's dropShadow"

	| aMorph |
	(aMorph _ costume renderedMorph) hasDropShadow ~~ aValue ifTrue: [aMorph toggleDropShadow]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:27'!
setEToyFriendly: aValue
	"Set the eToyFriendly preference."

	Preferences setPreference: #eToyFriendly toValue: aValue! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 5/30/2004 17:23'!
setEvaporationRate: aNumber

	self setCostumeSlot: #evaporationRate: toValue: aNumber.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:28'!
setFenceEnabled: aValue
	"Set the fenceEnabled preference."

	Preferences setPreference: #fenceEnabled toValue: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/14/2000 11:29'!
setFirstCharacter: aChar
	"Set my costume's first character to the indicated one"

	costume renderedMorph setFirstCharacter: aChar! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 2/22/2003 13:45'!
setFirstElement: aPlayer 
	"Caution - this is a replacement operation!!  Replace the receiver's costume's first element with the morph represented by aPlayer"

	| aCostume |
	(aPlayer == self or: [(aCostume := self costume) submorphs isEmpty]) 
		ifTrue: [^self].
	costume replaceSubmorph: aCostume submorphs first by: aPlayer costume! !

!Player methodsFor: 'slots-user' stamp: 'sw 7/4/2004 00:26'!
setFloatPrecisionFor: aReadout
	"If appropriate, set the floatPrecision for the given watcher readout (an UpdatingStringMorph), whose getter is assumed already to be established."
	
	| precision  |
	(precision _ self defaultFloatPrecisionFor: aReadout getSelector) ~= 1 ifTrue: [aReadout floatPrecision: precision]! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:11'!
setFogColor: x
	self costume renderedMorph fogColor: x! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:23'!
setFogDensity: x
	self costume renderedMorph fogDensity: x * 0.01! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:23'!
setFogRangeEnd: x
	self costume renderedMorph fogRangeEnd: x * 0.01! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:23'!
setFogRangeStart: x
	self costume renderedMorph fogRangeStart: x * 0.01! !

!Player methodsFor: 'slots-wonderland' stamp: 'ar 5/15/2001 14:12'!
setFogType: x
	self costume renderedMorph fogType: x! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 10/30/2000 11:18'!
setGetListSelector: sel
	"Set the receiver's get-list-selector as indicated."

	costume renderedMorph getListSelector: sel
	! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 6/12/2004 10:02'!
setGraphic: aForm
	"Set the receiver's graphic as indicated"

	| aMorph |
	^ ((aMorph _ costume renderedMorph) isSketchMorph)
		ifTrue:
			[aMorph form: aForm]
		ifFalse:
			[aMorph isPlayfieldLike
				ifTrue: 
					[aMorph backgroundForm: aForm]
				ifFalse:
					["what to do?"]]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/12/2004 22:55'!
setGreenComponentIn: aPatch to: value
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/14/2004 15:45'!
setGrouped: aBoolean

	self setCostumeSlot: #isGroup: toValue: aBoolean.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 09:55'!
setHResizing: aValue
	"Setter for costume's hResizing"

	costume hResizing: aValue asSymbol! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/16/2005 15:34'!
setHeading: newHeading
	"Set the heading as indicated"

	| aCostume |
	aCostume _ self costume.
	aCostume isWorldMorph ifTrue: [^ self].
	(newHeading closeTo: aCostume heading) ifTrue: [^ self].
	aCostume heading: newHeading.
	aCostume _ self costume. "in case we just got flexed for no apparent reason"
	(aCostume isFlexMorph and:[aCostume hasNoScaleOrRotation]) 
		ifTrue:	[aCostume removeFlexShell]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/17/2002 10:17'!
setHeadingTheta: anAngle
	"Set the heading theta"

	self setHeading: (450 - anAngle)! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/28/2000 11:12'!
setHeight: w
	"Set the height of the object as indicated"

	^ self costume height: w! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/8/2000 11:18'!
setIndexInOwner: amt
	"Move my costume to a different z-position within its container.  This is primarily in service of auto-line-layout views in which the z-ordering determines the left/right and top/bottom placement."

	costume setIndexInOwner: amt! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 12/10/2001 00:40'!
setIsLocked: aBoolean
	"Set my costume's isLocked"

	costume lock: aBoolean! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 01:29'!
setKeepTickingWhilePainting: aValue
	"Set the keepTickingWhilePainting preferences"

	Preferences setPreference: #keepTickingWhilePainting toValue: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/7/2000 17:36'!
setKnobColor: aColor
	self setCostumeSlot: #sliderColor: toValue: aColor! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 10/30/2000 08:33'!
setLabel: aLab
	"Set the receiver's costume's label as specified"

	costume renderedMorph label: aLab! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/10/2004 00:54'!
setLastCharacter: aChar
	"Set my costume's last character to the indicated value, usually a string of length one."

	costume renderedMorph setLastCharacter: aChar! !

!Player methodsFor: 'slot getters/setters' stamp: 'nk 10/13/2004 11:30'!
setLastKeystroke: aString
	"Set the last keystroke fielded"

	self setCostumeSlot: #lastKeystroke: toValue: aString! !

!Player methodsFor: 'slot getters/setters' stamp: 'jm 4/21/1999 11:46'!
setLastValue: aNumber

	self setCostumeSlot: #lastValue: toValue: aNumber.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:34'!
setLayoutInset: aValue
	"Setter for costume's layoutInset"

	costume layoutInset: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/16/2005 15:32'!
setLeft: w
	"Set the object's left coordinate as indicated"

	| topLeftNow cost |
	cost _ self costume.
	cost isWorldMorph ifTrue: [^ self].
	topLeftNow _ cost cartesianBoundsTopLeft.
	^ cost left: cost left - topLeftNow x + w
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 16:35'!
setLeftEdgeMode: aSymbol

	self setCostumeSlot: #leftEdgeMode: toValue: aSymbol.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 10/23/2004 14:44'!
setLength: aLength
	"Set the length of the receiver."

	| cost lengthToUse |

	self hasCostumeThatIsAWorld ifTrue:[^ self].

	((cost _ self costume) isLineMorph)
		ifTrue:
			[^ cost unrotatedLength: aLength].
	lengthToUse _ cost isRenderer
		ifTrue:
			[aLength / cost scaleFactor]
		ifFalse:
			[aLength].
	cost renderedMorph height: lengthToUse! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:33'!
setListCentering: val
	"Setter for costume's listCentering"

	costume listCentering: val! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:33'!
setListDirection: aValue
	"Setter for costume's listDirection"

	costume listDirection: aValue asSymbol! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/6/2000 17:14'!
setMaxVal: aNumber
	self setCostumeSlot: #setMaxVal: toValue: aNumber! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/6/2000 17:16'!
setMinVal: aNumber
	self setCostumeSlot: #setMinVal: toValue: aNumber! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/17/1998 17:13'!
setName: aName
	^ self costume renameTo: aName! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/1/2000 10:37'!
setNumberAtCursor: aNumber
	"Place the given number into the morph residing at my costume's current cursor position"

	| renderedMorph aCostume |
	aCostume _ self costume.
	((renderedMorph _ aCostume renderedMorph) respondsTo: #valueAtCursor:) ifTrue: [^ renderedMorph valueAtCursor setNumericValue: aNumber]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 9/15/2000 06:12'!
setNumericValue: amt
	"Set the receiver's numeric value to the amount.  This is passed on to the costume"

	costume renderedMorph setNumericValue: amt! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 11/22/2000 08:44'!
setOffsetX: aNumber

	^ self costume offsetX: aNumber! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 11/22/2000 08:56'!
setOffsetY: aNumber

	^ self costume offsetY: aNumber! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/3/2005 13:54'!
setOliveHandleForScriptedObjects: aBoolean
	"Set the value of the oliveHandleForScriptedObjects preference."

	Preferences setPreference: #oliveHandleForScriptedObjects toValue: aBoolean! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/7/2002 13:34'!
setPageNumber: aNumber
	"Set the page number of my book as indicated."

	self bookEmbodied goToPage: aNumber! !

!Player methodsFor: 'costume' stamp: 'sw 7/22/2002 17:44'!
setParameterFor: aSelector toType: aTypeSymbol
	"Set the parameter type for the given selector"

	| aUniclassScript |
	aTypeSymbol isEmptyOrNil ifTrue: [^ self].
	(self typeforParameterFor: aSelector) = aTypeSymbol ifTrue: [^ self].
	aUniclassScript _ self class scripts at: aSelector.
	aUniclassScript argumentVariables first variableType: aTypeSymbol.
	aUniclassScript currentScriptEditorDo:
		[:aScriptEditor | aScriptEditor assureParameterTilesValid].
	self updateAllViewersAndForceToShow: #scripts
	

! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/7/2004 18:37'!
setPatchValueIn: aPatch to: value
! !

!Player methodsFor: 'pen' stamp: 'tk 10/4/2001 16:48'!
setPenArrowheads: penDown
	"Set whether the pen will draw arrowheads on the ends of strokes"

	self actorState setPenArrowheads: penDown.
! !

!Player methodsFor: 'pen' stamp: 'sw 11/28/2000 09:03'!
setPenColor: aColor
	"Set the turtle pen color as indicated"

	self actorState setPenColor: aColor! !

!Player methodsFor: 'pen' stamp: 'sw 11/28/2000 09:03'!
setPenDown: penDown
	"Set the penDown state as indicated, to true or false"

	| morph trailMorph tfm |
	self actorState setPenDown: penDown.
	((morph _ self costume) notNil and: [(trailMorph _ morph trailMorph) notNil])
		ifTrue:
		[tfm _ morph owner transformFrom: trailMorph.
		trailMorph notePenDown: penDown forPlayer: self
					at: (tfm localPointToGlobal: morph referencePosition)]
! !

!Player methodsFor: 'pen' stamp: 'sw 11/28/2000 09:03'!
setPenSize: aSize
	"Set the pen size as indicated"

	self actorState setPenSize: aSize! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/11/2004 01:17'!
setPixelsPerPatch: aNumber

	^ self setCostumeSlot: #pixelsPerPatch: toValue: aNumber.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 2/15/2004 21:11'!
setPosition: amt 
	"Set the receiver's numeric value to the amount. This is passed 
	on to the costume"
	costume renderedMorph setPosition: amt! !

!Player methodsFor: 'slots-user' stamp: 'yo 8/1/2004 19:46'!
setPrecisionFor: slotName 
	"Set the precision for the given slot name"

	| aList aMenu reply val aGetter places |
	aGetter := Utilities getterSelectorFor: slotName.
	places := Utilities 
				decimalPlacesForFloatPrecision: (self defaultFloatPrecisionFor: aGetter).
	aList := #('0' '1' '2' '3' '4' '5' '6').
	aMenu := SelectionMenu labels: aList
				selections: (aList collect: [:m | m asNumber]).
	reply := aMenu 
				startUpWithCaption: ('How many decimal places? (currently {1})' translated
						format: {places}).
	reply ifNotNil: 
			[(self slotInfo includesKey: slotName) 
				ifTrue: 
					["it's a user slot"

					(self slotInfoAt: slotName) 
						floatPrecision: (Utilities floatPrecisionForDecimalPlaces: reply).
					self class allInstancesDo: 
							[:anInst | 
							reply == 0 
								ifFalse: 
									[((val := anInst instVarNamed: slotName asString) isInteger) 
										ifTrue: [anInst instVarNamed: slotName asString put: val asFloat]].
							anInst updateAllViewers]]
				ifFalse: 
					["it's specifying a preference for precision on a system-defined numeric slot"

					self noteDecimalPlaces: reply forGetter: aGetter.
					self updateAllViewers]]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 08:51'!
setRadialGradientFill: aBoolean
	"Setter for costume's radialGradientFill"

	| aStyle |
	(aStyle _ costume renderedMorph fillStyle) isGradientFill
		ifTrue:
			[aStyle isRadialFill ~~ aBoolean ifTrue:
				[aStyle radial: aBoolean.
				costume renderedMorph changed]]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 10/12/2004 22:54'!
setRedComponentIn: aPatch to: value
! !

!Player methodsFor: 'playing commands' stamp: 'dgd 2/15/2004 22:07'!
setRepeat: aBoolean 
	 self sendMessageToCostume: #setRepeat: with: aBoolean! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/15/2001 16:34'!
setResistsRemoval: aBoolean
	"Set the resistsRemoval property"

	^ costume resistsRemoval: aBoolean! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/16/2005 15:32'!
setRight: w
	"Set the right coordinate to the given value"

	| topLeftNow cost |
	cost _ self costume.
	cost isWorldMorph ifTrue: [^ self].
	topLeftNow _ cost cartesianBoundsTopLeft.
	^ cost right: cost left - topLeftNow x + w
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 16:35'!
setRightEdgeMode: aSymbol

	self setCostumeSlot: #rightEdgeMode: toValue: aSymbol.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 7/14/2004 21:35'!
setRotationStyle: aStyleSymbol
	"Set the rotation style to the indicated symbol; the external symbols seen are different, as you'll observe..."

	costume renderedMorph rotationStyle: 
		(#(normal none leftRight upDown ) at:
		(#(rotate #'do not rotate' #'flip left right' #'flip up down') indexOf: aStyleSymbol))! !

!Player methodsFor: 'card/stack commands' stamp: 'sw 10/30/2000 08:34'!
setRoundedCorners: aBoolean
	"Set the rounded-corners attribute as indicated"

	costume renderedMorph cornerStyle: (aBoolean ifTrue: [#rounded] ifFalse: [#square])! !

!Player methodsFor: 'slot getters/setters' stamp: 'jm 4/22/1999 15:46'!
setSampleAtCursor: aNumber
	"Note: Performance hacked to allow real-time sound. Assumes costume is a GraphMorph."

	self setCostumeSlot: #valueAtCursor: toValue: aNumber.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'RAA 11/22/2000 08:35'!
setScale: aNumber

	^ self costume scale: aNumber! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/16/2005 15:33'!
setScaleFactor: aNumber
	"Set the scale factor to be the given value"

	| cost |
	cost _ self costume.
	cost isWorldMorph ifTrue: [^ self].
	cost scaleFactor: ((aNumber asFloat max: 0.1) min: 10.0)! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 12/9/2001 23:55'!
setSecondColor: aColor
	"Setter for costume's second color, if it's using gradient fill; if not, does nothing"

	| aFillStyle aMorph |
	^ (aFillStyle _ (aMorph _ costume renderedMorph) fillStyle) isGradientFill
		ifTrue:
			[aFillStyle lastColor: aColor forMorph: aMorph hand: ActiveHand]! !

!Player methodsFor: 'slot getters/setters' stamp: 'tak 3/15/2005 11:17'!
setShadowColor: aValue
	"Setter for costume's shadowColor"

	costume  renderedMorph shadowColor: aValue! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 5/30/2004 16:57'!
setSniffRange: aNumber

	self setCostumeSlot: #sniffRange: toValue: aNumber.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:33'!
setSticky: val
	"Setter for costume's sticky"

	costume sticky: val! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 3/8/2004 21:40'!
setSubtitlesFileName: aString 
	"Set my costume's subtitlesFileName as indicated"
	costume renderedMorph setSubtitlesFileName: aString! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/16/2005 15:35'!
setTheta: aTheta
	"Set the object's position such that its rho is unchanged but the angle between the positive x-axis and the vector connecting the origin and the object's position is as given."

	| cost |
	cost _ self costume.
	cost isWorldMorph ifTrue: [^ self].
	cost cartesianXY: (Point r: self getDistance degrees: aTheta)! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/16/2005 15:32'!
setTop: w
	"Set the top coordinate as indicated, using cartesian sense"

	| topLeftNow cost |
	cost _ self costume.
	cost isWorldMorph ifTrue: [^ self].
	topLeftNow _ cost cartesianBoundsTopLeft.
	^ cost top: cost top + topLeftNow y - w! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/17/2004 16:35'!
setTopEdgeMode: aSymbol

	self setCostumeSlot: #topEdgeMode: toValue: aSymbol.
! !

!Player methodsFor: 'pen' stamp: 'sw 3/11/2003 11:23'!
setTrailStyle: aTrailStyle
	"Set the trail style"

	self actorState trailStyle: aTrailStyle
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 3/12/2000 11:52'!
setTruncate: aBoolean
	self setCostumeSlot: #truncate: toValue: aBoolean! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/4/2004 01:19'!
setTurtleCount: aNumber

	self setCostumeSlot: #turtleCount: toValue: aNumber.
! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 3/7/2005 07:34'!
setTurtleVisible: aBoolean
! !

!Player methodsFor: 'slot getters/setters' stamp: 'yo 3/14/2005 13:45'!
setUseGradientFill: aBoolean
	"Setter for costume's useGradientFill"

	costume renderedMorph fillStyle isGradientFill
		ifTrue:
			[aBoolean ifFalse: [costume renderedMorph useSolidFill]]
		ifFalse:
			[aBoolean ifTrue: [costume renderedMorph useGradientFill]]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 6/15/2004 05:15'!
setUseLogDisplay: aBoolean

	^ self setCostumeSlot: #useLogDisplay: toValue: aBoolean.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 7/10/2005 03:41'!
setUseVectorVocabulary: aValue
	"Set the useVectorVocabulary preferences"

	Preferences setPreference: #useVectorVocabulary toValue: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 09:55'!
setVResizing: aValue
	"Setter for costume's vResizing"

	costume vResizing: aValue asSymbol! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 8/17/1998 17:15'!
setValueAtCursor: aPlayer
	| renderedMorph aCostume |
	aCostume _ self costume.
	((renderedMorph _ aCostume renderedMorph) respondsTo: #valueAtCursor:) ifTrue: [^ renderedMorph valueAtCursor: aPlayer costume].

	(aCostume respondsTo: #valueAtCursor:) ifTrue: [aCostume valueAtCursor: aPlayer costume]! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 3/8/2004 18:24'!
setVideoFileName: aString 
	"Set my costume's videoFileName as indicated"
	costume renderedMorph setVideoFileName: aString! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 10/30/2000 08:35'!
setViewingByIcon: aVal
	"Set the user's  costume's view-by-icon attribute as indictated"

	(aVal == false)
		ifTrue:
			["problematical - we always need *some* view"
			costume renderedMorph viewByName]
		ifFalse:
			[costume renderedMorph viewByIcon]! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 2/15/2004 21:15'!
setVolume: amt 
	"Set the receiver's numeric value to the amount. This is passed 
	on to the costume"
	costume renderedMorph setVolume: amt! !

!Player methodsFor: 'slot getters/setters' stamp: 'dgd 4/4/2006 16:32'!
setWidth: aWidth 
	"Set the width"
	
	| cost widthToUse |

	self hasCostumeThatIsAWorld
		ifTrue: [^ self].

	(cost := self costume) isLineMorph
		ifTrue: [^ cost unrotatedWidth: aWidth].

	widthToUse := cost isRenderer
				ifTrue: [aWidth / cost scaleFactor]
				ifFalse: [aWidth].

	cost renderedMorph width: widthToUse.
! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/16/2001 07:32'!
setWrapDirection: aValue
	"Setter for costume's wrapDirection"

	costume wrapDirection: aValue! !

!Player methodsFor: 'slot getters/setters' stamp: 'di 9/12/2001 20:49'!
setX: val
	"Set the x coordinate as indicated"

	| aCostume |
	(aCostume _ self costume) isInWorld ifFalse: [^ self].
	aCostume isWorldOrHandMorph ifTrue: [^ self].
	aCostume owner isHandMorph ifTrue: [^ self].
	^ aCostume x: val! !

!Player methodsFor: 'slot getters/setters' stamp: 'di 9/12/2001 20:51'!
setY: val
	"Set the y coordinate as indicated"

	| aCostume |
	(aCostume _ self costume) isInWorld ifFalse: [^ self].
	aCostume isWorldOrHandMorph ifTrue: [^ self].
	aCostume owner isHandMorph ifTrue: [^ self].
	^ aCostume y: val! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/28/2000 10:56'!
show
	"Make the object be visible, as opposed to hidden"

	self costume show! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/8/2000 18:39'!
shuffleContents
	"Tell my costume to rearrange its submorphs randomly"

	costume shuffleSubmorphs! !

!Player methodsFor: 'macpal' stamp: 'sw 4/8/98 11:56'!
slotInfo
	^ self class slotInfo! !

!Player methodsFor: 'slots-user' stamp: 'md 3/1/2006 09:16'!
slotInfoAt: slotName 
	| info |
	info := self slotInfo at: slotName ifAbsent: [nil].
	info ifNil: [self slotInfo at: slotName put: (info := SlotInformation new)].
	(info isSymbol) ifTrue: ["bkward compat"
			self slotInfo at: slotName put: (info := SlotInformation new type: info)].
	^info! !

!Player methodsFor: 'slots-user' stamp: 'sw 7/26/2001 12:01'!
slotInfoAt: slotName ifAbsent: aBlock
	"If the receiver has a slot of the given name, answer its slot info, else answer nil"

	| info |
	info _ self slotInfo at: slotName ifAbsent: [^ aBlock value].
	^ info! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 8/3/2005 14:48'!
slotInfoButtonHitFor: aGetterSymbol inViewer: aViewer
	"The user made a gesture asking for slot menu for the given getter symbol in a viewer; put up the menu."

	| aMenu slotSym aType typeVocab interface selector |

	 (#(+ - * /) includes: aGetterSymbol)
		ifTrue:
			 [^ self inform: aGetterSymbol, ' is used for vector operations'].

	slotSym _ Utilities inherentSelectorForGetter: aGetterSymbol.
	aType _ self typeForSlotWithGetter: aGetterSymbol asSymbol.
	aMenu _ MenuMorph new defaultTarget: self.
	interface := aViewer currentVocabulary methodInterfaceAt: aGetterSymbol ifAbsent: [nil].
	selector := interface isNil
		ifTrue: [slotSym asString]
		ifFalse: [interface selector].
	aMenu addTitle: (selector, ' (', (aType asString translated), ')').

	aType = #Patch ifTrue: [
		aMenu add: 'grab morph' translated
			target: (self perform: aGetterSymbol)
			selector: #grabPatchMorph
			argument: #().
			aMenu addLine.
	].

	(typeVocab _ Vocabulary vocabularyForType: aType) addWatcherItemsToMenu: aMenu forGetter: aGetterSymbol.

	(self slotInfo includesKey: slotSym)
		ifTrue:
			[aMenu add: 'change value type' translated selector: #chooseSlotTypeFor: argument: aGetterSymbol.
			typeVocab addUserSlotItemsTo: aMenu slotSymbol: slotSym.
			aMenu add: ('remove "{1}"' translated format: {slotSym}) selector: #removeSlotNamed: argument: slotSym.
			aMenu add: ('rename "{1}"' translated format: {slotSym}) selector: #renameSlot: argument: slotSym.			aMenu addLine].

	typeVocab addExtraItemsToMenu: aMenu forSlotSymbol: slotSym.  "e.g. Player type adds hand-me-tiles"

	aMenu add: 'show categories....' translated target: aViewer selector: #showCategoriesFor: argument: aGetterSymbol.
	self addIdiosyncraticMenuItemsTo: aMenu forSlotSymol: slotSym.

	aMenu items isEmpty ifTrue:
		[aMenu add: 'ok' translated action: #yourself].

	aMenu popUpForHand: aViewer primaryHand in: aViewer world! !

!Player methodsFor: 'slots-user' stamp: 'sw 5/16/2001 13:01'!
slotInfoForGetter: aGetter
	"Answer a SlotInformation object which describes an instance variable of mine retrieved via the given getter, or nil if none"

	^ self slotInfo at: (Utilities inherentSelectorForGetter: aGetter) ifAbsent: [nil]! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 2/15/98 13:08'!
slotNames
	^ self class instVarNames  "Could also get it from the slotInfo dictionary"
			! !

!Player methodsFor: 'slots-user' stamp: 'sw 5/16/2001 18:29'!
slotNamesOfType: aType
	"Answer a list of potential slot names of the given type in the receiver"

	| fullList forViewer gettersToOffer |
	fullList _ (ScriptingSystem systemSlotNamesOfType: aType),
		(self class slotGettersOfType: aType).
	forViewer _ costume renderedMorph selectorsForViewer select:
		[:aSel | aSel beginsWith: 'get'].
	gettersToOffer _ fullList select: [:anItem | forViewer includes: anItem].
	^ gettersToOffer collect:
		[:aSel | Utilities inherentSelectorForGetter: aSel]! !

!Player methodsFor: 'scripts-kernel' stamp: 'sw 11/5/1998 10:33'!
sourceCodeFor: sel
	^ self class sourceCodeAt: sel ifAbsent: 
		[Player sourceCodeAt: sel ifAbsent: ['this space for rent']]
	! !

!Player methodsFor: 'misc' stamp: 'sw 10/27/2000 06:37'!
stack
	"Answer the stack to which the receiver belongs.  This only searches via the costume's parent pointer, so there is no guarantee that the stack that is found actually contains the receiver in its card list"

	^ costume ifNotNil: [costume stack]! !

!Player methodsFor: 'slot getters/setters' stamp: 'sw 11/4/2002 19:31'!
stackEmbodied
	"Answer the stack embodied by the receiver's costume; usually this is directly the receiver's costume, but in case it is not, we look up the owner chain for one.  This allows card-number messages to be sent to a *page* of the stack, as Alan is wont to do, and have them still find their way to the right place"

	| aMorph |
	^ ((aMorph _ self costume renderedMorph) isKindOf: StackMorph)
		ifTrue:
			[aMorph]
		ifFalse:
			[aMorph ownerThatIsA: StackMorph]! !

!Player methodsFor: 'costume' stamp: 'RAA 5/18/2001 12:41'!
stamp

	"stamp an image of ourself onto the pen trails form"

	costume pasteUpMorph addImageToPenTrailsFor: costume topRendererOrSelf! !

!Player methodsFor: 'costume' stamp: 'RAA 5/18/2001 09:21'!
stampAndErase

	self stamp.
	self erase.! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/4/2001 06:59'!
startAll: scriptName
	"Change the status of my script of the given name to be #ticking in me and all of my siblings."

	self assignStatus: #ticking toAllFor: scriptName! !

!Player methodsFor: 'costume' stamp: 'sw 9/30/2004 04:49'!
startHavingParameterFor: aSelector
	"Start having a parameter for the given selector.  After this change, the script name will change by the addition of a colon."

	| newSelector |
	self renameScript: aSelector newSelector: (newSelector _ (aSelector, ':') asSymbol).
	(self scriptEditorFor: newSelector) install! !

!Player methodsFor: 'scripts-execution' stamp: 'sw 8/21/1998 16:34'!
startRunning
	self costume arrangeToStartStepping.
	self instantiatedUserScriptsDo:
		[:aScript | aScript startRunningIfPaused]! !

!Player methodsFor: 'scripts-execution' stamp: 'sw 8/21/1998 16:34'!
startRunningScripts
	self startRunning.
	self costume arrangeToStartStepping.  "emergency patch, if not going already"
	self costume presenter startRunningScripts! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/28/2000 10:56'!
startScript: scriptName
	"Change the status of my script of the given name to be #ticking"

	self changeScript: scriptName toStatus: #ticking! !

!Player methodsFor: 'menus' stamp: 'ar 2/12/2001 18:50'!
step
	"obsolete"
	^self stepAt: Time millisecondClockValue.! !

!Player methodsFor: 'scripts-execution' stamp: 'ar 2/12/2001 18:04'!
stepAt: nowTick
	self runAllTickingScripts: nowTick! !

!Player methodsFor: 'playing commands' stamp: 'dgd 2/15/2004 21:31'!
stop
	self sendMessageToCostume: #stop! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 9/4/2001 07:24'!
stopAll: scriptName
	"Change the status of my script of the given name to be #normal in me and all of my siblings."

		self assignStatus: #normal toAllFor: scriptName! !

!Player methodsFor: 'scripts-execution' stamp: 'jm 6/16/1999 14:37'!
stopProgramatically
	"stop running my ticking scripts -- called from running code"
	self instantiatedUserScriptsDo:
		[:aUserScript | aUserScript stopTicking].
	(costume renderedMorph isKindOf: SpeakerMorph)
		ifTrue: [costume renderedMorph stopSound].  "turn off buffered speaker sound"
! !

!Player methodsFor: 'scripts-execution' stamp: 'sw 2/12/98 14:51'!
stopRunning
	self stopProgramatically! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/28/2000 10:56'!
stopScript: scriptName
	"Change the status of my script of the given name to be #normal"

	self changeScript: scriptName toStatus: #normal! !

!Player methodsFor: 'objects from disk' stamp: 'tk 8/6/1999 17:14'!
storeDataOn: aDataStream
	"Discard all non-showing script editors"

	self releaseCachedState.
	super storeDataOn: aDataStream.
! !

!Player methodsFor: 'printing' stamp: 'sw 1/6/1999 17:44'!
stringForReadout
	^ self externalName! !

!Player methodsFor: 'costume' stamp: 'sw 3/11/2003 00:32'!
tearOffButtonToFireScriptForSelector: aSelector
	"Tear off a button to fire the script for the given selector"

	| aButton props |
	Preferences useButtonProprtiesToFire ifFalse:
		[aButton _ ScriptActivationButton new.
		aButton initializeForPlayer: self uniclassScript:  (self class scripts at: aSelector).
		^ aButton openInHand].

	(aButton _ RectangleMorph new) useRoundedCorners; color: Color yellow.
	props _ aButton ensuredButtonProperties.
	props
		target: self;
		actionSelector: #runScript:;
		arguments: {aSelector};
		delayBetweenFirings: 80;
		actWhen: #mouseUp;
		mouseDownHaloWidth: 8;
		wantsRolloverIndicator: true;
		mouseOverHaloWidth: 5;
		establishEtoyLabelWording.
	aButton width: aButton submorphs first width + 20; height: 20.
	self currentHand attachMorph: aButton.
! !

!Player methodsFor: 'slots-user' stamp: 'tak 4/6/2005 14:20'!
tearOffFancyWatcherFor: aGetter
	"Hand the user a labeled readout for viewing a numeric value"

	(WatcherWrapper new fancyForPlayer: self getter: aGetter) openInHand! !

!Player methodsFor: 'misc' stamp: 'gm 2/22/2003 14:54'!
tearOffTileForSelf
	| tiles |
	self currentHand attachMorph: (tiles := self tileReferringToSelf).
	(tiles isSyntaxMorph) 
		ifTrue: 
			[Preferences tileTranslucentDrag 
				ifTrue: [tiles lookTranslucent]
				ifFalse: 
					[tiles align: tiles topLeft
						with: self currentHand position + tiles cursorBaseOffset]]! !

!Player methodsFor: 'slots-user' stamp: 'tak 4/6/2005 11:40'!
tearOffUnlabeledWatcherFor: aGetter 
	"Hand the user anUnlabeled readout for viewing a numeric value"
	(WatcherWrapper new unlabeledForPlayer: self getter: aGetter) openInHand! !

!Player methodsFor: 'slots-user' stamp: 'sw 8/12/2004 02:27'!
tearOffWatcherFor: aSlotGetter
	"Tear off a simple textual watcher for the slot whose getter is provided"

	| aWatcher anInterface info isNumeric |

	info _ self slotInfoForGetter: aSlotGetter.
	info
		ifNotNil:
			[isNumeric _ info type == #Number]
		ifNil:
			[anInterface _ Vocabulary eToyVocabulary methodInterfaceAt: aSlotGetter ifAbsent: [nil].
			isNumeric _ anInterface notNil and: [anInterface resultType == #Number]].
	aWatcher _ UpdatingStringMorph new.
	
	aWatcher
		growable: true;
		getSelector: aSlotGetter;
		putSelector: (info notNil
			ifTrue:
				[ScriptingSystem setterSelectorForGetter: aSlotGetter]
			ifFalse:
				[anInterface companionSetterSelector]);
		setNameTo: (info notNil
			ifTrue:
				[Utilities inherentSelectorForGetter: aSlotGetter]
			ifFalse:
				[anInterface wording]);
 		target: self.
	isNumeric
		ifFalse:
			[aWatcher useStringFormat]
		ifTrue:
			[self setFloatPrecisionFor: aWatcher].
	aWatcher
		step;
		fitContents;
		openInHand! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 2/20/2003 13:14'!
tellAllContents: aMessageSelector
	"Send the given message selector to all the content players within the receiver's morph"

	costume renderedMorph tellAllContents: aMessageSelector! !

!Player methodsFor: 'scripts-standard' stamp: 'nk 8/21/2004 12:39'!
tellAllSiblings: aMessageSelector
	"Send the given message selector to all my sibling instances, but not to myself"

	Symbol hasInterned: aMessageSelector
		ifTrue: [ :sel |
	self belongsToUniClass
		ifTrue: [self class allSubInstancesDo:
				[:anInstance | anInstance ~~ self ifTrue: [ anInstance triggerScript: sel ]]]
		ifFalse:
			[(sel ~~ #emptyScript) ifTrue:
				[ScriptingSystem reportToUser: ('Cannot "tell" ', aMessageSelector, ' to ', self externalName) ]]]! !

!Player methodsFor: 'scripts-standard' stamp: 'nk 8/21/2004 12:42'!
tellSelfAndAllSiblings: aMessageSelector
	"Send the given message selector to all my sibling instances, including myself"

	Symbol hasInterned: aMessageSelector
		ifTrue: [ :sel |
	self belongsToUniClass
		ifTrue: [self class allSubInstancesDo:
				[:anInstance | anInstance triggerScript: sel ]]
		ifFalse:
			[(sel ~~ #emptyScript) ifTrue:
				[ScriptingSystem reportToUser: ('Cannot "tell" ', aMessageSelector, ' to ', self externalName) ]]]! !

!Player methodsFor: 'misc' stamp: 'sw 11/6/2000 12:25'!
thumbnailMenuEvt: anEvent forMorph: aMorph
	"The mouse went down in the thumbnail of a Viewer for the receiver"

	^ self offerViewerMenuForEvt: anEvent morph: aMorph! !

!Player methodsFor: 'misc' stamp: 'yo 8/15/2005 18:08'!
tileReferringToSelf
	"answer a tile that refers to the receiver"

	| aTile  nn tile |
	Preferences universalTiles ifTrue:
		[nn _ self externalName. 	"name it, if necessary, and put in References"
		(References includesKey: nn asSymbol) ifFalse: [
			 References at: nn asSymbol put: self].
		tile _ SyntaxMorph new parseNode: 
			(VariableNode new name: nn key: nn code: nil).
		tile layoutInset: 1; addMorph: (tile addString: nn special: false).
		tile color: (SyntaxMorph translateColor: #variable).
		tile extent: tile firstSubmorph extent + (2@2).
		^ tile].

	aTile _ TileMorph new setToReferTo: self.
	aTile updateWordingToMatchVocabulary.
	^ aTile! !

!Player methodsFor: 'viewer' stamp: 'sw 5/4/2001 05:19'!
tileToRefer
	"Answer a reference tile that comprises an alias to me.  Forgive this temporary and seemingly gratuituous revectoring as worlds collide"

	^ self tileReferringToSelf! !

!Player methodsFor: 'scripts-kernel' stamp: 'mir 7/12/2004 19:36'!
tilesToCall: aMethodInterface
	"Answer a phrase for the non-typed command represented by aMethodInterface."

	| resultType cmd argType argTile selfTile aPhrase balloonTextSelector aDocString universal |
	self class namedTileScriptSelectors.

	resultType _ aMethodInterface resultType.
	cmd _ aMethodInterface selector.
	(universal _ self isUniversalTiles)
		ifTrue:
			[aPhrase _ self universalTilesForInterface: aMethodInterface]
		ifFalse: [cmd numArgs == 0
			ifTrue:
				[aPhrase _ PhraseTileMorph new setOperator: cmd
					type: resultType
					rcvrType: #Player]
			ifFalse:
				["only one arg supported in classic tiles, so if this is fed
				with a selector with > 1 arg, results will be very strange"
				argType _ aMethodInterface typeForArgumentNumber: 1.
				aPhrase _ PhraseTileMorph new setOperator: cmd
					type: resultType
					rcvrType: #Player
					argType: argType.
				argTile _ ScriptingSystem tileForArgType: argType.
				argTile position: aPhrase lastSubmorph position.
				aPhrase lastSubmorph addMorph: argTile]].

	(self slotInfo includesKey: cmd)
		ifTrue: [balloonTextSelector _ #userSlot].

	(self belongsToUniClass and: [self class includesSelector: cmd])
		ifTrue:
			[aDocString _ (self class userScriptForPlayer: self selector: cmd) documentation.
			aDocString
				ifNotNil: [aPhrase submorphs second setBalloonText: aDocString]
				ifNil: [balloonTextSelector _ #userScript]].

	(universal ifTrue: [aPhrase submorphs second] ifFalse: [aPhrase operatorTile]) balloonTextSelector: 
			(balloonTextSelector ifNil: [cmd]).
	universal ifFalse:
		[selfTile _ self tileToRefer.
		selfTile position: aPhrase firstSubmorph position.
		aPhrase firstSubmorph addMorph: selfTile.
		aPhrase makeAllTilesGreen.
		aPhrase justGrabbedFromViewer: false].
	^ aPhrase! !

!Player methodsFor: 'playing commands' stamp: 'jm 9/28/2004 16:30'!
totalFrames

	^ self sendMessageToCostume: #totalFrames
! !

!Player methodsFor: 'playing commands' stamp: 'jm 9/28/2004 16:30'!
totalSeconds

	^ self sendMessageToCostume: #totalSeconds
! !

!Player methodsFor: 'misc' stamp: 'sw 9/6/2002 13:11'!
touchesA: aPrototypicalPlayer
	"Answer whether the receiver overlaps any player who wears a Sketch costume and who is of the same class as the prototypicalPlayer and who is wearing the same bitmap, but who is *not that player itself*!!  This is an extreme case of a function highly customized (by Bob Arning) to suit a single, idiosycratic, and narrow demo need of Alan's.  Consult:
http://groups.yahoo.com/group/squeak/message/40560"

	| envelope trueNeighbor trueGoal trueSelf itsPlayer |
	aPrototypicalPlayer ifNil: [^ false].
	envelope _ costume owner ifNil: [^ false].
	trueSelf _ costume renderedMorph.
	trueGoal _ aPrototypicalPlayer costume renderedMorph.
	envelope submorphs do: [:each |
		trueNeighbor _ each renderedMorph.
		(trueNeighbor == trueGoal or: [trueNeighbor == trueSelf]) ifFalse:
			[(itsPlayer _ each player) ifNotNil:
				[(itsPlayer overlaps: self) ifTrue:
					[(trueGoal appearsToBeSameCostumeAs: trueNeighbor) ifTrue: [^ true]]]]].
	^ false
! !

!Player methodsFor: 'pen' stamp: 'sw 3/11/2003 11:22'!
trailStyleForAllPens: aTrailStyle
	"Only for the Player of a World"

	self costume renderedMorph trailStyleForAllPens: aTrailStyle! !

!Player methodsFor: 'customevents-custom events' stamp: 'nk 8/26/2003 10:50'!
triggerCustomEvent: aSymbol
	"Trigger whatever scripts may be connected to the custom event named aSymbol"

	self costume renderedMorph triggerCustomEvent: aSymbol! !

!Player methodsFor: 'customevents-custom events' stamp: 'nk 11/1/2004 10:48'!
triggerScript: aSymbol 
	"Perform the script of the given name,
	which is guaranteed to exist.
	However, it's possible that the script may still result in a DNU,
	which will be swallowed and reported to the Transcript."

	^ [[self perform: aSymbol]
		on: GetTriggeringObjectNotification do: [ :ex |
			ex isNested
				ifTrue: [ ex pass ]
				ifFalse: [ ex resume: self ]]]
		on: MessageNotUnderstood
		do: [:ex | 
			ScriptingSystem
				reportToUser: (String
						streamContents: [:s | s nextPutAll: self externalName;
								 nextPutAll: ': exception in script ';
								 print: aSymbol;
								 nextPutAll: ' : ';
								 print: ex]).
			ex return: self
			"ex pass"]! !

!Player methodsFor: 'name' stamp: 'sw 11/2/2000 22:00'!
tryToRenameTo: aName
	self costume topRendererOrSelf tryToRenameTo: aName! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 1/6/2001 06:24'!
turn: degrees
	"Rotate the heading of the object by the given number of degrees"

	degrees ifNil: [^ self].
	degrees = 0 ifTrue: [^ self].
	self setHeading: (self getHeading + degrees asFloat) \\ 360.0
! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 7/29/2005 17:25'!
turnToward: aPlayer
	"Turn to face the given player, unless our positions coincide."

	|  aCostume myPosition itsPosition |
	(aPlayer == nil or: [aPlayer == self]) ifTrue: [^ self].
	aCostume _ self costume.
	aCostume isWorldMorph ifTrue: [^ self].
	(self overlaps: aPlayer) ifFalse:
		[((myPosition _ aCostume referencePosition) = (itsPosition _ aPlayer costume referencePosition))
			ifFalse: "avoid division by zero ;-("
				[self setHeading: (myPosition bearingToPoint: itsPosition)]]
! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 5/29/2001 13:57'!
typeForSlot: aSlotName
	"Answer the data type for values of the instance variable of the given name"

	| getter |
	(self slotInfo includesKey: aSlotName) ifTrue: [^ (self slotInfoAt: aSlotName) type].
	getter _ (aSlotName beginsWith: 'get')
		ifTrue:
			[aSlotName]
		ifFalse:
			[Utilities getterSelectorFor: aSlotName].
	^ (self currentVocabulary methodInterfaceAt: getter ifAbsent: [self error: 'Unknown slot name: ', aSlotName]) resultType! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 5/24/2001 14:29'!
typeForSlot: aSlotName vocabulary: aVocabulary
	"Answer the data type for values of the instance variable of the given name.  Presently has no senders but retained for a while..."

	| getter inherentSelector |
	inherentSelector _ Utilities inherentSelectorForGetter: aSlotName.
	(self slotInfo includesKey: inherentSelector) ifTrue: [^ (self slotInfoAt: inherentSelector) type].
	getter _ (aSlotName beginsWith: 'get')
		ifTrue:
			[aSlotName]
		ifFalse:
			[Utilities getterSelectorFor: aSlotName].
	^ (aVocabulary methodInterfaceAt: getter ifAbsent: [self error: 'Unknown slot name: ', aSlotName]) resultType! !

!Player methodsFor: 'slots-kernel' stamp: 'sw 8/3/2005 14:21'!
typeForSlotWithGetter: aGetter
	"Answer the data type for values of the instance variable of the given name"

	| getter inherentSelector |
	(#(color:sees: seesColor: touchesA: overlaps: overlapsAny:) includes: aGetter) ifTrue: [^ #Boolean].
	(#(+ * - /) includes: aGetter) ifTrue: [^ #Player].  "weird vector stuff"
  	"Annoying special cases"

	inherentSelector _ Utilities inherentSelectorForGetter: aGetter.
	(self slotInfo includesKey: inherentSelector) ifTrue: [^ (self slotInfoAt: inherentSelector) type].
	getter _ (aGetter beginsWith: 'get')
		ifTrue:
			[aGetter]
		ifFalse:
			[Utilities getterSelectorFor: aGetter].
	^ (Vocabulary eToyVocabulary methodInterfaceAt: getter ifAbsent: [self error: 'Unknown slot name: ', aGetter]) resultType! !

!Player methodsFor: 'costume' stamp: 'sw 7/5/2002 22:16'!
typeforParameterFor: aSelector
	"Answer the type of the parameter for the given selector"

	(self class scripts at: aSelector ifAbsent: [nil]) ifNotNilDo:
		[:aScript | ^ aScript argumentVariables first variableType].
	self error: 'No parameter type for ', aSelector.
	^ #Number! !

!Player methodsFor: 'misc' stamp: 'sw 1/1/1999 22:27'!
unhideHiddenObjects
	self sendMessageToCostume: #unhideHiddenObjects! !

!Player methodsFor: 'misc' stamp: 'sw 11/14/2000 11:20'!
uninstallFrom: aPlayfield
	"The receiver is about to be supplanted by another instance which is about to be installed as the current 'card' in the playfield.  Exit gracefully"

	self runAllClosingScripts.
	self commitCardPlayerDataFrom: aPlayfield! !

!Player methodsFor: 'viewer' stamp: 'sw 10/30/2000 17:13'!
uniqueNameForReference
	"Answer a unique name for referring to the receiver"

	| itsReferent |
	self flag: #deferred.  "The once-and-maybe-future ObjectRepresentativeMorph scheme is for the moment disenfranchised"

	"(costume isKindOf: ObjectRepresentativeMorph) ifTrue:
		[((itsReferent _ costume objectRepresented) isKindOf: Class)
			ifTrue:
				[^ itsReferent name].
		itsReferent == Smalltalk ifTrue: [^ #Smalltalk].
		itsReferent == ScriptingSystem ifTrue: [^ #ScriptingSystem]]."

	^  super uniqueNameForReference

! !

!Player methodsFor: 'scripts-kernel' stamp: 'tk 9/29/2001 22:20'!
universalTilesForInterface: aMethodInterface
	"Return universal tiles for the given method interface.  Record who self is."

	| ms argTile itsSelector aType argList makeSelfGlobal phrase |
	itsSelector _ aMethodInterface selector.
	argList _ OrderedCollection new.
	aMethodInterface argumentVariables doWithIndex:
		[:anArgumentVariable :anIndex | 
			argTile _ ScriptingSystem tileForArgType: (aType _ aMethodInterface typeForArgumentNumber: anIndex).
			argList add: (aType == #Player 
				ifTrue: [argTile actualObject]
				ifFalse: [argTile literal]).	"default value for each type"].

	ms _ MessageSend receiver: self selector: itsSelector arguments: argList asArray.
	"For CardPlayers, use 'self'.  For others, name me, and use my global name."
	makeSelfGlobal _ self class officialClass ~~ CardPlayer.
	phrase _ ms asTilesIn: self class globalNames: makeSelfGlobal.
	makeSelfGlobal ifFalse: [phrase setProperty: #scriptedPlayer toValue: self].
	^ phrase
! !

!Player methodsFor: 'slots-user' stamp: 'sw 1/5/2005 22:17'!
unlabeledWatcherFor: aGetter
	"Answer an unnlabeled readout for viewing a numeric-valued slot of mine"

	| aWatcher info anInterface watcherWording itsType vocab aSetter |
	info _ self slotInfoForGetter: aGetter.
	info ifNotNil:
			[itsType _ info type.
			watcherWording _ Utilities inherentSelectorForGetter: aGetter.
			aSetter _ Utilities setterSelectorFor: watcherWording]
		ifNil:
			[anInterface _Vocabulary eToyVocabulary methodInterfaceAt: aGetter ifAbsent: [nil].
			anInterface
				ifNotNil:
					[itsType _ anInterface resultType.
					aSetter _ anInterface companionSetterSelector]
				ifNil:
					[itsType _ #Unknown.
					aSetter _ nil].
			watcherWording _ anInterface ifNotNil: [anInterface wording] ifNil: ['*']].
	vocab _ Vocabulary vocabularyForType: itsType.
	aWatcher _ vocab updatingTileForTarget: self partName: watcherWording getter: aGetter setter: aSetter.

	aWatcher setNameTo: (self externalName, '''s ', watcherWording).
	aWatcher minHeight: (vocab wantsArrowsOnTiles ifTrue: [22] ifFalse: [14]).
	^ aWatcher! !

!Player methodsFor: 'misc' stamp: 'dgd 9/1/2003 14:17'!
unusedScriptName
	"answer a name of the form 'scriptN', where N is one higher than the highest-numbered similarly-named script"

	| highestThus aPair |
	highestThus _ 0.
	self class tileScriptNames do:
		[:aName |
			aPair _ (aName copyWithout: $:) stemAndNumericSuffix.
			aPair first = 'script' translated ifTrue: [highestThus _ highestThus max: aPair last]].
	^ ('script' translated, (highestThus + 1) printString) asSymbol! !

!Player methodsFor: 'misc' stamp: 'sw 1/19/2001 15:35'!
updateAllViewers
	"The receiver's structure has changed, so viewers on it and its siblings need to be reconstituted."

	| aPresenter |
	(aPresenter _ self costume presenter) ifNil: [^ self].
	self allOpenViewersOnReceiverAndSiblings do:
		[:aViewer | aPresenter updateViewer: aViewer]! !

!Player methodsFor: 'misc' stamp: 'sw 1/19/2001 15:36'!
updateAllViewersAndForceToShow: aCategory
	"The receiver's structure has changed, so viewers on it and all its siblings need to be reconstituted."

	| aPresenter |
	(aPresenter _ self costume presenter) ifNil: [^ self].
	self allOpenViewersOnReceiverAndSiblings do:
		[:aViewer | aPresenter updateViewer: aViewer forceToShow: aCategory]! !

!Player methodsFor: 'slots-kernel' stamp: 'dgd 4/4/2006 16:35'!
usableMethodInterfacesIn: methodInterfaceList
	"Filter the list given by methodInterfaceList, to remove items inappropriate to the receiver"

	self hasCostumeThatIsAWorld ifTrue:
		[^ methodInterfaceList select: [:anInterface |
			#(append: prepend: beep: clearTurtleTrails doScript: getColor "color" getCursor "cursor" deleteCard doMenuItem emptyScript firstPage goToFirstCardInBackground goToFirstCardOfStack goToLastCardInBackground goToLastCardOfStack goToNextCardInStack goToPreviousCardInStack initiatePainting insertCard  liftAllPens lowerAllPens trailStyleForAllPens: arrowheadsOnAllPens noArrowheadsOnAllPens getMouseX getMouseY "mouseX mouseY" pauseScript: reverse roundUpStrays shuffleContents startScript: stopScript: unhideHiddenObjects getValueAtCursor "valueAtCursor"
startAll: pauseAll: stopAll:  
viewAllMessengers clobberAllMessengers openAllScriptsTool handScriptControlButtons viewAllReferencedObjects jumpToProject: #getLength #getWidth )

includes: anInterface selector]].

	self hasAnyBorderedCostumes ifTrue: [^ methodInterfaceList].

	^ self hasOnlySketchCostumes
		ifTrue:
			[methodInterfaceList select: [:anInterface | (#(getColor getAlpha getSecondColor getBorderColor getBorderWidth getBorderStyle getRoundedCorners getUseGradientFill getRadialGradientFill ) includes: anInterface selector) not]]
		ifFalse:
			[methodInterfaceList select: [:anInterface | (#(getBorderColor getBorderWidth) includes: anInterface selector) not]]! !

!Player methodsFor: 'slot-kedama' stamp: 'yo 9/16/2004 18:50'!
useTurtle: aPlayer

	aPlayer costume renderedMorph kedamaWorld: costume renderedMorph.
	aPlayer costume renderedMorph install.
! !

!Player methodsFor: 'slots-user' stamp: 'sw 7/4/1998 18:03'!
valueOfType: aType from: oldValue
	"The user has changed a slot's type to aType; convert its former value, oldValue, to something of the appropriate type.  For now, does not take oldValue into account"
	^ self initialValueForSlotOfType: aType! !

!Player methodsFor: 'copying' stamp: 'tk 9/4/2001 11:10'!
veryDeepFixupWith: deepCopier
	| old |
	"Any uniClass inst var may have been weakly copied.  If they were in the tree being copied, fix them up, otherwise point to the originals."

super veryDeepFixupWith: deepCopier.
Player instSize + 1 to: self class instSize do:
	[:ii | old _ self instVarAt: ii.
	self instVarAt: ii put: (deepCopier references at: old ifAbsent: [old])].
! !

!Player methodsFor: 'copying' stamp: 'tk 9/5/2001 09:43'!
veryDeepInner: deepCopier
	"Special code that handles user-added instance variables of a uniClass.
	Copy all of my instance variables.  Some need to be not copied at all, but shared.  This is special code for the dictionary.  See DeepCopier."
	| instVar weak subAss |

	super veryDeepInner: deepCopier.
	"my own instance variables are completely normal"
	costume _ costume veryDeepCopyWith: deepCopier.
	costumes _ costumes veryDeepCopyWith: deepCopier.

	Player instSize + 1 to: self class instSize do: [:index |
		instVar _ self instVarAt: index.
		weak _ instVar isMorph | instVar isPlayerLike. 
		(subAss _ deepCopier references associationAt: instVar ifAbsent: [nil])
				"use association, not value, so nil is an exceptional value"
			ifNil: [weak ifFalse: [
					self instVarAt: index put: (instVar veryDeepCopyWith: deepCopier)]]
			ifNotNil: [self instVarAt: index put: subAss value].
		].
! !

!Player methodsFor: 'misc' stamp: 'ccn 12/15/2000 19:03'!
viewerFlapTab
	"If a viewer in a flap exists for me, return it."

	(costume world ifNil: [self currentWorld])
		submorphsDo: [:mm |
			(mm isKindOf: ViewerFlapTab)
				ifTrue:
					[mm scriptedPlayer == self
						ifTrue: [^mm]]].
	^nil! !

!Player methodsFor: 'testing' stamp: 'sw 10/19/1999 08:30'!
wantsSteps
	"UnscriptedPlayer, with no scripts, overrides to false"
	^ true! !

!Player methodsFor: 'costume' stamp: 'mir 6/13/2001 15:29'!
wearCostumeOf: anotherPlayer
	"Put on a costume similar to the one currently worn by anotherPlayer"

	self renderedCostume: (anotherPlayer costume renderedMorph asWearableCostumeOfExtent: self costume extent) remember: anotherPlayer costume shouldRememberCostumes! !

!Player methodsFor: 'costume' stamp: 'sw 12/3/1998 10:05'!
wearCostumeOfClass: aClass
	"Assume that the costume in the library has player = nil"
	| newCostume |
	(costume renderedMorph isKindOf: aClass) ifTrue: [^ self].
	costumes ifNotNil:
		[costumes do:
			[:aCostume | (aCostume class  == aClass)
				ifTrue:
					[^ self renderedCostume: aCostume]]].

	newCostume _ aClass new.
	self renderedCostume: newCostume! !

!Player methodsFor: 'costume' stamp: 'sw 2/20/98 00:32'!
wearCostumeOfName: aName
	| classToUse |
	classToUse _ Smalltalk at: (aName, 'Morph') asSymbol ifAbsent: 
		[Smalltalk at: aName asSymbol].
	self wearCostumeOfClass: classToUse! !

!Player methodsFor: 'misc' stamp: 'sw 2/1/98 02:36'!
width
	^ self costume width! !

!Player methodsFor: 'scripts-standard' stamp: 'sw 11/28/2000 11:02'!
wrap
	"If the object has gone outside the bounds of its container, zap it over to the opposite edge of the container, providing a 'wrapping' impression "

	self costume wrap! !

Player subclass: #UnscriptedPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

UnscriptedPlayer class
	instanceVariableNames: 'ephemeralPlayerRef'!

!UnscriptedPlayer commentStamp: '<historical>' prior: 0!
My instances are Player objects that have not been scripted, and which hence do not require a unique scripts dictionary, etc.  As soon as the needed, I am transformed automatically into a unique subclass of Player.!

UnscriptedPlayer class
	instanceVariableNames: 'ephemeralPlayerRef'!

UnscriptedPlayer subclass: #UnscriptedCardPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!UnscriptedCardPlayer methodsFor: 'uniclass' stamp: 'sw 1/30/2001 22:42'!
rootClassForUniclasses
	"Answer the class that should be subclassed when the receiver is made into a uniclass"

	^ CardPlayer! !

!UnscriptedPlayer classSide methodsFor: 'reference' stamp: 'sw 9/30/1998 09:14'!
ephemeralPlayerRef
	"UnscriptedPlayer ephemeralPlayerRef"
	^ ephemeralPlayerRef! !

!UnscriptedPlayer classSide methodsFor: 'testing' stamp: 'sw 8/17/1998 07:33'!
isSystemDefined
	^ true! !

!UnscriptedPlayer classSide methodsFor: 'instance creation' stamp: 'sw 10/23/1999 22:51'!
isUniClass
	^ false! !

!UnscriptedPlayer classSide methodsFor: 'instance creation' stamp: 'sw 9/15/1998 13:33'!
newUserInstance
	"Answer an instance of an appropriate class to serve as a user object in the containment hierarchy"

	^ self new! !

!UnscriptedPlayer classSide methodsFor: 'testing' stamp: 'sw 8/17/1998 07:34'!
officialClass
	^ self! !

!UnscriptedPlayer classSide methodsFor: 'namespace' stamp: 'sw 9/30/1998 09:05'!
referenceSelectorFor: anObject
	"The use of this is for immediate evaluation of lines of script in a Viewer.  The class inst var 'ephemeralPlayerRef' is constantly reused for this purpose."

	ephemeralPlayerRef _ anObject.
	^ 'ephemeralPlayerRef'! !

!UnscriptedPlayer classSide methodsFor: 'compiling' stamp: 'sw 9/15/1998 13:49'!
wantsChangeSetLogging
	^ true! !

!UnscriptedPlayer methodsFor: 'viewer' stamp: 'sw 1/30/2001 22:43'!
assureUniClass
	"Create a uniclass and become the receiver into it"

	| anInstance |
	anInstance _ self rootClassForUniclasses instanceOfUniqueClass.
	anInstance initializeCostumesFrom: self.
	self become: anInstance.
	^ anInstance! !

!UnscriptedPlayer methodsFor: 'viewer' stamp: 'sw 8/10/1998 16:23'!
belongsToUniClass
	^ false! !

!UnscriptedPlayer methodsFor: 'copying' stamp: 'tk 10/4/2001 13:43'!
copyUniClassWith: deepCopier
	
	self error: 'oops, copyUniClass sent to an UnscriptedPlayer'! !

!UnscriptedPlayer methodsFor: 'error handling' stamp: 'nk 8/21/2004 11:44'!
doesNotUnderstand: aMessage
	"I do not attempt the special setters that Player does.
	I merely ignore not-understood messages."
	^nil! !

!UnscriptedPlayer methodsFor: 'slots-user' stamp: 'sw 12/28/1998 10:27'!
hasUserDefinedScripts
	^ false! !

!UnscriptedPlayer methodsFor: 'viewer' stamp: 'sw 12/28/1998 10:27'!
hasUserDefinedSlots
	^ false! !

!UnscriptedPlayer methodsFor: 'uniclass' stamp: 'sw 1/30/2001 22:42'!
rootClassForUniclasses
	"Answer the class that should be subclassed when the receiver is made into a uniclass"

	^ Player! !

!UnscriptedPlayer methodsFor: 'testing' stamp: 'sw 10/19/1999 08:30'!
wantsSteps
	"Has no scripts"
	^ false! !

Model subclass: #ScriptingDomain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!ScriptingDomain methodsFor: 'user interface' stamp: 'dgd 8/30/2003 21:54'!
addModelMenuItemsTo: aCustomMenu forMorph: aMorph hand: aHandMorph 
	super addModelMenuItemsTo: aCustomMenu forMorph: self hand: aHandMorph.  "nominally nothing"
	aCustomMenu add: 'take out of window' translated action: #takeOutOfWindow

	! !

!ScriptingDomain methodsFor: 'user interface' stamp: 'sw 10/27/1999 11:39'!
defaultBackgroundColor
	^ Color r: 0.91 g: 0.91 b: 0.91! !

!ScriptingDomain methodsFor: 'user interface' stamp: 'sw 9/30/1998 13:00'!
initialExtent
	^ 640 @ 480! !

!PolygonMorph methodsFor: '*eToys-geometry eToy' stamp: 'di 9/24/2000 09:36'!
heading: newHeading
	"Set the receiver's heading (in eToy terms).
	Note that polygons never use flex shells."
	self rotationDegrees: newHeading.! !

SymbolListType subclass: #ScriptNameType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Protocols-Type Vocabularies'!

!ScriptNameType commentStamp: '<historical>' prior: 0!
ScriptNameType is a data type representing selectors of user-written scripts.  The choices offered as values for data of this type are all the symbols that are implemented as names of user-written scripts in the current project.!

!ScriptNameType methodsFor: 'queries' stamp: 'sw 12/19/2003 23:42'!
choices
	"Answer an alphabetized list of known script selectors in the current project"

	^ ActiveWorld presenter allKnownUnaryScriptSelectors
! !

!ScriptNameType methodsFor: 'tiles' stamp: 'sw 12/19/2003 23:11'!
defaultArgumentTile
	"Answer a tile to represent the type"

	| aTile  |
	aTile _ ScriptNameTile new dataType: self vocabularyName.
	aTile addArrows.
	aTile setLiteral: #emptyScript.
	^ aTile! !

!ScriptNameType methodsFor: 'initialization' stamp: 'sw 12/3/2001 21:30'!
initialize
	"Initialize the ScriptNameType vocabulary"

	super initialize.
	self vocabularyName: #ScriptName! !

!ScriptNameType methodsFor: 'tile' stamp: 'sw 12/19/2003 23:07'!
newReadoutTile
	"Answer a tile that can serve as a readout for data of this type"

	^ ScriptNameTile new dataType: self vocabularyName
! !

!ScriptNameType methodsFor: 'queries' stamp: 'mir 7/15/2004 10:35'!
representsAType
	"Answer whether this vocabulary represents an end-user-sensible data type"

	^true! !

!SymbolListType methodsFor: '*eToys-tiles' stamp: 'sw 12/3/2001 19:15'!
defaultArgumentTile
	"Answer a tile to represent the type"

	| aTile choices |
	aTile _ SymbolListTile new choices: (choices _ self choices) dataType: self vocabularyName.
	aTile addArrows.
	aTile setLiteral: choices first.
	^ aTile! !

!SymbolListType methodsFor: '*eToys-tiles' stamp: 'sw 12/3/2001 21:00'!
newReadoutTile
	"Answer a tile that can serve as a readout for data of this type"

	^ SymbolListTile new choices: self choices dataType: self vocabularyName
! !

SymbolListType subclass: #UserCustomEventNameType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-CustomEvents'!

!UserCustomEventNameType commentStamp: 'nk 6/12/2004 14:09' prior: 0!
This is a data type that enumerates user-defined custom event names.

You can turn off the display of such events in the script status popups by turning off the

	allowEtoyUserCustomEvents
	
Preference.!

!UserCustomEventNameType classSide methodsFor: 'class initialization' stamp: 'nk 6/12/2004 14:18'!
allowEtoyUserCustomEventsPreferenceChanged
	Cursor wait showWhile: [ Vocabulary changeMadeToViewerAdditions ]! !

!UserCustomEventNameType classSide methodsFor: 'class initialization' stamp: 'nk 8/18/2004 18:02'!
initialize
	Vocabulary embraceAddedTypeVocabularies.
	Preferences
		addPreference: #allowEtoyUserCustomEvents
		categories:  #('scripting')
		default: false
		balloonHelp: 'If true, you can define your own events that can trigger scripts within a World.'
		projectLocal:  false
		changeInformee:  self
		changeSelector: #allowEtoyUserCustomEventsPreferenceChanged! !

!UserCustomEventNameType methodsFor: 'queries' stamp: 'nk 9/26/2003 23:36'!
choices
	"Answer an alphabetized list of known user custom event selectors"

	| choices |
	choices _ ScriptingSystem userCustomEventNames.
	^choices isEmpty ifTrue: [ #('no event') ] ifFalse: [ choices ]! !

!UserCustomEventNameType methodsFor: 'initialization' stamp: 'nk 7/21/2003 20:42'!
initialize
	"Initialize the CustomEvents vocabulary"

	super initialize.
	self vocabularyName: #CustomEvents! !

!UserCustomEventNameType methodsFor: 'tiles' stamp: 'nk 8/18/2004 17:48'!
representsAType
	"Answer whether this vocabulary represents an end-user-sensible data type"

	^true! !

!SpeakerMorph class methodsFor: '*eToys-scripting' stamp: 'sw 9/26/2001 04:27'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^  #((speaker

((slot conePosition  'the position of the speaker cone' Number readWrite Player getConePosition Player setConePosition:))))



! !

!Debugger methodsFor: '*eToys-code pane' stamp: 'di 11/16/2000 16:03'!
createSyntaxMorph

	| methodNode rootMorph |
	methodNode _ self selectedClass compilerClass new
			parse: contents
			in: self selectedClass
			notifying: nil.
	(rootMorph _ methodNode asMorphicSyntaxUsing: SyntaxMorph)
		parsedInClass: self selectedClass;
		debugger: self.
	self addDependent: rootMorph.
	^rootMorph

! !

!Debugger methodsFor: '*eToys-code pane' stamp: 'di 1/31/2001 11:14'!
toggleSyntaxMorph
"
	syntaxMorph ifNil:
		[syntaxMorph _ self createSyntaxMorph inAScrollPane.
		syntaxMorph color: Color paleOrange].
	standardTextMorph visible ifTrue: [
		standardTextMorph owner replacePane: standardTextMorph with: syntaxMorph.
		syntaxMorph scroller firstSubmorph update: #contentsSelection.
	] ifFalse: [
		syntaxMorph owner replacePane: syntaxMorph with: standardTextMorph.
	].
"
! !

!Class methodsFor: '*eToys-class name' stamp: 'sw 12/1/2000 20:40'!
uniqueNameForReference
	"Answer a unique name by which the receiver can be referred to from user scripts, for example"

	^ name! !

!BooleanType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:20'!
defaultArgumentTile
	"Answer a tile to represent the type"

	^ true newTileMorphRepresentative typeColor: self typeColor! !

!BooleanType methodsFor: '*eToys-color' stamp: 'sw 9/27/2001 17:20'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(0.94 1.0 0.06)! !

UpdatingStringMorph subclass: #SyntaxUpdatingStringMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Tile Scriptors'!

!SyntaxUpdatingStringMorph methodsFor: 'drawing' stamp: 'tk 1/31/2002 09:53'!
drawOn: aCanvas

	| tempForm strm where chars wid spaceWidth putLigature topOfLigature sizeOfLigature colorOfLigature dots charZero canvas f |

	tempForm _ Form extent: self extent depth: aCanvas depth.
	canvas _ tempForm getCanvas.
	f _ self fontToUse.
	spaceWidth _ f widthOf: Character space.
	strm _ ReadStream on: contents.
	charZero _ Character value: 0.	"a marker for center dot "
	where _ 0@0.
	topOfLigature _ self height // 2 - 1.
	sizeOfLigature _ (spaceWidth-2)@(spaceWidth-2).
	colorOfLigature _ Color black alpha: 0.45	"veryLightGray".
	dots _ OrderedCollection new.
	putLigature _ [
		dots add: ((where x + 1) @ topOfLigature extent: sizeOfLigature).
		where _ where + (spaceWidth@0)].
	strm peek = charZero ifTrue: [
		strm next.
		putLigature value].
	[strm peek = charZero] whileTrue: [strm next].
	[strm atEnd] whileFalse: [
		chars _ strm upTo: charZero.
		wid _ f widthOfString: chars.
		canvas drawString: chars at: where.
		where _ where + (wid@0).
		strm atEnd ifFalse: [putLigature value.
			[strm peek = charZero] whileTrue: [strm next]].
	].
	aCanvas paintImage: tempForm at: self topLeft.
	dots do: [ :each |
		aCanvas 
			fillRectangle: (each translateBy: self topLeft) 
			fillStyle: colorOfLigature.
	].
! !

!UpdatingStringMorph methodsFor: '*eToys-card in a stack' stamp: 'tk 9/26/2001 06:04'!
couldHoldSeparateDataForEachInstance
	"Answer whether this type of morph is inherently capable of holding separate data for each instance ('card data')"

	^ true! !

!UpdatingStringMorph methodsFor: '*eToys-player' stamp: 'tk 9/26/2001 06:06'!
currentDataValue
	"Answer the current data value held by the receiver"

	^ self valueFromContents! !

!UpdatingStringMorph methodsFor: '*eToys-target access' stamp: 'sw 3/7/2004 15:49'!
hasStructureOfComplexWatcher
	"Answer whether the receiver has precisely the structure of a so-called complex watcher, as used in the etoy system."

	| top |
	top _ (self owner ifNil: [^ false]) owner.
	^ ((((top isMemberOf: AlignmentMorph)
		and: [top submorphs size = 4])
			and: [top submorphs first isMemberOf: TileMorph])
				and: [top submorphs third isMemberOf: AlignmentMorph])! !

!UpdatingStringMorph methodsFor: '*eToys-menus' stamp: 'tk 9/26/2001 06:08'!
putOnBackground
	"Place the receiver, formerly private to its card, onto the shared background.  If the receiver needs data carried on its behalf by the card, such data will be represented on every card."

	"If I seem to have per-card data, then set that up."
	target class superclass == CardPlayer ifTrue: [
		(self hasOwner: target costume) ifTrue: [	
			self setProperty: #holdsSeparateDataForEachInstance toValue: true]].
	super putOnBackground.! !

!UpdatingStringMorph methodsFor: '*eToys-card & stack' stamp: 'tk 9/25/2001 11:43'!
setNewContentsFrom: stringOrNumberOrNil

	self acceptValue: stringOrNumberOrNil! !

!UpdatingStringMorph methodsFor: '*eToys-player' stamp: 'tk 9/26/2001 06:10'!
variableDocks
	"Answer a list of VariableDock objects for docking up my data with an instance held in my containing playfield.  For a numeric-readout tile."

	"Is CardPlayer class holding my variableDock, or should I be using the caching mechanism in Morph>>variableDocks?"
	^ Array with: (VariableDock new 
			variableName: (getSelector allButFirst: 3) withFirstCharacterDownshifted 
			type: #number 
			definingMorph: self 
			morphGetSelector: #valueFromContents 
			morphPutSelector: #acceptValue:)! !

!PluggableTextMorphWithModel methodsFor: '*eToys-card in a stack' stamp: 'sw 10/24/2000 14:52'!
couldHoldSeparateDataForEachInstance
	"Answer whethre the receiver is structurally capable of holding uniqe data for each ard instance"

	^ true! !

!PluggableTextMorphWithModel methodsFor: '*eToys-player' stamp: 'sw 10/30/2000 11:14'!
currentDataValue
	"Answer the current data value of the receiver"

	^ myContents! !

!PluggableTextMorphWithModel methodsFor: '*eToys-player' stamp: 'sw 10/25/2000 07:00'!
variableDocks
	"Answer a list of VariableDocks that will handle the interface between me and instance data stored on my behalf on a card"

	^ Array with: (VariableDock new variableName: self defaultVariableName type: #text definingMorph: self morphGetSelector: #getMyText morphPutSelector: #setMyText:)! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:36'!
addExtraItemsToMenu: aMenu forSlotSymbol: slotSym
	"If the receiver has extra menu items to add to the slot menu, here is its chance to do it"! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:38'!
addUserSlotItemsTo: aMenu slotSymbol: slotSym
	"Optionally add items to the menu that pertain to a user-defined slot of the given symbol"
! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 1/12/2005 08:35'!
addWatcherItemsToMenu: aMenu forGetter: aGetter
	"Add watcher items to the menu if appropriate, provided the getter is not an odd-ball one for which a watcher makes no sense"

	(Vocabulary gettersForbiddenFromWatchers includes: aGetter) ifFalse:
		[aMenu add: 'simple watcher' translated selector: #tearOffUnlabeledWatcherFor: argument: aGetter.
		aMenu add: 'detailed watcher' translated selector: #tearOffFancyWatcherFor: argument: aGetter.
		aMenu addLine]! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 02:29'!
affordsCoercionToBoolean
	"Answer true if a tile of this data type, when dropped into a pane that demands a boolean, could plausibly be expanded into a comparison (of the form  frog < toad   or frog = toad) to provide a boolean expression"

	^ true! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 02:53'!
comparatorForSampleBoolean
	"Answer the comparator to use in tile coercions involving the receiver; normally, the equality comparator is used but NumberType overrides"

	^ #=! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 13:15'!
defaultArgumentTile
	"Answer a tile to represent the type"

	^ 'arg' newTileMorphRepresentative typeColor: self typeColor! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:37'!
newReadoutTile
	"Answer a tile that can serve as a readout for data of this type"

	^ StringReadoutTile new typeColor: Color lightGray lighter! !

!DataType methodsFor: '*eToys-initialization' stamp: 'sw 9/27/2001 17:32'!
setFormatForDisplayer: aDisplayer
	"Set up the displayer to have the right format characteristics"

	aDisplayer useDefaultFormat.
	aDisplayer growable: true
	! !

!DataType methodsFor: '*eToys-color' stamp: 'sw 8/28/2004 20:30'!
subduedColorFromTriplet: anRGBTriplet
	"Currently:  as an expedient, simply return a standard system-wide constant; this is used only for the border-color of tiles...
	Formerly:  Answer a subdued color derived from the rgb-triplet to use as a tile color."

	^ ScriptingSystem standardTileBorderColor
"
	^ (Color fromRgbTriplet: anRGBTriplet) mixed: ScriptingSystem colorFudge with: ScriptingSystem uniformTileInteriorColor"! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 1/4/2005 00:45'!
updatingTileForTarget: aTarget partName: partName getter: getter setter: setter
	"Answer, for classic tiles, an updating readout tile for a part with the receiver's type, with the given getter and setter"

	| aTile displayer actualSetter |
	actualSetter _ setter ifNotNil:
		[(#(none #nil unused) includes: setter) ifTrue: [nil] ifFalse: [setter]].

	aTile _ self newReadoutTile.

	displayer _ UpdatingStringMorph new
		getSelector: getter;
		target: aTarget;
		growable: true;
		minimumWidth: 24;
		putSelector: actualSetter.
	"Note that when typeSymbol = #number, the #target: call above will have dealt with floatPrecision details"

	self setFormatForDisplayer: displayer.
	aTile addMorphBack: displayer.
	(actualSetter notNil and: [self wantsArrowsOnTiles]) ifTrue: [aTile addArrows].	
	getter numArgs == 0 ifTrue:
		[aTile setLiteralInitially: (aTarget perform: getter)].
	^ aTile
! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:33'!
wantsArrowsOnTiles
	"Answer whether this data type wants up/down arrows on tiles representing its values"

	^ true! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 9/26/2001 03:11'!
wantsAssignmentTileVariants
	"Answer whether an assignment tile for a variable of this type should show variants to increase-by, decrease-by, multiply-by.  NumberType says yes, the rest of us say no"

	^ false! !

!DataType methodsFor: '*eToys-tiles' stamp: 'sw 9/26/2001 03:18'!
wantsSuffixArrow
	"Answer whether a tile showing data of this type would like to have a suffix arrow"

	^ false! !

DataType subclass: #PlayerType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Protocols-Type Vocabularies'!

PlayerType subclass: #KedamaPatchType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaPatchType commentStamp: 'yo 6/18/2004 18:32' prior: 0!
EToys Patch type.
!

!KedamaPatchType methodsFor: 'tile protocol' stamp: 'yo 10/13/2004 04:16'!
defaultArgumentTile
	"Answer a tile to represent the type"
	| patch ks k p |
	patch _ KedamaPatchTile new typeColor: self typeColor.
	ks _ ActiveWorld allMorphs select: [:e | e isKindOf: KedamaMorph].
	ks isEmpty ifFalse: [
		k _ ks first.
		p _ k player getPatch.
	] ifTrue: [
		k _ KedamaPatchMorph new.
		k assuredPlayer.
		p _ k player.
	].
	patch usePatch: p.
	^ patch.
! !

!KedamaPatchType methodsFor: 'tile protocol' stamp: 'yo 10/13/2004 01:53'!
defaultArgumentTileFor: aPlayer
	"Answer a tile to represent the type"
	| patch morph |
	patch _ KedamaPatchTile new typeColor: self typeColor.
	morph _ aPlayer costume renderedMorph.
	(morph isKindOf: KedamaMorph) ifTrue: [
		patch usePatch: aPlayer costume renderedMorph player getPatch.
	].
	(morph isKindOf: KedamaPatchMorph) ifTrue: [
		patch usePatch: morph player.
	].
	^ patch.
! !

!KedamaPatchType methodsFor: 'tile protocol' stamp: 'yo 6/17/2004 19:44'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"

	^ aPlayer costume renderedMorph player getPatch.
! !

!KedamaPatchType methodsFor: 'initialization' stamp: 'yo 6/5/2004 16:18'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	self vocabularyName: #Patch.
! !

!KedamaPatchType methodsFor: 'tile protocol' stamp: 'yo 6/5/2004 16:19'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(0.806 1.0 0.806)	! !

!KedamaPatchType methodsFor: 'tile protocol' stamp: 'yo 9/13/2005 21:23'!
updatingTileForTarget: aTarget partName: partName getter: getter setter: setter

	| aTile displayer actualSetter |
	actualSetter _ setter ifNotNil:
		[(#(none nil unused) includes: setter) ifTrue: [nil] ifFalse: [setter]].

	aTile _ self newReadoutTile.

	displayer _ UpdatingStringMorph new
		getSelector: #externalName;
		target: (aTarget perform: getter) costume renderedMorph;
		growable: true;
		minimumWidth: 24;
		putSelector: nil.
	displayer stepTime: 1000.
	"Note that when typeSymbol = #number, the #target: call above will have dealt with floatPrecision details"

	self setFormatForDisplayer: displayer.
	aTile addMorphBack: displayer.
	(actualSetter notNil and: [self wantsArrowsOnTiles]) ifTrue: [aTile addArrows].	
	getter numArgs == 0 ifTrue:
		[aTile setLiteralInitially: (aTarget perform: getter)].
	displayer useStringFormat.

	^ aTile
! !

!PlayerType methodsFor: 'tiles' stamp: 'tak 1/18/2005 13:13'!
addExtraItemsToMenu: aMenu forSlotSymbol: slotSym
	"If the receiver has extra menu items to add to the slot menu, here is its chance to do it"

	aMenu add: 'tiles to get...' translated selector: #offerGetterTiles: argument: slotSym.
	aMenu add:  'reveal me' translated target: aMenu defaultTarget selector: #revealPlayerNamed:in: argumentList: { slotSym. ActiveWorld}.! !

!PlayerType methodsFor: 'tiles' stamp: 'sw 9/27/2001 17:30'!
defaultArgumentTile
	"Answer a tile to represent the type"

	^ ActiveWorld presenter standardPlayer tileToRefer! !

!PlayerType methodsFor: 'initial value' stamp: 'sw 9/27/2001 17:29'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"

	^ aPlayer costume presenter standardPlayer! !

!PlayerType methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:24'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	self vocabularyName: #Player! !

!PlayerType methodsFor: 'color' stamp: 'sw 9/27/2001 17:21'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(1.0  0 0.065)! !

!PlayerType methodsFor: 'tiles' stamp: 'sw 9/25/2001 21:04'!
updatingTileForTarget: aTarget partName: partName getter: getter setter: setter
	"Answer, for classic tiles, an updating readout tile for a part with the receiver's type, with the given getter and setter"

	^ PlayerReferenceReadout new objectToView: aTarget viewSelector: getter putSelector: setter! !

!PlayerType methodsFor: 'tiles' stamp: 'sw 9/27/2001 17:33'!
wantsArrowsOnTiles
	"Answer whether this data type wants up/down arrows on tiles representing its values"

	^ false! !

!BookMorph classSide methodsFor: '*eToys-scripting' stamp: 'sw 11/7/2002 13:20'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((#'book navigation'
			((command goto: 'go to the given page' Player)
			(command nextPage 'go to next page')
			(command previousPage 'go to previous page')
			(command firstPage 'go to first page')
			(command lastPage 'go to last page')
			(slot pageNumber 'The ordinal number of the current page' Number readWrite Player getPageNumber Player setPageNumber:))))! !

!BookMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 10/2/97 15:22'!
configureForKids
	super configureForKids.
	pages do:
		[:aPage | aPage configureForKids].! !

!BookMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 8/11/1998 16:50'!
succeededInRevealing: aPlayer
	currentPage ifNotNil: [currentPage player == aPlayer ifTrue: [^ true]].
	pages do:
		[:aPage |
			(aPage succeededInRevealing: aPlayer) ifTrue:
				[self goToPageMorph: aPage.
				^ true]].
	^ false! !

BookMorph subclass: #StackMorph
	instanceVariableNames: 'cards'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Stacks'!

!StackMorph commentStamp: '<historical>' prior: 0!
A book that is very much like a HyperCard stack.  

Each book page represents a different background.  The page stays while different cards are projected onto it.  
	The data for a single card is stored in a CardPlayer.  There is a list of objects that only appear on this card (privateMorphs) and the card-specific text to be inserted into the background fields.

Item					How it is stored
a background			a page of the StackMorph
a card					data is in an instance of a subclass of CardPlayer.
						A list of CardPlayers is in the 'cards' inst var of the StackMorph.
a background field		a TextMorph on a page of the StackMorph
a background picture	a morph of any kind on a page of the StackMorph
script for bkgnd button		method in Player.  Button is its costume.
text in a background field		value of inst var 'field1' in a CardPlayer.
								(The CardPlayer is also pointed at by the #cardInstance 
								property of the bkgnd field (TextMorph))
text in a card field		in the TextMorph in privateMorphs in the CardPlayer.
picture on a card		a morph of any kind in privateMorphs in the CardPlayer.
script for card button	method in the CardPlayer.  Button is its costume.

See VariableDock.!

!StackMorph classSide methodsFor: 'scripting' stamp: 'sw 11/2/2002 15:47'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ # ((#'stack navigation'
			((command goToNextCardInStack 'Go to the next card')
			(command goToPreviousCardInStack  'Go to the previous card')
			(command goToFirstCardInBackground 'Go to the first card of the current background')
			(command goToFirstCardOfStack 'Go to the first card of the entire stack')
			(command goToLastCardInBackground 'Go to the last card of the current background')
			(command goToLastCardOfStack 'Go to the last card of the entire stack')
			(command deleteCard 'Delete the current card')
			(command insertCard 'Create a new card')
			(slot cardNumber 'The ordinal number of the current card' Number readWrite Player getCardNumber Player setCardNumber:))))! !

!StackMorph classSide methodsFor: 'scripting' stamp: 'sw 10/9/2000 07:43'!
authoringPrototype
	"Answer an instance of the receiver suitable for placing in a parts bin for authors"
	
	| book |
	book _ self new markAsPartsDonor.
	book pageSize: (480 @ 320); color: (Color gray: 0.7).
	book borderWidth: 1; borderColor: Color black.
	book currentPage extent: book pageSize.
	book showPageControls: book fullControlSpecs.
	^ book

"self currentHand attachMorph: StackMorph authoringPrototype"! !

!StackMorph classSide methodsFor: 'parts bin' stamp: 'sw 8/2/2001 12:52'!
descriptionForPartsBin
	^ self partName:	'Stack'
		categories:		#('Presentation')
		documentation:	'A database of any sort -- slide show, rolodex, and any point in between'! !

!StackMorph classSide methodsFor: 'authoring prototype' stamp: 'nk 7/12/2003 08:59'!
designationsExplainer
	"Answer a morph that contains designation explanation"

	| aMorph aSwatch aTextMorph |
	aMorph _ AlignmentMorph newColumn color: Color black; layoutInset: 1.
	#((green		
'Shared items on
Background.
Exact same item
shared by every card')
	(orange
'Data items on
Background
Each card has its
own data')
	(red
'Instance-specific
items
unique
to this card')) do:

	[:aPair |
		aSwatch _ AlignmentMorph new extent: 132 @80; color: (Color perform: aPair first); lock.
		aSwatch hResizing: #rigid; vResizing: #rigid; layoutInset: 0.
		aSwatch borderColor: Color black.
		aTextMorph _ TextMorph new string: aPair second fontName: Preferences standardEToysFont familyName size: 18.
		aTextMorph width: 130.
		aTextMorph centered.
		aSwatch addMorphBack: aTextMorph.
		aMorph addMorphBack: aSwatch].
	aMorph hResizing: #shrinkWrap; vResizing: #shrinkWrap.

	^ aMorph

	"StackMorph designationsExplainer openInHand"
! !

!StackMorph classSide methodsFor: 'misc' stamp: 'tk 12/14/2001 19:23'!
discoverSlots: aMorph
	"Examine the parts of the morph for ones that couldHoldSeparateData.  Return a pair of lists: Named morphs, and unnamed morphs (which may be labels, and non-data).  Examine all submorphs."

	| named unnamed got sn generic |
	named _ OrderedCollection new.
	unnamed _ OrderedCollection new.
	aMorph submorphsDo: [:direct | 
		got _ false.
		direct allMorphsDo: [:sub |
			sub couldHoldSeparateDataForEachInstance ifTrue: [
				(sn _ sub knownName) ifNotNil: [
					generic _ (#('Number (fancy)' 'Number (mid)' 'Number (bare)')
									includes: sn).
					(sn beginsWith: 'shared' "label") | generic ifFalse: [
						named add: sub.
						got _ true]]]].
		got ifFalse: [unnamed add: direct]].
	^ Array with: named with: unnamed
		! !

!StackMorph classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 11:30'!
initialize

	self registerInFlapsRegistry.	! !

!StackMorph classSide methodsFor: 'navigation buttons' stamp: 'sw 10/27/2000 10:53'!
nextCardButton
	"Answer a button that advances the user to the next card in the stack"

	| aButton |
	aButton _ SimpleButtonMorph new.
	aButton target: aButton; actionSelector: #goToNextCardInStack; label: '>'; color: Color yellow; borderWidth: 0.
	aButton setNameTo: 'next'.
	^ aButton! !

!StackMorph classSide methodsFor: 'navigation buttons' stamp: 'sw 10/27/2000 10:53'!
previousCardButton
	"Answer a button that will take the user to the preceding card in the stack"

	| aButton |
	aButton _ SimpleButtonMorph new.
	aButton target: aButton; actionSelector: #goToPreviousCardInStack; label: '<'; color: Color yellow ; borderWidth: 0.
	aButton setNameTo: 'previous'.
	^ aButton! !

!StackMorph classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 11:36'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: #(StackMorph	authoringPrototype	'Stack'		'A multi-card data base'	)
						forFlapNamed: 'Scripting'.
						cl registerQuad: #(StackMorph	authoringPrototype	'Stack'		'A multi-card data base'	)
						forFlapNamed: 'Stack Tools'.
						cl registerQuad: #(StackMorph	stackHelpWindow	'Stack Help'	'Some hints about how to use Stacks')
						forFlapNamed: 'Stack Tools'.
						cl registerQuad: #(StackMorph	previousCardButton	'Previous Card'	'A button that takes the user to the previous card in the stack')
						forFlapNamed: 'Stack Tools'.
						cl registerQuad: #(StackMorph	nextCardButton	'Next Card'		'A button that takes the user to the next card in the stack')
						forFlapNamed: 'Stack Tools']! !

!StackMorph classSide methodsFor: 'parts bin' stamp: 'sw 4/8/2002 09:30'!
stackHelpWindow
	^ (Workspace new contents: 'A "stack" is a place where you can create, store, view and retrieve data "fields" from a set of "cards".  Data that you want to occur on every card (such as a name and an address in an Address Stack) are represented by objects such as "Simple Text", "Fancy Text", and "Scrolling Text" that you obtain from the Stack Tools flap.

When you look at a card in a Stack, you may be seeing three different kinds of material.  Press the  button in the stack''s controls to see the current designations, and use the "explain designations" to get a reminder of what the three different colors mean.
  Things that are designated to be seen on every card, and have the same contents whichever card is being shown. (green)
  Things that are designated to be seen on every card, with each card having its own value for them. (orange)
  Things that are designated to occur only on the particular card at hand. (red)

Use the "stack/cards" menu (in an object''s halo menu) to change the designation of any object.  For example, if you have an object that is private to just one card, and you want to make it visible on all cards, use "place onto background".  If you further want it to hold a separate value for each separate card, use "start holding separate data for each instance".

The normal sequence to define a Stack''s structure is to obtain a blank Stack, then create your fields by grabbing what you want from the Stack Tools flap and dropping it where you want it in the stack.  For easiest use, give a name to each field (by editing the name in its halo) *before* you put it onto the background..  Those fields that you want to represent the basic data of the stack need to be given names, placed on the background, and then told to hold separate data.

When you hit the + button in a stack''s controls, a new card is created with default values in all the fields.  You can arrange for a particular default value to be used in a field -- do this either for one field at a time with "be default value on new card", or you can request that the all the values seen on a particular card serve as default by choosing "be defaults for new cards" from the stack''s  menu.

It is also possible to have multiple "backgrounds" in the same stack -- each different background defines a different data structure, and cards from multiple backgrounds can be freely mixed in the same stack.

Besides text fields, it is also possible to have picture-valued fields -- and potentially fields with data values of any other type as well.')

	embeddedInMorphicWindowLabeled: 'Stack Help'

	"StackMorph stackHelpWindow"! !

!StackMorph classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 12:40'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!StackMorph methodsFor: 'menu' stamp: 'dgd 8/30/2003 21:14'!
addBookMenuItemsTo: aMenu hand: aHandMorph
	"Add book-related items to the given menu"

	| controlsShowing subMenu |
	subMenu _ MenuMorph new defaultTarget: self.
	subMenu add: 'previous card' translated action: #goToPreviousCardInStack.
	subMenu add: 'next card' translated action: #goToNextCardInStack.
	subMenu add: 'go to card...' translated action: #goToCard.
	subMenu add: 'insert a card' translated action: #insertCard.
	subMenu add: 'delete this card' translated action: #deleteCard.

	controlsShowing _ self hasSubmorphWithProperty: #pageControl.
	controlsShowing
		ifTrue:
			[subMenu add: 'hide card controls' translated action: #hidePageControls.
			subMenu add: 'fewer card controls' translated action: #fewerPageControls]
		ifFalse:
			[subMenu add: 'show card controls' translated action: #showPageControls].

	subMenu addLine.
	subMenu add: 'sound effect for all backgrounds' translated action: #menuPageSoundForAll:.
	subMenu add: 'sound effect this background only' translated action: #menuPageSoundForThisPage:.
	subMenu add: 'visual effect for all backgrounds' translated action: #menuPageVisualForAll:.
	subMenu add: 'visual effect this background only' translated action: #menuPageVisualForThisPage:.

	subMenu addLine.
	subMenu add: 'sort pages' translated action: #sortPages:.
	subMenu add: 'uncache page sorter' translated action: #uncachePageSorter.
	(self hasProperty: #dontWrapAtEnd)
		ifTrue: [subMenu add: 'wrap after last page' translated selector: #setWrapPages: argument: true]
		ifFalse: [subMenu add: 'stop at last page' translated selector: #setWrapPages: argument: false].

	subMenu  addUpdating: #showingFullScreenString action: #toggleFullScreen.
	subMenu addLine.
	subMenu add: 'search for text' translated action: #textSearch.
	(self primaryHand pasteBuffer class isKindOf: PasteUpMorph class) ifTrue:
		[subMenu add: 'paste book page' translated action: #pasteBookPage].

	subMenu add: 'send all pages to server' translated action: #savePagesOnURL.
	subMenu add: 'send this page to server' translated action: #saveOneOnURL.
	subMenu add: 'reload all from server' translated action: #reload.
	subMenu add: 'copy page url to clipboard' translated action: #copyUrl.
	subMenu add: 'keep in one file' translated action: #keepTogether.
	subMenu add: 'save as new-page prototype' translated action: #setNewPagePrototype.
	newPagePrototype ifNotNil:
		[subMenu add: 'clear new-page prototype' translated action: #clearNewPagePrototype].

	aMenu add: 'book...' translated subMenu: subMenu
! !

!StackMorph methodsFor: 'background' stamp: 'nb 6/17/2003 12:25'!
addCardsFromAFile
	"Using the current background, create new cards by reading in data from a fileThe data are in each record are expected to be tab-delimited, and to occur in the same order as the instance variables of the current-background's cards "

	| aFileStream |
	(aFileStream _ FileList2 modalFileSelector) ifNil: [^ Beeper beep].
	self addCardsFromString: aFileStream contentsOfEntireFile.
	aFileStream close! !

!StackMorph methodsFor: 'background' stamp: 'nb 6/17/2003 12:25'!
addCardsFromClipboardData
	"Using the current background, paste data from the (textual) clipboard to create new records.  The data are in each record are expected to be tab-delimited, and to occur in the same order as the instance variables of the current-background's cards "

	| clip |
	(clip _ Clipboard clipboardText) isEmptyOrNil ifTrue: [^ Beeper beep].
	self addCardsFromString: clip! !

!StackMorph methodsFor: 'background' stamp: 'nb 6/17/2003 12:25'!
addCardsFromClipboardDataForInstanceVariables: slotNames
	"Using the current background, paste data from the (textual) clipboard to create new records.  No senders, but can be usefully called manually for selectively bringing in data in oddball format."

	| clip |
	(clip _ Clipboard clipboardText) isEmptyOrNil ifTrue: [^ Beeper beep].
	self addCardsFromString: clip slotNames: slotNames! !

!StackMorph methodsFor: 'background' stamp: 'nb 6/17/2003 12:25'!
addCardsFromFile: fileStream
	"Using the current background, take tab delimited data from the file to create new records."

	| aString |
	(aString _ fileStream contentsOfEntireFile) isEmptyOrNil ifTrue: [^ Beeper beep].
	self addCardsFromString: aString! !

!StackMorph methodsFor: 'background' stamp: 'sw 12/18/2001 11:35'!
addCardsFromString: aString
	"Using the current background, add cards from a string, which is expected be tab- and return-delimited.  The data are in each record are expected to be tab-delimited, and to occur in the same order as the instance variables of the current-background's cards "

	self addCardsFromString: aString slotNames: self currentCard slotNames
 
! !

!StackMorph methodsFor: 'background' stamp: 'em 3/31/2005 10:23'!
addCardsFromString: aString slotNames: slotNames 
	"Using the current background, add cards from a string, which is expected be tab- and return-delimited"

	| count |
	count := 0.
	aString asString linesDo: 
			[:aLine | 
			aLine notEmpty 
				ifTrue: 
					[count := count + 1.
					self 
						insertCardOfBackground: self currentPage
						withDataFrom: aLine
						forInstanceVariables: slotNames]].
	self inform: count asString , ' card(s) added' translated! !

!StackMorph methodsFor: 'page controls' stamp: 'sw 6/6/2003 13:59'!
addPageControlMorph: aMorph
	"Add the given morph as a page-control, at the appropriate place"

	aMorph setProperty: #pageControl toValue: true.
	self addPane: aMorph paneType: #pageControl! !

!StackMorph methodsFor: 'initialization' stamp: 'sw 6/5/2003 04:04'!
addPane: aPane paneType: aType
	| anIndex |
	anIndex _ self insertionIndexForPaneOfType: aType.
	self privateAddMorph: aPane atIndex: anIndex! !

!StackMorph methodsFor: 'submorphs-accessing' stamp: 'sw 3/18/2002 02:20'!
allNonSubmorphMorphs
	"Return a collection containing all morphs in this morph which are not currently in the submorph containment hierarchy.  Especially the non-showing pages in BookMorphs."

	| coll |
	coll _ OrderedCollection new.
	self privateCards do: [:cd | 
		cd privateMorphs ifNotNil: [coll addAll: cd privateMorphs]].
	^ coll! !

!StackMorph methodsFor: 'background' stamp: 'sw 10/26/2000 14:41'!
backgroundWithCard: aCard
	"Answer the background which contains aCard."

	^ self backgrounds detect:
		[:aBackground | aBackground containsCard: aCard] ifNone: [nil]! !

!StackMorph methodsFor: 'background' stamp: 'sw 10/30/2000 10:04'!
backgrounds
	"Answer the list of backgrounds available in the receiver"

	^ self pages! !

!StackMorph methodsFor: 'background' stamp: 'sw 12/6/2001 21:26'!
beDefaultsForNewCards
	"Make the values that I see here all be accepted as defaults for new cards"

	self currentPage submorphs do:
		[:aMorph | aMorph holdsSeparateDataForEachInstance ifTrue:
			[aMorph setAsDefaultValueForNewCard]]! !

!StackMorph methodsFor: 'card access' stamp: 'sw 10/23/2000 16:24'!
browseCardClass
	"Browse the class of the current card"

	| suffix |
	suffix _ self currentCard class name numericSuffix.
	HierarchyBrowser newFor: self currentCard class labeled: 'Background ', suffix asString
! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 02:20'!
cardIndexOf: aCard
	"Answer the ordinal position of aCard in the receiver's list"

	^ self privateCards indexOf: aCard ifAbsent: [nil]! !

!StackMorph methodsFor: 'accessing' stamp: 'sw 11/2/2002 15:51'!
cardNumberOf: aPlayer
	"Answer the card-number of the given player, in the which-card-of-the-stack sense."

	^ self cards identityIndexOf: aPlayer ifAbsent: [0]! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 02:12'!
cards
	"Answer a list of the cards of the receiver, in order"

	^ self privateCards copy! !

!StackMorph methodsFor: 'accessing' stamp: 'sw 3/18/2002 02:09'!
cardsOrPages
	"The turnable and printable entities"

	^ self cards! !

!StackMorph methodsFor: 'background' stamp: 'sw 11/2/2002 17:56'!
changeInstVarOrder
	"Change the order of the receiver's instance variables"

	| reply |
	reply _ FillInTheBlank request: 'rearrange, then accept; or cancel' initialAnswer:
		((self currentPage player class instVarNames asArray collect: [:v | v asSymbol]) storeString copyWithoutAll: #($# $( $))) asString.
	reply isEmptyOrNil ifTrue: [^ self].
	self flag: #deferred.  "Error checking and graceful escape wanted"
	self currentPage player class resortInstanceVariables: (Compiler evaluate:
		('#(', reply, ')'))! !

!StackMorph methodsFor: 'as yet unclassified' stamp: 'sw 10/23/2000 16:27'!
commitCardData
	"Make certain that the player data are written back to the player instance"

	^ self currentCard commitCardPlayerData 
! !

!StackMorph methodsFor: 'card access' stamp: 'sw 10/23/2000 16:27'!
currentCard
	"Answer the current card of the current background of the receiver"

	^ currentPage currentDataInstance! !

!StackMorph methodsFor: 'insert and delete' stamp: 'sw 10/30/2000 10:10'!
defaultNameStemForNewPages
	"Answer the stem to use as the default for names of cards in the stack"

	^ 'card'
! !

!StackMorph methodsFor: 'card access' stamp: 'em 3/31/2005 10:24'!
deleteAllCardsExceptThisOne
	"Delete all cards except the current one"

	self privateCards size <= 1 ifTrue: [^ Beeper beep].
	(self confirm: 'Really delete ' translated, self privateCards size asString, ' card(s) and all of their data?' translated) ifTrue:
		[self privateCards: (OrderedCollection with: self currentCard)].! !

!StackMorph methodsFor: 'card access' stamp: 'sd 11/13/2003 21:03'!
deleteCard
	"Delete the current card from the stack"

	| aCard |
	aCard _ self currentCard.
	self privateCards size = 1 ifTrue: [^ Beeper beep].
	(self confirm: 'Really delete this card and all of its data?' translated) ifTrue:
		[self goToNextCardInStack.
		self privateCards remove: aCard].! !

!StackMorph methodsFor: 'card access' stamp: 'nb 6/17/2003 12:25'!
deleteCard: aCard
	"Delete the current card from the stack."

	self privateCards size = 1 ifTrue: [^ Beeper beep].
	(aCard == self currentCard) ifTrue: [^ self deleteCard].

	self privateCards remove: aCard ifAbsent: []! !

!StackMorph methodsFor: 'card in a stack' stamp: 'sw 10/30/2000 10:03'!
explainDesignations
	"Give the user an explanation of what the designations mean"

	self currentPage explainDesignations
! !

!StackMorph methodsFor: 'menu' stamp: 'sw 3/18/2002 02:06'!
findText: wants
	"Turn to the next card that has all of the strings mentioned on it.  Highlight where it is found.  allText and allTextUrls have been set.  Case insensitive search.
	Resuming a search.  If container's text is still in the list and secondary keys are still in the page, (1) search rest of that container.  (2) search rest of containers on that page (3) pages till end of book, (4) from page 1 to this page again."

	"Later sort wants so longest key is first"
	| allText good thisWord here fromHereOn startToHere oldContainer oldIndex otherKeys strings |
	allText _ self valueOfProperty: #allText ifAbsent: [#()].
	here _ self privateCards identityIndexOf: self currentCard ifAbsent: [1].
	fromHereOn _ here+1 to: self privateCards size.
	startToHere _ 1 to: here.		"repeat this page"
	(self valueOfProperty: #searchKey ifAbsent: [#()]) = wants ifTrue: [
		"does page have all the other keys?  No highlight if found!!"
		otherKeys _ wants allButFirst.
		strings _ allText at: here.
		good _ true.
		otherKeys do: [:searchString | "each key"
			good ifTrue: [thisWord _ false.
				strings do: [:longString |
					(longString findWordStart: searchString startingAt: 1) > 0 ifTrue: [
							thisWord _ true]].
				good _ thisWord]].
		good ifTrue: ["all are on this page.  Look in rest for string again."
			oldContainer _ self valueOfProperty: #searchContainer.
			oldIndex _ self valueOfProperty: #searchOffset.
			(self findText: (OrderedCollection with: wants first) inStrings: strings	
				startAt: oldIndex+1 container: oldContainer 
				cardNum: here) ifTrue: [
					self setProperty: #searchKey toValue: wants.
					^ true]]]
		ifFalse: [fromHereOn _ here to: self privateCards size].	"do search this page"
	"other pages"
	fromHereOn do: [:cardNum |
		(self findText: wants inStrings: (allText at: cardNum) startAt: 1 container: nil 
				cardNum: cardNum) 
					ifTrue: [^ true]].
	startToHere do: [:cardNum |
		(self findText: wants inStrings: (allText at: cardNum) startAt: 1 container: nil 
				cardNum: cardNum) 
					ifTrue: [^ true]].
	"if fail"
	self setProperty: #searchContainer toValue: nil.
	self setProperty: #searchOffset toValue: nil.
	self setProperty: #searchKey toValue: nil.
	^ false! !

!StackMorph methodsFor: 'menu' stamp: 'gm 2/22/2003 13:13'!
findText: keys inStrings: rawStrings startAt: startIndex container: oldContainer cardNum: cardNum 
	"Call once to search a card of the stack.  Return true if found and highlight the text.  oldContainer should be NIL.  
	(oldContainer is only non-nil when (1) doing a 'search again' and (2) the page is in memory and (3) keys has just one element.  oldContainer is a TextMorph.)"

	| good thisWord index insideOf place container start strings old |
	good := true.
	start := startIndex.
	strings := oldContainer ifNil: 
					["normal case"

					rawStrings]
				ifNotNil: [self currentPage allStringsAfter: oldContainer text].
	keys do: 
			[:searchString | 
			"each key"

			good 
				ifTrue: 
					[thisWord := false.
					strings do: 
							[:longString | 
							(index := longString findWordStart: searchString startingAt: start) > 0 
								ifTrue: 
									[thisWord not & (searchString == keys first) 
										ifTrue: 
											[insideOf := longString.
											place := index].
									thisWord := true].
							start := 1].	"only first key on first container"
					good := thisWord]].
	good 
		ifTrue: 
			["all are on this page"

			"wasIn _ (pages at: pageNum) isInMemory."

			self goToCardNumber: cardNum
			"wasIn ifFalse: ['search again, on the real current text.  Know page is in.'.
			^ self findText: keys 
				inStrings: ((pages at: pageNum) allStringsAfter: nil)         recompute it	
				startAt: startIndex container: oldContainer 
				pageNum: pageNum]"].
	(old := self valueOfProperty: #searchContainer) ifNotNil: 
			[(old respondsTo: #editor) 
				ifTrue: 
					[old editor selectFrom: 1 to: 0.	"trying to remove the previous selection!!"
					old changed]].
	good 
		ifTrue: 
			["have the exact string object"

			(container := oldContainer) ifNil: 
					[container := self 
								highlightText: keys first
								at: place
								in: insideOf]
				ifNotNil: 
					[container userString == insideOf 
						ifFalse: 
							[container := self 
										highlightText: keys first
										at: place
										in: insideOf]
						ifTrue: 
							[(container isTextMorph) 
								ifTrue: 
									[container editor selectFrom: place to: keys first size - 1 + place.
									container changed]]].
			self setProperty: #searchContainer toValue: container.
			self setProperty: #searchOffset toValue: place.
			self setProperty: #searchKey toValue: keys.	"override later"
			ActiveHand newKeyboardFocus: container.
			^true].
	^false! !

!StackMorph methodsFor: 'menu' stamp: 'em 3/31/2005 10:20'!
findViaTemplate
	| list pl cardInst |
	"Current card is the template.  Only search cards in this background. Look at cards directly (not allText). Key must be found in the same field as in the template.  HyperCard style (multiple starts of words).  
	Put results in a list, outside the stack."

	list _ self templateMatches.
	list isEmpty ifTrue: [^ self inform: 'No matches were found.
Be sure the current card is mostly blank
and only has text you want to match.' translated]. 
	"put up a PluggableListMorph"
	cardInst _ self currentCard.
	cardInst matchIndex: 0.	"establish entries"
	cardInst results at: 1 put: list.
	self currentPage setProperty: #myStack toValue: self.	"way to get back"

	pl _ PluggableListMorph new
			on: cardInst list: #matchNames
			selected: #matchIndex changeSelected: #matchIndex:
			menu: nil "#matchMenu:shifted:" keystroke: nil.
	ActiveHand attachMorph: (self formatList: pl).
! !

!StackMorph methodsFor: 'menu' stamp: 'tk 6/2/2001 11:40'!
formatList: pl
	| rr ff |
	"Turn this plugglable list into a good looking morph."

	pl color: Color transparent; borderWidth: 0.
	pl font: ((TextStyle named: #Palatino) fontOfSize: 14).
	pl toggleCornerRounding; width: 252; retractableOrNot; hResizing: #spaceFill.
	rr _ (RectangleMorph new) toggleCornerRounding; extent: pl extent + (30@30).
	rr color: self currentPage color; fillStyle: (ff _ self currentPage fillStyle copy).
	ff isGradientFill ifTrue: [
		rr fillStyle direction: (ff direction * self currentPage extent / rr extent) rounded.
		rr fillStyle origin: rr bounds origin].
	rr addMorph: pl.
	rr layoutPolicy: TableLayout new.
	rr layoutInset: 10@15; cellInset: 10@15; wrapDirection: #leftToRight.
	rr listCentering: #center; borderWidth: 5; borderColor: #raised.
	"Up and down buttons on left with arrows in a holder."
	"lb _ (RectangleMorph new) color: transparent; borderWidth: 0."
	^ rr! !

!StackMorph methodsFor: 'page controls' stamp: 'tk 11/5/2001 08:21'!
fullControlSpecs
	"Answer specifications for the long form of iconic stack/book controls"

	^ #(
		spacer
		variableSpacer
		('-'			deleteCard					'Delete this card')
		spacer
		( ''		goToFirstCardOfStack			'First card')
		spacer
		( '<' 		goToPreviousCardInStack		'Previous card')
		spacer
		(''			invokeBookMenu 			'Click here to get a menu of options for this stack.')
		"spacer	(''			reshapeBackground  		'Reshape')	"

		spacer
		(''			showDesignationsOfObjects 	'Show designations')
		spacer
		('>'			goToNextCardInStack			'Next card')
		spacer
		( ''		goToLastCardOfStack			'Final card')
		spacer
		('+'			insertCard					'Add a new card after this one')
		variableSpacer
		(''			fewerPageControls			'Fewer controls
(if shift key pressed,
deletes controls)')
)! !

!StackMorph methodsFor: 'menu' stamp: 'sw 3/18/2002 02:07'!
getAllText
	"Collect the text for each card.  Just point at strings so don't have to recopy them.  (Parallel array of urls for ID of cards.  Remote cards not working yet.)
	allText = Array (cards size) of arrays (fields in it) of strings of text.
	allTextUrls = Array (cards size) of urls or card numbers."

	| oldUrls oldStringLists allText allTextUrls aUrl which |
	self writeSingletonData.
	oldUrls _ self valueOfProperty: #allTextUrls ifAbsent: [#()].
	oldStringLists _ self valueOfProperty: #allText ifAbsent: [#()].
	allText _ self privateCards collect: [:pg | OrderedCollection new].
	allTextUrls _ Array new: self privateCards size.
	self privateCards doWithIndex: [:aCard :ind | aUrl _ aCard url.  aCard isInMemory 
		ifTrue: [(allText at: ind) addAll: (aCard allStringsAfter: nil).
			aUrl ifNil: [aUrl _ ind].
			allTextUrls at: ind put: aUrl]
		ifFalse: ["Order of cards on server may be different.  (later keep up to date?)"
			"*** bug in this algorithm if delete a page?"
			which _ oldUrls indexOf: aUrl.
			allTextUrls at: ind put: aUrl.
			which = 0 ifFalse: [allText at: ind put: (oldStringLists at: which)]]].
	self setProperty: #allText toValue: allText.
	self setProperty: #allTextUrls toValue: allTextUrls.
	^ allText! !

!StackMorph methodsFor: 'card access' stamp: 'em 3/31/2005 10:23'!
goToCard
	"prompt the user for an ordinal number, and use that as a basis for choosing a new card to install in the receiver"

	| reply index |
	reply _ FillInTheBlank request: 'Which card number? ' translated initialAnswer: '1'.
	reply isEmptyOrNil ifTrue: [^ self].
	((index _ reply asNumber) > 0 and: [index <= self privateCards size])
		ifFalse: [^ self inform: 'no such card'].
	self goToCard: (self privateCards at: index)! !

!StackMorph methodsFor: 'card access' stamp: 'sw 11/8/2002 15:15'!
goToCard: destinationCard
	"Install the indicated destinationCard as the current card in the receiver.  Any viewer currently open on the current card will get retargeted to look at the new one."

	| aBackground existingCard oldViewers |
	destinationCard == self currentCard ifTrue: [^ self].
	self currentPlayerDo:
		[:aPlayer | aPlayer runAllClosingScripts].   "Like HyperCard 'on closeCard'"

	aBackground _ self backgroundWithCard: destinationCard.
	existingCard _ aBackground currentDataInstance.
	oldViewers _ existingCard ifNil: [#()] ifNotNil: [existingCard allOpenViewers].

	aBackground installAsCurrent: destinationCard.
	aBackground setProperty: #myStack toValue: self.	"pointer cardMorph -> stack"

	aBackground ~~ currentPage ifTrue:
		[self goToPageMorph: aBackground runTransitionScripts: false].
	self currentPlayerDo:
		[:aPlayer | aPlayer runAllOpeningScripts] .  "Like HyperCard 'on opencard'"

	oldViewers do: [:aViewer | aViewer retargetFrom: existingCard to: destinationCard]! !

!StackMorph methodsFor: 'card access' stamp: 'sw 11/11/2002 03:07'!
goToCardNumber: aCardNumber
	"Install the card whose ordinal number is provided as the current card in the stack"

	self goToCard: (self privateCards atWrap: aCardNumber)! !

!StackMorph methodsFor: 'card access' stamp: 'nb 6/17/2003 12:25'!
goToFirstCardInBackground
	"Install the initial card in the current background as the current card in the stack"

	| kind |
	kind _ currentPage player class baseUniclass.
	self goToCard: (self privateCards detect: [:aCard | aCard isKindOf: kind] ifNone: [^ Beeper beep])! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 02:01'!
goToFirstCardOfStack
	"Install the initial card in the stack as the current card"

	self goToCard: self privateCards first! !

!StackMorph methodsFor: 'card access' stamp: 'nb 6/17/2003 12:25'!
goToLastCardInBackground
	"Install the final card in the current background as the current card"

	| kind |
	kind _ currentPage player class baseUniclass.
	self goToCard: (self privateCards reversed detect: [:aCard | aCard isKindOf: kind] ifNone: [^ Beeper beep])! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 02:09'!
goToLastCardOfStack
	"Install the final card in the stack as the current card"

	self goToCard: self privateCards last! !

!StackMorph methodsFor: 'card in a stack' stamp: 'sw 3/18/2002 02:03'!
goToNextCardInStack
	"Make the card *after* the current card become the current card"

	| anIndex newCard |
	anIndex _ self privateCards indexOf: currentPage currentDataInstance.
	newCard _ self privateCards atWrap: anIndex + 1.
	self goToCard: newCard! !

!StackMorph methodsFor: 'card in a stack' stamp: 'sw 3/18/2002 02:01'!
goToPreviousCardInStack
	"Install the previous card as my current one"

	| anIndex newCard |
	anIndex _ self privateCards indexOf: currentPage currentDataInstance.
	newCard _ self privateCards atWrap: anIndex - 1.
	self goToCard: newCard! !

!StackMorph methodsFor: 'initialization' stamp: 'sw 3/18/2002 02:12'!
initialize
	"Initialize the stack"

	| initialBackground |
	super initialize.
	initialBackground _ pages first.
	initialBackground extent: (640@480); beSticky.
	initialBackground beAStackBackground.
	self beUnsticky.
	self setProperty: #controlsAtBottom toValue: true.
	self privateCards: (OrderedCollection with: initialBackground currentDataInstance).

"self currentHand attachMorph: StackMorph authoringPrototype"! !

!StackMorph methodsFor: 'parts bin' stamp: 'sw 8/2/2001 18:14'!
initializeToStandAlone
	
	self initialize.
	self pageSize: (480 @ 320); color: (Color gray: 0.7).
	self borderWidth: 1; borderColor: Color black.
	self currentPage extent: self pageSize.
	self showPageControls: self fullControlSpecs.
	^ self

"StackMorph initializedInstance openInHand"! !

!StackMorph methodsFor: 'initialization' stamp: 'sw 3/18/2002 02:13'!
initializeWith: aCardMorph
	"Install the card inside a new stack.  Make no border or controls, so I the card's look is unchanged.  Card already has a CardPlayer."
	
	| wld |
	wld _ aCardMorph world.
	self initialize.
	self pageSize: aCardMorph extent.
	self borderWidth: 0; layoutInset: 0; color: Color transparent.
	pages _ Array with: aCardMorph.
	currentPage _ aCardMorph.
	self privateCards: (OrderedCollection with: currentPage currentDataInstance).
	currentPage beAStackBackground.
	self position: aCardMorph position.
	submorphs last delete.
	self addMorph: currentPage.	
	self showPageControls: self fullControlSpecs.
	wld addMorph: self.
! !

!StackMorph methodsFor: 'background' stamp: 'em 3/30/2005 14:47'!
insertAsBackground: newPage resize: doResize
	"Make a new background for the stack.  Obtain a name for it from the user.  It starts out life empty"

	| aName |
	aName _ FillInTheBlank request: 'What should we call this new background?' translated initialAnswer: 'alternateBackground' translated.
	aName isEmptyOrNil ifTrue: [^ self].
	newPage beSticky.
	doResize ifTrue: [newPage extent: currentPage extent].
	newPage beAStackBackground.
	newPage setNameTo: aName.
	newPage vResizeToFit: false.
	pages isEmpty
		ifTrue: [pages add: newPage]
		ifFalse: [pages add: newPage after: currentPage].
	self privateCards add: newPage currentDataInstance after: currentPage currentDataInstance.
	self nextPage.
! !

!StackMorph methodsFor: 'card in a stack' stamp: 'sw 10/23/2000 16:02'!
insertCard
	"Create a new card of the current background and make it become the current card"

	self insertCardOfBackground: currentPage! !

!StackMorph methodsFor: 'as yet unclassified' stamp: 'em 3/31/2005 10:56'!
insertCardOfBackground
	"Prompt the user for choice of a background, and insert a new card of that background"

	| bgs aMenu aBackground |
	(bgs _ self backgrounds) size == 1 ifTrue:
		[self inform: 
'At this time, there IS only one kind of
background in this stack, so that''s
what you''ll get' translated.
		^ self insertCard].
	aMenu _ SelectionMenu
		labels: 		(bgs collect: [:bg | bg externalName])
		selections: 	bgs.
	(aBackground _ aMenu startUp) ifNotNil:
		[self insertCardOfBackground: aBackground]! !

!StackMorph methodsFor: 'as yet unclassified' stamp: 'sw 3/18/2002 02:02'!
insertCardOfBackground: aBackground
	"Insert a new card of the given background and have it become the current card"

	| newCard |
	newCard _  aBackground newCard.
	self privateCards add: newCard after: self currentCard.
	self goToCard: newCard! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 02:03'!
insertCardOfBackground: aBackground withDataFrom: aLine forInstanceVariables: slotNames
	"Insert a new card of the given background and have it become the current card. "

	| newCard |
	newCard _  aBackground newCard.
	self privateCards add: newCard after: self currentCard.
	newCard absorbBackgroundDataFrom: aLine forInstanceVariables: slotNames.
	self goToCard: newCard! !

!StackMorph methodsFor: 'submorphs-accessing' stamp: 'sw 6/5/2003 04:01'!
insertionIndexForPaneOfType: aType
	| naturalIndex insertionIndex |
	naturalIndex _ self naturalPaneOrder indexOf: aType.
	insertionIndex _ 1.
	(self naturalPaneOrder copyFrom: 1 to: (naturalIndex - 1)) do: "guys that would precede"
		[:sym | (self hasSubmorphWithProperty: sym)
			ifTrue:
				[insertionIndex _ insertionIndex + 1]].
	^ insertionIndex! !

!StackMorph methodsFor: 'debugging' stamp: 'sw 10/30/2000 10:13'!
inspectCurrentBackground
	"Open an inspector on the corrent background.  Ideally should put include the background name in the inspector's title."

	^ self currentPage inspectWithLabel: 'A Background'
! !

!StackMorph methodsFor: 'debugging' stamp: 'sw 10/23/2000 16:37'!
inspectCurrentCard
	"For debugging: open an Inspector on the receiver's current card"

	^ self currentCard inspectWithLabel: 'A Card'
! !

!StackMorph methodsFor: 'debugging' stamp: 'sw 10/30/2000 10:09'!
inspectCurrentStack
	"Triggered from the stack-debug menu, open an Inspector on the receiver"

	^ self inspectWithLabel: 'A Stack'
! !

!StackMorph methodsFor: 'menu' stamp: 'dgd 9/29/2004 20:47'!
invokeBookMenu
	"Invoke the book's control panel menu."

	| aMenu |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Stack' translated.
	Preferences noviceMode
		ifFalse: [aMenu addStayUpItem].
	aMenu addList: {
		{'find...' translated.					#textSearch}.
		{'find via this template' translated.			#findViaTemplate}.
		{'show designations' translated. 			#showDesignationsOfObjects}.
		{'explain designations' translated.			#explainDesignations}.
		#-.
		{'previous card' translated. 				#goToPreviousCardInStack}.
		{'next card' translated. 				#goToNextCardInStack}.
		{'first card' translated. 				#goToFirstCardOfStack}.
		{'last card' translated. 				#goToLastCardOfStack}.
		{'go to card...' translated. 				#goToCard}.
		#-.
		{'add a card of this background' translated. 		#insertCard}.
		{'add a card of background...' translated.		#insertCardOfBackground}.
		{'make a new background...' translated. 		#makeNewBackground}.
		#-.
		{'insert cards from clipboard data' translated.		#addCardsFromClipboardData.	'Create new cards from a formatted string on the clipboard' translated}.
		{'insert cards from a file...' translated.		#addCardsFromAFile.		'Create new cards from data in a file' translated}.
		#-.
		{'instance variable order...' translated.		#changeInstVarOrder.		'Caution -- DANGER. Change the order of the variables on the cards' translated}.
		{'be defaults for new cards' translated. 		#beDefaultsForNewCards.		'Make these current field values be the defaults for their respective fields on new cards' translated}.
		    {'sort cards by...' translated.			#sortCards.			'Sort all the cards of the current background using some field as the sort key' translated}.
		#-.
		{'delete this card' translated. 			#deleteCard}.
		{'delete all cards *except* this one' translated.	#deleteAllCardsExceptThisOne}.
		#-.
		{'move card to front of stack' translated.		#makeCurrentCardFirstInStack}.
		{'move card to back of stack' translated.		#makeCurrentCardLastInStack}.
		{'move card one position earlier' translated.		#moveCardOnePositionEarlier}.
		{'move card one position later' translated.		#moveCardOnePositionLater}.
		#-.
		{'scripts for this background' translated.		#browseCardClass}.
		#-.
		{'debug...' translated.					#offerStackDebugMenu}.
		{'bookish items...' translated. 			#offerBookishMenu}}.

	aMenu addUpdating: #showingPageControlsString action: #toggleShowingOfPageControls.
	aMenu addUpdating: #showingFullScreenString action: #toggleFullScreen.

	aMenu popUpEvent: self world activeHand lastEvent in: self world
! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 01:57'!
makeCurrentCardFirstInStack
	"Move the current card such that it becomes the first card in the stack"

	| aCard |
	aCard _ self currentCard.
	self privateCards remove: aCard ifAbsent: [];
		addFirst: aCard.
	self currentPage flash! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 02:03'!
makeCurrentCardLastInStack
	"Move the current card such that it becomes the last card in the stack"

	| aCard |
	aCard _ self currentCard.
	self privateCards remove: aCard ifAbsent: [];
		addLast: aCard.
	self currentPage flash! !

!StackMorph methodsFor: 'background' stamp: 'tk 10/30/2001 19:01'!
makeNewBackground
	"Make a new background for the stack.  Obtain a name for it from the user.  It starts out life empty"

	| newPage |
	(newPage _ PasteUpMorph newSticky) color: self color muchLighter.
	newPage borderWidth: currentPage borderWidth; borderColor: currentPage borderColor.
	self insertAsBackground: newPage resize: true. 
! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 02:08'!
moveCardOnePositionEarlier
	"Move the current card such that its ordinal position is one fewer than it formerly was.  If the current card is already the first one one in the stack, then do nothing"

	| aCard aPosition |
	aCard _ self currentCard.
	aCard == self privateCards first ifTrue: [^ self].
	aPosition _ self privateCards indexOf: aCard.
	self privateCards remove: aCard;
		add: aCard afterIndex: (aPosition - 2).
	self currentPage flash! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 02:05'!
moveCardOnePositionLater
	"Move the current card such that its ordinal position is one greater than it formerly was.  If the current card is already the last one one in the stack, then do nothing"

	| aCard aPosition privateCards |
	aCard _ self currentCard.
	privateCards _ self privateCards.
	aCard == privateCards last ifTrue: [^ self].
	aPosition _ privateCards indexOf: aCard.
	privateCards remove: aCard.
	privateCards add: aCard afterIndex: aPosition.
	self currentPage flash! !

!StackMorph methodsFor: 'submorphs-accessing' stamp: 'sw 6/5/2003 04:02'!
naturalPaneOrder
	^ #(header pageControl retrieve search index content)! !

!StackMorph methodsFor: 'menu' stamp: 'dgd 4/4/2006 16:49'!
offerBookishMenu
	"Offer a menu with book-related items in it"

	| aMenu |
	aMenu := MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Stack / Book' translated.
	Preferences noviceMode
		ifFalse: [aMenu addStayUpItem].
	aMenu addList:
		#(('sort pages' sortPages)
		('uncache page sorter' uncachePageSorter)).
	(self hasProperty: #dontWrapAtEnd)
		ifTrue: [aMenu add: 'wrap after last page' translated selector: #setWrapPages: argument: true]
		ifFalse: [aMenu add: 'stop at last page' translated selector: #setWrapPages: argument: false].
	aMenu addList:
		#(('make bookmark'	 bookmarkForThisPage)
		('make thumbnail' thumbnailForThisPage)).

	aMenu addLine.
	aMenu add: 'sound effect for all pages' translated action: #menuPageSoundForAll:.
	aMenu add: 'sound effect this page only' translated action: #menuPageSoundForThisPage:.
	aMenu add: 'visual effect for all pages' translated action: #menuPageVisualForAll:.
	aMenu add: 'visual effect this page only' translated action: #menuPageVisualForThisPage:.

	aMenu addLine.
	(self primaryHand pasteBuffer class isKindOf: PasteUpMorph class) ifTrue:
		[aMenu add: 'paste book page'   translated action: #pasteBookPage].

	aMenu add: 'save as new-page prototype' translated action: #setNewPagePrototype.
	newPagePrototype ifNotNil: [
		aMenu add: 'clear new-page prototype' translated action: #clearNewPagePrototype].

	aMenu add: (self dragNDropEnabled ifTrue: ['close' translated ] ifFalse: ['open' translated]) , ' dragNdrop' translated
			action: #toggleDragNDrop.
	aMenu addLine.
	aMenu add: 'make all pages this size' translated action: #makeUniformPageSize.
	aMenu addUpdating: #keepingUniformPageSizeString target: self action: #toggleMaintainUniformPageSize.
	aMenu addLine.
	aMenu add: 'send all pages to server' translated action: #savePagesOnURL.
	aMenu add: 'send this page to server' translated action: #saveOneOnURL.
	aMenu add: 'reload all from server' translated action: #reload.
	aMenu add: 'copy page url to clipboard' translated action: #copyUrl.
	aMenu add: 'keep in one file' translated action: #keepTogether.

	aMenu addLine.
	aMenu add: 'load PPT images from slide #1' translated action: #loadImagesIntoBook.
	aMenu add: 'background color for all pages...' translated action: #setPageColor.

	aMenu popUpEvent: self world activeHand lastEvent in: self world
! !

!StackMorph methodsFor: 'menu' stamp: 'dgd 9/29/2004 20:47'!
offerStackDebugMenu
	"Put up a menu offering debugging items for the stack"

	| aMenu |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Stack debugging'.
	Preferences noviceMode
		ifFalse: [aMenu addStayUpItem].
	aMenu addList: #(
		('reassess'								reassessBackgroundShape)
		('relax grip on variable names'			relaxGripOnVariableNames)
		('commit card data'						commitCardData)
		-
		('browse card uniclass'					browseCardClass)
		('inspect card'							inspectCurrentCard)
		('inspect background'					inspectCurrentBackground)
		('inspect stack'							inspectCurrentStack)).
	aMenu popUpInWorld: (self world ifNil: [self currentWorld])
! !

!StackMorph methodsFor: 'as yet unclassified' stamp: 'sw 10/30/2000 10:08'!
openInsideLook
	"Open an inside-look at the current page.  This is a previously-demoed feature not presently incorporated in released code,"

	true ifTrue: [self notYetImplemented] ifFalse: [self currentPage openInsideLook]
! !

!StackMorph methodsFor: 'controls' stamp: 'sw 10/30/2000 16:31'!
pageControlsMorphFrom: controlSpecs
	"Answer a controls morph derived from the spec supplied"

	| controls |
	controls _ super pageControlsMorphFrom: controlSpecs.
	controls eventHandler: nil.  "not grabbable"
	^ controls! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 01:56'!
privateCards
	"Private - answer the collection object that sits in my cards instance variable"

	^ cards! !

!StackMorph methodsFor: 'card access' stamp: 'sw 3/18/2002 02:51'!
privateCards: aCollection
	"Private - Make my cards be te given colllection"

	cards _ aCollection! !

!StackMorph methodsFor: 'card in a stack' stamp: 'tk 10/5/2001 06:27'!
reassessBackgroundShape
	"Have the current page reconsider its cards' instance structure"

	currentPage setProperty: #myStack toValue: self. 	"pointer cardMorph -> stack"
	^ self currentPage reassessBackgroundShape 
! !

!StackMorph methodsFor: 'card in a stack' stamp: 'sw 10/30/2000 10:04'!
relaxGripOnVariableNames
	"Have the current background relax its grip on existing variable name"

	^ self currentPage relaxGripOnVariableNames 
! !

!StackMorph methodsFor: 'card in a stack' stamp: 'sw 10/30/2000 10:15'!
reshapeBackground
	"Abandon any memory of variable-name preferences for the current background, and reassess its instance structure"

	^ self currentPage reshapeBackground 
! !

!StackMorph methodsFor: 'page controls' stamp: 'sw 10/30/2000 10:09'!
shortControlSpecs
	"Answer specficiations for the shorter form of stack controls"

	^ #(
		spacer
		variableSpacer
		( '<'		goToPreviousCardInStack		'Previous card')
		spacer
		(''		invokeBookMenu 			'Click here to get a menu for this stack.')
		spacer
		('>'		goToNextCardInStack			'Next card')
		variableSpacer
		(''	showMoreControls				'More controls
(if shift key pressed,
deletes controls)'))! !

!StackMorph methodsFor: 'card in a stack' stamp: 'sw 10/30/2000 10:10'!
showDesignationsOfObjects
	"Momentarily show which objects on the current card belong to which designation category"

	self currentPage showDesignationsOfObjects
! !

!StackMorph methodsFor: 'background' stamp: 'sw 3/18/2002 02:14'!
sortByField: varName
	"Perform a simple reordering of my cards, sorting by the given field name.  If there are multiple backgrounds, then sort the current one, placing all its cards first, followed by all others in unchanged order"

	| holdCards thisClassesInstances sortedList |
	holdCards _ self privateCards copy.

	thisClassesInstances _ self privateCards select: [:c | c isKindOf: self currentCard class].
	sortedList _ thisClassesInstances asSortedCollection:
		[:a :b | (a instVarNamed: varName) asString <= (b instVarNamed: varName) asString].
	sortedList _ sortedList asOrderedCollection.
	holdCards removeAllFoundIn: sortedList.
	self privateCards:  (sortedList asOrderedCollection, holdCards).
	self goToFirstCardOfStack
! !

!StackMorph methodsFor: 'background' stamp: 'sw 12/6/2001 22:08'!
sortCards
	"Let the user provide an inst var on which to sort the cards of a background."

	| names aMenu |
	names _ self currentPage player class instVarNames.
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Choose field by which to sort:'.
	names do: [:n | aMenu add: n selector: #sortByField: argument: n].
	aMenu popUpInWorld! !

!StackMorph methodsFor: 'card in a stack' stamp: 'sw 10/23/2000 17:37'!
stackDo: aBlock
	"Evaluate aBlock on behalf of the receiver stack"

	^ aBlock value: self! !

!StackMorph methodsFor: 'menu' stamp: 'sw 3/18/2002 01:58'!
templateMatches
	| template docks keys bkg |
	"Current card is the template.  Only search cards in this background. Look at cards directly (not allText). Key must be found in the same field as in the template.  HyperCard style (multiple starts of words).  
	Put results in a list, outside the stack."

	template _ self currentCard.
	template commitCardPlayerData.
	docks _ template class variableDocks.
	(keys _ template asKeys) ifNil: [^ #()]. "nothing to match against"
	bkg _ self currentPage.
	^ self privateCards select: [:cardPlayer | 
		(((cardPlayer == template) not) and: [cardPlayer costume == bkg]) 
			and: [cardPlayer match: keys fields: docks]].
! !

!StackMorph methodsFor: 'menu' stamp: 'sw 3/18/2002 02:00'!
writeSingletonData
	"Backgrounds that have just one card, may never get their data written into a CardPlayer. Make sure we do it."

	| sieve |
	sieve _ IdentityDictionary new.
	pages do: [:pp | sieve at: pp put: 0].
	self privateCards do: [:cc | sieve at: cc costume put: (sieve at: cc costume) + 1].
	sieve associationsDo: [:ass | 
		ass value = 1 ifTrue:
			[ass key player commitCardPlayerDataFrom: ass key]].
			"If currently showing card, may be some trouble... <- tk note 5/01"! !

!ProjectViewMorph methodsFor: '*eToys-user interface' stamp: 'RAA 7/10/2000 23:30'!
eToyStreamedRepresentationNotifying: aWidget

	| safeVariant outData |

	self flag: #bob.		"probably irrelevant"
	safeVariant _ self copy.
	[ outData _ SmartRefStream streamedRepresentationOf: safeVariant ] 
		on: ProgressInitiationException
		do: [ :ex | 
			ex sendNotificationsTo: [ :min :max :curr |
				aWidget ifNotNil: [aWidget flashIndicator: #working].
			].
		].
	^outData
! !

!MenuType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:30'!
defaultArgumentTile
	"Answer a tile to represent the type"

	^ MenuTile new typeColor: self typeColor! !

!MenuType methodsFor: '*eToys-color' stamp: 'sw 9/27/2001 17:21'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(0.4 0.4 0.4)	! !

!Quadrangle methodsFor: '*eToys-vocabulary' stamp: 'sw 5/4/2001 16:45'!
vocabularyDemanded
	"Answer the vocabulary that the receiver really would like to use in a Viewer"

	^ Vocabulary quadVocabulary! !

!MethodNode methodsFor: '*eToys-tiles' stamp: 'RAA 2/16/2001 15:44'!
asMorphicSyntaxIn: parent
	
	^parent
		methodNodeInner: self 
		selectorOrFalse: selectorOrFalse 
		precedence: precedence 
		arguments: arguments 
		temporaries: temporaries 
		primitive: primitive 
		block: block
! !

!MethodNode methodsFor: '*eToys-tiles' stamp: 'tk 8/5/2001 11:40'!
asMorphicSyntaxUsing: aClass
	
	^ Cursor wait showWhile: [
		(aClass methodNodeOuter: self) finalAppearanceTweaks]
		! !

Vocabulary subclass: #EToyVocabulary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Protocols'!

!EToyVocabulary commentStamp: '<historical>' prior: 0!
EToyVocabulary - a vocabulary mirroring the capabilities available to end users in Squeak's old 1997-2000 etoy prototype.!

EToyVocabulary subclass: #EToyVectorVocabulary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Protocols'!

!EToyVectorVocabulary commentStamp: '<historical>' prior: 0!
An extension of the etoy vocabulary in support of an experiment Alan Kay requested in summer 2001 for allowing any morph/player to be thought of as a vector.  In effect, adds a category #vector to the viewer for such all morphs.  Consult Ted Kaehler and Alan Kay for more information on this track.!

!EToyVectorVocabulary methodsFor: 'initialization' stamp: 'sw 9/10/2001 14:44'!
addCustomCategoriesTo: categoryList
	"Add any further categories to the default list of viewer categories for an object"

	categoryList add: #vector! !

!EToyVectorVocabulary methodsFor: 'method list' stamp: 'sw 7/9/2005 12:02'!
allMethodsInCategory: aCategorySymbol forInstance: anObject ofClass: aClass
	"Answer a list of all methods in the etoy interface which are in the given category, on behalf of anObject, or if it is nil, aClass"

	^ ((anObject isKindOf: Player) and: [aCategorySymbol == #vector])
			ifTrue:
				[anObject costume class vectorAdditions collect:
					[:anAddition | (self methodInterfaceFrom: anAddition) selector]]
			ifFalse:
				[super allMethodsInCategory: aCategorySymbol forInstance: anObject ofClass: aClass]! !

!EToyVectorVocabulary methodsFor: 'initialization' stamp: 'sw 9/26/2001 03:56'!
eToyVectorTable
	"Answer a table of specifications to send to #addFromTable: which add the 'players are vectors' extension to the etoy vocabulary."

	"(selector setterOrNil ((arg name  arg type)...) resultType (category ...) 'help msg' 'wording' autoUpdate)"

	^ #(

(+ nil ((aVector  Player)) Player (geometry) 'Adds two players together, treating each as a vector from the origin.')
(- nil ((aVector  Player)) Player (geometry) 'Subtracts one player from another, treating each as a vector from the origin.')
(* nil ((aVector  Number)) Player (geometry) 'Multiply a player by a number, treating the Player as a vector from the origin.')
(/ nil ((aVector  Number)) Player (geometry) 'Divide a player by a Number, treating the Player as a vector from the origin.')

(incr: nil ((aVector  Player)) unknown (geometry) 'Each Player is a vector from the origin.  Increase one by the amount of the other.' 'increase by')
(decr: nil ((aVector  Player)) unknown (geometry) 'Each Player is a vector from the origin.  Decrease one by the amount of the other.' 'decrease by')
(multBy: nil ((factor  Number)) unknown (geometry) 'A Player is a vector from the origin.  Multiply its length by the factor.' 'multiplied by')
(dividedBy: nil ((factor  Number)) unknown (geometry) 'A Player is a vector from the origin.  Divide its length by the factor.' 'divided by')

"distance and theta are already in Player.  See additionsToViewerCategoryGeometry"
).! !

!EToyVectorVocabulary methodsFor: 'initialization' stamp: 'mir 7/15/2004 19:29'!
initialize
	"Initialize the vocabulary"

	super initialize.
	self addFromTable: self eToyVectorTable.
	self vocabularyName: #Vector.
	self documentation: 'This vocabulary adds to the basic etoy experience an interpretation of "players are vectors", requested by Alan Kay and implemented by Ted Kaehler in summer 2001'.
! !

!EToyVocabulary methodsFor: 'initialization' stamp: 'sw 9/13/2001 16:36'!
addCustomCategoriesTo: categoryList
	"Add any further categories to the categoryList -- for benefit of subclasses wishing to override."! !

!EToyVocabulary methodsFor: 'method list' stamp: 'sw 10/7/2004 17:09'!
allMethodsInCategory: aCategoryName forInstance: anObject ofClass: aClass
	"Answer a list of all methods in the etoy interface which are in the given category, on behalf of anObject, or if it is nil, aClass"

	| aCategory unfiltered suitableSelectors isAll |

	aCategoryName ifNil: [^ OrderedCollection new].
	aClass isUniClass ifTrue:
		[aCategoryName = ScriptingSystem nameForScriptsCategory ifTrue:
			[^ aClass namedTileScriptSelectors].
		aCategoryName = ScriptingSystem nameForInstanceVariablesCategory ifTrue:
			[^ aClass slotInfo keys asSortedArray collect:
				[:anInstVarName | Utilities getterSelectorFor: anInstVarName]]].
	unfiltered _ (isAll _ aCategoryName = self allCategoryName)
		ifTrue:
			[methodInterfaces collect: [:anInterface | anInterface selector]]
		ifFalse:
			[aCategory _ categories detect: [:cat | cat categoryName = aCategoryName] 
							ifNone: [^ OrderedCollection new].
			aCategory elementsInOrder collect: [:anElement | anElement selector]].

	(anObject isKindOf: Player) ifTrue:
		[suitableSelectors _ anObject costume selectorsForViewer.
		unfiltered _ unfiltered  select:
			[:aSelector | suitableSelectors includes: aSelector]].
	(isAll and: [aClass isUniClass]) ifTrue:
		[unfiltered addAll: aClass namedTileScriptSelectors.
		unfiltered addAll: (aClass slotInfo keys asSortedArray collect:
			[:anInstVarName | Utilities getterSelectorFor: anInstVarName])].

	^ (unfiltered copyWithoutAll: #(dummy unused)) asSortedArray! !

!EToyVocabulary methodsFor: 'category list' stamp: 'nk 8/29/2004 17:17'!
categoryListForInstance: anObject ofClass: aClass limitClass: mostGenericClass
	"Answer the category list for the given object, considering only code implemented in aClass and lower"

	^ (anObject isPlayerLike)
		ifTrue:
			[self flag: #deferred.  "The bit commented out on next line is desirable but not yet workable, because it delivers categories that are not relevant to the costume in question"
			"#(scripts #'instance variables'), (super categoryListForInstance: anObject ofClass: aClass limitClass: mostGenericClass)]"

			self translatedWordingsFor: ((mostGenericClass == aClass)
				ifFalse:
					[anObject categoriesForVocabulary: self]
				ifTrue:
					[{ScriptingSystem nameForScriptsCategory.  ScriptingSystem nameForInstanceVariablesCategory}])]
		ifFalse:
			[super categoryListForInstance: anObject ofClass: aClass limitClass: mostGenericClass]! !

!EToyVocabulary methodsFor: 'initialization' stamp: 'sw 12/12/2000 06:06'!
encompassesAPriori: aClass
	"Answer whether an object, by its very nature, is one that the receiver embraces"

	^ aClass isKindOf: Player class! !

!EToyVocabulary methodsFor: 'initialization' stamp: 'sw 12/18/2000 14:33'!
includesSelector: aSelector forInstance: anInstance ofClass: aTargetClass limitClass: mostGenericClass
	"Answer whether the vocabulary includes the given selector for the given class (and instance, if provided), only considering method implementations in mostGenericClass and lower"

	| classToUse aClass theKeys |
	(aTargetClass isUniClass and:
		[(aTargetClass namedTileScriptSelectors includes: aSelector) or:
			[(((theKeys _ aTargetClass slotInfo keys collect:
				[:anInstVarName | Utilities getterSelectorFor: anInstVarName])) includes: aSelector)
					or:
						[(theKeys collect: [:anInstVarName | Utilities setterSelectorFor: anInstVarName]) includes: aSelector]]]) ifTrue: [^ true].

	(methodInterfaces includesKey: aSelector) ifFalse: [^ false].
	classToUse _ self classToUseFromInstance: anInstance ofClass: aTargetClass.
	^ (aClass _ classToUse whichClassIncludesSelector: aSelector)
		ifNil:
			[false]
		ifNotNil:
			[aClass includesBehavior: mostGenericClass]
! !

!EToyVocabulary methodsFor: 'method list' stamp: 'sw 8/3/2005 14:03'!
masterOrderingOfPhraseSymbols
	"Answer a dictatorially-imposed presentation list of phrase-symbols.  This governs the order in which suitable phrases are presented in etoy viewers using the etoy vocabulary.  For any given category, the default implementation is that any items that are in this list will occur first, in the order specified here; after that, all other items will come, in alphabetic order by formal selector."

	^ #(beep: forward: turn: getX getY getLocationRounded getHeading getScaleFactor

		getLeft getRight getTop getBottom  
		getLength getWidth 
		getTheta getDistance getHeadingTheta getUnitVector

		startScript: pauseScript: stopScript: startAll: pauseAll: stopAll: tellAllSiblings: doScript:

		getColor getUseGradientFill getSecondColor  getRadialGradientFill  getBorderWidth getBorderColor getBorderStyle getRoundedCorners getDropShadow getShadowColor 

		getVolume play playUntilPosition: stop rewind getIsRunning getRepeat getPosition getTotalFrames getTotalSeconds getFrameGraphic getVideoFileName getSubtitlesFileName

		getGraphic getBaseGraphic

		getAllowEtoyUserCustomEvents 

		#getAutoExpansion #getAutoLineLayout #getBatchPenTrails getDropProducesWatcher #getFenceEnabled #getIndicateCursor #getIsOpenForDragNDrop #getIsPartsBin #getMouseOverHalos #getOriginAtCenter #getShowThumbnail

	 getFenceEnabled getKeepTickingWhilePainting getOliveHandleForScriptedObjects  getUseVectorVocabulary 

)! !

!EToyVocabulary methodsFor: 'initialization' stamp: 'sw 9/13/2001 16:39'!
methodInterfaceFrom: elementTuple
	"Tedious revectoring:  The argument is a tuple of the sort that #additionsToViewerCategory: answers a list of; answer a MethodInterface"

	^ elementTuple first == #command
		ifTrue:
			[MethodInterface new initializeFromEToyCommandSpec: elementTuple category: nil]
		ifFalse:  "#slot format"
			[MethodInterface new initializeFromEToySlotSpec: elementTuple]! !

!EToyVocabulary methodsFor: 'initialization' stamp: 'RAA 6/4/2001 19:12'!
objectForDataStream: refStrm
	"I am about to be written on an object file.  Write a path to me in the other system instead."

	vocabularyName == #eToy ifFalse: [^ self].

	^ DiskProxy 
		global: #Vocabulary
		selector: #vocabularyNamed: 
		args: (Array with: vocabularyName)
! !

!EToyVocabulary methodsFor: 'method list' stamp: 'sw 7/14/2004 18:24'!
phraseSymbolsToSuppress
	"Answer a dictatorially-imposed list of phrase-symbols that are to be suppressed from viewers when the eToyFriendly preference is set to true.  This list at the moment corresponds to the wishes of Alan and Kim and the LA teachers using Squeak in school-year 2001-2"

	^ Preferences eToyFriendly
		ifTrue:
			[#(moveToward: followPath goToRightOf:
				getViewingByIcon initiatePainting
				append: prepend: getClipSubmorphs touchesA:)]
		ifFalse:
			[#()]! !

!EToyVocabulary methodsFor: 'initialization' stamp: 'nk 10/6/2004 11:56'!
setCategoryDocumentationStrings
	"Initialize the documentation strings associated with the old etoy categories, in English"

	self setCategoryStrings: #(
(basic					'basic'					'a few important things')
(#'book navigation'		'book navigation'		'relating to book, stacks, etc')
(button					'button'					'for thinking of this object as a push-button control')
(collections				'collections'				'for thinking of this object as a collection')
(fog					'fog'					'3D fog')
(geometry				'geometry' 				'measurements and coordinates')
(#'color & border'		'color & border'			'matters concerning the colors and borders of objects')
(graphics				'graphics'				'for thinking of this object as a picture')
(variables				'variables'				'variables added by this object')
(joystick				'joystick'				'the object as a Joystick')
(miscellaneous			'miscellaneous' 			'various commands')
(motion					'motion' 				'matters relating to moving and turning')
(paintbox				'paintbox'				'the painting palette')
(#'pen trails'			'pen trails'				'relating to trails put down by pens')
(#'pen use'				'pen use' 				'use of an object''s "pen"')
(playfield				'playfield'				'the object as a container for other visible objects')
(sampling				'sampling'				'sampling')
(scripting				'scripting'				'commands to start and stop scripts')
(scripts					'scripts'					'methods added by this object')
(slider					'slider'					'functions useful to sliders')
(speaker				'speaker'				'the object as an audio Speaker')
(#'stack navigation'		'stack navigation'		'navigation within a stck')
(storyboard				'storyboard'				'storyboard')
(tests					'tests'					'yes/no tests, to use in "Test" panes of scripts')
(text					'text'					'The object as text')
(vector					'vector'					'The object as a vector')
(viewing				'viewing'				'matters relating to viewing')
 ) ! !

!Vocabulary classSide methodsFor: '*eToys-customevents-custom events' stamp: 'nk 8/18/2004 17:57'!
addCustomEventsVocabulary
	| vocab |
	self addStandardVocabulary: (vocab := self newCustomEventsVocabulary).
	SymbolListTile updateAllTilesForVocabularyNamed: #CustomEvents.
	^vocab! !

!Vocabulary classSide methodsFor: '*eToys-eToy vocabularies' stamp: 'sw 9/13/2001 17:29'!
addEToyVectorVocabulary
	"Add the etoy Vector vocabulary to the standard list"

	AllStandardVocabularies at: #Vector put:  EToyVectorVocabulary new

		 ! !

!Vocabulary classSide methodsFor: '*eToys-eToy vocabularies' stamp: 'mir 7/15/2004 10:49'!
changeMadeToViewerAdditions
	"A change to some morph-subclass class-side #additionsToViewer... was made, which means that the existing etoy vocabularies need updating."
	"Vocabulary changeMadeToViewerAdditions"

	AllStandardVocabularies
		ifNotNil: [
			self addStandardVocabulary: EToyVocabulary new.
			self addStandardVocabulary: EToyVectorVocabulary new]

	
		 ! !

!Vocabulary classSide methodsFor: '*eToys-customevents-custom events' stamp: 'nk 8/21/2004 19:58'!
customEventsVocabulary
	"Vocabulary customEventsVocabulary"
	^(self vocabularyNamed: #CustomEvents)
		ifNil: [ self addCustomEventsVocabulary ]
! !

!Vocabulary classSide methodsFor: '*eToys-eToy vocabularies' stamp: 'mir 7/15/2004 10:50'!
eToyVocabulary
	"Answer the etoy vocabulary in the AllStandardVocabularies list, creating it if necessary."

	^ self allStandardVocabularies at: #eToy ifAbsentPut: [EToyVocabulary new]! !

!Vocabulary classSide methodsFor: '*eToys-customevents-custom events' stamp: 'nk 8/18/2004 17:56'!
newCustomEventsVocabulary
	^UserCustomEventNameType new.
! !

!Vocabulary classSide methodsFor: '*eToys-type vocabularies' stamp: 'mir 7/15/2004 10:54'!
newWonderlandVocabulary
	"Answer a Wonderland vocabulary -- highly experimental"

	| aVocabulary  |
	"Vocabulary newWonderlandVocabulary"
	aVocabulary _ Vocabulary new vocabularyName: #Wonderland.
	aVocabulary documentation: 'A simple vocabulary for scripting Alice objects'.
	aVocabulary initializeFromTable:  #(
		(color color: () Color (basic color) 'The color of the object' unused updating)
		"--"
		(getX setX: () Number (basic geometry) 'The x position' unused updating)
		(getY setY: () Number (basic geometry) 'The y position' unused updating)
		(getZ setZ: () Number (basic geometry) 'The z position' unused updating)
		"--"
		(width setWidth: () Number (geometry) 'The width of the object' unused updating)
		(height setHeight: () Number (geometry) 'The height of the object' unused updating)
		(depth setDepth: () Number (geometry) 'The depth of the object' unused updating)
		"--"
		(heading setHeading: () Number (basic geometry) 'The heading of the object' unused updating)
		(forwardBy: unused ((distance Number)) none (basic motion) 'Moves the object by the specified distance' 'forward by')
		(turnBy: unused ((angle Number)) none (basic motion) 'Turns the object by the specified number of degrees' 'turn by')
		(graphic setGraphic: () Graphic (basic graphics) 'The picture currently being worn' unused updating)
		(animationIndex setAnimationIndex: () Number (graphics) 'The index in the object''s animation chain' unused updating)
		(emptyScript unused () none (scripts) 'The empty script')
		(distanceToCamera setDistanceToCamera: () Number (geometry) 'The distance of the object from the camera' unused updating)
		(distanceTo: unused ((target Player)) Number (geometry) 'The distance of the object to the given target')
	).
	^ aVocabulary! !

!Vocabulary classSide methodsFor: '*eToys-type vocabularies' stamp: 'mir 7/15/2004 10:55'!
wonderlandVocabulary
	"Answer the Quad vocabulary lurking in my AllStandardVocabularies list, creating it if necessary"
	"Vocabulary newWonderlandVocabulary"

	^ self allStandardVocabularies at: #Wonderland ifAbsentPut: [self newWonderlandVocabulary]! !

!Vocabulary methodsFor: '*eToys-color' stamp: 'sw 9/27/2001 17:47'!
subduedColorFrom: aColor
	"Answer a subdued color derived from the given color"

	^ aColor mixed: ScriptingSystem colorFudge with: ScriptingSystem uniformTileInteriorColor! !

!Vocabulary methodsFor: '*eToys-queries' stamp: 'yo 2/19/2005 23:41'!
tileWordingForSelector: aSelector
	"Answer the wording to emblazon on tiles representing aSelector"

	| anInterface inherent |
	anInterface _ self methodInterfaceAt: aSelector asSymbol ifAbsent:
		[inherent _ Utilities inherentSelectorForGetter: aSelector.
		^ inherent
			ifNil:
				[self translatedWordingFor: aSelector]
			ifNotNil:
				[inherent translated]].
	^ anInterface wording! !

!Vocabulary methodsFor: '*eToys-queries' stamp: 'sw 9/26/2001 12:01'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFrom: Color green! !

!CodeHolder methodsFor: '*eToys-tiles' stamp: 'nk 4/28/2004 10:14'!
installTilesForSelection
	"Install universal tiles into the code pane."
	| source aSelector aClass tree syn tileScriptor aWindow codePane |
	(aWindow _ self containingWindow)
		ifNil: [self error: 'hamna dirisha'].
	tileScriptor _ ((aSelector _ self selectedMessageName) isNil
					or: [(aClass _ self selectedClassOrMetaClass whichClassIncludesSelector: aSelector) isNil])
				ifTrue: [PluggableTileScriptorMorph new]
				ifFalse: [source _ aClass sourceCodeAt: aSelector.
					tree _ Compiler new
								parse: source
								in: aClass
								notifying: nil.
					(syn _ tree asMorphicSyntaxUsing: SyntaxMorph) parsedInClass: aClass.
					syn inAPluggableScrollPane].
	codePane _ aWindow
				findDeepSubmorphThat: [:m | (m isKindOf: PluggableTextMorph)
						and: [m getTextSelector == #contents]]
				ifAbsent: [].
	codePane
		ifNotNil: [codePane hideScrollBars].
	codePane
		ifNil: [codePane _ aWindow
						findDeepSubmorphThat: [:m | m isKindOf: PluggableTileScriptorMorph]
						ifAbsent: [self error: 'no code pane']].
	tileScriptor color: aWindow paneColorToUse;
		 setProperty: #hideUnneededScrollbars toValue: true.
	aWindow replacePane: codePane with: tileScriptor.
	currentCompiledMethod _ aClass
				ifNotNil: [aClass compiledMethodAt: aSelector].
	tileScriptor owner clipSubmorphs: true.
	tileScriptor extent: codePane extent! !

!CodeHolder methodsFor: '*eToys-tiles' stamp: 'rhi 1/4/2002 11:15'!
showTiles: aBoolean
	"Set the showingTiles as indicated.  The fact that there are initially no senders of this reflects that fact that initially this trait is only directly settable through the UI; later there may be senders, such as if one wanted to set a system up so that all newly-opened browsers showed tiles rather than text."

	aBoolean
		ifTrue:
			[contentsSymbol _ #tiles]
		ifFalse:
			[contentsSymbol == #tiles ifTrue: [contentsSymbol _ #source]].
	self setContentsToForceRefetch.
	self changed: #contents! !

!CodeHolder methodsFor: '*eToys-tiles' stamp: 'sw 2/3/2001 00:10'!
showingTiles
	"Answer whether the receiver is currently showing tiles"

	^ contentsSymbol == #tiles
! !

!CodeHolder methodsFor: '*eToys-tiles' stamp: 'sw 5/20/2001 21:12'!
showingTilesString
	"Answer a string characterizing whether tiles are currently showing or not"

	^ (self showingTiles
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'tiles'! !

!CodeHolder methodsFor: '*eToys-tiles' stamp: 'sw 2/14/2001 15:27'!
toggleShowingTiles
	"Toggle whether tiles should be shown in the code pane"

	self okToChange ifTrue:
		[self showingTiles
			ifTrue:
				[contentsSymbol _ #source.
				self setContentsToForceRefetch.
				self installTextualCodingPane.
				self contentsChanged]
			ifFalse:
				[contentsSymbol _ #tiles.
				self installTilesForSelection.
				self changed: #tiles]]! !

!GraphicType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:30'!
defaultArgumentTile
	"Answer a tile to represent the type"

	^ GraphicTile new typeColor: self typeColor! !

!GraphicType methodsFor: '*eToys-color' stamp: 'sw 9/27/2001 17:21'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(0.806 1.0 0.806)	! !

!GraphicType methodsFor: '*eToys-tiles' stamp: 'sw 9/25/2001 21:06'!
updatingTileForTarget: aTarget partName: partName getter: getter setter: setter
	"Answer, for classic tiles, an updating readout tile for a part with the receiver's type, with the given getter and setter"

	^ ThumbnailMorph new objectToView: aTarget viewSelector: getter; extent: 21@21; yourself! !

!Number methodsFor: '*eToys-printing' stamp: 'sw 10/31/97 13:54'!
newTileMorphRepresentative
	^ TileMorph new addArrows; setLiteral: self; addSuffixIfCan
! !

!Number methodsFor: '*eToys-vocabulary' stamp: 'sw 8/3/2001 13:43'!
vocabularyDemanded
	"Answer the vocabulary normally preferred by this object"

	^ Vocabulary numberVocabulary! !

ScriptEditorMorph subclass: #BooleanScriptEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Support'!

!BooleanScriptEditor commentStamp: '<historical>' prior: 0!
A ScriptEditor required to hold a Boolean!

!BooleanScriptEditor methodsFor: 'other' stamp: 'tk 3/1/2001 11:24'!
hibernate 
	"do nothing"! !

!BooleanScriptEditor methodsFor: 'other' stamp: 'dgd 2/22/2003 14:44'!
storeCodeOn: aStream indent: tabCount 
	(submorphs notEmpty and: [submorphs first submorphs notEmpty]) 
		ifTrue: 
			[aStream nextPutAll: '(('.
			super storeCodeOn: aStream indent: tabCount.
			aStream nextPutAll: ') ~~ false)'.
			^self].
	aStream nextPutAll: ' true '! !

!BooleanScriptEditor methodsFor: 'other' stamp: 'tk 2/28/2001 21:07'!
unhibernate 
	"do nothing"! !

!BooleanScriptEditor methodsFor: 'dropping/grabbing' stamp: 'sw 3/15/2005 22:43'!
wantsDroppedMorph: aMorph event: evt 
	"Answer whether the receiver would be interested in accepting the morph"

	(submorphs detect: [:m | m isAlignmentMorph] ifNone: [nil]) 
		ifNotNil: [^ false].

	((aMorph isKindOf: ParameterTile) and: [aMorph scriptEditor == self topEditor])
		ifTrue: [^ true].
	^ (aMorph isKindOf: PhraseTileMorph orOf: WatcherWrapper) 
		and: [(#(#Command #Unknown) includes: aMorph resultType capitalized) not]! !

!StringType methodsFor: '*eToys-tiles' stamp: 'yo 3/14/2005 21:27'!
defaultArgumentTile
        "Answer a tile to represent the type"

        ^ 'abc' translated newTileMorphRepresentative typeColor: self typeColor! !

!StringType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:33'!
setFormatForDisplayer: aDisplayer
	"Set up the displayer to have the right format characteristics"

	aDisplayer useStringFormat
	! !

!StringType methodsFor: '*eToys-color' stamp: 'sw 9/27/2001 17:21'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(0.0 0.0 1.0)	! !

!StringType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:33'!
wantsArrowsOnTiles
	"Answer whether this data type wants up/down arrows on tiles representing its values"

	^ false! !

!TextFieldMorph classSide methodsFor: '*eToys-instance creation' stamp: 'sw 6/13/2001 22:48'!
exampleBackgroundField
	"Answer a scrollable background field for a parts bin"

	| aMorph |
	aMorph _ self authoringPrototype.
	aMorph contents: 'background field' asText allBold.
	aMorph setProperty: #shared toValue: true.
	aMorph setNameTo: 'scrollingField1'.
	aMorph setProperty: #holdsSeparateDataForEachInstance toValue: true.
	^ aMorph! !

!TextFieldMorph methodsFor: '*eToys-card in a stack' stamp: 'tk 1/6/2001 14:20'!
couldHoldSeparateDataForEachInstance
	"Answer whether this type of morph is inherently capable of holding separate data for each instance ('card data')"

	^ true! !

!TextFieldMorph methodsFor: '*eToys-player' stamp: 'sw 10/30/2000 09:03'!
currentDataValue
	"Answer the current data value held by the receiver"

	^ self contents! !

!TextFieldMorph methodsFor: '*eToys-card & stack' stamp: 'tk 1/6/2001 14:15'!
setNewContentsFrom: textOrString
	"talk to my text"
	| tm |

	(tm _ self findA: TextMorph) ifNil: [^ nil].
	tm valueOfProperty: #cardInstance ifAbsent: ["move it down"
		tm setProperty: #cardInstance toValue: (self valueOfProperty: #cardInstance)].
	tm valueOfProperty: #holdsSeparateDataForEachInstance ifAbsent: ["move it down"
		tm setProperty: #holdsSeparateDataForEachInstance toValue: 
			(self valueOfProperty: #holdsSeparateDataForEachInstance)].
	^ tm setNewContentsFrom: textOrString! !

!TextFieldMorph methodsFor: '*eToys-player' stamp: 'tk 1/6/2001 13:58'!
variableDocks
	"Answer a list of VariableDock objects for docking up my data with an instance held in my containing playfield"

	^ Array with: (VariableDock new variableName: self defaultVariableName type: #text definingMorph: self morphGetSelector: #contents morphPutSelector: #setNewContentsFrom:)! !

!HandMorph methodsFor: '*eToys-scripting' stamp: 'ar 3/17/2001 20:11'!
adaptedToWorld: aWorld
	"If I refer to a world or a hand, return the corresponding items in the new world."
	^aWorld primaryHand! !

!HandMorph methodsFor: '*eToys-pen' stamp: 'di 9/10/1998 16:20'!
trailMorph
	"You can't draw trails when picked up by the hand."
	^ nil! !

!AlignmentMorph class methodsFor: '*eToys-scripting' stamp: 'sw 11/16/2001 10:01'!
additionsToViewerCategories
	"Answer viewer additions for the 'layout' category"

	^#((
		layout 
		(
			(slot cellInset 'The cell inset' Number readWrite Player getCellInset Player setCellInset:)
			(slot layoutInset 'The layout inset' Number readWrite Player getLayoutInset Player setLayoutInset:)
			(slot listCentering 'The list centering' ListCentering readWrite Player getListCentering Player setListCentering:)
			(slot hResizing  	'Horizontal resizing' Resizing readWrite Player 	getHResizing Player setHResizing:)
			(slot vResizing  	'Vertical resizing' Resizing readWrite Player 	getVResizing Player setVResizing:)
			(slot listDirection  'List direction' ListDirection readWrite Player 	getListDirection Player setListDirection:)
			(slot wrapDirection 'Wrap direction' ListDirection readWrite Player 	getWrapDirection Player setWrapDirection:)
		)))
! !

!AlignmentMorph methodsFor: '*eToys-initialization' stamp: 'ar 10/25/2000 17:53'!
addUpDownArrowsFor: aMorph
	"Add a column of up and down arrows that serve to send upArrowHit and downArrowHit to aMorph when they're pressed/held down"

	| holder downArrow upArrow |
	holder _ Morph new extent: 16 @ 16; beTransparent.
	downArrow _ ImageMorph new image: (ScriptingSystem formAtKey: 'DownArrow').
	upArrow _ ImageMorph new image: (ScriptingSystem formAtKey: 'UpArrow').
	upArrow position: holder bounds topLeft + (2@2).
	downArrow align: downArrow bottomLeft
				with: holder topLeft + (0 @ TileMorph defaultH) + (2@-2).
	holder addMorph: upArrow.
	holder addMorph: downArrow.
	self addMorphBack: holder.
	upArrow on: #mouseDown send: #upArrowHit to: aMorph.
	upArrow on: #mouseStillDown send: #upArrowHit to: aMorph.
	downArrow on: #mouseDown send: #downArrowHit to: aMorph.
	downArrow on: #mouseStillDown send: #downArrowHit to: aMorph.! !

!AlignmentMorph methodsFor: '*eToys-e-toy support' stamp: 'panda 4/25/2000 15:44'!
configureForKids
	self disableDragNDrop.
	super configureForKids
! !

AlignmentMorph subclass: #AllScriptsTool
	instanceVariableNames: 'showingOnlyActiveScripts showingAllInstances showingOnlyTopControls'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!AllScriptsTool commentStamp: '<historical>' prior: 0!
A tool for controlling and viewing all scripts in a project.  The tool has an open and a closed form.  In the closed form, stop-step-go buttons are available, plus a control for opening the tool up.  In the open form, it has a second row of controls that govern which scripts should be shown, followed by the individual script items.!

!AllScriptsTool classSide methodsFor: 'instance creation' stamp: 'sw 6/12/2001 11:52'!
allScriptsToolForActiveWorld
	"Launch an AllScriptsTool to view scripts of the active world"

	| aTool |
	aTool _ self newColumn.
	aTool initializeFor: ActiveWorld presenter.
	^ aTool! !

!AllScriptsTool classSide methodsFor: 'printing' stamp: 'sw 11/13/2001 19:44'!
defaultNameStemForInstances
	"Answer the default name stem for new instances of this class"

	^ 'All Scripts'! !

!AllScriptsTool classSide methodsFor: 'parts bin' stamp: 'sw 11/13/2001 18:31'!
descriptionForPartsBin
	"Answer a description for use in parts bins"

	^ self partName: 	'All Scripts'
		categories:		#('Scripting')
		documentation:	'A tool allowing you to monitor and change the status of all scripts in your project'! !

!AllScriptsTool classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 10:28'!
initialize

	self registerInFlapsRegistry.	! !

!AllScriptsTool classSide methodsFor: 'instance creation' stamp: 'sw 1/30/2001 23:06'!
launchAllScriptsToolFor: aPresenter
	"Launch an AllScriptsTool to view scripts of the given presenter"

	| aTool |
	aTool _ self newColumn.
	aTool initializeFor: aPresenter.
	self currentHand attachMorph: aTool.
	aPresenter associatedMorph world startSteppingSubmorphsOf: aTool
! !

!AllScriptsTool classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 10:30'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: #(AllScriptsTool			allScriptsToolForActiveWorld	'All Scripts' 		'A tool that lets you see and control all the running scripts in your project')
						forFlapNamed: 'PlugIn Supplies'.
						cl registerQuad: #(AllScriptsTool			allScriptsToolForActiveWorld	'All Scripts' 		'A tool that lets you control all the running scripts in your world')
						forFlapNamed: 'Scripting'.
						cl registerQuad: #(AllScriptsTool			allScriptsToolForActiveWorld	'All Scripts' 		'A tool that lets you see and control all the running scripts in your project')
						forFlapNamed: 'Widgets']! !

!AllScriptsTool classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 12:30'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!AllScriptsTool methodsFor: 'initialization' stamp: 'dgd 9/19/2003 14:34'!
addSecondLineOfControls
	"Add the second line of controls"

	| aRow outerButton aButton worldToUse |
	aRow _ AlignmentMorph newRow listCentering: #center; color: Color transparent.
	outerButton _ AlignmentMorph newRow.
	outerButton wrapCentering: #center; cellPositioning: #leftCenter.
	outerButton color:  Color transparent.
	outerButton hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	outerButton addMorph: (aButton _ UpdatingThreePhaseButtonMorph checkBox).
	aButton
		target: self;
		actionSelector: #toggleWhetherShowingOnlyActiveScripts;
		getSelector: #showingOnlyActiveScripts.
	outerButton addTransparentSpacerOfSize: (4@0).
	outerButton addMorphBack: (StringMorph contents: 'tickers only' translated) lock.
	outerButton setBalloonText: 'If checked, then only scripts that are paused or ticking will be shown' translated.
	aRow addMorphBack: outerButton.

	aRow addTransparentSpacerOfSize: 20@0.
	aRow addMorphBack: self helpButton.

	aRow addTransparentSpacerOfSize: 20@0.

	outerButton _ AlignmentMorph newRow.
	outerButton wrapCentering: #center; cellPositioning: #leftCenter.
	outerButton color:  Color transparent.
	outerButton hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	outerButton addMorph: (aButton _ UpdatingThreePhaseButtonMorph checkBox).
	aButton
		target: self;
		actionSelector: #toggleWhetherShowingAllInstances;
		getSelector: #showingAllInstances.
	outerButton addTransparentSpacerOfSize: (4@0).
	outerButton addMorphBack: (StringMorph contents: 'all instances' translated) lock.
	outerButton setBalloonText: 'If checked, then entries for all instances will be shown, but if not checked, scripts for only one representative of each different kind of object will be shown.  Consult the help available by clicking on the purple ? for more information.' translated.
	aRow addMorphBack: outerButton.

	self addMorphBack: aRow.
	worldToUse _ self isInWorld ifTrue: [self world] ifFalse: [ActiveWorld].
	worldToUse presenter reinvigorateAllScriptsTool: self.
	self layoutChanged.! !

!AllScriptsTool methodsFor: 'initialization' stamp: 'dgd 8/31/2003 19:43'!
dismissButton
	"Answer a button whose action would be to dismiss the 
	receiver "
	| aButton |
	aButton := super dismissButton.
	aButton setBalloonText: 'Click here to remove this tool from the screen; you can get another one any time you want from the Widgets flap' translated.
	^ aButton! !

!AllScriptsTool methodsFor: 'initialization' stamp: 'sw 12/8/2004 11:26'!
initializeFor: ignored
	"Initialize the receiver as a tool which shows, and allows the user to change the status of, all the instantiations of all the user-written scripts in the scope of the containing pasteup's presenter"

	| aRow aButton |
	showingOnlyActiveScripts _ true.
	showingAllInstances _ true.
	showingOnlyTopControls _ true.
	self color: Color brown muchLighter muchLighter; wrapCentering: #center; cellPositioning: #topCenter; vResizing: #shrinkWrap; hResizing: #shrinkWrap.
	self useRoundedCorners.
	self borderWidth: 4; borderColor: Color brown darker.
	aRow _ AlignmentMorph newRow.
	aRow listCentering: #justified; color: Color transparent.
	aButton _ self tanOButton.
	aButton actionSelector: #delete.
	aRow addMorphFront: aButton.
	aRow addMorphBack: ScriptingSystem scriptControlButtons.
	aRow addMorphBack: self openUpButton.
	self addMorphFront: aRow.

! !

!AllScriptsTool methodsFor: 'parts bin' stamp: 'dgd 2/22/2003 19:37'!
initializeToStandAlone
	super initializeToStandAlone.
	self
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #spaceFill;
		extent: 1 @ 1;
		vResizing: #spaceFill;
		rubberBandCells: true.
	self initializeFor: self currentWorld presenter! !

!AllScriptsTool methodsFor: 'toggles' stamp: 'sw 12/8/2004 11:28'!
openUpButton
	"Answer a button whose action would be to open up the receiver or snap it back closed"

	| aButton aForm |
	aButton _ IconicButton new borderWidth: 0.
	aForm _ ScriptingSystem formAtKey: #PowderBlueOpener.
	aForm ifNil:
		[aForm _ Form extent: 13@22 depth: 16 fromArray: #( 0 0 12017 787558129 0 0 0 0 12017 787561309 995965789 787558129 0 0 0 787561309 995965789 995965789 995965789 787546112 0 12017 995965789 995965789 995965789 995965789 995962609 0 12017 995965789 995965789 995965789 995965789 995962609 0 787561309 995965789 995965789 995965789 995965789 995965789 787546112 787561309 995965789 995965789 995965789 995965789 995965789 787546112 787561309 995965789 995950593 80733 995965789 995965789 787546112 787561309 995965789 65537 65537 80733 995965789 787546112 787561309 995950593 80733 995950593 80733 995965789 787546112 787561309 995950593 80733 995950593 80733 995965789 787546112 787561309 995950593 65537 65537 80733 995965789 787546112 787561309 995965789 65537 65537 995965789 995965789 787546112 787561309 995965789 995965789 995965789 995965789 995965789 787546112 787561309 995965789 995965789 995965789 995965789 995965789 787546112 787561309 995965789 995965789 995965789 995965789 995965789 787546112 787561309 995965789 995965789 995965789 995965789 995965789 787546112 12017 995965789 995965789 995965789 995965789 995962609 0 12017 995965789 995965789 995965789 995965789 995962609 0 0 787561309 995965789 995965789 995965789 787546112 0 0 12017 787561309 995965789 787558129 0 0 0 0 12017 787558129 0 0 0) offset: 0@0.
		ScriptingSystem saveForm: aForm atKey: #PowderBlueOpener].
	aButton labelGraphic: aForm.
	aButton
		target: self;
		color: Color transparent;
		actionSelector: #toggleWhetherShowingOnlyTopControls;
		setBalloonText: 'open or close the lower portion that shows individual scripts' translated.
	^ aButton! !

!AllScriptsTool methodsFor: 'initialization' stamp: 'dgd 9/19/2003 14:35'!
presentHelp
	"Sent when a Help button is hit; provide the user with some form of help for the tool at hand"

	| aString |
	aString _ 
'This tool allows you to see all the scripts for all the objects in this project.

Sometimes you are only interested in those scripts that are ticking, or that are *ready* to tick when you hit the GO button (which are said to be "paused.")

Check "tickers only" if you only want to see such scripts -- i.e., scripts that are either paused or ticking.

If "tickers only" is *not* checked, then all scripts will be shown, whatever their status.

The other checkbox, labeled "all instances", only comes into play if you have created "multiple sibling instances" (good grief) of the same object, which share the same scripts; if you have such things, it is often convenient to see the scripts of just *one* such sibling, because it will take up less space and require less mindshare -- and note that you can control a script for an object *and* all its siblings from the menu of that one that you see, via menu items such as "propagate status to siblings".

If "all instances" is checked, scripts for all sibling instances will be shown, whereas if "all instances" is *not* checked, only one of each group of siblings will be selected to have its scripts shown.

But how do you get "multiple sibling instances" of the same object?  There are several ways:

(1)  Use the "make a sibling instance" or the "make multiple siblings..." menu item in the halo menu of a scripted object

(2)  Use the "copy" tile in a script.

(3)  Request "give me a copy now" from the menu associated with the "copy" item in a Viewer

If you have on your screen multiple sibling instances of the same object, then you may or may want to see them all in the All Scripts tool, and that is what the "all instances" checkbox governs.

Set "all instances" if you want a separate entry for each instance, as
opposed to a single representative of that kind of object.

Note that if you obtain a copy of an object by using the green halo handle, it will *not* be a sibling instance of the original.  It will in many ways seem to be, because it will start out its life having the same scripts as the original.  But it will then lead an independent life, so that changes to scripts of the original will not be reflected in it, and vice-versa.

This is an important distinction, and an unavoidable one because people sometimes want the deep sharing of sibling instances and sometimes they clearly do not.  But the truly understandable description of these concepts and distinctions certainly lies *ahead* of us!!'.

	(StringHolder new contents: aString translated)
		openLabel: 'About the All Scripts tool' translated! !

!AllScriptsTool methodsFor: 'toggles' stamp: 'sw 1/30/2001 23:18'!
showingAllInstances
	"Answer whether the receiver is currently showing controls for all instances of each uniclass."
 
	^ showingAllInstances ! !

!AllScriptsTool methodsFor: 'toggles' stamp: 'sw 1/30/2001 23:18'!
showingOnlyActiveScripts
	"Answer whether the receiver is currently showing only active scripts"
 
	^ showingOnlyActiveScripts ! !

!AllScriptsTool methodsFor: 'toggles' stamp: 'sw 11/13/2001 19:43'!
showingOnlyTopControls
	"Answer whether the receiver is currently showing only the top controls"
 
	^ showingOnlyTopControls ifNil: [showingOnlyTopControls _ true]! !

!AllScriptsTool methodsFor: 'stepping and presenter' stamp: 'sw 11/14/2001 00:31'!
step
	"If the list of scripts to show has changed, refresh my contents"

	self showingOnlyTopControls ifFalse:
		[self presenter reinvigorateAllScriptsTool: self].! !

!AllScriptsTool methodsFor: 'testing' stamp: 'sw 1/31/2001 23:12'!
stepTime
	"Answer the interval between steps -- in this case a leisurely 4 seconds"

	^ 4000! !

!AllScriptsTool methodsFor: 'toggles' stamp: 'sw 11/14/2001 00:32'!
toggleWhetherShowingAllInstances
	"Toggle whether the receiver is showing all instances or only one exemplar per uniclass"

	showingAllInstances _ showingAllInstances not.
	self presenter reinvigorateAllScriptsTool: self! !

!AllScriptsTool methodsFor: 'toggles' stamp: 'sw 11/14/2001 00:32'!
toggleWhetherShowingOnlyActiveScripts
	"Toggle whether the receiver is showing only active scripts"

	showingOnlyActiveScripts _ showingOnlyActiveScripts not.
	self presenter reinvigorateAllScriptsTool: self! !

!AllScriptsTool methodsFor: 'toggles' stamp: 'sw 11/14/2001 00:32'!
toggleWhetherShowingOnlyTopControls
	"Toggle whether the receiver is showing only the stop/step/go line or the full whammy"

	| aCenter |
	showingOnlyTopControls _ self showingOnlyTopControls not.
	aCenter _ self center x.
	self showingOnlyTopControls
		ifTrue:
			[self removeAllButFirstSubmorph]
		ifFalse:
			[self addSecondLineOfControls.
			self presenter reinvigorateAllScriptsTool: self].
	WorldState addDeferredUIMessage:
		[self center: (aCenter @ self center y)]
	! !

!AllScriptsTool methodsFor: 'testing' stamp: 'sw 1/31/2001 23:12'!
wantsSteps
	"Answer whether the receiver wishes to receive the #step message"

	 ^ true! !

AlignmentMorph subclass: #ObjectRepresentativeMorph
	instanceVariableNames: 'objectRepresented'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

ObjectRepresentativeMorph subclass: #ClassRepresentativeMorph
	instanceVariableNames: 'classRepresented'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

ObjectRepresentativeMorph subclass: #ListViewLine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!ListViewLine methodsFor: 'e-toy support' stamp: 'sw 10/23/2000 18:27'!
listViewLineForFieldList: aFieldList
	"Answer a list view line containing data representing the items in aFieldList"

	^ objectRepresented == self
		ifFalse:
			[objectRepresented listViewLineForFieldList: aFieldList]
		ifTrue:
			[super listViewLineForFieldList: aFieldList]! !

!ListViewLine methodsFor: 'thumbnail' stamp: 'sw 10/6/2002 02:00'!
morphRepresented
	"Answer the morph that I actually represent"

	^ objectRepresented! !

!ListViewLine methodsFor: 'object represented' stamp: 'ar 11/1/2000 15:55'!
objectRepresented: anObject
	"Set the object represented by the receiver to be as requested"

	objectRepresented _ anObject.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.
	self setNameTo: anObject name.
	self removeAllMorphs.
! !

!ObjectRepresentativeMorph methodsFor: 'scripting' stamp: 'sw 10/17/2000 11:35'!
categoriesForViewer
	"Answer a list of symbols representing the categories to offer in the viewer, in order"

	^ objectRepresented class organization categories
! !

!ObjectRepresentativeMorph methodsFor: 'naming' stamp: 'sw 10/31/2000 09:22'!
nameOfObjectRepresented
	"Answer the external name of the object represented"

	^ objectRepresented externalName! !

!ObjectRepresentativeMorph methodsFor: 'macpal' stamp: 'sw 10/17/2000 11:42'!
objectRepresented
	"Answer the object represented by the receiver"

	^ objectRepresented! !

!ObjectRepresentativeMorph methodsFor: 'as yet unclassified' stamp: 'ar 11/1/2000 15:55'!
objectRepresented: anObject
	"Set the receiver's representee.  This clears out any preexisting state in the receiver"

	objectRepresented _ anObject.
	self removeAllMorphs.
	self hResizing: #shrinkWrap.  
	self vResizing: #shrinkWrap.
	self addMorphBack: (StringMorph new contents: anObject name asString).
	self setNameTo: anObject name
	! !

!ObjectRepresentativeMorph methodsFor: 'as yet unclassified' stamp: 'ar 11/1/2000 15:55'!
objectRepresented: anObject labelString: aLabel
	"Set the receiver's representee as indicated, and use the given label to tag it"

	objectRepresented _ anObject.
	self removeAllMorphs.
	self hResizing: #shrinkWrap.  
	self vResizing: #shrinkWrap.
	self addMorphBack: (StringMorph new contents: aLabel asString).
	self setNameTo: aLabel asString
	! !

AlignmentMorph subclass: #PhraseWrapperMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!PhraseWrapperMorph commentStamp: '<historical>' prior: 0!
An alignment morph designed for use in scripting Viewers; it wraps a set of phrases in a category viewer, and repels attempts to drop phrases upon it.!

!PhraseWrapperMorph classSide methodsFor: 'new-morph participation' stamp: 'sw 11/9/1998 16:16'!
includeInNewMorphMenu
	"Not to be instantiated from the menu"
	^ false! !

!PhraseWrapperMorph methodsFor: 'parts bin' stamp: 'sw 8/12/97 14:16'!
isPartsBin
	^ true! !

!PhraseWrapperMorph methodsFor: 'dropping/grabbing' stamp: 'sw 6/1/2000 22:30'!
repelsMorph: aMorph event: ev
	^ (aMorph isKindOf: PhraseTileMorph) or:
		[aMorph hasProperty: #newPermanentScript]! !

AlignmentMorph subclass: #PlayerSurrogate
	instanceVariableNames: 'playerRepresented'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!PlayerSurrogate commentStamp: '<historical>' prior: 0!
An morph representing an E-Toy "Player" in an AllPlayersTool.!

!PlayerSurrogate methodsFor: 'menu' stamp: 'sw 7/28/2004 21:18'!
addCustomMenuItems: aMenu hand:  aHand
	"Add cu stom items to the menu"

	aMenu addList: #(
		('grab this object'	grabThisObject	'wherever it may be rip this object out of its container and hand it to me.')
		('reveal this object'	revealThisObject		'make this object visible and put up its halo')
		('hand me a tile'	handMeATile	'hand me a tile for this object')
		('open viewer'		viewerForThisObject	'open this object''s Viewer'))! !

!PlayerSurrogate methodsFor: 'updating' stamp: 'sw 7/28/2004 20:55'!
bringUpToDate
	"To react to changes  in the corrreponding player, rebuild the display with fresh information from its surrogate"

	self rebuildRow! !

!PlayerSurrogate methodsFor: 'menu' stamp: 'sw 7/28/2004 20:54'!
grabThisObject
	"Hand the user the object represented by the receiver.  Invoked from menu, formerly at least."

	playerRepresented grabPlayerInActiveWorld! !

!PlayerSurrogate methodsFor: 'menu' stamp: 'sw 7/28/2004 20:52'!
handMeATile
	"Hand the user a tile representing the player for which the receiver is a surrogate"

	playerRepresented tearOffTileForSelf! !

!PlayerSurrogate methodsFor: 'updating' stamp: 'sw 7/28/2004 20:54'!
isTileScriptingElement
	"Answer (for the purpose of updating) whether the receiver is a tile-scripting element"

	^ true! !

!PlayerSurrogate methodsFor: 'accessing' stamp: 'sw 7/19/2004 12:48'!
playerRepresented
	"Answer the value of playerRepresented"

	^ playerRepresented! !

!PlayerSurrogate methodsFor: 'accessing' stamp: 'sw 7/28/2004 21:16'!
playerRepresented: anObject
	"Set the value of playerRepresented"

	playerRepresented _ anObject.
	self rebuildRow.
	self setNameTo: anObject costume topRendererOrSelf externalName! !

!PlayerSurrogate methodsFor: 'accessing' stamp: 'sw 7/28/2004 22:19'!
rebuildRow
	"Rebuild the row"

	| aThumbnail aTileButton aViewerButton |
	self removeAllMorphs.
	self layoutInset: 2; cellInset: 3.
	self beTransparent.
	aThumbnail _ ThumbnailForAllPlayersTool new objectToView: playerRepresented viewSelector: #graphicForViewerTab.
	aThumbnail setBalloonText: 'Click here to reveal this object' translated.
	self addMorphBack: aThumbnail.
	aThumbnail on: #mouseUp send: #beRevealedInActiveWorld to: playerRepresented.
	
	"aMenuButton _ IconicButton new labelGraphic: Cursor menu.
	aMenuButton target: self;
		actionSelector: #playerButtonHit;

		color: Color transparent;
		borderWidth: 0;
		shedSelvedge;
		actWhen: #buttonDown.
	aMenuButton setBalloonText: 'Press here to get a menu'.
	self addMorphBack: aMenuButton."
	aViewerButton _ IconicButton new labelGraphic: (ScriptingSystem formAtKey: #Viewer).
	aViewerButton color: Color transparent; 
			actWhen: #buttonUp;
			actionSelector: #beViewed; target: playerRepresented;
			setBalloonText: 'click here to obtain this object''s Viewer' translated;
			color: Color transparent;
			borderWidth: 0;
			shedSelvedge.

	self addMorphBack: aViewerButton.

	aTileButton _ IconicButton  new borderWidth: 0.
	aTileButton labelGraphic: (TileMorph new setToReferTo: playerRepresented) imageForm.
	aTileButton color: Color transparent; 
			actWhen: #buttonDown;
			actionSelector: #tearOffTileForSelf; target: playerRepresented;
			setBalloonText: 'click here to obtain a tile that refers to this player.' translated.
	self addMorphBack: aTileButton.

"	aNameMorph _ UpdatingStringMorph new
		useStringFormat;
		target:  playerRepresented;
		getSelector: #nameForViewer;
		setNameTo: 'name';
		font: ScriptingSystem fontForNameEditingInScriptor.
	aNameMorph putSelector: #setName:.
		aNameMorph setProperty: #okToTextEdit toValue: true.
	aNameMorph step.
	self addMorphBack: aNameMorph.
	aNameMorph setBalloonText: 'Click here to edit the player''s name.'.	"

	! !

!PlayerSurrogate methodsFor: 'menu' stamp: 'sw 7/28/2004 18:05'!
revealThisObject
	"Reveal the object I represent"

	playerRepresented revealPlayerIn: ActiveWorld! !

!PlayerSurrogate methodsFor: 'menu' stamp: 'sw 7/28/2004 20:53'!
viewerForThisObject
	"Open a viewer for the object represented by the receiver"

	playerRepresented beViewed! !

AlignmentMorph subclass: #ScriptStatusControl
	instanceVariableNames: 'tickPauseWrapper tickPauseButtonsShowing scriptInstantiation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!ScriptStatusControl methodsFor: 'initialization' stamp: 'dgd 9/21/2003 15:54'!
assurePauseTickControlsShow
	"Add two little buttons that allow the user quickly to toggle between paused and ticking state"

	| colorSelector status |
	self beTransparent.
	(tickPauseWrapper isKindOf: TickIndicatorMorph) ifFalse:[
		"this was an old guy"
		tickPauseWrapper ifNotNil:[tickPauseWrapper delete].
		tickPauseWrapper _ TickIndicatorMorph new.
		tickPauseWrapper on: #mouseDown send: #mouseDownTick:onItem: to: self.
		tickPauseWrapper on: #mouseUp send: #mouseUpTick:onItem: to: self.
		tickPauseWrapper setBalloonText:'Press to toggle ticking state. Hold down to set tick rate.' translated.
		self addMorphFront: tickPauseWrapper.
	].
	status _ scriptInstantiation status.
	colorSelector _ ScriptingSystem statusColorSymbolFor: status.
	tickPauseWrapper color: (Color perform: colorSelector) muchLighter.
	tickPauseWrapper stepTime: (1000 // scriptInstantiation tickingRate max: 0).
	tickPauseWrapper isTicking: status == #ticking.
	tickPauseButtonsShowing _ true.! !

!ScriptStatusControl methodsFor: 'initialization' stamp: 'dgd 11/26/2003 15:02'!
initializeFor: aScriptInstantiation
	"Answer a control that will serve to reflect and allow the user to change the status of the receiver"

	|  statusReadout |
	self hResizing: #shrinkWrap.
	self cellInset: 2@0.
	scriptInstantiation _ aScriptInstantiation.
	tickPauseButtonsShowing _ false.

	self addMorphBack: (statusReadout _ UpdatingSimpleButtonMorph new).
	statusReadout label: aScriptInstantiation status asString font: Preferences standardButtonFont.
	statusReadout setNameTo: 'trigger'.
	statusReadout target: aScriptInstantiation; wordingSelector: #translatedStatus; actionSelector: #presentScriptStatusPopUp.
	statusReadout setBalloonText: 'when this script should run' translated.
	statusReadout actWhen: #buttonDown.

	self assurePauseTickControlsShow.
	aScriptInstantiation updateStatusMorph: self! !

!ScriptStatusControl methodsFor: 'initialization' stamp: 'ar 3/3/2001 20:34'!
intoWorld: aWorld
	super intoWorld: aWorld.
	aWorld ifNotNil:[self updateStatus].! !

!ScriptStatusControl methodsFor: 'initialization' stamp: 'ar 2/12/2001 18:24'!
maybeRemovePauseTickControls
	"If we're in the business of removing pauseTick controls when we're neither paused nor ticking, then do it now.  The present take is not to remove these controls, which explains why the body of this method is currently commented out."
	tickPauseButtonsShowing _ false.
	"note: the following is to change color of the tick control appropriately"
	self assurePauseTickControlsShow.! !

!ScriptStatusControl methodsFor: 'mouse gestures' stamp: 'ar 4/27/2001 11:33'!
mouseDownTick: evt onItem: aMorph
	aMorph color: Color veryLightGray.
	self addAlarm: #offerTickingMenu: with: aMorph after: 1000.! !

!ScriptStatusControl methodsFor: 'mouse gestures' stamp: 'ar 2/11/2001 21:02'!
mouseUpTick: evt onItem: aMorph
	self removeAlarm: #offerTickingMenu:.
	aMorph color: (Color r: 0.767 g: 0.767 b: 1.0).
	(scriptInstantiation status == #ticking) ifTrue:[
		scriptInstantiation status: #paused. 
		aMorph color: (Color r: 1.0 g: 0.774 b: 0.774).
		aMorph isTicking: false.
	] ifFalse:[
		scriptInstantiation status: #ticking. 
		aMorph color: (Color r: 0.767 g: 0.767 b: 1.0).
		aMorph isTicking: true.
	].
	scriptInstantiation updateAllStatusMorphs.! !

!ScriptStatusControl methodsFor: 'mouse gestures' stamp: 'ar 2/12/2001 18:40'!
offerTickingMenu: aMorph
	self assurePauseTickControlsShow. "to set the color"
	^scriptInstantiation presentTickingMenu! !

!ScriptStatusControl methodsFor: 'mouse gestures' stamp: 'sw 9/18/2000 19:42'!
pausedUp: ignored with: alsoIgnored
	"The paused button was hit -- respond to it"

	(scriptInstantiation status == #paused)
		ifFalse:
			[scriptInstantiation status: #paused; updateAllStatusMorphs]
! !

!ScriptStatusControl methodsFor: 'access' stamp: 'sw 9/21/2000 10:41'!
scriptInstantiation
	"Answer the scriptInstantiation object with which the receiver is associated"

	^ scriptInstantiation! !

!ScriptStatusControl methodsFor: 'script status' stamp: 'sw 9/18/2000 16:55'!
scriptIsPaused
	"Answer whether the script is paused"

	^ scriptInstantiation status == #paused! !

!ScriptStatusControl methodsFor: 'script status' stamp: 'sw 9/18/2000 16:55'!
scriptIsTicking
	"Answer whether the script is ticking"

	^ scriptInstantiation status == #ticking! !

!ScriptStatusControl methodsFor: 'mouse gestures' stamp: 'sw 9/18/2000 19:42'!
tickingUp: ignored with: alsoIgnored
	"The user hit the ticking control; make the status become one of ticking"

	scriptInstantiation status == #ticking
		ifFalse:
			[scriptInstantiation status: #ticking; updateAllStatusMorphs]
! !

!ScriptStatusControl methodsFor: 'access' stamp: 'ar 3/3/2001 20:32'!
updateStatus
	scriptInstantiation updateStatusMorph: self! !

AlignmentMorph subclass: #ScriptStatusLine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!ScriptStatusLine methodsFor: 'event handling' stamp: 'sw 11/13/2001 17:45'!
wantsKeyboardFocusFor: aSubmorph
	"No thanks"

	^ false! !

AlignmentMorph subclass: #SyntaxMorph
	instanceVariableNames: 'parseNode markerMorph'
	classVariableNames: 'AllSpecs ContrastFactor DownRightArrow SelfTile SizeScaleFactor'
	poolDictionaries: ''
	category: 'EToys-Tile Scriptors'!

!SyntaxMorph commentStamp: '<historical>' prior: 0!
A single class of morph that holds any piece of Smalltalk syntax, and allows it to be a tile.  Tiles can be dragged in or out of a method. 

In the message list pane of a Browser, choose 'tile scriptor'.  Bring up a second one to steal parts from.  If you use a Protocol Browser, and choose tiles, there will be two buttons that bring up menus with many tiles on them.

Clicking multiple times selects enclosing phrases of code.  Dragging lets you take away a copy.  Any tile may be replaced by dropping on it.  Shift-click to edit the text of any tile.  Change variable and message names, but do not change the part-of-speech (objects to selector).

Each SyntaxMorph holds a ParseNode.  After editing, the parseNode is only good as a part-of-speech indicator.  Only the Class of a parseNode is important.  It's state is not kept up to date with the tile edits (but maybe it should be).  (For MessageNodes, whether the receiver slot is nil is significant.)

The correspondence between SyntaxMorphs and parseNodes in the real parse tree is not one-to-one.  Several extra levels of SyntaxMorph were added as aligners to make the horizontal and vertical layout right.  These sometimes have nil for the parseNode.

When accept the method, we pass over the tree of SyntaxMorphs, gathering their printStrings and inserting punctuation.  See (SyntaxMorph>>printOn:indent:).  We send the result to the compiler.  (We do not use the parse tree we already have.)

To turn on type checking: 
Preferences enable: #eToyFriendly
or for testing:     World project projectParameters at: #fullCheck put: true.

Colors of tiles:  Each tile has a current color (inst car color) and a deselectedColor (a property).  The deselectedColor may be governed by the part of speech, or not.  (translateColor: is only used when a tile is created, to set deselectedColor.)  From deselectedColor (set by #setDeselectedColor), the color changes to:
	lightBrown when selected (not the submorphs) in #select
	translucent when held in the hand (allMorphs) in #lookTranslucent
	green when a drop target (allMorphs) (change the owners back) #dropColor, 
		#trackDropZones 
deselectedColor is moderated by the darkness setting, #scaleColorByUserPref:.  (as it is put into color in #color:)

Code to produce an individual tile is: 
	(SyntaxMorph new) attachTileForCode: '''abc''' nodeType: LiteralNode.
see offerTilesMenuFor:in: for many other phrases that produce useful tiles.

AssignmentNode:  If three submorphs, is a statement, and is a noun.  If one submorph, is just the left arrow.  When dropped on a variable, it creates a new assignment statement. !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'RAA 2/26/2001 22:59'!
column: aColor on: aParseNode

	| c color |
	color _ self translateColor: aColor.
	(c _ self newColumn)
		parseNode: aParseNode;
		layoutInset: c standardInset;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		color: color;
		borderWidth: 1;
		borderColor: c stdBorderColor;
		wrapCentering: #topLeft;
		cellPositioning: c standardCellPositioning.
	^c
! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'RAA 2/16/2001 15:37'!
methodNodeOuter: aNode

	^(self column: #method on: aNode) methodNodeOuter: aNode
! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'di 11/13/2000 21:12'!
noTileColor

	^ true! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'RAA 2/26/2001 23:00'!
row: aColor on: aParseNode

	| r color |
	color _ self translateColor: aColor.
	(r _ self newRow)
		parseNode: aParseNode;
		layoutInset: r standardInset;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		color: color;
		borderWidth: 1;
		borderColor: r stdBorderColor;
		wrapCentering: #topLeft;
		cellPositioning: r standardCellPositioning.
	^r! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'di 5/2/2001 09:59'!
setSize: oldExtent andMakeResizable: outerMorph
	| tw |
	(tw _ outerMorph findA: TwoWayScrollPane) ifNil: [^self].
	tw hResizing: #spaceFill;
		vResizing: #spaceFill;
		color: Color transparent;
		setProperty: #hideUnneededScrollbars toValue: true.
	outerMorph 
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		cellPositioning: #topLeft.
	outerMorph fullBounds.
! !

!SyntaxMorph classSide methodsFor: 'accessing' stamp: 'tk 9/18/2001 16:10'!
sourceCodeTemplate
	"Return the default tile method template"

	^ 'anEmpty: input1 method: input2
	"Edit the name above and the code below to make your own method"
	3 + 4.
	"Drag tiles in here.  Use the ''tiles'' and ''vars'' menus to get new tiles"
	^ ''this is a statement'' sort'  
! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'RAA 2/26/2001 22:56'!
standardInset

	^ self alansTest1 ifTrue: [1] ifFalse: [-1@-1]! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'RAA 8/24/1999 11:59'!
test

"
SyntaxMorph test
"
	self testClass: MessageNode andMethod: #asMorphicSyntaxIn:.
	"self testClass: MethodNode andMethod: #asMorphicSyntaxIn:."

! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'dvf 8/23/2003 12:20'!
testAll

	| source tree total count systNav|
"
SyntaxMorph testAll
"
	systNav _ self systemNavigation.
	count _ total _ 0.
	systNav allBehaviorsDo: [ :aClass | total _ total + 1].
'Testing all behaviors'
	displayProgressAt: Sensor cursorPoint
	from: 0 to: total
	during: [ :bar |
		systNav allBehaviorsDo: [ :aClass |
			bar value: (count _ count + 1).
			aClass selectors do: [ :aSelector |
				source _ (aClass compiledMethodAt: aSelector) getSourceFromFile.
				tree _ Compiler new 
					parse: source 
					in: aClass 
					notifying: nil.
				tree asMorphicSyntaxUsing: SyntaxMorph.
			].
		].	].


! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'dvf 8/23/2003 12:20'!
testAllMethodsOver: methodSize 
	"MessageTally spyOn: [SyntaxMorph testAllMethodsOver: 600]"
	"Add up the total layout area for syntax morphs representing all  
	methods over the given size. This is a stress-test for SyntaxMorph  
	layout. A small value for the total area is also a figure of merit in the  
	presentation of Squeak source code in general."
	"Results:  
	#(69 600 180820874 103700) 11/4  
	70% build morphs, 12% get source, 9% layout, 8% parse, 1% roundoff  
	Folded wide receivers, don't center keywords any more.  
	#(68 600 160033784 127727) 11/9  
	76% build morphs, 8% get source, 8% layout, 8% parse, 0% roundoff  
	Folded more messages, dropped extra vertical spacing in blocks.  
	#(68 600 109141704 137308) 11/10  
	79% build morphs, 6% get source, 8% layout, 7% parse  
	Folded more messages, dropped extra horizontal spacing.  
	#(68 600 106912968 132171) 11/10  
	80% build morphs, ??% get source, 11% layout, 7% parse  
	Unfolded keyword messages that will fit on one line.  
	#(68 600 96497372 132153) 11/10  
	81% build morphs, ??% get source, 8% layout, 8% parse  
	After alignment rewrite...  
	#(74 600 101082316 244799) 11/12  
	76% build morphs, 4% get source, 15% layout, 5% parse  
	After alignment rewrite...  
	#(74 600 101250620 204972) 11/15  
	74% build morphs, 6% get source, 13% layout, 7% parse  
	"
	| tree source biggies morph stats time area |
	biggies _ self systemNavigation 
				allMethodsSelect: [:cm | cm size > methodSize].
	stats _ OrderedCollection new.
	'Laying out all ' , biggies size printString , ' methods over ' , methodSize printString , ' bytes...'
		displayProgressAt: Sensor cursorPoint
		from: 1
		to: biggies size
		during: [:bar | biggies
				withIndexDo: [:methodRef :i | 
					bar value: i.
					Utilities
						setClassAndSelectorFrom: methodRef
						in: [:aClass :aSelector | 
							source _ (aClass compiledMethodAt: aSelector) getSourceFromFile.
							time _ Time
										millisecondsToRun: [tree _ Compiler new
														parse: source
														in: aClass
														notifying: nil.
											morph _ tree asMorphicSyntaxUsing: SyntaxMorph.
											area _ morph fullBounds area]].
					stats add: {methodRef. area. time}]].
	^ {{biggies size. methodSize. stats
		detectSum: [:a | a second]. stats
		detectSum: [:a | a third]}. (stats
		asSortedCollection: [:x :y | x third >= y third]) asArray}! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'di 7/30/2001 16:29'!
testClass: aClass andMethod: aSelector
	| tree |
	tree _ Compiler new 
		parse: (aClass sourceCodeAt: aSelector) 
		in: aClass 
		notifying: nil.
	(tree asMorphicSyntaxUsing: SyntaxMorph)
		parsedInClass: aClass;
		openInWindow! !

!SyntaxMorph classSide methodsFor: 'as yet unclassified' stamp: 'tk 7/19/2001 20:06'!
translateColor: aColorOrSymbol

	aColorOrSymbol isColor  ifTrue: [^ aColorOrSymbol].
	aColorOrSymbol == #comment  ifTrue: [^ Color blue lighter].
	aColorOrSymbol == #block  ifTrue: [^ Color r: 0.903 g: 1.0 b: 0.903].
	aColorOrSymbol == #method  ifTrue: [^ Color r: 0.903 g: 1.0 b: 0.903].
	aColorOrSymbol == #text  ifTrue: [^ Color r: 0.9 g: 0.9 b: 0.9].

	self noTileColor ifTrue: [^ Color r: 1.0 g: 0.839 b: 0.613].	"override"

	aColorOrSymbol == #assignment  ifTrue: [^ Color paleGreen].
	aColorOrSymbol == #keyword1  ifTrue: [^ Color paleBuff].	"binary"
	aColorOrSymbol == #keyword2  ifTrue: [^ Color paleBuff lighter].	"multipart" 
	aColorOrSymbol == #cascade  ifTrue: [^ Color paleYellow darker].	"has receiver"
	aColorOrSymbol == #cascade2  ifTrue: [^ Color paleOrange].	"one send in the cascade"
	aColorOrSymbol == #literal  ifTrue: [^ Color paleMagenta].
	aColorOrSymbol == #message  ifTrue: [^ Color paleYellow].
	aColorOrSymbol == #method  ifTrue: [^ Color white].
	aColorOrSymbol == #error  ifTrue: [^ Color red].
	aColorOrSymbol == #return  ifTrue: [^ Color lightGray].
	aColorOrSymbol == #variable  ifTrue: [^ Color paleTan].
	aColorOrSymbol == #brace  ifTrue: [^ Color paleOrange].
	aColorOrSymbol == #tempVariable  ifTrue: [^ Color paleYellow mixed: 0.75 with: Color paleGreen
		"Color yellow lighter lighter"].
	aColorOrSymbol == #blockarg2  ifTrue: [
			^ Color paleYellow mixed: 0.75 with: Color paleGreen].	"arg itself"
	aColorOrSymbol == #blockarg1  ifTrue: [^ Color paleRed].	"container"
		"yellow mixed: 0.5 with: Color white"

	^ Color tan	"has to be something!!"! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 16:03'!
aSimpleStringMorphWith: aString

	self alansTest1 ifTrue: [
		^StringMorph contents: aString font: self alansCurrentFontPreference
	].

	^StringMorph contents: aString! !

!SyntaxMorph methodsFor: 'menus' stamp: 'tk 9/27/2001 19:09'!
accept
	"Turn my current state into the text of a method.  Compile it in my class."

	^ self acceptInCategory: ClassOrganizer default! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 9/13/2001 15:28'!
acceptDroppingMorph: aMorph event: evt
	| itNoun old |
	"Two cases: 1) a phrase being dropped into a block.  Add a new line.
		2) aMorph is replacing self by dropping on it.
	For the moment, you have to drop it the right place (the end of a tile if it is complex).  We do not look at enclosing morphs"

	itNoun _ aMorph isNoun.
	self withAllOwnersDo:
		[:m | (m isSyntaxMorph and: [m isBlockNode])
				ifTrue: [m stopStepping; removeDropZones]].
	self isBlockNode & itNoun ifTrue:
		[(aMorph nodeClassIs: TempVariableNode) ifTrue:
				["If I am a BlockNode, and it is a TempVariableNode, add it into list"
				(self addBlockArg: aMorph)].
		"If I am a BlockNode and it is a noun add it as a new line"
		^ self addToBlock: aMorph event: evt].
				
	self isBlockNode ifTrue: [
		 (aMorph nodeClassIs: CommentNode) ifTrue: [^ self addToBlock: aMorph event: evt].
		 (aMorph nodeClassIs: ReturnNode) ifTrue: [^ self addToBlock: aMorph event: evt]].

	"Later add args and keywords.  later allow comments to be dropped"

	"Can't put statement, literal, assignment, or cascade into left side of assignment"
	(owner isSyntaxMorph) ifTrue:
		[(owner nodeClassIs: AssignmentNode) ifTrue:
			[(owner submorphIndexOf: self) = 1 ifTrue:
				[aMorph isAVariable ifFalse: [ ^ self]]]].

	(aMorph nodeClassIs: AssignmentNode) ifTrue: [
		itNoun ifFalse: ["create a new assignment"
			self isAVariable ifTrue: [^ self newAssignment]
					ifFalse: [^ self]]].	"only assign to a variable"

	aMorph deselect.
	(old _ owner) replaceSubmorph: self by: aMorph.	"do the normal replacement"
	(old isSyntaxMorph) ifTrue: [old cleanupAfterItDroppedOnMe].	"now owned by no one"
! !

!SyntaxMorph methodsFor: 'menus' stamp: 'tk 11/21/2000 16:35'!
acceptIfInScriptor
	| root |
	"If I am in a ScriptEditorMorph, tell my root to accept the new changes."

	(self ownerThatIsA: ScriptEditorMorph) ifNotNil: [
		root _ self rootTile.
		root ifNotNil: [root accept]]. ! !

!SyntaxMorph methodsFor: 'menus' stamp: 'RAA 2/14/2001 15:40'!
acceptIgnoring: aString
	"If I am inside a ScriptEditorMorph, tell my root to accept the new changes.  Ignore the argument, which is the string whose conents just changed."

	thisContext sender receiver removeProperty: #syntacticallyCorrectContents.
	self acceptIfInScriptor! !

!SyntaxMorph methodsFor: 'menus' stamp: 'tk 9/27/2001 17:15'!
acceptInCategory: categoryString
	"Turn my current state into the text of a method.  Compile it in my class."
	| cls sc sel |
	self isMethodNode ifFalse: [
		self rootTile == self ifTrue: [^ self].  "not in a script"
		^ self rootTile accept  "always accept at the root"].
	(cls _ self parsedInClass) ifNil: [^ self].
	sel _ cls compile: self decompile classified: categoryString.
	(sc _ self firstOwnerSuchThat: [:mm | mm class == ScriptEditorMorph]) 
		ifNotNil: [sc hibernate; unhibernate].	"rebuild the tiles"
	^ sel! !

!SyntaxMorph methodsFor: 'menus' stamp: 'tk 9/27/2001 19:12'!
acceptSilently
	"Turn my current state into the text of a method.
	Compile it in my class.  Don't rebuild the tiles."
	| cls |
	self isMethodNode ifFalse: [
		self rootTile == self ifTrue: [^ false].  "not in a script"
		^ self rootTile acceptSilently  "always accept at the root"].
	(self ownerThatIsA: ScriptEditorMorph) ifNil: [^ false].
	(cls _ self parsedInClass) ifNil: [^ false].
	cls compile: self decompile classified: 'scripts'.
	^ true! !

!SyntaxMorph methodsFor: 'menus' stamp: 'di 5/4/2001 12:14'!
acceptUnlogged
	"This is an exact copy of acceptSilently, except it does not log to the source file.
	Used for all but the last of scrolling number changes."
	| cls |
	self isMethodNode ifFalse:
		[self rootTile == self ifTrue: [^ self].  "not in a script"
		^ self rootTile acceptUnlogged  "always accept at the root"].
	(self ownerThatIsA: ScriptEditorMorph) ifNil: [^ self].
	(cls _ self parsedInClass) ifNil: [^ self].
	cls compile: self decompile
		classified: ClassOrganizer default
		withStamp: nil
		notifying: nil
		logSource: false.
! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 8/21/2001 09:36'!
actualObject
	| sub |
	"Who is self in these tiles?  Usually a Player."


	(self nodeClassIs: LiteralVariableNode) ifTrue: [
		(sub _ self findA: StringMorph) ifNil: [^ nil].
		"Need to decompile here for odd synonyms of 'self' ?"
		^ Compiler evaluate: sub contents for: Player logged: false].

	(self nodeClassIs: VariableNode) ifTrue: [
		(sub _ self findA: StringMorph) ifNil: [^ nil].
		^ References at: (self cleanUpString: sub) asSymbol ifAbsent: [nil]].

	(self nodeClassIs: LiteralNode) ifTrue: [
		(sub _ self findA: StringMorph) ifNil: [^ nil].
		^ Compiler evaluate: sub contents for: nil logged: false].

	(sub _ self findA: SyntaxMorph) ifNil: [^ nil].
	^ sub actualObject	"receiver"! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'aoy 2/15/2003 21:30'!
addArg: index 
	"I rep a SelectorNode.  My string has been replaced.  Append an argument to my owner."

	"See if any sample args are recorded"

	| sel rec aVocabulary mi sample descrip mthNode tiles |
	sel := self decompile asString asSymbol.
	rec := self receiverObject.
	sample := rec class == Error 
				ifFalse: 
					[aVocabulary := self vocabularyToUseWith: rec.
					mi := aVocabulary methodInterfaceAt: sel ifAbsent: [nil].
					mi ifNil: [5]
						ifNotNil: 
							[descrip := mi argumentVariables at: index.
							descrip sample]]
				ifTrue: [5]. 
	mthNode := self string: sample storeString toTilesIn: sample class.
	tiles := mthNode submorphs at: mthNode submorphs size - 1.	"before the ^ self"
	self owner addMorphBack: tiles! !

!SyntaxMorph methodsFor: 'layout' stamp: 'FBS 2/24/2004 14:24'!
addBlockArg: aMorph
	"Add a temporary to a block or the method.  Return true if succeed"
	"(aMorph nodeClassIs: TempVariableNode) is known to be true."
	"***NOTE: This method should be combined with addTempVar:"

	| tempHolder tt var nn |
	owner isMethodNode ifTrue: [
		^ (self addTempVar: aMorph)].	"Node for them is not inside the block"
		"If exists, drop the temp in this block and let user extend it."
	nn _ aMorph decompile string.	"name"
	(self isKnownVarName: nn) ifTrue: [^ false].	"already defined"

	tt _ self firstSubmorph.
	tempHolder _ tt firstSubmorph isSyntaxMorph 
				ifTrue: [(tt nodeClassIs: BlockArgsNode) 
							ifTrue: [tt] ifFalse: [nil]]
				ifFalse: [nil].

	tempHolder ifNil: ["make new row"
		tempHolder _ self addRow: #blockarg1 on: (BlockArgsNode new).
		tempHolder addNoiseString: self noiseBeforeBlockArg.
		tempHolder submorphs last firstSubmorph emphasis: TextEmphasis bold emphasisCode.
		tempHolder useRoundedCorners.

		self addMorphFront: tempHolder.
		aMorph parseNode name: nn key: nn code: nil.
		aMorph parseNode asMorphicSyntaxIn: tempHolder.
		tempHolder cleanupAfterItDroppedOnMe.
		^ true].

	"Know this variable is not present, so add it"

	aMorph parseNode name: nn key: nn code: nil.
	tempHolder addMorphBack: (tempHolder transparentSpacerOfSize: 4@4).
	var _ tempHolder addRow: #tempVariable on: aMorph parseNode.
	var layoutInset: 1.
	var addMorphBack: (self aSimpleStringMorphWith: nn).
	var cleanupAfterItDroppedOnMe.
	^ true
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'RAA 3/25/2001 16:15'!
addColumn: aColorOrSymbol on: aNode
	| col |
	self addMorphBack: (col _ self class column: aColorOrSymbol on: aNode).

"col setProperty: #howCreated toValue: thisContext longStack."

	self alansTest1 ifTrue: [
		(aColorOrSymbol == #block and: [self isMethodNode not]) ifTrue: [
			col setConditionalPartStyle.
		].
	].
	^ col
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'FBS 2/24/2004 14:44'!
addNoiseString: aNoiseString

	^self addNoiseString: aNoiseString emphasis: TextEmphasis normal emphasisCode.
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'RAA 3/25/2001 16:17'!
addNoiseString: aNoiseString emphasis: anInteger

	self alansTest1 ifFalse: [^self].
	^(self addColumn: #keyword1 on: nil)
		layoutInset: 1;
		addMorphBack: ((self noiseStringMorph: aNoiseString)  emphasis: anInteger)
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'RAA 2/15/2001 19:22'!
addRow: aColorOrSymbol on: aNode

	| row |
	self addMorphBack: (row _ self class row: aColorOrSymbol on: aNode).

"row setProperty: #howCreated toValue: thisContext longStack."

	^row
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 8/24/2001 15:15'!
addSingleKeywordRow: aStringLikeItem

	| row sMorph modifiedString |

	(row _ self class row: #text on: nil) borderWidth: 1.

	modifiedString _ self substituteKeywordFor: aStringLikeItem.
	sMorph _ self addString: modifiedString special: true.
	sMorph font: (self fontToUseForSpecialWord: modifiedString).
	modifiedString = aStringLikeItem ifFalse: [
		sMorph setProperty: #syntacticallyCorrectContents toValue: aStringLikeItem].

	row addMorph: sMorph.
	self addMorphBack: row.
	^row! !

!SyntaxMorph methodsFor: 'layout' stamp: 'RAA 4/4/2001 13:15'!
addString: literalOrVarName special: aBoolean

	| answer |
	"Create and return an UpdatingStringMorph containing the value.  Use an UpdatingStringMorph, so it can inform its owner when it has been edited. Keep the getSelector being nil"

	answer _ (self anUpdatingStringMorphWith: literalOrVarName special: aBoolean)
		target: self;
		putSelector: #acceptIgnoring:;
		useStringFormat.

	^answer
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'FBS 2/24/2004 14:23'!
addTempVar: aMorph 
	"know we are a block inside a MethodNode" 
	"(aMorph nodeClassIs: TempVariableNode) is known to be true."
	| tempHolder ii tt var nn |
	nn _ aMorph decompile string.	"name"
	(self isKnownVarName: nn) ifTrue: [^ false].	"already defined"

	tempHolder _ nil.
	(ii _ owner submorphIndexOf: self) = 1 ifFalse: [
		tt _ owner submorphs at: ii - 1.
		tt isSyntaxMorph ifTrue: [
			(tt nodeClassIs: MethodTempsNode) ifTrue: [tempHolder _ tt].
			(tt nodeClassIs: UndefinedObject) ifTrue: [tempHolder _ tt findA: MethodTempsNode]]].

	tempHolder ifNil: [
		tempHolder _ owner addRow: #tempVariable on: MethodTempsNode new.
		tempHolder addNoiseString: self noiseBeforeBlockArg.
		tempHolder submorphs last firstSubmorph emphasis: TextEmphasis bold emphasisCode.
		tempHolder useRoundedCorners.

		owner addMorph: tempHolder inFrontOf: self.
		aMorph parseNode name: nn key: nn code: nil.
		aMorph parseNode asMorphicSyntaxIn: tempHolder.
		tempHolder cleanupAfterItDroppedOnMe.
		^ true].

	aMorph parseNode name: nn key: nn code: nil.
	tempHolder addMorphBack: (tempHolder transparentSpacerOfSize: 4@4).
	var _ tempHolder addRow: #tempVariable on: aMorph parseNode.
	var layoutInset: 1.
	var addMorphBack: (self addString: nn special: false).
	var cleanupAfterItDroppedOnMe.
	^ true! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'FBS 2/24/2004 14:23'!
addTemporaries: temporaries 
	| tempMorph outerMorph w2 |
	temporaries notEmpty ifFalse: [^self].
	self alansTest1 
		ifFalse: 
			[tempMorph := self addRow: #tempVariable on: MethodTempsNode new.
			temporaries do: [:temp | temp asMorphicSyntaxIn: tempMorph]
				separatedBy: 
					[tempMorph addMorphBack: (tempMorph transparentSpacerOfSize: 4 @ 4)].
			^self].
	outerMorph := self addRow: #tempVariable on: nil.
	outerMorph setSpecialTempDeclarationFormat1.
	outerMorph 
		addMorphBack: (w2 := self noiseStringMorph: self noiseBeforeBlockArg).
	w2 emphasis: TextEmphasis bold emphasisCode.
	tempMorph := outerMorph addRow: #tempVariable on: MethodTempsNode new.
	tempMorph setSpecialTempDeclarationFormat2.
	temporaries do: 
			[:temp | 
			tempMorph 
				addToken: temp name
				type: #tempVariableDeclaration
				on: temp]
		separatedBy: [tempMorph addMorphBack: self tokenVerticalSeparator]! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'tk 7/31/2001 17:27'!
addTemporaryControls

	| row stdSize |
	
	stdSize _ 8@8.
	row _ AlignmentMorph newRow
		color: Color transparent;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap.
	self addMorphBack: row.

	{
		Morph new
			extent: stdSize; 
			color: Color paleBlue darker;
			setBalloonText: 'Change the contrast';
			on: #mouseUp send: #controlContrast2: to: self;
			on: #mouseMove send: #controlContrast2: to: self;
			on: #mouseDown send: #controlContrast2: to: self.

	"Removed because it's default is giant tiles, which no one wants. --tk
		Morph new
			extent: stdSize; 
			color: Color green;
			setBalloonText: 'Change basic spacing';
			on: #mouseUp send: #controlSpacing2: to: self;
			on: #mouseMove send: #controlSpacing2: to: self;
			on: #mouseDown send: #controlSpacing2: to: self.
	"

		Morph new
			extent: stdSize; 
			color: Color lightRed;
			setBalloonText: 'Change basic style';
			on: #mouseUp send: #changeBasicStyle to: self.

	} do: [ :each |
		row addMorphBack: each.
		row addMorphBack: (self transparentSpacerOfSize: stdSize).
	].
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 8/10/2001 09:57'!
addTextRow: aStringLikeItem

	| row tt |
	(row _ self class row: #text on: nil) borderWidth: 1.
	(tt _ TextMorph new) contents: aStringLikeItem.
	row addMorph: tt.
	"row addMorph: (self addString: (aStringLikeItem copyWithout: Character cr) special: false)."
	self addMorphBack: row.
	^row! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 7/25/2001 10:01'!
addToBlock: aMorph event: evt
	"Insert a new line of code.  Figure out who it goes before.  If evt Y is within an existing line (to the right of a tile), then replace that tile."

	| whereDropped dropBefore replace |
	whereDropped _ "self pointFromWorld:" evt cursorPoint.
	dropBefore _ self submorphs 
		detect: [:each | each isSyntaxMorph ifTrue: [
			whereDropped y < each top ifTrue: [true]	"before this one"
				ifFalse: [whereDropped y < each bottom 
							ifTrue: [replace _ true]	"replace this one"
							ifFalse: [false]]]] "try next line"
		ifNone: [nil].
	(aMorph nodeClassIs: ReturnNode) ifTrue: [dropBefore _ nil].
		"Returns are always at the end. (Watch out for comments)"

	dropBefore 
		ifNil: [self addMorphBack: aMorph]
		ifNotNil: [
			replace ifNotNil: [aMorph deselect.
				self replaceSubmorph: dropBefore by: aMorph.	"replace it!!"
				^ dropBefore cleanupAfterItDroppedOnMe].	"now owned by no one"
			self addMorph: aMorph inFrontOf: dropBefore].
	self cleanupAfterItDroppedOnMe.
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'FBS 2/24/2004 14:22'!
addToken: aString type: aColorOrSymbol on: aNode

	| sMorph modifiedString noiseWord row |

	row _ (self addRow: aColorOrSymbol on: aNode) layoutInset: 1.
	self alansTest1 ifFalse: [
		sMorph _ self addString: aString special: false.
		row addMorphBack: sMorph.
		^row
	].

	noiseWord _ [ :w |
		w ifNotNil: [
			row 
				addMorphBack: (self noiseStringMorph: w);
				addMorphBack: (self tokenVerticalSeparator)
		].
	].
	(self shouldBeBrokenIntoWords: aColorOrSymbol) ifTrue: [
		modifiedString _ self substituteKeywordFor: aString.
		sMorph _ self addString: modifiedString special: (aColorOrSymbol ~= #assignmentArrow).
			"(#(unary keywordGetz keywordSetter unaryGetter) includes: aColorOrSymbol)"
		modifiedString = aString ifFalse: [
			sMorph setProperty: #syntacticallyCorrectContents toValue: aString].
		sMorph setProperty: #syntacticReformatting toValue: aColorOrSymbol;
			contents: modifiedString.
	] ifFalse: [
		sMorph _ self addString: (modifiedString _ aString) special: false.
	].
	(#(keyword2 upArrow) includes: aColorOrSymbol) ifTrue: [
		sMorph 
			font: (self fontToUseForSpecialWord: modifiedString).
	].
	(#(keyword2 unary assignmentArrow methodHeader1 methodHeader2) includes: aColorOrSymbol) ifTrue: [
		sMorph emphasis: TextEmphasis bold emphasisCode.
	].
	aColorOrSymbol == #blockarg1 ifTrue: [
	].
	(aColorOrSymbol == #variable or: [aColorOrSymbol == #tempVariable]) ifTrue: [
		aString = 'self' ifTrue: [
			sMorph setProperty: #wordyVariantOfSelf toValue: true.
		].
		noiseWord value: (self noiseWordBeforeVariableNode: aNode string: aString).
	].

	row addMorphBack: sMorph.
	^row! !

!SyntaxMorph methodsFor: 'layout' stamp: 'RAA 3/25/2001 17:22'!
addTokenSpecialCase: aString type: aColorOrSymbol on: aNode

	| sMorph modifiedString noiseWord col |

	noiseWord _ nil.
	sMorph _ self addString: aString special: false.
	(aColorOrSymbol == #keyword2) ifTrue: [
		modifiedString _ aString = 'if:' ifTrue: ['Test'] ifFalse: ['Yes'].
		sMorph 
			font: (self fontToUseForSpecialWord: modifiedString); 
			setProperty: #syntacticallyCorrectContents toValue: aString;
			contents: modifiedString.
	].

	col _ (self addRow: aColorOrSymbol on: aNode) layoutInset: 1.
	noiseWord ifNotNil: [
		col 
			addMorphBack: (self noiseStringMorph: noiseWord);
			addMorphBack: (self transparentSpacerOfSize: 3@1)
	].
	col addMorphBack: sMorph.
	^col! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 8/24/2001 15:21'!
addUnaryRow: aStringLikeItem style: aSymbol

	| row sMorph modifiedString fontToUse |

	(row _ self class row: #text on: nil) borderWidth: 1.
	modifiedString _ self substituteKeywordFor: aStringLikeItem.
	sMorph _ self addString: modifiedString special: true.
	fontToUse _ self fontToUseForSpecialWord: modifiedString.

	sMorph 
		font: fontToUse emphasis: 1;
		setProperty: #syntacticReformatting toValue: #unary.
	modifiedString = aStringLikeItem ifFalse: [
		sMorph setProperty: #syntacticallyCorrectContents toValue: aStringLikeItem].
	row addMorph: sMorph.
	self addMorphBack: row.
	^row! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'yo 12/3/2004 17:01'!
alanBinaryPostRcvr: aNode key: key args: args

	| nodeWithNilReceiver row |

"==
Repeat for collection [ collect ( from foo. blah blah foo blah) ]
Repeat for 1 to 50 [ do ( from i. blah blab i blah ) ]
=="

	nodeWithNilReceiver _ aNode copy receiver: nil.
	(row _ self addRow: #keyword2 on: nodeWithNilReceiver)
		borderWidth: 1;
		parseNode: (nodeWithNilReceiver as: MessageNode);
		borderColor: row stdBorderColor.
	row addToken: key asString
		type: #binary
		on: (SelectorNode new key: key asString code: nil "fill this in?").
	args first asMorphicSyntaxIn: row.
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'tk 7/30/2001 14:52'!
alanKeywordMessage: aNode isAConditional: template key: key args: args

	| nodeWithNilReceiver column keywords row onlyOne |

	(key == #collect: and: [args first isKindOf: BlockNode]) ifTrue: [
		^self
			alanKwdCollect: aNode 
			isAConditional: template 
			key: key 
			args: args
	].
	key == #repeatFor:doing: ifTrue: [
		^self
			alanKwdRepeatForDoing: aNode 
			isAConditional: template 
			key: key 
			args: args
	].
	key == #if:do: ifTrue: [
		^self
			alanKwdIfDo: aNode 
			isAConditional: template 
			key: key 
			args: args
	].
	(args size = 1 and: [key endsWith: 'Getz:']) ifTrue: [
		^self
			alanKwdSetter: aNode 
			isAConditional: 0 
			key: key 
			args: args
	].
	(args size = 1 and: [self isStandardSetterKeyword: key]) ifTrue: [
		^self
			alanKwdSetter2: aNode 
			isAConditional: 0 
			key: key 
			args: args
	].
	nodeWithNilReceiver _ aNode copy receiver: nil.
	template = 1 ifTrue: [
		self listDirection: #topToBottom.
	].
	column _ self addColumn: #keyword1 on: nodeWithNilReceiver.
	keywords _ key keywords.
	onlyOne _ args size = 1.
	onlyOne ifFalse: ["necessary for three keyword messages!!"
		column setProperty: #deselectedBorderColor toValue: column compoundBorderColor].
	keywords
		with: (args first: keywords size)
		do: [:kwd :arg |
			template = 1 ifTrue: [
				column addMorphBack: (column transparentSpacerOfSize: 3@3).
			].
			(row _ column addRow: #keyword2 on: nodeWithNilReceiver)
				parseNode: (nodeWithNilReceiver as: 
						(onlyOne ifTrue: [MessageNode] ifFalse: [MessagePartNode]));
				borderColor: row stdBorderColor.
			template = 1 ifTrue: [row addMorphBack: (row transparentSpacerOfSize: 20@6)].
			row addToken: kwd
				type: #keyword2
				on: (onlyOne ifTrue: [SelectorNode new key: kwd code: nil "fill this in?"]
								ifFalse: [KeyWordNode new]).
			(arg asMorphicSyntaxIn: row) setConditionalPartStyle.
		].
	onlyOne ifTrue: [
		self replaceSubmorph: column by: row.
		column _ row.
	].
			
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'RAA 3/25/2001 16:31'!
alanKwdCollect: aNode isAConditional: template key: key args: args

	| nodeWithNilReceiver row kwdHolder |

	nodeWithNilReceiver _ aNode copy receiver: nil.
	(row _ self addRow: #keyword2 on: nodeWithNilReceiver)
		borderWidth: 1;
		parseNode: (nodeWithNilReceiver as: MessageNode);
		borderColor: row stdBorderColor.
	kwdHolder _ row
		addToken: key
		type: #keyword2
		on: (SelectorNode new key: key code: nil "fill this in?").
	kwdHolder firstSubmorph 
		setProperty: #syntacticallyCorrectContents toValue: key asString;
		contents: ''.

	args first asMorphicCollectSyntaxIn: row.
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'tk 7/30/2001 14:52'!
alanKwdIfDo: aNode isAConditional: template key: key args: args
	"(know it has more than one arg)"
	| nodeWithNilReceiver column keywords row |

	nodeWithNilReceiver _ aNode copy receiver: nil.
	column _ self addColumn: #keyword1 on: nodeWithNilReceiver.
	"column borderColor: column compoundBorderColor."
	keywords _ key keywords.
	keywords
		with: (args first: keywords size)
		do: [:kwd :arg |
			(row _ column addRow: #keyword2 on: nodeWithNilReceiver)
				parseNode: (nodeWithNilReceiver as: MessagePartNode).
			kwd = 'do:' ifTrue: [
				row addMorphBack: (row transparentSpacerOfSize: 26@6).
			] ifFalse: [
				row addMorphBack: (row transparentSpacerOfSize: 10@6).
			].
			row addTokenSpecialCase: kwd
				type: #keyword2
				on: KeyWordNode new.
			(arg asMorphicSyntaxIn: row) setConditionalPartStyle.
		].
			
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'RAA 2/28/2001 10:16'!
alanKwdRepeatForDoing: aNode isAConditional: template key: key args: args

	| nodeWithNilReceiver row column keywords |

	nodeWithNilReceiver _ aNode copy receiver: nil.
	column _ self addColumn: #keyword1 on: nodeWithNilReceiver.
	keywords _ key keywords.
	keywords
		with: (args first: keywords size)
		do: [:kwd :arg |
			(row _ column addRow: #keyword2 on: nodeWithNilReceiver)
				parseNode: (nodeWithNilReceiver as: MessagePartNode).
			row addToken: kwd
				type: #keyword2
				on: KeyWordNode new.
			(arg asMorphicSyntaxIn: row) setConditionalPartStyle.
		].
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'FBS 2/24/2004 14:22'!
alanKwdSetter2: aNode isAConditional: template key: key args: args
	"translates
		foo setHeading: 0
	to
		foo's heading _ 0
	"
	| kwdHolder wordy |
	kwdHolder _ self
		addToken: key
		type: #keywordSetter
		on: (SelectorNode new key: key code: nil "fill this in?").
	wordy _ self translateToWordySetter: key.
	kwdHolder firstSubmorph 
		setProperty: #syntacticReformatting toValue: #keywordSetter;
		contents: wordy;
		emphasis: TextEmphasis bold emphasisCode.
	wordy = key asString ifFalse: [
		kwdHolder firstSubmorph 
			setProperty: #syntacticallyCorrectContents toValue: key asString].

	(args first asMorphicSyntaxIn: self) setConditionalPartStyle
			
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'FBS 2/24/2004 14:22'!
alanKwdSetter: aNode isAConditional: template key: key args: args

	| nodeWithNilReceiver row kwdHolder |

	nodeWithNilReceiver _ aNode copy receiver: nil.
	(row _ self addRow: #keyword2 on: nodeWithNilReceiver)
		borderWidth: 1;
		parseNode: (nodeWithNilReceiver as: MessageNode);
		borderColor: row stdBorderColor.
	row addNoiseString: '''s' emphasis: TextEmphasis bold emphasisCode.
	kwdHolder _ row
		addToken: key
		type: #keywordGetz
		on: (SelectorNode new key: key code: nil "fill this in?").
	kwdHolder firstSubmorph 
		setProperty: #syntacticReformatting toValue: #keywordGetz;
		setProperty: #syntacticallyCorrectContents toValue: key asString;
		contents: (self splitAtCapsAndDownshifted: (key asString allButLast: 5));
		emphasis: TextEmphasis bold emphasisCode.
	row addNoiseString: '_' emphasis: TextEmphasis bold emphasisCode.

	(args first asMorphicSyntaxIn: row) setConditionalPartStyle
			
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'FBS 2/24/2004 14:21'!
alanUnaryGetter: aNode key: key
	"I am a MessageNode.  Fill me with a SelectorNode {getX} whose string is {'s x}.  All on one level."

	| selSyn usm wordy |
	selSyn _ self
		addToken: key
		type: #unaryGetter
		on: (SelectorNode new key: key code: nil "fill this in?").
	usm _ selSyn firstSubmorph.
	usm setProperty: #syntacticReformatting toValue: #unaryGetter.
	wordy _ self translateToWordyGetter: key.
	wordy = key asString ifFalse: [
		usm setProperty: #syntacticallyCorrectContents toValue: key asString].
	usm contents: wordy; emphasis: TextEmphasis bold emphasisCode.
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'RAA 4/4/2001 12:49'!
alanUnaryPostRcvr: aNode key: key selector: selector

	| row |

	(self isStandardGetterSelector: key) ifTrue: [
		^self alanUnaryGetter: aNode key: key
	].
	row _ (self addUnaryRow: key style: #unary) layoutInset: 1.
	^ row parseNode: selector
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 23:24'!
alansCurrentFontPreference

	^nil		"StrikeFont familyName: 'ComicBold' size: 16"! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'dgd 2/22/2003 13:38'!
alansMessageNode: aNode receiver: receiver selector: selector keywords: key arguments: args 
	| receiverMorph testAndReceiver anotherSelf wordyMorph template |
	template := self alansTemplateStyleFor: key.
	receiver ifNotNil: 
			["i.e. not a cascade"

			anotherSelf := self constructSelfVariant: receiver and: key.
			anotherSelf ifNotNil: 
					[wordyMorph := self addString: anotherSelf special: false.
					wordyMorph setProperty: #wordyVariantOfSelf toValue: true.
					self addMorph: wordyMorph.
					self layoutInset: 1.
					^self].
			testAndReceiver := self.
			template = 1 
				ifTrue: 
					[testAndReceiver := self addRow: #keyword1 on: nil.
					self setSpecialOuterTestFormat.
					testAndReceiver addNoiseString: 'Test'].
			false 
				ifTrue: 
					["template = 2"

					testAndReceiver := self addRow: #keyword1 on: nil.
					"self setSpecialOuterTestFormat."
					testAndReceiver addNoiseString: 'Repeat for'].
			receiverMorph := receiver asMorphicSyntaxIn: testAndReceiver.
			template = 1 ifTrue: [receiverMorph setConditionalPartStyle]].

	"unary messages"
	args isEmpty 
		ifTrue: 
			[^self 
				alanUnaryPostRcvr: aNode
				key: key
				selector: selector].

	"binary messages"
	key last = $: 
		ifFalse: 
			[^self 
				alanBinaryPostRcvr: aNode
				key: key
				args: args].

	"keyword messages"
	receiverMorph ifNotNil: [receiverMorph setConditionalPartStyle].
	self setSpecialOuterTestFormat.
	self 
		alanKeywordMessage: aNode
		isAConditional: template
		key: key
		args: args! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 13:38'!
alansTemplateStyleFor: key

	(#(ifTrue: ifFalse: ifTrue:ifFalse: ifFalse:ifTrue:) includes: key) ifTrue: [^1].
	(#(do: collect:) includes: key) ifTrue: [^2].
	(#(if:do:) includes: key) ifTrue: [^3].
	^0
! !

!SyntaxMorph methodsFor: 'formatting options' stamp: 'di 2/21/2001 11:42'!
alansTest1
	
	| root |

	root _ self rootTile ifNil: [self].
	^root valueOfProperty: #alansNewStyle ifAbsent: [self usingClassicTiles not]! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/28/2001 13:46'!
anUpdatingStringMorphWith: aString special: aBoolean

	self alansTest1 ifTrue: [
		^(aBoolean ifTrue: [SyntaxUpdatingStringMorph] ifFalse: [UpdatingStringMorph])
			 contents: aString
			font: self alansCurrentFontPreference
	].
	^UpdatingStringMorph contents: aString! !

!SyntaxMorph methodsFor: 'type checking' stamp: 'sw 2/23/2001 23:40'!
argTypeFor: aSelector
	"Answer the type of the argument of this selector.  Return #unknown if not found."

	| itsInterface |
	aSelector numArgs = 0 
		ifTrue: [self inform: aSelector, ' does not take an argument'. ^ #error "7"].
	itsInterface _ self currentVocabulary methodInterfaceAt: aSelector ifAbsent:
		[^ #unknown].
	^ itsInterface typeForArgumentNumber: 1! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 9/20/2001 13:21'!
argumentNodes
	"Return a collection of this message's argument nodes.  "

	| cls coll rec |
	parseNode ifNil: [^ #()].
	cls _ parseNode class.
	cls == SelectorNode ifTrue: [^ #()].
	cls == KeyWordNode ifTrue: [^ #()].

	coll _ OrderedCollection new.
	rec _ self receiverNode.
	submorphs do: [:sub | 
		(sub isSyntaxMorph and: [sub ~~ rec]) ifTrue: [
			sub isNoun ifTrue: [coll addLast: sub]	"complete arg"
				ifFalse: [coll _ coll, sub argumentNodes]]].	"MessagePartNode, MessageNode with no receiver"
	^ coll! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 8/24/2001 13:24'!
assignmentArrow
	"Offer to embed this variable in a new assignment statement.  (Don't confuse this with upDownAssignment:, which runs the up and down arrows that rotate among assignment types.)"
	| rr |

	self isAVariable ifFalse: [^ nil].
	self isDeclaration ifTrue: [^ nil].
	^ (rr _ RectangleMorph new)
		extent: 11@13; borderWidth: 1; color: Color lightGreen;
		borderColor: Color gray;
		addMorph: ((self noiseStringMorph: '_') topLeft: rr topLeft + (3@0));
		on: #mouseUp send: #newAssignment to: self
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'RAA 2/27/2001 14:19'!
assignmentNode: aNode variable: variable value: value

	| row v expMorph |

	row _ self addRow: #assignment on: aNode.
	v _ variable asMorphicSyntaxIn: row.
	self alansTest1 ifTrue: [v setConditionalPartStyle; layoutInset: 2].
	row addToken: ' _ ' type: #assignmentArrow on: aNode.
	expMorph _ value asMorphicSyntaxIn: row.
	self alansTest1 ifTrue: [
		row setSpecialOuterTestFormat.
		(expMorph hasProperty: #deselectedColor) ifFalse: [expMorph setConditionalPartStyle].
	].
	^row
! !

!SyntaxMorph methodsFor: 'new tiles' stamp: 'FBS 2/24/2004 14:32'!
attachTileForCode: expression nodeType: nodeClass
	| nn master tile |
	"create a new tile for a part of speech, and put it into the hand"

	"a few special cases"
	expression = 'self' ifTrue: [
		^ (((self string: expression toTilesIn: Object) 
				findA: ReturnNode) findA: nodeClass) attachToHand].

	expression = '<me by name>' ifTrue: ["Tile for the variable in References"
		nn _ nodeClass knownName ifNil: [#+].
		(References at: nn asSymbol ifAbsent: [nil]) == nodeClass ifTrue: [
			^ self attachTileForCode: nn nodeType: LiteralVariableNode].
		"otherwise just give a tile for self"
		^ self attachTileForCode: 'self' nodeType: VariableNode].

	expression = '<assignment>' ifTrue: ["do something really special"
		master _ self class new.
		master addNoiseString: '  _  ' emphasis: TextEmphasis bold emphasisCode.
		tile _ master firstSubmorph.
		^ (tile parseNode: AssignmentNode new) attachToHand].	"special marker"
		"When this is dropped on a variable, enclose it in 
			a new assignment statement"

	"general case -- a tile for a whole line of code is returned"
	^ ((self string: expression toTilesIn: Object) 
				findA: nodeClass) attachToHand.! !

!SyntaxMorph methodsFor: 'new tiles' stamp: 'tk 9/7/2001 11:21'!
attachToHand
	"Adjust my look and attach me to the hand"

	self roundedCorners.
	ActiveHand attachMorph: self.
	Preferences tileTranslucentDrag
		ifTrue: [self lookTranslucent.
			self align: self center with: ActiveHand position "+ self cursorBaseOffset"]
		ifFalse: [self align: self topLeft with: ActiveHand position + self cursorBaseOffset]
! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'RAA 8/24/1999 17:57'!
balloonText

	^(('Value: ',(self getCurrentValue ifNil: [^nil])) 
		withNoLineLongerThan: 35) truncateWithElipsisTo: 300! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'dgd 2/22/2003 13:39'!
blockNode: aNode arguments: arguments statements: statements 
	| row column |
	column := self addColumn: #block on: aNode.
	self alansTest1 ifFalse: [column layoutInset: 5 @ -1].
	self alansTest1 
		ifTrue: 
			[column setProperty: #deselectedBorderColor toValue: self lighterColor].
	aNode addCommentToMorph: column.
	arguments notEmpty 
		ifTrue: 
			[row := column addRow: #blockarg1 on: BlockArgsNode new.
			row addNoiseString: self noiseBeforeBlockArg.
			arguments do: 
					[:arg | 
					row 
						addToken: arg name
						type: #blockarg2
						on: arg]].
	statements do: 
			[:each | 
			(row := each asMorphicSyntaxIn: column) borderWidth: 1.
			self alansTest1 ifTrue: [row setSpecialOuterTestFormat].
			each addCommentToMorph: column].
	^column! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'FBS 2/24/2004 14:32'!
blockNodeCollect: aNode arguments: arguments statements: statements 
	| row column c2 r2 r3 |
	column := self addColumn: #blockCollectOnly on: aNode.
	self alansTest1 ifFalse: [column layoutInset: 5 @ -1].
	aNode addCommentToMorph: column.
	arguments notEmpty 
		ifTrue: 
			[row := column addRow: #blockarg1 on: BlockArgsNode new.
			row addNoiseString: 'collect using' emphasis: TextEmphasis bold emphasisCode.
			r3 := row addRow: #blockarg1b on: nil.	"aNode"
			r3 setConditionalPartStyle.
			arguments do: 
					[:arg | 
					r3 
						addToken: arg name
						type: #blockarg2
						on: arg]].
	r2 := column addRow: #block on: aNode.
	r2 setProperty: #ignoreNodeWhenPrinting toValue: true.
	r2 addNoiseString: self noiseBeforeBlockArg emphasis: TextEmphasis bold emphasisCode.
	c2 := r2 addColumn: #block on: aNode.
	c2 setProperty: #ignoreNodeWhenPrinting toValue: true.
	statements do: 
			[:each | 
			(each asMorphicSyntaxIn: c2) borderWidth: 1.
			each addCommentToMorph: c2].
	^column! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'di 12/13/2000 15:25'!
borderColor: colorOrSymbolOrNil

	borderColor = colorOrSymbolOrNil ifFalse: [
		borderColor _ colorOrSymbolOrNil.
		self bounds area < 40000
			ifTrue: [self invalidRect: self bounds]
			ifFalse: [(self bounds areasOutside: (self bounds insetBy: self borderWidth))
						do: [:r | self invalidRect: r]]].
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'RAA 2/22/2001 17:00'!
cascadeNode: aNode receiver: receiver messages: messages
	| row |

	self alansTest1 ifTrue: [
		row _ self addColumn: #cascade on: aNode.
		row setSpecialOuterTestFormat.
	] ifFalse: [
		row _ self addRow: #cascade on: aNode
	].
	receiver asMorphicSyntaxIn: row.
	messages do: [:m | m asMorphicSyntaxIn: row].
	^ row

"	(node2 _ aNode copy) receiver: nil messages: messages.
	cascadeMorph _ row addColumn: #cascade2 on: node2.
	messages do: [ :m | m asMorphicSyntaxIn: cascadeMorph].
	^row
"
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'RAA 2/16/2001 16:34'!
changeBasicStyle

	self removeAllMorphs.
	self setProperty: #alansNewStyle toValue: self alansTest1 not.
	self methodNodeOuter: parseNode
! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'gk 2/23/2004 21:08'!
changeSound: upDown
	| ind arg st soundChoices index it current |
	"move in the list of sounds.  Adjust arg tile after me"

	ind _ owner submorphs indexOf: self.
	arg _ owner submorphs atWrap: ind+1.
	arg isSyntaxMorph ifFalse: [^ self].
	st _ arg submorphs detect: [:mm | mm isKindOf: StringMorph] ifNone: [^ self].
	soundChoices _ SoundService default sampledSoundChoices.
	current _ st contents copyFrom: 2 to: st contents size-1.	"remove string quotes"
	index _ soundChoices indexOf: current.
	index > 0 ifTrue:
		[st contents: (it _ soundChoices atWrap: index + upDown) printString.
		self playSoundNamed: it].
! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'RAA 4/4/2001 12:36'!
cleanUpString: stringSubMorph

	| style rawData |
	^ stringSubMorph 
		valueOfProperty: #syntacticallyCorrectContents 
		ifAbsent: [
			style _ stringSubMorph valueOfProperty: #syntacticReformatting.
			rawData _ stringSubMorph contents.
			 (#(unary tempVariableDeclaration blockarg2 methodHeader1 tempVariable variable) includes: style) ifTrue: [
				rawData _ self unSpaceAndUpShift: rawData appending: nil.
			].
			style == #keywordGetz ifTrue: [
				rawData _ self unSpaceAndUpShift: rawData appending: 'Getz:'.
			].
			style == #keywordSetter ifTrue: [
				rawData _ self unSpaceAndUpShift: 'set ',rawData appending: ':'.
			].
			style == #unaryGetter ifTrue: [
				rawData _ self unSpaceAndUpShift: 'get ',rawData appending: nil.
			].
			(#(keyword2 methodHeader2) includes: style)  ifTrue: [
				rawData _ self unSpaceAndUpShift: rawData appending: ':'.
			].
			rawData
		]
! !

!SyntaxMorph methodsFor: 'dropping/grabbing' stamp: 'di 1/29/2001 16:23'!
cleanupAfterItDroppedOnMe
	"A tile just dropped into me.  Clean up"

	self layoutChanged.  "** Isn't this already implied by the addMorph: ?"
	"Auto-accept on drop if in a scriptor"
	self acceptIfInScriptor.! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 7/19/2001 20:04'!
color: aColorOrSymbol

	| deselectedColor cc |
	aColorOrSymbol isColor ifTrue: [
		self valueOfProperty: #deselectedColor ifAbsent: ["record my color the first time"
			self setProperty: #deselectedColor toValue: aColorOrSymbol.
			^ super color: (self scaleColorByUserPref: aColorOrSymbol)].
		^ super color: aColorOrSymbol].

	deselectedColor _ self valueOfProperty: #deselectedColor ifAbsent: [nil].
	deselectedColor ifNotNil: [^ super color: (self scaleColorByUserPref: deselectedColor)].

	aColorOrSymbol == #comment  ifTrue: [^ self color: Color blue lighter].
	SyntaxMorph noTileColor ifTrue: [	"override"
		^ self color: Color transparent].	"Fix this to be real color!!"

	(cc _ self class translateColor: aColorOrSymbol) isColor
		ifTrue: [^ self color: cc]
		ifFalse: [Transcript show: aColorOrSymbol, ' needs to be handled in translateColor:'; cr.
			^ self color: Color transparent].	"help!!"! !

!SyntaxMorph methodsFor: 'change reporting' stamp: 'tk 9/28/2001 13:36'!
colorChangedForSubmorph: colorPatch
	| sel newSel cc ms phrase completeMsg |
	"reporting a color change"

	(self nodeClassIs: MessageNode) ifFalse: [^ nil].
	(sel _ self selector) ifNil: [^ nil].
	(Color colorNames includes: sel) | (sel == #r:g:b:) ifFalse: [^ nil].
		"a standard color name"
	"replace self with new tiles from the color"
	(newSel _ (cc _ colorPatch color) name) 
		ifNil: [ms _ MessageSend receiver: Color selector: #r:g:b: arguments: 
				(Array with: cc red with: cc green with: cc blue).
			phrase _ ms asTilesIn: Color globalNames: true]
		ifNotNil: [ms _ MessageSend receiver: Color selector: newSel arguments: #().
			phrase _ ms asTilesIn: Color globalNames: true].
	self deletePopup.
	completeMsg _ self isNoun ifTrue: [self] ifFalse: [owner].
	completeMsg owner replaceSubmorph: completeMsg by: phrase.
	"rec setSelection: {rec. nil. rec}."
	phrase acceptIfInScriptor.! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 8/26/2001 14:31'!
colorPatch
	"Return a color patch button that lets the user choose a color and modifies the code"
	| cc patch sel completeMsg |
	
	
	((self nodeClassIs: MessageNode) "or: [self nodeClassIs: SelectorNode]") ifFalse: [^ nil].
	(sel _ self selector) ifNil: [^ nil].
	(Color colorNames includes: sel) | (sel == #r:g:b:) ifFalse: [^ nil].
		"a standard color name"
	completeMsg _ self isNoun ifTrue: [self] 
				ifFalse: [owner isNoun ifTrue: [owner] ifFalse: [owner owner]].

	(cc _ completeMsg try) class == Color ifFalse: [^ nil].
	patch _ ColorTileMorph new colorSwatchColor: cc.
		"sends colorChangedForSubmorph: to the messageNode"
	patch color: Color transparent; borderWidth: 0.  patch submorphs last delete.
	^ patch! !

!SyntaxMorph methodsFor: 'highlighting' stamp: 'tk 7/30/2001 14:48'!
compoundBorderColor 

	^ self valueOfProperty: #deselectedBorderColor ifAbsent: [Color veryLightGray]
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 13:39'!
constructSelfVariant: receiver and: key

	| wordy |
	(receiver isKindOf: VariableNode) ifFalse: [^nil].
	receiver name = 'self'  ifFalse: [^nil].
	(wordy _ self translateFromWordySelfVariant: key) ifNil: [^nil].
	^wordy

! !

!SyntaxMorph methodsFor: 'formatting options' stamp: 'RAA 2/26/2001 09:02'!
controlContrast2: evt

	| origin scale startingContrastX |

	evt isMouseUp ifTrue: [
		^self removeProperty: #startingPointForSomeAdjustment
	].
	evt isMouseDown ifTrue: [
		^self setProperty: #startingPointForSomeAdjustment toValue: evt cursorPoint
	].
	ContrastFactor ifNil: [ContrastFactor _ 0.5].
	scale _ 200.0.
	startingContrastX _ ContrastFactor * scale.
	origin _ self valueOfProperty: #startingPointForSomeAdjustment.
	ContrastFactor _ (evt cursorPoint x - origin x + startingContrastX) / scale min: 1.0 max: 0.0.
	self finalAppearanceTweaks.
! !

!SyntaxMorph methodsFor: 'formatting options' stamp: 'RAA 2/26/2001 09:07'!
controlContrast: evt

	"old version. may be some scripts saved with me, so don't crash"
	^self! !

!SyntaxMorph methodsFor: 'formatting options' stamp: 'RAA 5/11/2001 07:41'!
controlSpacing2: evt

	| origin scale startingContrastX |

	evt isMouseUp ifTrue: [
		^self removeProperty: #startingPointForSomeAdjustment
	].
	evt isMouseDown ifTrue: [
		^self setProperty: #startingPointForSomeAdjustment toValue: evt cursorPoint
	].
	SizeScaleFactor ifNil: [SizeScaleFactor _ 0.15].
	scale _ 200.0.
	startingContrastX _ SizeScaleFactor * scale.
	origin _ self valueOfProperty: #startingPointForSomeAdjustment.
	SizeScaleFactor _ (evt cursorPoint x - origin x + startingContrastX) / scale min: 1.0 max: 0.0.
	self finalAppearanceTweaks.
! !

!SyntaxMorph methodsFor: 'formatting options' stamp: 'RAA 2/26/2001 09:07'!
controlSpacing: evt

	"old version. may be some scripts saved with me, so don't crash"
	^self! !

!SyntaxMorph methodsFor: 'card in a stack' stamp: 'tk 9/25/2001 11:46'!
couldHoldSeparateDataForEachInstance
	"Answer whether this type of morph is inherently capable of holding separate data for each instance ('card data')"

	^ true! !

!SyntaxMorph methodsFor: 'player' stamp: 'tk 9/26/2001 06:05'!
currentDataValue
	"Answer the current data value held by the receiver"

	^ self readOut valueFromContents! !

!SyntaxMorph methodsFor: 'selection' stamp: 'di 11/17/2000 08:10'!
currentSelectionDo: blockForSelection
	| rootTile |
	(rootTile _ self rootTile) isMethodNode ifFalse:
		 [^ blockForSelection value: nil value: nil value: nil].
	rootTile valueOfProperty: #selectionSpec ifPresentDo:
		[:selectionSpec | ^ blockForSelection
							value: selectionSpec first
							value: selectionSpec second
							value: selectionSpec third].
	^ blockForSelection value: nil value: nil value: nil! !

!SyntaxMorph methodsFor: 'macpal' stamp: 'sw 6/4/2001 19:26'!
currentVocabulary
	"Answer the current vocabulary associated with the receiver.  If none is yet set, determine an appropriate vocabulary and cache it within my properties dictionary."

	| aVocab aSym |
	aSym _ self valueOfProperty: #currentVocabularySymbol ifAbsent: [nil].
	aSym ifNil:
		[aVocab _ self valueOfProperty: #currentVocabulary ifAbsent: [nil].
		aVocab ifNotNil:
			[aSym _ aVocab vocabularyName.
			self removeProperty: #currentVocabulary.
			self setProperty: #currentVocabularySymbol toValue: aSym]].

	aSym ifNotNil:
		[^ Vocabulary vocabularyNamed: aSym].
	aVocab _ super currentVocabulary.
	self setProperty: #currentVocabularySymbol toValue: aVocab vocabularyName.
	^ aVocab! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'di 11/8/2000 22:05'!
cursorBaseOffset

	^ 7@14
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 23:20'!
darkerColor

	^(Color r: 1.0 g: 0.839 b: 0.613)	"Color lightBrown lighter lighter."
! !

!SyntaxMorph methodsFor: 'debugging' stamp: 'di 11/17/2000 07:59'!
debugger

	^ self rootTile valueOfProperty: #debugger! !

!SyntaxMorph methodsFor: 'debugging' stamp: 'di 11/17/2000 07:59'!
debugger: x

	self rootTile setProperty: #debugger toValue: x! !

!SyntaxMorph methodsFor: 'menus' stamp: 'md 8/15/2005 11:02'!
decompile
	| stream |
	"Produce Smalltalk code.  We have a tree of SyntaxMorphs, but not a tree of ParseNodes.  The user has dragged in many SyntaxMorphs, each with its own parseNode, but those nodes are not sewn together in a tree.  The only data we get from a ParseNode is its class.
	We produce really ugly code.  But we compile it and decompile (prettyPrint) again for user to see."

	stream _ ColoredCodeStream on: (Text new: 400).
	self printOn: stream indent: 1.	"Tree walk and produce text of the code"
	^ stream contents! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 2/21/2001 16:37'!
deleteLine
	| temp |
	temp _ owner.
	self deletePopup.
	self delete.
	temp setSelection: nil.
	temp acceptIfInScriptor.! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'di 12/13/2000 13:05'!
deletePopup

	self valueOfProperty: #myPopup ifPresentDo:
		[:panel | panel delete. self removeProperty: #myPopup]! !

!SyntaxMorph methodsFor: 'selection' stamp: 'RAA 2/14/2001 11:33'!
deselect
	self allMorphsDo:
		[:m | m isSyntaxMorph ifTrue: [m setDeselectedColor]].

	"Note following is wasteful because we do a deselect before each select, and it is often the same morph."
	self deletePopup! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 2/21/2001 13:09'!
dismisser
	"Return the icon to delete this line of tiles.  I am an entire line in a block."
	| handle handleSpec colorToUse iconName form |

	(owner isSyntaxMorph and: [owner nodeClassIs: BlockNode]) ifFalse: [^ nil].
	handleSpec _ Preferences haloSpecifications fourth.	"dismiss"
	handle _ EllipseMorph
			newBounds: (Rectangle center: 10@10 extent: 16 asPoint)
			color: (colorToUse _ Color colorFrom: handleSpec color).
	iconName _ handleSpec iconSymbol.
	form _ ScriptingSystem formAtKey: iconName.	"#'Halo-Dismiss'"
	handle addMorphCentered: (ImageMorph new
				image: form; 
				color: colorToUse makeForegroundColor;
				lock).
	handle on: #mouseDown send: #deleteLine to: self.
	^ handle! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 9/20/2001 12:43'!
dissectMessage
	"I am a MessageNode.  Return {receiverNode or nil, selector, (keyword nodes), (argument nodes)}.  Ignore all spacing morphs."

	! !

!SyntaxMorph methodsFor: 'drawing' stamp: 'RAA 3/25/2001 16:16'!
drawOn: aCanvas

	super drawOn: aCanvas.
	self isBlockNode ifFalse: [^self].
	self alansTest1 ifTrue: [^self].

	self immediatelyBelowTheMethodNode ifTrue: [
		aCanvas fillRectangle: (self topLeft + (0@-1) extent: self width@1) color: Color gray
	] ifFalse: [
		aCanvas fillRectangle: (self topLeft + (1@1) extent: 2@(self height-2)) color: Color gray.
		aCanvas fillRectangle: (self topLeft + (1@1) extent: 4@1) color: Color gray.
		aCanvas fillRectangle: (self bottomLeft + (1@-1) extent: 4@1) color: Color gray
	].
! !

!SyntaxMorph methodsFor: 'highlighting' stamp: 'di 11/5/2000 07:26'!
dropColor
	^ Color green darker! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 10/17/2001 13:38'!
dupTile: evt

	| dup |
	self deletePopup.
	"self deselect."
	dup _ self duplicateMorph: evt.
	Preferences tileTranslucentDrag
		ifTrue: [dup align: dup center with: evt hand position.
				dup lookTranslucent]
		ifFalse: [dup align: dup topLeft
					with: evt hand position + self cursorBaseOffset].
! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 10/17/2001 13:29'!
duplicator
	"Return the icon to duplicate this tile."
	| handle handleSpec colorToUse iconName form |

	handleSpec _ Preferences haloSpecifications at: 11.	"duplicate"
	handle _ EllipseMorph
			newBounds: (Rectangle center: 10@10 extent: 16 asPoint)
			color: (colorToUse _ Color colorFrom: handleSpec color).
	iconName _ handleSpec iconSymbol.
	form _ ScriptingSystem formAtKey: iconName.	"#'Halo-Dup'"
	handle addMorphCentered: (ImageMorph new
				image: form; 
				color: colorToUse makeForegroundColor;
				lock).
	handle on: #mouseDown send: #dupTile: to: self.
	^ handle! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'sw 2/3/2001 01:40'!
editor
	"In parallel with the interface for text morphs, we respond to this, but in our case we are our own editor"

	^ self! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 9/19/2001 15:39'!
enclosingPane
	"The object that owns this script layout"

	| oo higher |
	oo _ self owner.
	[higher _ oo isSyntaxMorph.
	higher _ higher or: [oo class == TransformMorph].
	higher _ higher or: [oo class == TwoWayScrollPane].
	higher ifFalse: [^ oo].
	higher] whileTrue: [oo _ oo owner].
! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'ar 3/18/2001 17:28'!
event: arg1 arrow: arg2 upDown: arg3
	"Reorder the arguments for existing event handlers"
	(arg3 isMorph and:[arg3 eventHandler notNil]) ifTrue:[arg3 eventHandler fixReversedValueMessages].
	^self upDown: arg1 event: arg2 arrow: arg3! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 9/28/2001 13:38'!
extend
	| messageNodeMorph first |
	"replace this noun with a new message like (arg + 1).  If type is not known, ask the user to type in a selector.  Use nil as arg.  Let user drag something to it afterwards."

	"Later do evaluation of self to see what type and offer right selector"
	self deselect.
	messageNodeMorph _ (MessageSend receiver: 1 selector: #+ arguments: #(1))
								asTilesIn: Player globalNames: false.
	owner replaceSubmorph: self by: messageNodeMorph.
	first _ messageNodeMorph submorphs detect: [:mm | mm isSyntaxMorph].
	messageNodeMorph replaceSubmorph: first by: self.
	self acceptIfInScriptor.! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 8/24/2001 13:14'!
extendArrow
	"Return the extend arrow button.  It replaces the argument with a new message.
	I am a number or getter messageNode."
	| patch |
	
	self isNoun ifFalse: [^ nil].
	self isDeclaration ifTrue: [^ nil].
	patch _ (ImageMorph new image: (TileMorph classPool at: #SuffixPicture)).
	patch on: #mouseDown send: #extend to: self.
	^ patch! !

!SyntaxMorph methodsFor: 'viewer' stamp: 'tk 11/5/2001 08:32'!
externalName

	^ self knownName ifNil: [
		parseNode ifNil: ['Syntax -- (extra layer)']
				ifNotNil: [self parseNode class printString]]! !

!SyntaxMorph methodsFor: 'visual properties' stamp: 'tk 7/31/2001 16:53'!
fillStyle: aFillStyle

	aFillStyle isColor 
		ifTrue: [self color: aFillStyle]	"so we will process it"
		ifFalse: [super fillStyle: aFillStyle].
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'gm 2/22/2003 13:42'!
finalAppearanceTweaks
	| deletes lw |
	SizeScaleFactor ifNil: [SizeScaleFactor := 0.15].
	SizeScaleFactor := 0.0.	"disable this feature.  Default was for giant tiles"
	self usingClassicTiles 
		ifTrue: 
			[self 
				allMorphsDo: [:each | (each isSyntaxMorph) ifTrue: [each lookClassic]].
			^self].
	deletes := OrderedCollection new.
	self allMorphsDo: 
			[:each | 
			(each respondsTo: #setDeselectedColor) ifTrue: [each setDeselectedColor].
			"(each hasProperty: #variableInsetSize) ifTrue: [
			each layoutInset: 
				((each valueOfProperty: #variableInsetSize) * SizeScaleFactor) rounded]."
			each isSyntaxMorph 
				ifTrue: 
					[lw := each layoutInset.
					lw isPoint ifTrue: [lw := lw x].
					each layoutInset: lw @ 0	"(6 * SizeScaleFactor) rounded"]].
	deletes do: [:each | each delete]! !

!SyntaxMorph methodsFor: 'submorphs-accessing' stamp: 'tk 1/13/2001 20:41'!
findA: aClass
	| ans |
	"Allow finding on the class of the parseNode"

	(ans _ super findA: aClass) ifNotNil: [^ ans].
	submorphs do: [:ss | 
		ss isSyntaxMorph ifTrue: [
			ss parseNode class == aClass ifTrue: [^ ss]]].
	^ nil! !

!SyntaxMorph methodsFor: 'layout' stamp: 'RAA 2/15/2001 19:43'!
foldMessage
	"I am a message whose receiver is wide, and whose message part is a column.
	Rearrange me so that the message part appears indented under the receiver part."
	| messageRow node2 |
	node2 _ parseNode copy receiver: nil.
	messageRow _ SyntaxMorph row: #keyword1 on: node2.

	messageRow 
		addMorph: (self transparentSpacerOfSize: 20@10);
		addMorphBack: submorphs last.		"<<handle noise words better"
	self listDirection: #topToBottom;
		wrapCentering: #topLeft;
		addMorphBack: (self transparentSpacerOfSize: 4@4);
		addMorphBack: messageRow.
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 1/15/2001 11:15'!
foldMessageOneArg
	"I am a message that is wide, a row with receiver and a row with selector and arg.
	Rearrange me so that the message part appears indented under the receiver part."
	| messageRow node2 |
	node2 _ parseNode copy receiver: nil.
	messageRow _ SyntaxMorph row: #keyword1 on: node2.
	messageRow addMorph: (self transparentSpacerOfSize: 20@10);
			addMorphBack: submorphs second;
			addMorphBack: submorphs second.  "was the third"
	self listDirection: #topToBottom;
		wrapCentering: #topLeft;
		addMorphBack: messageRow.
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 23:23'!
fontToUseForSpecialWord: aString

	^(#('Yes' 'No' 'Test') includes: aString) ifTrue: [
		(StrikeFont familyName: 'Helvetica' size: 14)
	] ifFalse: [
		nil	"(StrikeFont familyName: 'ComicBold' size: 16)"
	]! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 9/7/2001 16:13'!
getCurrentValue

	parseNode ifNil: [^nil].
	parseNode class == Symbol ifTrue: [^nil].	"special"
	^parseNode currentValueIn: self hostContext! !

!SyntaxMorph methodsFor: 'printing' stamp: 'tk 9/23/2001 01:33'!
getHeader: strm
	| se |
	"We are in an EToy scriptor and the method header line has been removed.  Try to recover the method name.  Fail if method has args (deal with this later)."

	(se _ self ownerThatIsA: ScriptEditorMorph) ifNotNil: [
		se scriptName numArgs > 0 ifTrue: [^ false].	"abort"
		strm nextPutAll: se scriptName].
	^ true! !

!SyntaxMorph methodsFor: 'menus' stamp: 'di 11/13/2000 20:23'!
getMenuBlock

	^ nil! !

!SyntaxMorph methodsFor: 'highlighting' stamp: 'tk 7/23/2001 18:28'!
grabColor

	"Not the select color, but the mouseOver border color.  Means it could be grabbed"
	^ Color paleOrange mixed: 0.5 with: Color brown! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'di 11/6/2000 16:20'!
handlesKeyboard: evt
	^ evt keyCharacter = Character backspace! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'dgd 2/22/2003 13:39'!
handlesMouseDown: evt 
	evt yellowButtonPressed ifTrue: [^true].
	parseNode isNil ifTrue: [^false].
	owner isSyntaxMorph 
		ifTrue: [(owner isMethodNode and: [self isBlockNode not]) ifTrue: [^false]].	"Can only take block out of a MethodNode"
	^true! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'tk 10/26/2000 16:58'!
handlesMouseOver: evt
	"Am I a tile that could be picked up?"

	^ true! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'di 11/6/2000 08:21'!
handlesMouseOverDragging: evt

	^ evt hand hasSubmorphs
		and: [evt hand firstSubmorph isSyntaxMorph]
! !

!SyntaxMorph methodsFor: 'dropping/grabbing' stamp: 'di 5/4/2001 13:16'!
highlightForDrop: evt

	(self wantsDroppedMorph: evt hand firstSubmorph event: evt)
		ifTrue: [self color: self dropColor].! !

!SyntaxMorph methodsFor: 'highlighting' stamp: 'di 11/6/2000 09:22'!
highlightForGrab: evt

	self borderColor: self grabColor.! !

!SyntaxMorph methodsFor: 'debugging' stamp: 'RAA 8/24/1999 12:35'!
hostContext

	^nil		"we don't have one"! !

!SyntaxMorph methodsFor: 'node types' stamp: 'RAA 2/14/2001 20:34'!
immediatelyBelowTheMethodNode

	^(owner respondsTo: #isMethodNode) and: [owner isMethodNode]! !

!SyntaxMorph methodsFor: 'initialization' stamp: 'sw 3/6/2001 11:26'!
inAPluggableScrollPane
	"Answer a PluggableTileScriptorMorph that holds the receiver"

	| widget |
	widget _ PluggableTileScriptorMorph new.
	widget extent: 10@10; borderWidth: 0.
	widget scroller addMorph: self.
	widget setScrollDeltas.
	widget hResizing: #spaceFill; vResizing: #spaceFill.
	^ widget

! !

!SyntaxMorph methodsFor: 'initialization' stamp: 'sw 6/26/2001 10:58'!
inAScrollPane
	"Answer a scroll pane in which the receiver is scrollable"

	^ self inATwoWayScrollPane! !

!SyntaxMorph methodsFor: 'new tiles' stamp: 'tk 8/30/2001 06:22'!
instVarTile: aName
	"Make and put into hand a tile for an instance variable"

	| sm |
	sm _ ((VariableNode new
					name: aName
					index: 1
					type: 1 "LdInstType") asMorphicSyntaxIn: SyntaxMorph new).
	sm roundedCorners.
	ActiveHand attachMorph: sm.
	Preferences tileTranslucentDrag
		ifTrue: [sm lookTranslucent.
			sm align: sm center with: ActiveHand position "+ self cursorBaseOffset"]
		ifFalse: [sm align: sm topLeft with: ActiveHand position + self cursorBaseOffset]
! !

!SyntaxMorph methodsFor: 'node types' stamp: 'tk 8/24/2001 15:41'!
isAVariable
	"There are three kinds of variable nodes"

	((parseNode class == TempVariableNode) or: [
		(parseNode class == LiteralVariableNode) or: [
			parseNode class == VariableNode]]) ifFalse: [^ false].
	^ (ClassBuilder new reservedNames includes: 
			self decompile string withoutTrailingBlanks) not! !

!SyntaxMorph methodsFor: 'node types' stamp: 'di 11/17/2000 08:31'!
isBlockNode
	^ parseNode class == BlockNode! !

!SyntaxMorph methodsFor: 'node types' stamp: 'tk 9/26/2001 05:50'!
isDeclaration
	"Return true if I am a TempVarNode inside a declaration of some kind, including a method arg"

	| opc |
	owner isSyntaxMorph ifFalse: [^ false].
	opc _ owner parseNode class.
	opc == BlockArgsNode ifTrue: [^ true].
	opc == MethodTempsNode ifTrue: [^ true].
	opc == SelectorNode ifTrue: [^ true].
	^ false! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 8/22/2001 16:30'!
isKnownVarName: newVarName
	"Return true if this variable is already known, as an argument, temp var, block temp, or instance variable."

	| syntLevel |
	(self parsedInClass allInstVarNames includes: newVarName) ifTrue: [^ true].
	syntLevel _ self.
	[syntLevel tempVarNodesDo: [:node | 
		node decompile string = newVarName ifTrue: [^ true]].
	 (syntLevel _ syntLevel owner) isSyntaxMorph] whileTrue.
	^ false! !

!SyntaxMorph methodsFor: 'node types' stamp: 'gm 2/22/2003 12:30'!
isLeafTile
	self hasSubmorphs ifFalse: [^false].
	(self firstSubmorph isSyntaxMorph) ifTrue: [^false].
	(self firstSubmorph isMemberOf: Morph) ifTrue: [^false].
	^true! !

!SyntaxMorph methodsFor: 'node types' stamp: 'di 11/17/2000 08:31'!
isMethodNode
	^ parseNode class == MethodNode! !

!SyntaxMorph methodsFor: 'node types' stamp: 'tk 9/13/2001 15:28'!
isNoun
	"Consider these to be nouns:  MessageNode with receiver, CascadeNode with receiver, AssignmentNode, TempVariableNode, LiteralNode, VariableNode, LiteralVariableNode."

	(#(TempVariableNode LiteralNode VariableNode LiteralVariableNode) includes:
		(parseNode class name)) ifTrue: [^ true].

	(self nodeClassIs: MessageNode) ifTrue: [^ parseNode receiver notNil].
	(self nodeClassIs: CascadeNode) ifTrue: [^ parseNode receiver notNil].
	(self nodeClassIs: AssignmentNode) ifTrue: [^ submorphs size >= 3].

	^ false! !

!SyntaxMorph methodsFor: 'selection' stamp: 'tk 1/17/2001 15:07'!
isSelectable
	| ss |
	"Spacer morphs enclose other morphs with the same parseNode"
	self submorphs size > 1 ifTrue: [
		ss _ self submorphs second.
		ss isSyntaxMorph ifTrue: [
			ss parseNode == parseNode ifTrue: [
				^ self submorphs first class ~~ Morph]]].
		
"	(self nodeClassIs: SelectorNode) ifTrue: [^ false].
	(self nodeClassIs: KeyWordNode) ifTrue: [^ false].
"
	self isMethodNode ifTrue: [^ false].
	parseNode ifNil: [^ false].
	^ true! !

!SyntaxMorph methodsFor: 'node types' stamp: 'tk 9/23/2001 00:17'!
isSelfTile

	^ parseNode class == VariableNode and: [self decompile asString = 'self ']
	! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'yo 11/11/2002 10:30'!
isStandardGetterSelector: key

	self flag: #yoCharCases.

	key size > 3 ifFalse: [^false].
	(key beginsWith: 'get') ifFalse: [^false].
	key fourth isUppercase ifFalse: [^false].
	^true
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'yo 11/11/2002 10:31'!
isStandardSetterKeyword: key

	self flag: #yoCharCases.

	key size > 4 ifFalse: [^false].
	(key endsWith: ':') ifFalse: [^false].
	(key beginsWith: 'set') ifFalse: [^false].
	key fourth isUppercase ifFalse: [^false].
	^true
! !

!SyntaxMorph methodsFor: 'classification' stamp: 'di 11/2/2000 13:25'!
isSyntaxMorph
	^ true! !

!SyntaxMorph methodsFor: 'dropping/grabbing' stamp: 'tk 7/19/2001 19:04'!
justDroppedInto: aMorph event: evt
	aMorph isSyntaxMorph ifFalse:
		[Preferences tileTranslucentDrag
			ifTrue: [self setDeselectedColor]
			ifFalse: [self align: self topLeft with: self topLeft - self cursorBaseOffset]].
	self removeProperty: #beScript.
	^ super justDroppedInto: aMorph event: evt! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'di 11/17/2000 08:21'!
keyStroke: evt
	"Handle a keystroke event."
	| spacer |
	evt keyCharacter = Character backspace ifTrue:
		[(owner notNil and: [owner isSyntaxMorph]) ifTrue:
			[owner isBlockNode ifTrue:
				["Delete a statement."
				(spacer _ self submorphAfter) class == AlignmentMorph
						ifTrue: [spacer delete].
				self delete].
			]].
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/28/2001 09:35'!
lighterColor

	^Color gray: 0.9		
"(Color r: 0.935 g: 0.935 b: 0.935)"
"paleGreen lighter"
! !

!SyntaxMorph methodsFor: 'vocabulary' stamp: 'gm 2/22/2003 12:31'!
limitClassToUseWith: aValue vocabulary: aVocabulary 
	"Answer the most generic whose method should be shown in a selector pop-up in the receiver that is put up on behalf of aValue"

	(aValue isNumber) ifTrue: [^Number].
	"Ted: This hook allows you to intervene as suits your purposes here if you don't like the defaults."
	^aValue defaultLimitClassForVocabulary: aVocabulary! !

!SyntaxMorph methodsFor: 'formatting options' stamp: 'di 2/21/2001 12:30'!
lookClassic
	self isLeafTile ifTrue: [self layoutInset: 2@4]! !

!SyntaxMorph methodsFor: 'drawing' stamp: 'tk 9/13/2001 15:13'!
lookTranslucent

	self setDeselectedColor.
	super color: (self color alpha: 0.25).
	submorphs do: [:mm | (mm respondsTo: #lookTranslucent) 
		ifTrue: [mm lookTranslucent]
		ifFalse: ["mm color: color"]].
! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'dgd 2/22/2003 18:48'!
messageNode
	"Return the enclosing messageNode that is the full message.  It has a receiver."

	^self orOwnerSuchThat: [:oo | oo receiverNode notNil]! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'dgd 2/22/2003 13:39'!
messageNode: aNode receiver: receiver selector: selector keywords: key arguments: args 
	| keywords column row receiverMorph receiverWidth messageWidth onlyOne nodeWithNilReceiver isAConditional |
	self alansTest1 
		ifTrue: 
			[^self 
				alansMessageNode: aNode
				receiver: receiver
				selector: selector
				keywords: key
				arguments: args].
	isAConditional := #(#ifTrue: #ifFalse: #ifTrue:ifFalse: #ifFalse:ifTrue:) 
				includes: key.
	receiver ifNotNil: 
			["i.e. not a cascade"

			receiverMorph := receiver asMorphicSyntaxIn: self].
	keywords := key keywords.
	args isEmpty 
		ifTrue: 
			[row := (self addSingleKeywordRow: key) layoutInset: 1.
			^row parseNode: selector].
	receiverWidth := receiver ifNil: [0]
				ifNotNil: [receiverMorph fullBounds width].
	onlyOne := args size = 1.
	(receiverWidth <= 80 and: [onlyOne]) 
		ifTrue: 
			[self 
				messageOneArg: key
				receiver: receiver
				selector: selector
				args: args.
			^self].
	nodeWithNilReceiver := aNode copy receiver: nil.
	column := self addColumn: #keyword1 on: nodeWithNilReceiver.
	"onlyOne ifTrue: [column parseNode: nil].	is a spacer"
	messageWidth := 0.
	keywords with: (args copyFrom: 1 to: keywords size)
		do: 
			[:kwd :arg | 
			isAConditional 
				ifTrue: [column addMorphBack: (column transparentSpacerOfSize: 3 @ 3)].
			(row := column addRow: #keyword2 on: nodeWithNilReceiver)
				borderWidth: 1;
				parseNode: (nodeWithNilReceiver 
							as: (onlyOne ifTrue: [MessageNode] ifFalse: [MessagePartNode]));
				borderColor: row stdBorderColor.
			isAConditional 
				ifTrue: [row addMorphBack: (row transparentSpacerOfSize: 20 @ 6)].
			row 
				addToken: kwd
				type: #keyword2
				on: (onlyOne 
						ifTrue: [SelectorNode new key: kwd code: nil	"fill this in?"]
						ifFalse: [KeyWordNode new]).
			arg asMorphicSyntaxIn: row.
			messageWidth := messageWidth + row fullBounds width].
	onlyOne 
		ifTrue: 
			[self replaceSubmorph: column by: row.
			column := row].
	receiverMorph ifNil: [^self].
	receiverWidth + messageWidth < 350 
		ifTrue: 
			[isAConditional ifFalse: [self unfoldMessage].
			^self].
	((receiverWidth > 200 
		or: [receiverWidth > 80 and: [column fullBounds height > 20]]) or: 
				[receiverMorph fullBounds width > 30 
					and: [column fullBounds height > 100 or: [column fullBounds width > 250]]]) 
		ifTrue: [^self foldMessage]! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'RAA 2/26/2001 14:04'!
messageOneArg: key receiver: receiver selector: selector args: args

	| row firstArgMorph |

	row _ (self addSingleKeywordRow: key) layoutInset: 1.
	row parseNode: selector.
	firstArgMorph _ args first asMorphicSyntaxIn: self.
	receiver ifNil: [^ self].
	(firstArgMorph fullBounds height > 100
			or: [firstArgMorph fullBounds width > 250])
		ifTrue: [self foldMessageOneArg].
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'RAA 2/26/2001 17:12'!
methodNodeInner: aNode selectorOrFalse: selectorOrFalse precedence: precedence arguments: arguments temporaries: temporaries primitive: primitive block: block
	| header selNode |

	selNode _ selectorOrFalse class == SelectorNode 
		ifTrue: [selectorOrFalse] 
		ifFalse: [SelectorNode new key: selectorOrFalse code: nil].
	header _ self addRow: Color white on: selNode.
	precedence = 1
		ifTrue: [header addToken: aNode selector type: #methodHeader1 on: selNode]
		ifFalse: [aNode selector keywords with: arguments do:
					[:kwd :arg | 
					header addToken: kwd type: #methodHeader2 on: selNode.
					(arg asMorphicSyntaxIn: header) color: #blockarg2]].
	aNode addCommentToMorph: self.
	self addTemporaries: temporaries.
	(primitive > 0 and: [(primitive between: 255 and: 519) not]) ifTrue:
		["Dont decompile <prim> for, eg, ^ self "
		self addTextRow: (String streamContents: [ :strm | aNode printPrimitiveOn: strm])].
	block asMorphicSyntaxIn: self.
	^ self
! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'tk 7/31/2001 17:37'!
methodNodeOuter: aNode

	| block |
	
	self borderWidth: 0.
	aNode asMorphicSyntaxIn: self.
	self alansTest1 ifTrue: [self addTemporaryControls].
	self finalAppearanceTweaks.
		"self setProperty: #deselectedColor toValue: Color transparent."
	block _ self findA: BlockNode.
		"block setProperty: #deselectedColor toValue: Color transparent."
	block submorphs size = 1 ifTrue: [^ self].	"keep '^ self' if that is the only thing in method"
	block submorphs last decompile string = '^  self ' ifTrue: [
		block submorphs last delete].
	^ self! !

!SyntaxMorph methodsFor: 'dropping/grabbing' stamp: 'tk 9/30/2001 11:09'!
morphToDropInPasteUp: aPasteUp
	"If property #beScript is true, create a scriptor around me."

	| actualObject itsSelector aScriptor adjustment handy tw blk |
	(self valueOfProperty: #beScript ifAbsent: [false]) ifFalse: [^ self].
	self removeProperty: #beScript.
	actualObject _ self actualObject ifNil: [
					self valueOfProperty: #scriptedPlayer ifAbsent: [nil]].
	actualObject ifNil: [^ self].
	self removeProperty: #scriptedPlayer.
	actualObject assureUniClass.

	itsSelector _ self userScriptSelector.
	aScriptor _ itsSelector isEmptyOrNil
		ifFalse:
			[adjustment _ 0@0.
			actualObject scriptEditorFor: itsSelector]
		ifTrue:
			[adjustment _ 60 @ 20.
			actualObject newScriptorAround: self].
	aScriptor ifNil: [^self].
	handy _ aPasteUp primaryHand.

	aScriptor position: handy position - adjustment.
	aPasteUp addMorphFront: aScriptor.	"do this early so can find World"
	aScriptor showingMethodPane ifFalse: [
		"(tw _ aScriptor findA: TwoWayScrollPane) ifNil:
			[itsSelector ifNil: ['blank script'.
				tw _ aScriptor findA: TwoWayScrollPane.
				blk _ (tw scroller findA:  SyntaxMorph ""MethodNode"") findA: BlockNode.
				blk addMorphFront: self]].
		"
		SyntaxMorph setSize: nil andMakeResizable: aScriptor.
		].
	^ aScriptor
! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'tk 9/26/2001 05:56'!
mouseDown: evt 
	| dup rootTile |
	evt yellowButtonPressed ifTrue: [^ self showMenu: evt].
	(rootTile _ self rootTile) isMethodNode ifTrue:
		[self currentSelectionDo:
			[:innerMorph :mouseDownLoc :outerMorph |
			(outerMorph notNil and: [self == innerMorph])
				ifTrue: ["Click on prior selection -- record click point."
						self setSelection: {self. evt cursorPoint. outerMorph}]
				ifFalse: ["A new selection sequence."
						self setSelection: {self. evt cursorPoint. nil}]].
		^ self].

	"Out in the world -- treat as a unit"
	rootTile isSticky ifTrue: [^ self].	"later may allow to be selected"
	rootTile isPartsDonor 
		ifTrue: [dup _ rootTile duplicate.
				dup setProperty: #beScript toValue: true]
		ifFalse: [dup _ rootTile].
	evt hand attachMorph: dup.
	Preferences tileTranslucentDrag
		ifTrue: [^ dup lookTranslucent]
		ifFalse: [^ dup align: dup topLeft with: evt hand position + self cursorBaseOffset]
! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'tk 7/19/2001 20:21'!
mouseEnter: evt
	"Highlight this level as a potential grab target"

"Transcript cr; print: self; show: ' enter'."
	self rootTile isMethodNode ifFalse: [^ self]. 	"not in a script"
	self unhighlightOwnerBorder.
	self highlightForGrab: evt.
	evt hand newKeyboardFocus: self.
! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'tk 7/25/2001 10:09'!
mouseEnterDragging: evt
	"Highlight this level as a potential drop target"

"self isBlockNode ifTrue: [Transcript cr; print: self; show: ' enterDragging']."
	self rootTile isMethodNode ifFalse: [^ self]. 	"not in a script"

	evt hand hasSubmorphs ifFalse: [^ self].  "Don't react to empty hand"
	self unhighlightOwnerBorder.
	self isBlockNode ifFalse: [self highlightForDrop: evt.
		(self firstOwnerSuchThat: [:m | m isSyntaxMorph and: [m color = self dropColor]])
			ifNotNilDo: [:m | m unhighlight]].

	self isBlockNode ifTrue:
		[(self firstOwnerSuchThat: [:m | m isSyntaxMorph and: [m isBlockNode]])
			ifNotNilDo: [:m | "Suspend outer block."
						m stopStepping; removeDropZones].
		self startStepping]
! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'dgd 2/22/2003 18:48'!
mouseLeave: evt 
	"Move grab highlight back out a level"

	"Transcript cr; print: self; show: ' leave'."

	self rootTile isMethodNode ifFalse: [^self].	"not in a script"
	self unhighlightBorder.
	(owner notNil and: [owner isSyntaxMorph]) 
		ifTrue: [owner highlightForGrab: evt]! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'dgd 2/22/2003 18:48'!
mouseLeaveDragging: evt 
	"Transcript cr; print: self; show: ' leaveDragging'."

	self rootTile isMethodNode ifFalse: [^self].	"not in a script"
	self isBlockNode 
		ifTrue: 
			[self
				stopStepping;
				removeDropZones.
			(self firstOwnerSuchThat: [:m | m isSyntaxMorph and: [m isBlockNode]]) 
				ifNotNilDo: [:m | m startStepping].	"Activate outer block."
			self submorphs do: 
					[:ss | 
					"cancel drop color in line beside mouse"

					ss color = self dropColor ifTrue: [ss setDeselectedColor]]].

	"Move drop highlight back out a level"
	self unhighlight.
	(owner notNil and: [owner isSyntaxMorph]) 
		ifTrue: [owner isBlockNode ifFalse: [owner highlightForDrop: evt]]! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'tk 10/17/2001 13:41'!
mouseMove: evt
	| dup selection |
	owner isSyntaxMorph ifFalse: [^ self].

false ifTrue: ["for now, do not drag off a tile"
	self currentSelectionDo:
		[:innerMorph :mouseDownLoc :outerMorph |
		mouseDownLoc ifNotNil: [
			(evt cursorPoint dist: mouseDownLoc) > 4 ifTrue:
				["If drag 5 pixels, then tear off a copy of outer selection."
				selection _ outerMorph ifNil: [self].
				selection deletePopup.
				evt hand attachMorph: (dup _ selection duplicate).
				Preferences tileTranslucentDrag
					ifTrue: [dup lookTranslucent]
					ifFalse: [dup align: dup topLeft
								with: evt hand position + self cursorBaseOffset].
				self setSelection: nil.	"Why doesn't this deselect?"
				(self firstOwnerSuchThat: [:m | m isSyntaxMorph and: [m isBlockNode]])
					ifNotNilDo: [:m | "Activate enclosing block."
								m startStepping]]]].
	].! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'di 11/17/2000 08:13'!
mouseUp: evt
	| newSel |
	self rootTile isMethodNode ifFalse: [^ self].
	self currentSelectionDo:
		[:innerMorph :mouseDownLoc :outerMorph |
		newSel _ outerMorph
			ifNil: [self "first click"]
			ifNotNil: [(outerMorph firstOwnerSuchThat:
							[:m | m isSyntaxMorph and: [m isSelectable]]) ifNil: [self]].
		newSel isMethodNode ifTrue: [^ self setSelection: nil].
		self setSelection: {self. nil. newSel}]
! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 9/28/2001 11:35'!
newAssignment
	"I am a variableNode.  Place me inside an assignment statement."

	| new old |
	parseNode name: self decompile.	"in case user changed name"
	new _ owner assignmentNode: AssignmentNode new variable: parseNode 
					value: parseNode copy.
	self deselect.
	(old _ owner) replaceSubmorph: self by: new.	"do the normal replacement"
	(old isSyntaxMorph) ifTrue: [old cleanupAfterItDroppedOnMe].	"now owned by no one"
! !

!SyntaxMorph methodsFor: 'node types' stamp: 'di 11/6/2000 15:26'!
nodeClassIs: aParseNodeClass
	"Test the class of my parseNode"

	^ parseNode class == aParseNodeClass! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'tk 8/7/2001 23:15'!
noiseBeforeBlockArg

	^ self alansTest1 ifTrue: [' Use'] ifFalse: [' from']! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 3/25/2001 17:16'!
noiseStringMorph: aNoiseString

	| sMorph |

	sMorph _ self aSimpleStringMorphWith: aNoiseString.
	sMorph 
		font: (self fontToUseForSpecialWord: aNoiseString); 
		setProperty: #noiseWord toValue: true.

	^sMorph
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'tk 8/7/2001 23:12'!
noiseWordBeforeVariableNode: aNode string: aString

	(#('self' 'nil') includes: aString) ifFalse: [
		aNode code ifNil: [^'my'].
		aNode type < 4 ifTrue: [^'my']
	].
	^nil! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'dgd 2/22/2003 13:40'!
offerPopUp
	"Put up a halo to allow user to change
		Literals (Integer, true),
		Selector (beep: sound, +,-,*,//,\\, r:g:b:, setX: incX: decX: for any X,),
		Variable (Color),
		not AssignmentNode (_ inc dec),
	Extend arrows on each literal, variable, and message, (block that is by itself).
	Retract arrows on each literal or variable, or message or block that is an argument.
	Any literal can be changed by Shift-clicking and typing."

	| panel any upDown retract extend colorPatch edge dismiss rr duplicate |
	(self hasProperty: #myPopup) ifTrue: [^self].	"already has one"
	any := false.
	(upDown := self upDownArrows) ifNotNil: [any := true].	"includes menu of selectors"
	(retract := self retractArrow) ifNotNil: [any := true].
	(extend := self extendArrow) ifNotNil: [any := true].
	(dismiss := self dismisser) ifNotNil: [any := true].
	(duplicate := self duplicator) ifNotNil: [any := true].
	"(assign _ self assignmentArrow) ifNotNil: [any _ true].
			get from menu or any other assignment"
	submorphs last class == ColorTileMorph 
		ifFalse: [(colorPatch := self colorPatch) ifNotNil: [any := true]].
	any ifFalse: [^self].
	"Transcript cr; print: parseNode class; space; 
		print: (self hasProperty: #myPopup); endEntry."
	panel := (RectangleMorph new)
				color: Color transparent;
				borderWidth: 0.
	upDown ifNotNil: 
			[panel addMorphBack: upDown first.
			upDown first align: upDown first topLeft with: panel topLeft + (0 @ 0).
			panel addMorphBack: upDown second.
			upDown second align: upDown second topLeft
				with: upDown first bottomLeft + (0 @ 1).
			upDown size > 2 
				ifTrue: 
					[panel addMorphBack: upDown third.
					upDown third align: upDown third topLeft
						with: upDown first topRight + (2 @ 3)]].
	rr := self right.
	colorPatch ifNotNil: 
			[rr := rr + colorPatch submorphs first width + 1.
			self addMorphBack: colorPatch	"always in tile"
			"colorPatch align: colorPatch topLeft 
					with: panel topLeft + (1@1)"].
	retract ifNotNil: 
			[edge := panel submorphs isEmpty 
						ifTrue: [panel left]
						ifFalse: [panel submorphs last right].
			panel addMorphBack: retract.
			retract align: retract topLeft with: (edge + 2) @ (panel top + 3)].
	extend ifNotNil: 
			[edge := panel submorphs isEmpty 
						ifTrue: [panel left]
						ifFalse: [panel submorphs last right].
			panel addMorphBack: extend.
			extend align: extend topLeft with: (edge + 2) @ (panel top + 3)].
	duplicate ifNotNil: 
			[edge := panel submorphs isEmpty 
						ifTrue: [panel left]
						ifFalse: [panel submorphs last right].
			panel addMorphBack: duplicate.
			duplicate align: duplicate topLeft with: (edge + 2) @ (panel top + 1)].
	dismiss ifNotNil: 
			[edge := panel submorphs isEmpty 
						ifTrue: [panel left]
						ifFalse: [panel submorphs last right].
			panel addMorphBack: dismiss.
			dismiss align: dismiss topLeft with: (edge + 2) @ (panel top + 1)].
	"	assign ifNotNil: [
		edge _ panel submorphs isEmpty 
			ifTrue: [panel left] 
			ifFalse: [panel submorphs last right].
		panel addMorphBack: assign.
		assign align: assign topLeft with: (edge+2) @ (panel top + 2)].
"
	panel align: panel topLeft with: rr @ (self top - 2).
	panel extent: panel submorphs last bottomRight - panel topLeft.
	self setProperty: #myPopup toValue: panel.
	self addMorphBack: panel	"Any reason ever to have panel below?"
	"(owner listDirection = #topToBottom and: [self listDirection = #leftToRight])
		ifTrue: [self addMorphBack: panel]
		ifFalse: [owner addMorph: panel after: self]."! !

!SyntaxMorph methodsFor: 'menus' stamp: 'tk 9/23/2001 02:05'!
offerTilesMenuFor: aReceiver in: aLexiconModel
	"Offer a menu of tiles for assignment and constants"

	| menu |
	menu _ MenuMorph new addTitle: 'Hand me a tile for...'.
	menu addLine.
	menu add: '(accept method now)' target: aLexiconModel selector: #acceptTiles.
	menu submorphs last color: Color red darker.
	menu addLine.

	menu add: 'me, by name' target: self  selector: #attachTileForCode:nodeType: 
				argumentList: {'<me by name>'. aReceiver}.
	menu add: 'self' target: self  selector: #attachTileForCode:nodeType: 
				argumentList: {'self'. VariableNode}.
	menu add: '_   (assignment)' target: self  selector: #attachTileForCode:nodeType: 
				argumentList: {'<assignment>'. nil}.
	menu add: '"a Comment"' target: self  selector: #attachTileForCode:nodeType: 
				argumentList: {'"a comment"\' withCRs. CommentNode}.
	menu submorphs last color: Color blue.
	menu add: 'a Number' target: self  selector: #attachTileForCode:nodeType: 
				argumentList: {'5'. LiteralNode}.
	menu add: 'a Character' target: self  selector: #attachTileForCode:nodeType: 
				argumentList: {'$z'. LiteralNode}.
	menu add: '''abc''' target: self selector: #attachTileForCode:nodeType: 
				argumentList: {'''abc'''. LiteralNode}.
	menu add: 'a Symbol constant' target: self selector: #attachTileForCode:nodeType: 
				argumentList: {'#next'. LiteralNode}.
	menu add: 'true' target: self selector: #attachTileForCode:nodeType: 
				argumentList: {'true'. VariableNode}.
	menu add: 'a Test' target: self  selector: #attachTileForCode:nodeType: 
				argumentList: {'true ifTrue: [self] ifFalse: [self]'. MessageNode}.
	menu add: 'a Loop' target: self selector: #attachTileForCode:nodeType: 
				argumentList: {'1 to: 10 do: [:index | self]'. MessageNode}.
	menu add: 'a Block' target: self selector: #attachTileForCode:nodeType: 
				argumentList: {'[self]'. BlockNode}.
	menu add: 'a Class or Global' target: self selector: #attachTileForCode:nodeType: 
				argumentList: {'Character'. LiteralVariableNode}.
	menu add: 'a Reply' target: self selector: #attachTileForCode:nodeType: 
				argumentList: {'| temp | temp'. ReturnNode}.
	menu popUpAt: ActiveHand position forHand: ActiveHand in: World.
! !

!SyntaxMorph methodsFor: 'menus' stamp: 'tk 9/17/2001 13:38'!
offerVarsMenuFor: aReceiver in: aLexiconModel
	"Offer a menu of tiles for assignment and constants"

	| menu instVarList cls |
	menu _ MenuMorph new addTitle: 'Hand me a tile for...'.
	menu addLine.
	menu add: '(accept method now)' target: aLexiconModel selector: #acceptTiles.
	menu submorphs last color: Color red darker.
	menu addLine.
	menu add: 'new temp variable' target: self selector: #attachTileForCode:nodeType: 
				argumentList: {'| temp | temp'. TempVariableNode}.

	instVarList _ OrderedCollection new.
	cls _ aReceiver class.
	[instVarList addAllFirst: cls instVarNames.
	 cls == aLexiconModel limitClass] whileFalse: [cls _ cls superclass].
	instVarList do: [:nn |
		menu add: nn target: self selector: #instVarTile: argument: nn].
	menu popUpAt: ActiveHand position forHand: ActiveHand in: World.

! !

!SyntaxMorph methodsFor: 'type checking' stamp: 'sw 2/15/2001 14:25'!
okToBeReplacedBy: aSyntaxMorph
	"Return true if it is OK to replace me with aSyntaxMorph.  Enforce the type rules in the old EToy green tiles."

	| itsType myType |
	(Preferences eToyFriendly or: [Preferences typeCheckingInTileScripting])
		ifFalse: [^ true].	"not checking unless one of those prefs is true"
	(parseNode class == BlockNode and: [aSyntaxMorph parseNode class == BlockNode]) 
		ifTrue: [^ true].
	(parseNode class == ReturnNode and: [aSyntaxMorph parseNode class == ReturnNode]) 
		ifTrue: [^ true].
	parseNode class == KeyWordNode ifTrue: [^ false].
	aSyntaxMorph parseNode class == KeyWordNode ifTrue: [^ false].
	parseNode class == SelectorNode ifTrue: [^ false].
	aSyntaxMorph parseNode class == SelectorNode ifTrue: [^ false].
	owner isSyntaxMorph ifFalse: [^ true].	"only within a script"
		"Transcript show: aSyntaxMorph resultType printString, ' dropped on ', 
			self receiverOrArgType printString; cr.
		"
	(itsType _ aSyntaxMorph resultType) == #unknown ifTrue: [^ true].
	(myType _ self receiverOrArgType) == #unknown ifTrue: [^ true].
		"my type in enclosing message"
	^ myType = itsType! !

!SyntaxMorph methodsFor: 'initialization' stamp: 'di 1/31/2001 10:14'!
openInWindow

	| window widget sel |
	sel _ ''.
	self firstSubmorph allMorphs do: [:rr | 
			(rr isKindOf: StringMorph) ifTrue: [sel _ sel, rr contents]].
	window _ (SystemWindow labelled: 'Tiles for ', self parsedInClass printString, '>>',sel).
	widget _ self inAScrollPane.
	widget color: Color paleOrange.
	window
		addMorph: widget
		frame: (0@0 extent: 1.0@1.0).
	window openInWorldExtent: (
		self extent + (20@40) min: (Display boundingBox extent * 0.8) rounded
	)

! !

!SyntaxMorph methodsFor: 'printing' stamp: 'dgd 2/22/2003 13:40'!
ownerPrecedence
	"Return the selector precedence of my owner.  1 for unary (asInteger), 2 for binary arithmetic (+), and 3 for keyword selectors (from:to:).  Subtract 0.5 if self is an arg, not the receiver (the case of a + (b + c))"

	| oo below sel pp |
	oo := owner.
	below := self.
	
	[oo isSyntaxMorph ifFalse: [^10].	"I do not need parens"
	oo parseNode isNil] 
			whileTrue: 
				[below := oo.
				oo := oo owner].
	(sel := oo selector) ifNil: [^10].
	(pp := sel precedence) = 3 ifTrue: [^2.5].	"keyword messages need parens"
	^oo receiverNode == below ifTrue: [pp] ifFalse: [pp - 0.5]! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'RAA 8/15/1999 16:23'!
parseNode
	
	^parseNode
		
	! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'RAA 8/15/1999 16:11'!
parseNode: x
	
	parseNode _ x
		
	! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'di 11/17/2000 08:07'!
parsedInClass

	^ self rootTile parseNode encoder classEncoding! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 11/15/2000 14:39'!
parsedInClass: x

	self parsedInClass == x ifFalse: [self error: 'inconsistent value']! !

!SyntaxMorph methodsFor: 'printing' stamp: 'tk 9/23/2001 02:11'!
printAssignmentNodeOn: strm indent: level
	"sometimes an assignment is in parens"
	| parens above |

	parens _ submorphs size >= 3.
	parens ifTrue: [
		above _ self ownerPrecedence.	"high if not in an expression"
		parens _ above <= 3].	"assignment is a noun inside a message"
	parens ifTrue: [strm nextPut: $( ].
	self
		submorphsDoIfSyntax: [ :sub |
			sub printOn: strm indent: level.
			strm ensureASpace.
		]
		ifString: [ :sub |
			strm ensureNoSpace. 	"_ will have a leading space"
			self printSimpleStringMorph: sub on: strm
		].
	parens ifTrue: [strm ensureNoSpace; nextPut: $) ].
! !

!SyntaxMorph methodsFor: 'printing' stamp: 'RAA 3/4/2001 12:19'!
printBlockArgsNodeOn: strm indent: level

	| argString |

	self
		submorphsDoIfSyntax: [ :sub |
			(argString _ sub decompile) isEmpty ifFalse: [
				strm 
					nextPut: $:;
					nextPutAll: argString;
					space
			].
		] 
		ifString: [ :sub |
			"self printSimpleStringMorph: sub on: strm	<<<< do we need this??"
		].
	strm nextPut: $|; crtab: level.

! !

!SyntaxMorph methodsFor: 'printing' stamp: 'tk 9/23/2001 01:17'!
printBlockNodeOn: strm indent: level

	| lev inASyntaxButNotOutermost subNodeClass |

	lev _ level.
	inASyntaxButNotOutermost _ owner isSyntaxMorph and: [ owner isMethodNode not].
	inASyntaxButNotOutermost ifTrue: [strm nextPut: $[.  lev _ lev+1].
	self
		submorphsDoIfSyntax: [ :sub |
			sub printOn: strm indent: lev.
			subNodeClass _ sub parseNode class.
			(#(BlockArgsNode ReturnNode CommentNode) includes: subNodeClass name) ifFalse: [
				strm ensureNoSpace; nextPut: $.].
			subNodeClass == BlockArgsNode
				ifTrue: [strm space]
				ifFalse: [strm crtab: lev].
		] 
		ifString: [ :sub |
			self printSimpleStringMorph: sub on: strm
		].
	inASyntaxButNotOutermost ifTrue: [strm nextPut: $] ].

! !

!SyntaxMorph methodsFor: 'printing' stamp: 'tk 9/23/2001 01:19'!
printCascadeNodeOn: strm indent: level

	| parens cnt me above |

	parens _ parseNode receiver notNil.
	parens ifTrue: [me _ self selector precedence.
		above _ self ownerPrecedence.	"high if not in an expression"
		parens _ me > above].
	parens ifTrue: [strm nextPut: $( ].
	cnt _ 0.
	self
		submorphsDoIfSyntax: [ :sub |
			cnt _ cnt + 1.
			"maybe we want to test sub isCascadePart for the following???"
			cnt > 2 ifTrue: [strm nextPutAll: '; '].
			sub printOn: strm indent: level.
			strm ensureASpace.
		]
		ifString: [ :sub |
			self printSimpleStringMorph: sub on: strm
		].
	parens ifTrue: [strm ensureNoSpace; nextPut: $) ].
! !

!SyntaxMorph methodsFor: 'printing' stamp: 'tk 9/23/2001 01:17'!
printMessageNodeOn: strm indent: level

	| parens me above |

	parens _ parseNode receiver notNil.
	parens ifTrue: [me _ self selector precedence.
		above _ self ownerPrecedence.	"high if not in an expression"
		parens _ me > above].
	parens ifTrue: [strm nextPut: $( ].
	self
		submorphsDoIfSyntax: [ :sub |
			sub printOn: strm indent: level.
			strm ensureASpace.
		]
		ifString: [ :sub |
			self printSimpleStringMorph: sub on: strm
		].
	parens ifTrue: [strm ensureNoSpace; nextPut: $) ].
! !

!SyntaxMorph methodsFor: 'printing' stamp: 'tk 9/23/2001 02:12'!
printMethodNodeOn: strm indent: level

	(self findA: SelectorNode) ifNil: [
		(self getHeader: strm) ifFalse: [^ self].		"might fail"
		strm crtab: level].
	self 
		submorphsDoIfSyntax: [ :sub |
			sub printOn: strm indent: level.
			strm crtab: level.
		]
		ifString: [ :sub |
			self printSimpleStringMorph: sub on: strm
		]. 
	strm last == $. ifTrue: [strm skip: -1].  "ugh!!  erase duplicate final period"! !

!SyntaxMorph methodsFor: 'printing' stamp: 'RAA 3/4/2001 12:08'!
printMethodTempsNodeOn: strm indent: level

	strm nextPut: $|; space.
	self
		submorphsDoIfSyntax: [ :sub |
			sub printOn: strm indent: level.
			strm space.
		]
		ifString: [ :sub |
			self printSimpleStringMorph: sub on: strm
		].
	strm nextPut: $|; crtab: level.
! !

!SyntaxMorph methodsFor: 'printing' stamp: 'tk 10/22/2000 20:43'!
printOn: strm

	super printOn: strm.
	strm space; nextPutAll: parseNode class name.! !

!SyntaxMorph methodsFor: 'printing' stamp: 'tk 9/14/2001 13:52'!
printOn: strm indent: level

	| nodeClass |

	(self hasProperty: #ignoreNodeWhenPrinting) ifFalse: [
		nodeClass _ parseNode class.
		nodeClass == VariableNode ifTrue: [^self printVariableNodeOn: strm indent: level].
		nodeClass == LiteralVariableNode ifTrue: [^self printVariableNodeOn: strm indent: level].
		nodeClass == MessageNode ifTrue: [^self printMessageNodeOn: strm indent: level].
		nodeClass == BlockNode ifTrue: [^self printBlockNodeOn: strm indent: level].
		nodeClass == BlockArgsNode ifTrue: [^self printBlockArgsNodeOn: strm indent: level].
		nodeClass == MethodNode ifTrue: [^self printMethodNodeOn: strm indent: level].
		nodeClass == MethodTempsNode ifTrue: [^self printMethodTempsNodeOn: strm indent: level].
		nodeClass == CascadeNode ifTrue: [^self printCascadeNodeOn: strm indent: level].
		nodeClass == AssignmentNode ifTrue: [^self printAssignmentNodeOn: strm indent: level].
	].
	self
		submorphsDoIfSyntax: [ :sub |
			sub printOn: strm indent: level.
			strm ensureASpace.
		]
		ifString: [ :sub |
			self printSimpleStringMorph: sub on: strm
		].
! !

!SyntaxMorph methodsFor: 'printing' stamp: 'RAA 2/16/2001 18:16'!
printSimpleStringMorph: aMorph on: strm

	| trialContents |

	(aMorph hasProperty: #wordyVariantOfSelf) ifTrue: [
		strm nextPutAll: 'self '.
		strm nextPutAll: ((self translateToWordySelfVariant: aMorph contents) ifNil: [^self]).
		^self
	].
	(aMorph hasProperty: #noiseWord) ifFalse: [
		trialContents _ self cleanUpString: aMorph.
		strm nextPutAll: trialContents
	].
! !

!SyntaxMorph methodsFor: 'printing' stamp: 'RAA 2/13/2001 23:10'!
printStatementsOn: aStream indent: indent

	"seemed to be necessary to see top node in explorer"

	^parseNode printStatementsOn: aStream indent: indent! !

!SyntaxMorph methodsFor: 'printing' stamp: 'ar 8/16/2001 13:29'!
printVariableNodeOn: strm indent: level

	"nil out any old association"
	parseNode key isVariableBinding ifTrue: [
		parseNode 
			name: parseNode name 
			key: nil 
			code: parseNode code
	].
	self
		submorphsDoIfSyntax: [ :sub |
			sub printOn: strm indent: level.
			strm ensureASpace.
		]
		ifString: [ :sub |
			self printSimpleStringMorph: sub on: strm
		].
! !

!SyntaxMorph methodsFor: 'menus' stamp: 'tk 12/14/2001 11:58'!
putOnBackground
	"Place the receiver, formerly private to its card, onto the shared background.  If the receiver needs data carried on its behalf by the card, such data will be represented on every card."

	| updStr |
	(updStr _ self readOut) ifNotNil: ["If has a place to put per-card data, set that up."
		updStr getSelector ifNotNil: [
			self setProperty: #holdsSeparateDataForEachInstance toValue: true]].
	super putOnBackground.! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 9/25/2001 11:28'!
readOut
	"Find and return an UpdatingStringMorph, possibly in a NumericReadoutTile"

	^ ((self findA: NumericReadoutTile) ifNil: [^ nil]) findA: UpdatingStringMorph! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 7/23/2001 18:04'!
receiverNode
	"If I am (have) a MessageNode, return the node of the receiver.  Watch out for foolish noise words."

	parseNode class == MessageNode ifFalse: [^ nil].
	parseNode receiver ifNil: [^ nil].
	submorphs do: [:ss | 
		ss isSyntaxMorph ifTrue: [
			ss parseNode ifNotNil: ["not noise word"
				ss isNoun ifTrue: [^ ss] 
					ifFalse: [^ nil "found selector"]]]].
	^ nil! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 9/23/2001 00:27'!
receiverObject
	"Return some object that could be the receiver to me (a selector).  Either the actual object who is the receiver in this message, or a guy of the right class."

	| rec value mm |
	(rec _ owner) isSyntaxMorph ifFalse: [^ nil].
	rec _ rec receiverNode.
	rec ifNil: [(rec _ owner owner) isSyntaxMorph ifFalse: [^ nil].
				rec _ rec receiverNode].	
	rec ifNil: [(rec _ owner owner owner) isSyntaxMorph ifFalse: [^ nil].
				rec _ rec receiverNode].
	rec isSelfTile ifTrue: [
		^ ((mm _ self containingWindow model) respondsTo: #targetObject) 
			ifTrue: [mm targetObject]
			ifFalse: [mm selectedClassOrMetaClass new]].
	value _ rec ifNotNil: [rec try].
	value class == Error ifTrue: [
		value _ Vocabulary instanceWhoRespondsTo: self selector].
	^ value! !

!SyntaxMorph methodsFor: 'type checking' stamp: 'tk 2/9/2001 15:56'!
receiverOrArgType
	| ty |
	"Return my type in my role as a receiver or as an argument.  Ask my enclosing message first, then ask myself.  (If owner accepts any #object, and I am a #point, do return #object.)"

	^ (ty _ self receiverOrArgTypeAbove) == #unknown
		ifTrue: [self resultType]
		ifFalse: [ty]! !

!SyntaxMorph methodsFor: 'type checking' stamp: 'dgd 2/22/2003 18:48'!
receiverOrArgTypeAbove
	"Return the type for me according to the message that encloses me."

	| enclosing sub list |
	(self nodeClassIs: BlockNode) ifTrue: [^#command].
	enclosing := owner.
	sub := self.
	
	[enclosing isSyntaxMorph ifFalse: [^#unknown].
	(enclosing nodeClassIs: MessageNode) 
		ifTrue: 
			[list := enclosing submorphs 
						select: [:ss | ss isSyntaxMorph and: [ss parseNode notNil]].
			list size = 1 
				ifFalse: 
					[^(list indexOf: sub) = 1 
						ifTrue: [enclosing receiverTypeFor: enclosing selector]
						ifFalse: [enclosing argTypeFor: enclosing selector]]].
	(enclosing nodeClassIs: BlockNode) ifTrue: [^#command].
	sub := enclosing.
	enclosing := enclosing owner.
	true] 
			whileTrue! !

!SyntaxMorph methodsFor: 'type checking' stamp: 'sw 2/27/2001 09:11'!
receiverTypeFor: aSelector
	"Answer the type of the receiver of this selector.  Return #unknown if not found."

	| itsInterface |

	aSelector ifNil: [^ #unknown].
	itsInterface _ self currentVocabulary methodInterfaceAt: aSelector ifAbsent:
		[^ #unknown].
	^ itsInterface receiverType! !

!SyntaxMorph methodsFor: 'insertion drop zones' stamp: 'di 1/30/2001 21:00'!
removeDropZones
	"Remove the insertion drop-zone morphs."

	self submorphsDo:
		[:mm | (mm isMemberOf: BorderedMorph) ifTrue: [mm delete]].
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 8/1/2001 13:06'!
removeReturnNode
	| blk |
	"If last line is ^ self, remove it.  I am a methodNode.  Keep if no other tiles in the block."

	blk _ self findA: BlockNode.
	blk submorphs last decompile string = '^self ' ifTrue: [
		(blk submorphs count: [:ss | ss isSyntaxMorph]) > 1 ifTrue: [
			blk submorphs last delete]].! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 1/13/2001 20:08'!
rename: newSelector
	| keywords mainSel list last |
	"Attempt to change the name as listed in my tiles.  Can change the number of argumtents.  MethodNode (SelectorNode (SelectorNode (string))) or MethodNode (SelectorNode (SelectorNode (string) TempVarNode() SelectorNode (string) TempVarNode()))"

	self isMethodNode ifFalse: [
		self rootTile == self ifTrue: [^ self].  "not in a script"
		^ self rootTile rename: newSelector  "always do at the root"].

	keywords _ newSelector keywords.
	mainSel _ self findA: SelectorNode.
	list _ mainSel submorphs select: [:mm | 
		mm isSyntaxMorph and: [mm parseNode class == SelectorNode]].
	1 to: (list size min: keywords size) do: [:ind |
		((list at: ind) findA: UpdatingStringMorph) contents: (keywords at: ind)].
	keywords size + 1 to: list size do: [:ind | "removing keywords"
		[last _ mainSel submorphs last.
		 (last isSyntaxMorph and: [last parseNode class == TempVariableNode])] whileFalse: [
				last delete].
		[last _ mainSel submorphs last.
		 (last isSyntaxMorph and: [last parseNode class == SelectorNode])] whileFalse: [
				last delete].	"the TempVariableNode and others"
		mainSel submorphs last delete.	"the SelectorNode"
		].
	list size + 1 to: keywords size do: [:ind | "adding keywords"
		"add a SelectorNode, add a spacer, add a TempVarNode"
		mainSel addToken: (keywords at: ind) type: #keyword1 
			on: (SelectorNode new key: (keywords at: ind) code: nil).
		mainSel addMorphBack: (mainSel transparentSpacerOfSize: 4@4).
		(TempVariableNode new name: 'arg', ind printString index: ind type: nil scope: nil)
			 asMorphicSyntaxIn: mainSel].! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 9/28/2001 13:39'!
replaceKeyWord: evt menuItem: stringMorph
	"Replace my entire message (which may be multi-part) with the one specified.  Preserve all argument tiles, either in the new message or in the world outside the scriptor.  I am a SelectorNode or KeyWordNode."

	| menu new news newSel mm newTree newRec newArgs top oldArgNodes share ctrY |
	(menu _ stringMorph owner owner) class == RectangleMorph ifTrue: [
		menu delete].
	new _ stringMorph contents.
	new first = $( ifTrue: [^ self].	"Cancel"
	new first = $  ifTrue: [^ self].	"nothing"
	news _ String streamContents: [:strm | "remove fake args"
		(new findBetweenSubStrs: #(' 5' $ )) do: [:part | strm nextPutAll: part]].
	newSel _ stringMorph valueOfProperty: #syntacticallyCorrectContents.
	newSel ifNil: [newSel _ news].
	mm _ MessageSend receiver: 5 selector: newSel 
			arguments: ((Array new: newSel numArgs) atAllPut: 5).
	newTree _ mm asTilesIn: Object globalNames: false.
	newRec _ newTree receiverNode.
	newArgs _ newTree argumentNodes.
	ctrY _ self fullBoundsInWorld center y.
	top _ self messageNode.
	newRec owner replaceSubmorph: newRec by: top receiverNode.
	oldArgNodes _ top argumentNodes.
	share _ newArgs size min: oldArgNodes size.
	(newArgs first: share) with: (oldArgNodes first: share) do: [:newNode :oldNode | 
		newNode owner replaceSubmorph: newNode by: oldNode].
	"later get nodes for objects of the right type for new extra args"

	top owner replaceSubmorph: top by: newTree.

	"Deposit extra args in the World"
	(oldArgNodes copyFrom: share+1 to: oldArgNodes size) do: [:leftOver |
		(leftOver parseNode class == LiteralNode and: [leftOver decompile asString = '5']) 
			ifFalse: [newTree pasteUpMorph addMorphFront: leftOver.
				leftOver position: newTree enclosingPane fullBoundsInWorld right - 20 @ ctrY.
				ctrY _ ctrY + 26]
			ifTrue: [leftOver delete]].
	newTree acceptIfInScriptor.! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 9/19/2001 21:32'!
replaceSel: evt menuItem: stringMorph
	"I rep a SelectorNode.  Replace my selector with new one that was just chosen from a menu"

	| menu new old newSel ms oa na case news |
	(menu _ stringMorph owner owner) class == RectangleMorph ifTrue: [
		menu delete].
	new _ stringMorph contents.
	new first = $( ifTrue: [^ self].	"Cancel"
	new first = $  ifTrue: [^ self].	"nothing"
	news _ String streamContents: [:strm | "remove fake args"
		(new findBetweenSubStrs: #(' 5' $ )) do: [:part | strm nextPutAll: part]].
	newSel _ stringMorph valueOfProperty: #syntacticallyCorrectContents.
	newSel ifNil: [newSel _ news].
	old _ (ms _ self findA: StringMorph) valueOfProperty: #syntacticallyCorrectContents.
	old ifNil: [old _ (self findA: StringMorph) contents].
	oa _ old numArgs.  na _ newSel numArgs.  case _ 5.
	(oa = 1) & (na = 1) ifTrue: [case _ 1]. 
	(oa = 0) & (na = 0) ifTrue: [case _ 2].
	(oa = 1) & (na  = 0) ifTrue: [case _ 3].
	(oa = 0) & (na  = 1) ifTrue: [case _ 4].
	case <= 4 ifTrue: ["replace the selector"
		ms contents: news.	"not multi-part"
		ms setProperty: #syntacticallyCorrectContents toValue: newSel].
	case = 3 ifTrue: [owner tossOutArg: 1].
	case = 4 ifTrue: [self addArg: 1].
	"more cases here.  Rebuild the entire MessageNode"
	
	self acceptIfInScriptor.! !

!SyntaxMorph methodsFor: 'type checking' stamp: 'yo 7/22/2005 16:43'!
resultType
	"Look up my result type.  If I am a constant, use that class.  If I am a message, look up the selector."

	| list value soundChoices |
	parseNode class == BlockNode ifTrue: [^#blockContext].
	parseNode class == AssignmentNode ifTrue: [^#command].
	parseNode class == ReturnNode ifTrue: [^#command].	"Need more restriction than this"
	list := submorphs 
				select: [:ss | ss isSyntaxMorph and: [ss parseNode notNil]].
	list size > 1 ifTrue: [^self resultTypeFor: self selector].
	list size = 1 
		ifTrue: 
			["test for levels that are just for spacing in layout"

			(list first isSyntaxMorph and: [list first nodeClassIs: MessageNode]) 
				ifTrue: [^list first resultType]].	"go down one level"
	value := self try.
	value class == Error ifTrue: [^#unknown].
	(value isNumber) ifTrue: [^#Number].
	(value isKindOf: Boolean) ifTrue: [^#Boolean].
	(value isForm) ifTrue: [^#Graphic].
	(value isPlayerLike and: [value costume renderedMorph isMemberOf: KedamaPatchMorph]) ifTrue: [^#Patch].
	value isString
		ifTrue: 
			[soundChoices := #('silence').	"default, if no SampledSound class"
			Smalltalk at: #SampledSound
				ifPresent: [:sampledSound | soundChoices := sampledSound soundNames].
			(soundChoices includes: value) ifTrue: [^#Sound]].
	(value isPlayerLike) ifTrue: [^#Player].
	^value class name asLowercase	"asSymbol (not needed)"! !

!SyntaxMorph methodsFor: 'type checking' stamp: 'sw 2/24/2001 12:13'!
resultTypeFor: aSelector
	"Answer the result type of selector.  Return #unknown if not found."

	| itsInterface |
	aSelector ifNil: [self inform: 'Please tell Ted how you caused this'.
		^ #abs "a bogus type"].
	itsInterface _ self currentVocabulary methodInterfaceAt: aSelector ifAbsent:
		[^ #unknown].
	^ itsInterface resultType! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 9/28/2001 10:06'!
retract
	"replace this message with its receiver.  I am the message node."
	| rec cascade msg |
	(self nodeClassIs: CascadeNode) ifTrue:
		["This is a piece of a cascaded message -- just delete it"
		self deletePopup.
		cascade _ owner.
		self delete.
		cascade setSelection: {cascade. nil. cascade}.
		^ cascade acceptIfInScriptor].
	self deletePopup.
	(rec _ self receiverNode)
		ifNil: [msg _ owner.
			rec _ owner receiverNode.
			msg owner replaceSubmorph: msg by: rec]
		ifNotNil: [owner replaceSubmorph: self by: rec].
	rec setSelection: {rec. nil. rec}.
	rec acceptIfInScriptor.! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'di 12/13/2000 12:57'!
retractArrow
	"Return the retract arrow button.  It replaces the current message with its receiver.
	I am in a MessageNode whose first subnode is not a MessagePartNode.  I did not encounter a block on the way up to it.  I am the last subnode in every owner up to it."
	| patch |

	(self nodeClassIs: MessageNode) ifFalse: [^ nil].
	(owner isSyntaxMorph and: [owner parseNode == parseNode]) ifTrue: [^ nil].

	patch _ (ImageMorph new image: (TileMorph classPool at: #RetractPicture)).
	patch on: #mouseDown send: #retract to: self.
	^ patch! !

!SyntaxMorph methodsFor: 'initialization' stamp: 'FBS 2/24/2004 14:21'!
returnNode: aNode expression: expr

	| row expMorph sMorph aNoiseString |
	row _ self addRow: #return on: aNode.
	self alansTest1 ifTrue: [
		row setSpecialOuterTestFormat.
		aNoiseString _ ' Reply '.
		sMorph _ self aSimpleStringMorphWith: aNoiseString.
		sMorph 
			emphasis: TextEmphasis bold emphasisCode;
			setProperty: #syntacticallyCorrectContents toValue: '^'.

		row addMorphBack: sMorph.
	] ifFalse: [
		row addToken: '^ ' type: #upArrow on: aNode.
	].
	expMorph _ expr asMorphicSyntaxIn: row.
	self alansTest1 ifTrue: [
		(expMorph hasProperty: #deselectedColor) ifFalse: [expMorph setConditionalPartStyle].
	].
	expr addCommentToMorph: row.
	^row
! !

!SyntaxMorph methodsFor: 'node types' stamp: 'dgd 2/22/2003 13:40'!
rootTile
	^self 
		orOwnerSuchThat: [:m | m owner isNil or: [m owner isSyntaxMorph not]]! !

!SyntaxMorph methodsFor: 'initialization' stamp: 'tk 1/19/2001 13:29'!
sample: arg1
"a comment"
| temp1 |
temp1 _ 5.
temp1 yourself.
temp1 min: arg1.! !

!SyntaxMorph methodsFor: 'selection' stamp: 'tk 7/31/2001 17:18'!
scaleColorByUserPref: aColor
	
	| myRoot underLyingColor |

	myRoot _ self rootTile.
	underLyingColor _ myRoot ifNil: [Color transparent] ifNotNil: [myRoot color].
	[underLyingColor isTransparent and: [(myRoot _ myRoot owner) notNil]] whileTrue: [
		underLyingColor _ myRoot color.
	].
	
	"rude hack to get the desired effect before we have an owner"

	underLyingColor isTransparent ifTrue: [underLyingColor _ Color r: 0.903 g: 1.0 b: 0.903].
	^aColor mixed: (ContrastFactor ifNil: [0.3]) with: underLyingColor

"Would like to be able to make MethodNode and outer Block be transparent.  This method does not allow that.  Consider (^ myRoot color) inside the whileTrue.  Consider setting underLyingColor to (myRoot valueOfProperty: #deselectedBorderColor ifAbsent: [myRoot color]) in second line."! !

!SyntaxMorph methodsFor: 'selection' stamp: 'tk 7/23/2001 18:31'!
select
	self deselect.
	"Outer block is not colored and has no popup"
	(owner isSyntaxMorph and: [owner nodeClassIs: MethodNode]) 
		ifTrue: [self setDeselectedColor "normal"]
		ifFalse: [self color: Color lightBrown].
	self borderColor: #raised.
	self offerPopUp.! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'dgd 2/22/2003 13:41'!
selector
	"Find the selector I represent, or have inside of me.  My parseNode is a SelectorNode or a MessageNode."

	| sel cnt |
	parseNode class == SelectorNode 
		ifTrue: [^self decompile asString asSymbol].
	parseNode class == KeyWordNode ifTrue: [^self decompile asString asSymbol].
	parseNode class == MessageNode | (parseNode class == MessagePartNode) 
		ifFalse: [^nil].
	"Must be one of those to have a selector"
	"Beware of messageParts.  If MessagePartNode, only returns this one keyword."
	sel := ''.
	cnt := 0.
	submorphs do: 
			[:mm | 
			mm isSyntaxMorph 
				ifTrue: 
					[cnt := cnt + 1.
					(mm nodeClassIs: SelectorNode) ifTrue: [^mm selector].
					(mm nodeClassIs: MessagePartNode) ifTrue: [sel := sel , mm selector].
					(mm nodeClassIs: KeyWordNode) ifTrue: [sel := sel , mm decompile asString].
					(mm nodeClassIs: ReturnNode) ifTrue: [cnt := cnt - 1].
					(mm nodeClassIs: MessageNode) 
						ifTrue: 
							[parseNode receiver ifNil: [sel := mm selector].
							cnt = 2 & (sel isEmpty) 
								ifTrue: 
									["not the receiver.  Selector and arg"

									sel := mm selector]]]].
	sel ifNil: [^nil].
	sel notEmpty ifTrue: [^sel asSymbol].
	^nil! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 9/18/2001 16:27'!
selectorMenu
	"Put up a menu of all selectors that my receiver could be sent.  Replace me with the one chosen.  (If fewer args, put the tiles for the extra arg to the side, in script's owner (world?).)
	Go ahead and eval receiver to find out its type.  Later, mark selectors for side effects, and don't eval those.
	Put up a table.  Each column is a viewer category."

	| cats value catNames interfaces list setter wording all words ind aVocabulary limitClass |
	cats _ #().
	all _ Set new.
	value _ self receiverObject.
	value class == Error ifTrue: [^ nil].
	
	aVocabulary _ self vocabularyToUseWith: value.
	limitClass _ self limitClassToUseWith: value vocabulary: aVocabulary.
	catNames _ value categoriesForVocabulary: aVocabulary limitClass: limitClass.
	cats _ catNames collect: [:nn | 
		list _ OrderedCollection new.
		interfaces _ value methodInterfacesForCategory: nn 
						inVocabulary: aVocabulary limitClass: limitClass.
		interfaces do: [:mi | 
			(all includes: mi selector) ifFalse: [
				"list add: (self aSimpleStringMorphWith: mi elementWording).  Expensive"
				words _ mi selector.
				(words beginsWith: 'get ') ifTrue: [words _ words allButFirst: 4].
				mi selector last == $: ifTrue: [
					words _ String streamContents: [:strm | "add fake args"
						(words findTokens: $:) do: [:part | strm nextPutAll: part; nextPutAll: ' 5 ']].
					words _ words allButLast].
				mi selector isInfix ifTrue: [words _ words, ' 5'].
				words _ self splitAtCapsAndDownshifted: words.	
				list add: (self anUpdatingStringMorphWith: words special: true).
				words = mi selector ifFalse: [
					list last setProperty: #syntacticallyCorrectContents toValue: mi selector].
				all add: mi selector].
			setter _ mi companionSetterSelector asString.
			(setter = 'nil') | (all includes: setter) ifFalse: ["need setters also"
				wording _ (self translateToWordySetter: setter).
				list add:  (self aSimpleStringMorphWith: wording, ' 5').
				wording = setter ifFalse: [
					list last setProperty: #syntacticallyCorrectContents 
						toValue: setter].
				all add: setter]].
		list].
	(ind _ catNames indexOf: 'scripts') > 0 ifTrue: [
		(cats at: ind) first contents = 'empty script' ifTrue: [(cats at: ind) removeFirst]].
	cats first addFirst: (self aSimpleStringMorphWith: ' ').	"spacer"
	cats first addFirst: (self aSimpleStringMorphWith: '( from ', value class name, ' )').
	cats first first color: (Color green mixed: 0.25 with: Color black).
	self selectorMenuAsk: cats.	"The method replaceSel:menuItem: does the work.  
		and replaces the selector."
	! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 9/20/2001 16:04'!
selectorMenuAsk: listOfLists
	"I represent a SelectorNode to be replaced by one of the selectors in one of the category lists.  Each list has pre-built StringMorphs in it."

	| menu col |
	listOfLists isEmpty ifTrue: [^ nil].
	listOfLists first addFirst: (self aSimpleStringMorphWith: '( Cancel )').
	listOfLists first first color: Color red.
	menu _ RectangleMorph new.
	menu listDirection: #leftToRight; layoutInset: 3; cellInset: 1@0.
	menu layoutPolicy: TableLayout new; hResizing: #shrinkWrap; 
		vResizing: #shrinkWrap; color: (Color r: 0.767 g: 1.0 b: 0.767);
		useRoundedCorners; cellPositioning: #topLeft.
	listOfLists do: [:ll |
		col _ Morph new.
	 	col listDirection: #topToBottom; layoutInset: 0; cellInset: 0@0.
		col layoutPolicy: TableLayout new; hResizing: #shrinkWrap.
		col color: Color transparent; vResizing: #shrinkWrap.
		menu addMorphBack: col.
		ll do: [:ss | 
			col addMorphBack: ss.
			ss on: #mouseUp send: #replaceKeyWord:menuItem: to: self]
		].
	self world addMorph: menu.
	menu setConstrainedPosition: (owner localPointToGlobal: self topRight) + (10@-30) 
			hangOut: false.
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 22:50'!
setConditionalPartStyle

	self specialColor: self lighterColor andBorder: self darkerColor.
	self useRoundedCorners.
	self borderWidth: 1.
! !

!SyntaxMorph methodsFor: 'selection' stamp: 'tk 7/19/2001 17:52'!
setDeselectedColor
	"The normal color of the tile, stored with the tile"
	| deselectedColor deselectedBorderColor |

	deselectedColor _ self valueOfProperty: #deselectedColor ifAbsent: [nil].
	deselectedBorderColor _ self valueOfProperty: #deselectedBorderColor ifAbsent: [nil].
	deselectedColor ifNotNil: [
		deselectedColor _ self scaleColorByUserPref: deselectedColor].
	deselectedBorderColor ifNotNil: [
		deselectedBorderColor _ self scaleColorByUserPref: deselectedBorderColor].
	self 
		color: (deselectedColor ifNil: [Color transparent]);
		borderColor: (deselectedBorderColor ifNil: [Color transparent])! !

!SyntaxMorph methodsFor: 'card & stack' stamp: 'tk 9/25/2001 11:41'!
setNewContentsFrom: stringOrNumberOrNil
	"Using stringOrNumberOrNil as a guide, set the receiver's contents afresh.  If the input parameter is nil, the a default value stored in a property of the receiver, if any, will supply the new initial content.  This method is only called when a VariableDock is attempting to put a new value."

	(self readOut ifNil: [^ self]) setNewContentsFrom: stringOrNumberOrNil.! !

!SyntaxMorph methodsFor: 'selection' stamp: 'dgd 2/22/2003 13:41'!
setSelection: newSpec 
	"A selectionSpec is {Inner morph.  Where clicked.  Outer morph}.
	First mouseDown starts a selection (with outerMorph isNil).
	Dragging more than 4 pixels means to grab a copy of the current outer selection.
		The current selection is the outerMorph, or the inner if it is nil.
	Each mouseUp extends the selection to the next outer morph that is selectable.
		Except if this is the first click."

	| rootTile |
	(rootTile := self rootTile) valueOfProperty: #selectionSpec
		ifPresentDo: [:oldSpec | oldSpec third ifNotNilDo: [:m | m deselect]].
	(newSpec isNil or: [newSpec third isNil and: [self isMethodNode]]) 
		ifTrue: 
			[self deselect.
			^rootTile removeProperty: #selectionSpec].

	"Select outer morph of the new selection"
	newSpec third isNil 
		ifTrue: [self select	"first click down"]
		ifFalse: [newSpec third select	"subsequent clicks"].
	rootTile setProperty: #selectionSpec toValue: newSpec! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'sw 3/18/2004 00:35'!
setSelector: stringLike in: stringMorph
	"Store the new selector and accept method."

	| aSymbol myType str |
	aSymbol _ stringLike asSymbol.
	(ScriptingSystem helpStringOrNilFor: aSymbol) ifNotNilDo:
		[:aString |
			self setBalloonText: aString translated].
	myType _ stringMorph valueOfProperty: #syntacticReformatting ifAbsent: [#none].
	str _ aSymbol.
	(self isStandardSetterKeyword: str) ifTrue: [str _ self translateToWordySetter: str].
	(self isStandardGetterSelector: str) ifTrue: [str _ self translateToWordyGetter: str].
	(self shouldBeBrokenIntoWords: myType) 
		ifTrue: [str _ self substituteKeywordFor: str].
	stringMorph contents: str.
	"parseNode key: aSymbol code: nil."
	str = stringLike ifFalse:
		[stringMorph setProperty: #syntacticallyCorrectContents toValue: aSymbol].
	self acceptSilently! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/27/2001 07:34'!
setSpecialOuterTestFormat

	self 
		specialColor: self darkerColor 
		andBorder: self lighterColor.
	self useRoundedCorners.
	self layoutInset: 1.
	"self setProperty: #variableInsetSize toValue: 6."
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 22:50'!
setSpecialTempDeclarationFormat1

	"the outer template for temp defs"

	self 
		specialColor: self darkerColor 
		andBorder: self lighterColor.
	"self 
		specialColor: (Color lightYellow) 
		andBorder: (Color r: 0.581 g: 0.774 b: 0.903)."
	self useRoundedCorners.
	self layoutInset: 1.
	self cellPositioning: #center.
	"self setProperty: #variableInsetSize toValue: 6."
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 22:50'!
setSpecialTempDeclarationFormat2

	"the inner template for temp defs"

	self 
		specialColor: self lighterColor 
		andBorder:  self darkerColor.
	"self 
		specialColor: (Color r: 1.0 g: 1.0 b: 0.548) 
		andBorder:  (Color r: 0.581 g: 0.774 b: 0.903)."
	self useRoundedCorners.
	self layoutInset: 1.
	"self setProperty: #variableInsetSize toValue: 6."
! !

!SyntaxMorph methodsFor: 'card & stack' stamp: 'tk 11/4/2001 21:47'!
setTarget: aPlayer
	"Find my UpdatingStringMorph and set its getSelector, putSelector, and target"

	| updatingString |
	(updatingString _ self readOut) ifNil: [^ self].
	updatingString putSelector: (Utilities setterSelectorFor: self knownName).
	updatingString getSelector: (Utilities getterSelectorFor: self knownName).
	updatingString target: aPlayer. ! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'tk 7/27/2001 16:53'!
shouldBeBrokenIntoWords: aSymbol

	^#(methodHeader1 methodHeader2 keyword2 upArrow 
		tempVariable tempVariableDeclaration blockarg2 variable
		keywordGetz keywordSetter unaryGetter
		assignmentArrow) includes: aSymbol! !

!SyntaxMorph methodsFor: 'menus' stamp: 'di 11/17/2000 09:00'!
showCode
	"Turn my current state into the text of a method.  Put it in a window."

	(Workspace new contents: self rootTile decompile) openLabel: self printString,' code'

	
! !

!SyntaxMorph methodsFor: 'menus' stamp: 'tk 12/10/2001 17:48'!
showMenu: evt
	| menu |
	menu _ MenuMorph new.
	self rootTile isMethodNode ifTrue:
		[menu add: 'accept method' target: self selector: #accept.
		menu addLine.

		menu add: 'new temp variable' target: self selector: #attachTileForCode:nodeType: 
					argumentList: {'| temp | temp'. TempVariableNode}.
		menu addLine.

		self parsedInClass allInstVarNames do: [:nn |
			menu add: nn,' tile' target: self selector: #instVarTile: argument: nn].
		menu addLine.

		menu add: 'show code' target: self selector: #showCode.
		menu add: 'try out' target: self selector: #try.
		menu popUpAt: evt hand position forHand: evt hand in: World].



! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 13:44'!
specialColor: c1 andBorder: c2

	self color: (self scaleColorByUserPref: c1).
	self setProperty: #deselectedColor toValue: c1.
	self borderColor: (self scaleColorByUserPref: c2).
	self setProperty: #deselectedBorderColor toValue: c2.
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'yo 11/11/2002 10:32'!
splitAtCapsAndDownshifted: aString

	self flag: #yoCharCases.

	^String streamContents: [ :strm |
		aString do: [ :each | 
			each = $: ifFalse: [
				each isUppercase ifTrue: [strm nextPut: (Character value: 0);  
						 	nextPut: (Character value: 0); 
						 	nextPut: (Character value: 0); 
							nextPut: each asLowercase]
					ifFalse: [strm nextPut: each]
			].
		]
	].! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 23:01'!
standardCellPositioning

	^ self alansTest1 ifTrue: [#leftCenter] ifFalse: [#topLeft]! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'tk 7/25/2001 17:33'!
standardInset

	parseNode class == BlockNode ifTrue: [^ 5@1].
		"allow pointing beside a line so can replace it"
	^ self alansTest1 ifTrue: [1] ifFalse: [-1]! !

!SyntaxMorph methodsFor: 'highlighting' stamp: 'tk 7/19/2001 19:09'!
stdBorderColor 

	"put choices of how to do the border here"
	^ self valueOfProperty: #deselectedBorderColor ifAbsent: [Color transparent]! !

!SyntaxMorph methodsFor: 'stepping and presenter' stamp: 'tk 7/25/2001 11:26'!
step
	super step.
	self isBlockNode ifTrue: [self trackDropZones].
! !

!SyntaxMorph methodsFor: 'testing' stamp: 'di 11/3/2000 08:03'!
stepTime

	^ 50! !

!SyntaxMorph methodsFor: 'new tiles' stamp: 'tk 9/13/2001 13:44'!
string: anExpression toTilesIn: playerClass
	| code tree methodNode |
	"Construct SyntaxMorph tiles for some code.  Returns the main BlockNode of a doIt."

	"This is really cheating!!  Make a true parse tree later. -tk"
	code _ String streamContents: [:strm | 
		strm nextPutAll: 'doIt'; cr; tab; nextPutAll: anExpression].
	"decompile to tiles"
	tree _ Compiler new 
		parse: code 
		in: playerClass
		notifying: nil.
	methodNode _ tree asMorphicSyntaxUsing: SyntaxMorph.
	anExpression first == $" ifTrue: ["a comment" 
		"(methodNode findA: CommentNode) firstSubmorph color: Color blue."
		^ methodNode].
	^ methodNode submorphs detect: [:mm | 
		(mm respondsTo: #parseNode) 
			ifTrue: [mm parseNode class == BlockNode] 
			ifFalse: [false]].
! !

!SyntaxMorph methodsFor: 'printing' stamp: 'di 11/13/2000 07:43'!
structure
	"Print my structure from inner to outer."
	^ String streamContents: [:s |
		self withAllOwnersDo:
			[:m | m isSyntaxMorph ifTrue:
				[s cr; print: m parseNode class.
				((m nodeClassIs: MessageNode) or: [m nodeClassIs: TileMessageNode]) ifTrue:
					[s space; nextPutAll: m parseNode selector key]]]]! !

!SyntaxMorph methodsFor: 'dropping/grabbing' stamp: 'tk 9/24/2001 10:04'!
structureMatchWith: aMorph
	| meNoun itNoun |
	"Return true if the node types would allow aMorph to replace me.  This tests the gross structure of the method only."

	meNoun _ self isNoun.
	itNoun _ aMorph isNoun.

	"Consider these nouns to be equal:  TempVariableNode, LiteralNode, VariableNode, (MessageNode with receiver), CascadeNode, AssignmentNode"
	meNoun & itNoun ifTrue: [^ true].
	meNoun & aMorph isBlockNode ifTrue: [^ true].

	"If I am a BlockNode, and it is a TempVariableNode, add it into list"
	"If I am a BlockNode, and it is a noun, add it as a new line"
	self isBlockNode ifTrue:
		[itNoun ifTrue: [^ true].
		(aMorph nodeClassIs: ReturnNode) ifTrue:
			[^ (self submorphs
				detect: [:mm | ((mm isSyntaxMorph) and: [mm nodeClassIs: ReturnNode])]
				ifNone: [nil]) isNil].	"none already in this block"
				"If I am a BlockNode, and it is a ReturnNode, add to end"
		(aMorph nodeClassIs: CommentNode) ifTrue: [^ true]].

	(self isMethodNode) ifTrue: [^ false].	"Later add args and keywords"
		"Later allow comments to be dropped in"
		"Add MethodTemps by dropping into the main block"

	(self nodeClassIs: ReturnNode) & (aMorph parseNode class == MessageNode) 
		ifTrue: [^ true].		"Command replace Return"
	(self nodeClassIs: MessageNode) & (aMorph parseNode class == ReturnNode) ifTrue: [
		(owner submorphs select: [:ss | ss isSyntaxMorph]) last == self
			ifTrue: [^ true]].	"Return replace last command"

	(aMorph nodeClassIs: AssignmentNode) ifTrue: [
		itNoun ifFalse: ["create a new assignment"
			^ self isAVariable & self isDeclaration not]].	"only assign to a variable"

	"If nodes are of equal class, replace me with new one."
	(self nodeClassIs: aMorph parseNode class) ifTrue: [
		(self nodeClassIs: MessageNode) 
				ifFalse: [^ true]	"normal match"
				ifTrue: [^ self receiverNode == aMorph receiverNode]].	"both nil"

	^ false "otherwise reject"
! !

!SyntaxMorph methodsFor: 'printing' stamp: 'RAA 3/4/2001 11:55'!
submorphsDoIfSyntax: block1 ifString: block2 

	^self submorphsDoIfSyntax: block1 ifString: block2 otherwise: [ :sub | ]
! !

!SyntaxMorph methodsFor: 'printing' stamp: 'gm 2/22/2003 12:34'!
submorphsDoIfSyntax: block1 ifString: block2 otherwise: block3 
	submorphs do: 
			[:sub | 
			sub isSyntaxMorph 
				ifTrue: [block1 value: sub]
				ifFalse: 
					[(sub isKindOf: StringMorph) 
						ifTrue: [block2 value: sub]
						ifFalse: 
							[(sub isTextMorph) 
								ifTrue: [block2 value: sub]
								ifFalse: [block3 value: sub]]]]! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 4/4/2001 13:12'!
substituteKeywordFor: aString

	aString isEmpty ifTrue: [^aString asString].
	aString asString = '^ ' ifTrue: [^'answer'].
	aString asString = 'ifTrue:' ifTrue: [^'Yes'].
	aString asString = 'ifFalse:' ifTrue: [^'No'].
	aString asString = 'self' ifTrue: [^'self'].
	aString first isUppercase ifTrue: [^aString asString].

	^self splitAtCapsAndDownshifted: aString! !

!SyntaxMorph methodsFor: 'scripting' stamp: 'tk 9/26/2001 06:01'!
tearOffTile
	"For a SyntaxMorph, this means give a copy of me"

	| dup |
	dup _ self duplicate.
	ActiveHand attachMorph: dup.
	Preferences tileTranslucentDrag
		ifTrue: [^ dup lookTranslucent]
		ifFalse: [^ dup align: dup topLeft with: ActiveHand position + self cursorBaseOffset]
! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 8/22/2001 16:35'!
tempVarNodesDo: aBlock
	"Execute the block for any block temporary variables, method temps, or method args we have"

	| tempHolder argsHolder |
	((self parseNode class == MethodNode) or: [self parseNode class == BlockNode]) ifTrue: [
		self submorphsDoIfSyntax: [:sub | 
				(sub nodeClassIs: MethodTempsNode) ifTrue: [tempHolder _ sub].
				((sub nodeClassIs: UndefinedObject) and: [tempHolder isNil]) ifTrue: [
					tempHolder _ sub findA: MethodTempsNode].
				(sub nodeClassIs: BlockArgsNode) ifTrue: [tempHolder _ sub].
				(sub nodeClassIs: SelectorNode) ifTrue: [argsHolder _ sub].
				]
			ifString: [:sub | ].
		tempHolder ifNotNil: ["Temp variables"
			tempHolder submorphsDoIfSyntax: [:sm | 
					(sm nodeClassIs: TempVariableNode) ifTrue: [aBlock value: sm]]
				ifString: [:sm | ]].
		argsHolder ifNotNil: ["arguments"
			argsHolder submorphsDoIfSyntax: [:sm | 
					(sm nodeClassIs: TempVariableNode) ifTrue: [aBlock value: sm]]
				ifString: [:sm | ]].
		].
	"otherwise do nothing"! !

!SyntaxMorph methodsFor: 'tests' stamp: 'di 11/6/2000 10:47'!
test
	3 > 4 ifTrue: [].
	^ self! !

!SyntaxMorph methodsFor: 'tests' stamp: 'gm 2/22/2003 12:35'!
testForNode: targetNode andDo: aBlock 
	targetNode == parseNode ifTrue: [aBlock value: self].
	self submorphsDo: 
			[:each | 
			(each isSyntaxMorph) 
				ifTrue: [each testForNode: targetNode andDo: aBlock]]! !

!SyntaxMorph methodsFor: 'tests' stamp: 'di 11/14/2000 14:54'!
toDo
"
Biggies...
[ ]	Integrate with EToy scriptors
	releaseCachedState can discard all morphic structure.

[ ]	Options:
	Show / hide syntax markers (like [], (), ., :, ;, etc)
	No color / color-in-focus / full color
	Tiles / textiles / text

[ ]	ParsedTextMorph -- looks like text but has all same substructure

[ ]	Introduce notion of an UnParsedNode -- maybe a flag in ParseNode
	Text -> UnParsed -> Parsed -> CodeGen

[ ]	Need DnD evaluator, or some sort of '!!' button on any entity (halo?)
	Also inspector / browser

[ ]	All the type help we can get

Details ...
[ ]	Open up the parse of BraceNodes

[ ]	Verify that all pastes are OK

[ ]	Colors not yet right for colored version.

[ ]	Start work on show / hide of syntax markers -- (), [], etc.

[ ]	Start work on textiles (grabable entites in 'normal' text)

[ ]	Need autoscroll during drag for drop

[ ]	Use, eg, shift-drag to move, del to delete

[ ]	What about invalid drops -- stick on cursor?

System...
[ ]	Only keep history 7 deep; option to clear on quit
	clear command above spaceLeft

[ ]	Compute each page of prefs viewer on demand instead of as now.

[ ]	Offer a search command that will gather up all preferences that match a given string (name or help string)

Preferences enable: #noTileColor.
Preferences disable: #noTileColor.
Smalltalk browseAllSelect: [:cm | cm size > 600]
SyntaxMorph testAll
"! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/28/2001 15:03'!
tokenVerticalSeparator

	^Morph new 
		color: Color transparent;
		extent: 3@3;
		lock
! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 9/19/2001 15:47'!
tossOutArg: extras

	"Remove the tiles for the last N keywords and arguments.  Place the tiles beside the current window.  I am a SyntaxMorph for a MessageNode."

	| cnt ctr |
	cnt _ 0.
	 submorphs copy reverseDo: [:sub |
		ctr _ sub fullBoundsInWorld center.
		sub delete.
		(sub isSyntaxMorph and: [sub parseNode notNil]) ifTrue: [
			sub isNoun ifTrue: [
				self pasteUpMorph addMorphFront: sub.
				sub position: self enclosingPane fullBoundsInWorld right - 20 @ ctr y].
			(cnt _ cnt + 1) >= extras ifTrue: [^ self]]].! !

!SyntaxMorph methodsFor: 'insertion drop zones' stamp: 'tk 9/13/2001 15:24'!
trackDropZones
	| hand i localPt insertion insHt ii prevBot nxtHt d c1 c2 ht2 spacer1 spacer2 wid ht1 dc each |
	hand _ self primaryHand.
	("hand lastEvent redButtonPressed &" hand hasSubmorphs
		and: [(self hasOwner: hand) not]) ifFalse: [^ self].

	insertion _ hand firstSubmorph renderedMorph.
	insertion isSyntaxMorph ifFalse: [^ self].
	insertion isNoun ifFalse: [(insertion nodeClassIs: CommentNode) ifFalse: [^ self]].
	localPt _ self globalPointToLocal: hand position.
	insHt _ insertion height.  "**just use standard line height here"
	self removeDropZones.  "Maybe first check if in right place, then just tweak heights."
	i _ (ii _ self indexOfMorphAbove: localPt) min: submorphs size-1.
	prevBot _ i <= 0 ifTrue: [(self innerBounds) top]
					ifFalse: [(self submorphs at: i) bottom].
	nxtHt _ (submorphs isEmpty
		ifTrue: [insertion]
		ifFalse: [self submorphs at: i+1]) height.
	d _ ii > i ifTrue: [nxtHt "for consistent behavior at bottom"]
			ifFalse: [0 max: (localPt y - prevBot min: nxtHt)].

	"Top and bottom spacer heights cause continuous motion..."
	c1 _ Color transparent.  c2 _ Color transparent.
	ht2 _ d*insHt//nxtHt.  ht1 _ insHt - ht2.
	wid _ self width - (2*borderWidth) - (2*self layoutInset).
	wid isPoint ifTrue: [wid _ wid x].
	(spacer1 _ BorderedMorph newBounds: (0@0 extent: wid@ht1)
				color: (ht1 > (insHt//2) ifTrue: [c1] ifFalse: [c2]))
					borderWidth: 1; borderColor: spacer1 color.
	self privateAddMorph: spacer1 atIndex: (i+1 max: 1).
	(spacer2 _ BorderedMorph newBounds: (0@0 extent: wid@ht2)
				color: (ht2 > (insHt//2+1) ifTrue: [c1] ifFalse: [c2]))
					borderWidth: 1; borderColor: spacer2 color.
	spacer1 setProperty: #dropZone toValue: true.
	spacer2 setProperty: #dropZone toValue: true.
	self privateAddMorph: spacer2 atIndex: (i+3 min: submorphs size+1).
	self fullBounds.  "Force layout prior to testing for cursor containment"

	"Maintain the drop target highlight -- highlight spacer if hand is in it."
	{spacer1. spacer2} do:
		[:spacer | (spacer containsPoint: localPt) ifTrue:
			[spacer color: self dropColor.
			"Ignore border color.  Maybe do it later.
			self borderColor = self dropColor
				ifTrue: [self borderColor: self stdBorderColor]"]].
	"If no submorph (incl spacers) highlighted, then re-highlight the block."
	"Ignore border color.  Maybe do it later.
	((self wantsDroppedMorph: insertion event: hand lastEvent) and:
		[(self submorphs anySatisfy: [:m | m containsPoint: localPt]) not])
			ifTrue: [self borderColor: self dropColor].
	"

	"Dragging a tile within a Block, if beside a tile, color it a dropzone"
	"Transcript show: localPt y printString; space; show: submorphs first top 
		printString; space; show: submorphs last top printString; cr."
	dc _ self dropColor.
	1 to: ((ii+4 min: submorphs size) max: 1) do: [:ind | 
		each _ submorphs at: ind.
		each isSyntaxMorph ifTrue: [
			localPt y >= each top 
				ifTrue: ["in this one or beyond"
					(localPt y < each bottom) 
						ifTrue: [(each submorphs anySatisfy: [:m | 
								m containsPoint: localPt])
							ifTrue: [each setDeselectedColor]
							ifFalse: [each color: dc]]
						ifFalse: [each color = dc ifTrue: [each setDeselectedColor]]]
				ifFalse: [each color = dc ifTrue: [each setDeselectedColor]]]].
! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 13:45'!
translateFromWordySelfVariant: key

	#selfWrittenAsMe == key ifTrue: [^'me'].
	#selfWrittenAsMy == key ifTrue: [^'my'].
	#selfWrittenAsIll == key ifTrue: [^'I''ll'].
	#selfWrittenAsIm == key ifTrue: [^'I''m'].
	#selfWrittenAsThis == key ifTrue: [^'this'].
	^nil

! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'tk 7/28/2001 09:05'!
translateToWordyGetter: key
	"  setBlob:  becomes  's blob _  "

	^ '''s ', 
	  (self splitAtCapsAndDownshifted: (key asString allButFirst: 3) 
			withFirstCharacterDownshifted)! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'RAA 2/26/2001 13:45'!
translateToWordySelfVariant: aString

	| lc |
	lc _ aString asLowercase.
	lc = 'me' ifTrue: [^#selfWrittenAsMe].
	lc = 'my' ifTrue: [^#selfWrittenAsMy].
	lc = 'i''ll' ifTrue: [^#selfWrittenAsIll].
	lc = 'i''m' ifTrue: [^#selfWrittenAsIm].
	lc = 'this' ifTrue: [^#selfWrittenAsThis].
	^nil

! !

!SyntaxMorph methodsFor: 'alans styles' stamp: 'tk 7/27/2001 17:26'!
translateToWordySetter: key
	"  setBlob:  becomes  's blob _  "

	^ '''s ', 
	  (self splitAtCapsAndDownshifted: (key asString allButFirst: 3) allButLast 
			withFirstCharacterDownshifted), 
	  ' _'! !

!SyntaxMorph methodsFor: 'layout' stamp: 'tk 2/12/2001 14:39'!
try
	"Evaluate me once"

	(#(MessageNode LiteralNode VariableNode) includes: parseNode class name) 
		ifFalse: [^ Error new].
	^ [Compiler evaluate: self decompile
				for: self actualObject
				logged: false.	"should do something to the player"
		] ifError: [ :a :b | Error new].! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'aoy 2/15/2003 21:31'!
unSpaceAndUpShift: aString appending: extraChars 
	| answer upShiftNext |
	answer := WriteStream on: String new.
	upShiftNext := false.
	aString do: 
			[:ch | 
			upShiftNext :=( ch == Character space) 
				ifTrue: [ true]
				ifFalse: 
					[answer nextPut: (upShiftNext ifTrue: [ch asUppercase] ifFalse: [ch]).
					 false]].
	answer := answer contents.
	extraChars isEmptyOrNil ifTrue: [^answer].
	(answer endsWith: extraChars) ifFalse: [answer := answer , extraChars].
	^answer! !

!SyntaxMorph methodsFor: 'layout' stamp: 'ar 8/10/2003 18:19'!
unfoldMessage
	"I am a message whose message part is a column.
	Rearrange me so that the entire message is one row."
	| messageRow |
	messageRow _ self submorphs last.
	self removeMorph: messageRow.
	messageRow submorphs do: [:m | self addMorphBack: m].

! !

!SyntaxMorph methodsFor: 'highlighting' stamp: 'tk 7/19/2001 19:50'!
unhighlight

	self setDeselectedColor.


false ifTrue: [
	self currentSelectionDo: [:innerMorph :mouseDownLoc :outerMorph |
		self color: ( false
			"(self == outerMorph or: [owner notNil and: [owner isSyntaxMorph not]])"
				ifTrue: [self valueOfProperty: #deselectedBorderColor ifAbsent: [#raised]]
				ifFalse: [self color: Color transparent]
		)
	]].
! !

!SyntaxMorph methodsFor: 'highlighting' stamp: 'di 5/4/2001 13:21'!
unhighlightBorder

	self currentSelectionDo: [:innerMorph :mouseDownLoc :outerMorph |
		self borderColor: (
			(self == outerMorph or: [owner notNil and: [owner isSyntaxMorph not]])
				ifTrue: [self valueOfProperty: #deselectedBorderColor ifAbsent: [#raised]]
				ifFalse: [self stdBorderColor]
		)
	]
! !

!SyntaxMorph methodsFor: 'highlighting' stamp: 'dgd 2/22/2003 18:48'!
unhighlightOwner
	"Unhighlight my owner"

	(owner notNil and: [owner isSyntaxMorph]) ifTrue: [owner unhighlight]! !

!SyntaxMorph methodsFor: 'highlighting' stamp: 'dgd 2/22/2003 18:49'!
unhighlightOwnerBorder
	"Unhighlight my owner's border"

	(owner notNil and: [owner isSyntaxMorph]) 
		ifTrue: [owner unhighlightBorder]! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 8/24/2001 13:35'!
upDown: delta event: evt arrow: arrowMorph

	| st |
	st _ submorphs detect: [:mm | mm isKindOf: StringMorph] ifNone: [^ self].
	(self nodeClassIs: LiteralNode) ifTrue:
		[ "+/- 1"
		st contents: (self decompile asNumber + delta) printString.
		^ self acceptUnlogged].
	(self nodeClassIs: VariableNode) ifTrue:
		[ "true/false"
		st contents: (self decompile string = 'true') not printString.
		^ self acceptSilently ifFalse: [self changed].
			"maybe set parseNode's key"].

	(self upDownArithOp: delta) ifTrue: [^ self].	"+ - // *   < > <= =   beep:"

	(self upDownAssignment: delta) ifTrue: [^ self].
		"Handle assignment --  increaseBy:  <-   multiplyBy:"
! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 7/24/2001 17:54'!
upDownArithOp: delta
	"Change a + into a -.  Also do sounds (change the arg to the beep:)."

	| aList index st |
	st _ submorphs detect: [:mm | mm isKindOf: StringMorph] ifNone: [^ self].
	(self nodeClassIs: SelectorNode) ifTrue:
		[aList _ #(+ - * / // \\ min: max:).
		(index _ aList indexOf: self decompile asString) > 0 ifTrue:
			[self setSelector: (aList atWrap: index + delta) in: st.  ^ true].

		aList _ #(= ~= > >= isDivisibleBy: < <=).
		(index _ aList indexOf: self decompile asString) > 0 ifTrue:
			[self setSelector: (aList atWrap: index + delta) in: st.  ^ true].

		aList _ #(== ~~).
		(index _ aList indexOf: self decompile asString) > 0 ifTrue:
			[self setSelector: (aList atWrap: index + delta) in: st.  ^ true].

		'beep:' = self decompile asString ifTrue:
			["replace sound arg"
			self changeSound: delta.
			self acceptSilently.  ^ true].
		].
	^ false! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 9/20/2001 16:33'!
upDownArrows
	"Return an array of two up/down arrow buttons.
	It replaces the selector or arg with a new one.
	I am a number or boolean or a selector (beep:, +,-,*,//,\\, or setX: incX: decX: for any X."
	| patch any noMenu |
	any _ (self nodeClassIs: LiteralNode) and: [parseNode key isNumber].
	any _ any or: [(self nodeClassIs: VariableNode) and:
						[(#('true' 'false') includes: self decompile asString)]].
	noMenu _ any.

	any _ any or: [self nodeClassIs: SelectorNode].	"arrows and menu of selectors"
	any _ any or: [self nodeClassIs: KeyWordNode].
	any ifFalse: [^ nil].

	patch _ {(ImageMorph new image: TileMorph upPicture)
				on: #mouseDown send: #upDown:event:arrow: to: self withValue: 1;
				on: #mouseStillDown send: #upDownMore:event:arrow: 
					to: self withValue: 1;
				on: #mouseUp send: #upDownDone to: self.
			(ImageMorph new image: TileMorph downPicture)
				on: #mouseDown send: #upDown:event:arrow: to: self withValue: -1;
				on: #mouseStillDown send: #upDownMore:event:arrow: 
					to: self withValue: -1;
				on: #mouseUp send: #upDownDone to: self}.
	noMenu ifFalse: [patch _ patch, {(RectangleMorph new)
						extent: 6@10; borderWidth: 1;
						borderColor: Color gray;
						on: #mouseUp send: #selectorMenu to: self}.
					patch last color: ((self nodeClassIs: SelectorNode) 
						ifTrue: [Color lightGreen] ifFalse: [Color red darker])].
	^ patch! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'tk 8/24/2001 12:33'!
upDownAssignment: delta
	"Rotate between increaseBy:  decreaseBy:   _  multiplyBy:"

	| st now want instVar |
	st _ submorphs detect: [:mm | mm isKindOf: StringMorph] ifNone: [^ self].
	(self nodeClassIs: SelectorNode) ifTrue:
		["kinds of assignment"
		((now _ self decompile asString) beginsWith: 'set') ifTrue:
			["a setX: 3"
			want _ 1+delta.  instVar _ (now allButFirst: 3) allButLast].
		(now endsWith: 'IncreaseBy:') ifTrue:
			["a xIncreaseBy: 3 a setX: (a getX +3)."
			want _ 2+delta.  instVar _ now allButLast: 11].
		(now endsWith: 'DecreaseBy:') ifTrue:
			["a xDecreaseBy: 3 a setX: (a getX -3)."
			want _ 3+delta.  instVar _ now allButLast: 11].
		(now endsWith: 'MultiplyBy:') ifTrue:
			["a xMultiplyBy: 3 a setX: (a getX *3)."
			want _ 4+delta.  instVar _ now allButLast: 11].
		want ifNil: [^ false].
		instVar _ instVar asLowercase.
		want _ #(1 2 3 4) atWrap: want.
		want = 1 ifTrue:
			["setter method is present"
			self setSelector: ('set', instVar capitalized, ':') in: st.  ^ true].
		want = 2 ifTrue:
			["notUnderstood will create the method if needed"
			self setSelector: instVar, 'IncreaseBy:' in: st.  ^ true].
		want = 3 ifTrue:
			["notUnderstood will create the method if needed"
			self setSelector: instVar, 'DecreaseBy:' in: st.  ^ true].
		want = 4 ifTrue:
			["notUnderstood will create the method if needed"
			self setSelector: instVar, 'MultiplyBy:' in: st.  ^ true].
		].
	^ false
! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'di 5/4/2001 11:49'!
upDownDone

	(self nodeClassIs: LiteralNode) ifTrue:
		[self acceptSilently.  "Final compilation logs source"
		self removeProperty: #timeOfLastTick;
			removeProperty: #currentDelay].
! !

!SyntaxMorph methodsFor: 'pop ups' stamp: 'di 5/4/2001 12:26'!
upDownMore: delta event: evt arrow: arrowMorph

	| st delay1 delay2 now timeOfLastTick currentDelay |
	(self nodeClassIs: LiteralNode) ifFalse: [^ self].
	st _ submorphs detect: [:mm | mm isKindOf: StringMorph] ifNone: [^ self].
	delay1 _ 300.  "ms"
	delay2 _ 50.  "ms"
	now _ Time millisecondClockValue.
	timeOfLastTick _ (self valueOfProperty: #timeOfLastTick) ifNil: [now - delay1].
	currentDelay _ (self valueOfProperty: #currentDelay) ifNil: [delay1].
	now >= (timeOfLastTick + currentDelay) ifTrue:
		[self setProperty: #timeOfLastTick toValue: now.
		"decrease the delay"
		self setProperty: #currentDelay toValue: (currentDelay*8//10 max: delay2).
		st contents: (self decompile asNumber + delta) printString.
		^ self acceptUnlogged].
! !

!SyntaxMorph methodsFor: 'updating' stamp: 'di 11/13/2000 20:30'!
update: aSymbol

	| bingo saveOwner newMorph db |

	(db _ self debugger) ifNil: [^super update: aSymbol].
	aSymbol == #contents ifTrue: [
		saveOwner _ owner.
		db removeDependent: self.
		markerMorph ifNotNil: [markerMorph delete. markerMorph _ nil].
		newMorph _ db createSyntaxMorph.
		self delete.
		saveOwner addMorph: newMorph.
		saveOwner owner setScrollDeltas.
		newMorph update: #contentsSelection.
	].
	aSymbol == #contentsSelection ifTrue: [
		markerMorph ifNil: [
			markerMorph _ RectangleMorph new.
			markerMorph
				color: Color transparent;
				borderWidth: 2;
				borderColor: Color red;
				lock.
			owner addMorphFront: markerMorph.
		].
		bingo _ parseNode rawSourceRanges keyAtValue: db pcRange ifAbsent: [nil].
		self testForNode: bingo andDo: [ :foundMorph | 
			markerMorph
				position: foundMorph position;
				extent: foundMorph extent.
			owner owner scrollIntoView: foundMorph bounds extra: 0.5.
			^self
		].
	].
	super update: aSymbol! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 11/12/2000 14:42'!
userScriptSelector
	"user wrote this script"

	^ self valueOfProperty: #userScriptSelector! !

!SyntaxMorph methodsFor: 'accessing' stamp: 'tk 11/12/2000 14:41'!
userScriptSelector: sel
	"user wrote this script"

	self setProperty: #userScriptSelector toValue: sel.! !

!SyntaxMorph methodsFor: 'formatting options' stamp: 'tk 7/18/2001 16:00'!
usingClassicTiles 

	^ Preferences uniTilesClassic! !

!SyntaxMorph methodsFor: 'node to morph' stamp: 'RAA 2/15/2001 19:49'!
vanillaMessageNode: aNode receiver: receiver selector: selector arguments: arguments

	| substitute row sel |
	sel _ #message.
	((self nodeClassIs: CascadeNode) and: [self parseNode receiver ~~ aNode]) ifTrue: [
		sel _ #keyword2.
		receiver ifNotNil: [self inform: 'receiver should be nil']].
	row _ self addRow: sel on: aNode.
	substitute _ aNode as: TileMessageNode.
	(aNode macroPrinter == #printCaseOn:indent:) ifTrue: [
		aNode asMorphicCaseOn: row indent: nil.
		^ self].
	aNode macroPrinter
		ifNotNil: 
			[substitute perform: aNode macroPrinter with: row with: nil]
		ifNil: 
			[substitute 
				printKeywords: selector key
				arguments: arguments
				on: row
				indent: nil].
	^ row addTransparentSpacerOfSize: 3@0.	"horizontal spacing only"
! !

!SyntaxMorph methodsFor: 'player' stamp: 'tk 2/15/2002 13:03'!
variableDocks
	"Answer a list of VariableDock objects for docking up my data with an instance held in my containing playfield.  For a numeric-readout tile."

	"Is CardPlayer class holding my variableDock, or should I be using the caching mechanism in Morph>>variableDocks?"
	| updatingString lab nn aGetter |
	(updatingString _ self readOut) ifNil: [^ #()].
	updatingString getSelector ifNil: [
		lab _ self submorphNamed: 'label' ifNone: [self defaultName].
		nn _ lab contents asString.
		"nn at: 1 put: nn first asUppercase."
		updatingString getSelector: (aGetter _ 'get',nn) asSymbol;
			putSelector: (ScriptingSystem setterSelectorForGetter: aGetter).
		].
	^ Array with: (VariableDock new 
			variableName: (updatingString getSelector allButFirst: 3) withFirstCharacterDownshifted 
			type: #number 
			definingMorph: updatingString 
			morphGetSelector: #valueFromContents 
			morphPutSelector: #acceptValue:)! !

!SyntaxMorph methodsFor: 'vocabulary' stamp: 'gm 2/22/2003 12:48'!
vocabularyToUseWith: aValue 
	"Answer a vocabulary to use with the given value"

	(aValue isNumber) ifTrue: [^Vocabulary numberVocabulary].
	(aValue isKindOf: Time) ifTrue: [^Vocabulary vocabularyForClass: Time].
	(aValue isString) ifTrue: [^Vocabulary vocabularyForClass: String].
	aValue class isUniClass ifTrue: [^Vocabulary eToyVocabulary].
	^self currentVocabulary! !

!SyntaxMorph methodsFor: 'dropping/grabbing' stamp: 'gm 2/22/2003 12:49'!
wantsDroppedMorph: aMorph event: evt 
	"For the moment, you have to drop it the right place.  We do not look at enclosing morphs"

	"Two ways to do this:  Must always destroy old node, then drag in new one.
		Or, drop replaces what you drop on.  Nasty with blocks."

	(aMorph isSyntaxMorph) ifFalse: [^false].
	(self structureMatchWith: aMorph) ifFalse: [^false].	"gross structure"

	"Only look at types if NoviceMode -- building EToys"
	^self okToBeReplacedBy: aMorph	"test the types"

	"^ true"! !

!SyntaxMorph methodsFor: 'event handling' stamp: 'tk 12/1/2000 15:42'!
wantsKeyboardFocusFor: aSubmorph
	| doEdit |
	"only let strings edit on shift-click.  Editing on ordinary click defeats the brown selection and tile dragging."

	doEdit _ self world primaryHand lastEvent shiftPressed.
	doEdit ifTrue: ["remove the arrows during editing"
		self valueOfProperty: #myPopup ifPresentDo: [:panel |
			panel delete. self removeProperty: #myPopup]].
	^ doEdit! !

!SyntaxMorph methodsFor: 'testing' stamp: 'di 1/30/2001 11:22'!
wantsSteps
	"Only step this morph if we explicitly send startStepping"

	^ false! !

AlignmentMorph subclass: #TileLikeMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

TileLikeMorph subclass: #CommandTilesMorph
	instanceVariableNames: 'morph playerScripted'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!CommandTilesMorph commentStamp: '<historical>' prior: 0!
An entire Smalltalk statement in tiles.  A line of code.!

!CommandTilesMorph methodsFor: 'initialization' stamp: 'ar 11/9/2000 21:13'!
initialize

	super initialize.
	self wrapCentering: #center; cellPositioning: #leftCenter.
	self hResizing: #shrinkWrap.
	borderWidth _ 0.
	self layoutInset: 0.
	self extent: 5@5.  "will grow to fit"
! !

!CommandTilesMorph methodsFor: 'initialization' stamp: 'sw 1/29/98 18:32'!
setMorph: aMorph
	playerScripted _ aMorph playerScripted
! !

!CommandTilesMorph methodsFor: 'miscellaneous' stamp: ''!
tileRows

	^ Array with: self submorphs! !

TileLikeMorph subclass: #CompoundTileMorph
	instanceVariableNames: 'type testPart yesPart noPart'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!CompoundTileMorph commentStamp: '<historical>' prior: 0!
A statement with other whole statements inside it.  If-Then.  Test.!

!CompoundTileMorph classSide methodsFor: 'new-morph participation' stamp: 'di 6/22/97 09:07'!
includeInNewMorphMenu
	"Not to be instantiated from the menu"
	^ false! !

!CompoundTileMorph methodsFor: 'layout' stamp: ''!
acceptDroppingMorph: aMorph event: evt
	"Forward the dropped morph to the appropriate part."

	(self targetPartFor: aMorph) acceptDroppingMorph: aMorph event: evt.
! !

!CompoundTileMorph methodsFor: 'access' stamp: 'sw 8/11/1998 16:42'!
associatedPlayer
	^ nil! !

!CompoundTileMorph methodsFor: 'code generation' stamp: 'sw 9/2/1999 15:22'!
codeString
	^ String streamContents: [:aStream | self storeCodeOn: aStream indent: 1]
! !

!CompoundTileMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:21'!
defaultBorderWidth
"answer the default border width for the receiver"
	^ 1! !

!CompoundTileMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:21'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color orange muchLighter! !

!CompoundTileMorph methodsFor: 'event handling' stamp: 'tk 2/28/2001 21:22'!
handlesMouseDown: evt
	^true! !

!CompoundTileMorph methodsFor: 'event handling' stamp: 'di 10/17/97 21:36'!
handlesMouseOver: evt

	^ true
! !

!CompoundTileMorph methodsFor: 'event handling' stamp: 'di 9/14/1998 07:50'!
handlesMouseOverDragging: evt

	^ true
! !

!CompoundTileMorph methodsFor: 'initialization' stamp: 'sw 7/22/2004 00:13'!
initialize
	"initialize the state of the receiver"

	| r stringMorph |
	super initialize.
	self layoutInset: 2.
	self listDirection: #topToBottom.
	self hResizing: #shrinkWrap; vResizing: #shrinkWrap; cellInset: (0 @ 1); minCellSize: (200@14).
	"NB: hResizing gets reset to #spaceFill below, after the standalone structure is created"
	r _ AlignmentMorph newRow color: color;
				 layoutInset: 0.
	r setProperty: #demandsBoolean toValue: true.
	r addMorphBack: (Morph new color: color;
			 extent: 2 @ 5).
	"spacer"
	stringMorph _ StringMorph new contents: 'Test' translated.
	stringMorph name: 'Test'.
	r addMorphBack: stringMorph.
	r addMorphBack: (Morph new color: color;
			 extent: 5 @ 5).
	"spacer"
	r addMorphBack: (testPart _ BooleanScriptEditor new borderWidth: 0;
					 layoutInset: 1).
	testPart color: Color transparent.
	testPart hResizing: #spaceFill.
	self addMorphBack: r.
	r _ AlignmentMorph newRow color: color;
				 layoutInset: 0.
	r addMorphBack: (Morph new color: color;
			 extent: 30 @ 5).
	"spacer"
	stringMorph _ StringMorph new contents: 'Yes' translated.
	stringMorph name: 'Yes'.
	r addMorphBack: stringMorph.
	r addMorphBack: (Morph new color: color;
			 extent: 5 @ 5).
	"spacer"
	r addMorphBack: (yesPart _ ScriptEditorMorph new borderWidth: 0;
					 layoutInset: 2).
	yesPart hResizing: #spaceFill.
	yesPart color: Color transparent.
	self addMorphBack: r.
	r _ AlignmentMorph newRow color: color;
				 layoutInset: 0.
	r addMorphBack: (Morph new color: color;
			 extent: 35 @ 5).
	"spacer"
	stringMorph _ StringMorph new contents: 'No' translated.
	stringMorph name: 'No'.
	r addMorphBack: stringMorph.
	r addMorphBack: (Morph new color: color;
			 extent: 5 @ 5).
	"spacer"
	r addMorphBack: (noPart _ ScriptEditorMorph new borderWidth: 0;
					 layoutInset: 2).
	noPart hResizing: #spaceFill.
	noPart color: Color transparent.
	self addMorphBack: r.
	self bounds: self fullBounds.
	self updateWordingToMatchVocabulary.
 	self hResizing:#spaceFill
! !

!CompoundTileMorph methodsFor: 'miscellaneous' stamp: 'sw 10/18/97 18:03'!
install
	"Backstop for obscure cases"! !

!CompoundTileMorph methodsFor: 'e-toy support' stamp: 'ar 2/7/2001 17:57'!
isTileEditor
	"Yes I am"
	^true! !

!CompoundTileMorph methodsFor: 'testing' stamp: 'yo 11/4/2002 20:33'!
isTileScriptingElement

	^ true
! !

!CompoundTileMorph methodsFor: 'event handling' stamp: 'tk 2/28/2001 21:25'!
mouseDown: evt 
	"Pretend we picked up the tile and then put it down for a trial  
	positioning."
	"The essence of ScriptEditor mouseEnter:"
	| ed ss guyToTake |
"	self isPartsDonor ifTrue:[
		dup _ self duplicate.
		evt hand attachMorph: dup.
		dup position: evt position.
		^self].
	submorphs isEmpty 			never true
		ifTrue: [^ self].
"
	(ed _ self enclosingEditor) ifNil: [^evt hand grabMorph: self].

	guyToTake _ self.
	owner class == TilePadMorph
		ifTrue: ["picking me out of another phrase"
			(ss _ submorphs first) class == TilePadMorph
				ifTrue: [ss _ ss submorphs first].
			guyToTake _  ss veryDeepCopy].
	evt hand grabMorph: guyToTake.
	ed startStepping.
	ed mouseEnterDragging: evt.
	ed setProperty: #justPickedUpPhrase toValue: true.
! !

!CompoundTileMorph methodsFor: 'event handling' stamp: 'jm 10/18/97 21:03'!
mouseEnter: evt
	"Resume drop-tracking in enclosing editor"
	| ed |
	(ed _ self enclosingEditor) ifNotNil:
		[ed mouseLeave: evt]! !

!CompoundTileMorph methodsFor: 'event handling' stamp: 'di 9/14/1998 08:07'!
mouseEnterDragging: evt
	"Test button state elsewhere if at all"
	^ self mouseEnter: evt! !

!CompoundTileMorph methodsFor: 'event handling' stamp: 'jm 10/18/97 21:02'!
mouseLeave: evt
	"Resume drop-tracking in enclosing editor"
	| ed |
	(ed _ self enclosingEditor) ifNotNil:
		[ed mouseEnter: evt]! !

!CompoundTileMorph methodsFor: 'event handling' stamp: 'di 9/14/1998 08:08'!
mouseLeaveDragging: evt
	"Test button state elsewhere if at all"
	^ self mouseLeave: evt! !

!CompoundTileMorph methodsFor: 'access' stamp: 'nk 10/14/2004 11:37'!
myMorph
	^nil! !

!CompoundTileMorph methodsFor: 'mouse' stamp: 'sw 2/1/98 16:40'!
prepareToUndoDropOf: aMorph
	"needs to be here, as a no-op, owing to being hit obscurely on occasion"! !

!CompoundTileMorph methodsFor: 'miscellaneous' stamp: 'ar 2/6/2001 22:07'!
recompileScript
	"Pertains only when the test is outside a script?!!"
! !

!CompoundTileMorph methodsFor: 'miscellaneous' stamp: 'sw 9/27/2001 17:27'!
resultType
	"Answer the result type of the receiver"

	^ #Command! !

!CompoundTileMorph methodsFor: 'miscellaneous' stamp: 'sw 5/13/1998 15:19'!
rowOfRightTypeFor: aLayoutMorph forActor: anActor
	aLayoutMorph demandsBoolean ifTrue:
		[^ self error: 'oops, cannot do that, please close this'].
	^ self! !

!CompoundTileMorph methodsFor: 'miscellaneous' stamp: 'sw 10/13/97 21:23'!
scriptEdited
	 "Pertains only when the test is outside a script?!!"! !

!CompoundTileMorph methodsFor: 'access' stamp: 'sw 10/13/97 21:23'!
scriptee
	 "Pertains only when the test is outside a script?!!"
	^ nil! !

!CompoundTileMorph methodsFor: 'code generation' stamp: 'jm 5/29/1998 10:26'!
storeCodeBlockFor: scriptPart on: aStream indent: tabCount

	| rows r |
	rows _ scriptPart tileRows.
	1 to: rows size do: [:i |
		tabCount timesRepeat: [aStream tab].
		r _ rows at: i.
		r do: [:t | t storeCodeOn: aStream indent: tabCount].
		i < rows size ifTrue: [aStream nextPut: $.; cr]].
! !

!CompoundTileMorph methodsFor: 'code generation' stamp: 'jm 5/29/1998 10:31'!
storeCodeOn: aStream indent: tabCount

	aStream nextPut: $(.
	testPart storeCodeOn: aStream indent: 0.
	aStream nextPut: $); cr.

	tabCount + 1 timesRepeat: [aStream tab].
	aStream nextPutAll: 'ifTrue: ['; cr.
	self storeCodeBlockFor: yesPart on: aStream indent: tabCount + 2.
	aStream nextPut: $]; cr.
	tabCount + 1 timesRepeat: [aStream tab].
	aStream nextPutAll: 'ifFalse: ['; cr.
	self storeCodeBlockFor: noPart on: aStream indent: tabCount + 2.
	aStream nextPut: $].
! !

!CompoundTileMorph methodsFor: 'mouse' stamp: ''!
targetPartFor: aMorph
	"Return the row into which the given morph should be inserted."

	| centerY |
	centerY _ aMorph fullBounds center y.
	(Array with: testPart with: yesPart with: noPart) do: [:m |
		(centerY <= m bounds bottom) ifTrue: [^ m]].
	^ noPart
! !

!CompoundTileMorph methodsFor: 'miscellaneous' stamp: 'di 5/6/1998 21:10'!
tile: tile isOnLineAfter: previousTile
	"Return true if the given tile is not on the same line at the previous tile or if the previous tile is nil."

	| tileRow previousRow |
	previousTile ifNil: [^ true].
	tileRow _ tile owner.
	[tileRow isMemberOf: AlignmentMorph]
		whileFalse: [tileRow _ tileRow owner].  "find the owning row"
	previousRow _ previousTile owner.
	[previousRow isMemberOf: AlignmentMorph]
		whileFalse: [previousRow _ previousRow owner].  "find the owning row"
	^ tileRow ~~ previousRow
! !

!CompoundTileMorph methodsFor: 'miscellaneous' stamp: 'tk 2/15/2001 16:36'!
tileRows
	"Answer a list of tile rows, in this case just one though it's compound"

	^ Array with: (Array with: self veryDeepCopy)! !

!CompoundTileMorph methodsFor: 'miscellaneous' stamp: ''!
type

	^ #compound
! !

!CompoundTileMorph methodsFor: 'initialization' stamp: 'nk 10/8/2004 11:56'!
updateWordingToMatchVocabulary
	| labels |
	labels _ OrderedCollection new.
	self submorphs do: [:submorph |
		submorph submorphs do: [:subsubmorph |
			subsubmorph class == StringMorph ifTrue: [labels add: subsubmorph]]].
	labels do: [:label | label knownName ifNotNilDo: [ :nm | label acceptValue: nm translated ]]
! !

!CompoundTileMorph methodsFor: 'dropping/grabbing' stamp: 'sw 12/13/2001 16:42'!
wantsDroppedMorph: aMorph event: evt
	"Removing this method entirely would be okay someday"

	^ false
"
	^ (aMorph isKindOf: TileMorph) or:
	   [(aMorph isKindOf: ScriptEditorMorph) or:
	   [(aMorph isKindOf: CompoundTileMorph) or:
	   [aMorph isKindOf: CommandTilesMorph]]]"
! !

TileLikeMorph subclass: #PhraseTileMorph
	instanceVariableNames: 'resultType brightenedOnEnter userScriptSelector justGrabbedFromViewer vocabulary vocabularySymbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!PhraseTileMorph commentStamp: '<historical>' prior: 0!
Phrase Tile World: A single smalltalk expression in tiles.  Like (car forwardBy: number), having 3 tiles.  

type = command
rcvrType = #actor


In the Old Single tile world:  Holder for a phrase of tiles as it came from the viewer and while it is being dragged by the hand.

 !

!PhraseTileMorph classSide methodsFor: 'scripting' stamp: 'dgd 8/26/2004 12:11'!
defaultNameStemForInstances
	^ 'PhraseTile'! !

!PhraseTileMorph classSide methodsFor: 'backward compatibility' stamp: 'sw 1/19/2001 22:25'!
markViewerOrigination
	"For bringing old content forward"

	| hadIt gotIt didntWantIt |
	hadIt _ 0.
	gotIt _ 0.
	didntWantIt _ 0.
	self allSubInstancesDo:
		[:m | (m ownerThatIsA: CategoryViewer)
			ifNil:
				[m justGrabbedFromViewer: false.
				didntWantIt _ didntWantIt + 1]
			ifNotNil:
				[(m justGrabbedFromViewerOrNil == true)
					ifTrue:
						[hadIt _ hadIt + 1]
					ifFalse:
						[m justGrabbedFromViewer: true.
						gotIt _ gotIt + 1]]].
	Transcript cr; show: 'updating phrase tiles -- already ok: '; show: hadIt; show: '  marked as in-viewer: '; show: gotIt; show: '  marked as not-in-viewer: '; show: didntWantIt.

	"PhraseTileMorph markViewerOrigination"! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 8/11/1998 16:39'!
actualObject
	"Answer the player that's the object of my attention"

	^ self associatedPlayer! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'dgd 8/30/2003 21:56'!
addCustomMenuItems:  aMenu hand: aHand
	"Add additional items to the halo manu"

	super addCustomMenuItems: aMenu hand: aHand.
	aMenu add: 'Sprout a new scriptor around this phrase' translated target: self action: #sproutNewScriptor! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'tk 8/6/1999 13:59'!
associatedPlayer
	"Answer the player that's the object of my attention"

	| pp |
	pp _ self firstSubmorph.
	[pp isKindOf: PhraseTileMorph] whileTrue: [pp _ pp firstSubmorph].
	^ pp firstSubmorph actualObject! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 9/8/2000 16:42'!
bePossessive
	"No way for doubly possessive stuff to work at present, so we just catch and swallow this request here."

	self flag: #deferred! !

!PhraseTileMorph methodsFor: 'scripting' stamp: 'sw 4/22/1998 14:59'!
bringUpToDate
	"Nothing here to do in the current architecture"! !

!PhraseTileMorph methodsFor: 'mouse' stamp: 'tak 3/15/2005 11:40'!
catchDivideByZero: aStream indent: tabCount 
	"See if I am have divide as my operator. If so, insert a test in the argument to divide."

	| exp |
	submorphs second type = #operator ifFalse: [^false].	"not me"
	exp _ submorphs second operatorOrExpression.
	(#(/ // \\) includes: exp) ifFalse: [^false].	"not me"
	aStream space.
	aStream nextPutAll: '(self beNotZero: '.
	(submorphs third) storeCodeOn: aStream indent: tabCount.
	aStream nextPut: $).
	^true! !

!PhraseTileMorph methodsFor: 'code generation' stamp: 'sw 9/2/1999 15:33'!
codeString
	^ String streamContents: [:aStream | self storeCodeOn: aStream indent: 1]
! !

!PhraseTileMorph methodsFor: 'macpal' stamp: 'sw 6/4/2001 19:35'!
currentVocabulary
	"Answer the current vocabulary"

	vocabulary "fix up old strutures"
		ifNotNil: 
			[vocabularySymbol _ vocabulary vocabularyName.
			vocabulary _ nil].

	^ vocabularySymbol
		ifNotNil:
			[Vocabulary vocabularyNamed: vocabularySymbol]
		ifNil:
			[super currentVocabulary]
! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 12/22/2004 01:34'!
dismissViaHalo
	"The user has clicked in the delete halo-handle.."

	| ed |
	ed _ self topEditor.
	super dismissViaHalo.
	ed ifNotNil: [ed scriptEdited]! !

!PhraseTileMorph methodsFor: 'kedama' stamp: 'sw 10/24/2005 02:48'!
duplicate
	"Make and return a duplicate of the receiver."

	| newMorph |
	newMorph _ super duplicate.
	(self ownerThatIsA: Viewer) ifNotNilDo:
		[:aViewer | newMorph replacePlayerInReadoutWith: aViewer scriptedPlayer].

	^ newMorph! !

!PhraseTileMorph methodsFor: 'kedama' stamp: 'yo 4/6/2005 14:59'!
firstMorphBearingKedamaPlayer

	self allMorphsWithPlayersDo: [:e :p | (p isKindOf: KedamaExamplerPlayer) ifTrue: [^ e]].
	^ nil.
! !

!PhraseTileMorph methodsFor: 'event handling' stamp: 'ar 10/6/2000 23:38'!
handlesMouseDown: evt
	^true! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'sw 8/28/2004 14:23'!
initialize
	"Initialize a nascent instance"

	super initialize.
	resultType _ #unknown.
	brightenedOnEnter _ false.
	self wrapCentering: #center; cellPositioning: #leftCenter.
	self hResizing: #shrinkWrap.
	borderWidth _ 0.
	self layoutInset: 0.
	self extent: 5@5.  "will grow to fit"
	self minCellSize: (0 @ (Preferences standardEToysFont height rounded + 10)).
	justGrabbedFromViewer _ true.  "All new PhraseTileMorphs that go through the initialize process (rather than being copied) are placed in viewers; the clones dragged out from them will thus have this set the right way; the drop code resets this to false"
! !

!PhraseTileMorph methodsFor: 'queries' stamp: 'sw 9/28/2001 07:46'!
isBoolean
	"Answer whether the receiver has a boolean type"

	^ self resultType = #Boolean! !

!PhraseTileMorph methodsFor: 'e-toy support' stamp: 'sw 7/28/1999 17:02'!
isCandidateForAutomaticViewing
	^ false! !

!PhraseTileMorph methodsFor: 'queries' stamp: 'nk 7/8/2000 13:35'!
isCommand
	"Answer whether the receiver is a true line of phrase-command. If not,  
	it is a fragment that will not be able to serve as a line of script on its  
	own"
	| rcvrTile pad |
	submorphs isEmpty
		ifTrue: [^ false].
	pad _ submorphs first.
	(pad isKindOf: TilePadMorph)
		ifTrue: [(submorphs second isKindOf: AssignmentTileMorph)
				ifTrue: [^ true].
			(((rcvrTile _ pad submorphs first) isKindOf: TileMorph)
					and: [rcvrTile isPossessive])
				ifTrue: [^ false]].
	^ true! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 3/7/2004 13:04'!
isPlayer: aPlayer ofReferencingTile: tile
	"Answer whether a given player is the object referred to by the given tile, or a sibling of that object."

	^ aPlayer class == self actualObject class! !

!PhraseTileMorph methodsFor: 'queries' stamp: 'di 8/11/1998 13:15'!
isPossessive
	^ false! !

!PhraseTileMorph methodsFor: 'scripting' stamp: 'sw 4/21/1998 21:35'!
isTileScriptingElement
	^ true! !

!PhraseTileMorph methodsFor: 'dropping/grabbing' stamp: 'sw 2/9/2001 00:15'!
justDroppedInto: newOwner event: evt
	"Phrase tiles only auto-expand if they originate from viewers.  Any phrase tile, once dropped, loses its auto-phrase-expansion thing"

	justGrabbedFromViewer _ false.
	super justDroppedInto: newOwner event: evt! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 1/19/2001 15:06'!
justGrabbedFromViewer
	"Answer whether the receiver originated in a Viewer.  Only tiles that originated in a viewer will ever do that infernal sprouting of a new script around them.  The nil branch is only for backward compatibility."

	^ justGrabbedFromViewer ifNil: [justGrabbedFromViewer _ true]! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 1/19/2001 14:25'!
justGrabbedFromViewer: aBoolean
	"Set the receiver's justGrabbedFromViewer instance variable"

	justGrabbedFromViewer _ aBoolean! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 1/19/2001 20:26'!
justGrabbedFromViewerOrNil
	"Answer the value of the receiver's justGrabbedFromViewer slot.  Needed only for conversion methods"

	^ justGrabbedFromViewer! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'tk 8/16/2000 10:01'!
lastTile
	"The tile that might get an extension arrow"

	^ self lastSubmorph lastTile! !

!PhraseTileMorph methodsFor: 'mouse' stamp: 'tak 4/6/2005 14:20'!
morphToDropInPasteUp: aPasteUp
	"Answer the morph to drop in aPasteUp, given that the receiver is the putative droppee"

	| actualObject itsSelector aScriptor pos aWatcher op |

	((actualObject _ self actualObject) isNil or: [actualObject costume isInWorld not]) ifTrue: [^ self].
	self isCommand ifFalse:  "Can't expand to a scriptor, but maybe launch a watcher..."
		[^ (Preferences dropProducesWatcher and: [(#(unknown command) includes: self resultType) not] and:
			[(op _ self operatorTile operatorOrExpression) notNil] and: [op numArgs = 0] and: [(Vocabulary gettersForbiddenFromWatchers includes: op) not])
			ifTrue:
				[aWatcher _ WatcherWrapper new fancyForPlayer: self associatedPlayer getter: op.
				aWatcher position: self position]
			ifFalse:
				[self]].

	self justGrabbedFromViewer ifFalse: [^ self].
	actualObject assureUniClass.
	itsSelector _ self userScriptSelector.
	pos _ self position.
	aScriptor _ itsSelector isEmptyOrNil
		ifFalse:
			[actualObject scriptEditorFor: itsSelector]
		ifTrue:
			["It's a system-defined selector; construct an anonymous scriptor around it"
			actualObject newScriptorAround: self].
	aScriptor ifNil:[^self].
	(self hasOwner: aScriptor) ifTrue:[
		aScriptor fullBounds. "force layout"
		aScriptor position: pos - self position.
	] ifFalse:[
		aScriptor position: self position.
	].
	^ aScriptor! !

!PhraseTileMorph methodsFor: 'kedama' stamp: 'yo 4/3/2005 17:36'!
morphsBearingPlayers

	| c |
	c _ OrderedCollection new.
	self allMorphsWithPlayersDo: [:e :p | c add: e].
	^ c asArray.
! !

!PhraseTileMorph methodsFor: 'mouse' stamp: 'sw 6/17/2003 16:03'!
mouseDown: evt 
	"Handle a mouse-down on the receiver"

	| ed guyToTake dup enclosingPhrase |
	self isPartsDonor ifTrue:
		[dup _ self duplicate.
		dup eventHandler: nil.   "Remove viewer-related evt mouseover feedback"
		evt hand attachMorph: dup.
		dup position: evt position.
		"So that the drag vs. click logic works"
		dup formerPosition: evt position.
		^ self].
	submorphs isEmpty
		ifTrue: [^ self].

	guyToTake _ self.
	[(enclosingPhrase _ guyToTake ownerThatIsA: PhraseTileMorph) notNil] whileTrue:
		[guyToTake _ enclosingPhrase].  "This logic always grabs the outermost phrase, for now anyway"
	
	"the below had comment: 'picking me out of another phrase'"
	"owner class == TilePadMorph
		ifTrue:
			[(ss _ submorphs first) class == TilePadMorph
				ifTrue: [ss _ ss submorphs first].
			guyToTake _  ss veryDeepCopy]."

	(ed _ self enclosingEditor) ifNil: [^ evt hand grabMorph: guyToTake].
	evt hand grabMorph: guyToTake.
	ed startStepping.
	ed mouseEnterDragging: evt.
	ed setProperty: #justPickedUpPhrase toValue: true.
! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 9/1/2000 08:41'!
operatorTile
	"Answer the submorph which comprises the operator tile of the receiver.  Ouch!!"

	^ submorphs second! !

!PhraseTileMorph methodsFor: 'kedama' stamp: 'yo 4/6/2005 14:59'!
replacePlayerInReadoutWith: aPlayer 

	| tile |
	tile _ self firstMorphBearingKedamaPlayer.
	tile ifNil: [^ self].
	(tile isMemberOf: TileMorph) ifFalse: [^ self].
	tile type = #objRef ifFalse: [^ self].
	tile referToSimilarObject: aPlayer.

	self allMorphsDo: [:e |
		((e isMemberOf: UpdatingStringMorph) or: [e isMemberOf: UpdatingRectangleMorph]) ifTrue: [
			e target isPlayerLike ifTrue: [
				e target: aPlayer
			].
		].
	].! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 9/27/2001 17:28'!
resultType
	"Answer the result type of the receiver"

	^ resultType! !

!PhraseTileMorph methodsFor: 'all' stamp: 'sw 12/13/2001 17:41'!
rowOfRightTypeFor: aLayoutMorph forActor: aPlayer
	"Answer a phrase of the right type for the putative container"

	| aTemporaryViewer aPhrase |
	aLayoutMorph demandsBoolean ifTrue:
		[self isBoolean ifTrue: [^ self].
		aTemporaryViewer _ CategoryViewer new invisiblySetPlayer: aPlayer.
		aPhrase _ aTemporaryViewer booleanPhraseFromPhrase: self.
		aPhrase justGrabbedFromViewer: false.
		^ aPhrase].
	^ self! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 6/18/2004 16:01'!
setAngleToOperator: opSymbol type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ KedamaAngleToTile new adoptVocabulary: self currentVocabulary.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'customevents-scripting' stamp: 'nk 11/1/2004 11:14'!
setAsActionInButtonProperties: buttonProperties

	userScriptSelector ifNil: [
		buttonProperties
			target: self associatedPlayer;
			actionSelector: #evaluateUnloggedForSelf:;
			arguments: {self codeString}.
		^true
	].
	buttonProperties
		target: self objectViewed player;
		actionSelector: #triggerScript: ;
		arguments: {userScriptSelector}.
	^true

"==== or 

	buttonProperties
		target: (self morphToDropInPasteUp: nil);
		actionSelector: #tryMe;
		arguments: #().
	^true


	==="! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'sw 10/29/1998 16:01'!
setAssignmentRoot: opSymbol type: opType rcvrType: rcvrType argType: argType
	resultType _ opType.
	self color: (ScriptingSystem colorForType: opType).
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	self addMorphBack: ((AssignmentTileMorph new setRoot: opSymbol asString dataType: argType) typeColor: color).
	self addMorphBack: (TilePadMorph new setType: argType)
! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'sw 8/12/2004 18:58'!
setAssignmentRoot: opSymbol type: opType rcvrType: rcvrType argType: argType vocabulary: aVocabulary
	"Add submorphs to make me constitute a setter of the given symbol"

	| anAssignmentTile |
	resultType _ opType.
	self color: (ScriptingSystem colorForType: opType).
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	anAssignmentTile _ AssignmentTileMorph new rawVocabulary: aVocabulary.
	self addMorphBack: (anAssignmentTile typeColor: color).
	anAssignmentTile setRoot: opSymbol asString dataType: argType.
	anAssignmentTile setAssignmentSuffix: #:.
	self addMorphBack: (TilePadMorph new setType: argType)! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 9/28/2004 18:24'!
setBounceOnColorOperator: opSymbol type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ KedamaBounceOnColorTile new adoptVocabulary: self currentVocabulary.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 9/27/2004 14:58'!
setBounceOnOperator: opSymbol type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ KedamaBounceOnTile new adoptVocabulary: self currentVocabulary.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 10/12/2004 22:43'!
setColorComponentRoot: opSymbol componentName: componentName type: opType rcvrType: rcvrType argType: argType vocabulary: aVocabulary
	"Add submorphs to make me constitute a setter of the given symbol"

	| anAssignmentTile |
	resultType _ opType.
	self color: (ScriptingSystem colorForType: opType).
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	anAssignmentTile _ KedamaSetColorComponentTile new rawVocabulary: aVocabulary.
	anAssignmentTile componentName: componentName.
	self addMorphBack: (anAssignmentTile typeColor: color).
	anAssignmentTile setRoot: opSymbol asString dataType: argType.
	self addMorphBack: (TilePadMorph new setType: argType)! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 6/18/2004 16:01'!
setDistanceToOperator: opSymbol type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ KedamaDistanceToTile new adoptVocabulary: self currentVocabulary.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 1/30/2005 11:02'!
setGetColorComponentOperator: opSymbol componentName: componentName type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ KedamaGetColorComponentTile new adoptVocabulary: self currentVocabulary.
	aTileMorph componentName: componentName.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 6/18/2004 16:50'!
setGetPixelOperator: opSymbol type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ KedamaGetPixelValueTile new adoptVocabulary: self currentVocabulary.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'sw 2/5/1999 14:51'!
setOperator: opSymbol type: opType rcvrType: rcvrType
	self setOperator: opSymbol type: opType rcvrType: rcvrType argType: nil.
! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'sw 9/26/2001 11:58'!
setOperator: opSymbol type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ TileMorph new adoptVocabulary: self currentVocabulary.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 6/18/2004 16:02'!
setPixelValueRoot: opSymbol type: opType rcvrType: rcvrType argType: argType vocabulary: aVocabulary
	"Add submorphs to make me constitute a setter of the given symbol"

	| anAssignmentTile |
	resultType _ opType.
	self color: (ScriptingSystem colorForType: opType).
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	anAssignmentTile _ KedamaSetPixelValueTile new rawVocabulary: aVocabulary.
	self addMorphBack: (anAssignmentTile typeColor: color).
	anAssignmentTile setRoot: opSymbol asString dataType: argType.
	self addMorphBack: (TilePadMorph new setType: argType)! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:41'!
setSlotRefOperator: opSymbol type: opType
	"Set the given symbol as the receiver's slot-reference operator, adding tiles to the receiver appropriately"

	resultType _ opType.
	self color: (ScriptingSystem colorForType: opType).
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: #Player).
	self addMorphBack: ((TileMorph new setSlotRefOperator: opSymbol asString) typeColor: color)
! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 10/9/2004 15:38'!
setTurtleAtOperator: opSymbol type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ KedamaTurtleAtTile new adoptVocabulary: self currentVocabulary.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 3/7/2005 08:39'!
setTurtleOfOperator: opSymbol type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ KedamaTurtleOfTile new adoptVocabulary: self currentVocabulary.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'yo 6/18/2004 16:02'!
setUpHillOperator: opSymbol type: opType rcvrType: rcvrType argType: argType
	"Set the operator, type, receiver type, and argument type for the phrase"

	| aTileMorph |

	resultType _ opType.
	opType ifNotNil: [self color: (ScriptingSystem colorForType: opType)].
	self removeAllMorphs.
	self addMorph: (TilePadMorph new setType: rcvrType).
	aTileMorph _ KedamaUpHillTile new adoptVocabulary: self currentVocabulary.
	self addMorphBack: ((aTileMorph setOperator: opSymbol asString) typeColor: color).
	opSymbol numArgs = 1 ifTrue:
		[self addMorphBack: (TilePadMorph new setType: (argType ifNil: [#Object]))]! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'tk 2/14/2001 14:07'!
sproutNewScriptor
	"The receiver, operating as a naked phrase tile, wishes to get iself placed in a nascent script"

	| newScriptor |

	self actualObject assureUniClass.
	newScriptor _ self actualObject newScriptorAround:
		((self ownerThatIsA: Viewer orA: ScriptEditorMorph)
			ifNotNil:
				[self veryDeepCopy]
			ifNil:
				[self]).
	self currentHand attachMorph: newScriptor! !

!PhraseTileMorph methodsFor: 'code generation' stamp: 'dgd 2/22/2003 19:08'!
storeCodeOn: aStream indent: tabCount 
	"Add in some smarts for division by zero."

	aStream nextPut: $(.
	submorphs first storeCodeOn: aStream indent: tabCount.
	aStream space.
	submorphs second storeCodeOn: aStream indent: tabCount.
	submorphs size > 2 
		ifTrue: 
			[(self catchDivideByZero: aStream indent: tabCount) 
				ifFalse: 
					[aStream space.
					(submorphs third) storeCodeOn: aStream indent: tabCount]].
	aStream nextPut: $)! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'tk 2/15/2001 16:37'!
tileRows
	"Answer a list of tile rows -- in this case exactly one row -- representing the receiver.  The fullCopy is deeply problematical here in the presence of the formerOwner property, so it the latter is temporarily set aside"

	^ Array with: (Array with: self veryDeepCopy)! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'yo 4/4/2005 17:15'!
try
	"Evaluate the given phrase once"

	| aPlayer |
	(userScriptSelector notNil and: [userScriptSelector numArgs = 0])
		ifTrue:
			[aPlayer _ self objectViewed player.
			aPlayer triggerScript: userScriptSelector]
		ifFalse:
			[Compiler evaluate:
				self codeString
				for: self associatedPlayer
				logged: false]! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 1/29/98 01:49'!
userScriptSelector
	^ userScriptSelector! !

!PhraseTileMorph methodsFor: 'miscellaneous' stamp: 'sw 1/29/98 01:49'!
userScriptSelector: s
	userScriptSelector _ s! !

!PhraseTileMorph methodsFor: 'initialization' stamp: 'gm 2/24/2003 18:06'!
vocabulary: aVocab 
	"Set the vocabulary"

	vocabularySymbol := (aVocab isKindOf: Symbol) 
				ifTrue: [aVocab]
				ifFalse: [aVocab vocabularyName]! !

PhraseTileMorph subclass: #SystemQueryPhrase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!SystemQueryPhrase methodsFor: 'miscellaneous' stamp: 'sw 8/3/1998 14:48'!
actualObject
	^ nil! !

!SystemQueryPhrase methodsFor: 'miscellaneous' stamp: 'sw 8/11/1998 16:42'!
associatedPlayer
	^ nil! !

!SystemQueryPhrase methodsFor: 'initialization' stamp: 'sw 9/26/2001 03:04'!
initialize
	"Initialize the receiver.  In this case we primarily seek to undo the damage done by inherited implementors of #initialize"

	super initialize.
	self removeAllMorphs.
	resultType _ #Boolean.
	self vResizing: #shrinkWrap! !

!SystemQueryPhrase methodsFor: 'access' stamp: 'sw 8/3/1998 14:50'!
isBoolean
	^ true! !

!SystemQueryPhrase methodsFor: 'code generation' stamp: 'sw 8/3/1998 14:51'!
storeCodeOn: aStream indent: tabCount
	submorphs first storeCodeOn: aStream indent: tabCount! !

!TileLikeMorph methodsFor: 'user interface' stamp: ''!
fixLayoutOfSubmorphsNotIn: aCollection 
	self minCellSize: 0 @ (Preferences standardEToysFont height rounded + 10).
	self
		allMorphsDo: [:m | (aCollection includes: m)
				ifFalse: [(m respondsTo: #fixLayoutOfSubmorphsNotIn:)
						ifTrue: [m ~~ self
								ifTrue: [m fixLayoutOfSubmorphsNotIn: aCollection]]
						ifFalse: [m layoutChanged].
					aCollection add: m]].
	self layoutChanged; fullBounds! !

!TileLikeMorph methodsFor: 'scripting' stamp: 'ar 10/7/2000 16:50'!
isTileLike
	^true! !

!TileLikeMorph methodsFor: 'dropping/grabbing' stamp: 'ar 2/9/2001 23:57'!
justGrabbedFrom: formerOwner
	| editor |
	formerOwner ifNil:[^self].
	editor _ formerOwner topEditor.
	editor ifNotNil:[editor scriptEdited].! !

!TileLikeMorph methodsFor: 'initialization' stamp: 'mir 7/15/2004 15:20'!
localeChanged
	"Update myself to reflect the change in locale"

	self updateWordingToMatchVocabulary! !

!TileLikeMorph methodsFor: 'user interface' stamp: 'sw 3/10/2004 19:55'!
updateWordingToMatchVocabulary
	"If appropriate, change the wording on the receiver to match up with a changed vocabulary."! !

AlignmentMorph subclass: #Viewer
	instanceVariableNames: 'scriptedPlayer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!Viewer commentStamp: 'sw 9/6/2002 13:14' prior: 0!
An abstract superclass for both CategoryViewer and StandardViewer.  A viewer is always associated with a particular 'scriptedPlayer' -- the object whose protocol it shows in tile form.!

Viewer subclass: #CategoryViewer
	instanceVariableNames: 'namePane chosenCategorySymbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!CategoryViewer commentStamp: '<historical>' prior: 0!
A viewer on an object.  Consists of three panes:
   Header pane -- category-name, arrows for moving among categories, etc.
   List pane -- contents are a list of subparts in the chosen category.
   Editing pane -- optional, a detail pane with info relating to the selected element of the list pane.!

!CategoryViewer methodsFor: 'header pane' stamp: 'sw 8/31/2004 14:01'!
addHeaderMorph
	"Add the header at the top of the viewer, with a control for choosing the category, etc."

	| header aButton |
	header _ AlignmentMorph newRow color: self color; wrapCentering: #center; cellPositioning: #leftCenter.
	aButton _ self tanOButton.
	header addMorph: aButton.
	aButton actionSelector: #delete;
		setBalloonText: 'remove this pane from the screen
don''t worry -- nothing will be lost!!.' translated.
	self maybeAddArrowsTo: header.
	header beSticky.
	self addMorph: header.
	self addNamePaneTo: header.
	chosenCategorySymbol _ #basic! !

!CategoryViewer methodsFor: 'entries' stamp: 'sw 7/25/2004 15:37'!
addIsOverColorDetailTo: aRow
	"Special-casee code for the boolean-valued phrase variously known as is-over-color or sees-color."

	| clrTile |
	aRow addMorphBack: (Morph new color: self color; extent: 2@10).  "spacer"
	aRow addMorphBack: (clrTile _ Color blue newTileMorphRepresentative).

"The following commented-out code put a readout up; the readout was very nice, but was very consumptive of cpu time, which is why the is-over-color tile got removed from the viewer long ago.  Now is-over-color is reinstated to the viewer, minus the expensive readout..."

"	aRow addMorphBack: (AlignmentMorph new beTransparent).
	readout _ UpdatingStringMorphWithArgument new
			target: scriptedPlayer; getSelector: #seesColor:; growable: false; putSelector: nil;
			argumentTarget: clrTile colorSwatch argumentGetSelector: #color.
	readout useDefaultFormat.
	aTile _ StringReadoutTile new typeColor: Color lightGray lighter.
	aTile addMorphBack: readout.
	aRow addMorphBack: aTile.
	aTile setLiteralTo: (scriptedPlayer seesColor: clrTile colorSwatch color) printString width: 30"! !

!CategoryViewer methodsFor: 'header pane' stamp: 'nk 7/12/2004 23:15'!
addNamePaneTo: header 
	"Add the namePane, which may be a popup or a type-in 
	depending on the type of CategoryViewer"
	| aButton |
	namePane := RectangleMorph newSticky color: Color brown veryMuchLighter.
	namePane borderWidth: 0.
	aButton := (StringButtonMorph
				contents: '-----'
				font: Preferences standardButtonFont)
				color: Color black.
	aButton target: self;
		 arguments: Array new;
		 actionSelector: #chooseCategory.
	aButton actWhen: #buttonDown.
	namePane addMorph: aButton.
	aButton position: namePane position.
	namePane align: namePane topLeft with: bounds topLeft + (50 @ 0).
	namePane setBalloonText: 'category (click here to choose a different one)' translated.
	header addMorphBack: namePane.
	(namePane isKindOf: RectangleMorph)
		ifTrue: [namePane addDropShadow.
			namePane shadowColor: Color gray]
! !

!CategoryViewer methodsFor: 'entries' stamp: 'md 11/14/2003 16:20'!
addOverlapsDetailTo: aRow
	"Disreputable magic: add necessary items to a viewer row abuilding for the overlaps phrase"

	aRow addMorphBack: (Morph new color: self color; extent: 2@10).  "spacer"
	aRow addMorphBack:  self tileForSelf.
	aRow addMorphBack: (AlignmentMorph new beTransparent).  "flexible spacer"

! !

!CategoryViewer methodsFor: 'entries' stamp: 'md 11/14/2003 16:21'!
addTouchesADetailTo: aRow
	| clrTile |
	aRow addMorphBack: (Morph new color: self color; extent: 2@10).  " spacer"
	aRow addMorphBack: (clrTile _ self tileForSelf).
	aRow addMorphBack: (AlignmentMorph new beTransparent).  "flexible spacer"

	"readout _ UpdatingStringMorphWithArgument new
			target: scriptedPlayer; getSelector: #seesColor:; growable: false; putSelector: nil;
			argumentTarget: clrTile colorSwatch argumentGetSelector: #color.
	readout useDefaultFormat.
	aTile _ StringReadoutTile new typeColor: Color lightGray lighter.
	aTile addMorphBack: readout.
	aRow addMorphBack: aTile.
	aTile setLiteralTo: (scriptedPlayer seesColor: clrTile colorSwatch color) printString width: 30"! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 7/7/2004 21:59'!
adjustColorsAndBordersWithin
	"Adjust the colors and borders of submorphs to suit current fashion"

	self allMorphsDo: [:aMorph | 
		(aMorph isKindOf: ViewerLine) ifTrue:
			[aMorph layoutInset: 1].
		(aMorph isKindOf: TilePadMorph) ifTrue:
			[aMorph beTransparent].
		(aMorph isKindOf: PhraseTileMorph) ifTrue:
			[aMorph beTransparent.
			aMorph borderWidth: 0].

		(aMorph isKindOf: TileMorph)
			ifTrue:
				[aMorph borderWidth: 1]].

	self borderWidth: 1! !

!CategoryViewer methodsFor: 'e-toy support' stamp: 'sw 9/13/2001 19:16'!
adoptVocabulary: aVocabulary
	"Adopt the given vocabulary as the one used in this viewer."

	| aCategory |
	chosenCategorySymbol ifNil: [^ self delete].
	aCategory _ aVocabulary categoryAt: chosenCategorySymbol.
	aCategory ifNil: [self delete] ifNotNil: [self updateCategoryNameTo: aCategory wording].
	super adoptVocabulary: aVocabulary! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'dgd 9/1/2003 13:51'!
arrowSetterButton: sel args: argArray

	| m |
	m _ RectangleMorph new
		color: (ScriptingSystem colorForType: #command);
		extent: 24@TileMorph defaultH;
		borderWidth: 0.
	m addMorphCentered: (ImageMorph new image: (ScriptingSystem formAtKey: 'Gets')).
	m setBalloonText: 'drag from here to obtain an assignment phrase.' translated.
	m on: #mouseDown send: sel
		to: self
		withValue: argArray.
	^ m
! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'dgd 9/1/2003 13:51'!
arrowSetterButtonFor: partName type: partType

	| m |
	m _ RectangleMorph new
		color: (ScriptingSystem colorForType: #command);
		extent: 24@TileMorph defaultH;
		borderWidth: 0.
	m addMorphCentered: (ImageMorph new image: (ScriptingSystem formAtKey: 'Gets')).
	m setBalloonText: 'drag from here to obtain an assignment phrase.' translated.
	m on: #mouseDown send: #makeSetter:event:from:
		to: self
		withValue: (Array with: partName with: partType).
	^ m
! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 8/22/2002 14:00'!
beReplacedByCategory: chosenCategory
	"Be replaced by a category pane pointed at the chosen category"

	self outerViewer replaceSubmorph: self by: (self outerViewer categoryViewerFor: chosenCategory)
	! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 9/28/2004 18:24'!
booleanPhraseForBounceOnColorOfType: retrieverType retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setBounceOnColorOperator: retrieverOp type: retrieverType rcvrType: #Player argType: nil.
	getterPhrase submorphs third delete.
	getterPhrase submorphs second setSlotRefOperator: retrieverOp.
	"getterPhrase submorphs second setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer)."
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	"self halt."
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 9/27/2004 15:00'!
booleanPhraseForBounceOnOfType: retrieverType retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setBounceOnOperator: retrieverOp type: retrieverType rcvrType: #Player argType: nil.
	getterPhrase submorphs third delete.
	getterPhrase submorphs second setSlotRefOperator: retrieverOp.
	"getterPhrase submorphs second setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer)."
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	"self halt."
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 6/17/2004 10:02'!
booleanPhraseForGetAngleToOfType: retrieverType retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setAngleToOperator: retrieverOp type: retrieverType rcvrType: #Player argType: nil.
	getterPhrase submorphs third delete.
	getterPhrase submorphs second setSlotRefOperator: retrieverOp.
	getterPhrase submorphs second setTurtleDefaultTo: scriptedPlayer.
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 10/13/2004 01:39'!
booleanPhraseForGetColorComponentOfType: retrieverType componentName: componentName retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setGetColorComponentOperator: retrieverOp componentName: componentName type: retrieverType rcvrType: #Player argType: nil.
	getterPhrase submorphs third delete.
	getterPhrase submorphs second setSlotRefOperator: retrieverOp.
	getterPhrase submorphs second setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer).
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	"self halt."
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 6/15/2004 18:45'!
booleanPhraseForGetDistanceToOfType: retrieverType retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setDistanceToOperator: retrieverOp type: retrieverType rcvrType: #Player argType: nil.
	getterPhrase submorphs third delete.
	getterPhrase submorphs second setSlotRefOperator: retrieverOp.
	getterPhrase submorphs second setTurtleDefaultTo: scriptedPlayer.
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	"self halt."
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 6/15/2004 16:38'!
booleanPhraseForGetPatchValueOfType: retrieverType retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setGetPixelOperator: retrieverOp type: retrieverType rcvrType: #Player argType: nil.
	getterPhrase submorphs third delete.
	getterPhrase submorphs second setSlotRefOperator: retrieverOp.
	getterPhrase submorphs second setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer).
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	"self halt."
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 10/9/2004 10:39'!
booleanPhraseForGetTurtleAtOfType: retrieverType retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setTurtleAtOperator: retrieverOp type: retrieverType rcvrType: #Player argType: nil.
	getterPhrase submorphs third delete.
	getterPhrase submorphs second setSlotRefOperator: retrieverOp.
	getterPhrase submorphs second setTurtleDefaultTo: scriptedPlayer.
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	"self halt."
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 3/7/2005 08:41'!
booleanPhraseForGetTurtleOfOfType: retrieverType retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setTurtleOfOperator: retrieverOp type: retrieverType rcvrType: #Player argType: nil.
	getterPhrase submorphs third delete.
	getterPhrase submorphs second setSlotRefOperator: retrieverOp.
	"getterPhrase submorphs second setTurtleDefaultTo: scriptedPlayer."
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	"self halt."
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 6/15/2004 16:39'!
booleanPhraseForGetUpHillOfType: retrieverType retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setUpHillOperator: retrieverOp type: retrieverType rcvrType: #Player argType: nil.
	getterPhrase submorphs third delete.
	getterPhrase submorphs second setSlotRefOperator: retrieverOp.
	getterPhrase submorphs second setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer).
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	"self halt."
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'sw 9/27/2001 13:28'!
booleanPhraseForRetrieverOfType: retrieverType retrieverOp: retrieverOp player: aPlayer
	"Answer a boolean-valued phrase derived from a retriever (e.g. 'car's heading'); this is in order to assure that tiles laid down in a TEST area will indeed produce a boolean result"

	| outerPhrase getterPhrase receiverTile  rel finalTile |
	rel _ (Vocabulary vocabularyForType:  retrieverType) comparatorForSampleBoolean.
	outerPhrase _ PhraseTileMorph new setOperator: rel type: #Boolean rcvrType: retrieverType argType: retrieverType.
	getterPhrase _  PhraseTileMorph new setOperator: retrieverOp type: retrieverType rcvrType: #Player.
	getterPhrase submorphs last setSlotRefOperator: retrieverOp.
	getterPhrase submorphs first changeTableLayout.
	receiverTile _ aPlayer tileToRefer bePossessive.
	receiverTile position: getterPhrase firstSubmorph position.
	getterPhrase firstSubmorph addMorph: receiverTile.

	outerPhrase firstSubmorph addMorph: getterPhrase.
	finalTile _ ScriptingSystem tileForArgType: retrieverType.	"comes with arrows"
	outerPhrase submorphs last addMorph: finalTile.
	outerPhrase submorphs second submorphs last setBalloonText: (ScriptingSystem helpStringForOperator: rel).    
	^ outerPhrase! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 3/24/2005 11:18'!
booleanPhraseFromPhrase: phrase
	"Answer, if possible, a boolean-valued phrase derived from the phrase provided"

	|  retrieverOp retrieverTile |
	(phrase isKindOf: ParameterTile) ifTrue: [^ phrase booleanComparatorPhrase].
	phrase isBoolean ifTrue: [^ phrase].
	((scriptedPlayer respondsTo: #costume) 
		and:[scriptedPlayer costume isInWorld not]) ifTrue: [^ Array new].
	((retrieverTile _ phrase submorphs last) isKindOf: TileMorph) ifFalse: [^ phrase].
	retrieverOp _ retrieverTile operatorOrExpression.

	(Vocabulary vocabularyForType: phrase resultType)
		affordsCoercionToBoolean ifTrue: [
			retrieverOp =  #getPatchValueIn: ifTrue: [
				^ self booleanPhraseForGetPatchValueOfType: phrase resultType retrieverOp: retrieverOp player: phrase actualObject.
			].
			retrieverOp =  #getRedComponentIn: ifTrue: [
				^ self booleanPhraseForGetColorComponentOfType: phrase resultType componentName: #red  retrieverOp: retrieverOp player: phrase actualObject.
			].
			retrieverOp =  #getGreenComponentIn: ifTrue: [
				^ self booleanPhraseForGetColorComponentOfType: phrase resultType componentName: #green  retrieverOp: retrieverOp player: phrase actualObject.
			].
			retrieverOp =  #getBlueComponentIn: ifTrue: [
				^ self booleanPhraseForGetColorComponentOfType: phrase resultType componentName: #blue retrieverOp: retrieverOp player: phrase actualObject.
			].
			retrieverOp = #getUphillIn: ifTrue: [
				^ self booleanPhraseForGetUpHillOfType: phrase resultType retrieverOp: retrieverOp player: phrase actualObject.
			].
			retrieverOp = #getDistanceTo: ifTrue: [
				^ self booleanPhraseForGetDistanceToOfType: phrase resultType retrieverOp: retrieverOp player: phrase actualObject.
			].
			retrieverOp = #getAngleTo: ifTrue: [
				^ self booleanPhraseForGetAngleToOfType: phrase resultType retrieverOp: retrieverOp player: phrase actualObject.
			].
			retrieverOp = #bounceOn: ifTrue: [
				^ self booleanPhraseForBounceOnOfType: phrase resultType retrieverOp: retrieverOp player: phrase actualObject.
			].
			(retrieverOp = #bounceOn:color: or: [retrieverOp = #bounceOnColor:]) ifTrue: [
				^ self booleanPhraseForBounceOnColorOfType: phrase resultType retrieverOp: retrieverOp player: phrase actualObject.
			].
			retrieverOp = #getTurtleAt: ifTrue: [
				^ self booleanPhraseForGetTurtleAtOfType: phrase resultType retrieverOp: retrieverOp player: phrase actualObject.
			].
			retrieverOp = #getTurtleOf: ifTrue: [
				^ self booleanPhraseForGetTurtleOfOfType: phrase resultType retrieverOp: retrieverOp player: phrase actualObject.
			].

			^ self booleanPhraseForRetrieverOfType: phrase resultType retrieverOp: retrieverOp player: phrase actualObject.

		].
	^ phrase! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:55'!
categoryNameWhoseTranslatedWordingIs: aWording
	"Answer the category name with the given wording"

	| result |
	result _ self currentVocabulary categoryWhoseTranslatedWordingIs: aWording.
	^ result
		ifNotNil:
			[result categoryName]
		ifNil:
			[aWording]! !

!CategoryViewer methodsFor: 'support' stamp: 'sw 8/17/2002 01:11'!
categoryRestorationInfo
	"Answer info needed to reincarnate myself"

	^ self chosenCategorySymbol! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/12/2001 20:34'!
categoryWhoseTranslatedWordingIs: aWording
	"Answer the elementCategory with the given wording"

	^ self currentVocabulary categoryWhoseTranslatedWordingIs: aWording! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 10/30/2001 13:45'!
categoryWording: aCategoryWording
	"Make the category with the given wording be my current one."

	| actualPane |
	(actualPane _ namePane renderedMorph) firstSubmorph contents: aCategoryWording; color: Color black.
	actualPane extent: actualPane firstSubmorph extent.

	self removeAllButFirstSubmorph. "that being the header"
	self addAllMorphs:
		((scriptedPlayer tilePhrasesForCategory: chosenCategorySymbol inViewer: self)).
	self enforceTileColorPolicy.
	self secreteCategorySymbol.
	self world ifNotNil: [self world startSteppingSubmorphsOf: self].
	self adjustColorsAndBordersWithin.

	owner ifNotNil: [owner isStandardViewer ifTrue: [owner fitFlap]]! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 3/2/2004 23:53'!
chooseCategory
	"The mouse went down on my category-list control; pop up a list of category choices"

	| aList aMenu reply aLinePosition lineList |
	aList _ scriptedPlayer categoriesForViewer: self.

	aLinePosition _ aList indexOf: #miscellaneous ifAbsent: [nil].
	aList _ aList collect:	
		[:aCatSymbol | self currentVocabulary categoryWordingAt: aCatSymbol].

	lineList _ aLinePosition ifNil: [#()] ifNotNil: [Array with: aLinePosition].
	aList size == 0 ifTrue: [aList add: ScriptingSystem nameForInstanceVariablesCategory translated].
	aMenu _ CustomMenu labels: aList lines: lineList selections: aList.
	reply _ aMenu startUpWithCaption: 'category' translated.
	reply ifNil: [^ self].
	self chooseCategoryWhoseTranslatedWordingIs: reply asSymbol
! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:56'!
chooseCategoryWhoseTranslatedWordingIs: aWording
	"Choose the category with the given wording"

	self chosenCategorySymbol: (self categoryNameWhoseTranslatedWordingIs: aWording)
! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 5/29/2001 22:43'!
chosenCategorySymbol
	"Answer the inherent category currently being shown, not necessarily the same as the translated word."

	^ chosenCategorySymbol ifNil: [self secreteCategorySymbol]! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:49'!
chosenCategorySymbol: aCategorySymbol
	"Make the given category be my current one."

	| aCategory wording |
	chosenCategorySymbol _ aCategorySymbol.
	aCategory _ self currentVocabulary categoryAt: chosenCategorySymbol.
	wording _ aCategory ifNil: [aCategorySymbol] ifNotNil: [aCategory wording].
	self categoryWording: wording! !

!CategoryViewer methodsFor: 'editing pane' stamp: 'nb 6/17/2003 12:25'!
contents: c notifying: k
	"later, spruce this up so that it can accept input such as new method source"
	
	Beeper beep.
	^ false! !

!CategoryViewer methodsFor: 'support' stamp: 'sw 10/30/1998 18:15'!
contentsSelection
	"Not well understood why this needs to be here!!"
	^ 1 to: 0! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 2/23/2001 22:29'!
currentCategory
	"Answer the symbol representing the receiver's currently-selected category"

	| current |
	current _ namePane renderedMorph firstSubmorph contents.
	^ current ifNotNil: [current asSymbol] ifNil: [#basic]! !

!CategoryViewer methodsFor: 'macpal' stamp: 'sw 5/4/2001 05:24'!
currentVocabulary
	"Answer the vocabulary currently installed in the viewer.  The outer StandardViewer object holds this information."

	| outerViewer |
	^  (outerViewer _ self outerViewer)
		ifNotNil:
			[outerViewer currentVocabulary]
		ifNil:
			[(self world ifNil: [ActiveWorld]) currentVocabularyFor: scriptedPlayer]! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 10/24/1998 14:24'!
downArrowHit
	self previousCategory! !

!CategoryViewer methodsFor: 'initialization' stamp: 'sw 8/22/2002 23:08'!
establishContents
	"Perform any initialization steps that needed to wait until I am installed in my outer viewer"! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'sw 9/27/2001 04:23'!
getterButtonFor: getterSelector type: partType
	"Answer a classic-tiles getter button for a part of the given name"

	| m inherent wording |
	m _ TileMorph new adoptVocabulary: self currentVocabulary.

	inherent _ Utilities inherentSelectorForGetter: getterSelector.
	wording _ (scriptedPlayer slotInfo includesKey: inherent)
		ifTrue: [inherent]
		ifFalse: [self currentVocabulary tileWordingForSelector: 
getterSelector].
	m setOperator: getterSelector andUseWording: wording.
	m typeColor: (ScriptingSystem colorForType: partType).
	m on: #mouseDown send: #makeGetter:event:from:
		to: self
		withValue: (Array with: getterSelector with: partType).
	^ m! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'yo 3/24/2005 13:12'!
getterTilesFor: getterSelector type: aType 
	"Answer classic getter for the given name/type"

	"aPhrase _ nil, assumed"

	| selfTile selector aPhrase |
	(#(#color:sees: #colorSees) includes: getterSelector) 
		ifTrue: [aPhrase := self colorSeesPhrase].
	(#(#getPatchValueIn:) includes: getterSelector)
		ifTrue: [aPhrase _ self patchValuePhrase].
	(#(#getRedComponentIn:) includes: getterSelector)
		ifTrue: [aPhrase _ self colorComponentPhraseFor: #red].
	(#(#getGreenComponentIn:) includes: getterSelector)
		ifTrue: [aPhrase _ self colorComponentPhraseFor: #green].
	(#(#getBlueComponentIn:) includes: getterSelector)
		ifTrue: [aPhrase _ self colorComponentPhraseFor: #blue].
	(#(#getUphillIn:) includes: getterSelector)
		ifTrue: [aPhrase _ self patchUphillPhrase].
	(#(#bounceOn:) includes: getterSelector)
		ifTrue: [aPhrase _ self bounceOnPhrase].
	(#(#bounceOn:color: #bounceOnColor:) includes: getterSelector)
		ifTrue: [aPhrase _ self bounceOnColorPhrase].
	(getterSelector = #getDistanceTo:)
		ifTrue: [aPhrase _ self distanceToPhrase].
	(getterSelector = #getAngleTo:)
		ifTrue: [aPhrase _ self angleToPhrase].
	(getterSelector = #getTurtleOf:)
		ifTrue: [aPhrase _ self turtleOfPhrase].
	(#(#seesColor: #isOverColor) includes: getterSelector) 
		ifTrue: [aPhrase := self seesColorPhrase].
	(#(#overlaps: #overlaps) includes: getterSelector) 
		ifTrue: [aPhrase := self overlapsPhrase].
	(#(#overlapsAny: #overlapsAny) includes: getterSelector) 
		ifTrue: [aPhrase := self overlapsAnyPhrase].
	(#(#touchesA: #touchesA) includes: getterSelector) 
		ifTrue: [aPhrase := self touchesAPhrase].
	aPhrase ifNil: 
			[aPhrase := PhraseTileMorph new setSlotRefOperator: getterSelector asSymbol
						type: aType].
	selfTile := self tileForSelf bePossessive.
	selfTile position: aPhrase firstSubmorph position.
	aPhrase firstSubmorph addMorph: selfTile.
	selector := aPhrase submorphs second.
	(#(#getPatchValueIn: getUphillIn:) includes: getterSelector) ifFalse: [
		(Vocabulary vocabularyNamed: aType capitalized) 
			ifNotNilDo: [:aVocab | aVocab wantsSuffixArrow ifTrue: [selector addSuffixArrow]].
	].
	selector updateLiteralLabel.
	aPhrase enforceTileColorPolicy.
	^aPhrase! !

!CategoryViewer methodsFor: 'entries' stamp: 'md 11/14/2003 16:20'!
infoButtonFor: aScriptOrSlotSymbol
	"Answer a fully-formed morph that will serve as the 'info button' alongside an entry corresponding to the given slot or script symbol.  If no such button is appropriate, answer a transparent graphic that fills the same space."

	| aButton |
	(self wantsRowMenuFor: aScriptOrSlotSymbol) ifFalse:
		["Fill the space with sweet nothing, since there is no meaningful menu to offer"
		aButton _ RectangleMorph new beTransparent extent: (17@20).
		aButton borderWidth: 0.
		^ aButton].

	aButton _ IconicButton new labelGraphic: Cursor menu.
	aButton target: scriptedPlayer;
		actionSelector: #infoFor:inViewer:;
		arguments: (Array with:aScriptOrSlotSymbol with: self);
		color: Color transparent;
		borderWidth: 0;
		shedSelvedge;
		actWhen: #buttonDown.
	aButton setBalloonText: 'Press here to get a menu' translated.
	^ aButton! !

!CategoryViewer methodsFor: 'initialization' stamp: 'sw 9/8/2000 10:58'!
initializeFor: aPlayer
	"Initialize the category pane to show the #basic category by default"

	^ self initializeFor: aPlayer categoryChoice: #basic
! !

!CategoryViewer methodsFor: 'initialization' stamp: 'dgd 8/16/2004 21:51'!
initializeFor: aPlayer categoryChoice: aChoice
	"Initialize the receiver to be associated with the player and category specified"

	self listDirection: #topToBottom;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 1;
		beSticky.
	self color: Color green muchLighter muchLighter.
	scriptedPlayer _ aPlayer.
	self addHeaderMorph.

	self chooseCategoryWhoseTranslatedWordingIs: aChoice
! !

!CategoryViewer methodsFor: 'support' stamp: 'sw 3/1/1999 11:57'!
invisiblySetPlayer: aPlayer
	scriptedPlayer _ aPlayer! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 8/11/2005 14:54'!
isSpecialPatchCase: aPlayer and: cmd

	((aPlayer costume renderedMorph class = KedamaMorph) and: [cmd = #addToPatchDisplayList:]) ifTrue: [
		^ true.
	].
	(aPlayer costume renderedMorph class = KedamaPatchMorph) ifTrue: [
		(#(#redComponentInto: #greenComponentInto: #blueComponentInto:
			#redComponentFrom: #greenComponentFrom: #blueComponentFrom:) includes: cmd) ifTrue: [
		^ true.
	].
	].

	^ false.
! !

!CategoryViewer methodsFor: 'support' stamp: 'yo 8/11/2005 14:48'!
isSpecialPatchReceiver: aPlayer and: cmd

	^ (aPlayer costume renderedMorph class = KedamaPatchMorph) and: [
		(#(#redComponentInto: #greenComponentInto: #blueComponentInto:
			#redComponentFrom: #greenComponentFrom: #blueComponentFrom:) includes: cmd)
	].
! !

!CategoryViewer methodsFor: 'scripting' stamp: 'sw 9/12/2001 22:58'!
isTileScriptingElement
	"Answer whether the receiver is a tile-scripting element"

	^ true! !

!CategoryViewer methodsFor: 'support' stamp: 'sw 8/6/2001 19:42'!
limitClass
	"Answer the receiver's limitClass"

	| outer |
	^ (outer _ self outerViewer)
		ifNotNil:
			[outer limitClass]
		ifNil:
			[ProtoObject]! !

!CategoryViewer methodsFor: 'e-toy support' stamp: 'mir 7/15/2004 15:19'!
localeChanged
	"Update myself to reflect the change in locale"

	chosenCategorySymbol ifNil: [^ self delete].
	self updateCategoryNameTo: ((self currentVocabulary ifNil: [Vocabulary eToyVocabulary]) categoryWordingAt: chosenCategorySymbol)! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'sw 4/6/2001 00:59'!
makeGetter: args event: evt from: aMorph
	"Hand the user tiles representing a classic getter on the slot represented by aMorph"

	| tiles |
	tiles _ self getterTilesFor: args first type: args second.
	owner
		ifNotNil:	[self primaryHand attachMorph: tiles]
		ifNil: 		[^ tiles]
! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'ar 3/18/2001 17:26'!
makeGetter: arg1 from: arg2 forPart: arg3
	"Reorder the arguments for existing event handlers"
	(arg3 isMorph and:[arg3 eventHandler notNil]) ifTrue:[arg3 eventHandler fixReversedValueMessages].
	^self makeGetter: arg1 event: arg2 from: arg3! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'yo 7/22/2005 16:52'!
makeSetter: selectorAndTypePair event: evt from: aMorph 
	"Classic tiles: make a Phrase that comprises a setter of a slot, and hand it to the user."

	| argType m argTile selfTile argValue actualGetter |
	selectorAndTypePair first = #getPatchValueIn: ifTrue: [^ self makeSetterForGetPatchValue: selectorAndTypePair event: evt from: aMorph].
	selectorAndTypePair first = #getRedComponentIn: ifTrue: [^ self makeSetterForColorComponent: selectorAndTypePair componentName: #red event: evt from: aMorph].
	selectorAndTypePair first = #getBlueComponentIn: ifTrue: [^ self makeSetterForColorComponent: selectorAndTypePair componentName: #blue event: evt from: aMorph].
	selectorAndTypePair first = #getGreenComponentIn: ifTrue: [^ self makeSetterForColorComponent: selectorAndTypePair componentName: #green event: evt from: aMorph].
	
	argType := selectorAndTypePair second.
	actualGetter := selectorAndTypePair first asSymbol.
	m := PhraseTileMorph new 
				setAssignmentRoot: (Utilities inherentSelectorForGetter: actualGetter)
				type: #command
				rcvrType: #Player
				argType: argType
				vocabulary: self currentVocabulary.
	argValue := self scriptedPlayer 
				perform: selectorAndTypePair first asSymbol.
	argValue isPlayerLike
		ifTrue: [argTile := argValue tileReferringToSelf]
		ifFalse: 
			[argTile := ScriptingSystem tileForArgType: argType.
			(argType == #Number and: [argValue isNumber]) 
				ifTrue: 
					[(scriptedPlayer decimalPlacesForGetter: actualGetter) 
						ifNotNilDo: [:places | (argTile findA: UpdatingStringMorph) decimalPlaces: places]].
			argTile
				setLiteral: argValue;
				updateLiteralLabel].
	argTile position: m lastSubmorph position.
	m lastSubmorph addMorph: argTile.
	selfTile := self tileForSelf bePossessive.
	selfTile position: m firstSubmorph position.
	m firstSubmorph addMorph: selfTile.
	m enforceTileColorPolicy.
	m openInHand! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'ar 3/18/2001 17:27'!
makeSetter: arg1 from: arg2 forPart: arg3
	"Reorder the arguments for existing event handlers"
	(arg3 isMorph and:[arg3 eventHandler notNil]) ifTrue:[arg3 eventHandler fixReversedValueMessages].	
	^self makeSetter: arg1 event: arg2 from: arg3! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'yo 8/10/2005 17:29'!
makeSetterForColorComponent: selectorAndTypePair componentName: componentName event: evt from: aMorph 

	| argType m argTile selfTile argValue actualGetter |
	argType := selectorAndTypePair second.
	componentName = #red ifTrue: [actualGetter _ #setRedComponentIn:].
	componentName = #green ifTrue: [actualGetter _ #setGreenComponentIn:].
	componentName = #blue ifTrue: [actualGetter _ #setBlueComponentIn:].
	m := PhraseTileMorph new 
				setColorComponentRoot: actualGetter
				componentName: componentName
				type: #command
				rcvrType: #Patch
				argType: argType
				vocabulary: self currentVocabulary.
	argValue := self scriptedPlayer 
				perform: selectorAndTypePair first asSymbol with: nil.
	(argValue isKindOf: Player) 
		ifTrue: [argTile := argValue tileReferringToSelf]
		ifFalse: 
			[argTile := ScriptingSystem tileForArgType: argType.
			(argType == #Number and: [argValue isNumber]) 
				ifTrue: 
					[(scriptedPlayer decimalPlacesForGetter: actualGetter) 
						ifNotNilDo: [:places | (argTile findA: UpdatingStringMorph) decimalPlaces: places]].
			argTile
				setLiteral: argValue;
				updateLiteralLabel].
	argTile position: m lastSubmorph position.
	m lastSubmorph addMorph: argTile.
	selfTile := self tileForSelf bePossessive.
	selfTile position: m firstSubmorph position.
	m firstSubmorph addMorph: selfTile.
	m enforceTileColorPolicy.
	m submorphs second setPatchDefaultTo: scriptedPlayer defaultPatchPlayer.

	m openInHand! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'yo 3/31/2005 15:54'!
makeSetterForGetPatchValue: selectorAndTypePair event: evt from: aMorph 

	| argType m argTile selfTile argValue actualGetter |
	argType := selectorAndTypePair second.
	actualGetter := #patchValueIn:.
	m := PhraseTileMorph new 
				setPixelValueRoot: actualGetter
				type: #command
				rcvrType: #Player
				argType: argType
				vocabulary: self currentVocabulary.
	argValue := self scriptedPlayer 
				perform: selectorAndTypePair first asSymbol with: nil.
	(argValue isPlayerLike) 
		ifTrue: [argTile := argValue tileReferringToSelf]
		ifFalse: 
			[argTile := ScriptingSystem tileForArgType: argType.
			(argType == #Number and: [argValue isNumber]) 
				ifTrue: 
					[(scriptedPlayer decimalPlacesForGetter: actualGetter) 
						ifNotNilDo: [:places | (argTile findA: UpdatingStringMorph) decimalPlaces: places]].
			argTile
				setLiteral: argValue;
				updateLiteralLabel].
	argTile position: m lastSubmorph position.
	m lastSubmorph addMorph: argTile.
	selfTile := self tileForSelf bePossessive.
	selfTile position: m firstSubmorph position.
	m firstSubmorph addMorph: selfTile.
	m enforceTileColorPolicy.
	m submorphs second setPatchDefaultTo: scriptedPlayer defaultPatchPlayer.

	m openInHand! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'RAA 4/6/2001 13:28'!
makeUniversalTilesGetter: aMethodInterface event: evt from: aMorph
	"Button in viewer performs this to make a universal-tiles getter and attach it to hand."

	| newTiles |
	newTiles _ self newGetterTilesFor: scriptedPlayer methodInterface: aMethodInterface.
	newTiles setProperty: #beScript toValue: true.
	owner
		ifNotNil:
			[ActiveHand attachMorph: newTiles.
			newTiles align: newTiles topLeft with: evt hand position + (7@14)]
		ifNil:
			[^ newTiles]
! !

!CategoryViewer methodsFor: 'header pane' stamp: 'dgd 9/1/2003 13:47'!
maybeAddArrowsTo: header
	"Maybe add up/down arrows to the header"

	| wrpr |
	header addTransparentSpacerOfSize: 5@5.
	header addUpDownArrowsFor: self.
	(wrpr _ header submorphs last) submorphs second setBalloonText: 'previous category' translated.	
	wrpr submorphs first  setBalloonText: 'next category' translated! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'nk 10/14/2004 10:53'!
newGetterTilesFor: aPlayer methodInterface: aMethodInterface
	"Return universal tiles for a getter on this property.  Record who self is."

	| ms argTile argArray |
	ms _ MessageSend receiver: aPlayer selector: aMethodInterface selector arguments: #().

	"Handle three idiosyncratic cases..."
	aMethodInterface selector == #color:sees: ifTrue:
		[argTile _ ScriptingSystem tileForArgType: #Color.
		argArray _ Array with: argTile colorSwatch color with: argTile colorSwatch color copy. 
		ms arguments: argArray].
	aMethodInterface selector == #seesColor: ifTrue:
		[argTile _ ScriptingSystem tileForArgType: #Color.
		ms arguments: (Array with: argTile colorSwatch color)].

	(#(touchesA: overlaps: overlapsAny:) includes: aMethodInterface selector) ifTrue:
		[argTile _ ScriptingSystem tileForArgType: #Player.
		ms arguments: (Array with: argTile actualObject)].

	^ ms asTilesIn: aPlayer class globalNames: (aPlayer class officialClass ~~ CardPlayer)
			"For CardPlayers, use 'self'.  For others, name it, and use its name."! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'sw 3/28/2001 14:17'!
newMakeGetter: arg event: evt from: aMorph
	"Button in viewer performs this to makea universal-tiles header tile and attach to hand."

	^ self makeUniversalTilesGetter: arg event: evt from: aMorph! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'sw 3/28/2001 13:04'!
newMakeGetter: arg1 from: arg2 forMethodInterface: arg3
	"Button in viewer performs this to make a new style tile and attach to hand. (Reorder the arguments for existing event handlers)"

	(arg3 isMorph and: [arg3 eventHandler notNil]) ifTrue:
		[arg3 eventHandler fixReversedValueMessages].
	 ^ self makeUniversalTilesGetter: arg1 event: arg2 from: arg3! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'ar 3/18/2001 17:27'!
newMakeGetter: arg1 from: arg2 forPart: arg3
	"Reorder the arguments for existing event handlers"
	(arg3 isMorph and:[arg3 eventHandler notNil]) ifTrue:[arg3 eventHandler fixReversedValueMessages].
	^self newMakeGetter: arg1 event: arg2 from: arg3! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'ar 3/17/2001 14:17'!
newMakeSetter: aSpec event: evt from: aMorph
	"Button in viewer performs this to make a new style tile and attach to hand."

	| m |
	m _ self newTilesFor: scriptedPlayer setter: aSpec.
	owner ifNotNil: [self primaryHand attachMorph: m.
			m align: m topLeft with: evt hand position + (7@14)]
		ifNil: [^ m].
! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'ar 3/18/2001 17:27'!
newMakeSetter: arg1 from: arg2 forPart: arg3
	"Reorder the arguments for existing event handlers"
	(arg3 isMorph and:[arg3 eventHandler notNil]) ifTrue:[arg3 eventHandler fixReversedValueMessages].
	^self newMakeSetter: arg1 event: arg2 from: arg3! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'RAA 4/6/2001 13:28'!
newMakeSetterFromInterface: aMethodInterface evt: evt from: aMorph 
	"Button in viewer performs this to make a new style tile and attach to hand."

	| m |
	m _ self newSetterTilesFor: scriptedPlayer methodInterface: aMethodInterface.
	m setProperty: #beScript toValue: true.

	owner
		ifNotNil: [self primaryHand attachMorph: m.
			m align: m topLeft with: evt hand position + (7@14)]
		ifNil: [^ m]
! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'tk 9/30/2001 11:20'!
newSetterTilesFor: aPlayer methodInterface: aMethodInterface
	"Return universal tiles for a setter on this property.  Record who self is."

	| ms argValue makeSelfGlobal phrase |
	argValue _ aPlayer perform: aMethodInterface selector.
	ms _ MessageSend receiver: aPlayer 
		selector: aMethodInterface companionSetterSelector 
		arguments: (Array with: argValue).
	makeSelfGlobal _ aPlayer class officialClass ~~ CardPlayer.
	phrase _ ms asTilesIn: aPlayer class globalNames: makeSelfGlobal.
			"For CardPlayers, use 'self'.  For others, name it, and use its name."
	makeSelfGlobal ifFalse: [phrase setProperty: #scriptedPlayer toValue: aPlayer].
	^ phrase! !

!CategoryViewer methodsFor: 'get/set slots' stamp: 'sw 11/16/2001 14:44'!
newTilesFor: aPlayer setter: aSpec
	| ms  argValue |
	"Return universal tiles for a getter on this property.  Record who self is."

	argValue _ aPlayer perform: (Utilities getterSelectorFor: aSpec second asSymbol).
	ms _ MessageSend receiver: aPlayer selector: aSpec ninth arguments: (Array with: argValue).
	^ ms asTilesIn: aPlayer class globalNames: (aPlayer class officialClass ~~ CardPlayer)
			"For CardPlayers, use 'self'.  For others, name it, and use its name."! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:57'!
nextCategory
	"Change the receiver to point at the category following the one currently seen"

	| aList anIndex newIndex already aChoice |
	aList _ (scriptedPlayer categoriesForViewer: self) collect:
		[:aCatSymbol | self currentVocabulary categoryWordingAt: aCatSymbol].

	already _ self outerViewer ifNil: [#()] ifNotNil: [self outerViewer categoriesCurrentlyShowing].
	anIndex _ aList indexOf: self currentCategory ifAbsent: [0].
	newIndex _ anIndex = aList size
		ifTrue:		[1]
		ifFalse:		[anIndex + 1].
	[already includes: (aChoice _ aList at: newIndex)] whileTrue:
		[newIndex _ (newIndex \\ aList size) + 1].
	self chooseCategoryWhoseTranslatedWordingIs: aChoice! !

!CategoryViewer methodsFor: 'support' stamp: 'sw 11/18/1999 16:04'!
outerViewer
	"Answer the StandardViewer or equivalent that contains this object"
	^ self ownerThatIsA: Viewer! !

!CategoryViewer methodsFor: 'entries' stamp: 'yo 8/18/2005 10:37'!
phraseForCommandFrom: aMethodInterface
	"Answer a phrase for the non-slot-like command represented by aMethodInterface - classic tiles"

	| aRow resultType cmd names argType argTile selfTile aPhrase balloonTextSelector stat inst aDocString universal tileBearingHelp |
	aDocString _ aMethodInterface documentation.
	aDocString = 'no help available' ifTrue: [aDocString _ nil].
	names _ scriptedPlayer class namedTileScriptSelectors.

	resultType _ aMethodInterface resultType.
	cmd _ aMethodInterface selector.
	(universal _ scriptedPlayer isUniversalTiles)
		ifTrue:
			[aPhrase _ scriptedPlayer universalTilesForInterface: aMethodInterface]
		ifFalse: [cmd numArgs == 0
			ifTrue:
				[aPhrase _ PhraseTileMorph new vocabulary: self currentVocabulary.
				aPhrase setOperator: cmd
					type: resultType
					rcvrType: #Player]
			ifFalse:
				["only one arg supported in classic tiles, so if this is fed
				with a selector with > 1 arg, results will be very strange"
				argType _ aMethodInterface typeForArgumentNumber: 1.
				aPhrase _ PhraseTileMorph new vocabulary: self currentVocabulary.
				(self isSpecialPatchReceiver: scriptedPlayer and: cmd) ifTrue: [
					aPhrase setOperator: cmd
						type: resultType
						rcvrType: #Patch
						argType: argType.
				] ifFalse: [
					aPhrase setOperator: cmd
						type: resultType
						rcvrType: #Player
						argType: argType.
				].
				(self isSpecialPatchCase: scriptedPlayer and: cmd) ifTrue: [
					argTile _ (Vocabulary vocabularyForType: argType) defaultArgumentTileFor: scriptedPlayer.
				] ifFalse: [
					argTile _ ScriptingSystem tileForArgType: argType.
				].
				(#(bounce: wrap:) includes: cmd) ifTrue:
					["help for the embattled bj"
					argTile setLiteral: 'silence'; updateLiteralLabel].
				argTile position: aPhrase lastSubmorph position.
				aPhrase lastSubmorph addMorph: argTile]].

	(scriptedPlayer slotInfo includesKey: cmd)
		ifTrue: [balloonTextSelector _ #userSlot].

	(scriptedPlayer belongsToUniClass and: [scriptedPlayer class includesSelector: cmd])
		ifTrue:
			[aDocString ifNil:
				[aDocString _ (scriptedPlayer class userScriptForPlayer: scriptedPlayer selector: cmd) documentation].
			aDocString ifNil:
				[balloonTextSelector _ #userScript]].

	tileBearingHelp _ universal ifTrue: [aPhrase submorphs second] ifFalse: [aPhrase operatorTile]. 
	aDocString
		ifNotNil:
			[tileBearingHelp setBalloonText: aDocString]
		ifNil:
			[balloonTextSelector ifNil:
				[tileBearingHelp setProperty: #inherentSelector toValue: cmd.
				balloonTextSelector _ #methodComment].
			tileBearingHelp balloonTextSelector: balloonTextSelector].
	aPhrase markAsPartsDonor.
	cmd == #emptyScript ifTrue:
		[aPhrase setProperty: #newPermanentScript toValue: true.
		aPhrase setProperty: #newPermanentPlayer toValue: scriptedPlayer.
		aPhrase submorphs second setBalloonText: 
'drag and drop to 
add a new script' translated].

	universal ifFalse:
		[selfTile _ self tileForSelf.
		selfTile position: aPhrase firstSubmorph position.
		aPhrase firstSubmorph addMorph: selfTile].

	aRow _ ViewerLine newRow borderWidth: 0; color: self color.
	aRow elementSymbol: cmd asSymbol.

	aRow addMorphBack: (ScriptingSystem tryButtonFor: aPhrase).
	aRow addMorphBack: (Morph new extent: 2@2; beTransparent).
	aRow addMorphBack: (self infoButtonFor: cmd).
	aRow addMorphBack: aPhrase.
	aPhrase on: #mouseEnter send: #addCommandFeedback to: aRow.
	aPhrase on: #mouseLeave send: #removeHighlightFeedback to: aRow.
	aPhrase on: #mouseLeaveDragging send: #removeHighlightFeedback to: aRow.

	(names includes: cmd) ifTrue:
		[aPhrase userScriptSelector: cmd.
		cmd numArgs == 0 ifTrue:
			[aPhrase beTransparent.
			aRow addMorphBack: AlignmentMorph newVariableTransparentSpacer.
			aRow addMorphBack: (stat _ (inst _ scriptedPlayer scriptInstantiationForSelector: cmd) statusControlMorph).
			inst updateStatusMorph: stat]].

	aRow beSticky; disableDragNDrop.

	^ aRow! !

!CategoryViewer methodsFor: 'entries' stamp: 'yo 3/7/2005 08:45'!
phraseForVariableFrom: aMethodInterface
	"Return a structure consisting of tiles and controls and a readout representing a 'variable' belonging to the player, complete with an appropriate readout when indicated.  Functions in both universalTiles mode and classic mode.  Slightly misnamed in that this path is used for any methodInterface that indicates an interesting resultType."

	| anArrow slotName getterButton cover inner aRow doc setter tryer universal hotTileForSelf spacer buttonFont varName |
	aRow _ ViewerLine newRow
		color: self color;
		beSticky;
		elementSymbol: (slotName _ aMethodInterface selector);
		wrapCentering: #center;
		cellPositioning: #leftCenter.
	(universal _ scriptedPlayer isUniversalTiles) ifFalse:
		[buttonFont _ Preferences standardEToysFont.
			aRow addMorphBack: (Morph new color: self color;
					 extent: (((buttonFont widthOfString: '!!') + 8) @ (buttonFont height + 6));
					 yourself)].  "spacer"
	aRow addMorphBack: (self infoButtonFor: slotName).
	aRow addMorphBack: (Morph new color: self color; extent: 0@10).  " vertical spacer"
	universal
		ifTrue:
			[inner _ scriptedPlayer universalTilesForGetterOf: aMethodInterface.
			cover _ Morph new color: Color transparent.
			cover extent: inner fullBounds extent.
			(getterButton _ cover copy) addMorph: cover; addMorphBack: inner.
			cover on: #mouseDown send: #makeUniversalTilesGetter:event:from: 
					to: self withValue: aMethodInterface.
			aRow addMorphFront:  (tryer _ ScriptingSystem tryButtonFor: inner).
			tryer color: tryer color lighter lighter]
		ifFalse:
			[hotTileForSelf _ self tileForSelf bePossessive.
			hotTileForSelf  on: #mouseDown send: #makeGetter:event:from:
				to: self
				withValue: (Array with: aMethodInterface selector with: aMethodInterface resultType).
			aRow addMorphBack: hotTileForSelf.
			aRow addMorphBack: (spacer _ Morph new color: self color; extent: 2@10).
			spacer on: #mouseEnter send: #addGetterFeedback to: aRow.
			spacer on: #mouseLeave send: #removeHighlightFeedback to: aRow.
			spacer on: #mouseLeaveDragging send: #removeHighlightFeedback to: aRow.
			spacer  on: #mouseDown send: #makeGetter:event:from:
				to: self
				withValue: (Array with: aMethodInterface selector with: aMethodInterface resultType).
			hotTileForSelf on: #mouseEnter send: #addGetterFeedback to: aRow.
			hotTileForSelf on: #mouseLeave send: #removeHighlightFeedback to: aRow.
			hotTileForSelf on: #mouseLeaveDragging send: #removeHighlightFeedback to: aRow.
			getterButton _ self getterButtonFor: aMethodInterface selector type: aMethodInterface resultType].
	aRow addMorphBack: getterButton.
	getterButton on: #mouseEnter send: #addGetterFeedback to: aRow.
	getterButton on: #mouseLeave send: #removeHighlightFeedback to: aRow.
	getterButton on: #mouseLeaveDragging send: #removeHighlightFeedback to: aRow.
	(doc _ aMethodInterface documentation) ifNotNil:
		[getterButton setBalloonText: doc].

	(scriptedPlayer slotInfo includesKey: (varName _ Utilities inherentSelectorForGetter: slotName)) "user slot"
		ifTrue:
			["aRow addTransparentSpacerOfSize: 3@0.
			aRow addMorphBack: (self slotTypeMenuButtonFor: varName)"].

	universal ifFalse:
		[(slotName == #seesColor:) ifTrue:
			[self addIsOverColorDetailTo: aRow.
			^ aRow].
		(slotName == #touchesA:) ifTrue:
			[self addTouchesADetailTo: aRow.
			^ aRow].
		(slotName == #overlaps: or: [ slotName == #overlapsAny:]) ifTrue:
			[self addOverlapsDetailTo: aRow.
			^ aRow]].
	aRow addMorphBack: (AlignmentMorph new beTransparent).  "flexible spacer"
	(setter _ aMethodInterface companionSetterSelector) ifNotNil:
		[aRow addMorphBack: (Morph new color: self color; extent: 2@10).  " spacer"
		anArrow _ universal 
			ifTrue: [self arrowSetterButton: #newMakeSetterFromInterface:evt:from:  
						args: aMethodInterface]
			ifFalse: [self arrowSetterButton: #makeSetter:from:forPart:
						args: (Array with: slotName with: aMethodInterface resultType)].
		anArrow beTransparent.
		universal ifFalse:
			[anArrow on: #mouseEnter send: #addSetterFeedback to: aRow.
			anArrow on: #mouseLeave send: #removeHighlightFeedback to: aRow.
			anArrow on: #mouseLeaveDragging send: #removeHighlightFeedback to: aRow].
		aRow addMorphBack: anArrow].
	(#(color:sees: playerSeeingColor copy touchesA: overlaps: getTurtleAt: getTurtleOf:) includes: slotName) ifFalse:
 		[(universal and: [slotName == #seesColor:]) ifFalse:
			[aMethodInterface wantsReadoutInViewer ifTrue: 
				[aRow addMorphBack: (self readoutFor: slotName type: aMethodInterface resultType readOnly: setter isNil getSelector: aMethodInterface selector putSelector: setter)]]].
	anArrow ifNotNil: [anArrow step].
	^ aRow! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:53'!
previousCategory
	"Change the receiver to point at the category preceding the one currently seen"

	| aList anIndex newIndex already aChoice |
	aList _ (scriptedPlayer categoriesForViewer: self) collect:
		[:aCatSymbol | self currentVocabulary categoryWordingAt: aCatSymbol].
	already _ self outerViewer ifNil: [#()] ifNotNil: [self outerViewer categoriesCurrentlyShowing].
	anIndex _ aList indexOf: self currentCategory ifAbsent: [aList size + 1].
	newIndex _ anIndex = 1
		ifTrue:		[aList size]
		ifFalse:		[anIndex - 1].
	[already includes: (aChoice _ aList at: newIndex)] whileTrue:
		[newIndex _ newIndex = 1 ifTrue: [aList size] ifFalse: [newIndex - 1]].

	self chooseCategoryWhoseTranslatedWordingIs: aChoice! !

!CategoryViewer methodsFor: 'entries' stamp: 'sw 7/4/2004 01:09'!
readoutFor: partName type: partType readOnly: readOnly getSelector: getSelector putSelector: putSelector
	"Answer a readout morph for the given part"

	| readout delta |
	readout _ (Vocabulary vocabularyForType: partType) updatingTileForTarget: scriptedPlayer partName: partName getter: getSelector setter: putSelector.

	(partType == #Number) ifTrue:
		[(delta _ scriptedPlayer arrowDeltaFor: getSelector) = 1
			ifFalse:
				[readout setProperty: #arrowDelta toValue: delta].
		scriptedPlayer setFloatPrecisionFor: readout updatingStringMorph].

	readout step.
	^ readout! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 9/13/2001 19:50'!
secreteCategorySymbol
	"Set my chosenCategorySymbol by translating back from its representation in the namePane.  Answer the chosenCategorySymbol"

	| aCategory |
	aCategory _ self currentVocabulary categoryWhoseTranslatedWordingIs: self currentCategory.
	^ chosenCategorySymbol _ aCategory
		ifNotNil:
			[aCategory categoryName]
		ifNil:
			[self currentCategory]! !

!CategoryViewer methodsFor: 'initialization' stamp: 'sw 8/17/2002 01:23'!
setCategorySymbolFrom: aChoice
	"Set my category symbol"

	self chosenCategorySymbol: aChoice asSymbol
! !

!CategoryViewer methodsFor: 'categories' stamp: 'nk 9/2/2004 19:37'!
showCategoriesFor: aSymbol
	"Put up a pop-up list of categories in which aSymbol is filed; replace the receiver with a CategoryViewer for the one the user selects, if any"

	| allCategories aVocabulary hits meths chosen |
	aVocabulary _ self currentVocabulary.
	allCategories _ scriptedPlayer categoriesForVocabulary: aVocabulary limitClass: ProtoObject.

	hits _ allCategories select:
		[:aCategory | 
			meths _ aVocabulary allMethodsInCategory: aCategory forInstance: scriptedPlayer ofClass: scriptedPlayer class.
			meths includes: aSymbol].

	hits isEmpty ifTrue: [ ^self ].

	chosen _ (SelectionMenu selections: hits) startUp.
	chosen isEmptyOrNil ifFalse:
		[self outerViewer addCategoryViewerFor: chosen atEnd: true]

	! !

!CategoryViewer methodsFor: 'support' stamp: 'sw 5/4/2001 05:32'!
tileForSelf
	"Return a tile representing the receiver's viewee"

	^ scriptedPlayer tileToRefer
! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 10/24/1998 14:25'!
upArrowHit
	self nextCategory! !

!CategoryViewer methodsFor: 'categories' stamp: 'sw 5/29/2001 11:41'!
updateCategoryNameTo: aName
	"Update the category name, because of a language change."

	| actualPane |
	(actualPane _ namePane firstSubmorph) contents: aName; color: Color black.
	namePane extent: actualPane extent.
	self world ifNotNil: [self world startSteppingSubmorphsOf: self]
! !

!CategoryViewer methodsFor: 'entries' stamp: 'nk 10/14/2004 10:54'!
wantsRowMenuFor: aSymbol
	"Answer whether a viewer row for the given symbol should have a menu button on it"

	| elementType |

	true ifTrue: [^ true].  "To allow show categories item.  So someday this method can be removed, and its sender can stop sending it..."

	elementType _ scriptedPlayer elementTypeFor: aSymbol vocabulary: self currentVocabulary.
	(elementType == #systemScript) ifTrue: [^ false].
	((elementType == #systemSlot) and:
		[#(color:sees: touchesA: overlaps: overlapsAny:) includes: aSymbol]) ifTrue: [^ false].
	^ true! !

CategoryViewer subclass: #KedamaCategoryViewer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaCategoryViewer methodsFor: 'support' stamp: 'yo 4/3/2005 23:14'!
booleanPhraseFromPhrase: phrase

	| outerPhrase |
	outerPhrase _ super booleanPhraseFromPhrase: phrase.
	outerPhrase replacePlayerInReadoutWith: scriptedPlayer.
	^ outerPhrase.
! !

!KedamaCategoryViewer methodsFor: 'get/set slots' stamp: 'yo 4/4/2005 08:41'!
getterTilesFor: getterSelector type: aType

	| phrase |
	phrase _ super getterTilesFor: getterSelector type: aType.
	phrase replacePlayerInReadoutWith: scriptedPlayer.
	^ phrase.
! !

!KedamaCategoryViewer methodsFor: 'get/set slots' stamp: 'yo 4/3/2005 23:10'!
makeSetter: selectorAndTypePair event: evt from: aMorph 
	"Classic tiles: make a Phrase that comprises a setter of a slot, and hand it to the user."

	| argType m argTile selfTile argValue actualGetter |
	selectorAndTypePair first = #getPatchValueIn: ifTrue: [^ self makeSetterForGetPatchValue: selectorAndTypePair event: evt from: aMorph].
	selectorAndTypePair first = #getRedComponentIn: ifTrue: [^ self makeSetterForColorComponent: selectorAndTypePair componentName: #red event: evt from: aMorph].
	selectorAndTypePair first = #getBlueComponentIn: ifTrue: [^ self makeSetterForColorComponent: selectorAndTypePair componentName: #blue event: evt from: aMorph].
	selectorAndTypePair first = #getGreenComponentIn: ifTrue: [^ self makeSetterForColorComponent: selectorAndTypePair componentName: #green event: evt from: aMorph].
	
	argType := selectorAndTypePair second.
	actualGetter := selectorAndTypePair first asSymbol.
	m := PhraseTileMorph new 
				setAssignmentRoot: (Utilities inherentSelectorForGetter: actualGetter)
				type: #command
				rcvrType: #Player
				argType: argType
				vocabulary: self currentVocabulary.
	argValue := self scriptedPlayer 
				perform: selectorAndTypePair first asSymbol.
	(argValue isKindOf: Player) 
		ifTrue: [argTile := argValue tileReferringToSelf]
		ifFalse: 
			[argTile := ScriptingSystem tileForArgType: argType.
			(argType == #Number and: [argValue isNumber]) 
				ifTrue: 
					[(scriptedPlayer decimalPlacesForGetter: actualGetter) 
						ifNotNilDo: [:places | (argTile findA: UpdatingStringMorph) decimalPlaces: places]].
			argTile
				setLiteral: argValue;
				updateLiteralLabel].
	argTile position: m lastSubmorph position.
	m lastSubmorph addMorph: argTile.
	selfTile := self tileForSelf bePossessive.
	selfTile position: m firstSubmorph position.
	m firstSubmorph addMorph: selfTile.
	m enforceTileColorPolicy.
	m replacePlayerInReadoutWith: scriptedPlayer.
	m openInHand! !

!KedamaCategoryViewer methodsFor: 'support' stamp: 'yo 4/4/2005 14:30'!
replaceTargetWith: player

	self submorphs allButFirst do: [:m |
		(m isMemberOf: ViewerLine) ifFalse: [self halt].
		(#(#systemScript #userScript) includes: m entryType) ifTrue: [
			m replacePlayerWith: player.
		].
		(#(#systemSlot #userSlot) includes: m entryType) ifTrue: [
			m replacePlayerInReadoutWith: player.
		].
	].
! !

CategoryViewer subclass: #KedamaWhoSearchingViewer
	instanceVariableNames: 'searchString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaWhoSearchingViewer methodsFor: 'initialization' stamp: 'yo 4/4/2005 14:33'!
addNamePaneTo: header
	"Add the namePane, which may be a popup or a type-in depending on the type of CategoryViewer"

	| plugTextMor searchButton |
	namePane _ AlignmentMorph newRow vResizing: #spaceFill; height: 14.
	namePane hResizing: #spaceFill.
	namePane listDirection: #leftToRight.

	plugTextMor _ PluggableTextMorph on: self
					text: #searchString accept: #searchString:notifying:
					readSelection: nil menu: nil.
	plugTextMor setProperty: #alwaysAccept toValue: true.
	plugTextMor askBeforeDiscardingEdits: false.
	plugTextMor acceptOnCR: true.
	plugTextMor setTextColor: Color brown.
	plugTextMor setNameTo: 'Turtle' translated.
	plugTextMor vResizing: #spaceFill; hResizing: #spaceFill.
	plugTextMor hideScrollBarsIndefinitely.
	plugTextMor setTextMorphToSelectAllOnMouseEnter.

	searchButton _ SimpleButtonMorph new 
		target: self;
		beTransparent;
		label: 'Turtle' translated;
		actionSelector: #doSearchFrom:;
		arguments: {plugTextMor}.
	searchButton setBalloonText: 'Type a number into the pane to specify the individual turtle.' translated.

	namePane addMorphFront: searchButton.
	namePane addTransparentSpacerOfSize: 6@0.
	namePane addMorphBack: plugTextMor.
	header addMorphBack: namePane! !

!KedamaWhoSearchingViewer methodsFor: 'support' stamp: 'yo 4/3/2005 09:16'!
categoryRestorationInfo
	"Answer info needed to reincarnate myself"

	^ Array with: self chosenCategorySymbol with: self searchString! !

!KedamaWhoSearchingViewer methodsFor: 'categories' stamp: 'yo 4/3/2005 09:16'!
categoryWording: aCategoryWording
	"okay, thanks"! !

!KedamaWhoSearchingViewer methodsFor: 'categories' stamp: 'yo 4/3/2005 09:16'!
currentCategory
	"Answer the symbol associated with the pane"

	^ #search! !

!KedamaWhoSearchingViewer methodsFor: 'search' stamp: 'yo 4/3/2005 09:16'!
doSearchFrom:  aSource
	"Perform the search operation"

	^ self doSearchFrom: aSource interactive: true! !

!KedamaWhoSearchingViewer methodsFor: 'search' stamp: 'yo 4/4/2005 18:03'!
doSearchFrom:  aSource interactive: isInteractive
	"Perform the search operation.  If interactive is true, this actually happened because a search button was pressed; if false, it was triggered some other way for which an informer would be inappropriate."

	| searchFor |
	searchString _ (aSource isKindOf: PluggableTextMorph)
		ifFalse:
			[aSource]
		ifTrue:
			[aSource text string].
	searchFor _ searchString asNumber asInteger.
	(self outerViewer isMemberOf: KedamaStandardViewer) ifTrue: [
		self outerViewer providePossibleRestrictedView: searchFor.
		self updateWhoString.
	].
! !

!KedamaWhoSearchingViewer methodsFor: 'initialization' stamp: 'yo 4/3/2005 09:16'!
establishContents 
	"Perform any initialization steps that needed to wait until I am installed in my outer viewer"

	searchString isEmptyOrNil ifFalse: [self doSearchFrom: searchString interactive: false]! !

!KedamaWhoSearchingViewer methodsFor: 'initialization' stamp: 'sw 8/3/2005 11:16'!
initializeFor: aPlayer categoryChoice: aChoice
	"Initialize the receiver to be associated with the player and category specified"

	super initializeFor: aPlayer categoryChoice: #search.
	self clipSubmorphs: true.
	(namePane findA: PluggableTextMorph) setText: aChoice second asText.
	self setCategorySymbolFrom: aChoice! !

!KedamaWhoSearchingViewer methodsFor: 'header pane' stamp: 'yo 4/3/2005 09:16'!
maybeAddArrowsTo: header
	"Maybe add up/down arrows to the header"

	header addTransparentSpacerOfSize: 5@5! !

!KedamaWhoSearchingViewer methodsFor: 'search' stamp: 'yo 4/3/2005 09:16'!
rawSearchString: aString
	"Set the search string as indicated"

	searchString _ aString asString! !

!KedamaWhoSearchingViewer methodsFor: 'search' stamp: 'yo 4/3/2005 09:16'!
searchString
	"Answer the search string"

	^ searchString ifNil: [searchString _ '']! !

!KedamaWhoSearchingViewer methodsFor: 'search' stamp: 'yo 4/3/2005 09:16'!
searchString: aString notifying: znak
	"Set the search string as indicated and carry out a search"

	searchString _ aString asString.
	self doSearchFrom: searchString! !

!KedamaWhoSearchingViewer methodsFor: 'initialization' stamp: 'yo 4/3/2005 09:16'!
setCategorySymbolFrom: aChoice
	"Set my category symbol"

	self chosenCategorySymbol: #search.
	self rawSearchString: aChoice second! !

!KedamaWhoSearchingViewer methodsFor: 'categories' stamp: 'yo 4/3/2005 09:16'!
updateCategoryNameTo: aName
	"Update the category name, because of a language change."

	self doSearchFrom: (namePane findA: PluggableTextMorph) text interactive: false.
	self flag: #deferred.  "A nice touch would be to change the Button wording here"
! !

!KedamaWhoSearchingViewer methodsFor: 'support' stamp: 'md 3/17/2006 16:58'!
updateWhoString

	(self outerViewer restrictedWho = 0) ifTrue: [
		self rawSearchString: ''.
		self changed: #searchString.
	].
! !

CategoryViewer subclass: #SearchingViewer
	instanceVariableNames: 'searchString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!SearchingViewer commentStamp: 'sw 8/16/2002 22:43' prior: 0!
A SearchingViewer is a custom Viewer which has a type-in 'search' pane; the user types a word or fragment into the search pane and hits the 'search' button (or hits Return or Enter) and the pane gets populated with all the phrases that match (in the currently-installed language) the search-string.!

!SearchingViewer methodsFor: 'initialization' stamp: 'tak 8/8/2005 11:31'!
addNamePaneTo: header
	"Add the namePane, which may be a popup or a type-in depending on the type of CategoryViewer"

	| plugTextMor searchButton |
	namePane _ AlignmentMorph newRow vResizing: #spaceFill; height: 14.
	namePane hResizing: #spaceFill.
	namePane listDirection: #leftToRight.

	plugTextMor _ PluggableTextMorph on: self
					text: #searchString accept: #searchString:notifying:
					readSelection: nil menu: nil.
	plugTextMor setProperty: #alwaysAccept toValue: true.
	plugTextMor askBeforeDiscardingEdits: false.
	plugTextMor acceptOnCR: true.
	plugTextMor setTextColor: Color brown.
	plugTextMor setNameTo: 'Search' translated.
	plugTextMor vResizing: #spaceFill; hResizing: #spaceFill.
	plugTextMor hideScrollBarsIndefinitely.
	plugTextMor setTextMorphToSelectAllOnMouseEnter.

	searchButton _ SimpleButtonMorph new 
		target: self;
		beTransparent;
		actionSelector: #doSearchFrom:;
		arguments: {plugTextMor}.

	namePane addMorphFront: searchButton.
	namePane addTransparentSpacerOfSize: 6@0.
	namePane addMorphBack: plugTextMor.
	header addMorphBack: namePane.
	self updateSearchButtonLabel.! !

!SearchingViewer methodsFor: 'support' stamp: 'sw 8/17/2002 01:13'!
categoryRestorationInfo
	"Answer info needed to reincarnate myself"

	^ Array with: self chosenCategorySymbol with: self searchString! !

!SearchingViewer methodsFor: 'categories' stamp: 'sw 12/11/2001 15:52'!
categoryWording: aCategoryWording
	"okay, thanks"! !

!SearchingViewer methodsFor: 'categories' stamp: 'sw 12/11/2001 19:13'!
currentCategory
	"Answer the symbol associated with the pane"

	^ #search! !

!SearchingViewer methodsFor: 'search' stamp: 'sw 4/10/2003 21:36'!
doSearchFrom:  aSource
	"Perform the search operation"

	^ self doSearchFrom: aSource interactive: true! !

!SearchingViewer methodsFor: 'search' stamp: 'nk 8/29/2004 17:21'!
doSearchFrom:  aSource interactive: isInteractive
	"Perform the search operation.  If interactive is true, this actually happened because a search button was pressed; if false, it was triggered some other way for which an informer would be inappropriate."

	| searchFor aVocab aList all anInterface useTranslations scriptNames addedMorphs |
	searchString _ (aSource isKindOf: PluggableTextMorph)
		ifFalse:
			[aSource]
		ifTrue:
			[aSource text string].
	searchFor _ searchString asString asLowercase withBlanksTrimmed.

	aVocab _ self outerViewer currentVocabulary.
	(useTranslations _ (scriptedPlayer isPlayerLike) and: [aVocab isEToyVocabulary])
		ifTrue:
			[all _ scriptedPlayer costume selectorsForViewer.
			all addAll: (scriptNames _ scriptedPlayer class namedTileScriptSelectors)]
		ifFalse:
			[all _ scriptNames _ scriptedPlayer class allSelectors].
	aList _ all select:
		[:aSelector | (aVocab includesSelector: aSelector forInstance: scriptedPlayer ofClass: scriptedPlayer class limitClass: ProtoObject) and:
			[(useTranslations and: [(anInterface _ aVocab methodInterfaceAt: aSelector ifAbsent: [nil]) notNil and: [anInterface wording includesSubstring: searchFor caseSensitive: false]])
				or:
					[((scriptNames includes: aSelector) or: [useTranslations not]) and:
						[aSelector includesSubstring: searchFor caseSensitive: false]]]].
	aList _ aList asSortedArray.

	self removeAllButFirstSubmorph. "that being the header"
	self addAllMorphs:
		((addedMorphs _ scriptedPlayer tilePhrasesForSelectorList: aList inViewer: self)).
	self enforceTileColorPolicy.
	self secreteCategorySymbol.
	self world ifNotNil: [self world startSteppingSubmorphsOf: self].
	self adjustColorsAndBordersWithin.

	owner ifNotNil: [owner isStandardViewer ifTrue: [owner fitFlap].

	(isInteractive and: [addedMorphs isEmpty]) ifTrue:
		[self inform: ('No matches found for "' translated), searchFor, '"']]! !

!SearchingViewer methodsFor: 'initialization' stamp: 'sw 10/15/2004 02:34'!
establishContents 
	"Perform any initialization steps that needed to wait until I am installed in my outer viewer"

	searchString isEmptyOrNil ifFalse: [self doSearchFrom: searchString interactive: false]! !

!SearchingViewer methodsFor: 'initialization' stamp: 'sw 8/3/2005 11:16'!
initializeFor: aPlayer categoryChoice: aChoice
	"Initialize the receiver to be associated with the player and category specified"

	super initializeFor: aPlayer categoryChoice: #search.
	self clipSubmorphs: true.
	(namePane findA: PluggableTextMorph) setText: aChoice second asText.
	self setCategorySymbolFrom: aChoice! !

!SearchingViewer methodsFor: 'header pane' stamp: 'sw 12/11/2001 19:12'!
maybeAddArrowsTo: header
	"Maybe add up/down arrows to the header"

	header addTransparentSpacerOfSize: 5@5! !

!SearchingViewer methodsFor: 'search' stamp: 'sw 8/22/2002 15:14'!
rawSearchString: aString
	"Set the search string as indicated"

	searchString _ aString asString! !

!SearchingViewer methodsFor: 'search' stamp: 'sw 8/11/2002 02:14'!
searchString
	"Answer the search string"

	^ searchString ifNil: [searchString _ '']! !

!SearchingViewer methodsFor: 'search' stamp: 'sw 8/22/2002 20:30'!
searchString: aString notifying: znak
	"Set the search string as indicated and carry out a search"

	searchString _ aString asString.
	self doSearchFrom: searchString! !

!SearchingViewer methodsFor: 'initialization' stamp: 'sw 8/22/2002 23:09'!
setCategorySymbolFrom: aChoice
	"Set my category symbol"

	self chosenCategorySymbol: #search.
	self rawSearchString: aChoice second! !

!SearchingViewer methodsFor: 'categories' stamp: 'tak 8/8/2005 11:31'!
updateCategoryNameTo: aName
	"Update the category name, because of a language change."

	self doSearchFrom: (namePane findA: PluggableTextMorph) text interactive: false.
	self updateSearchButtonLabel
! !

!SearchingViewer methodsFor: 'categories' stamp: 'tak 8/8/2005 11:30'!
updateSearchButtonLabel
	| button |
	button := self
				findDeepSubmorphThat: [:e | e class = SimpleButtonMorph]
				ifAbsent: [].
	button label: 'Search' translated.
	button setBalloonText: 'Type some letters into the pane at right, and then press this Search button (or hit RETURN) and all method selectors that match what you typed will appear in the list below.' translated! !

Viewer subclass: #StandardViewer
	instanceVariableNames: 'firstPanel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!StandardViewer commentStamp: 'sw 8/17/2002 02:04' prior: 0!
A structure that allows you to view state and behavior of an object; it consists of a header and then any number of CategoryViewers.!

StandardViewer subclass: #KedamaStandardViewer
	instanceVariableNames: 'stub restrictedIndex restrictedWho'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaStandardViewer methodsFor: 'categories' stamp: 'yo 4/4/2005 15:19'!
categoryViewerFor: categoryInfo 
	"Answer a category viewer for the given category info"

	| aViewer |
	aViewer := ((categoryInfo isCollection) 
				and: [categoryInfo first == #search]) 
					ifFalse: [KedamaCategoryViewer new]
					ifTrue: [KedamaWhoSearchingViewer new].
	aViewer initializeFor: scriptedPlayer categoryChoice: categoryInfo.
	(aViewer isMemberOf: KedamaWhoSearchingViewer) ifTrue: [^ aViewer].
	restrictedWho = 0
		ifTrue: [aViewer replaceTargetWith: scriptedPlayer]
		ifFalse: [aViewer replaceTargetWith: stub].
	^aViewer.
! !

!KedamaStandardViewer methodsFor: 'initialization' stamp: 'yo 4/4/2005 18:14'!
initializeFor: aPlayer barHeight: anInteger includeDismissButton: aBoolean showCategories: categoryInfo

	stub _ aPlayer clonedSequentialStub.
	stub who: 0.
	restrictedWho _ 0.
	restrictedIndex _ 0.
	super initializeFor: aPlayer barHeight: anInteger includeDismissButton: aBoolean showCategories: categoryInfo.
! !

!KedamaStandardViewer methodsFor: 'turtle view' stamp: 'yo 4/4/2005 18:05'!
providePossibleRestrictedView: anInteger

	| who |
	restrictedIndex _ anInteger.
	who _ scriptedPlayer whoAt: anInteger.
	restrictedWho = who ifTrue: [^ self].
	restrictedWho _ who.
	stub who: who.
	who = 0 ifTrue: [self replaceTargetsWithExampler] ifFalse: [self replaceTargetsWithStub].
	self searchingViewerMorphs do: [:v | v updateWhoString].
! !

!KedamaStandardViewer methodsFor: 'turtle view' stamp: 'yo 4/4/2005 10:53'!
replaceTargetsWithExampler

	self submorphsDo: [:e |
		(e isMemberOf: KedamaCategoryViewer) ifTrue: [
			e replaceTargetWith: scriptedPlayer.
		].
	].
! !

!KedamaStandardViewer methodsFor: 'turtle view' stamp: 'yo 4/3/2005 17:25'!
replaceTargetsWithStub

	self submorphsDo: [:e |
		(e isMemberOf: KedamaCategoryViewer) ifTrue: [
			e replaceTargetWith: stub.
		].
	].
! !

!KedamaStandardViewer methodsFor: 'turtle view' stamp: 'yo 4/4/2005 17:48'!
resetWhoIfNecessary

	restrictedWho <= 0 ifTrue: [^ self].
	self providePossibleRestrictedView: 0.
! !

!KedamaStandardViewer methodsFor: 'turtle view' stamp: 'yo 4/4/2005 18:15'!
restrictedIndex

	^ restrictedIndex.
! !

!KedamaStandardViewer methodsFor: 'turtle view' stamp: 'yo 4/4/2005 18:01'!
restrictedWho

	^ restrictedWho.
! !

!KedamaStandardViewer methodsFor: 'turtle view' stamp: 'yo 4/4/2005 18:00'!
searchingViewerMorphs

	^ self submorphsSatisfying: [:m | m isKindOf: KedamaWhoSearchingViewer].! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 12/28/1998 15:22'!
addCategoryViewer	
	self addCategoryViewerFor: self likelyCategoryToShow! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 8/23/2002 14:50'!
addCategoryViewerFor: categoryInfo
	"Add a category viewer for the given category info"

	self addCategoryViewerFor: categoryInfo atEnd: true! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 8/23/2002 14:56'!
addCategoryViewerFor: categoryInfo atEnd: atEnd
	"Add a category viewer for the given category info.  If atEnd is true, add it at the end, else add it just after the header morph"

	| aViewer |
	aViewer _ self categoryViewerFor: categoryInfo.
	atEnd
		ifTrue:
			[self addMorphBack: aViewer]
		ifFalse:
			[self addMorph: aViewer after: submorphs first].
	aViewer establishContents.
	self world ifNotNil: [self world startSteppingSubmorphsOf: aViewer].
	self fitFlap! !

!StandardViewer methodsFor: 'initialization' stamp: 'nk 9/2/2004 11:30'!
addHeaderMorphWithBarHeight: anInteger includeDismissButton: aBoolean
	"Add the header morph to the receiver, using anInteger as a guide for its height, and if aBoolean is true, include a dismiss buton for it"

	| header aFont aButton aTextMorph nail wrpr costs headWrapper |
	header _ AlignmentMorph newRow color: self color muchLighter; wrapCentering: #center; cellPositioning: #leftCenter.
	aFont _ Preferences standardButtonFont.
	aBoolean ifTrue:
		[aButton _ self tanOButton.
		header addMorph: aButton.
		aButton target: self;
				actionSelector: #dismiss;
				setBalloonText: 'remove this entire Viewer from the screen
don''t worry -- nothing will be lost!!.' translated.
		header addTransparentSpacerOfSize: 4@1].

	aButton _ IconicButton new borderWidth: 0;
			labelGraphic: (ScriptingSystem formAtKey: #AddCategoryViewer); color: Color transparent; 
			actWhen: #buttonDown;
			target: self;
			actionSelector: #addCategoryViewer;
			setBalloonText: 'click here to add
another category pane' translated;
			shedSelvedge.
	header addMorphBack: aButton.
	header addTransparentSpacerOfSize: 4@1.

	costs _ scriptedPlayer costumes.
	costs ifNotNil:
	[(costs size > 1 or: [costs size = 1 and: [costs first ~~ scriptedPlayer costume]]) ifTrue:
		[header addUpDownArrowsFor: self.
		(wrpr _ header submorphs last) submorphs second setBalloonText: 'switch to previous costume' translated.	
		wrpr submorphs first  setBalloonText: 'switch to next costume' translated]].	

	nail _ (self hasProperty: #noInteriorThumbnail)
		ifFalse:
			[ThumbnailMorph new objectToView: scriptedPlayer viewSelector: #costume]
		ifTrue:
			[ImageMorph new image: Cursor menu].
	nail on: #mouseDown send: #offerViewerMenuForEvt:morph: to: scriptedPlayer.
	header addMorphBack: nail.
	nail setBalloonText: 'click here to get a menu
that will allow you to
add a variable,
tear off a tile, etc..' translated.
	(self hasProperty: #noInteriorThumbnail)
		ifFalse:
			[nail borderWidth: 3; borderColor: #raised].

	header addTransparentSpacerOfSize: 5@5.

"	aButton _ SimpleButtonMorph new target: self; actionSelector: #newEmptyScript; label: 'S' translated font: (aFont _ StrikeFont familyName: #ComicBold size: 16);  color: Color transparent; borderWidth: 0; actWhen: #buttonDown.
	aButton setBalloonText: 'drag from here to
create a new script
for this object' translated.	
	header addMorphBack: aButton.

	header addTransparentSpacerOfSize: 8@5."
	
	aButton _ SimpleButtonMorph new target: scriptedPlayer; actionSelector: #addInstanceVariable; label: 'v' translated font: (aFont emphasized: 1);  color: Color transparent; borderWidth: 1; actWhen: #buttonUp.
	"aButton firstSubmorph color: Color gray."
	aButton setBalloonText: 'click here to add a variable
to this object.' translated.
	header addMorphBack: aButton.

	header addTransparentSpacerOfSize: 5@5.
	self viewsMorph ifTrue: [scriptedPlayer costume assureExternalName].
	aTextMorph _ UpdatingStringMorph new
		useStringFormat;
		target:  scriptedPlayer;
		getSelector: #nameForViewer;
		setNameTo: 'name';
		font: ScriptingSystem fontForNameEditingInScriptor.
	self viewsMorph ifTrue:
		[aTextMorph putSelector: #setName:.
		aTextMorph setProperty: #okToTextEdit toValue: true].
	aTextMorph step.
	header  addMorphBack: aTextMorph.
	aTextMorph setBalloonText: 'Click here to edit the player''s name.' translated.	

	header beSticky.
	anInteger > 0
		ifTrue:
			[headWrapper _ AlignmentMorph newColumn color: self color.
			headWrapper addTransparentSpacerOfSize: (0 @ anInteger).
			headWrapper addMorphBack: header.
			self addMorph: headWrapper]
		ifFalse:
			[self addMorph: header]! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 8/23/2002 14:18'!
addSearchPane
	"Add a search pane"

	self addCategoryViewerFor: #(search '') atEnd: false! !

!StandardViewer methodsFor: 'initialization' stamp: 'gm 2/22/2003 13:44'!
affordsUniclass
	"Answer true iff the receiver operates on behalf of an object that is, or could become, a member of a Uniclass"

	| viewee |
	^(viewee := self objectViewed) belongsToUniClass or: 
			[((viewee isInteger) not and: [viewee isBehavior not]) 
				and: [self userLevel > 0]]! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 11/18/1999 16:00'!
categoriesCurrentlyShowing
	^ self categoryMorphs collect: [:m | m currentCategory]! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 10/30/1998 18:38'!
categoryMorphs
	^ self submorphsSatisfying: [:m | m isKindOf: CategoryViewer]! !

!StandardViewer methodsFor: 'categories' stamp: 'gm 2/22/2003 13:01'!
categoryViewerFor: categoryInfo 
	"Answer a category viewer for the given category info"

	| aViewer |
	aViewer := ((categoryInfo isCollection) 
				and: [categoryInfo first == #search]) 
					ifFalse: [CategoryViewer new]
					ifTrue: [SearchingViewer new].
	aViewer initializeFor: scriptedPlayer categoryChoice: categoryInfo.
	^aViewer! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 8/3/2001 18:22'!
chooseLimitClass
	"Put up a menu allowing the user to choose the most generic class to show"

	| aMenu limitClass |
	aMenu _ MenuMorph new defaultTarget: self.
	limitClass _ self limitClass.
	scriptedPlayer class withAllSuperclasses do:
		[:aClass | 
			aClass == ProtoObject
				ifTrue:
					[aMenu addLine].
			aMenu add: aClass name selector: #setLimitClass: argument: aClass.
			aClass == limitClass ifTrue:
				[aMenu lastItem color: Color red].
			aClass == limitClass ifTrue: [aMenu addLine]].
	aMenu addTitle: 'Show only methods
implemented at or above...'.  "heh heh -- somebody please find nice wording here!!"
	aMenu popUpInWorld: self currentWorld! !

!StandardViewer methodsFor: 'macpal' stamp: 'sw 6/4/2001 18:05'!
currentVocabulary
	"Answer the vocabulary currently associated with the receiver"

	| aSym aVocab |
	aSym _ self valueOfProperty: #currentVocabularySymbol ifAbsent: [nil].
	aSym ifNil:
		[aVocab _ self valueOfProperty: #currentVocabulary ifAbsent: [nil].
		aVocab ifNotNil:
			[aSym _ aVocab vocabularyName.
			self removeProperty: #currentVocabulary.
			self setProperty: #currentVocabularySymbol toValue: aSym]].
	^ aSym
		ifNotNil:
			[Vocabulary vocabularyNamed: aSym]
		ifNil:
			[(self world ifNil: [ActiveWorld]) currentVocabularyFor: scriptedPlayer]! !

!StandardViewer methodsFor: 'user interface' stamp: 'sw 10/25/1999 22:08'!
dismiss
	| aFlapTab |
	"User hit the dismiss button."
	(owner isKindOf: TabbedPalette)
		ifTrue:
			[^ owner showNoPalette].
	(aFlapTab _ self pasteUpMorph correspondingFlapTab) ifNotNil:
		[^ aFlapTab dismissViaHalo].
	self topRendererOrSelf delete! !

!StandardViewer methodsFor: 'user interface' stamp: 'sw 10/24/1998 14:34'!
downArrowHit
	self nextCostume! !

!StandardViewer methodsFor: 'initialization' stamp: 'ar 6/30/2001 13:23'!
fitFlap
	(owner notNil and:[owner isFlap]) ifTrue:[
		owner width < self fullBounds width ifTrue:[
			owner assureFlapWidth: self fullBounds width + 25.
		].
	].! !

!StandardViewer methodsFor: 'user interface' stamp: 'sw 10/26/1999 01:07'!
hasDismissButton
	submorphs isEmptyOrNil ifTrue: [^ false].
	^ (submorphs first allMorphs detect:
		[:possible |  (possible isKindOf: SimpleButtonMorph) and: [possible actionSelector == #dismiss]]
			ifNone: [nil]) notNil! !

!StandardViewer methodsFor: 'initialization' stamp: 'sw 12/23/1998 23:26'!
initialHeightToAllow
	^ 300! !

!StandardViewer methodsFor: 'initialization' stamp: 'sw 6/25/1999 23:04'!
initializeFor: aPlayer barHeight: anInteger
	^ self initializeFor: aPlayer barHeight: anInteger includeDismissButton: true! !

!StandardViewer methodsFor: 'initialization' stamp: 'sw 8/4/2000 13:02'!
initializeFor: aPlayer barHeight: anInteger includeDismissButton: aBoolean
	self initializeFor: aPlayer barHeight: anInteger includeDismissButton: aBoolean showCategories: nil! !

!StandardViewer methodsFor: 'initialization' stamp: 'yo 8/18/2005 13:43'!
initializeFor: aPlayer barHeight: anInteger includeDismissButton: aBoolean showCategories: categoryInfo
	"Initialize the receiver to be a look inside the given Player.  The categoryInfo, if present, describes which categories should be present in it, in which order"

	scriptedPlayer _ aPlayer.
	self listDirection: #topToBottom;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		borderWidth: 1.
	self color: self standardViewerColor.
	self addHeaderMorphWithBarHeight: anInteger includeDismissButton: aBoolean.

	categoryInfo isEmptyOrNil
		ifFalse:  "Reincarnating an pre-existing list"
			[categoryInfo do:
				[:aCat | self addCategoryViewerFor: aCat]]
		ifTrue:  "starting fresh"
			[self addSearchPane. 
			self addCategoryViewer.
			self addCategoryViewer.
			(scriptedPlayer costume isMemberOf: KedamaMorph) ifTrue: [self addCategoryViewer].
		].! !

!StandardViewer methodsFor: 'classification' stamp: 'ar 6/30/2001 13:13'!
isStandardViewer
	^true! !

!StandardViewer methodsFor: 'categories' stamp: 'ka 11/29/2005 20:53'!
likelyCategoryToShow
	"Choose a category to show based on what's already showing and on some predefined heuristics"

	| possible all aCat currVocab |
	all := (scriptedPlayer categoriesForViewer: self) asOrderedCollection.
	possible _ all copy.

	currVocab := self currentVocabulary.
	self categoryMorphs do: 
			[:m | 
			aCat := currVocab categoryWhoseTranslatedWordingIs: m currentCategory.
			aCat ifNotNil: [possible remove: aCat wording ifAbsent: []]].

	(possible includes: ScriptingSystem nameForInstanceVariablesCategory translated) ifTrue:
		[^ ScriptingSystem nameForInstanceVariablesCategory].

	(currVocab isEToyVocabulary) 
		ifTrue: 
			[(possible includes: ScriptingSystem nameForScriptsCategory translated) 
				ifTrue: [^ ScriptingSystem nameForScriptsCategory]].
	{'kedama' translated. #basic translated} 
		do: [:preferred | (possible includes: preferred) ifTrue: [^ preferred]].
	((scriptedPlayer isPlayerLike) 
		and: [scriptedPlayer hasOnlySketchCostumes]) 
			ifTrue: [(possible includes: #tests translated) ifTrue: [^#tests translated]].
	{#'color & border' translated. #tests translated. #color translated. #flagging translated. #comparing translated.} 
		do: [:preferred | (possible includes: preferred) ifTrue: [^ preferred]].
	^ possible isEmpty ifFalse: [possible first] ifTrue: [all first]! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 8/3/2001 18:17'!
limitClass
	"Answer the limit class to use in this viewer"

	| aClass |
	(aClass _ self valueOfProperty: #limitClass)  ifNotNil:
		[^ aClass].

	aClass _ scriptedPlayer defaultLimitClassForVocabulary: self currentVocabulary.
	self setProperty: #limitClass toValue: aClass.
	^ aClass! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 8/3/2001 18:31'!
limitClass: aClass
	"Set aClass as the limit class for this viewer, without side effects"

	self setProperty: #limitClass toValue: aClass
! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 12/11/2000 10:51'!
outerViewer
	"Answer the StandardViewer or equivalent that contains this object"

	^ self! !

!StandardViewer methodsFor: 'initialization' stamp: 'sw 6/4/2001 18:06'!
rawVocabulary: aVocabulary
	"Mark the receiver as having aVocabulary as its vocabulary"

	self setProperty: #currentVocabularySymbol toValue: aVocabulary vocabularyName! !

!StandardViewer methodsFor: 'categories' stamp: 'di 2/19/2001 10:39'!
recreateCategories
	"To change from old to new tiles"
	| cats |
	cats _ self categoriesCurrentlyShowing.
	self removeAllMorphsIn: self categoryMorphs.
	cats do: [:cat | self addCategoryViewerFor: cat]! !

!StandardViewer methodsFor: 'user interface' stamp: 'sw 4/20/2001 21:22'!
relaunchViewer
	"Launch a new viewer to replace the receiver."

	self presenter updateViewer: self forceToShow: nil! !

!StandardViewer methodsFor: 'categories' stamp: 'yo 4/4/2005 14:42'!
searchingViewerMorphs
	^ self submorphsSatisfying: [:m | m isKindOf: SearchingViewer].! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 8/3/2001 18:31'!
setLimitClass: aClass
	"Set aClass as the limit class for this viewer"

	self limitClass: aClass.
	self relaunchViewer
! !

!StandardViewer methodsFor: 'user interface' stamp: 'sw 12/23/1998 22:45'!
standardViewerColor
	 ^ Color r: 0.572 g: 0.883 b: 0.572! !

!StandardViewer methodsFor: 'initialization' stamp: 'nk 8/29/2004 17:18'!
switchToVocabulary: aVocabulary
	"Make the receiver show categories and methods as dictated by aVocabulary.  If this constitutes a switch, then wipe out existing category viewers, which may be showing the wrong thing."

	self adoptVocabulary: aVocabulary.  "for benefit of submorphs"
	self setProperty: #currentVocabularySymbol toValue: aVocabulary vocabularyName.
	((scriptedPlayer isPlayerLike) and: [self isUniversalTiles not]) ifTrue:
		[scriptedPlayer allScriptEditors do:
			[:aScriptEditor |
				aScriptEditor adoptVocabulary: aVocabulary]]! !

!StandardViewer methodsFor: 'categories' stamp: 'sw 5/29/2001 22:43'!
symbolsOfCategoriesCurrentlyShowing
	"Answer the category symbols of my categoryMorphs"

	^ self categoryMorphs collect: [:m | m chosenCategorySymbol]! !

!StandardViewer methodsFor: 'user interface' stamp: 'sw 10/24/1998 14:33'!
upArrowHit
	self previousCostume! !

!StandardViewer methodsFor: 'initialization' stamp: 'sw 6/4/2001 19:40'!
useVocabulary: aVocabulary
	"Make the receiver show categories and methods as dictated by aVocabulary"

	| itsName |
	((self valueOfProperty: #currentVocabularySymbol ifAbsent: [nil]) == (itsName _ aVocabulary vocabularyName)) ifFalse:
		[self setProperty: #currentVocabularySymbol toValue: itsName.
		self removeProperty: #currentVocabulary.  "grandfathered"
		(self submorphs select: [:m | m isKindOf: CategoryViewer]) do: [:m | m delete]]! !

!StandardViewer methodsFor: 'initialization' stamp: 'sw 10/26/2000 09:42'!
userLevel
	"Answer the user level for this viewer, which can be used in figuring out what to display in the viewer.  Initially, we make little use of this, but in past prototypes, and in future deployments, it may be handy."

	^ self valueOfProperty: #userLevel ifAbsent: [1]! !

!StandardViewer methodsFor: 'debug and other' stamp: 'sw 6/20/2001 12:47'!
viewMorphDirectly
	"Launch a new viewer to replace the receiver."

	self delete.
	self presenter viewObjectDirectly: scriptedPlayer costume renderedMorph! !

!StandardViewer methodsFor: 'initialization' stamp: 'nk 8/29/2004 17:18'!
viewsMorph
	"Answer whether the receiver views a morph.  Traditional viewers up until late 2000 *all* viewed morphs (as per the morph/player architecture), but viewers on non-morph/players have now become possible"

	^ scriptedPlayer isPlayerLike! !

!Viewer methodsFor: 'queries' stamp: ''!
aboutMethod: aSelector 
	"Give the user documentation for the selector"
	| aClass help |
	aClass _ scriptedPlayer class whichClassIncludesSelector: aSelector.
	help _ aClass firstCommentAt: aSelector.
	help
		ifNotNil: [self
				inform: (help string withNoLineLongerThan: 25)]! !

!Viewer methodsFor: 'special phrases' stamp: 'yo 6/18/2004 16:01'!
angleToPhrase

	| outerPhrase getTile |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Number
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	(outerPhrase submorphs second) delete.	"color"
	getTile _ KedamaAngleToTile new.
	outerPhrase addMorphBack: getTile.
	^outerPhrase! !

!Viewer methodsFor: 'special phrases' stamp: 'yo 9/28/2004 18:24'!
bounceOnColorPhrase

	| outerPhrase bounceOn |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Boolean
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	(outerPhrase submorphs second) delete.	"color"
	bounceOn _ KedamaBounceOnColorTile new.
	"upHill setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer)."
	outerPhrase addMorphBack: bounceOn.
	^outerPhrase.
! !

!Viewer methodsFor: 'special phrases' stamp: 'yo 9/27/2004 15:07'!
bounceOnPhrase

	| outerPhrase upHill |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Boolean
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	(outerPhrase submorphs second) delete.	"color"
	upHill _ KedamaBounceOnTile new.
	"upHill setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer)."
	outerPhrase addMorphBack: upHill.
	^outerPhrase.
! !

!Viewer methodsFor: 'queries' stamp: 'sd 4/16/2003 08:44'!
browseImplementorsOf: aSelector
	"Open a browser on implementors of aSelector"

	self systemNavigation browseAllImplementorsOf: aSelector

! !

!Viewer methodsFor: 'queries' stamp: ''!
browseMethodFull: aSelector 
	"Open a traditional browser on aSelector, in whatever class implements 
	aSelector "
	| aClass |
	aClass _ scriptedPlayer class whichClassIncludesSelector: aSelector.
	Browser fullOnClass: aClass selector: aSelector! !

!Viewer methodsFor: 'queries' stamp: 'sd 1/16/2004 21:10'!
browseMethodInheritance: aSelector 
	"Open an inheritance browser on aSelector"
	| aClass |
	aClass _ scriptedPlayer class whichClassIncludesSelector: aSelector.
	self systemNavigation methodHierarchyBrowserForClass: aClass selector: aSelector! !

!Viewer methodsFor: 'queries' stamp: 'dvf 8/23/2003 11:51'!
browseSendersOf: aSelector 
	"Open a browser on senders of aSelector"

	self systemNavigation browseAllCallsOn: aSelector! !

!Viewer methodsFor: 'queries' stamp: ''!
browseVersionsOf: aSelector 
	"Open a browser on versions of aSelector"
	| aClass |
	aClass _ scriptedPlayer class whichClassIncludesSelector: aSelector.
	Utilities browseVersionsForClass: aClass selector: aSelector! !

!Viewer methodsFor: 'commands' stamp: 'yo 1/14/2005 19:57'!
chooseVocabulary
	"Put up a menu allowing the user to specify which protocol to use in this viewer"

	| aMenu |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Choose a vocabulary' translated.
	"aMenu addStayUpItem."  "For debugging only"
	Vocabulary allStandardVocabularies do:
		[:aVocabulary |
			(scriptedPlayer class implementsVocabulary: aVocabulary)
				ifTrue:
					[aMenu add: aVocabulary vocabularyName selector: #switchToVocabulary: argument: aVocabulary.
					aVocabulary == self currentVocabulary ifTrue:
						[aMenu lastItem color: Color blue]. 
					aMenu balloonTextForLastItem: aVocabulary documentation]].
	aMenu popUpInWorld: self currentWorld! !

!Viewer methodsFor: 'special phrases' stamp: 'yo 10/12/2004 22:56'!
colorComponentPhraseFor: componentName

	| outerPhrase getTile |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Number
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	(outerPhrase submorphs second) delete.	"color"
	getTile _ KedamaGetColorComponentTile new.
	getTile componentName: componentName.
	getTile setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer).
	outerPhrase addMorphBack: getTile.
	^outerPhrase! !

!Viewer methodsFor: 'special phrases' stamp: 'dgd 2/22/2003 19:02'!
colorSeesPhrase
	"In classic tiles, answer a complete phrase that represents the colorSees test"

	| outerPhrase |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Boolean
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	"Install (ColorSeerTile new) in middle position"
	(outerPhrase submorphs second) delete.	"operator"
	outerPhrase addMorphBack: ColorSeerTile new.
	(outerPhrase submorphs second) goBehind.	"Make it third"
	outerPhrase submorphs last addMorph: (ColorTileMorph new 
				typeColor: (ScriptingSystem colorForType: #Color)).
	^outerPhrase! !

!Viewer methodsFor: 'special phrases' stamp: 'yo 6/18/2004 16:01'!
distanceToPhrase

	| outerPhrase getTile |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Number
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	(outerPhrase submorphs second) delete.	"color"
	getTile _ KedamaDistanceToTile new.
	outerPhrase addMorphBack: getTile.
	^outerPhrase! !

!Viewer methodsFor: 'initialization' stamp: 'ar 11/9/2000 22:00'!
initialize
	super initialize.
	self useRoundedCorners.
	self hResizing: #shrinkWrap; vResizing: #shrinkWrap.! !

!Viewer methodsFor: 'e-toy support' stamp: 'sw 10/21/1998 15:55'!
isAViewer
	^ true! !

!Viewer methodsFor: 'e-toy support' stamp: 'sw 7/6/1999 08:43'!
isCandidateForAutomaticViewing
	^ false! !

!Viewer methodsFor: 'commands' stamp: 'sw 1/26/2001 15:26'!
newPermanentScript
	"Create a new, empty script and attach it to the hand"

	| aMorph |
	self scriptedPlayer assureUniClass.
	aMorph _ ImageMorph new image: (ScriptingSystem formAtKey: 'newScript').
	aMorph setProperty: #newPermanentScript toValue: true.
	aMorph setProperty: #newPermanentPlayer toValue: self scriptedPlayer.
	self primaryHand attachMorph: aMorph! !

!Viewer methodsFor: 'commands' stamp: 'nb 6/17/2003 12:25'!
nextCostume
	| aList aPlayer itsCurrent anIndex newIndex |
	aList _ (aPlayer _ scriptedPlayer) availableCostumesForArrows.
	aList isEmptyOrNil ifTrue: [^ Beeper beep].
	itsCurrent _ aPlayer costume renderedMorph.
	anIndex _ aList indexOf: itsCurrent ifAbsent: [nil].
	newIndex _ anIndex
		ifNil:		[1]
		ifNotNil:	[anIndex + 1].
	newIndex > aList size ifTrue: [newIndex _ 1].
	aPlayer renderedCostume: (aList at: newIndex).
	self presenter ifNotNil: [self presenter updateViewer: self]! !

!Viewer methodsFor: 'transition' stamp: 'sw 11/5/1998 15:48'!
noteDeletionOf: aMorph
	| pal |
	aMorph player == scriptedPlayer ifTrue:
		[(pal _ self standardPalette)
			ifNotNil: [pal showNoPalette]
			ifNil:	[self delete]]   "Viewer on a discarded player"! !

!Viewer methodsFor: 'e-toy support' stamp: 'nk 8/29/2004 17:18'!
objectViewed
	"Answer the graphical object to which the receiver's phrases apply"

	^ (scriptedPlayer isPlayerLike)
		ifTrue:
			[scriptedPlayer costume]
		ifFalse:
			[scriptedPlayer]! !

!Viewer methodsFor: 'special phrases' stamp: 'nk 10/14/2004 10:59'!
overlapsAnyPhrase
	"Answer a conjured-up overlaps phrase in classic tile"

	| outerPhrase |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Boolean
				rcvrType: #Player
				argType: #Player.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	outerPhrase addMorphBack: (TileMorph new setOperator: #overlapsAny:).
	(outerPhrase submorphs second) goBehind.	"Make it third"
	outerPhrase submorphs last addMorph: scriptedPlayer tileToRefer.
	^outerPhrase! !

!Viewer methodsFor: 'special phrases' stamp: 'dgd 2/22/2003 19:02'!
overlapsPhrase
	"Answer a conjured-up overlaps phrase in classic tile"

	| outerPhrase |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Boolean
				rcvrType: #Player
				argType: #Player.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	outerPhrase addMorphBack: (TileMorph new setOperator: #overlaps:).
	(outerPhrase submorphs second) goBehind.	"Make it third"
	outerPhrase submorphs last addMorph: scriptedPlayer tileToRefer.
	^outerPhrase! !

!Viewer methodsFor: 'special phrases' stamp: 'yo 6/18/2004 16:02'!
patchUphillPhrase

	| outerPhrase upHill |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Number
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	(outerPhrase submorphs second) delete.	"color"
	upHill _ KedamaUpHillTile new.
	upHill setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer).
	outerPhrase addMorphBack: upHill.
	^outerPhrase.
! !

!Viewer methodsFor: 'special phrases' stamp: 'yo 6/18/2004 16:50'!
patchValuePhrase

	| outerPhrase getTile |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Number
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	(outerPhrase submorphs second) delete.	"color"
	getTile _ KedamaGetPixelValueTile new.
	getTile setPatchDefaultTo: (scriptedPlayer defaultPatchPlayer).
	outerPhrase addMorphBack: getTile.
	^outerPhrase! !

!Viewer methodsFor: 'commands' stamp: 'nb 6/17/2003 12:25'!
previousCostume
	| aList aPlayer itsCurrent anIndex newIndex |
	aList _ (aPlayer _ scriptedPlayer) availableCostumesForArrows.
	aList isEmptyOrNil ifTrue: [^ Beeper beep].
	itsCurrent _ aPlayer costume renderedMorph.
	anIndex _ aList indexOf: itsCurrent ifAbsent: [nil].
	newIndex _ anIndex
		ifNil:		[aList size]
		ifNotNil:	[anIndex - 1].
	newIndex < 1 ifTrue: [newIndex _ aList size].
	aPlayer renderedCostume: (aList at: newIndex).
	self presenter ifNotNil: [self presenter updateViewer: self]! !

!Viewer methodsFor: 'dropping/grabbing' stamp: 'ar 2/13/2001 19:58'!
repelsMorph: aMorph event: ev
	"viewers in flaps are resistant to drop gestures"
	owner isFlap ifTrue:[^true].
	^false! !

!Viewer methodsFor: 'transition' stamp: 'sw 10/6/2000 22:41'!
retargetFrom: oldPlayer to: newPlayer
	"Retarget the receiver so that instead of viewing oldPlayer, it now views newPlayer, which are expected to be different instances of the same uniclass"

	scriptedPlayer == oldPlayer
		ifTrue:
			[self allMorphs do:  "nightmarishly special-cased, sorry"
				[:aMorph | 
					(aMorph isKindOf: Viewer) ifTrue:  "includes self"
						[aMorph scriptedPlayer: newPlayer].
					((aMorph isKindOf: UpdatingStringMorph) and: [aMorph target == oldPlayer]) ifTrue:
						[aMorph target: newPlayer].
					(aMorph isKindOf: TileMorph) ifTrue:
						[aMorph retargetFrom: oldPlayer to: newPlayer]]]! !

!Viewer methodsFor: 'access' stamp: 'sw 10/21/1998 15:56'!
scriptedPlayer
	^ scriptedPlayer! !

!Viewer methodsFor: 'access' stamp: 'sw 10/20/1998 13:20'!
scriptedPlayer: aPlayer
	scriptedPlayer _ aPlayer! !

!Viewer methodsFor: 'special phrases' stamp: 'dgd 2/22/2003 19:02'!
seesColorPhrase
	"In classic tiles, answer a complete phrase that represents the seesColor test"

	| outerPhrase seesColorTile |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Boolean
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	"Install (ColorSeerTile new) in middle position"
	(outerPhrase submorphs second) delete.	"operator"
	seesColorTile := TileMorph new setOperator: #seesColor:.
	outerPhrase addMorphBack: seesColorTile.
	(outerPhrase submorphs second) goBehind.	"Make it third"
	"	selfTile _ self tileForSelf bePossessive.	Done by caller.
	selfTile position: 1.
	outerPhrase firstSubmorph addMorph: selfTile.
"
	outerPhrase submorphs last addMorph: (ColorTileMorph new 
				typeColor: (ScriptingSystem colorForType: #Color)).
	^outerPhrase! !

!Viewer methodsFor: 'special phrases' stamp: 'dgd 2/22/2003 19:02'!
touchesAPhrase
	"Answer a conjured-up touchesA phrase in classic tile"

	| outerPhrase |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Boolean
				rcvrType: #Player
				argType: #Player.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	outerPhrase addMorphBack: (TileMorph new setOperator: #touchesA:).
	(outerPhrase submorphs second) goBehind.	"Make it third"
	outerPhrase submorphs last addMorph: scriptedPlayer tileToRefer.
	^outerPhrase! !

!Viewer methodsFor: 'special phrases' stamp: 'yo 3/7/2005 08:39'!
turtleOfPhrase

	| outerPhrase getTile |
	outerPhrase := PhraseTileMorph new 
				setOperator: #+
				type: #Player
				rcvrType: #Player
				argType: #Color.	"temp dummy"
	(outerPhrase submorphs second) delete.	"operator"
	(outerPhrase submorphs second) delete.	"color"
	getTile _ KedamaTurtleOfTile new.
	outerPhrase addMorphBack: getTile.
	^outerPhrase! !

!Viewer methodsFor: 'copying' stamp: 'tk 1/7/1999 17:01'!
veryDeepFixupWith: deepCopier
	"If target and arguments fields were weakly copied, fix them here.  If they were in the tree being copied, fix them up, otherwise point to the originals!!!!"

super veryDeepFixupWith: deepCopier.
scriptedPlayer _ deepCopier references at: scriptedPlayer ifAbsent: [scriptedPlayer].! !

!Viewer methodsFor: 'copying' stamp: 'tk 1/8/1999 07:42'!
veryDeepInner: deepCopier
	"Copy all of my instance variables.  Some need to be not copied at all, but shared.  	Warning!!!!  Every instance variable defined in this class must be handled.  We must also implement veryDeepFixupWith:.  See DeepCopier class comment."

super veryDeepInner: deepCopier.
scriptedPlayer _ scriptedPlayer.		"Weakly copied"
	"Store into it to satisfy checkVariables"! !

!Viewer methodsFor: 'dropping/grabbing' stamp: 'ar 9/18/2000 18:34'!
wantsToBeDroppedInto: aMorph
	"Return true if it's okay to drop the receiver into aMorph"
	^aMorph isWorldMorph "only into worlds"! !

AlignmentMorph subclass: #ViewerEntry
	instanceVariableNames: 'helpPane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!ViewerEntry methodsFor: 'menu' stamp: 'sw 11/9/1998 13:57'!
codePaneMenu: aMenu shifted: shifted
	^ aMenu 
		labels: 'menu
eventually
will
be
useful'
		lines: #(1)
		selections: #(beep flash beep flash beep)! !

!ViewerEntry methodsFor: 'contents' stamp: 'nb 6/17/2003 12:25'!
contents: c notifying: k
	"later, spruce this up so that it can accept input such as new method source"
	| info |
	(info _ self userSlotInformation)
		ifNotNil:
			[info documentation: c.
			^ true].
	Beeper beep.
	^ false! !

!ViewerEntry methodsFor: 'contents' stamp: 'sw 11/5/1998 09:31'!
contentsSelection
	"Not well understood why this needs to be here!!"
	^ 1 to: 0! !

!ViewerEntry methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:32'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color
		r: 1.0
		g: 0.985
		b: 0.985! !

!ViewerEntry methodsFor: 'access' stamp: 'sw 11/5/1998 10:09'!
entryType
	^ self viewerRow entryType! !

!ViewerEntry methodsFor: 'initialization' stamp: 'dgd 2/14/2003 21:07'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	self layoutInset: 0! !

!ViewerEntry methodsFor: 'access' stamp: 'sw 12/28/1998 13:01'!
playerBearingCode
	^ owner owner scriptedPlayer! !

!ViewerEntry methodsFor: 'slot' stamp: 'sw 1/13/1999 12:57'!
slotName
	"Assuming the receiver represents a slot, return its name"

	^  self viewerRow elementSymbol! !

!ViewerEntry methodsFor: 'slot' stamp: 'sw 12/28/1998 12:50'!
userSlotInformation
	"If the receiver represents a user-defined slot, then return its info; if not, retun nil"
	| aSlotName info |
	((self entryType == #systemSlot) or: [self entryType == #userSlot])
		ifFalse:
			[^ nil].
	aSlotName _ self slotName.
	^ ((info _ self playerBearingCode slotInfo) includesKey: aSlotName)
		ifTrue:
			[info at: aSlotName]
		ifFalse:
			[nil]! !

!ViewerEntry methodsFor: 'access' stamp: 'sw 11/5/1998 10:09'!
viewerRow
	"Answer the ViewerRow object, that contains the controls and the phraseTile"
	^ submorphs first! !

AlignmentMorph subclass: #ViewerLine
	instanceVariableNames: 'elementSymbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!ViewerLine commentStamp: 'sw 8/28/2004 20:32' prior: 0!
Serves as a wrapper around a line in a Viewer, enforcing the desired layout properties.!

!ViewerLine methodsFor: 'slot' stamp: 'sw 6/6/2003 21:47'!
addCommandFeedback
	"Add screen feedback showing what would be torn off in a drag"

	| aMorph |
	aMorph _ RectangleMorph new bounds: ((submorphs fourth topLeft - (2@1)) corner: (submorphs last bottomRight) + (2@0)).
	aMorph useRoundedCorners; beTransparent; borderWidth: 2; borderColor: (Color r: 1.0 g: 0.548 b: 0.452); lock.
	aMorph setProperty: #highlight toValue: true.
	ActiveWorld addMorphFront: aMorph! !

!ViewerLine methodsFor: 'slot' stamp: 'sw 8/31/2004 23:50'!
addGetterFeedback
	"Add feedback during mouseover of a getter"

	| aMorph endMorph |
	endMorph _
		(#(touchesA: #seesColor: #overlaps:) includes: self elementSymbol)
			ifTrue:
				[submorphs eighth]
			ifFalse:
				[submorphs sixth].
	aMorph _ RectangleMorph new useRoundedCorners bounds: ((submorphs fourth topLeft - (2@-1)) corner: (endMorph bottomRight + (2@-1))).
	aMorph beTransparent; borderWidth: 2; borderColor: (Color r: 1.0 g: 0.355 b: 0.839); lock.
	aMorph setProperty: #highlight toValue: true.
	ActiveWorld addMorphFront: aMorph

"
Color fromUser (Color r: 1.0 g: 0.355 b: 0.839)
"! !

!ViewerLine methodsFor: 'slot' stamp: 'sw 5/22/2003 04:30'!
addSetterFeedback
	"Add screen feedback showing what would be torn off to make a setter"

	| aMorph |
	aMorph _ RectangleMorph new bounds: ((submorphs fourth topLeft - (2@1)) corner: (submorphs last bottomRight) + (2@0)).
	aMorph useRoundedCorners; beTransparent; borderWidth: 2; borderColor: (Color r: 1.0 g: 0.548 b: 0.452); lock.
	aMorph setProperty: #highlight toValue: true.
	ActiveWorld addMorphFront: aMorph! !

!ViewerLine methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:32'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color
		r: 1.0
		g: 0.985
		b: 0.985! !

!ViewerLine methodsFor: 'access' stamp: 'sw 10/30/2001 12:43'!
elementSymbol
	"Answer the element symbol"

	^ elementSymbol! !

!ViewerLine methodsFor: 'access' stamp: 'sw 10/30/2001 12:43'!
elementSymbol: aSymbol
	"Set the element symbol"

	elementSymbol _ aSymbol! !

!ViewerLine methodsFor: 'access' stamp: 'sw 5/25/2001 10:36'!
entryType
	"Answer one of: #systemSlot #userSlot #systemScript #userScript"

	^ self playerBearingCode elementTypeFor: elementSymbol vocabulary: self currentVocabulary! !

!ViewerLine methodsFor: 'access' stamp: 'yo 4/6/2005 14:59'!
firstMorphBearingKedamaPlayer

	self allMorphsWithPlayersDo: [:e :p | (p isKindOf: KedamaExamplerPlayer) ifTrue: [^ e]].
	^ nil.
! !

!ViewerLine methodsFor: 'initialization' stamp: 'sw 8/28/2004 20:30'!
initialize
	"Initialize the receiver"

	super initialize.
	self layoutInset: 0.
	self minCellSize: (0 @ (Preferences standardEToysFont height rounded + 10))! !

!ViewerLine methodsFor: 'access' stamp: 'yo 4/1/2005 09:39'!
morphBearingPlayer

	self allMorphsWithPlayersDo:[:e :p | ^ e].
	^ nil.
! !

!ViewerLine methodsFor: 'access' stamp: 'yo 4/1/2005 10:55'!
morphsBearingPlayers

	| c |
	c _ OrderedCollection new.
	self allMorphsWithPlayersDo: [:e :p | c add: e].
	^ c asArray.
! !

!ViewerLine methodsFor: 'access' stamp: 'yo 4/1/2005 09:57'!
playerBearingCode

	self allMorphsWithPlayersDo: [:e :p | ^ e playerBearingCode].
	^ nil.
! !

!ViewerLine methodsFor: 'slot' stamp: 'sw 6/4/2003 02:36'!
removeGetterFeedback
	"Remove any existing getter feedback.  Backward-compatibility only"

	self removeHighlightFeedback! !

!ViewerLine methodsFor: 'slot' stamp: 'sw 6/4/2003 02:30'!
removeHighlightFeedback
	"Remove any existing highlight feedback"

	(ActiveWorld submorphs select: [:m | m hasProperty: #highlight]) do:
		[:m | m delete]! !

!ViewerLine methodsFor: 'slot' stamp: 'sw 6/4/2003 02:37'!
removeSetterFeedback
	"Remove any existing setter feedback"

	self removeHighlightFeedback  "backward compatibility with previously-launched viewer panes only"! !

!ViewerLine methodsFor: 'access' stamp: 'yo 4/6/2005 14:59'!
replacePlayerInReadoutWith: aPlayer 

	| tile |
	tile _ self firstMorphBearingKedamaPlayer.
	tile ifNil: [^ self].
	(tile isMemberOf: TileMorph) ifFalse: [^ self].
	tile type = #objRef ifFalse: [^ self].
	tile referToSimilarObject: aPlayer.

	self allMorphsDo: [:e |
		((e isMemberOf: UpdatingStringMorph) or: [e isMemberOf: UpdatingRectangleMorph]) ifTrue: [
			e target isPlayerLike ifTrue: [
				e target: aPlayer
			].
		].
	].! !

!ViewerLine methodsFor: 'access' stamp: 'yo 4/6/2005 14:59'!
replacePlayerWith: aPlayer

	| tile |
	tile _ self firstMorphBearingKedamaPlayer.
	tile ifNil: [^ self].
	(tile isMemberOf: TileMorph) ifFalse: [^ self].
	tile type = #objRef ifFalse: [^ self].
	tile referToSimilarObject: aPlayer.
! !

!ViewerLine methodsFor: 'slot' stamp: 'sw 10/30/2001 11:46'!
slotName
	"Assuming the receiver represents a slot, return its name"

	^  self elementSymbol! !

AlignmentMorph subclass: #ViewerRow
	instanceVariableNames: 'elementSymbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!ViewerRow methodsFor: 'access' stamp: 'sw 2/1/1999 13:00'!
elementSymbol
	^ elementSymbol! !

!ViewerRow methodsFor: 'access' stamp: 'sw 1/13/1999 12:47'!
elementSymbol: aSymbol
	elementSymbol _ aSymbol! !

!ViewerRow methodsFor: 'access' stamp: 'sw 5/25/2001 10:36'!
entryType
	"Answer one of: #systemSlot #userSlot #systemScript #userScript"

	^ self playerBearingCode elementTypeFor: elementSymbol vocabulary: self currentVocabulary! !

!ViewerRow methodsFor: 'initialization' stamp: 'ar 11/9/2000 20:44'!
initialize
	super initialize.
	self layoutInset: 1! !

!ViewerRow methodsFor: 'access' stamp: 'sw 12/28/1998 13:10'!
playerBearingCode
	^ owner playerBearingCode! !

AlignmentMorph subclass: #WatcherWrapper
	instanceVariableNames: 'player variableName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Support'!

!WatcherWrapper commentStamp: 'sw 1/6/2005 00:06' prior: 0!
A wrapper around either kind of watcher.  My primary raison d'etre is so that I can automatically change names when my player changes names.!

!WatcherWrapper methodsFor: 'accessing' stamp: 'sw 1/12/2005 09:33'!
associatedPlayer
	"Answer the player with which I'm associated"

	^ player! !

!WatcherWrapper methodsFor: 'updating' stamp: 'sw 1/6/2005 00:11'!
bringUpToDate
	"Given that my player may have been renamed, reformulate my external name"

	self reconstituteName
	! !

!WatcherWrapper methodsFor: 'initialization' stamp: 'tak 4/6/2005 13:06'!
buildForPlayer: aPlayer getter: aGetter 
	"Build up basic structure"
	| aColor |
	self
		player: aPlayer
		variableName: (Utilities inherentSelectorForGetter: aGetter).
	aColor := Color
				r: 0.387
				g: 0.581
				b: 1.0.
	self listDirection: #leftToRight;
		 hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap;
		 color: aColor;
		 layoutInset: -1;
		 borderWidth: 1;
		 borderColor: aColor darker;
		 listCentering: #center.
	self
		addMorphBack: (self buildReadout: aGetter)! !

!WatcherWrapper methodsFor: 'initialization' stamp: 'tak 4/6/2005 12:14'!
buildReadout: aGetter 
	| triplet type wording setter vocab readout |
	triplet := self readoutInformation: aGetter.
	type := triplet first.
	wording := triplet second.
	setter := triplet third.
	vocab := Vocabulary vocabularyForType: type.
	readout := vocab
				updatingTileForTarget: player
				partName: wording
				getter: aGetter
				setter: setter.
	readout
		setNameTo: ('{1}''s {2}' translated format: {player externalName. wording}).
	readout
		minHeight: (vocab wantsArrowsOnTiles
				ifTrue: [22]
				ifFalse: [14]).
	^ readout! !

!WatcherWrapper methodsFor: 'initialization' stamp: 'tak 4/6/2005 13:25'!
fancyForPlayer: aPlayer getter: aGetter 
	"build a labeled readout"
	| aColor aLabel |
	self buildForPlayer: aPlayer getter: aGetter.
	aColor := Color
				r: 0.387
				g: 0.581
				b: 1.0.
	aLabel := StringMorph contents: variableName translated , ' = ' font: ScriptingSystem fontForTiles.
	aLabel setProperty: #watcherLabel toValue: true.
	self addMorphFront: aLabel.
	self addMorphFront: (aPlayer tileReferringToSelf borderWidth: 0;
			 layoutInset: 4 @ 0;
			 typeColor: aColor;
			 color: aColor;
			 bePossessive)! !

!WatcherWrapper methodsFor: 'accessing' stamp: 'sw 1/6/2005 04:18'!
getterTilesForDrop
	"Answer getter tiles to use if there is an attempt to drop me onto a tile pad"

	| aCategoryViewer |
	aCategoryViewer _ CategoryViewer new initializeFor: player categoryChoice: #basic.
	^ aCategoryViewer getterTilesFor: (Utilities getterSelectorFor: variableName)  type: self resultType! !

!WatcherWrapper methodsFor: 'accessing' stamp: 'sw 1/6/2005 03:49'!
isTileLike
	"Answer whether the receiver can participate tile-like"

	^ true! !

!WatcherWrapper methodsFor: 'identification' stamp: 'sw 1/6/2005 03:44'!
isTileScriptingElement
	"Answer whether the receiver is a tile-scripting element"

	^ true! !

!WatcherWrapper methodsFor: 'accessing' stamp: 'sw 1/6/2005 04:00'!
justGrabbedFrom: formerOwner
	"An attempt to make these guys easier to involve in tile scripting.  But in the end too strange, so for the moment the active ingredients commented out"

"	self center: ActiveHand position.
	self left: ActiveHand position x."

	super justGrabbedFrom: formerOwner! !

!WatcherWrapper methodsFor: 'updating' stamp: 'tak 8/5/2005 01:14'!
localeChanged! !

!WatcherWrapper methodsFor: 'initialization' stamp: 'tak 4/6/2005 13:07'!
player: aPlayer variableName: aVariableName
	"Set up my initial state"
	player _ aPlayer.
	variableName _ aVariableName.
	self reconstituteName
! !

!WatcherWrapper methodsFor: 'initialization' stamp: 'tak 4/6/2005 13:14'!
readoutInformation: aGetter 
	"Answer a triplet of {type. wording. setter}"
	| info anInterface |
	info := player slotInfoForGetter: aGetter.
	"In case of a variable"
	info
		ifNotNil: [^ {info type. Utilities inherentSelectorForGetter: aGetter. Utilities setterSelectorFor: variableName}].
	"In case of a slot"
	anInterface := Vocabulary eToyVocabulary
				methodInterfaceAt: aGetter
				ifAbsent: [^ {#Unknown. '*'. nil}].
	^ {anInterface resultType. anInterface wording. anInterface companionSetterSelector}! !

!WatcherWrapper methodsFor: 'accessing' stamp: 'sw 1/6/2005 02:47'!
readoutMorph
	"Answer the submorph of mine that serves as my readout"

	^ self allMorphs detect:
		[:m | m isEtoyReadout] ifNone: [nil]! !

!WatcherWrapper methodsFor: 'updating' stamp: 'yo 3/17/2005 08:47'!
reconstituteName
	"Reconstitute the external name of the receiver"

	variableName ifNotNil:
		[self setNameTo: ('{1}''s {2}' translated format: {player externalName. variableName translated}).
		(self submorphWithProperty: #watcherLabel) ifNotNilDo:
			[:aLabel | aLabel contents: variableName asString, ' = ']]! !

!WatcherWrapper methodsFor: 'accessing' stamp: 'sw 1/6/2005 03:13'!
resultType
	"Answer the result type the receiver would produce."

	^ player typeForSlotWithGetter: (Utilities getterSelectorFor: variableName)! !

!WatcherWrapper methodsFor: 'accessing' stamp: 'sw 1/12/2005 09:32'!
tileRows
	"Answer, for the benefit of d&d scripting, a structure appropriate for dropping nto a script"

	^ Array with: (Array with: self getterTilesForDrop)! !

!WatcherWrapper methodsFor: 'initialization' stamp: 'tak 4/6/2005 13:47'!
unlabeledForPlayer: aPlayer getter: aGetter 
	"build a simple watcher"
	| readout |
	self buildForPlayer: aPlayer getter: aGetter.
	readout := self submorphs last.
	(readout isKindOf: TileMorph)
		ifTrue: [readout labelMorph lock: true.
			readout labelMorph beSticky]! !

FloatArray variableWordSubclass: #KedamaFloatArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaFloatArray methodsFor: 'arithmetic' stamp: 'yo 10/25/2004 15:40'!
* other

	| result |
	other isNumber ifTrue: [
		result _ KedamaFloatArray new: self size.
		^ self primMulScalar: self and: other into: result.
	].
	((other isMemberOf: WordArray) or: [other isMemberOf: KedamaFloatArray]) ifTrue: [	
		result _ KedamaFloatArray new: self size.
		^ self primMulArray: self and: other into: result.
	].
	^ super * other.
! !

!KedamaFloatArray methodsFor: 'arithmetic' stamp: 'yo 10/25/2004 15:43'!
*= other

	other isNumber ifTrue: [
		^ self primMulScalar: self and: other asFloat into: self.
	].
	((other isMemberOf: WordArray) or: [other isMemberOf: KedamaFloatArray]) ifTrue: [	
		^ self primMulArray: self and: other into: self.
	].
	^ super *= other.
! !

!KedamaFloatArray methodsFor: 'arithmetic' stamp: 'yo 10/25/2004 15:30'!
+ other

	| result |
	other isNumber ifTrue: [
		result _ KedamaFloatArray new: self size.
		^ self primAddScalar: self and: other into: result.
	].
	((other isMemberOf: WordArray) or: [other isMemberOf: KedamaFloatArray]) ifTrue: [	
		result _ KedamaFloatArray new: self size.
		^ self primAddArray: self and: other into: result.
	].
	^ super + other.
! !

!KedamaFloatArray methodsFor: 'arithmetic' stamp: 'yo 10/25/2004 15:44'!
+= other

	other isNumber ifTrue: [
		^ self primAddScalar: self and: other into: self.
	].
	((other isMemberOf: WordArray) or: [other isMemberOf: KedamaFloatArray]) ifTrue: [	
		^ self primAddArray: self and: other into: self.
	].
	^ super += other.
! !

!KedamaFloatArray methodsFor: 'arithmetic' stamp: 'yo 10/25/2004 15:40'!
- other

	| result |
	other isNumber ifTrue: [
		result _ KedamaFloatArray new: self size.
		^ self primSubScalar: self and: other into: result.
	].
	((other isMemberOf: WordArray) or: [other isMemberOf: KedamaFloatArray]) ifTrue: [	
		result _ KedamaFloatArray new: self size.
		^ self primSubArray: self and: other into: result.
	].
	^ super - other.
! !

!KedamaFloatArray methodsFor: 'arithmetic' stamp: 'yo 10/25/2004 15:44'!
-= other

	other isNumber ifTrue: [
		^ self primSubScalar: self and: other into: self.
	].
	((other isMemberOf: WordArray) or: [other isMemberOf: KedamaFloatArray]) ifTrue: [	
		^ self primSubArray: self and: other into: self.
	].
	^ super -= other.
! !

!KedamaFloatArray methodsFor: 'arithmetic' stamp: 'yo 10/25/2004 15:41'!
/ other

	| result |
	other isNumber ifTrue: [
		result _ KedamaFloatArray new: self size.
		^ self primDivScalar: self and: other into: result.
	].
	((other isMemberOf: WordArray) or: [other isMemberOf: KedamaFloatArray]) ifTrue: [	
		result _ KedamaFloatArray new: self size.
		^ self primDivArray: self and: other into: result.
	].
	^ super / other.
! !

!KedamaFloatArray methodsFor: 'arithmetic' stamp: 'yo 10/25/2004 15:45'!
/= other

	other isNumber ifTrue: [
		^ self primDivScalar: self and: other into: self.
	].
	((other isMemberOf: WordArray) or: [other isMemberOf: KedamaFloatArray]) ifTrue: [	
		^ self primDivArray: self and: other into: self.
	].
	^ super /= other.
! !

!KedamaFloatArray methodsFor: 'primitives' stamp: 'yo 10/25/2004 15:39'!
primAddArray: rcvr and: other into: result

	<primitive: 'primitiveAddArrays' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primitiveAddArrays."

	1 to: rcvr size do: [:i |
		result at: i put: (rcvr at: i) + (other at: i)
	].
	^ result.
! !

!KedamaFloatArray methodsFor: 'primitives' stamp: 'yo 10/25/2004 15:15'!
primAddScalar: rcvr and: other into: result

	<primitive: 'primitiveAddScalar' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primitiveAddScalar."

	1 to: rcvr size do: [:i |
		result at: i put: (rcvr at: i) + other.
	].
	^ result.
! !

!KedamaFloatArray methodsFor: 'primitives' stamp: 'yo 10/25/2004 15:39'!
primDivArray: rcvr and: other into: result

	<primitive: 'primitiveDivArrays' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primitiveDivArrays."

	1 to: rcvr size do: [:i |
		result at: i put: (rcvr at: i) / (other at: i)
	].
	^ result.
! !

!KedamaFloatArray methodsFor: 'primitives' stamp: 'yo 10/25/2004 15:39'!
primDivScalar: rcvr and: other into: result

	<primitive: 'primitiveDivScalar' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primitiveDivScalar."

	1 to: rcvr size do: [:i |
		result at: i put: (rcvr at: i) / other.
	].
	^ result.
! !

!KedamaFloatArray methodsFor: 'primitives' stamp: 'yo 10/25/2004 15:39'!
primMulArray: rcvr and: other into: result

	<primitive: 'primitiveMulArrays' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primitiveMulArrays."

	1 to: rcvr size do: [:i |
		result at: i put: (rcvr at: i) * (other at: i)
	].
	^ result.
! !

!KedamaFloatArray methodsFor: 'primitives' stamp: 'yo 10/25/2004 15:38'!
primMulScalar: rcvr and: other into: result

	<primitive: 'primitiveMulScalar' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primitiveMulScalar."

	1 to: rcvr size do: [:i |
		result at: i put: (rcvr at: i) * other.
	].
	^ result.
! !

!KedamaFloatArray methodsFor: 'primitives' stamp: 'yo 10/25/2004 15:39'!
primSubArray: rcvr and: other into: result

	<primitive: 'primitiveSubArrays' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primitiveSubArrays."

	1 to: rcvr size do: [:i |
		result at: i put: (rcvr at: i) - (other at: i)
	].
	^ result.
! !

!KedamaFloatArray methodsFor: 'primitives' stamp: 'yo 10/25/2004 15:38'!
primSubScalar: rcvr and: other into: result

	<primitive: 'primitiveSubScalar' module:'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primitiveSubScalar."

	1 to: rcvr size do: [:i |
		result at: i put: (rcvr at: i) - other.
	].
	^ result.
! !

!TextMorph class methodsFor: '*eToys-scripting' stamp: 'sw 10/3/2004 01:34'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(
(#'color & border' (
(slot backgroundColor 'The color of the background behind the text' Color readWrite Player getBackgroundColor Player setBackgroundColor:)))

(text (
(slot backgroundColor 'The color of the background behind the text' Color readWrite Player getBackgroundColor Player setBackgroundColor:)
(slot characters	'The characters in my contents' String	readWrite Player getCharacters Player setCharacters:)

(slot cursor 'The position among my characters that replacement text would go' Number readWrite Player getCursor Player setCursor:)
(slot characterAtCursor 'The character at the my cursor position' String readWrite Player getCharacterAtCursor Player setCharacterAtCursor:)
(slot count 'How many characters I have' Number readOnly Player getCount unused unused)

(slot firstCharacter  'The first character in my contents' String  readWrite Player getFirstCharacter  Player  setFirstCharacter:)

(slot lastCharacter  'The last character in my contents' String  readWrite Player getLastCharacter  Player  setLastCharacter:)
(slot allButFirst 'All my characters except the first one' String readWrite Player getAllButFirstCharacter Player  setAllButFirstCharacter:)
(command insertCharacters: 'insert the given string at my cursor position' String)
(command insertContentsOf: 'insert the characters from another object at my cursor position' Player)
(slot numericValue 'The number represented by my contents' Number readWrite Player getNumericValue Player  setNumericValue:)))

(basic (
(slot characters	'The characters in my contents' String	readWrite Player getCharacters Player setCharacters:))))


! !

!TextMorph class methodsFor: '*eToys-parts bin' stamp: 'sw 6/13/2001 22:46'!
exampleBackgroundField
	"Answer a background field for a parts bin"

	| aMorph |
	aMorph _ TextMorph authoringPrototype.
	aMorph contents: 'background field' asText allBold.
	aMorph setProperty: #shared toValue: true.
	aMorph setNameTo: 'field1'.
	aMorph setProperty: #holdsSeparateDataForEachInstance toValue: true.	
	^ aMorph
! !

!TextMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 10/2/97 15:16'!
configureForKids
	super configureForKids.
	self lock! !

!TextMorph methodsFor: '*eToys-card in a stack' stamp: 'sw 10/30/2000 09:02'!
couldHoldSeparateDataForEachInstance
	"Answer whether this type of morph is inherently capable of holding separate data for each instance ('card data')"

	^ true! !

!TextMorph methodsFor: '*eToys-player' stamp: 'sw 10/30/2000 09:01'!
currentDataValue
	"Answer the current data value held by the receiver"

	^ text! !

!TextMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 9/15/2000 06:14'!
getNumericValue
	"Obtain a numeric value from the receiver; if no digits, return zero"

	| aString |
	^ [(aString _ text string) asNumber] ifError: [:a :b | ^ aString asInteger ifNil: [0]]! !

!TextMorph methodsFor: '*eToys-card & stack' stamp: 'sw 5/31/2000 01:18'!
newContents: stringOrText fromCard: aCard
	"Accept new text contents."
	| newText setter |

	newText _ stringOrText asText.
	text = newText ifTrue: [^ self].  "No substantive change"

	text ifNotNil: [
		text embeddedMorphs do: [ :m | m delete ] ].

	text _ newText.

	"add all morphs off the visible region; they'll be moved into the right place when they become visible.  (this can make the scrollable area too large, though)"
	stringOrText asText embeddedMorphs do: [ :m | 
		self addMorph: m. 
		m position: (-1000@0)].

	self releaseParagraph.  "update the paragraph cache"
	self paragraph.  "re-instantiate to set bounds"

	self holdsSeparateDataForEachInstance
		ifTrue:
			[setter _ self valueOfProperty: #setterSelector.
			setter ifNotNil:
				[aCard perform: setter with: newText]].

	self world ifNotNil:
		[self world startSteppingSubmorphsOf: self ].
! !

!TextMorph methodsFor: '*eToys-card & stack' stamp: 'tk 1/8/2001 13:14'!
setNewContentsFrom: stringOrTextOrNil
	"Using stringOrTextOrNil as a guide, set the receiver's contents afresh.  If the input parameter is nil, the a default value stored in a property of the receiver, if any, will supply the new initial content.  This method is only called when a VariableDock is attempting to put a new value.  This is still messy and ill-understood and not ready for prime time."

	| defaultValue tt atts |
	stringOrTextOrNil ifNotNil: [^ self newContents: stringOrTextOrNil 
		fromCard: (self valueOfProperty: #cardInstance)].
		   "Well, totally yuk -- emergency measure late on eve of demo"
	defaultValue _ self valueOfProperty: #defaultValue 
					ifAbsent: [atts _ text attributesAt: 1.	"Preserve size, emphasis"
						tt _ text copyReplaceFrom: 1 to: text size
								with: 'blankText'.
						atts do: [:anAtt | tt addAttribute: anAtt].
						tt].
	self contents: defaultValue deepCopy wrappedTo: self width.
! !

!TextMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 9/1/2000 10:44'!
setNumericValue: aValue
	"Set the contents of the receiver to be a string obtained from aValue"

	self newContents: aValue asString! !

!TextMorph methodsFor: '*eToys-player' stamp: 'sw 10/25/2000 07:02'!
variableDocks
	"Answer a list of VariableDock objects for docking up my data with an instance held in my containing playfield"

	^ Array with: (VariableDock new variableName: self defaultVariableName type: #text definingMorph: self morphGetSelector: #contents morphPutSelector: #setNewContentsFrom:)! !

!Inspector methodsFor: '*eToys-menu commands' stamp: 'sw 10/23/2000 18:25'!
tearOffTile
	"Tear off a tile that refers to the receiver's selection, and place it in the mophic hand"

	| objectToRepresent |
	objectToRepresent _ self selectionIndex == 0 ifTrue: [object] ifFalse: [self selection].
	self currentHand attachMorph: (TileMorph new referTo: objectToRepresent)
	! !

!MorphicModel class methodsFor: '*eToys-queries' stamp: 'sw 2/27/2002 14:58'!
baseUniclass
	"Answer the uniclass that new instances should be instances of.  This protocol is primarily intended for the Player lineage, but can get sent to a MorphicModel subclass when the project-loading mechanism is scrambling to fix up projects that have naming conflicts with the project being loaded."

	| curr |
	curr _ self.
	[curr theNonMetaClass superclass name endsWithDigit]
		whileTrue:
			[curr _ curr superclass].
	^ curr

"PlayWithMe1 baseUniclass"! !

!SmallInteger methodsFor: '*eToys-printing' stamp: 'sw 10/26/2000 09:01'!
uniqueNameForReference
	"Answer a nice name by which the receiver can be referred to by other objects.   For SmallIntegers, we can actually just use the receiver's own printString, though this is pretty strange in some ways."

	^ self asString! !

!MessageSend methodsFor: '*eToys-tiles' stamp: 'tk 11/15/2000 15:19'!
asTilesIn: playerClass
	| code keywords num tree syn block phrase |
	"Construct SyntaxMorph tiles for me."

	"This is really cheating!!  Make a true parse tree later. -tk"
	code _ String streamContents: [:strm | 
		strm nextPutAll: 'doIt'; cr; tab.
		strm nextPutAll: (self stringFor: receiver).
		keywords _ selector keywords.
		strm space; nextPutAll: keywords first.
		(num _ selector numArgs) > 0 ifTrue: [strm space. 
					strm nextPutAll: (self stringFor: arguments first)].
		2 to: num do: [:kk |
			strm space; nextPutAll: (keywords at: kk).
			strm space; nextPutAll: (self stringFor: (arguments at: kk))]].
	"decompile to tiles"
	tree _ Compiler new 
		parse: code 
		in: playerClass
		notifying: nil.
	syn _ tree asMorphicSyntaxUsing: SyntaxMorph.
	block _ syn submorphs detect: [:mm | 
		(mm respondsTo: #parseNode) ifTrue: [
			mm parseNode class == BlockNode] ifFalse: [false]].
	phrase _ block submorphs detect: [:mm | 
		(mm respondsTo: #parseNode) ifTrue: [
			mm parseNode class == MessageNode] ifFalse: [false]].
	^ phrase

! !

!MessageSend methodsFor: '*eToys-tiles' stamp: 'tk 9/28/2001 13:41'!
asTilesIn: playerClass globalNames: makeSelfGlobal
	| code keywords num tree syn block phrase |
	"Construct SyntaxMorph tiles for me.  If makeSelfGlobal is true, name the receiver and use that name, else use 'self'.  (Note that this smashes 'self' into the receiver, regardless of what it was.)"

	"This is really cheating!!  Make a true parse tree later. -tk"
	code _ String streamContents: [:strm | 
		strm nextPutAll: 'doIt'; cr; tab.
		strm nextPutAll: 
			(makeSelfGlobal ifTrue: [self stringFor: receiver] ifFalse: ['self']).
		keywords _ selector keywords.
		strm space; nextPutAll: keywords first.
		(num _ selector numArgs) > 0 ifTrue: [strm space. 
					strm nextPutAll: (self stringFor: arguments first)].
		2 to: num do: [:kk |
			strm space; nextPutAll: (keywords at: kk).
			strm space; nextPutAll: (self stringFor: (arguments at: kk))]].
	"decompile to tiles"
	tree _ Compiler new 
		parse: code 
		in: playerClass
		notifying: nil.
	syn _ tree asMorphicSyntaxUsing: SyntaxMorph.
	block _ syn submorphs detect: [:mm | 
		(mm respondsTo: #parseNode) ifTrue: [
			mm parseNode class == BlockNode] ifFalse: [false]].
	phrase _ block submorphs detect: [:mm | 
		(mm respondsTo: #parseNode) ifTrue: [
			mm parseNode class == MessageNode] ifFalse: [false]].
	^ phrase

! !

!MessageSend methodsFor: '*eToys-tiles' stamp: 'sw 6/20/2001 14:17'!
stringFor: anObject
	"Return a string suitable for compiling.  Literal or reference from global ref dictionary.  self is always named via the ref dictionary."

	| generic aName |
	anObject isLiteral ifTrue: [^ anObject printString].
	anObject class == Color ifTrue: [^ anObject printString].
	anObject class superclass == Boolean ifTrue: [^ anObject printString].
	anObject class == BlockContext ifTrue: [^ '[''do nothing'']'].	"default block"
		"Real blocks need to construct tiles in a different way"
	anObject class isMeta ifTrue: ["a class" ^ anObject name].
	generic _ anObject knownName.	"may be nil or 'Ellipse' "
	aName _ anObject uniqueNameForReference.
	generic ifNil:
		[(anObject respondsTo: #renameTo:) 
			ifTrue: [anObject renameTo: aName]
			ifFalse: [aName _ anObject storeString]].	"for Fraction, LargeInt, etc"
	^ aName
! !

Morph subclass: #DrawErrorMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Experimental'!

!DrawErrorMorph commentStamp: '<historical>' prior: 0!
This morph simply invokes errors during drawing and stepping.!

!DrawErrorMorph methodsFor: 'drawing' stamp: 'ar 4/2/1999 12:13'!
drawOn: aCanvas
	aCanvas error:'DrawErrorMorph drawOn: invoked'! !

!DrawErrorMorph methodsFor: 'printing' stamp: 'ar 4/2/1999 12:15'!
printOn: aStream
	"Indirectly invokes an error during stepping in an Inspector"
	aStream error:'DrawErrorMorph>>printOn: invoked'! !

Morph subclass: #KedamaMorph
	instanceVariableNames: 'dimensions wrapX wrapY pixelsPerPatch patchesToDisplay patchVarDisplayForm lastTurtleID turtleCount turtlesDict turtlesDictSemaphore turtlesToDisplay magnifiedDisplayForm autoChanged topEdgeMode bottomEdgeMode leftEdgeMode rightEdgeMode topEdgeModeMnemonic bottomEdgeModeMnemonic leftEdgeModeMnemonic rightEdgeModeMnemonic'
	classVariableNames: 'RandomSeed'
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaMorph commentStamp: 'yo 6/18/2004 18:29' prior: 0!
A tile-scriptable variant of StarSqueak.
!

!KedamaMorph classSide methodsFor: 'class initialization' stamp: 'yo 9/14/2004 20:27'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."
	^ #(

 	(kedama (
		(command addToPatchDisplayList: 'add patch to display list' Patch)
		(command removeAllFromPatchDisplayList 'clear the patch display list')
		(slot patchDisplayList 'patches to display' String readOnly Player getPatchesList unused unused)
		(command addToTurtleDisplayList: 'add turtle to display list' Player)
		(command removeAllFromTurtleDisplayList 'clear the turtle display list')
		(slot turtleDisplayList 'turtles to display' String readOnly Player getTurtlesList unused unused)
		(slot pixelsPerPatch 'the display scale' Number readWrite Player getPixelsPerPatch Player setPixelsPerPatch:)
		(slot color 'The color of the object' Color readWrite Player getColor  Player  setColor:)
		"(command makeTurtlesMap 'Internally create the map of turtles')"
		(slot leftEdgeMode 'the mode of left edge' EdgeMode readWrite Player getLeftEdgeMode Player setLeftEdgeMode:)
		(slot rightEdgeMode 'the mode of right edge' EdgeMode readWrite Player getRightEdgeMode Player setRightEdgeMode:)
		(slot topEdgeMode 'the mode of top edge' EdgeMode readWrite Player getTopEdgeMode Player setTopEdgeMode:)
		(slot bottomEdgeMode 'the mode of bottom edge' EdgeMode readWrite Player getBottomEdgeMode Player setBottomEdgeMode:)
	))
).
! !

!KedamaMorph classSide methodsFor: 'class initialization' stamp: 'yo 6/9/2004 11:09'!
cleanUp
"
	self cleanUp
"
	self allInstancesDo: [:e | e cleanUp].
! !

!KedamaMorph classSide methodsFor: 'class initialization' stamp: 'yo 6/17/2004 19:04'!
defaultDimensions

	"Answer the dimensions of this StarSqueak simulation. Subclasses can override this method to define their own world size."

	^ 100@100.
! !

!KedamaMorph classSide methodsFor: 'class initialization' stamp: 'yo 9/26/2005 22:58'!
defaultNameStemForInstances

	^ 'KedamaWorld' translated.
! !

!KedamaMorph classSide methodsFor: 'global primitive backup' stamp: 'yo 10/11/2004 11:47'!
degreesToRadians: degrees

	| deg q headingRadians |
	deg _ 90.0 - degrees.
	q _ (deg / 360.0) asInteger.
	deg < 0.0 ifTrue: [q _ q - 1].
	headingRadians _ (deg - (q * 360.0)) * 0.0174532925199433.
	^ headingRadians.
! !

!KedamaMorph classSide methodsFor: 'class initialization' stamp: 'yo 9/13/2005 22:09'!
descriptionForPartsBin
	^ self partName:	'KedamaWorld'
		categories:		#('Kedama')
		documentation:	'A tile scriptable particle system' translated! !

!KedamaMorph classSide methodsFor: 'class initialization' stamp: 'yo 9/2/2004 23:23'!
initialize
	"Kedama initialize"

	RandomSeed _ 17.
! !

!KedamaMorph classSide methodsFor: 'class initialization' stamp: 'yo 10/23/2005 20:33'!
newSet

	| k p t s w |
	k _ self new.
	p _ k assuredPlayer getPatch costume renderedMorph.
	t _ k assuredPlayer newTurtleForSet.

	s _ SelectionMorph new.

	w _ PasteUpMorph new.
	w extent: 400@400.
	p position: 275@50.
	t position: 300@175.
	k position: 25@25.
	w addMorph: k.
	w addMorph: t.
	w addMorph: p.
	w addMorph: s.
	s bounds: w bounds.
	s selectSubmorphsOf: w.
	^ s.
! !

!KedamaMorph classSide methodsFor: 'global primitive backup' stamp: 'yo 10/11/2004 12:45'!
radiansToDegrees: radians

	| degrees deg |
	degrees _ radians / 0.0174532925199433.
	deg _ 90.0 - degrees.
	deg > 0.0 ifFalse: [deg _ deg + 360.0].
	^ deg.

! !

!KedamaMorph classSide methodsFor: 'global primitive backup' stamp: 'yo 10/10/2004 09:07'!
scalarXAt: index xArray: xArray headingArray: headingArray value: val destWidth: destWidth leftEdgeMode: leftEdgeMode rightEdgeMode: rightEdgeMode

	| newX headingRadians |
	newX _ val.
	newX < 0.0 ifTrue: [
		leftEdgeMode = 1 ifTrue: [
			"wrap"
			newX _ newX + destWidth.
		].
		leftEdgeMode = 2 ifTrue: [
			"stick"
			newX _ 0.0.
		].
		leftEdgeMode = 3 ifTrue: [
			"bounce"
			newX _ 0.0 - newX.
			headingRadians _ headingArray at: index.
			headingRadians <  3.141592653589793
				ifTrue: [headingArray at: index put: 3.141592653589793 - headingRadians]
				ifFalse: [headingArray at: index put: 9.42477796076938 - headingRadians].
		].
	].

	newX >= destWidth ifTrue: [
		rightEdgeMode = 1 ifTrue: [
			newX _ newX - destWidth.
		].
		rightEdgeMode = 2 ifTrue: [
			newX _ destWidth - 0.000001.
		].
		rightEdgeMode = 3 ifTrue: [
			newX _ (destWidth - 0.000001) - (newX - destWidth).
			headingRadians _ headingArray at: index.
			headingRadians < 3.141592653589793
				ifTrue: [headingArray at: index put: (3.141592653589793 - headingRadians)]
				ifFalse: [headingArray at: index put: (9.42477796076938 - headingRadians)].
		]
	].
	xArray at: index put: newX.
! !

!KedamaMorph classSide methodsFor: 'global primitive backup' stamp: 'yo 10/10/2004 09:09'!
scalarYAt: index yArray: yArray headingArray: headingArray value: val destHeight: destHeight topEdgeMode: topEdgeMode bottomEdgeMode: bottomEdgeMode

	| newY |
	newY _ val.
	newY < 0.0 ifTrue: [
		topEdgeMode = 1 ifTrue: [
			"wrap"
			newY _ newY + destHeight.
		].
		topEdgeMode = 2 ifTrue: [
			"stick"
			newY _ 0.0.
		].
		topEdgeMode = 3 ifTrue: [
			"bounce"
			newY _ 0.0 - newY.
			headingArray at: index put: (6.283185307179586 - (headingArray at: index)).
		].
	].

	newY >= destHeight ifTrue: [
		bottomEdgeMode = 1 ifTrue: [
			newY _ newY - destHeight.
		].
		bottomEdgeMode = 2 ifTrue: [
			newY _ destHeight - 0.000001.
		].
		bottomEdgeMode = 3 ifTrue: [
			newY _ (destHeight - 0.000001) - (newY - destHeight).
			headingArray at: index put: (6.283185307179586 - (headingArray at: index)).
		]
	].
	yArray at: index put: newY.
! !

!KedamaMorph methodsFor: 'event handling' stamp: 'ar 7/8/2006 20:25'!
acceptDroppingMorph: morphToDrop event: evt

	| f turtle |
	(morphToDrop renderedMorph isKindOf: SketchMorph) ifFalse: [
		^morphToDrop rejectDropMorphEvent: evt.
	].

	f _ morphToDrop renderedMorph rotatedForm.
	f _ f magnify: f boundingBox by: (1.0 / self pixelsPerPatch asFloat) smoothing: 1.

	turtle _ self player newTurtleSilently.
	turtle createTurtlesAsIn: f originAt: ((morphToDrop topLeft - self topLeft) / self pixelsPerPatch asFloat) asIntegerPoint.
	"turtle isGroup: true."
	turtle color: (self dominantColorWithoutTransparent: f).
! !

!KedamaMorph methodsFor: 'setup' stamp: 'yo 9/9/2004 15:24'!
addToPatchDisplayList: p

	| a |
	a _ patchesToDisplay copyWithout: p.
	patchesToDisplay _ a copyWith: p.
! !

!KedamaMorph methodsFor: 'setup' stamp: 'yo 9/14/2004 20:24'!
addToTurtleDisplayList: p

	| a |
	(p isKindOf: KedamaExamplerPlayer) ifFalse: [^ self].
	a _ turtlesToDisplay copyWithout: p.
	turtlesToDisplay _ a copyWith: p.
! !

!KedamaMorph methodsFor: 'submorphs-accessing' stamp: 'yo 6/17/2004 19:45'!
allSubmorphNamesDo: aBlock

	super allSubmorphNamesDo: aBlock.
	aBlock value: self player getPatch externalName.
! !

!KedamaMorph methodsFor: 'drawing' stamp: 'yo 6/1/2004 16:53'!
areasRemainingToFill: aRectangle
	"Drawing optimization. Since I completely fill my bounds with opaque pixels, this method tells Morphic that it isn't necessary to draw any morphs covered by me."
	
	^ aRectangle areasOutside: self bounds
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 6/17/2004 09:51'!
bottomEdgeMode

	^ bottomEdgeMode.
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 1/20/2005 19:52'!
bottomEdgeMode: aSymbol

	bottomEdgeMode _ aSymbol asSymbol.
	bottomEdgeMode == #wrap ifTrue: [
		bottomEdgeModeMnemonic _ 1.
		^ self
	].
	bottomEdgeMode == #stick ifTrue: [
		bottomEdgeModeMnemonic _ 2.
		^ self
	].
	(bottomEdgeMode == #bounce or: [bottomEdgeMode == #bouncing]) ifTrue: [
		bottomEdgeModeMnemonic _ 3.
		^ self
	].
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 9/16/2004 18:59'!
bottomEdgeModeMnemonic

	^ bottomEdgeModeMnemonic.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'KR 4/14/2006 15:52'!
calcTurtlesCount

	turtleCount _ 0.
	turtlesDict do: [:a | turtleCount _ turtleCount + a size].

	SmalltalkImage current vmParameterAt: 5 put: ((turtleCount * 3) min: 16000 max: 4000).
	SmalltalkImage current vmParameterAt: 6 put: ((turtleCount * 6) min: 32000 max: 8000).
! !

!KedamaMorph methodsFor: 'etoys' stamp: 'yo 6/11/2004 11:20'!
categoriesForViewer
	"Answer a list of symbols representing the categories to offer in the viewer, in order"
	^ super categoriesForViewer.
"
	| aList |
	aList _ OrderedCollection new.
	aList addAllFirstUnlessAlreadyPresent: (self class additionsToViewerCategories collect:
				[:categorySpec | categorySpec first]).
	^ aList
"! !

!KedamaMorph methodsFor: 'setup' stamp: 'yo 6/9/2004 11:09'!
cleanUp

	self extension actorState: nil.
	self extension player: nil.
	self initializeTurtlesDict.
! !

!KedamaMorph methodsFor: 'setup' stamp: 'yo 9/27/2004 10:26'!
clearAll
	"Reset this StarSqueak world. All patch variables are cleared, all turtles are removed, and all demons are turned off."

	patchVarDisplayForm _ Form extent: dimensions depth: 32.
	self initializePatch.
	self recreateMagnifiedDisplayForm.
	self initializeTurtlesDict.

	turtleCount _ 0.
	lastTurtleID _ 0.

	self color: Color black.

! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 4/6/2005 12:59'!
colorAt: aLocalPoint

	| pix |
	pix _ patchVarDisplayForm pixelValueAt: (aLocalPoint // pixelsPerPatch) asIntegerPoint.
	^ Color colorFromPixelValue: (pix bitOr: 16rFF000000) depth: 32.
! !

!KedamaMorph methodsFor: 'deleting' stamp: 'yo 10/8/2004 11:11'!
delete

	| c |
	super delete.
	turtlesDict keysDo: [:k |
		self deleteAllTurtlesOfExampler: k.
		c _ k costume.
		c ifNotNil: [c renderedMorph delete].
	].

! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/28/2004 14:12'!
deleteAllTurtlesOfExampler: examplerPlayer

	turtlesDict removeKey: examplerPlayer ifAbsent: [].
	self removeFromTurtleDisplayList: examplerPlayer.
	self calcTurtlesCount.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 7/20/2005 22:34'!
deleteTurtleID: who of: examplerPlayer
	"Delete the given turtle from this world."

	| array |
	array _ examplerPlayer turtles.
	array ifNil: [^ self].
	turtlesDictSemaphore critical: [
		array deleteTurtleID: who.
	].
	self calcTurtlesCount.
	examplerPlayer costume renderedMorph privateTurtleCount: array size.
	"examplerPlayer allOpenViewers do: [:v | v resetWhoIfNecessary]."
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 6/1/2004 16:53'!
dimensions

	^ dimensions
! !

!KedamaMorph methodsFor: 'drawing' stamp: 'yo 6/17/2004 17:56'!
display
	"Display this world on the Display. Used for debugging."

	| c |
	c _ FormCanvas extent: (dimensions * pixelsPerPatch) depth: 32.
	c _ c copyOffset: bounds origin negated.
	self drawOn: c.
	c form display.
! !

!KedamaMorph methodsFor: 'event handling' stamp: 'ar 7/8/2006 20:25'!
dominantColorWithoutTransparent: aForm
	| tally max maxi |
	aForm depth > 16 ifTrue:
		[^self dominantColorWithoutTransparent: (aForm asFormOfDepth: 16)].
	tally _ aForm tallyPixelValues.
	max _ maxi _ 0.
	tally withIndexDo: [:n :i | n > max ifTrue: [ i ~= 1 ifTrue: [max _ n. maxi _ i]]].
	^ Color colorFromPixelValue: maxi - 1 depth: aForm depth
! !

!KedamaMorph methodsFor: 'drawing' stamp: 'yo 9/27/2004 10:28'!
drawOn: aCanvas
	"Display this StarSqueak world."

	| result |
	"Time millisecondClockValue printString displayAt: 0@0."
	self player ifNil: [^ aCanvas fillRectangle: (self bounds) color: self color].
	patchVarDisplayForm fillColor: self color.
	patchesToDisplay do: [:p |
		p displayPatchVariableOn: patchVarDisplayForm.
	].
	self drawTurtlesOnForm: patchVarDisplayForm.
	pixelsPerPatch = 1 ifTrue: [
		aCanvas drawImage: patchVarDisplayForm at: bounds origin.
	] ifFalse: [
		result _ self zoom: patchVarDisplayForm into: magnifiedDisplayForm factor: pixelsPerPatch.
		result ifNil: [
			aCanvas warpImage: patchVarDisplayForm transform: (MatrixTransform2x3 withScale: pixelsPerPatch) at: self innerBounds origin.
		] ifNotNil: [
			aCanvas drawImage: magnifiedDisplayForm at: bounds origin.
		]
	].

	autoChanged ifTrue: [self changed].

! !

!KedamaMorph methodsFor: 'drawing' stamp: 'yo 9/27/2004 16:28'!
drawTurtlesOnForm: aForm

	turtlesToDisplay do: [:exampler |
		(self isVisible: exampler) ifTrue: [
			turtlesDictSemaphore critical: [
				exampler turtles drawOn: aForm.
			].
		].
	].
! !

!KedamaMorph methodsFor: 'geometry' stamp: 'yo 6/1/2004 16:53'!
extent: aPoint
	"Do nothing; my extent is determined by my StarSqueak world dimensions and pixelsPerPatch."
! !

!KedamaMorph methodsFor: 'utils' stamp: 'yo 8/18/2005 15:32'!
hasNoTurtleBreed

	^ turtlesDict isEmpty.
! !

!KedamaMorph methodsFor: 'initialization' stamp: 'yo 9/17/2004 19:34'!
initialize

	super initialize.
	dimensions _ self class defaultDimensions.  "dimensions of this StarSqueak world in patches"
	wrapX _ dimensions x asFloat.
	wrapY _ dimensions y asFloat.
	pixelsPerPatch _ 2.
	super extent: dimensions * pixelsPerPatch.
	self assuredPlayer assureUniClass.
	self clearAll.  "be sure this is done once in case setup fails to do it"
	autoChanged _ true.
	self leftEdgeMode: #wrap.
	self rightEdgeMode: #wrap.
	self topEdgeMode: #wrap.
	self bottomEdgeMode: #wrap.

	turtlesDictSemaphore _ Semaphore forMutualExclusion.
! !

!KedamaMorph methodsFor: 'initialization' stamp: 'yo 9/9/2004 15:15'!
initializePatch

	| f |
	f _ self player addPatchVarNamed: #patch.
	patchesToDisplay _ Array new: 0.
	self addToPatchDisplayList: f.
	^ f.
! !

!KedamaMorph methodsFor: 'initialization' stamp: 'yo 9/28/2004 14:12'!
initializeTurtlesDict

	turtlesDict _ IdentityDictionary new.
	turtlesToDisplay _ Array new.
! !

!KedamaMorph methodsFor: 'private' stamp: 'yo 9/14/2004 18:47'!
isVisible: examplerPlayer

	| turtleMorph |
	turtleMorph _ examplerPlayer costume.
	turtleMorph visible ifFalse: [^ false].
	turtleMorph owner isRenderer ifFalse: [^ true].
	^ turtleMorph owner visible.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/20/2004 09:28'!
lastWhoOf: exampler

	| turtles |
	turtles _ turtlesDict at: exampler ifAbsent: [nil].
	^ (turtles arrays first) at: turtles arrays first size.
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 6/17/2004 09:51'!
leftEdgeMode

	^ leftEdgeMode.
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 1/20/2005 19:52'!
leftEdgeMode: aSymbol

	leftEdgeMode _ aSymbol asSymbol.
	leftEdgeMode == #wrap ifTrue: [
		leftEdgeModeMnemonic _ 1.
		^ self
	].
	leftEdgeMode == #stick ifTrue: [
		leftEdgeModeMnemonic _ 2.
		^ self
	].
	(leftEdgeMode == #bounce or: [leftEdgeMode == #bouncing]) ifTrue: [
		leftEdgeModeMnemonic _ 3.
		^ self
	].
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 9/16/2004 18:59'!
leftEdgeModeMnemonic

	^ leftEdgeModeMnemonic.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/14/2004 18:39'!
makePrototypeOfExampler: examplerPlayer

	^ self makePrototypeOfExampler: examplerPlayer color: nil.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/27/2004 16:29'!
makePrototypeOfExampler: examplerPlayer color: cPixel

	| array inst info |
	array _ examplerPlayer turtles.
	info _ array info.
	array size > 0 ifTrue: [
		inst _ array makePrototypeFromFirstInstance.
		cPixel ifNotNil: [inst at: (info at: #color) put: cPixel].
		^ inst.
	].

	inst _ Array new: array instSize.
	info associationsDo: [:assoc |
		inst at: (assoc value) put: (examplerPlayer perform: (Utilities getterSelectorFor: assoc key)).
	].
	cPixel ifNotNil: [inst at: (info at: #color) put: cPixel] ifNil: [inst at: (info at: #color) put: ((examplerPlayer getColor pixelValueForDepth: 32) bitAnd: 16rFFFFFF)].
	inst at: (info at: #visible) put: ((inst at: (info at: #visible)) ifTrue: [1] ifFalse: [0]).
	^ inst.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/27/2004 16:29'!
makeReplicatedTurtles: count examplerPlayer: tp color: c ofPrototype: prototype randomize: randomizeFlag

	| array inst |
	array _ tp turtles.

	inst _ prototype ifNil: [self makePrototypeOfExampler: tp color: c].

	turtlesDictSemaphore critical: [
		array addTurtlesCount: count ofPrototype: inst for: self randomize: randomizeFlag.
	].
	self calcTurtlesCount.
	self changed.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/28/2004 14:13'!
makeTurtles: count examplerPlayer: tp color: c ofPrototype: prototype turtles: turtles randomize: randomizeFlag

	| array inst |
	array _ tp turtles.
	(turtlesDict keys includes: tp) ifFalse: [
		self addToTurtleDisplayList: tp.
		turtlesDict at: tp put: (array _ turtles).
	].

	inst _ prototype ifNil: [self makePrototypeOfExampler: tp color: c].

	turtlesDictSemaphore critical: [array setTurtlesCount: count prototype: inst for: self randomize: randomizeFlag].
	self calcTurtlesCount.
	self changed.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/28/2004 14:16'!
makeTurtlesAtPositionsIn: positionAndColorArray examplerPlayer: tp ofPrototype: prototype

	| array inst |
	array _ tp turtles.

	inst _ prototype ifNil: [self makePrototypeOfExampler: tp].

	turtlesDictSemaphore critical: [array addTurtlesCount: positionAndColorArray first size ofPrototype: inst for: self positionAndColorArray: positionAndColorArray].
	self calcTurtlesCount.
	self changed.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/14/2004 17:34'!
nextTurtleID

	^ lastTurtleID _ lastTurtleID + 1.
! !

!KedamaMorph methodsFor: 'menu' stamp: 'yo 8/18/2005 12:09'!
offerCostumeViewerMenu: aMenu
	aMenu add: 'add a new patch variable' translated action: #newPatch.
	aMenu balloonTextForLastItem: 'Add a new patch variable' translated.
	aMenu add: 'add a new breed of turtle' translated action: #newTurtle.
	aMenu balloonTextForLastItem: 'Add a new turtle' translated.
	aMenu addLine.
! !

!KedamaMorph methodsFor: 'drawing' stamp: 'yo 6/4/2004 18:13'!
patchesToDisplayAsString

	^ String streamContents: [:strm |
		strm nextPutAll: '#('.
		patchesToDisplay do: [:p |
			strm nextPutAll: p externalName.
			strm nextPut: Character space.
		].
		strm nextPutAll: ')'.
	].
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 6/1/2004 16:53'!
pixelsPerPatch

	^ pixelsPerPatch
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 6/11/2004 01:06'!
pixelsPerPatch: anInteger
	"Set the width of one patch in pixels. Larger numbers scale up this StarSqueak world, but numbers larger than 2 or 3 result in a blocky look. The useful range is 1 to 10."

	pixelsPerPatch _ (anInteger rounded max: 1) min: 10.
	super extent: dimensions * pixelsPerPatch.
	self recreateMagnifiedDisplayForm
! !

!KedamaMorph methodsFor: 'private-primitives' stamp: 'yo 3/7/2005 13:27'!
primSetRandomSeed: seed

	<primitive: 'kedamaSetRandomSeed' module: 'KedamaPlugin'>
	^ nil.
! !

!KedamaMorph methodsFor: 'private-primitives' stamp: 'yo 6/18/2004 15:18'!
primZoom: src into: dst srcWidth: sWidth height: sHeight multX: xFactor y: yFactor

	<primitive: 'zoomBitmap' module: 'KedamaPlugin'>
	"^ KedamaSqueakPlugin doPrimitive: #zoomBitmap."
	^ nil.
! !

!KedamaMorph methodsFor: 'utils' stamp: 'yo 3/23/2005 12:24'!
random: range
	"Answer a random integer between 0 and range."

	| r val |
	<primitive: 'randomRange' module: 'KedamaPlugin'>
	r _ range < 0 ifTrue: [range negated] ifFalse: [range].
	RandomSeed _ ((RandomSeed * 1309) + 13849) bitAnd: 65535.
	val _ (RandomSeed * (r + 1)) >> 16.
	^ range < 0 ifTrue: [val negated] ifFalse: [^ val].

! !

!KedamaMorph methodsFor: 'private' stamp: 'yo 6/6/2004 23:25'!
recreateMagnifiedDisplayForm

	magnifiedDisplayForm _ Form extent: self dimensions * pixelsPerPatch depth: 32.
	self changed.
! !

!KedamaMorph methodsFor: 'drawing' stamp: 'yo 9/9/2004 15:16'!
removeAllFromPatchDisplayList

	patchesToDisplay _ #().
! !

!KedamaMorph methodsFor: 'drawing' stamp: 'yo 9/9/2004 15:19'!
removeAllFromTurtleDisplayList

	turtlesToDisplay _ #().
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/14/2004 20:28'!
removeFromTurtleDisplayList: examplerPlayer

	turtlesToDisplay _ turtlesToDisplay copyWithout: examplerPlayer.
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 6/17/2004 09:51'!
rightEdgeMode

	^ rightEdgeMode.
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 1/20/2005 19:52'!
rightEdgeMode: aSymbol

	rightEdgeMode _ aSymbol asSymbol.
	rightEdgeMode == #wrap ifTrue: [
		rightEdgeModeMnemonic _ 1.
		^ self
	].
	rightEdgeMode == #stick ifTrue: [
		rightEdgeModeMnemonic _ 2.
		^ self
	].
	(rightEdgeMode == #bounce or: [rightEdgeMode == #bouncing]) ifTrue: [
		rightEdgeModeMnemonic _ 3.
		^ self
	].
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 9/16/2004 18:59'!
rightEdgeModeMnemonic

	^ rightEdgeModeMnemonic.
! !

!KedamaMorph methodsFor: 'menu' stamp: 'yo 6/16/2004 08:56'!
setScale

	| reply |
	reply _ FillInTheBlank
		request: 'Set the number of pixels per patch (a number between 1 and 10)?'
		 initialAnswer: pixelsPerPatch printString.
	reply isEmpty ifTrue: [^ self].
	self pixelsPerPatch: reply asNumber.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/27/2004 16:30'!
setTurtlesCount: count examplerPlayer: tp color: cPixel

	| prototype |
	prototype _ self makePrototypeOfExampler: tp color: cPixel.
	turtlesDictSemaphore critical: [(tp turtles) setTurtlesCount: count prototype: prototype for: self randomize: true].
	self calcTurtlesCount.
! !

!KedamaMorph methodsFor: 'setup' stamp: 'yo 6/1/2004 16:53'!
setup
	"Subclasses should override this to setup the initial conditions of this StarSqueak world. The method should start with 'self clearAll'."

	self clearAll.
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 6/17/2004 09:52'!
topEdgeMode

	^ topEdgeMode.
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 1/20/2005 19:53'!
topEdgeMode: aSymbol

	topEdgeMode _ aSymbol asSymbol.
	topEdgeMode == #wrap ifTrue: [
		topEdgeModeMnemonic _ 1.
		^ self
	].
	topEdgeMode == #stick ifTrue: [
		topEdgeModeMnemonic _ 2.
		^ self
	].
	(topEdgeMode == #bounce or: [topEdgeMode == #bouncing])  ifTrue: [
		topEdgeModeMnemonic _ 3.
		^ self
	].
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 9/16/2004 18:59'!
topEdgeModeMnemonic

	^ topEdgeModeMnemonic.
! !

!KedamaMorph methodsFor: 'etoys' stamp: 'yo 6/2/2004 16:36'!
turtleCount

	^ turtleCount.
! !

!KedamaMorph methodsFor: 'turtles' stamp: 'yo 9/27/2004 16:30'!
turtlesCountOf: exampler

	| array |
	array _ exampler turtles.
	array ifNil: [^ 0].
	^ array size.
! !

!KedamaMorph methodsFor: 'drawing' stamp: 'yo 9/9/2004 15:21'!
turtlesToDisplayAsString

	^ String streamContents: [:strm |
		strm nextPutAll: '#('.
		turtlesToDisplay do: [:p |
			strm nextPutAll: p externalName.
			strm nextPut: Character space.
		].
		strm nextPutAll: ')'.
	].
! !

!KedamaMorph methodsFor: 'event handling' stamp: 'yo 8/10/2004 12:23'!
wantsDroppedMorph: aMorph event: anEvent

	^ aMorph isMemberOf: SketchMorph.
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 9/9/2004 15:08'!
wrapX

	^ wrapX.
! !

!KedamaMorph methodsFor: 'accessing' stamp: 'yo 9/9/2004 15:08'!
wrapY

	^ wrapY.
! !

!KedamaMorph methodsFor: 'private' stamp: 'yo 6/7/2004 12:04'!
zoom: src into: dst factor: f

	src unhibernate.
	dst unhibernate.
	^ self primZoom: src bits into: dst bits srcWidth: src width height: src height multX: f y: f.
! !

Morph subclass: #KedamaPatchMorph
	instanceVariableNames: 'form displayMax shiftAmount useLogDisplay displayForm diffusionRate scaledEvaporationRate sniffRange formChanged tmpForm autoChanged displayType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaPatchMorph commentStamp: 'yo 6/18/2004 18:31' prior: 0!
I represent the patch variable.
!

!KedamaPatchMorph classSide methodsFor: 'scripting' stamp: 'yo 3/7/2005 12:22'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(

	(kedama (
		(slot evaporationRate 'evaporation rate' Number readWrite Patch getEvaporationRate Patch setEvaporationRate:)
		(slot diffusionRate 'diffusion rate' Number readWrite Patch getDiffusionRate Patch setDiffusionRate:)
		(slot sniffRange 'sniff range' Number readWrite Patch getSniffRange Patch setSniffRange:)
		(slot shiftAmount 'shift amount when log-based color conversion is not used' Number readWrite Player getDisplayShiftAmount Player setDisplayShiftAmount:)
		(slot scaleMax 'scale when log-based color conversion is used' Number readWrite Player getDisplayScaleMax Player setDisplayScaleMax:)
		"(slot useLogDisplay 'log-based color conversion' Boolean readWrite Player getUseLogDisplay Player setUseLogDisplay:)"
		(slot displayType 'how to map the value in cells to color' PatchDisplayMode readWrite Player getDisplayType Player setDisplayType:)
		(slot color 'The color of the object' Color readWrite Player getColor  Player  setColor:)

		"(slot autoUpdate 'Updating screen always' Boolean readWrite Player getAutoUpdate  Player setAutoUpdate:)"
		(command clear 'clear all patch')
		(command diffusePatchVariable 'diffuse')
		(command decayPatchVariable 'decay')

		(command redComponentInto: 'split red component into another patch' Patch)
		(command greenComponentInto: 'split green component into another patch' Patch)
		(command blueComponentInto: 'split blue component into another patch' Patch)

		(command redComponentFrom: 'merge red component from another patch' Patch)
		(command greenComponentFrom: 'merge green component from another patch' Patch)
		(command blueComponentFrom: 'merge blue component from another patch' Patch)

)))
! !

!KedamaPatchMorph classSide methodsFor: 'accessing' stamp: 'yo 4/6/2005 14:17'!
classVersion

	^ 1.
! !

!KedamaPatchMorph classSide methodsFor: 'scripting' stamp: 'yo 1/20/2005 21:24'!
defaultNameStemForInstances

	^ 'patch' translated.
! !

!KedamaPatchMorph classSide methodsFor: 'class initialization' stamp: 'yo 6/5/2004 23:30'!
initialize
"
	self initialize
"
	Smalltalk addToStartUpList: self.
	self startUp.
! !

!KedamaPatchMorph classSide methodsFor: 'instance creation' stamp: 'yo 2/2/2005 17:45'!
newExtent: aPoint

	| inst |
	inst _ self new.
	inst newExtent: aPoint.
	^ inst.
! !

!KedamaPatchMorph classSide methodsFor: 'class initialization' stamp: 'yo 6/10/2004 13:41'!
startUp
"
	self startUp
"
	self allInstancesDo: [:e | e startUp].
! !

!KedamaPatchMorph classSide methodsFor: 'class initialization' stamp: 'yo 10/11/2004 18:58'!
startUp: aBoolean
"
	self startUp
"

	self startUp.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/7/2004 10:21'!
autoChanged

	^ autoChanged.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/7/2004 10:21'!
autoChanged: aBoolean

	autoChanged _ aBoolean.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 8/18/2005 10:56'!
basicPixelAtX: xPos y: yPos

	| x y i |
	x _ xPos truncated.
	y _ yPos truncated.
	((x < 0) or: [y < 0]) ifTrue: [^ 0].
	((x >= form width) or: [y >= form height]) ifTrue: [^ 0].
	i _ ((y * form width) + x) + 1.
	form bits class == ByteArray ifTrue: [form unhibernate].
	^ form bits at: i.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 11/30/2005 12:07'!
basicPixelAtX: xPos y: yPos put: value

	| x y i v |
	x _ xPos truncated.
	y _ yPos truncated.
	v _ value asInteger min: 16rFFFFFFFF max: 0.
	((x < 0) or: [y < 0]) ifTrue: [^ self].
	((x >= form width) or: [y >= form height]) ifTrue: [^ self].
	i _ ((y * form width) + x) + 1.
	form bits class == ByteArray ifTrue: [form unhibernate].
	form bits at: i put: v.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 9/30/2004 11:46'!
basicUphillForTurtleX: tX turtleY: tY turtleHeading: tH
	"Answer the heading the points in the direction of increasing value for the given patch variable. If there is no gradient, or if the turtle is outside the world bounds, answer the turtles current heading."

	| turtleX turtleY startX endX startY endY maxVal rowOffset thisVal maxValX maxValY |
	turtleX := tX truncated.
	turtleY := tY truncated.
	turtleX := turtleX max: 0.
	turtleY := turtleY max: 0.
	turtleX := turtleX min: form width - 1.
	turtleY := turtleY min: form height - 1.
	startX := turtleX - sniffRange max: 0.
	endX := (turtleX + sniffRange) min: (form width - 1).
	startY := (turtleY - sniffRange) max: 0.
	endY := (turtleY + sniffRange) min: (form height - 1).
	"form bits class == Bitmap ifFalse: [form unhibernate]."
	maxVal := form bits at: turtleY * form width + turtleX + 1.
	maxValX := nil.
	startY to: endY
		do: 
			[:y | 
			rowOffset := y * form width.
			startX to: endX
				do: 
					[:x | 
					thisVal := form bits at: rowOffset + x + 1.
					thisVal > maxVal 
						ifTrue: 
							[maxValX := x.
							maxValY := y.
							maxVal := thisVal]]].
	nil = maxValX ifTrue: [^ (90.0 - tH radiansToDegrees) \\ 360.0].
	^ (((maxValX - turtleX) @ (maxValY - turtleY)) degrees + 90.0) \\ 360.0
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 10/13/2004 01:59'!
blueComponentFrom: another

	| pix anotherPix |
	0 to: self height - 1 do: [:y |
		0 to: self width -1 do: [:x |
			pix _ self pixelAtX: x y: y.
			anotherPix _ (another pixelAtX: x y: y) bitAnd: 16rFF.
			pix _ (pix bitAnd: 16rFFFF00) bitOr: (anotherPix).
			self pixelAtX: x y: y put: pix.
		].
	].
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 10/13/2004 00:22'!
blueComponentInto: another

	0 to: self height - 1 do: [:y |
		0 to: self width -1 do: [:x |
			another pixelAtX: x y: y put: ((self pixelAtX: x y: y) bitAnd: 16rFF).
		].
	].
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/3/2004 21:31'!
clear

	form fillColor: Color transparent.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/5/2004 20:21'!
color: aColor

	super color: aColor.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/3/2004 14:00'!
colorAtX: xPos y: yPos

	^ Color colorFromPixelValue: (self pixelAtX: xPos y: yPos) depth: 32.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/3/2004 21:29'!
colorAtX: xPos y: yPos put: aColor

	| pixel |
	pixel _ aColor pixelValueForDepth: 32.
	self pixelAtX: xPos y: yPos put: pixel.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'private' stamp: 'yo 3/7/2005 12:19'!
convertToCurrentVersion: varDict refStream: smartRefStrm
	
	varDict at: 'useLogDisplay' ifPresent: [ :x | 
		displayType _ x ifTrue: [#logScale] ifFalse: [#linear].
	].
	displayType ifNil: [displayType _ #logScale].
	^super convertToCurrentVersion: varDict refStream: smartRefStrm.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 8/18/2005 10:54'!
diffuse

	| dim newV |
	dim _ form extent.
	form bits class == ByteArray ifTrue: [form unhibernate].
	newV _ Bitmap new: form bits size.
	self primDiffuseFrom: form bits
		to: newV
		width: dim x
		height: dim y
		delta: diffusionRate truncated.
	form bits: newV.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/3/2004 20:48'!
diffusionRate

	^ diffusionRate.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/3/2004 20:48'!
diffusionRate: newRate
	"Set the diffusion rate to an integer between 0 and 10. The diffusion rate gives the number of patches on one size of the area averaged to compute the next value of the variable for a given patch. Larger numbers cause faster diffusion. Zero means no diffusion."

	diffusionRate _ (newRate rounded max: 0) min: 10.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/15/2004 05:12'!
displayMax

	^ displayMax at: 1.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/16/2004 04:34'!
displayMax: anInteger

	displayMax _ WordArray with: (anInteger asInteger min: 16rFFFFFFFF max: 0).
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'drawing' stamp: 'yo 8/18/2005 11:42'!
displayPatchVariableOn: aForm

	| patchVar pixelValue |
	form ifNil: [^self].

	formChanged ifTrue: [
		"displayForm fillColor: Color transparent."
		pixelValue _ (self color pixelValueForDepth: 32) bitAnd: 16rFFFFFF.
		form bits class == ByteArray ifTrue: [form unhibernate].
		patchVar _ form bits.
		displayForm bits class == ByteArray ifTrue: [displayForm unhibernate].
		displayType = #linear ifTrue: [
			self primMakeMaskOf: patchVar in: displayForm bits colorPixel: pixelValue shift: shiftAmount.
		].
		displayType = #logScale ifTrue: [
			self primMakeMaskOf: patchVar in: displayForm bits colorPixel: pixelValue max: displayMax.
		].
		displayType = #color ifTrue: [
			form displayOn: displayForm.
			displayForm fixAlpha.
		].
	].

	tmpForm fillColor: Color black.
	displayForm displayOn: tmpForm at: 0@0 rule: 24.
	aForm == tmpForm ifFalse: [
		displayForm displayOn: aForm at: 0@0 rule: 24.
	].
	formChanged _ false.

! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/14/2004 16:21'!
displayShiftAmount

	^ shiftAmount.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/14/2004 16:20'!
displayShiftAmount: anInteger

	shiftAmount _ anInteger min: 8 max: -32.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 3/7/2005 12:05'!
displayType

	^ displayType.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 4/5/2005 10:11'!
displayType: aSymbol

	displayType _ aSymbol asSymbol.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'drawing' stamp: 'yo 3/7/2005 11:57'!
drawOn: aCanvas

	formChanged ifTrue: [
		self displayPatchVariableOn: tmpForm.
	].
	aCanvas drawImage: tmpForm at: self innerBounds origin.

	autoChanged ifTrue: [self changed].
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 8/18/2005 10:56'!
evaporate

	form bits class == ByteArray ifTrue: [form unhibernate].
	self primEvaporate: form bits rate: scaledEvaporationRate.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/3/2004 20:50'!
evaporationRate

	^ 1024 - scaledEvaporationRate.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/3/2004 20:50'!
evaporationRate: newRate
	"Set the evaporation rate. The useful range is 0 to 25 or so. Larger numbers cause faster evaporation. Zero means no evaporization."

	scaledEvaporationRate _ ((1024 - newRate truncated) max: 1) min: 1024.
! !

!KedamaPatchMorph methodsFor: 'drawing' stamp: 'yo 6/6/2004 23:23'!
formChanged

	formChanged _ true.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 10/13/2004 01:06'!
greenComponentFrom: another

	| pix anotherPix |
	0 to: self height - 1 do: [:y |
		0 to: self width -1 do: [:x |
			pix _ self pixelAtX: x y: y.
			anotherPix _ (another pixelAtX: x y: y) bitAnd: 16rFF.
			pix _ (pix bitAnd: 16rFF00FF) bitOr: (anotherPix bitShift: 8).
			self pixelAtX: x y: y put: pix.
		].
	].
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 10/13/2004 00:22'!
greenComponentInto: another

	0 to: self height - 1 do: [:y |
		0 to: self width -1 do: [:x |
			another pixelAtX: x y: y put: (((self pixelAtX: x y: y) bitShift: -8) bitAnd: 16rFF).
		].
	].
! !

!KedamaPatchMorph methodsFor: 'initialization' stamp: 'yo 3/7/2005 12:17'!
initialize

	super initialize.

	self evaporationRate: 6.
	self diffusionRate: 1.
	self sniffRange: 1.

	displayType _ #logScale.
	displayMax _ WordArray with: 1024.
	shiftAmount _ -2.

	autoChanged _ true.

	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'initialization' stamp: 'yo 2/2/2005 17:37'!
initializeForm: aForm 

	form _ aForm. 
	form fillColor: Color transparent.

	displayForm _ (Form extent: aForm extent depth: 32).
	tmpForm _ (Form extent: aForm extent depth: 32).
	tmpForm fillColor: Color black.

	super extent: form extent.
	self changed.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'initialization' stamp: 'yo 2/2/2005 17:46'!
newExtent: aPoint

	self initializeForm: (Form extent: aPoint depth: 32).
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 8/18/2005 10:38'!
pixelAtX: xPos y: yPos

	| x y i |
	x _ xPos truncated.
	y _ yPos truncated.
	((x < 0) or: [y < 0]) ifTrue: [^ 0].
	((x >= form width) or: [y >= form height]) ifTrue: [^ 0].
	i _ ((y * form width) + x) + 1.
	form bits class == ByteArray ifTrue: [form unhibernate].
	^ form bits at: i.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 11/30/2005 12:07'!
pixelAtX: xPos y: yPos put: value

	| x y i v |
	x _ xPos truncated.
	y _ yPos truncated.
	v _ (value asInteger max: 0) bitAnd: 16rFFFFFFFF.
	((x < 0) or: [y < 0]) ifTrue: [^ self].
	((x >= form width) or: [y >= form height]) ifTrue: [^ self].
	i _ ((y * form width) + x) + 1.
	form bits class == ByteArray ifTrue: [form unhibernate].
	form bits at: i put: v.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 8/18/2005 10:55'!
pixelsAtXArray: xArray yArray: yArray into: aWordArray

	form bits class == ByteArray ifTrue: [form unhibernate].
	self primPixelsAtXArray: xArray yArray: yArray bits: form bits width: form width height: form height into: aWordArray.
! !

!KedamaPatchMorph methodsFor: 'primitives' stamp: 'yo 6/3/2004 20:45'!
primDiffuseFrom: srcBitmap to: dstBitmap width: width height: height delta: delta 
	"Diffuse the integer values of the source patch variable Bitmap into the output Bitmap. Each cell of the output is the average of the NxN area around it in the source, where N = (2 * delta) + 1."

	| area startY endY startX endX sum rowStart |
	<primitive: 'primitiveDiffuseFromToWidthHeightDelta' module: 'StarSqueakPlugin'>
	area := (2 * delta + 1) * (2 * delta + 1).
	1 to: height
		do: 
			[:y | 
			startY := y - delta.
			startY := startY max: 1.
			endY := y + delta.
			endY := endY min: height.
			1 to: width
				do: 
					[:x | 
					startX := x - delta.
					startX := startX max: 1.
					endX := x + delta.
					endX := endX min: width.
					sum := 0.
					startY to: endY
						do: 
							[:y2 | 
							rowStart := (y2 - 1) * width.
							startX to: endX do: [:x2 | sum := sum + (srcBitmap at: rowStart + x2)]].
					dstBitmap at: (y - 1) * width + x put: sum // area]]! !

!KedamaPatchMorph methodsFor: 'primitives' stamp: 'yo 6/3/2004 20:50'!
primEvaporate: aBitmap rate: rate
	"Evaporate the integer values of the source Bitmap at the given rate, an integer between 0 and 1024, where 1024 is a scale factor of 1.0 (i.e., no evaporation). That is, replace each integer element v with (rate * v) / 1024."

	<primitive: 'primitiveEvaporateRate' module: 'StarSqueakPlugin'>
	1 to: aBitmap size do: [:i |
		aBitmap at: i put: (((aBitmap at: i) * rate) bitShift: -10)].
! !

!KedamaPatchMorph methodsFor: 'primitives' stamp: 'yo 6/26/2004 20:07'!
primMakeMaskOf: dataBits in: maskBits colorPixel: pixel max: max

	| highMask alpha maxLog data |
	<primitive: 'makeMaskLog' module: 'KedamaPlugin'>
	"^ KedamaSqueakPlugin doPrimitive: #makeMaskLog."


	highMask _ 16rFF000000.
	"maxLog _ self cCode: 'log(max)' inSmalltalk: [max first ln]."
	maxLog _ max first ln.
	maxLog _ 255.0 / maxLog.

	1 to: dataBits size do: [:i |
		data _ dataBits at: i.
		data = 0 ifTrue: [alpha _ 0] ifFalse: [
			"alpha _ ((255.0 / maxLog) * (self cCode: 'log(data)' inSmalltalk: [data ln])) asInteger."
			alpha _ (maxLog * (data ln)) asInteger.

		].
		(alpha > 255) ifTrue: [alpha _ 255].
		maskBits at: i put: (((alpha << 24) bitAnd: highMask) bitOr: pixel).
	].
	^ self.
! !

!KedamaPatchMorph methodsFor: 'primitives' stamp: 'yo 6/18/2004 15:18'!
primMakeMaskOf: dataBits in: maskBits colorPixel: pixel shift: shift

	| highMask data alpha |
	<primitive: 'makeMask' module: 'KedamaPlugin'>
	"^ KedamaSqueakPlugin doPrimitive: #makeMask."

	highMask _ 16rFF000000.
	1 to: dataBits size do: [:i |
		data _ dataBits at: i.
		alpha _ data bitShift: shift.
		(alpha > 255) ifTrue: [alpha _ 255].
		maskBits at: i put: (((alpha << 24) bitAnd: highMask) bitOr: pixel).
	].

	^ self.
! !

!KedamaPatchMorph methodsFor: 'primitives' stamp: 'yo 8/19/2005 12:24'!
primPixelsAtXArray: xArray yArray: yArray bits: bits width: width height: height into: aWordArray
	| x y formIndex val |
	<primitive: 'primPixelsAtXY' module: 'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primPixelsAtXY."
	1 to: aWordArray size do: [:i |
		val _ nil.
		x _ (xArray at: i) truncated.
		y _ (yArray at: i) truncated.
		((x < 0) or: [y < 0]) ifTrue: [val _ 0].
		((x >= form width) or: [y >= form height]) ifTrue: [val _ 0].
		val ifNil: [
			formIndex _ ((y * form width) + x) + 1.
			val _ bits at: formIndex.
		].
		aWordArray at: i put: val.
	].
! !

!KedamaPatchMorph methodsFor: 'primitives' stamp: 'yo 10/25/2004 16:56'!
primSetPixelsAtXArray: xArray yArray: yArray bits: bits width: width height: height value: value

	| v |
	<primitive: 'primSetPixelsAtXY' module: 'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primSetPixelsAtXY."

	value isNumber ifTrue: [v _ value].
	1 to: xArray size do: [:i |
		value isNumber ifFalse: [
			v _ value at: i.
		].		
		self pixelAtX: (xArray at: i) y: (yArray at: i) put: v.
	].
! !

!KedamaPatchMorph methodsFor: 'primitives' stamp: 'yo 9/30/2004 10:35'!
primUpHillX: tX y: tY heading: tH bits: bits width: width height: height sniffRange: s

	<primitive: 'primUpHill' module: 'KedamaPlugin'>
	"^ KedamaPlugin doPrimitive: #primUpHill."
	^ nil
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 10/13/2004 01:05'!
redComponentFrom: another

	| pix anotherPix |
	0 to: self height - 1 do: [:y |
		0 to: self width -1 do: [:x |
			pix _ self pixelAtX: x y: y.
			anotherPix _ (another pixelAtX: x y: y) bitAnd: 16rFF.
			pix _ (pix bitAnd: 16r00FFFF) bitOr: (anotherPix bitShift: 16).
			self pixelAtX: x y: y put: pix.
		].
	].
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 10/13/2004 00:21'!
redComponentInto: another

	0 to: self height - 1 do: [:y |
		0 to: self width -1 do: [:x |
			another pixelAtX: x y: y put: (((self pixelAtX: x y: y) bitShift: -16) bitAnd: 16rFF).
		].
	].
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 8/18/2005 10:55'!
setPixelsAtXArray: xArray yArray: yArray value: value

	form bits class == ByteArray ifTrue: [form unhibernate].
	self primSetPixelsAtXArray: xArray yArray: yArray bits: form bits width: form width height: form height value: value.
	self formChanged.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/4/2004 01:37'!
sniffRange

	^ sniffRange.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 6/4/2004 01:36'!
sniffRange: aNumber

	sniffRange _ aNumber.
! !

!KedamaPatchMorph methodsFor: 'initialization' stamp: 'yo 6/10/2004 13:41'!
startUp

	form unhibernate.
	displayForm unhibernate.
	tmpForm unhibernate.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 8/18/2005 10:54'!
uphillForTurtleX: tX turtleY: tY turtleHeading: tH
	"Answer the heading the points in the direction of increasing value for the given patch variable. If there is no gradient, or if the turtle is outside the world bounds, answer the turtles current heading."

	| ret |
	form bits class == ByteArray ifTrue: [form unhibernate].
	ret _ self primUpHillX: tX y: tY heading: tH bits: form bits width: form width height: form height sniffRange: sniffRange.
	ret ifNotNil: [^ ret].
	^ self basicUphillForTurtleX: tX turtleY: tY turtleHeading: tH.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 3/7/2005 12:19'!
useLogDisplay

	^ displayType == #logScale.
! !

!KedamaPatchMorph methodsFor: 'commands and slots' stamp: 'yo 3/7/2005 12:20'!
useLogDisplay: aBoolean

	displayType _ aBoolean ifTrue: [#logScale] ifFalse: [#linear].
	self formChanged.
! !

Morph subclass: #KedamaTurtleMorph
	instanceVariableNames: 'kedamaWorld turtleCount isGroup'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaTurtleMorph commentStamp: 'yo 6/18/2004 18:33' prior: 0!
The exampler of turtles.
!

!KedamaTurtleMorph classSide methodsFor: 'scripting' stamp: 'yo 3/7/2005 08:46'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."
	"self halt."
	^ #(

	('kedama turtle' (
		(command die 'delete this turtle')

		(slot upHill 'uphill of the implicit at my location' Number readOnly Player getUphillIn: unspecified unspecified)
		(slot bounceOn 'detect collision and bounce back' Boolean readOnly Player bounceOn: unspecified unspecified)
		(slot bounceOnColor 'detect collision and bounce back on a color' Boolean readOnly Player bounceOnColor: unspecified unspecified)
		(slot patchValueIn 'get the value at this position' Number readWrite Number getPatchValueIn: Number setPatchValueIn:to:)
		(slot distanceTo 'The distance to another turtle' Number readOnly Player getDistanceTo: unused unused)
		(slot angleTo 'The angle to another turtle' Number readOnly Player getAngleTo: unused unused)

		(slot getReplicated 'returns a copy of this turtle' Player readOnly Player getReplicated unused unused)
		(slot x 'The x coordinate' Number readWrite Player getX Player setX:)
		(slot y  	'The y coordinate' Number readWrite Player 	getY Player setY:)
		(slot heading 'Which direction the object is facing.  0 is straight up' Number readWrite Player getHeading Player setHeading:)
		(command forward: 'Moves the object forward in the direction it is heading' Number)
		(command turn: 'Change the heading of the object by the specified amount' Number)
			(slot color 'The color of the object' Color readWrite Player getColor  Player  setColor:)
	"		(slot headingTheta 'The angle, in degrees, that my heading vector makes with the positive x-axis' Number readWrite Player getHeadingTheta Player setHeadingTheta:)

			(slot theta 'The angle between the positive x-axis and the vector connecting the origin to the object''s position' Number readWrite Player getTheta Player setTheta: )
"
		(slot turtleVisible 'The flag that governs the visibility of turtle' Boolean readWrite Player getTurtleVisible Player setTurtleVisible:)
			"(command turtleShow 'make the object visible')
			(command turtleHide 'make the object invisible')"

		(slot turtleOf 'returns a turtle of specified breed at my position.' Player readOnly Player getTurtleOf: unused unused)
		"(slot normal 'The normal for bouncing' Number readWrite Player getNormal Player  setNormal:)"


	))
	('kedama turtle breed' (
		(slot turtleCount 'set the number of turtles' Number readWrite Number getTurtleCount Number setTurtleCount:)
		(slot grouped 'turtles bahaves as one connected objects' Boolean readWrite Boolean getGrouped Boolean setGrouped:)
	))

	('kedama turtle color' (
		(slot redComponentIn 'The red component in specified patch.' Number readWrite Player getRedComponentIn: Player setRedComponentIn:to:)
		(slot greenComponentIn 'The green component in specified patch.' Number readWrite Player getGreenComponentIn: Player setGreenComponentIn:to:)
		(slot blueComponentIn 'The blue component in specified patch.' Number readWrite Player getBlueComponentIn: Player setBlueComponentIn:to:)
		(command colorFromPatch: 'make my color specified in the patch' Patch)
		(command colorToPatch: 'store my color into the patch' Patch)
	))

)

! !

!KedamaTurtleMorph classSide methodsFor: 'class initialization' stamp: 'yo 6/9/2004 12:50'!
cleanUp
"
	self cleanUp
"
	self allInstancesDo: [:e | e cleanUp].
! !

!KedamaTurtleMorph classSide methodsFor: 'scripting' stamp: 'yo 10/22/2004 08:12'!
copyAllMethodsAgain
"
	KedamaTurtleMorph copyAllMethodsAgain.
"
	KedamaTurtleMorph allInstancesDo: [:e | e player ifNotNil: [e player copyAllMethodsAgain]].
! !

!KedamaTurtleMorph classSide methodsFor: 'scripting' stamp: 'yo 1/20/2005 21:24'!
defaultNameStemForInstances

	^ 'turtle' translated.
! !

!KedamaTurtleMorph classSide methodsFor: 'class initialization' stamp: 'yo 10/24/2004 18:58'!
useKedamaFloatArray
"
	KedamaTurtleMorph useKedamaFloatArray.
"
	KedamaTurtleMorph allInstancesDo: [:e | e player ifNotNil: [e player turtles useKedamaFloatArray]].
! !

!KedamaTurtleMorph methodsFor: 'viewer access' stamp: 'yo 9/29/2004 15:13'!
addCustomMenuItems: aCustomMenu hand: aHandMorph
	"Include our modest command set in the ctrl-menu"

	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	self player ifNotNil: [
		aCustomMenu addLine.
		aCustomMenu add: 'copy methods' target: self player action: #copyAllMethodsAgain.
	].
! !

!KedamaTurtleMorph methodsFor: 'viewer access' stamp: 'yo 6/11/2004 11:18'!
categoriesForViewer
	"Answer a list of symbols representing the categories to offer in the viewer, in order"
	"^ super categoriesForViewer."

	| aList |
	aList _ OrderedCollection new.
	aList addAllFirstUnlessAlreadyPresent: (self class additionsToViewerCategories collect:
				[:categorySpec | categorySpec first]).
	^ aList
! !

!KedamaTurtleMorph methodsFor: 'initialization' stamp: 'yo 6/9/2004 12:51'!
cleanUp

	self player: nil.
	self actorState: nil.
! !

!KedamaTurtleMorph methodsFor: 'group turtle creation' stamp: 'yo 10/11/2004 16:00'!
createTurtlesAsIn: aForm originAt: origin

	| c xArray yArray colorArray newX newY |
	xArray _ OrderedCollection new: aForm width * aForm height.
	yArray _ OrderedCollection new: aForm width * aForm height.
	colorArray _ OrderedCollection new: aForm width * aForm height.
	0 to: aForm height do: [:y |
		0 to: aForm width do: [:x |
			c _ aForm colorAt: (x@y).
			c isTransparent ifFalse: [
				newX _ x + origin x.
				newY _ y + origin y.
				((newX >= 0 and: [newX < kedamaWorld dimensions x]) and: [newY >= 0 and: [newY < kedamaWorld dimensions y]]) ifTrue: [
					xArray add: newX.
					yArray add: newY.
					colorArray add: (c pixelValueForDepth: 32).
				].
			].
		].
	].
	kedamaWorld makeTurtlesAtPositionsIn: {xArray asArray. yArray asArray. colorArray asArray} examplerPlayer: self player ofPrototype: nil.
	self privateTurtleCount: (kedamaWorld turtlesCountOf: self player).
! !

!KedamaTurtleMorph methodsFor: 'initialization' stamp: 'yo 10/8/2004 10:50'!
delete

	super delete.
	kedamaWorld deleteAllTurtlesOfExampler: self player.
	self player delete.
! !

!KedamaTurtleMorph methodsFor: 'initialization' stamp: 'ar 7/8/2006 20:26'!
initialize

	super initialize.
	turtleCount _ 1.
	self assuredPlayer assureUniClass.
	self extent: 40@40.

	isGroup _ false.
	self color: self saturatedRandomColor.

! !

!KedamaTurtleMorph methodsFor: 'initialization' stamp: 'yo 9/23/2004 18:38'!
install

	| t |
	self player kedamaWorld: kedamaWorld.
	t _ self player createTurtles.
	kedamaWorld makeTurtles: turtleCount examplerPlayer: self player color: ((self color pixelValueForDepth: 32) bitAnd: 16rFFFFFF) ofPrototype: nil turtles: t randomize: true.
	self player createSequenceStub.
! !

!KedamaTurtleMorph methodsFor: 'accessing' stamp: 'yo 9/14/2004 15:46'!
isGroup

	^ isGroup.
! !

!KedamaTurtleMorph methodsFor: 'accessing' stamp: 'yo 9/14/2004 15:46'!
isGroup: aBoolean

	isGroup _ aBoolean.
! !

!KedamaTurtleMorph methodsFor: 'accessing' stamp: 'yo 9/16/2004 18:49'!
kedamaWorld

	^ kedamaWorld.
! !

!KedamaTurtleMorph methodsFor: 'accessing' stamp: 'yo 9/16/2004 18:49'!
kedamaWorld: k

	kedamaWorld _ k.
! !

!KedamaTurtleMorph methodsFor: 'initialization' stamp: 'yo 9/17/2004 08:09'!
newPlayerInstance

	^ (KedamaExamplerPlayer newUserInstance).! !

!KedamaTurtleMorph methodsFor: 'accessing' stamp: 'yo 6/5/2004 19:38'!
privateTurtleCount: c

	turtleCount _ c.
! !

!KedamaTurtleMorph methodsFor: 'initialization' stamp: 'ar 7/8/2006 20:26'!
saturatedRandomColor
	"Return a random color that isn't too dark or under-saturated."

	^ Color basicNew
		setHue: (360 atRandom)
		saturation: 1.0
		brightness: 1.0.
! !

!KedamaTurtleMorph methodsFor: 'accessing' stamp: 'yo 6/4/2004 01:18'!
turtleCount

	^ turtleCount.
! !

!KedamaTurtleMorph methodsFor: 'accessing' stamp: 'yo 4/4/2005 17:47'!
turtleCount: count

	| c |
	c _ count asInteger max: 0.
	kedamaWorld setTurtlesCount: c examplerPlayer: self player color: ((self color pixelValueForDepth: 32) bitAnd: 16rFFFFFF).
	turtleCount _ c.
	"turtleCount <= 0 ifTrue: [self player allOpenViewers do: [:v | v providePossibleRestrictedView: 0]]."! !

!KedamaTurtleMorph methodsFor: 'menus' stamp: 'yo 8/17/2005 16:53'!
wantsHaloHandleWithSelector: aSelector inHalo: aHaloMorph
	"Answer whether the receiver would like to offer the halo handle with the given selector (e.g. #addCollapseHandle:)"

	(#(addDupHandle: addMakeSiblingHandle:) includes: aSelector) ifTrue:
		[^ false].

	^ super wantsHaloHandleWithSelector: aSelector inHalo: aHaloMorph! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'sw 9/27/2001 17:40'!
additionsToViewerCategoryBasic
	"Answer viewer additions for the 'basic' category"

	^#(
		basic 
		(
			(slot x 'The x coordinate' Number readWrite Player getX Player setX:)
			(slot y  	'The y coordinate' Number readWrite Player 	getY Player setY:)
			(slot heading  'Which direction the object is facing.  0 is straight up' Number readWrite Player getHeading Player setHeading:)
			(command forward: 'Moves the object forward in the direction it is heading' Number)
			(command turn: 'Change the heading of the object by the specified amount' Number)
			(command beep: 'Make the specified sound' Sound)
		)
	)
! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'sw 4/20/2002 00:47'!
additionsToViewerCategoryDragAndDrop
	"Answer viewer additions for the 'drag & drop' category"

	^#(
		#'drag & drop'
 
		(
			(slot 'drop enabled' 'Whether drop is enabled' Boolean readWrite Player getDropEnabled Player setDropEnabled:)
			(slot 'resist being picked up' 'Whether a simple mouse-drag on this object should allow it to be picked up' Boolean readWrite Player getSticky Player setSticky:)
			(slot 'resist deletion' 'Whether this is resistant to easy removal via the pink X halo handle.' Boolean readWrite Player getResistsRemoval Player setResistsRemoval:)
			(slot 'be locked' 'Whether this object should be blind to all input' Boolean readWrite Player getIsLocked Player setIsLocked:)
		
		))! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'sw 9/17/2002 13:58'!
additionsToViewerCategoryGeometry
	"answer additions to the geometry viewer category"

	^ #(geometry 
		(
			(slot x   'The x coordinate' Number readWrite Player  getX   Player setX:)
			(slot y   'The y coordinate' Number readWrite Player  getY  Player setY:)
			(slot heading  'Which direction the object is facing.  0 is straight up' Number readWrite Player getHeading  Player setHeading:)

			(slot  scaleFactor 'The factor by which the object is magnified' Number readWrite Player getScaleFactor Player setScaleFactor:)
			(slot  left   'The left edge' Number readWrite Player getLeft  Player  setLeft:)
			(slot  right  'The right edge' Number readWrite Player getRight  Player  setRight:)
			(slot  top  'The top edge' Number readWrite Player getTop  Player  setTop:) 
			(slot  bottom  'The bottom edge' Number readWrite Player getBottom  Player  setBottom:) 
			(slot  length  'The length' Number readWrite Player getLength  Player  setLength:) 
			(slot  width  'The width' Number readWrite Player getWidth  Player  setWidth:)

			(slot headingTheta 'The angle, in degrees, that my heading vector makes with the positive x-axis' Number readWrite Player getHeadingTheta Player setHeadingTheta:)

			(slot distance 'The length of the vector connecting the origin to the object''s position' Number readWrite Player getDistance Player setDistance:)
			(slot theta 'The angle between the positive x-axis and the vector connecting the origin to the object''s position' Number readWrite Player getTheta Player setTheta: )
		)
	)


! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'sw 11/16/2001 10:21'!
additionsToViewerCategoryLayout
	"Answer viewer additions for the 'layout' category"

	^#(
		layout 
		(
			(slot clipSubmorphs 'Whether or not to clip my submorphs' Boolean readWrite Player getClipSubmorphs Player setClipSubmorphs:)

		))
! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'sw 7/8/2004 00:20'!
additionsToViewerCategoryMiscellaneous
	"Answer viewer additions for the 'miscellaneous' category"

	^#(
		miscellaneous 
		(
			(command doMenuItem: 'do the menu item' Menu)
			(command show 'make the object visible')
			(command hide 'make the object invisible')
			(command wearCostumeOf: 'wear the costume of...' Player)

			(command fire 'trigger any and all of this object''s button actions')
			(slot copy 'returns a copy of this object' Player readOnly Player getNewClone	 unused unused)
			(slot elementNumber 'my index in my container' Number readWrite Player getIndexInOwner Player setIndexInOwner:)
			(slot holder 'the object''s container' Player readOnly Player getHolder Player setHolder:)
			(command stamp 'add my image to the pen trails')
			(command erase 'remove this object from the screen')
			(command stampAndErase 'add my image to the pen trails and go away')
		)
	)

! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'dgd 8/8/2003 22:17'!
additionsToViewerCategoryMotion
	"Answer viewer additions for the 'motion' category"

	^#(
		motion 
		(
			(slot x 'The x coordinate' Number readWrite Player getX Player setX:)
			(slot y  	'The y coordinate' Number readWrite Player 	getY Player setY:)
			(slot heading  'Which direction the object is facing.  0 is straight up' Number readWrite Player getHeading Player setHeading:)
			(command forward: 'Moves the object forward in the direction it is heading' Number)
			(slot obtrudes 'whether the object sticks out over its container''s edge' Boolean readOnly Player getObtrudes unused unused) 
			(command turnToward: 'turn toward the given object' Player) 
			(command moveToward: 'move toward the given object' Player) 
			(command turn: 'Change the heading of the object by the specified amount' Number)
			(command bounce: 'bounce off the edge if hit' Sound) 
			(command wrap 'wrap off the edge if appropriate') 
			(command followPath 'follow the yellow brick road') 
			(command goToRightOf: 'place this object to the right of another' Player)
		)
	)

! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'sw 12/9/2001 23:26'!
additionsToViewerCategoryObservation
	"Answer viewer additions for the 'observations' category"

	^#(
		observation
 
		(
			(slot colorUnder 'The color under the center of the object' Color readOnly Player getColorUnder unused  unused )
			(slot brightnessUnder 'The brightness under the center of the object' Number readOnly Player getBrightnessUnder unused unused)
			(slot luminanceUnder 'The luminance under the center of the object' Number readOnly Player getLuminanceUnder unused unused)
			(slot saturationUnder 'The saturation under the center of the object' Number readOnly Player getSaturationUnder unused unused)
		
		))
! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'sw 4/17/2003 12:05'!
additionsToViewerCategoryPenUse
	"Answer viewer additions for the 'pen use' category"

	^#(
		#'pen use' 
		(
			(slot penColor 'the color of ink used by the pen' Color readWrite Player getPenColor Player setPenColor:) 
			(slot penSize 'the width of the pen' Number readWrite Player getPenSize Player setPenSize:) 
			(slot penDown 'whether the pen is currently down' Boolean readWrite Player getPenDown Player setPenDown:)
			(slot trailStyle 'determines whether lines, arrows, arrowheads, or dots are used when I put down a pen trail' TrailStyle readWrite Player getTrailStyle Player setTrailStyle:)
			(slot dotSize 'diameter of dot to use when trailStyle is dots' Number readWrite Player getDotSize Player setDotSize:)
			(command clearOwnersPenTrails 'clear all pen trails in my containing playfield')
		)
	)
! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'sw 2/19/2003 18:04'!
additionsToViewerCategoryScripting
	"Answer viewer additions for the 'scripting' category"

	^#(
		scripting 
		(

			(command startScript: 'start the given script ticking' ScriptName)
			(command pauseScript: 'make the given script be "paused"' ScriptName)
			(command stopScript: 'make the given script be "normal"' ScriptName)

			(command startAll: 'start the given script ticking in the object and all of its siblings.' ScriptName)
			(command pauseAll: 'make the given script be "paused" in the object and all of its siblings' ScriptName)
			(command stopAll: 'make the given script be "normal" in the object and all of its siblings' ScriptName)

			(command doScript: 'run the given script once, on the next tick' ScriptName)
			(command tellSelfAndAllSiblings: 'run the given script in the object and in all of its siblings' ScriptName)
			(command tellAllSiblings: 'send a message to all siblings' ScriptName)))! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'RAA 5/18/2001 12:48'!
additionsToViewerCategoryScripts

	"note: if you change the thing below you also need to change #tileScriptCommands."

	^#(
		scripts 
		(
			(command emptyScript 'an empty script')
		)
	)

! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'nk 10/14/2004 10:59'!
additionsToViewerCategoryTests
	"Answer viewer additions for the 'tests' category."

"Note:  Because of intractable performance problems in continuously evaluating isOverColor in a Viewer, the isOverColor entry is not given a readout"

	^#(
		#tests 
		(
			(slot isOverColor 'whether any part of the object is over the given color' Boolean	readOnly Player seesColor: unused unused)
			(slot isUnderMouse 'whether the object is under the current mouse position' Boolean readOnly	Player getIsUnderMouse unused unused)
			(slot colorSees	'whether the given color sees the given color' Boolean readOnly	Player color:sees:	unused	unused)
			(slot overlaps    'whether I overlap a given object' Boolean readOnly Player overlaps: unused unused)
			(slot overlapsAny    'whether I overlap a given object or one of its siblings or similar objects' Boolean readOnly Player overlapsAny: unused unused)
			(slot touchesA	'whether I overlap any  Sketch that is showing the same picture as a particular prototype.' Boolean readOnly Player touchesA:	unused	unused)
			(slot obtrudes 'whether the object sticks out over its container''s edge' Boolean readOnly Player getObtrudes unused unused)
		)
	)
! !

!Morph class methodsFor: '*eToys-customevents-user events' stamp: 'sw 6/16/2005 01:26'!
additionsToViewerCategoryUserEvents
	"Answer further viewer additions relating to user-defined events; these appear in the 'scripting' category"

	^ Preferences allowEtoyUserCustomEvents
		ifTrue: [ #(scripting (
			(command triggerCustomEvent: 'trigger a user-defined (global) event' CustomEvents)
			(slot triggeringObject 'the object that is triggering an event, either user-defined or pre-defined' Player readOnly Player getTriggeringObject unused unused)))]
		ifFalse: [#(scripting ())]! !

!Morph class methodsFor: '*eToys-new-morph participation' stamp: 'sw 10/24/2001 15:51'!
partName: aName categories: aList documentation: aDoc sampleImageForm: aForm
	"Answer a DescriptionForPartsBin which will represent a launch of a new instance of my class via the #newStandAlone protocol sent to my class. Use the category-list and documentation provided.  This variant allows an overriding image form to be provided, useful in cases where we don't want to launch a sample instance just to get the form"


	| descr |
	descr _ DescriptionForPartsBin new
		formalName: aName
		categoryList: aList
		documentation: aDoc
		globalReceiverSymbol: self name
		nativitySelector: #newStandAlone.
	descr sampleImageForm: aForm.
	^ descr
! !

!Morph class methodsFor: '*eToys-scripting' stamp: 'sw 9/17/2002 10:00'!
vectorAdditions
	"Answer slot/command definitions for the vector experiment"

	^ # (
(slot x   'The x coordinate' Number readWrite Player  getX   Player setX:)
(slot y   'The y coordinate' Number readWrite Player  getY  Player setY:)
(slot heading  'Which direction the object is facing.  0 is straight up' Number readWrite Player getHeading  Player setHeading:)
(slot distance 'The length of the vector connecting the origin to the object''s position' Number readWrite Player getDistance Player setDistance:)
(slot theta 'The angle between the positive x-axis and the vector connecting the origin to the object''s position' Number readWrite Player getTheta Player setTheta: )
(slot headingTheta 'The angle that my heading vector makes with the positive x-axis' Number readWrite Player getHeadingTheta Player setHeadingTheta:)

(command + 'Adds two players together, treating each as a vector from the origin.' Player)
(command - 'Subtracts one player from another, treating each as a vector from the origin.' Player)
(command * 'Multiply a player by a Number, treating the Player as a vector from the origin.' Number)
(command / 'Divide a player by a Number, treating the Player as a vector from the origin.' Number)

(command incr: 'Each Player is a vector from the origin.  Increase one by the amount of the other.' Player)
(command decr: 'Each Player is a vector from the origin.  Decrease one by the amount of the other.' Player)
(command multBy: 'A Player is a vector from the origin.  Multiply its length by the factor.' Number)
(command dividedBy: 'A Player is a vector from the origin.  Divide its length by the factor.' Number)
	)! !

!Morph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/24/2003 17:31'!
instantiatedUserScriptsDo: aBlock
	self actorStateOrNil ifNotNilDo: [ :aState | aState instantiatedUserScriptsDictionary do: aBlock]! !

!Morph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/25/2003 11:36'!
removeAllEventTriggers
	"Remove all the event registrations for my Player.
	User custom events are triggered at the World,
	while system custom events are triggered on individual Morphs."

	| player |
	(player _ self player) ifNil: [ ^self ].
	self removeAllEventTriggersFor: player.
	self currentWorld removeAllEventTriggersFor: player.! !

!Morph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/24/2003 17:46'!
removeAllEventTriggersFor: aPlayer
	"Remove all the event registrations for aPlayer.
	User custom events are triggered at the World,
	while system custom events are triggered on individual Morphs."

	self removeActionsSatisfying: 
			[:action | action receiver == aPlayer and: [(#(#doScript: #triggerScript:) includes: action selector) ]].! !

!Morph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/25/2003 11:37'!
removeEventTrigger: aSymbol
	"Remove all the event registrations for my Player that are triggered by aSymbol.
	User custom events are triggered at the World,
	while system custom events are triggered on individual Morphs."

	| player |
	(player _ self player) ifNil: [ ^self ].
	self removeEventTrigger: aSymbol for: player.
	self currentWorld removeEventTrigger: aSymbol for: player.! !

!Morph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/25/2003 11:24'!
removeEventTrigger: aSymbol for: aPlayer 
	"Remove all the event registrations for aPlayer that are triggered by 
	aSymbol. User custom events are triggered at the World, 
	while system custom events are triggered on individual Morphs."
	self removeActionsSatisfying: [:action | action receiver == aPlayer
				and: [(#(#doScript: #triggerScript: ) includes: action selector)
						and: [action arguments first == aSymbol]]]! !

!Morph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/25/2003 11:11'!
renameScriptActionsFor: aPlayer from: oldSelector to: newSelector

	self updateableActionMap keysAndValuesDo: [ :event :sequence |
		sequence asActionSequence do: [ :action |
			((action receiver == aPlayer)
				and: [ (#(doScript: triggerScript:) includes: action selector)
					and: [ action arguments first == oldSelector ]])
						ifTrue: [ action arguments at: 1 put: newSelector ]]]
! !

!Morph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 11/1/2004 11:00'!
triggerCustomEvent: aSymbol
	"Trigger whatever scripts may be connected to the custom event named aSymbol"

	self currentWorld triggerEtoyEvent: aSymbol from: self! !

!Morph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 11/1/2004 10:54'!
triggerEtoyEvent: aSymbol
	"Trigger whatever scripts may be connected to the event named aSymbol.
	If anyone comes back to ask who sent it, return our player."

	[ self triggerEvent: aSymbol ]
		on: GetTriggeringObjectNotification do: [ :ex |
			ex isNested
				ifTrue: [ ex pass ]
				ifFalse: [ ex resume: self assuredPlayer ]]
! !

!Morph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 11/1/2004 10:58'!
triggerEtoyEvent: aSymbol from: aMorph
	"Trigger whatever scripts may be connected to the event named aSymbol.
	If anyone comes back to ask who sent it, return aMorph's player."

	[ self triggerEvent: aSymbol ]
		on: GetTriggeringObjectNotification do: [ :ex |
			ex isNested
				ifTrue: [ ex pass ]
				ifFalse: [ ex resume: aMorph assuredPlayer ]]
! !

Morph subclass: #TilePadMorph
	instanceVariableNames: 'type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Tiles'!

!TilePadMorph commentStamp: '<historical>' prior: 0!
The drop target for colored tiles.  Landing pad.  In the hierarchy, but not a tile itself.  Would like to eliminate this, but an attempt at it failed. !

!TilePadMorph classSide methodsFor: 'color' stamp: 'sw 5/8/1998 13:40'!
colorFudge
	^ 0.4! !

!TilePadMorph methodsFor: 'layout' stamp: 'sw 1/6/2005 03:25'!
acceptDroppingMorph: aMorph event: evt 
	"Accept the given morph within my bowels"

	| editor wasPossessive morphToUse |
	wasPossessive := submorphs notEmpty and: [submorphs first isPossessive].
	morphToUse _ self morphToDropFrom: aMorph.
	self prepareToUndoDropOf: morphToUse.
	self removeAllMorphs.
	morphToUse position: self position.
	self addMorph: morphToUse.
	wasPossessive ifTrue: [morphToUse bePossessive].
	morphToUse lastTile addRetractArrow.	"if can"
	(editor := self topEditor) ifNotNil: [editor install]! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'dgd 8/30/2003 22:19'!
addCustomMenuItems: aCustomMenu hand: aHandMorph
	"Add custom menu items to the menu"

	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	aCustomMenu add: 'restore default tile' translated action: #restoreDefaultTile.! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'sw 9/17/1999 08:04'!
basicWidth
	^ 18
! !

!TilePadMorph methodsFor: 'mouse' stamp: 'sw 5/1/2005 03:17'!
canAccept: aMorph
	"Answer whether this pad can accept the given morph"

	| itsType myType |
	((aMorph isKindOf: PhraseTileMorph) or: [aMorph isKindOf: TileMorph orOf: WatcherWrapper]) 		ifTrue:
			[^ ((itsType _ aMorph resultType capitalized) = (myType _ self type capitalized)) or:
				[(myType = #Graphic) and: [itsType = #Player]]].
	^ false! !

!TilePadMorph methodsFor: 'event handling' stamp: 'ar 2/8/2001 19:54'!
handlesMouseOverDragging: evt
	^true! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'ar 8/16/2001 13:30'!
isColorConstant: aParseNode
	"Is this a Color constant, of the form (MessageNode (VariableNode Color->Color) (SelectorNode #r:g:b:) (LiteralNode LiteralNode LiteralNode))"

	| rec |
	((rec _ aParseNode receiver) isKindOf: VariableNode) ifFalse: [^ false].
	rec key isVariableBinding ifFalse: [^ false].
	rec key value == Color ifFalse: [^ false].
	aParseNode selector key == #r:g:b: ifFalse: [^ false].
	aParseNode arguments  size = 3 ifFalse: [^ false].
	^ true
! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'tk 8/3/1999 13:25'!
isOutsideRef: aParseNode
	"Is this a reference to an outside Player, of the form (self class refUnscriptedPlayer1)?
(MessageNode (VariableNode 'self') (SelectorNode 'class')) (SelectorNode 'refUnscriptedPlayer1')"

	| rec |
	((rec _ aParseNode receiver) isKindOf: MessageNode) ifFalse: [^ false].
	rec receiver isSelfPseudoVariable ifFalse: [^ false].
	rec selector key == #class ifFalse: [^ false].
	aParseNode selector key numArgs = 0 ifFalse: [^ false].
	(aParseNode selector key beginsWith: 'ref') ifFalse: [^ false].
	^ true
! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'tk 8/16/2000 10:01'!
lastTile
	"The tile that might get an extension arrow"

	^ self lastSubmorph lastTile! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'sw 5/1/2005 03:42'!
morphToDropFrom: aMorph 
	"Given a morph being carried by the hand, which the hand is about to drop, answer the actual morph to be deposited.  Normally this would be just the morph itself, but several unusual cases arise, which this method is designed to service."

	| vwr |
	(aMorph isKindOf: WatcherWrapper)
		ifTrue: [^ aMorph getterTilesForDrop].
	^ ((self type capitalized = #Graphic)  "Special-case requested by Alan 4/30/05"
		and: [aMorph isKindOf: TileMorph] and: [aMorph resultType = #Player])
			ifFalse:
				[aMorph]
			ifTrue:
				[vwr _ CategoryViewer new initializeFor: aMorph associatedPlayer categoryChoice: #basic.
				vwr getterTilesFor: (Utilities getterSelectorFor: #graphic)  type: #Graphic]! !

!TilePadMorph methodsFor: 'event handling' stamp: 'ar 2/8/2001 19:56'!
mouseEnterDragging: evt
	evt hand hasSubmorphs ifFalse:[^self].
	(self wantsDroppedMorph: evt hand firstSubmorph event: evt) ifTrue:[
		self firstSubmorph color: Color green.
	].
! !

!TilePadMorph methodsFor: 'event handling' stamp: 'ar 2/8/2001 19:59'!
mouseLeaveDragging: evt
	evt hand hasSubmorphs ifFalse:[^self].
	(self wantsDroppedMorph: evt hand firstSubmorph event: evt) ifTrue:[
		self firstSubmorph useUniformTileColor.
	].! !

!TilePadMorph methodsFor: 'mouse' stamp: 'dgd 2/22/2003 14:43'!
prepareToUndoDropOf: aMorph 
	| m |
	m := self owner.
	[m isNil] whileFalse: 
			[(m isKindOf: ScriptEditorMorph) ifTrue: [^m prepareToUndoDropOf: aMorph].
			m := m owner]! !

!TilePadMorph methodsFor: 'printing' stamp: 'sma 6/1/2000 09:19'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' type='; print: type! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'sw 7/22/2002 18:32'!
restoreDefaultTile
	"Restore the receiver to showing only its default literal tile"

	self setToBearDefaultLiteral.
	(self ownerThatIsA: ScriptEditorMorph) ifNotNilDo:
		[:aScriptEditor | aScriptEditor install]! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'sw 7/22/2002 17:50'!
setToBearDefaultLiteral
	"Set the receiver so that it contains only a tile reflecting the default literal value for a pad of this type"

	self removeAllMorphs.
	self addMorphBack: (Vocabulary vocabularyForType: type) defaultArgumentTile! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'sw 9/17/1999 08:03'!
setType: aSymbol

	type _ aSymbol.
	self color: (ScriptingSystem colorForType: type).
	self extent: (self basicWidth @ TileMorph defaultH)
! !

!TilePadMorph methodsFor: 'code generation' stamp: 'jm 5/28/1998 19:10'!
storeCodeOn: aStream indent: tabCount

	submorphs do: [:m | m storeCodeOn: aStream indent: tabCount].
! !

!TilePadMorph methodsFor: 'miscellaneous' stamp: 'sw 10/3/97 18:00'!
type
	^ type! !

!TilePadMorph methodsFor: 'dropping/grabbing' stamp: 'jm 6/26/97 08:30'!
wantsDroppedMorph: aMorph event: evt

	^ self canAccept: aMorph
! !

SimpleButtonMorph subclass: #ScriptActivationButton
	instanceVariableNames: 'uniclassScript'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!ScriptActivationButton commentStamp: 'sw 3/11/2003 00:24' prior: 0!
A button associated with a particular player and script.  You can "tear off" such a button for any etoy script, using menu items available in both Viewers and Scriptors.  The button initially is given a label reflecting its player and script name, but this can be edited via the button's halo menu, as can its other appearance parameters.  Such buttons are automatically kept in synch when the object's name or the script name change.!

!ScriptActivationButton classSide methodsFor: 'scripting' stamp: 'sw 9/26/2001 04:26'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((button (
			(slot color 'The color of the object' Color readWrite Player getColor  Player  setColor:)
			(slot height  'The height' Number readWrite Player getHeight  Player  setHeight:) 
			(slot borderColor 'The color of the object''s border' Color readWrite Player getBorderColor Player  setBorderColor:)
			(slot borderWidth 'The width of the object''s border' Number readWrite Player getBorderWidth Player setBorderWidth:)
			(slot roundedCorners 'Whether corners should be rounded' Boolean readWrite Player getRoundedCorners Player setRoundedCorners:) 
			(slot actWhen 'When the script should fire' ButtonPhase  readWrite Player getActWhen Player setActWhen: ))))! !

!ScriptActivationButton methodsFor: 'menu' stamp: 'sw 3/2/2004 20:58'!
addCustomMenuItems: aMenu hand: aHandMorph
	"Add morph-specific items to the given menu which was invoked by the given hand."

	super addCustomMenuItems: aMenu hand: aHandMorph.
	aMenu addLine.
	aMenu add: 'open underlying scriptor' translated target: target selector: #openUnderlyingScriptorFor: argument: arguments first

! !

!ScriptActivationButton methodsFor: 'miscellaneous' stamp: 'sw 3/11/2003 00:35'!
bringUpToDate
	"The object's name, or the script name, or both, may have changed.  Make sure I continue to look and act right"

	uniclassScript ifNotNil:
		[arguments _ Array with: uniclassScript selector].
	self establishLabelWording! !

!ScriptActivationButton methodsFor: 'label' stamp: 'yo 11/30/2005 14:14'!
establishLabelWording
	"Set the label wording, unless it has already been manually edited"

	| itsName |
	itsName _ target externalName.
	(self hasProperty: #labelManuallyEdited)
		ifFalse:
			[self label: (itsName, ' ', arguments first)].
	self setBalloonText: 
		('click to run the script "{1}" in player named "{2}"' translated format: {arguments first. itsName}).
! !

!ScriptActivationButton methodsFor: 'initialization' stamp: 'sw 3/11/2003 00:31'!
initializeForPlayer: aPlayer uniclassScript: aUniclassScript
	"Initialize the receiver for the given player and uniclass script"

	target _ aPlayer.
	uniclassScript _ aUniclassScript.
	actionSelector _ #runScript:.
	arguments _ Array with: uniclassScript selector.
	self establishLabelWording
	! !

!ScriptActivationButton methodsFor: 'label' stamp: 'sw 10/1/1999 21:42'!
isTileScriptingElement
	^ true! !

!ScriptActivationButton methodsFor: 'e-toy support' stamp: 'nk 9/3/2004 16:06'!
localeChanged
	"Do nothing"! !

!ScriptActivationButton methodsFor: 'miscellaneous' stamp: 'sw 2/28/2001 21:42'!
setLabel
	"Allow the user to enter a new label for this button"

	| newLabel existing |
	existing _ self label.
	newLabel _ FillInTheBlank
		request: 'Please enter a new label for this button'
		initialAnswer: existing.
	(newLabel isEmptyOrNil not and: [newLabel ~= existing]) ifTrue:
		[self setProperty: #labelManuallyEdited toValue: true.
		self label: newLabel].
! !

!ScriptActivationButton methodsFor: 'access' stamp: 'sw 3/10/2003 23:57'!
uniclassScript
	"Answer the receiver's uniclassScript.  For old buttons, this might initially be nil but will get set, when possible herein."

	^ uniclassScript ifNil:
		[uniclassScript _ target class scripts at: arguments first ifAbsent: [nil]]! !

!SimpleButtonMorph class methodsFor: '*eToys-scripting' stamp: 'sw 2/6/2001 23:54'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((button (
			(command fire 'trigger any and all of this object''s button actions'))))
! !

!FullVocabulary methodsFor: '*eToys-initialization' stamp: 'sw 9/27/2001 17:21'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(1.0 0.26 0.98)	! !

!MessageNode methodsFor: '*eToys-tiles' stamp: 'RAA 2/15/2001 19:34'!
asMorphicSyntaxIn: parent

	^parent 
		vanillaMessageNode: self 
		receiver: receiver 
		selector: selector 
		arguments: arguments
! !

MessageNode subclass: #MessagePartNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Tile Scriptors'!

MessageNode subclass: #TileMessageNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Tile Scriptors'!

!TileMessageNode methodsFor: 'printing' stamp: 'RAA 6/9/2000 18:25'!
printIfNilNotNil: aMorph indent: level

	| newNode |
	newNode _ aMorph parseNode clone.
	newNode receiver ifNotNil:
		[newNode receiver: newNode receiver ifNilReceiver].	"fudge so it prints right"

	(arguments first isJust: NodeNil) ifTrue:
		[^ newNode morphFromKeywords: #ifNotNil:
				arguments: { arguments second }
				on: aMorph indent: level].
	(arguments second isJust: NodeNil) ifTrue:
		[^ newNode morphFromKeywords: #ifNil:
				arguments: { arguments first }
				on: aMorph indent: level].
	^ newNode morphFromKeywords: #ifNil:ifNotNil:
			arguments: arguments
			on: aMorph indent: level! !

!TileMessageNode methodsFor: 'printing' stamp: 'BJP 6/9/2000 12:37'!
printIfOn: aStream indent: level 
"Just copied the old MessageNode one down here."
	(arguments last isJust: NodeNil)
		ifTrue: [^ self
				printKeywords: #ifTrue:
				arguments: (Array with: arguments first)
				on: aStream
				indent: level].
	(arguments last isJust: NodeFalse)
		ifTrue: [^ self
				printKeywords: #and:
				arguments: (Array with: arguments first)
				on: aStream
				indent: level].
	(arguments first isJust: NodeNil)
		ifTrue: [^ self
				printKeywords: #ifFalse:
				arguments: (Array with: arguments last)
				on: aStream
				indent: level].
	(arguments first isJust: NodeTrue)
		ifTrue: [^ self
				printKeywords: #or:
				arguments: (Array with: arguments last)
				on: aStream
				indent: level].
	self
		printKeywords: #ifTrue:ifFalse:
		arguments: arguments
		on: aStream
		indent: level! !

!TileMessageNode methodsFor: 'printing' stamp: 'RAA 8/15/1999 16:23'!
printKeywords: key arguments: args on: morph indent: level

	^morph parseNode
		morphFromKeywords: key 
		arguments: args 
		on: morph 
		indent: level
! !

!TileMessageNode methodsFor: 'printing' stamp: 'RAA 6/9/2000 15:32'!
printReceiver: rcvr on: aMorph indent: level
					
	"I don't think we need this to do anything since we already printed the receiver ourself"
! !

!TileMessageNode methodsFor: 'printing' stamp: 'dgd 2/21/2003 22:52'!
printToDoOn: aMorph indent: level 
	| limitNode |
	limitNode := (arguments last isNil 
				or: [(arguments last isMemberOf: AssignmentNode) not]) 
					ifTrue: [arguments first]
					ifFalse: [arguments last value].
	(selector key = #to:by:do: 
		and: [arguments second isConstantNumber and: [arguments second key = 1]]) 
			ifTrue: 
				[self 
					printKeywords: #to:do:
					arguments: (Array with: limitNode with: (arguments third))
					on: aMorph
					indent: level]
			ifFalse: 
				[self 
					printKeywords: selector key
					arguments: (Array with: limitNode) , arguments allButFirst
					on: aMorph
					indent: level]! !

!TileMessageNode methodsFor: 'printing' stamp: 'RAA 6/9/2000 17:15'!
printWhileOn: aMorph indent: level

	(arguments first isJust: NodeNil) ifTrue: [
		selector _ SelectorNode new
			key: (selector key == #whileTrue:
				ifTrue: [#whileTrue] ifFalse: [#whileFalse])
			code: #macro.
		arguments _ Array new
	].
	self printKeywords: selector key arguments: arguments
		on: aMorph indent: level! !

StandardScriptingSystem subclass: #EToySystem
	instanceVariableNames: ''
	classVariableNames: 'EToyVersion EToyVersionDate'
	poolDictionaries: ''
	category: 'EToys-Experimental'!

!EToySystem commentStamp: '<historical>' prior: 0!
A global object holding onto properties and code of the overall E-toy system of the moment.  Its code is entirely held on the class side; the class is never instantiated.!

!StandardScriptingSystem methodsFor: '*eToys-universal slots & scripts' stamp: 'sw 1/4/2005 02:20'!
acceptableSlotNameFrom: originalString forSlotCurrentlyNamed: currentName asSlotNameIn: aPlayer world: aWorld
	"Produce an acceptable slot name, derived from the current name, for aPlayer.  This method will always return a valid slot name that will be suitable for use in the given situation, though you might not like its beauty sometimes."

	| aString stemAndSuffix proscribed stem suffix putative |
	aString _ originalString asIdentifier: false.  "get an identifier not lowercase"
	stemAndSuffix _ aString stemAndNumericSuffix.
	proscribed _ #(self super thisContext costume costumes dependents #true #false size), aPlayer class allInstVarNames.

	stem _ stemAndSuffix first.
	suffix _ stemAndSuffix last.
	putative _ aString asSymbol.
	
	[(putative ~~ currentName) and: [(proscribed includes: putative)
		or:	[(aPlayer respondsTo: putative)
		or:	[Smalltalk includesKey: putative]]]]
	whileTrue:
		[suffix _ suffix + 1.
		putative _ (stem, suffix printString) asSymbol].
	^ putative! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 11/1/2004 07:47'!
addCustomEventFor: registrantClass named: aSymbol help: helpString targetMorphClass: targetClass
	| registration |
	registration _ self customEventsRegistry at: aSymbol ifAbsentPut: [ IdentityDictionary new ].
	registration at: registrantClass put: { helpString. targetClass }.
! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 9/26/2003 23:23'!
addUserCustomEventNamed: aSymbol help: helpString
	self currentWorld addUserCustomEventNamed: aSymbol help: helpString.
	"Vocabulary addStandardVocabulary: UserCustomEventNameType new."
	Vocabulary customEventsVocabulary.
	SymbolListTile updateAllTilesForVocabularyNamed: #CustomEvents! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 10/30/2000 16:37'!
allKnownClassVariableNames
	"Answer a set of all the knwon class variable names in the system.  This normally retrieves them from a cache, and at present there is no organized mechanism for invalidating the cache.  The idea is to avoid, in the References scheme, names that may create a conflict"

	^ ClassVarNamesInUse ifNil: [ClassVarNamesInUse _ self allClassVarNamesInSystem]

	"ClassVarNamesInUse _ nil.
	Time millisecondsToRun: [ScriptingSystem allKnownClassVariableNames]"
! !

!StandardScriptingSystem methodsFor: '*eToys-parts bin' stamp: 'tk 10/20/2004 15:52'!
anyButtonPressedTiles
	"Answer tiles representing the query 'is any button pressed?'"

	^ self tilesForQuery: '(ActiveHand anyButtonPressed)' label: 'button down?' translated! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'kfr 9/23/2003 09:44'!
arithmeticalOperatorsAndHelpStrings
	"Answer an array consisting of lists of the standard arithmetical operator tiles and of the corresponding balloon help for them"

	^ #((+ - * / // \\ max: min:)
	 	('add' 'subtract' 'multiply' 'divide' 'divide & truncate' 'remainder when divided by' 'larger value' 'smaller value' ))! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'tk 10/20/2004 15:52'!
buttonDownTile
	"Answer a boolean-valued tile which reports whether the button is down"

	^ self systemQueryPhraseWithActionString: '(ActiveHand anyButtonPressed)' labelled: 'button down?' translated! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'tk 10/20/2004 15:52'!
buttonUpTile
	"Answer a boolean-valued tile which reports whether the button is up"

	^ self systemQueryPhraseWithActionString: '(ActiveHand noButtonPressed)' labelled: 'button up?' translated! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'tk 8/21/2000 12:59'!
cleanupsForRelease
	"Miscellaneous space cleanups to do before a release."
	"EToySystem cleanupsForRelease"

	Socket deadServer: ''.  "Don't reveal any specific server name"
	HandMorph initialize.  "free cached ColorChart"
	PaintBoxMorph initialize.	"forces Prototype to let go of extra things it might hold"
	Smalltalk removeKey: #AA ifAbsent: [].
	Smalltalk removeKey: #BB ifAbsent: [].
	Smalltalk removeKey: #CC ifAbsent: [].
	Smalltalk removeKey: #DD ifAbsent: [].
	Smalltalk removeKey: #Temp ifAbsent: [].

	ScriptingSystem reclaimSpace.
	Smalltalk cleanOutUndeclared.
	Smalltalk reclaimDependents.
	Smalltalk forgetDoIts.
	Smalltalk removeEmptyMessageCategories.
	Symbol rehash! !

!StandardScriptingSystem methodsFor: '*eToys-font & color choices' stamp: 'sw 5/2/1998 14:26'!
colorBehindTiles
	^ Color r: 0.903 g: 1.0 b: 0.903! !

!StandardScriptingSystem methodsFor: '*eToys-tile colors' stamp: 'sw 8/28/2004 15:19'!
colorForType: typeSymbol
	"Answer the color to use to represent the given type symbol"

	true ifTrue:
		[^ self standardTileBorderColor].

	typeSymbol capitalized = #Command ifTrue:
		[^ Color fromRgbTriplet: #(0.065 0.258 1.0)].
	"Command is historical and idiosyncratic and should be regularized"

	^ (Vocabulary vocabularyForType: typeSymbol) typeColor! !

!StandardScriptingSystem methodsFor: '*eToys-tile colors' stamp: 'sw 10/29/1998 16:18'!
colorFudge
	^ 0.4! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 11/1/2004 08:19'!
customEventNamesAndHelpStringsFor: aPlayer
	| retval help helpStrings morph |
	morph := aPlayer costume renderedMorph.
	retval := SortedCollection sortBlock: [ :a :b | a first < b first ].
	self customEventsRegistry
		keysAndValuesDo: [ :k :v |
			helpStrings := Array streamContents: [ :hsStream |
				v keysAndValuesDo: [ :registrant :array |
					(morph isKindOf: array second) ifTrue: [
						help := String streamContents: [ :stream |
										v size > 1
											ifTrue: [ stream nextPut: $(;
													nextPutAll: array second name;
													nextPut: $);
													space ].
										stream nextPutAll: array first ].
						hsStream nextPut: help ]]].
			helpStrings isEmpty ifFalse: [retval add: { k. helpStrings } ]].
	^ retval! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 9/26/2003 23:31'!
customEventStati
	^self globalCustomEventNames,
	self userCustomEventNames! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 10/12/2003 13:14'!
customEventsRegistry
	^Smalltalk at: #CustomEventsRegistry ifAbsentPut: [ IdentityDictionary new ].! !

!StandardScriptingSystem methodsFor: '*eToys-universal slots & scripts' stamp: 'kfr 9/23/2003 09:29'!
doesOperatorWantArrows: aSymbol
	aSymbol = #, ifTrue:[^ false].
	^ aSymbol isInfix or: [#(isDivisibleBy:) includes: aSymbol]! !

!StandardScriptingSystem methodsFor: '*eToys-font & color choices' stamp: 'dgd 7/12/2003 12:06'!
fontForNameEditingInScriptor
	^ Preferences standardEToysFont! !

!StandardScriptingSystem methodsFor: '*eToys-font & color choices' stamp: 'dgd 7/12/2003 12:05'!
fontForTiles
	^ Preferences standardEToysFont! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 9/26/2003 23:30'!
globalCustomEventNames
	^self customEventsRegistry keys asArray sort! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 11/1/2004 07:56'!
globalCustomEventNamesFor: aPlayer
	| morph names |
	morph := aPlayer costume renderedMorph.
	names := SortedCollection new.
	self customEventsRegistry keysAndValuesDo: [ :k :v |
		(v anySatisfy: [ :array | morph isKindOf: array second ])
			ifTrue: [ names add: k ]].
	^names asArray! !

!StandardScriptingSystem methodsFor: '*eToys-script-control' stamp: 'dgd 9/19/2003 14:40'!
goButton
	| aButton |
	aButton _  ThreePhaseButtonMorph new.
	aButton image:  (ScriptingSystem formAtKey: 'GoPicOn');
			offImage: (ScriptingSystem formAtKey: 'GoPic');
			pressedImage: (ScriptingSystem formAtKey: 'GoPicOn');
			actionSelector: #goUp:with:; 
			arguments: (Array with: nil with: aButton);
			actWhen: #buttonUp;
			target: self;
			setNameTo: 'Go Button';
			setBalloonText:
'Resume running all paused scripts' translated.
	^ aButton! !

!StandardScriptingSystem methodsFor: '*eToys-script-control' stamp: 'sw 11/11/1998 15:24'!
goUp: evt with: aGoButton
	aGoButton presenter startRunningScriptsFrom: aGoButton! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 3/10/2004 23:24'!
helpStringForOperator: anOperator
	"Answer the help string associated with the given operator. If none found, return a standard no-help-available reply"

	^ (self helpStringOrNilForOperator: anOperator) ifNil:
		['Sorry, no help available here' translated]  "This should never be seen, but is provided as a backstop"! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 6/27/2004 11:11'!
helpStringOrNilForOperator: anOperator
	"Answer the help string associated with the given operator, nil if none found."

	| anIndex opsAndHelp |
	(anIndex _ (opsAndHelp _ self arithmeticalOperatorsAndHelpStrings) first indexOf: anOperator) > 0
		ifTrue:	[^ (opsAndHelp second at: anIndex) translated].

	(anIndex _ (opsAndHelp _ self numericComparitorsAndHelpStrings) first indexOf: anOperator) > 0
		ifTrue:	[^ (opsAndHelp second at: anIndex) translated].

	anOperator = #, ifTrue:
		[^ 'Concatenate two Strings' translated].

	^ nil! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'nk 7/12/2003 08:59'!
holderWithAlphabet
	"Answer a fully instantiated Holder that has submorphs that represent the letters of the uppercase alphabet, with each one having an 'index' slot which bears the letter's index in the alphabet -- 1 for A, 2 for B, etc.   A few special characters are provided as per ack request 10/00; for these the index provided is rather arbitrarily assigned"

	| aMorph aPlayer newMorph oneCharString aContainer aWrapper |

	"ScriptingSystem holderWithAlphabet openInHand"

	aContainer _ self prototypicalHolder useRoundedCorners.
	aContainer borderColor: Color blue lighter.

	aWrapper _ AlignmentMorph new hResizing: #shrinkWrap; vResizing: #shrinkWrap; layoutInset: 0.
	aWrapper addMorphBack: (aMorph _ TextMorph new contents: 'A').
	aMorph beAllFont: ((TextStyle named: Preferences standardEToysFont familyName) fontOfSize: 24).
	aMorph width: 14; lock.
	aWrapper beTransparent; setNameTo: 'A'.
	aPlayer _ aWrapper assuredPlayer.
	aPlayer addInstanceVariableNamed: #index type: #Number value: 1.
	aContainer addMorphBack: aWrapper.
	2 to: 26 do:
		[:anIndex |
			newMorph _ aWrapper usableSiblingInstance.
			newMorph player perform: #setIndex: with: anIndex.
			newMorph firstSubmorph contents: (oneCharString _ ($A asciiValue + anIndex - 1) asCharacter asString).
			newMorph setNameTo: oneCharString.

			aContainer addMorphBack: newMorph].

	#(' ' '.' '#') with: #(27 28 29) do:
		[:aString :anIndex |
			newMorph _ aWrapper usableSiblingInstance.
			newMorph player perform: #setIndex: with: anIndex.
			newMorph firstSubmorph contents: aString.
			aString = ' '
				ifTrue:
					[newMorph setNameTo: 'space'.
					newMorph color: (Color gray alpha: 0.2)]
				ifFalse:
					[newMorph setNameTo: aString].
			aContainer addMorphBack: newMorph].

	aContainer setNameTo: 'alphabet'.
	aContainer isPartsBin: true.
	aContainer enableDrop: false.
	aContainer indicateCursor: false; width: 162.
	aContainer color: (Color r: 0.839 g: 1.0 b: 1.0).  "Color fromUser"
	^ aContainer! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 9/15/2000 06:24'!
informScriptingUser: aString
	"This provides a hook for logging messages that the user or the developer may wish to see; at present it simply logs the message to the Transcript, with a standard prefix to signal their provenance.  Such messages will fall on the floor if there is no Transcript window open"

	Transcript cr; show: 'SCRIPT NOTE: ', aString! !

!StandardScriptingSystem methodsFor: '*eToys-form dictionary' stamp: 'sw 5/12/1999 10:09'!
installSolidMenuForm
	"ScriptingSystem installSolidMenuForm"
	self saveForm:
		(Form extent: 14@16 depth: 16
	fromArray: #( 1 0 0 0 0 0 0 65537 65536 0 0 0 65537 0 65537 65537 65537 65537 65537 65537 65536 65537 65537 65537 65537 65537 1600061441 65536 65537 1600085855 1600085855 1600085855 1600085855 1600061441 65536 65537 1600085855 65537 65537 65537 65537 65536 65537 1600085855 65537 65537 65537 1600061441 65536 65537 1600085855 1600085855 1600085855 1600085855 1600085855 65537 65537 1600085855 65537 65537 65537 1600085855 65537 65537 1600085855 1600061441 65537 65537 89951 65537 65537 1600085855 1600085855 1600085855 1600085855 1600085855 65537 65537 1600085855 1600061441 65537 65537 65537 65537 65537 1600085855 65537 65537 65537 65536 65537 65537 65537 65537 65537 65537 65537 65537 1 65537 65537 65537 65537 65537 65536 0 65536 0 0 0 0 0) offset: 0@0)
		atKey: 'SolidMenu'! !

!StandardScriptingSystem methodsFor: '*eToys-form dictionary' stamp: 'sw 11/26/1999 15:37'!
mergeGraphicsFrom: aDictionary
	"aDictionary is assumed to hold associations of the form <formName> -> <form>.   Merge the graphics held by that dictionary into the internal FormDictionary, overlaying any existing entries with the ones found in aDictionary"

	aDictionary associationsDo:
		[:assoc | self saveForm: assoc value atKey: assoc key]

		"works ok even if keys in aDictionary are strings rather than symbols"! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 2/26/2003 22:44'!
nameForInstanceVariablesCategory
	"Answer the name to use for the viewer category that contains instance variables"

	^ #variables    
	"^ #'instance variables'"

"ScriptingSystem nameForInstanceVariablesCategory"! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 2/6/2003 18:00'!
nameForScriptsCategory
	"Answer the name to use for the viewer category that contains scripts"

	^ #scripts! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 2/18/2001 17:50'!
newScriptingSpace
	"Answer a complete scripting space - raa 19 sept 2000 - experiment for Alan, a variant *not* in a window, now adopted as the only true scripting space"

	^ self newScriptingSpace2! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 11/13/2001 14:41'!
newScriptingSpace2
	"Answer a complete scripting space"

	| aTemplate  aPlayfield aControl |
	
	(aTemplate _ PasteUpMorph new)
		setNameTo: 'etoy';
		extent: 638 @ 470;
		color: Color white;
		impartPrivatePresenter;
		setProperty: #automaticPhraseExpansion toValue: true;
		beSticky.
	aTemplate useRoundedCorners; borderWidth: 2. 
	aControl _  ScriptingSystem scriptControlButtons setToAdhereToEdge: #bottomLeft.
	aControl beSticky; borderWidth: 0; beTransparent.
	aTemplate addMorphBack: aControl.
	aTemplate presenter addTrashCan.

	aTemplate addMorph: (aPlayfield _ PasteUpMorph new).
	aPlayfield
		setNameTo: 'playfield';
		useRoundedCorners;
		setToAdhereToEdge: #topLeft;
		extent: 340@300;
		position: aTemplate topRight - (400@0);
		beSticky;
		automaticViewing: true;
		wantsMouseOverHalos: true.
	aTemplate presenter standardPlayfield: aPlayfield.
	
	^ aTemplate

! !

!StandardScriptingSystem methodsFor: '*eToys-parts bin' stamp: 'sw 11/16/2004 13:56'!
noButtonPressedTiles
	"Answer tiles representing the query 'is no button pressed?'"

	^ self tilesForQuery: '(ActiveHand noButtonPressed)' label: 'button up?' translated! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 9/21/2000 22:39'!
numericComparitorsAndHelpStrings
	"Answer an array whose first element is the list of comparitors, and whose second element is a list of the corresponding help strings"

	^ #((< <= = ~= > >= isDivisibleBy:)
	 	('less than' 'less than or equal' 'equal' 'not equal' 'greater than' 'greater than or equal' 'divisible by' ))! !

!StandardScriptingSystem methodsFor: '*eToys-form dictionary' stamp: 'sw 2/20/2002 01:09'!
patchInNewStandardPlayerForm
	"Patch in a darker and larger representation of a Dot.  No senders -- called from the postscript of an update"

	"ScriptingSystem patchInNewStandardPlayerForm"

	FormDictionary at: #standardPlayer put:
		(Form
	extent: 13@13
	depth: 16
	fromArray: #( 0 0 0 65536 0 0 0 0 0 65537 65537 65536 0 0 0 65537 65537 65537 65537 65536 0 0 65537 65537 65537 65537 65536 0 1 65537 65537 65537 65537 65537 0 1 65537 65537 65537 65537 65537 0 65537 65537 65537 65537 65537 65537 65536 1 65537 65537 65537 65537 65537 0 1 65537 65537 65537 65537 65537 0 0 65537 65537 65537 65537 65536 0 0 65537 65537 65537 65537 65536 0 0 0 65537 65537 65536 0 0 0 0 0 65536 0 0 0)
	offset: 0@0)! !

!StandardScriptingSystem methodsFor: '*eToys-parts bin' stamp: 'sw 5/3/1999 22:40'!
prototypicalHolder
	| aHolder |
	aHolder _ PasteUpMorph authoringPrototype color: Color orange muchLighter; borderColor: Color orange lighter.
	aHolder setNameTo: 'holder'; extent: 160 @ 110.
	^ aHolder behaveLikeHolder.
! !

!StandardScriptingSystem methodsFor: '*eToys-form dictionary' stamp: 'sw 5/2/1998 14:01'!
readFormsFromFileNamed: aFileName
	"Read the entire FormDictionary in from a designated file on disk"

	| aReferenceStream |
	aReferenceStream _ ReferenceStream fileNamed: aFileName.
	FormDictionary _ aReferenceStream next.
	aReferenceStream close

	"ScriptingSystem readFormsFromFileNamed: 'EToyForms22Apr'"! !

!StandardScriptingSystem methodsFor: '*eToys-form dictionary' stamp: 'sw 9/14/2000 21:29'!
readFormsFromFileNamed: aFileName andStoreIntoGlobal: globalName
	"Read the a FormDictionary in from a designated file on disk and save it in the designated global"

	| aReferenceStream |
	aReferenceStream _ ReferenceStream fileNamed: aFileName.
	Smalltalk at: globalName put: aReferenceStream next.
	aReferenceStream close

	"ScriptingSystem readFormsFromFileNamed: 'SystemFormsFromFwdF.forms' andStoreIntoGlobal: #FormsTemp"

	"ScriptingSystem saveForm:  (FormsTemp at: #StackElementDesignationHelp) atKey: #StackElementDesignationHelp"! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 5/2/1998 14:17'!
reclaimSpace
	"Reclaim space from the scripting system, and report the result in an informer"
	"ScriptingSystem reclaimSpace"

	| reclaimed |
	(reclaimed _ self spaceReclaimed)  > 0
		ifTrue:	[self inform: reclaimed printString, ' bytes reclaimed']
		ifFalse:	[self inform: 'Hmm...  Nothing gained this time.']! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 10/30/2000 09:08'!
referenceAt: aSymbol
	"Answer the object referred to by aSymbol in the 'References' scheme of things, or nil if none"

	^ References at: aSymbol ifAbsent: [nil]! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 10/30/2000 09:33'!
referenceAt: aSymbol put: anObject
	"Store a reference to anObject at the given symbol in the References directory"

	^ References at: aSymbol put: anObject! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 6/9/2000 18:44'!
reinvigorateThumbnailsInViewerFlapTabs
	"It has happened that the thumbnail in a viewer flap tab will go solid gray because it got associated with some passing and disused player temporarily created during the initial painting process.  This method takes a sledge hammer to repair such thumbnails.   At its genesis, this method is called only from the postscript of its defining fileout."
	| vwr thumbnail |
	ViewerFlapTab allInstancesDo:
		[:aTab | 
			vwr _ aTab referent findA: StandardViewer.
			thumbnail _ aTab findA: ThumbnailMorph.
			(vwr notNil and: [thumbnail notNil]) ifTrue:
				[thumbnail objectToView: vwr scriptedPlayer]]

	"ScriptingSystem reinvigorateThumbnailsInViewerFlapTabs"! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 7/20/2003 12:37'!
removeCustomEventNamed: aSymbol for: registrant
	| registration helpString |
	registration _ self customEventsRegistry at: aSymbol ifAbsent: [ ^nil ].
	helpString _ registration removeKey: registrant ifAbsent: [].
	registration isEmpty ifTrue: [ self customEventsRegistry removeKey: aSymbol ].
	^helpString! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 9/26/2003 23:26'!
removeUserCustomEventNamed: eventName
	| retval |
	retval _ self currentWorld removeUserCustomEventNamed: eventName.
	"Vocabulary addStandardVocabulary: UserCustomEventNameType new."
	Vocabulary customEventsVocabulary.
	SymbolListTile updateAllTilesForVocabularyNamed: #CustomEvents.
	^retval! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 12/20/2003 18:02'!
reportToUser: aString
	"Make a message accessible to the user.  For the moment, we simply defer to the Transcript mechanism"

	Transcript cr; show: aString! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 10/30/2000 16:47'!
resetAllScriptingReferences
	"Clear out all the elements in the References directory"
	
	Smalltalk at: #References put: IdentityDictionary new

	"ScriptingSystem resetAllScriptingReferences"! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 10/30/2000 16:45'!
resetStaleScriptingReferences
	"Remove all scripting references that are no longer needed"

	References  removeUnreferencedKeys

	"ScriptingSystem resetStaleScriptingReferences"
! !

!StandardScriptingSystem methodsFor: '*eToys-parts bin' stamp: 'sw 10/27/1998 13:35'!
resetStandardPartsBin
	"ScriptingSystem resetStandardPartsBin"

	StandardPartsBin _ nil! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 7/25/2004 17:27'!
restoreClassicEToyLook
	"Restore classic EToy look, as closely as possible.  If ComicBold is present, restore it as the standard etoy and button font.  Substitute ComicSansMS and Accuny as respective alternatives if the classic fonts are absent.  If those also aren't available, do nothing."

	| aTextStyle aFont | 
	(aTextStyle _ TextStyle named: #ComicBold)
		ifNotNil:
			[aFont _ aTextStyle fontOfSize: 16.
			Preferences setEToysFontTo: aFont.
			Preferences setButtonFontTo: aFont]
		ifNil:
			[(aTextStyle _ TextStyle named: #ComicSansMS) ifNotNil:
				[Preferences setEToysFontTo: (aTextStyle fontOfSize: 18)].
			(aTextStyle _ TextStyle named: #Accuny) ifNotNil:
				[Preferences setButtonFontTo: (aTextStyle fontOfSize: 12)]].

	(aTextStyle _ TextStyle named: #NewYork)
		ifNotNil:
			[Preferences setSystemFontTo: (aTextStyle fontOfSize: 12)]! !

!StandardScriptingSystem methodsFor: '*eToys-form dictionary' stamp: 'sw 4/23/1999 11:24'!
restorePrivateGraphics
	"ScriptingSystem restorePrivateGraphics"
	| aReferenceStream |
	aReferenceStream _ ReferenceStream fileNamed: 'disGraphics'.
	self mergeGraphicsFrom: aReferenceStream next.
	aReferenceStream close.
! !

!StandardScriptingSystem methodsFor: '*eToys-form dictionary' stamp: 'sw 5/6/1998 17:46'!
saveFormsToFileNamed: aFileName
	"Save the current state of form dictionary to disk for possible later retrieval"
  	 (ReferenceStream fileNamed: aFileName) nextPut: FormDictionary; close

	"ScriptingSystem saveFormsToFileNamed: 'SystemForms06May98.forms'"! !

!StandardScriptingSystem methodsFor: '*eToys-script-control' stamp: 'sw 1/23/2001 11:39'!
scriptControlButtons
	"Answer a composite object that serves to control the stop/stop/go status of a Presenter"

	| wrapper |
	wrapper _ AlignmentMorph newRow setNameTo: 'script controls'.
	wrapper vResizing: #shrinkWrap.
	wrapper hResizing: #shrinkWrap.
	wrapper addMorph: self stopButton.
	wrapper addMorphBack: self stepButton.
	wrapper addMorphBack: self goButton.
	wrapper beTransparent.
	^ wrapper! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 5/16/2001 12:58'!
setterSelectorForGetter: aGetterSymbol
	"Answer the setter selector corresponding to a given getter"

	^ (('s', (aGetterSymbol copyFrom: 2 to: aGetterSymbol size)), ':') asSymbol

	"ScriptingSystem setterSelectorForGetter: #getCursor"! !

!StandardScriptingSystem methodsFor: '*eToys-font & color choices' stamp: 'nk 7/12/2003 08:59'!
smallBoldFont
	"Answer a small bold font for use in some standard scripting-support structures"

	^ StrikeFont familyName: Preferences standardEToysFont familyName size: 12! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'di 3/3/2001 08:47'!
spaceReclaimed
	"Reclaim space from the EToy system, and return the number of bytes reclaimed"
	"ScriptingSystem spaceReclaimed"

	| oldFree  |
	oldFree _ Smalltalk garbageCollect.
	ThumbnailMorph recursionReset.
	Player removeUninstantiatedSubclassesSilently.
	Smalltalk cleanOutUndeclared.
	Smalltalk reclaimDependents.
	^ Smalltalk garbageCollect - oldFree.! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 6/30/2004 18:16'!
standardEventStati
	"Answer the events that can be directed to a particular morph by its event handler."
	^ #(mouseDown	"run when mouse goes down on me"
		mouseStillDown	"while mouse still down"
		mouseUp		"when mouse comes back up"
		mouseEnter	"when mouse enters my bounds, button up"
		mouseLeave	"when mouse exits my bounds, button up"
		mouseEnterDragging	"when mouse enters my bounds, button down"
		mouseLeaveDragging	"when mouse exits my bounds, button down"
		"keyStroke"
		"gesture"
	)
! !

!StandardScriptingSystem methodsFor: '*eToys-form dictionary' stamp: 'sw 12/7/1998 16:47'!
standardForms
	"ScriptingSystem standardForms"
	^ FormDictionary collect: [:f | f]! !

!StandardScriptingSystem methodsFor: '*eToys-tile colors' stamp: 'sw 8/28/2004 20:31'!
standardTileBorderColor
	"Answer the color to use for tile borders"

	^ Color r: 0.804 g: 0.76 b: 0.564! !

!StandardScriptingSystem methodsFor: '*eToys-font & color choices' stamp: 'sw 9/14/1998 14:41'!
statusColorSymbolFor: statusSymbol
	#(	(normal					green)
		(ticking					blue)
		(paused					red)
		(mouseDown				yellow)
		(mouseStillDown			lightYellow)
		(mouseUp				lightBlue)
		(mouseEnter				lightBrown)
		(mouseLeave			lightRed)
		(mouseEnterDragging	lightGray)
		(mouseLeaveDragging	darkGray)
		(keyStroke				lightGreen)) do:

			[:pair | statusSymbol == pair first ifTrue: [^ pair second]].

		^ #blue! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-help dictionary' stamp: 'nk 11/1/2004 08:21'!
statusHelpStringFor: aPlayer
	^String streamContents: [ :stream |
		stream nextPutAll: 'normal -- run when called
paused -- ready to run all the time
ticking -- run all the time
mouseDown -- run when mouse goes down on me
mouseStillDown -- while mouse still down
mouseUp -- when mouse comes back up
mouseEnter -- when mouse enters my bounds, button up
mouseLeave -- when mouse exits my bounds, button up
mouseEnterDragging -- when mouse enters my bounds, button down
mouseLeaveDragging -- when mouse exits my bounds, button down
opening -- when I am being opened
closing -- when I am being closed' translated.

"'keyStroke -- run when user hits a key' "

	stream cr; cr; nextPutAll: 'More events:' translated; cr.

	(self customEventNamesAndHelpStringsFor: aPlayer) do: [ :array |
		stream cr;
		nextPutAll: array first;
		nextPutAll: ' -- '.
		array second do: [ :help | stream nextPutAll: help translated ]
			separatedBy: [ stream nextPutAll: ' or ' translated ]].

	(Preferences allowEtoyUserCustomEvents) ifTrue: [
	self userCustomEventNames isEmpty ifFalse: [
		stream cr; cr; nextPutAll: 'User custom events:' translated; cr.
		self currentWorld userCustomEventsRegistry keysAndValuesDo: [ :key :value |
			stream cr; nextPutAll: key; nextPutAll: ' -- '; nextPutAll: value ]]]]! !

!StandardScriptingSystem methodsFor: '*eToys-script-control' stamp: 'dgd 9/19/2003 14:41'!
stepButton
	| aButton |
	self flag: #deferred.  "ambiguity about recipients"
	aButton _ ThreePhaseButtonMorph new.
		aButton
			image:  (ScriptingSystem formAtKey: 'StepPicOn');
			offImage: (ScriptingSystem formAtKey: 'StepPic');
			pressedImage:  (ScriptingSystem formAtKey: 'StepPicOn');
			arguments: (Array with: nil with: aButton);
		 	actionSelector: #stepStillDown:with:; 
			target: self;
			setNameTo: 'Step Button'; 
			actWhen: #whilePressed;
			on: #mouseDown send: #stepDown:with: to: self;
			on: #mouseStillDown send: #stepStillDown:with: to: self;
			on: #mouseUp send: #stepUp:with: to: self;
			setBalloonText:
'Run every paused script exactly once.  Keep the mouse button down over "Step" and everything will keep running until you release it' translated.
	^ aButton! !

!StandardScriptingSystem methodsFor: '*eToys-script-control' stamp: 'sw 10/30/1998 15:33'!
stepDown: evt with: aMorph
	aMorph presenter stopRunningScripts! !

!StandardScriptingSystem methodsFor: '*eToys-script-control' stamp: 'sw 10/30/1998 15:35'!
stepStillDown: dummy with: theButton
	theButton presenter stepStillDown: dummy with: theButton! !

!StandardScriptingSystem methodsFor: '*eToys-script-control' stamp: 'sw 10/30/1998 15:35'!
stepUp: evt with: aMorph
	aMorph presenter stepUp: evt with: aMorph! !

!StandardScriptingSystem methodsFor: '*eToys-script-control' stamp: 'dgd 9/19/2003 14:41'!
stopButton
	"Answer a new button that can serve as a stop button"
	| aButton |
	aButton _ ThreePhaseButtonMorph new.
	aButton
		image:  (ScriptingSystem formAtKey: 'StopPic');
		offImage: (ScriptingSystem formAtKey: 'StopPic');
		pressedImage:  (ScriptingSystem formAtKey: 'StopPicOn').
		aButton actionSelector: #stopUp:with:; 
		arguments: (Array with: nil with: aButton);
		actWhen: #buttonUp;
		target: self;
		setNameTo: 'Stop Button'; 
		setBalloonText: 'Pause all ticking scripts.' translated.
	^ aButton! !

!StandardScriptingSystem methodsFor: '*eToys-script-control' stamp: 'sw 11/11/1998 15:16'!
stopUp: dummy with: theButton
	| aPresenter |
	(aPresenter _ theButton presenter) flushPlayerListCache.  "catch guys not in cache but who're running"
	aPresenter stopRunningScriptsFrom: theButton! !

!StandardScriptingSystem methodsFor: '*eToys-universal slots & scripts' stamp: 'sw 9/27/2001 04:08'!
systemSlotNamesOfType: aType
	"Answer the type of the slot name, or nil if not found."
	
	| aList |
	self flag: #deferred.  "Hard-coded etoyVocabulary needed here to make this work."
	aList _ OrderedCollection new.
	Vocabulary eToyVocabulary methodInterfacesDo:
		 [:anInterface |
			anInterface resultType = aType ifTrue:
				[aList add: anInterface selector]].
	^ aList! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'sw 9/27/2001 13:28'!
tileForArgType: aType
	"Anwer a default tile to represent a datum of the given argument type, which may be either a symbol (e.g. #Color) or a class"

	(aType isKindOf: Class)  "Allowed in Ted's work"
		ifTrue:
			[^ aType name asString newTileMorphRepresentative typeColor: Color gray].

	^ (Vocabulary vocabularyForType: aType) defaultArgumentTile! !

!StandardScriptingSystem methodsFor: '*eToys-parts bin' stamp: 'sw 7/3/2001 08:01'!
tilesForQuery: expressionString label: aLabel
	"Answer scripting tiles that represent the query,"

	| aPhrase aTile |
	aPhrase _ SystemQueryPhrase new.
	aTile _ BooleanTile new.
	aTile setExpression: expressionString  label: aLabel.
	aPhrase addMorph: aTile.
	^ aPhrase
! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'dgd 7/12/2003 12:05'!
tryButtonFor: aPhraseTileMorph 
	| aButton |
	aButton := SimpleButtonMorph new.
	aButton target: aPhraseTileMorph;
		 actionSelector: #try;
		
		label: '!!'
		font: Preferences standardEToysFont;
		 color: Color yellow;
		 borderWidth: 0.
	aButton actWhen: #whilePressed.
	aButton balloonTextSelector: #try.
	^ aButton! !

!StandardScriptingSystem methodsFor: '*eToys-font & color choices' stamp: 'sw 5/2/1998 14:23'!
uniformTileInteriorColor
	^ Color r: 0.806 g: 1.0 b: 0.806! !

!StandardScriptingSystem methodsFor: '*eToys-viewer' stamp: 'sw 10/30/2000 09:07'!
uniqueNameForReference
	"Answer a more-or-less global name by which the receiver can be referred to in scripts"

	^ #ScriptingSystem! !

!StandardScriptingSystem methodsFor: '*eToys-customevents-custom events' stamp: 'nk 9/26/2003 23:22'!
userCustomEventNames
	^ self currentWorld userCustomEventNames! !

!StandardScriptingSystem methodsFor: '*eToys-utilities' stamp: 'nk 10/14/2004 11:19'!
wordingForOperator: aString
	"Answer the wording to be seen by the user for the given operator symbol/string"

	| toTest |
	toTest _ aString asString.
	#(	(append:				'include at end')
		(arrowheadsOnAllPens	'arrowheads on all pens')
		(beep:					'make sound')
		(bounce:				'bounce')
		(clearTurtleTrails		'clear pen trails')
		(clearOwnersPenTrails	'clear all pen trails')
		(colorSees				'color  sees')
		(color:sees:				'color sees')
		(doMenuItem:			'do menu item')
		(doScript:				'do')
		(forward:				'forward by')
		(goToRightOf:			'align after')
		(includeAtCursor:		'include at cursor')
		(isDivisibleBy:			'is divisible by')
		(liftAllPens				'lift all pens')
		(lowerAllPens			'lower all pens')
		(makeNewDrawingIn:	'start painting in')
		(max:					'max')
		(min:					'min')
		(moveToward:			'move toward')
		(noArrowheadsOnAllPens	'no arrowheads on pens')
		(overlapsAny			'overlaps any')
		(pauseAll:				'pause all')
		(pauseScript:			'pause script')
		(prepend:				'include at beginning')
		(seesColor:				'is over color')
		(startAll:				'start all')
		(startScript:				'start script')
		(stopProgramatically	'stop')
		(stopAll:					'stop all')
		(stopScript:				'stop script')
		(tellAllSiblings:			'tell all siblings')
		(tellSelfAndAllSiblings:	'send to all')
		(turn:					'turn by')
		(turnToward:				'turn toward')
		(wearCostumeOf:		'look like'))

	do:
		[:pair | toTest = pair first ifTrue: [^ pair second]].

	^ toTest

	"StandardScriptingSystem initialize"

! !

!MatrixTransformMorph methodsFor: '*eToys-geometry eToy' stamp: 'ar 6/12/2001 06:03'!
heading: newHeading
	"Set the receiver's heading (in eToy terms)"
	self rotateBy: ((newHeading - self forwardDirection) - self innerAngle).! !

!GrabPatchMorph methodsFor: '*eToys-misc' stamp: 'sw 7/5/2004 01:49'!
isCandidateForAutomaticViewing
	"Answer whether the receiver is a candidate for automatic viewing.  Only relevant if a now-seldom-used feature, automaticViewing, is in play"

	^ self isPartsDonor not! !

!NumberType methodsFor: '*eToys-tiles' stamp: 'dgd 9/6/2003 20:30'!
addExtraItemsToMenu: aMenu forSlotSymbol: slotSym
	"If the receiver has extra menu items to add to the slot menu, here is its chance to do it.  The defaultTarget of the menu is the player concerned."

	aMenu add: 'decimal places...' translated selector: #setPrecisionFor: argument: slotSym.
	aMenu balloonTextForLastItem: 'Lets you choose how many decimal places should be shown in readouts for this variable' translated! !

!NumberType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 02:53'!
comparatorForSampleBoolean
	"Answer the comparator to use in tile coercions involving the receiver; normally, the equality comparator is used but NumberType overrides"

	^ #<! !

!NumberType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:30'!
defaultArgumentTile
	"Answer a tile to represent the type"

	^ 5 newTileMorphRepresentative typeColor: self typeColor! !

!NumberType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:37'!
newReadoutTile
	"Answer a tile that can serve as a readout for data of this type"

	^ NumericReadoutTile new typeColor: Color lightGray lighter! !

!NumberType methodsFor: '*eToys-color' stamp: 'sw 9/27/2001 17:21'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(0.8 0.4 0.2)! !

!NumberType methodsFor: '*eToys-tiles' stamp: 'sw 9/26/2001 03:11'!
wantsAssignmentTileVariants
	"Answer whether an assignment tile for a variable of this type should show variants to increase-by, decrease-by, multiply-by."

	^ true! !

!NumberType methodsFor: '*eToys-tiles' stamp: 'sw 9/26/2001 03:18'!
wantsSuffixArrow
	"Answer whether a tile showing data of this type would like to have a suffix arrow"

	^ true! !

!UndefinedObject methodsFor: '*eToys-*Morphic-customevents-class initialization' stamp: 'nk 11/1/2004 07:48'!
initialize
	"AlansTextPlusMorph initialize"
	ScriptingSystem addCustomEventFor: self named: #scrolledIntoView help: 'when I am scrolled into view in a GeeMailMorph' targetMorphClass: Morph.
	ScriptingSystem addCustomEventFor: self named: #scrolledOutOfView help: 'when I am scrolled out of view in a GeeMailMorph'  targetMorphClass: Morph.
! !

!UndefinedObject methodsFor: '*eToys-printing' stamp: 'sw 10/29/1998 16:34'!
newTileMorphRepresentative
	^ UndescribedTile new! !

!Boolean methodsFor: '*eToys-*morphic' stamp: 'sw 8/20/1999 17:42'!
newTileMorphRepresentative
	^ TileMorph new addArrows; setLiteral: self
! !

!ButtonPhaseType methodsFor: '*eToys-color' stamp: 'sw 9/27/2001 17:20'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(0.806 1.0 0.806)	! !

!MorphExtension methodsFor: '*eToys-accessing' stamp: 'dgd 2/16/2003 21:56'!
actorState
	"answer the redeiver's actorState"
	^ actorState ! !

!MorphExtension methodsFor: '*eToys-accessing' stamp: 'dgd 2/16/2003 21:53'!
actorState: anActorState 
"change the receiver's actorState"
	actorState _ anActorState! !

!MorphExtension methodsFor: '*eToys-accessing' stamp: 'dgd 2/16/2003 21:42'!
player
	"answer the receiver's player"
	^ player! !

!MorphExtension methodsFor: '*eToys-accessing' stamp: 'dgd 2/16/2003 21:53'!
player: anObject 
	"change the receiver's player"
	player _ anObject ! !

!FlapTab methodsFor: '*eToys-e-toy support' stamp: 'sw 1/25/2000 11:07'!
isCandidateForAutomaticViewing
	^ false! !

!FlapTab methodsFor: '*eToys-e-toy support' stamp: 'sw 7/28/2001 01:31'!
succeededInRevealing: aPlayer
	"Try to reveal aPlayer, and answer whether we succeeded"

	(super succeededInRevealing: aPlayer) ifTrue: [^ true].
	self flapShowing ifTrue: [^ false].
	(referent succeededInRevealing: aPlayer)
		ifTrue:
			[self showFlap.
			aPlayer costume goHome; addHalo.
			^ true].
	^ false! !

Notification subclass: #GetTriggeringObjectNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-CustomEvents'!

!GetTriggeringObjectNotification commentStamp: '<historical>' prior: 0!
This is used to report on the sender of #triggerScript:!

!ColorPickerMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 7/6/1999 09:00'!
isCandidateForAutomaticViewing
	^ false! !

!SystemDictionary methodsFor: '*eToys-copying' stamp: 'sw 11/21/2001 15:08'!
assureUniClass
	"Assure that the receiver has a uniclass.  Or rather, in this case, stop short of fulfilling such a request"

	self error: 'We do not want uniclasses descending from here'
! !

!SystemDictionary methodsFor: '*eToys-copying' stamp: 'sw 10/23/2001 10:23'!
vocabularyDemanded
	"Answer the vocabulary that the receiver really would like to use in a Viewer"

	^ Vocabulary vocabularyNamed: #System! !

!MenuItemMorph class methodsFor: '*eToys-scripting' stamp: 'sw 2/7/2001 00:04'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((button (
			(command fire 'trigger any and all of this object''s button actions'))))
! !

!PaintInvokingMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 6/30/1999 20:31'!
isCandidateForAutomaticViewing
	^ self isPartsDonor not! !

!Preferences class methodsFor: '*eToys-customevents-preferences' stamp: 'sw 6/16/2005 01:38'!
allowEtoyUserCustomEvents
	^ self
		valueOfFlag: #allowEtoyUserCustomEvents
		ifAbsent: [false]! !

AlignmentMorphBob1 subclass: #GenericPropertiesMorph
	instanceVariableNames: 'myTarget thingsToRevert'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Experimental'!

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 16:23'!
addARow: anArray

	^(super addARow: anArray) cellPositioning: #topLeft! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'ar 4/10/2005 22:18'!
buildFakeSlider: nameStringOrSymbol selector: aSymbol help: helpString 
	| col |
	col := self inAColumn: { 
						(nameStringOrSymbol isSymbol) 
							ifTrue: 
								[(UpdatingStringMorph new)
									useStringFormat;
									getSelector: nameStringOrSymbol;
									target: self;
									growable: true;
									minimumWidth: 24;
									lock]
							ifFalse: [self lockedString: nameStringOrSymbol]}.
	col
		borderWidth: 2;
		borderColor: color darker;
		color: color muchLighter;
		hResizing: #shrinkWrap;
		setBalloonText: helpString;
		on: #mouseMove
			send: #mouseAdjust:in:
			to: self;
		on: #mouseDown
			send: #mouseAdjust:in:
			to: self;
		on: #mouseUp
			send: #clearSliderFeedback
			to: self;
		setProperty: #changeSelector toValue: aSymbol.
	^col! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'dgd 11/2/2004 21:26'!
buttonNamed: aString action: aSymbol color: aColor help: helpString

	| f col |
	f _ SimpleButtonMorph new
		target: self;
		label: aString;
		color: aColor;
		borderColor: aColor muchDarker;
		actionSelector: aSymbol;
		setBalloonText: helpString.
	col _ (self inAColumn: {f}) hResizing: #shrinkWrap.
	^col! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 16:41'!
clearSliderFeedback

	| feedBack |

	feedBack _ self valueOfProperty: #sliderFeedback ifAbsent: [^self].
	feedBack delete! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 16:15'!
colorPickerFor: target getter: getterSymbol setter: setterSymbol

	^ColorPickerMorph new
		initializeForPropertiesPanel;
		target: target;
		selector: setterSymbol;
		originalColor: (target perform: getterSymbol)! !

!GenericPropertiesMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:15'!
defaultBorderWidth
"answer the default border width for the receiver"
	^ 4! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 14:36'!
directToggleButtonFor: target getter: getterSymbol setter: setterSymbol help: helpText

	^(EtoyUpdatingThreePhaseButtonMorph checkBox)
		target: target;
		actionSelector: setterSymbol;
		arguments: #();
		getSelector: getterSymbol;
		setBalloonText: helpText;
		step
! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 14:36'!
doAccept

	self delete! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 19:40'!
doCancel

	thingsToRevert keysAndValuesDo: [ :k :v |
		myTarget perform: k with: v
	].
	self delete! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 16:24'!
doEnables! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/9/2001 12:50'!
doMainProperties

	myTarget openAPropertySheet.
	self delete.
! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/15/2001 12:57'!
doTextProperties

	myTarget openATextPropertySheet.
	self delete.
! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 16:23'!
enable: aMorph when: aBoolean

	aBoolean = (aMorph hasProperty: #disabledMaskColor) ifFalse: [^self].
	aBoolean ifTrue: [
		aMorph 
			removeProperty: #disabledMaskColor;
			lock: false;
			changed.
		^self
	].
	aMorph 
		setProperty: #disabledMaskColor toValue: (Color black alpha: 0.5);
		lock: true;
		changed
! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 16:22'!
inAColumn: aCollectionOfMorphs

	| col |
	col _ AlignmentMorphBob1 newColumn
		color: Color transparent;
		vResizing: #shrinkWrap;
		layoutInset: 1;
		wrapCentering: #center;
		cellPositioning: #topCenter.
	aCollectionOfMorphs do: [ :each | col addMorphBack: each].
	^col! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 16:22'!
inAColumn: anArray named: aString

	^(self inAColumn: anArray) setNamePropertyTo: aString! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/9/2001 12:27'!
inARow: aCollectionOfMorphs

	| row |
	row _ AlignmentMorphBob1 newRow
		color: Color transparent;
		vResizing: #shrinkWrap;
		layoutInset: 1;
		wrapCentering: #center;
		cellPositioning: #leftCenter.
	aCollectionOfMorphs do: [ :each | row addMorphBack: each].
	^row
! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 16:22'!
inARow: anArray named: aString

	^(self inARow: anArray) setNamePropertyTo: aString! !

!GenericPropertiesMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:16'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""

	self layoutInset: 4.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.
	thingsToRevert _ Dictionary new.
	self useRoundedCorners! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 14:37'!
lockedString: s

	^(StringMorph contents: s) lock.
! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/9/2001 10:22'!
mouseAdjust: evt in: aMorph

	| fractionalPosition feedBack testExtent |

	feedBack _ self showSliderFeedback: nil.
	feedBack world ifNil: [
		feedBack bottomLeft: evt cursorPoint - (0@8)
	].
	testExtent _ 100@100.		"the real extent may change"
	fractionalPosition _ (evt cursorPoint - aMorph topLeft) / testExtent.
	self 
		perform: (aMorph valueOfProperty: #changeSelector)
		with: fractionalPosition
! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/9/2001 13:09'!
openNearTarget

	| w wb tb leftOverlap rightOverlap topOverlap bottomOverlap best |

	w _ myTarget world ifNil: [World].
	wb _ w bounds.
	self fullBounds.
	tb _ myTarget boundsInWorld.
	leftOverlap _ self width - (tb left - wb left).
	rightOverlap _ self width - (wb right - tb right).
	topOverlap _ self height - (tb top - wb top).
	bottomOverlap _ self height - (wb bottom - tb bottom).
	best _ nil.
	{
		{leftOverlap. #topRight:. #topLeft}.
		{rightOverlap. #topLeft:. #topRight}.
		{topOverlap. #bottomLeft:. #topLeft}.
		{bottomOverlap. #topLeft:. #bottomLeft}.
	} do: [ :tuple |
		(best isNil or: [tuple first < best first]) ifTrue: [best _ tuple].
	].
	self perform: best second with: (tb perform: best third).
	self bottom: (self bottom min: wb bottom) rounded.
	self right: (self right min: wb right) rounded.
	self top: (self top max: wb top) rounded.
	self left: (self left max: wb left) rounded.

	self openInWorld: w.! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/9/2001 10:12'!
showSliderFeedback: aString

	| feedBack |

	feedBack _ self 
		valueOfProperty: #sliderFeedback 
		ifAbsent: [
			feedBack _ AlignmentMorph newRow
				hResizing: #shrinkWrap;
				vResizing: #shrinkWrap;
				color: (Color yellow" alpha: 0.6");
				addMorph: (
					TextMorph new 
						contents: '?';
						beAllFont: ((TextStyle default fontOfSize: 24) emphasized: 1)
				).
			self setProperty: #sliderFeedback toValue: feedBack.
			feedBack
		].
	aString ifNotNil: [
		feedBack firstSubmorph contents: aString asString.
		feedBack world ifNil: [feedBack openInWorld].
	].
	^feedBack! !

!GenericPropertiesMorph methodsFor: 'stepping and presenter' stamp: 'RAA 3/8/2001 16:24'!
step

	super step.
	self doEnables! !

!GenericPropertiesMorph methodsFor: 'testing' stamp: 'RAA 3/8/2001 16:24'!
stepTime

	^500! !

!GenericPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:01'!
targetMorph: x

	myTarget _ x! !

!GenericPropertiesMorph methodsFor: 'dropping/grabbing' stamp: 'tk 7/11/2001 14:00'!
wantsToBeDroppedInto: aMorph
	"Return true if it's okay to drop the receiver into aMorph"
	^aMorph isWorldMorph or:[Preferences systemWindowEmbedOK]! !

GenericPropertiesMorph subclass: #ObjectPropertiesMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Experimental'!

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:04'!
adjustTargetBorderWidth: aFractionalPoint

	| n |

	myTarget borderWidth: (n _ (aFractionalPoint x * 10) rounded max: 0).
	self showSliderFeedback: n.! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:02'!
adjustTargetGradientDirection: aFractionalPoint

	| fs p |

	(fs _ myTarget fillStyle) isGradientFill ifFalse: [^self].
	fs direction: (p _ (aFractionalPoint * myTarget extent) rounded).
	self showSliderFeedback: p.
	myTarget changed.
! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:05'!
adjustTargetGradientOrigin: aFractionalPoint

	| fs p |

	(fs _ myTarget fillStyle) isGradientFill ifFalse: [^self].
	fs origin: (p _ myTarget topLeft + (aFractionalPoint * myTarget extent) rounded).
	self showSliderFeedback: p.
	myTarget changed.
! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:03'!
adjustTargetShadowOffset: aFractionalPoint

	| n |

	myTarget changed; layoutChanged.
	myTarget shadowOffset: (n _ (aFractionalPoint * 4) rounded).
	self showSliderFeedback: n.
	myTarget changed; layoutChanged.
! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'sd 11/13/2003 21:03'!
borderPrototype: aBorderStyle help: helpString
	| selector proto |
	selector _ BorderedMorph new.
	selector borderWidth: 0.
	selector color: Color transparent.
	proto _ Morph new extent: 16@16.
	proto color:  Color transparent.
	proto borderStyle: aBorderStyle.
	selector extent: proto extent + 4.
	selector addMorphCentered: proto.
	(myTarget canDrawBorder: aBorderStyle) ifTrue:[
		selector setBalloonText: helpString.
		selector on: #mouseDown send: #toggleBorderStyle:with:from: to: self withValue: proto.
		(myTarget borderStyle species == aBorderStyle species and:[
			myTarget borderStyle style == aBorderStyle style]) ifTrue:[selector borderWidth: 1].
	] ifFalse:[
		selector setBalloonText: 'This border style cannot be used here' translated.
		selector on: #mouseDown send: #beep to: Beeper.
		selector addMorphCentered: ((Morph new) color: (Color black alpha: 0.5); extent: selector extent).
	].
	^selector! !

!ObjectPropertiesMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:44'!
defaultBorderColor
"answer the default border color/fill style for the receiver"
	^ self defaultColor darker! !

!ObjectPropertiesMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:44'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color
		r: 0.548
		g: 0.839
		b: 0.452! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:03'!
doEnables

	| itsName fs |

	fs _ myTarget fillStyle.
	self allMorphsDo: [ :each |
		itsName _ each knownName.
		itsName == #pickerForColor ifTrue: [
			self enable: each when: fs isSolidFill | fs isGradientFill
		].
		itsName == #pickerForBorderColor ifTrue: [
			self enable: each when: (myTarget respondsTo: #borderColor:)
		].
		itsName == #pickerForShadowColor ifTrue: [
			self enable: each when: myTarget hasDropShadow
		].
		itsName == #pickerFor2ndGradientColor ifTrue: [
			self enable: each when: fs isGradientFill
		].
	].
! !

!ObjectPropertiesMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:44'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	myTarget
		ifNil: [myTarget _ RectangleMorph new openInWorld].
	self rebuild! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:03'!
makeTargetGradientFill

	myTarget useGradientFill! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:02'!
makeTargetSolidFill

	myTarget useSolidFill! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:04'!
numberOneColor

	myTarget fillStyle isGradientFill ifFalse: [^myTarget color].
	^myTarget fillStyle colorRamp first value
! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'gm 2/16/2003 20:36'!
numberOneColor: aColor 
	myTarget fillStyle isGradientFill 
		ifFalse: 
			[^(myTarget isSystemWindow) 
				ifTrue: [myTarget setWindowColor: aColor]
				ifFalse: [myTarget fillStyle: aColor]].
	myTarget fillStyle 
		firstColor: aColor
		forMorph: myTarget
		hand: nil! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:21'!
paneFor2ndGradientColorPicker

	^self 
		inAColumn: {
			(self inAColumn: {
				self colorPickerFor: self getter: #tgt2ndGradientColor setter: #tgt2ndGradientColor:.
				self lockedString: '2nd gradient color' translated.
				self paneForRadialGradientToggle hResizing: #shrinkWrap.
				(
					self inARow: {self paneForGradientOrigin. self paneForGradientDirection}
				) hResizing: #shrinkWrap.
			}
			named: #pickerFor2ndGradientColor) layoutInset: 0.
			self paneForGradientFillToggle hResizing: #shrinkWrap 
		}
! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:23'!
paneForBorderColorPicker

	^self 
		inAColumn: {
			self 
				colorPickerFor: self
				getter: #targetBorderColor
				setter: #targetBorderColor:.
			self lockedString: 'Border Color' translated.
			(self paneForBorderStyle) hResizing: #shrinkWrap; layoutInset: 5.
			self lockedString: 'Border style' translated.
			self paneForBorderWidth.
		} 
		named: #pickerForBorderColor.

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:25'!
paneForBorderStyle

	^self inARow: {
		self borderPrototype: (BorderStyle width: 4 color: Color black)
			help:'Click to select a simple colored border' translated.
		self borderPrototype: (BorderStyle raised width: 4)
			help:'Click to select a simple raised border' translated.
		self borderPrototype: (BorderStyle inset width: 4)
			help:'Click to select a simple inset border' translated.
		self borderPrototype: (BorderStyle complexFramed width: 4)
			help:'Click to select a complex framed border' translated.
		self borderPrototype: (BorderStyle complexRaised width: 4)
			help:'Click to select a complex raised border' translated.
		self borderPrototype: (BorderStyle complexInset width: 4)
			help:'Click to select a complex inset border' translated.
		self borderPrototype: (BorderStyle complexAltFramed width: 4)
			help:'Click to select a complex framed border' translated.
		self borderPrototype: (BorderStyle complexAltRaised width: 4)
			help:'Click to select a complex raised border' translated.
		self borderPrototype: (BorderStyle complexAltInset width: 4)
			help:'Click to select a complex inset border' translated.
	}

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:29'!
paneForBorderWidth

	^(self inARow: {
		self
			buildFakeSlider: 'Border width' translated
			selector: #adjustTargetBorderWidth:
			help: 'Drag in here to change the border width' translated
	}) hResizing: #shrinkWrap

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:18'!
paneForCornerRoundingToggle

	^self inARow: {
		self
			directToggleButtonFor: myTarget 
			getter: #wantsRoundedCorners setter: #toggleCornerRounding
			help: 'Turn rounded corners on or off' translated.
		self lockedString: ' Rounded corners' translated.
	}

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:40'!
paneForDropShadowToggle

	^self inARow: {
		self
			directToggleButtonFor: myTarget 
			getter: #hasDropShadow setter: #toggleDropShadow
			help: 'Turn drop shadows on or off' translated.
		self lockedString: ' Drop shadow color' translated.
	}
! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:22'!
paneForGradientDirection

	^(self inARow: {
		self
			buildFakeSlider: 'Direction' translated
			selector: #adjustTargetGradientDirection:
			help: 'Drag in here to change the direction of the gradient' translated
	}) hResizing: #shrinkWrap

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:22'!
paneForGradientFillToggle

	^self inARow: {
		self
			directToggleButtonFor: self 
			getter: #targetHasGradientFill
			setter: #toggleTargetGradientFill
			help: 'Turn gradient fill on or off' translated.
		self lockedString: ' Gradient fill' translated.
	}
! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:21'!
paneForGradientOrigin

	^(self inARow: {
		self
			buildFakeSlider: 'Origin' translated
			selector: #adjustTargetGradientOrigin:
			help: 'Drag in here to change the origin of the gradient' translated
	}) hResizing: #shrinkWrap

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:19'!
paneForLockedToggle

	^self inARow: {
		self
			directToggleButtonFor: myTarget 
			getter: #isLocked setter: #toggleLocked
			help: 'Turn lock on or off' translated.
		self lockedString: ' Lock' translated.
	}

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:20'!
paneForMainColorPicker

	^self 
		inAColumn: {
			self 
				colorPickerFor: self 
				getter: #numberOneColor 
				setter: #numberOneColor:.
			self lockedString: 'Color' translated.
			(self paneForSolidFillToggle)  hResizing: #shrinkWrap.
		} 
		named: #pickerForColor.

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:21'!
paneForRadialGradientToggle

	^self inARow: {
		self
			directToggleButtonFor: self 
			getter: #targetRadial setter: #toggleTargetRadial
			help: 'Turn radial gradient on or off' translated.
		self lockedString: ' Radial gradient' translated.
	}

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'RAA 3/8/2001 18:03'!
paneForShadowColorPicker

	^self 
		inAColumn: {
			(self inAColumn: {
				self colorPickerFor: myTarget getter: #shadowColor setter: #shadowColor:.
				self paneForShadowOffset.
			}
			named: #pickerForShadowColor) layoutInset: 0.
			self paneForDropShadowToggle hResizing: #shrinkWrap.
		}

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:36'!
paneForShadowOffset

	^(self inARow: {
		self
			buildFakeSlider: 'Offset' translated
			selector: #adjustTargetShadowOffset:
			help: 'Drag in here to change the offset of the shadow' translated
	}) hResizing: #shrinkWrap

! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:20'!
paneForSolidFillToggle

	^self inARow: {
		self
			directToggleButtonFor: self 
			getter: #targetHasSolidFill
			setter: #toggleTargetSolidFill
			help: 'Turn solid fill on or off' translated.
		self lockedString: ' Solid fill' translated.
	}
! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'dgd 8/31/2003 21:18'!
paneForStickinessToggle

	^self inARow: {
		self
			directToggleButtonFor: myTarget 
			getter: #isSticky setter: #toggleStickiness
			help: 'Turn stickiness on or off' translated.
		self lockedString: ' Sticky' translated.
	}

! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'dgd 10/8/2003 19:35'!
rebuild

	self removeAllMorphs.
	self addARow: {
		self lockedString: ('Properties for {1}' translated format: {myTarget name}).
	}.
	self addARow: {
		self inAColumn: {
			self paneForCornerRoundingToggle.
			self paneForStickinessToggle.
			self paneForLockedToggle.
		}.
	}.

	self addARow: {
		self paneForMainColorPicker.
		self paneFor2ndGradientColorPicker.
	}.
	self addARow: {
		self paneForBorderColorPicker.
		self paneForShadowColorPicker.
	}.

	self addARow: {
		self 
			buttonNamed: 'Accept' translated action: #doAccept color: color lighter 
			help: 'keep changes made and close panel' translated.
		self 
			buttonNamed: 'Cancel' translated action: #doCancel color: color lighter 
			help: 'cancel changes made and close panel' translated.
	}, self rebuildOptionalButtons.

	thingsToRevert _ Dictionary new.
	"thingsToRevert at: #fillStyle: put: myTarget fillStyle."
	myTarget isSystemWindow ifTrue: [
		thingsToRevert at: #setWindowColor: put: myTarget paneColorToUse
	].
	thingsToRevert at: #hasDropShadow: put: myTarget hasDropShadow.
	thingsToRevert at: #shadowColor: put: myTarget shadowColor.
	(myTarget respondsTo: #borderColor:) ifTrue: [
		thingsToRevert at: #borderColor: put: myTarget borderColor.
	].

	thingsToRevert at: #borderWidth: put: myTarget borderWidth.
	thingsToRevert at: #cornerStyle: put: myTarget cornerStyle.
	thingsToRevert at: #sticky: put: myTarget isSticky.
	thingsToRevert at: #lock: put: myTarget isLocked.
! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'wiz 9/21/2005 22:42'!
rebuildOptionalButtons

	| answer |

	answer _ #() .
	
	myTarget isTextMorph ifTrue: [
		answer _ answer, {
			self 
				buttonNamed: 'Text' translated action: #doTextProperties color: color lighter 
				help: 'open a text properties panel for the morph' translated.
		}.
	].
	^answer! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'ar 8/25/2001 18:30'!
targetBorderColor
	^myTarget borderStyle baseColor! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'ar 11/26/2001 15:29'!
targetBorderColor: aColor
	"Need to replace the borderStyle or BorderedMorph will not 'feel' the change"
	myTarget borderStyle: (myTarget borderStyle copy baseColor: aColor).! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:04'!
targetHasGradientFill

	^myTarget fillStyle isGradientFill! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:01'!
targetHasSolidFill

	^myTarget fillStyle isSolidFill! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:03'!
targetRadial

	myTarget fillStyle isGradientFill ifFalse: [^false].
	^myTarget fillStyle radial! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:03'!
tgt2ndGradientColor

	myTarget fillStyle isGradientFill ifFalse: [^Color black].
	^myTarget fillStyle colorRamp last value! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:03'!
tgt2ndGradientColor: aColor

	myTarget fillStyle lastColor: aColor forMorph: myTarget hand: nil
! !

!ObjectPropertiesMorph methodsFor: 'panes' stamp: 'ar 8/25/2001 18:35'!
toggleBorderStyle: provider with: arg1 from: arg2
	| oldStyle newStyle |
	oldStyle _ myTarget borderStyle.
	newStyle _ provider borderStyle copy.
	oldStyle width = 0 
		ifTrue:[newStyle width: 2]
		ifFalse:[newStyle width: oldStyle width].
	newStyle baseColor: oldStyle baseColor.
	myTarget borderStyle: newStyle.
	provider owner owner submorphsDo:[:m| m borderWidth: 0].
	provider owner borderWidth: 1.! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 2/20/2001 17:45'!
toggleTargetGradientFill

	self targetHasGradientFill ifTrue: [
		self makeTargetSolidFill
	] ifFalse: [
		self makeTargetGradientFill
	].
	self doEnables! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 18:02'!
toggleTargetRadial

	| fs |

	(fs _ myTarget fillStyle) isGradientFill ifFalse: [^self].
	fs radial: fs radial not.
	myTarget changed.
	self doEnables.! !

!ObjectPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 2/20/2001 17:48'!
toggleTargetSolidFill

	self targetHasSolidFill ifTrue: [
		self makeTargetGradientFill
	] ifFalse: [
		self makeTargetSolidFill
	].
	self doEnables! !

GenericPropertiesMorph subclass: #TextPropertiesMorph
	instanceVariableNames: 'activeTextMorph applyToWholeText lastGlobalColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Experimental'!

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/15/2001 11:41'!
activeEditor

	^self activeTextMorph editor! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/15/2001 11:41'!
activeTextMorph

	^activeTextMorph! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/21/2001 11:26'!
adjustTargetMargin: aFractionalPoint

	| n |

	n _ (aFractionalPoint * 4) rounded.
	myTarget margins: n.
	self showSliderFeedback: n.
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/15/2001 11:57'!
applyToWholeText

	^applyToWholeText! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:06'!
changeKernBy: delta

	self changeSelectionAttributeTo: (TextKern kern: delta)! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:05'!
changeSelectionAttributeTo: newAttribute

	self applyToWholeText ifTrue: [self activeEditor selectAll].
	self activeEditor replaceSelectionWith: (
		self activeEditor selection asText addAttribute: newAttribute
	).
	self activeTextMorph updateFromParagraph.! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'md 10/22/2003 15:22'!
changeStyle

	| aList reply style |

	aList _ StrikeFont actualFamilyNames.
	aList addFirst: 'DefaultTextStyle'.
	reply _ (SelectionMenu labelList: aList lines: #(1) selections: aList) startUp.
	reply ifNil: [^self].

	(style _ TextStyle named: reply) ifNil: [Beeper beep. ^ true].
	self applyToWholeText ifTrue: [self activeEditor selectAll].
	self activeEditor changeStyleTo: style copy.
	self activeTextMorph updateFromParagraph.! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/21/2001 11:07'!
changeTargetColorTo: aColor

	self applyToWholeText ifTrue: [
		lastGlobalColor _ aColor
	].
	self changeSelectionAttributeTo: (TextColor color: aColor)! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:07'!
changeToNormalText

	self changeSelectionAttributeTo: (TextEmphasis normal)! !

!TextPropertiesMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:55'!
defaultBorderColor
"answer the default border color/fill style for the receiver"
	^ self defaultColor darker! !

!TextPropertiesMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:55'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color
		r: 0.355
		g: 0.742
		b: 0.839! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/21/2001 11:06'!
doAccept

	myTarget 
		text: self activeTextMorph contents textStyle: self activeTextMorph textStyle;
		releaseCachedState;
		changed.
	lastGlobalColor ifNotNil: [myTarget textColor: lastGlobalColor].
	super doAccept.! !

!TextPropertiesMorph methodsFor: 'initialization' stamp: 'nk 2/12/2004 13:06'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	applyToWholeText _ true.
	myTarget
		ifNil: [""
			myTarget _ TextMorph new openInWorld.
			myTarget contents: ''].

	activeTextMorph _ myTarget copy.
	activeTextMorph extent: 300 @ 100;	 
			 releaseCachedState.
	thingsToRevert
		at: #wrapFlag: put: myTarget isWrapped;
		 at: #autoFit: put: myTarget isAutoFit;
		 at: #margins: put: myTarget margins;
		at: #extent: put: myTarget extent.
	self rebuild! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:00'!
kernMinus

	self changeKernBy: -1! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:01'!
kernPlus

	self changeKernBy: 1! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/15/2001 12:01'!
offerFontMenu
	"Present a menu of available fonts, and if one is chosen, apply it to the current selection.  
	Use only names of Fonts of this paragraph  "

	| aList reply |

	aList _ self activeTextMorph textStyle fontNamesWithPointSizes.
	reply _ (SelectionMenu labelList: aList selections: aList) startUp.
	reply ifNil: [^self].
	self applyToWholeText ifTrue: [self activeEditor selectAll].
	self activeEditor replaceSelectionWith:
		(Text string: self activeEditor selection asString 
			attribute: (TextFontChange fontNumber: (aList indexOf: reply))).
	self activeTextMorph updateFromParagraph.! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'dgd 8/31/2003 21:54'!
paneForApplyToWholeText

	^self inARow: {
		self
			directToggleButtonFor: self 
			getter: #applyToWholeText
			setter: #toggleApplyToWholeText
			help: 'Whether to apply style changes to entire text or just selection' translated.
		self lockedString: ' Apply changes to entire text ' translated.
	}
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'dgd 8/31/2003 21:54'!
paneForAutoFitToggle

	^self inARow: {
		self
			directToggleButtonFor: self 
			getter: #targetHasAutoFit
			setter: #toggleTargetAutoFit
			help: 'Turn auto-fit on or off' translated.
		self lockedString: ' Auto-Fit' translated.
	}
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'dgd 8/31/2003 21:54'!
paneForMargins

	^(self inARow: {
		self
			buildFakeSlider: 'Margins' translated 
			selector: #adjustTargetMargin:
			help: 'Drag in here to change the margins of the text' translated
	}) hResizing: #shrinkWrap

! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'dgd 8/31/2003 21:54'!
paneForTextColorPicker

	^self 
		inAColumn: {
			self 
				colorPickerFor: self
				getter: #targetTextColor
				setter: #changeTargetColorTo:.
			self lockedString: 'Text Color' translated.
		} 
		named: #pickerForTextColor.

! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'dgd 8/31/2003 21:54'!
paneForWrappingToggle

	^self inARow: {
		self
			directToggleButtonFor: self 
			getter: #targetHasWrapping
			setter: #toggleTargetWrapping
			help: 'Turn line wrapping on or off' translated.
		self lockedString: ' Wrapping' translated.
	}
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'dgd 8/31/2003 21:56'!
rebuild

	| buttonColor c |

	self removeAllMorphs.
	self addAColumn: {
		self lockedString: ('Text Properties for {1}' translated format:{myTarget name}).
	}.
	self addAColumn: {
		self paneForApplyToWholeText.
	}.


	c _ self addAColumn: {
		self activeTextMorph.
	}.
	c 
		wrapCentering: #topLeft;
		color: Color white;
		borderWidth: 2;
		borderColor: color darker.
	self addAColumn: {
		self paneForTextColorPicker.
	}.

	self addARow: {
		self paneForAutoFitToggle.
	}.
	self addARow: {
		self paneForWrappingToggle.
	}.
	self addARow: {
		self paneForMargins.
	}.

	buttonColor _ color lighter.
	self addARow: {
		self inAColumn: {
			self addARow: {
				self 
					buttonNamed: 'Size' translated action: #offerFontMenu color: buttonColor
					help: 'font changing' translated.
				self 
					buttonNamed: 'Style' translated action: #changeStyle color: buttonColor
					help: 'font changing' translated.
				self 
					buttonNamed: 'N' translated action: #changeToNormalText color: buttonColor
					help: 'normal text' translated.
				self 
					buttonNamed: 'B' translated action: #toggleBold color: buttonColor
					help: 'bold text' translated.
				self 
					buttonNamed: 'I' translated action: #toggleItalic color: buttonColor
					help: 'italic text' translated.
				self 
					buttonNamed: 'n' translated action: #toggleNarrow color: buttonColor
					help: 'narrow text' translated.
				self 
					buttonNamed: 'U' translated action: #toggleUnderlined color: buttonColor
					help: 'underlined text' translated.
				self 
					buttonNamed: 'S' translated action: #toggleStruckOut color: buttonColor
					help: 'struck out text' translated.
				self 
					buttonNamed: 'Kern-' translated action: #kernMinus color: buttonColor
					help: 'decrease kern' translated.
				self 
					buttonNamed: 'Kern+' translated action: #kernPlus color: buttonColor
					help: 'increase kern' translated.
			}.
		}.
	}.
	self addARow: {
		self inAColumn: {
			self addARow: {
				self 
					buttonNamed: 'Accept' translated action: #doAccept color: buttonColor
					help: 'keep changes made and close panel' translated.
				self 
					buttonNamed: 'Cancel' translated action: #doCancel color: buttonColor
					help: 'cancel changes made and close panel' translated.
			}.
		}.
	}.
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/21/2001 11:14'!
targetHasAutoFit

	^myTarget isAutoFit
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/21/2001 11:19'!
targetHasWrapping

	^myTarget isWrapped
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 08:56'!
targetTextColor

	^Color black! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/15/2001 11:50'!
toggleApplyToWholeText

	applyToWholeText _ applyToWholeText not.
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:16'!
toggleBold

	self toggleSelectionAttribute: TextEmphasis bold! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:16'!
toggleItalic

	self toggleSelectionAttribute: TextEmphasis italic! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:16'!
toggleNarrow

	self toggleSelectionAttribute: TextEmphasis narrow! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:15'!
toggleSelectionAttribute: newAttribute

	| selText oldAttributes |

	self applyToWholeText ifTrue: [self activeEditor selectAll].
	selText _ self activeEditor selection asText.
	oldAttributes _ selText attributesAt: 1 forStyle: self activeTextMorph textStyle.
	oldAttributes do: [:att |
		(att dominates: newAttribute) ifTrue: [newAttribute turnOff]
	].
	self activeEditor replaceSelectionWith: (selText addAttribute: newAttribute).
	self activeTextMorph updateFromParagraph.! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:17'!
toggleStruckOut

	self toggleSelectionAttribute: TextEmphasis struckOut! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/21/2001 11:14'!
toggleTargetAutoFit

	^myTarget autoFitOnOff
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 8/21/2001 11:16'!
toggleTargetWrapping

	^myTarget wrapOnOff
! !

!TextPropertiesMorph methodsFor: 'as yet unclassified' stamp: 'RAA 3/16/2001 09:17'!
toggleUnderlined

	self toggleSelectionAttribute: TextEmphasis underlined! !

!HaloMorph methodsFor: '*eToys-handles' stamp: 'sw 3/30/2005 04:26'!
doDupOrMakeSibling: evt with: dupHandle
	"Ask hand to duplicate my target, if shift key *not* pressed, or make a sibling if shift key *is* pressed"

	^ (evt shiftPressed and: [target couldMakeSibling])
		ifTrue:
			[dupHandle color: Color green muchDarker.
			self doMakeSibling: evt with: dupHandle]
		ifFalse:
			[self doDup: evt with: dupHandle]! !

!HaloMorph methodsFor: '*eToys-handles' stamp: 'sw 3/30/2005 04:28'!
doMakeSiblingOrDup: evt with: dupHandle
	"Ask hand to duplicate my target, if shift key *is* pressed, or make a sibling if shift key *not* pressed"

	^ (evt shiftPressed or: [target couldMakeSibling not])
		ifFalse:
			[self doMakeSibling: evt with: dupHandle]
		ifTrue:
			[dupHandle color: Color green.
			self doDup: evt with: dupHandle]! !

!TheWorldMenu methodsFor: '*eToys-scripting' stamp: 'ar 3/17/2001 20:16'!
adaptedToWorld: aWorld
	"Can use me but need to adapt myself"
	self adaptToWorld: aWorld.! !

!TheWorldMenu methodsFor: '*eToys-popups' stamp: 'RAA 5/26/2000 10:26'!
playfieldDo
	"Build the playfield menu for the world."

	self doPopUp: myWorld playfieldOptionsMenu! !

!TheWorldMenu methodsFor: '*eToys-construction' stamp: 'RAA 5/24/2000 22:34'!
playfieldMenu

	^ myWorld playfieldOptionsMenu! !

!TheWorldMenu methodsFor: '*eToys-popups' stamp: 'RAA 5/26/2000 10:27'!
scriptingDo

	self doPopUp: self scriptingMenu! !

!TheWorldMenu methodsFor: '*eToys-action' stamp: 'dgd 9/1/2003 15:05'!
uniTilesClassicString
	^ ((myProject
			parameterAt: #uniTilesClassic
			ifAbsent: [false])
		ifTrue: ['<yes>']
		ifFalse: ['<no>']), 'classic tiles' translated! !

!VariableNode methodsFor: '*eToys-tiles' stamp: 'ab 7/13/2004 13:54'!
asMorphicSyntaxIn: parent

	^ parent addToken: self name
			type: #variable 
			on: self clone	"don't hand out the prototype!! See VariableNode>>initialize"
! !

!VariableNode methodsFor: '*eToys-tiles' stamp: 'RAA 8/24/1999 16:34'!
currentValueIn: aContext

	aContext ifNil: [^nil].
	^((self variableGetterBlockIn: aContext) ifNil: [^nil]) value printString
	

! !

!VariableNode methodsFor: '*eToys-tiles' stamp: 'ab 7/13/2004 13:53'!
explanation

	self isSelfPseudoVariable ifTrue: [^'the pseudo variable <self> (refers to the receiver)'].
	^(#('instance' 'temporary' 'LIT3' 'global') 
			at: self type 
			ifAbsent: ['UNK',self type printString]),' variable <',self name,'>'
		

	"LdInstType _ 1.
	LdTempType _ 2.
	LdLitType _ 3.
	LdLitIndType _ 4.
"

! !

!VariableNode methodsFor: '*eToys-tiles' stamp: 'ab 7/13/2004 13:54'!
variableGetterBlockIn: aContext

	| temps index ivars |

	(self type = 4 and: [self key isVariableBinding]) ifTrue: [
		^[self key value]
	].
	aContext ifNil: [^nil].
	self isSelfPseudoVariable ifTrue: [^[aContext receiver]].
	self type = 1 ifTrue: [
		ivars _ aContext receiver class allInstVarNames.
		index _ ivars indexOf: self name ifAbsent: [^nil].
		^[aContext receiver instVarAt: index]
	].
	self type = 2 ifTrue: [
		temps _ aContext tempNames.
		index _ temps indexOf: self name ifAbsent: [^nil].
		^[aContext tempAt: index]
	].
	^nil
! !

!ClassDescription methodsFor: '*eToys-accessing method dictionary' stamp: 'sw 3/20/2001 13:26'!
namedTileScriptSelectors
	"Answer a list of all the selectors of named tile scripts.  Initially, only Player reimplements, but if we switch to a scheme in which every class can have uniclass subclasses, this would kick in elsewhere"

	^ OrderedCollection new! !

!ReturnNode methodsFor: '*eToys-tiles' stamp: 'RAA 2/26/2001 06:44'!
asMorphicSyntaxIn: parent

	^parent returnNode: self expression: expr
! !

!ReturnNode methodsFor: '*eToys-tiles' stamp: 'RAA 8/15/1999 19:31'!
explanation

	^'Exit this method returning the value of ',expr explanation
! !

UpdatingThreePhaseButtonMorph subclass: #EtoyUpdatingThreePhaseButtonMorph
	instanceVariableNames: ''
	classVariableNames: 'CheckedForm MouseDownForm UncheckedForm'
	poolDictionaries: ''
	category: 'EToys-Widgets'!

!EtoyUpdatingThreePhaseButtonMorph commentStamp: '<historical>' prior: 0!
A slight variation wherein the actionSelector and getSelector both take argument(s).!

!EtoyUpdatingThreePhaseButtonMorph classSide methodsFor: 'instance creation' stamp: 'RAA 8/8/2000 13:34'!
checkBox
	"Answer a button pre-initialized with checkbox images."

	"(Form extent: 12@12 depth: 32) morphEdit"
	CheckedForm ifNil: [
		self setForms
	].
	^self new
		onImage: CheckedForm;
		pressedImage: MouseDownForm;
		offImage: UncheckedForm;
		extent: CheckedForm extent;
		yourself
! !

!EtoyUpdatingThreePhaseButtonMorph classSide methodsFor: 'as yet unclassified' stamp: 'RAA 8/8/2000 13:34'!
setForms

CheckedForm _ (Form
	extent: 12@12
	depth: 32
	fromArray: #( 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 4278190081 4278190081 4278190081 0 0 0 0 0 0 0 0 4278190081 2003331177 4278190081 4278190081 0 0 0 0 0 0 0 4278190081 2003331177 0 4278190081 4278190081 0 0 0 0 0 0 4278190081 2003331177 0 0 4278190081 4278190081 0 4278190081 0 0 0 4278190081 2003331177 0 0 0 4278190081 4278190081 0 2003331177 4278190081 0 4278190081 2003331177 0 0 0 0 4278190081 4278190081 0 0 2003331177 4278190081 2003331177 0 0 0 0 0 4278190081 4278190081 0 0 0 2003331177 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081)
	offset: 0@0).
MouseDownForm _ UncheckedForm _ (Form
	extent: 12@12
	depth: 32
	fromArray: #( 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 0 0 0 0 0 0 0 0 0 0 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081 4278190081)
	offset: 0@0)! !

!EtoyUpdatingThreePhaseButtonMorph methodsFor: 'stepping and presenter' stamp: 'RAA 7/11/2000 12:18'!
step
	| newBoolean |

	state == #pressed ifTrue: [^ self].
	newBoolean _ target perform: getSelector withArguments: arguments.
	newBoolean == self isOn
		ifFalse:
			[self state: (newBoolean ifTrue: [#on] ifFalse: [#off])]
! !

Object subclass: #ActorState
	instanceVariableNames: 'owningPlayer penDown penSize penColor fractionalPosition instantiatedUserScriptsDictionary penArrowheads trailStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Support'!

!ActorState commentStamp: '<historical>' prior: 0!
Holds a record of data representing actor-like slots in the Morph, on behalf of an associated Player.  Presently also holds onto the scriptInstantion objects that represent active scripts in an instance, but this will probably change soon.!

!ActorState methodsFor: 'other' stamp: 'sw 4/22/1998 17:02'!
addPlayerMenuItemsTo: aMenu hand: aHandMorph
	self getPenDown
		ifTrue: [aMenu add: 'pen up' action: #liftPen]
		ifFalse: [aMenu add: 'pen down' action: #lowerPen].
	aMenu add: 'pen size' action: #choosePenSize.
	aMenu add: 'pen color' action: #choosePenColor:.! !

!ActorState methodsFor: 'pen' stamp: 'nk 6/12/2004 16:36'!
choosePenColor: evt
	owningPlayer costume changeColorTarget: owningPlayer costume selector: #penColor: originalColor: owningPlayer getPenColor hand: evt hand.! !

!ActorState methodsFor: 'pen' stamp: 'sw 4/22/1998 13:44'!
choosePenSize
	| menu sz |
	menu _ CustomMenu new.
	1 to: 10 do: [:w | menu add: w printString action: w].
	sz _ menu startUp.
	sz ifNotNil: [penSize _ sz]! !

!ActorState methodsFor: 'other' stamp: 'sw 4/13/1998 19:36'!
costume
	^ owningPlayer costume! !

!ActorState methodsFor: 'pen' stamp: 'sw 2/4/98 15:16'!
defaultPenColor
	^ Color blue! !

!ActorState methodsFor: 'pen' stamp: 'sw 2/4/98 15:03'!
defaultPenSize
	^ 1! !

!ActorState methodsFor: 'position' stamp: 'jm 4/24/1998 21:34'!
fractionalPosition
	"Return my player's costume's position including the fractional part. This allows the precise position to be retained to avoid cummulative rounding errors, while letting Morphic do all its calculations with integer pixel coordinates. See the implementation of forward:."

	^ fractionalPosition
! !

!ActorState methodsFor: 'position' stamp: 'jm 4/24/1998 21:31'!
fractionalPosition: aPoint

	fractionalPosition _ aPoint asFloatPoint.
! !

!ActorState methodsFor: 'pen' stamp: 'tk 10/4/2001 16:42'!
getPenArrowheads
	^ penArrowheads == true! !

!ActorState methodsFor: 'pen' stamp: 'sw 4/22/1998 13:35'!
getPenColor
	penColor ifNil: [penColor _ self defaultPenColor].
	^ penColor! !

!ActorState methodsFor: 'pen' stamp: 'sw 4/22/1998 13:40'!
getPenDown
	^ penDown == true! !

!ActorState methodsFor: 'pen' stamp: 'sw 4/22/1998 13:43'!
getPenSize
	penSize ifNil: [penSize _ self defaultPenSize].
	^ penSize! !

!ActorState methodsFor: 'initialization' stamp: 'sw 5/13/1998 16:37'!
initializeFor: aPlayer
	| aNewDictionary |
	owningPlayer _ aPlayer.
	instantiatedUserScriptsDictionary ifNil: [^ self].
	aNewDictionary _ IdentityDictionary new.
	instantiatedUserScriptsDictionary associationsDo: 
		[:assoc |
			aNewDictionary at: assoc key put: (assoc value shallowCopy player: aPlayer)].
	instantiatedUserScriptsDictionary _ aNewDictionary.! !

!ActorState methodsFor: 'script instantiations' stamp: 'sw 4/9/98 22:35'!
instantiatedUserScriptsDictionary
	instantiatedUserScriptsDictionary ifNil: [instantiatedUserScriptsDictionary _ IdentityDictionary new].
	^ instantiatedUserScriptsDictionary! !

!ActorState methodsFor: 'pen' stamp: 'sw 2/4/98 18:07'!
liftPen
	penDown _ false! !

!ActorState methodsFor: 'pen' stamp: 'sw 2/4/98 14:58'!
lowerPen
	penDown _ true! !

!ActorState methodsFor: 'pen' stamp: 'sw 2/4/98 18:03'!
penColor: aColor
	penColor _ aColor! !

!ActorState methodsFor: 'printing' stamp: 'sw 5/12/1998 23:35'!
printOn: aStream
	aStream nextPutAll: 'ActorState for ', owningPlayer externalName, ' '.
	penDown ifNotNil: [aStream cr; nextPutAll: 'penDown ', penDown printString].
	penColor ifNotNil: [aStream cr; nextPutAll: 'penColor ', penColor printString].
	penSize ifNotNil: [aStream cr; nextPutAll: 'penSize ', penSize printString].
	instantiatedUserScriptsDictionary ifNotNil:
		[aStream cr; nextPutAll:
			'+ ', instantiatedUserScriptsDictionary size printString, ' user scripts'].
! !

!ActorState methodsFor: 'filter streaming' stamp: 'MPW 1/1/1901 21:53'!
printOnStream: aStream
	aStream print: 'ActorState for '; print:owningPlayer externalName; print:' '.
	penDown ifNotNil: [aStream cr; print: 'penDown '; write:penDown].
	penColor ifNotNil: [aStream cr; print: 'penColor '; write:penColor].
	penSize ifNotNil: [aStream cr; print: 'penSize '; write:penSize].
	instantiatedUserScriptsDictionary ifNotNil:
		[aStream cr; print:
			'+ '; write: instantiatedUserScriptsDictionary size; print:' user scripts'].
! !

!ActorState methodsFor: 'pen' stamp: 'tk 10/4/2001 16:43'!
setPenArrowheads: aBoolean
	penArrowheads _ aBoolean! !

!ActorState methodsFor: 'pen' stamp: 'sw 4/22/1998 13:51'!
setPenColor: aColor
	penColor _ aColor
! !

!ActorState methodsFor: 'pen' stamp: 'sw 4/22/1998 13:47'!
setPenDown: aBoolean
	penDown _ aBoolean! !

!ActorState methodsFor: 'pen' stamp: 'sw 4/22/1998 13:45'!
setPenSize: aNumber
	penSize _ aNumber! !

!ActorState methodsFor: 'pen' stamp: 'sw 4/16/2003 12:26'!
trailStyle
	"Answer the receiver's trailStyle.  For backward compatibility, if the old penArrowheads slot is in found to be set, use it as a guide for initialization"

	^ trailStyle ifNil: [trailStyle _ penArrowheads == true ifTrue: [#arrows] ifFalse: [#lines]]! !

!ActorState methodsFor: 'pen' stamp: 'sw 3/11/2003 11:28'!
trailStyle: aSymbol
	"Set the trail style to the given symbol"

	trailStyle _ aSymbol! !

Object subclass: #ButtonProperties
	instanceVariableNames: 'target actionSelector arguments actWhen wantsRolloverIndicator mouseDownTime nextTimeToFire visibleMorph delayBetweenFirings mouseOverHaloWidth mouseOverHaloColor mouseDownHaloWidth mouseDownHaloColor stateCostumes currentLook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Buttons'!

!ButtonProperties commentStamp: '<historical>' prior: 0!
ButtonProperties test1

ButtonProperties test2

ButtonProperties test3

!

!ButtonProperties classSide methodsFor: 'printing' stamp: 'sw 2/16/98 01:31'!
defaultNameStemForInstances
	^ 'button'! !

!ButtonProperties classSide methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 17:29'!
ellipticalButtonWithText: aStringOrText

	| m prop |

	m _ EllipseMorph new.
	prop _ m ensuredButtonProperties.
	prop
		target: #(1 2 3);
		actionSelector: #inspect;
		actWhen: #mouseUp;
		addTextToButton: aStringOrText;
		wantsRolloverIndicator: true.
	^m! !

!ButtonProperties classSide methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 08:31'!
test1

	| m prop |
	m _ EllipseMorph new.
	prop _ m ensuredButtonProperties.
	prop
		target: #(1 2 3);
		actionSelector: #inspect;
		actWhen: #mouseUp.
	m openInWorld.! !

!ButtonProperties classSide methodsFor: 'as yet unclassified' stamp: 'RAA 3/8/2001 08:41'!
test2

	(self ellipticalButtonWithText: 'Hello world') openInWorld.! !

!ButtonProperties classSide methodsFor: 'as yet unclassified' stamp: 'nb 6/17/2003 12:25'!
test3

	| m |

	(m _ self ellipticalButtonWithText: 'Hello world') openInWorld.
	m ensuredButtonProperties
		target: Beeper;
		actionSelector: #beep;
		delayBetweenFirings: 1000.! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/9/2001 09:43'!
actWhen

	^ actWhen! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/9/2001 09:43'!
actWhen: condition

	(#(buttonDown mouseDown) includes: condition) ifTrue: [ actWhen _ #mouseDown ].
	(#(buttonUp mouseUp) includes: condition) ifTrue: [ actWhen _ #mouseUp ].
	(#(whilePressed mouseStillDown) includes: condition) ifTrue: [ actWhen _ #mouseStillDown ].
	self setEventHandlers: true.! !

!ButtonProperties methodsFor: 'accessing' stamp: ''!
actionSelector

	^ actionSelector
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 08:46'!
actionSelector: aSymbolOrString

	aSymbolOrString isEmptyOrNil ifTrue: [^actionSelector _ nil].
	aSymbolOrString = 'nil' ifTrue: [^actionSelector _ nil].
	actionSelector _ aSymbolOrString asSymbol.
! !

!ButtonProperties methodsFor: 'initialization' stamp: 'ar 3/17/2001 20:12'!
adaptToWorld: aWorld
	super adaptToWorld: aWorld.
	target _ target adaptedToWorld: aWorld.! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/16/2001 08:28'!
addMouseOverHalo

	self wantsRolloverIndicator ifTrue: [
		visibleMorph 
			addMouseActionIndicatorsWidth: mouseOverHaloWidth 
			color: mouseOverHaloColor.
	].
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/15/2001 09:35'!
addTextToButton: aStringOrText

	| tm existing |

	existing _ self currentTextMorphsInButton.
	existing do: [ :x | x delete].
	aStringOrText ifNil: [^self].
	tm _ TextMorph new contents: aStringOrText.
	tm 
		fullBounds;
		lock;
		align: tm center with: visibleMorph center;
		setProperty: #textAddedByButtonProperties toValue: true;
		setToAdhereToEdge: #center.
	"maybe the user would like personal control here"
	"visibleMorph extent: (tm extent * 1.5) rounded."
	visibleMorph addMorphFront: tm.
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 07:35'!
adjustPositionsAfterSizeChange

	"re-center label, etc??"! !

!ButtonProperties methodsFor: 'accessing' stamp: ''!
arguments

	^ arguments
! !

!ButtonProperties methodsFor: 'accessing' stamp: ''!
arguments: aCollection

	arguments _ aCollection asArray copy.
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/9/2001 11:40'!
bringUpToDate

	self establishEtoyLabelWording
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/15/2001 09:18'!
currentLook

	^currentLook ifNil: [currentLook _ #normal]! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/14/2001 18:18'!
currentTextInButton

	| existing |

	existing _ self currentTextMorphsInButton.
	existing isEmpty ifTrue: [^nil].
	^existing first
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/14/2001 18:17'!
currentTextMorphsInButton

	^visibleMorph submorphsSatisfying: [ :x | 
		x hasProperty: #textAddedByButtonProperties
	]
! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/8/2001 09:29'!
delayBetweenFirings

	^delayBetweenFirings! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/8/2001 09:55'!
delayBetweenFirings: millisecondsOrNil

	delayBetweenFirings _ millisecondsOrNil! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/15/2001 09:21'!
displayCostume: aSymbol

	self currentLook == aSymbol ifTrue: [^true].
	self stateCostumes at: aSymbol ifPresent: [ :aForm |
		currentLook _ aSymbol.
		visibleMorph wearCostume: aForm.
		^true
	].
	^false
! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/9/2001 08:58'!
doButtonAction
	
	self doButtonAction: nil! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/9/2001 17:08'!
doButtonAction: evt

	| arity |

	target ifNil: [^self].
	actionSelector ifNil: [^self].
	arguments ifNil: [arguments _ #()].
	Cursor normal showWhile: [
		arity _ actionSelector numArgs.
		arity = arguments size ifTrue: [
			target perform: actionSelector withArguments: arguments
		].
		arity = (arguments size + 1) ifTrue: [
			target perform: actionSelector withArguments: {evt},arguments
		].
		arity = (arguments size + 2) ifTrue: [
			target perform: actionSelector withArguments: {evt. visibleMorph},arguments
		].
	]! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/14/2001 19:01'!
editButtonsScript: evt
	"The user has touched my Scriptor halo-handle.  Bring up a Scriptor on the script of the button."

	| cardsPasteUp cardsPlayer anEditor scriptSelector |

	cardsPasteUp _ self pasteUpMorph.
	(cardsPlayer _ cardsPasteUp assuredPlayer) assureUniClass.
	scriptSelector _ self figureOutScriptSelector.
	scriptSelector ifNil: [
		scriptSelector _ cardsPasteUp scriptSelectorToTriggerFor: self.
		anEditor _ cardsPlayer newTextualScriptorFor: scriptSelector.
		evt hand attachMorph: anEditor.
		^self
	].

	(cardsPlayer class selectors includes: scriptSelector) ifTrue: [
		anEditor _ cardsPlayer scriptEditorFor: scriptSelector.
		evt hand attachMorph: anEditor.
		^self
	].
	"Method somehow got removed; I guess we start aftresh"
	scriptSelector _ nil.
	^ self editButtonsScript! !

!ButtonProperties methodsFor: 'accessing' stamp: 'yo 11/30/2005 14:20'!
establishEtoyLabelWording
	"Set the label wording, unless it has already been manually edited"

	| itsName |

	self isTileScriptingElement ifFalse: [^self].
	itsName _ target externalName.
	self addTextToButton: itsName, ' ', arguments first.
	visibleMorph setBalloonText: 
		('click to run the script "{1}" in player named "{2}"' translated format: {arguments first. itsName}).
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/14/2001 19:01'!
figureOutScriptSelector

	self halt! !

!ButtonProperties methodsFor: 'initialization' stamp: 'RAA 3/9/2001 09:47'!
initialize

	wantsRolloverIndicator _ false.
	delayBetweenFirings _ nil.
	mouseOverHaloWidth _ 10.
	mouseOverHaloColor _ Color blue alpha: 0.3.
	mouseDownHaloWidth _ 15.
	mouseDownHaloColor _ Color blue alpha: 0.7.
	arguments _ #().! !

!ButtonProperties methodsFor: 'accessing' stamp: 'nk 8/29/2004 17:16'!
isTileScriptingElement

	actionSelector == #runScript: ifFalse: [^false].
	arguments isEmptyOrNil ifTrue: [^false].
	^target isPlayerLike! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/14/2001 18:19'!
lockAnyText

	self currentTextMorphsInButton do: [ :x | x lock: true].! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/14/2001 18:40'!
mouseDown: evt

	self displayCostume: #mouseDown.
	mouseDownTime _ Time millisecondClockValue.
	nextTimeToFire _ nil.
	delayBetweenFirings ifNotNil: [
		nextTimeToFire _ mouseDownTime + delayBetweenFirings.
	].
	self wantsRolloverIndicator ifTrue: [
		visibleMorph 
			addMouseActionIndicatorsWidth: mouseDownHaloWidth 
			color: mouseDownHaloColor.
	].
	actWhen == #mouseDown ifFalse: [^self].
	(visibleMorph containsPoint: evt cursorPoint) ifFalse: [^self].
	self doButtonAction: evt.

"=====

	aMorph .

	now _ Time millisecondClockValue.
	oldColor _ color. 
	actWhen == #buttonDown
		ifTrue: [self doButtonAction]
		ifFalse: [	self updateVisualState: evt; refreshWorld].
	dt _ Time millisecondClockValue - now max: 0.
	dt < 200 ifTrue: [(Delay forMilliseconds: 200-dt) wait].
	self mouseStillDown: evt.
====="! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 16:15'!
mouseDownHaloColor

	^mouseDownHaloColor! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 16:15'!
mouseDownHaloColor: x

	mouseDownHaloColor _ x! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 16:31'!
mouseDownHaloWidth

	^mouseDownHaloWidth! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 16:31'!
mouseDownHaloWidth: x

	mouseDownHaloWidth _ x! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/14/2001 18:43'!
mouseDownLook: aFormOrMorph

	self setLook: #mouseDown to: aFormOrMorph
! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/16/2001 08:29'!
mouseEnter: evt

	self displayCostume: #mouseEnter.
	self addMouseOverHalo.
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/14/2001 18:43'!
mouseEnterLook: aFormOrMorph

	self setLook: #mouseEnter to: aFormOrMorph
! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/14/2001 18:39'!
mouseLeave: evt

	self displayCostume: #normal.
	visibleMorph deleteAnyMouseActionIndicators.
! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/8/2001 07:57'!
mouseMove: evt

	actWhen == #mouseDown ifTrue: [^ self].
	self updateVisualState: evt.! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 16:16'!
mouseOverHaloColor

	^mouseOverHaloColor! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 16:16'!
mouseOverHaloColor: x

	mouseOverHaloColor _ x! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 16:30'!
mouseOverHaloWidth

	^mouseOverHaloWidth! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 16:31'!
mouseOverHaloWidth: x

	mouseOverHaloWidth _ x! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/9/2001 08:57'!
mouseStillDown: evt

	(visibleMorph containsPoint: evt cursorPoint) ifFalse: [^self].
	nextTimeToFire ifNil: [^self].
	nextTimeToFire <= Time millisecondClockValue ifTrue: [
		self doButtonAction: evt.
		nextTimeToFire _ Time millisecondClockValue + self delayBetweenFirings.
		^self
	].
! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/16/2001 08:29'!
mouseUp: evt

	(self displayCostume: #mouseEnter) ifFalse: [self displayCostume: #normal].
	self addMouseOverHalo.
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'gm 2/22/2003 14:53'!
privateSetLook: aSymbol to: aFormOrMorph 
	| f |
	f := (aFormOrMorph isForm) 
				ifTrue: [aFormOrMorph]
				ifFalse: [aFormOrMorph imageForm].
	self stateCostumes at: aSymbol put: f! !

!ButtonProperties methodsFor: 'events' stamp: 'RAA 3/9/2001 12:27'!
replaceVisibleMorph: aNewMorph

	| old oldOwner oldText |

	old _ visibleMorph.
	oldText _ self currentTextInButton.
	self visibleMorph: nil.
	old buttonProperties: nil.
	aNewMorph buttonProperties: self.
	self visibleMorph: aNewMorph.
	self addTextToButton: oldText.
	oldOwner _ old owner ifNil: [^self].
	oldOwner replaceSubmorph: old by: aNewMorph.! !

!ButtonProperties methodsFor: 'menu' stamp: 'yo 3/16/2005 20:58'!
setActWhen

	| selections |
	selections _ #(mouseDown mouseUp mouseStillDown).
	actWhen _ (SelectionMenu labels: (selections collect: [:t | t translated]) selections: selections)
		startUpWithCaption: 'Choose one of the following conditions' translated
! !

!ButtonProperties methodsFor: 'menu' stamp: 'yo 3/16/2005 20:53'!
setActionSelector

	| newSel |
	newSel _ FillInTheBlank
		request:
'Please type the selector to be sent to
the target when this button is pressed' translated
		initialAnswer: actionSelector.
	newSel isEmpty ifFalse: [self actionSelector: newSel].
! !

!ButtonProperties methodsFor: 'menu' stamp: 'yo 3/14/2005 13:07'!
setArguments

	| s newArgs newArgsArray |
	s _ WriteStream on: ''.
	arguments do: [:arg | arg printOn: s. s nextPutAll: '. '].
	newArgs _ FillInTheBlank
		request:
'Please type the arguments to be sent to the target
when this button is pressed separated by periods' translated
		initialAnswer: s contents.
	newArgs isEmpty ifFalse: [
		newArgsArray _ Compiler evaluate: '{', newArgs, '}' for: self logged: false.
		self arguments: newArgsArray].
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/10/2001 13:57'!
setEventHandlers: enabled

	enabled ifTrue: [
		visibleMorph on: #mouseDown send: #mouseDown: to: self.
		visibleMorph on: #mouseStillDown send: #mouseStillDown: to: self.
		visibleMorph on: #mouseUp send: #mouseUp: to: self.
		visibleMorph on: #mouseEnter send: #mouseEnter: to: self.
		visibleMorph on: #mouseLeave send: #mouseLeave: to: self.
	] ifFalse: [
		#(mouseDown mouseStillDown mouseUp mouseEnter mouseLeave) do: [ :sel |
			visibleMorph on: sel send: nil to: nil
		].
	].
! !

!ButtonProperties methodsFor: 'menu' stamp: ''!
setLabel

	| newLabel |
	newLabel _ FillInTheBlank
		request:
'Please a new label for this button'
		initialAnswer: self label.
	newLabel isEmpty ifFalse: [self label: newLabel].
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/15/2001 09:14'!
setLook: aSymbol to: aFormOrMorph

	(self stateCostumes includesKey: #normal) ifFalse: [
		self privateSetLook: #normal to: visibleMorph.
	].
	self privateSetLook: aSymbol to: aFormOrMorph.
! !

!ButtonProperties methodsFor: 'menu' stamp: 'di 12/20/1998 16:55'!
setPageSound: event

	^ target menuPageSoundFor: self event: event! !

!ButtonProperties methodsFor: 'menu' stamp: 'di 12/20/1998 16:55'!
setPageVisual: event

	^ target menuPageVisualFor: self event: event! !

!ButtonProperties methodsFor: 'menu' stamp: 'wiz 12/4/2006 00:20'!
setTarget: evt 
	| rootMorphs |
	rootMorphs := self world rootMorphsAt: evt targetPoint.
	target := rootMorphs size > 1 
		ifTrue: [rootMorphs second]
		ifFalse: [nil]! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/14/2001 18:30'!
stateCostumes

	^stateCostumes ifNil: [stateCostumes _ Dictionary new]! !

!ButtonProperties methodsFor: 'accessing' stamp: ''!
target

	^ target
! !

!ButtonProperties methodsFor: 'accessing' stamp: ''!
target: anObject

	target _ anObject
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/14/2001 18:19'!
unlockAnyText

	self currentTextMorphsInButton do: [ :x | x lock: false].! !

!ButtonProperties methodsFor: 'copying' stamp: 'jm 7/28/97 11:52'!
updateReferencesUsing: aDictionary
	"If the arguments array points at a morph we are copying, then point at the new copy.  And also copies the array, which is important!!"

	super updateReferencesUsing: aDictionary.
	arguments _ arguments collect:
		[:old | aDictionary at: old ifAbsent: [old]].
! !

!ButtonProperties methodsFor: 'visual properties' stamp: 'RAA 3/8/2001 14:24'!
updateVisualState: evt
	
"	oldColor ifNil: [^self].

	self color: 
		((self containsPoint: evt cursorPoint)
			ifTrue: [oldColor mixed: 1/2 with: Color white]
			ifFalse: [oldColor])"! !

!ButtonProperties methodsFor: 'copying' stamp: 'tk 1/6/1999 17:55'!
veryDeepFixupWith: deepCopier
	"If target and arguments fields were weakly copied, fix them here.  If they were in the tree being copied, fix them up, otherwise point to the originals!!!!"

super veryDeepFixupWith: deepCopier.
target _ deepCopier references at: target ifAbsent: [target].
arguments _ arguments collect: [:each |
	deepCopier references at: each ifAbsent: [each]].
! !

!ButtonProperties methodsFor: 'copying' stamp: 'RAA 3/16/2001 08:21'!
veryDeepInner: deepCopier
	"Copy all of my instance variables.  Some need to be not copied at all, but shared.  	Warning!!!!  Every instance variable defined in this class must be handled.  We must also implement veryDeepFixupWith:.  See DeepCopier class comment."

super veryDeepInner: deepCopier.
"target _ target.		Weakly copied"
"actionSelector _ actionSelector.		a Symbol"
"arguments _ arguments.		All weakly copied"
actWhen _ actWhen veryDeepCopyWith: deepCopier.
"oldColor _ oldColor veryDeepCopyWith: deepCopier."
visibleMorph _ visibleMorph.	"I guess this will have been copied already if needed"
delayBetweenFirings _ delayBetweenFirings.
mouseDownHaloColor _ mouseDownHaloColor.
stateCostumes _ stateCostumes veryDeepCopyWith: deepCopier.
currentLook _ currentLook.! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 15:43'!
visibleMorph: x

	visibleMorph ifNotNil: [self setEventHandlers: false].
	visibleMorph _ x.
	visibleMorph ifNotNil: [self setEventHandlers: true].
! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/8/2001 09:09'!
wantsRolloverIndicator

	^wantsRolloverIndicator ifNil: [false]! !

!ButtonProperties methodsFor: 'accessing' stamp: 'RAA 3/10/2001 13:59'!
wantsRolloverIndicator: aBoolean

	wantsRolloverIndicator _ aBoolean.
	wantsRolloverIndicator ifTrue: [
		self setEventHandlers: true.
	].! !

Object subclass: #KedamaAttributeDictionary
	instanceVariableNames: 'dictionaries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaAttributeDictionary methodsFor: 'all' stamp: 'yo 8/28/2004 00:45'!
getAttribute: aSymbol of: anObject

	| attributes |
	attributes _ dictionaries at: anObject ifAbsent: [^ nil].
	^ attributes at: aSymbol ifAbsent: [nil].
! !

!KedamaAttributeDictionary methodsFor: 'all' stamp: 'yo 8/28/2004 00:46'!
initialize

	dictionaries _ IdentityDictionary new.
! !

!KedamaAttributeDictionary methodsFor: 'all' stamp: 'yo 8/28/2004 00:46'!
setAttribute: aSymbol of: anObject to: value

	| attributes |
	attributes _ dictionaries at: anObject ifAbsentPut: [IdentityDictionary new].
	attributes at: aSymbol put: value.

! !

Object subclass: #KedamaVectorizer
	instanceVariableNames: 'attributes root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-StarSqueak'!

!KedamaVectorizer methodsFor: 'private' stamp: 'yo 9/24/2004 11:47'!
checkFirstPairFor: stmt in: obj

	^ self checkRequireVectorIn: stmt for: obj.
"

	| receiver selector ret |
	receiver _ Compiler evaluate: (self getReceiverFromStatement: stmt) name for: obj logged: false.
	selector _ (self getSelectorFromStatement: stmt) key.
	ret _ self isVectorizationRequiredWithPlayer: receiver andSelector: selector.
	attributes setAttribute: #firstTurtle of: stmt to: receiver.
	attributes setAttribute: #requireVector of: stmt to: ret.
	^ ret.
"! !

!KedamaVectorizer methodsFor: 'private' stamp: 'yo 2/14/2005 15:33'!
checkRequireVectorIn: aMessageNode for: obj

	| players playersSet sel playerNodes |
	"self halt."
	playerNodes _ self getAllPlayersIn: aMessageNode for: obj.
	players _ playerNodes collect: [:e | Compiler evaluate: e name for: obj logged: false.].
	playersSet _ players asSet.
	(playersSet select: [:e | e isPrototypeTurtlePlayer]) size = 0 ifTrue: [
		attributes setAttribute: #firstTurtle of: aMessageNode to: (Compiler evaluate: playerNodes first name for: obj logged: false).
		attributes setAttribute: #requireVector of: aMessageNode to: false.
		^ self.
	].
	(playersSet select: [:e | e isPrototypeTurtlePlayer]) size > 0 ifTrue: [	
		playerNodes with: players do: [:n :p |
			p isPrototypeTurtlePlayer ifTrue: [
				sel _ self getSelectorRootFor: p fromMessageNode: aMessageNode for: obj ignoreSelectors: #(beNotZero: setTurtleCount: getTurtleCount setGrouped: getGrouped).
				sel ifNotNil: [
					(self isVectorizationRequiredWithPlayer: p andSelector: sel) ifTrue: [
						attributes setAttribute: #requireVector of: aMessageNode to: true.
						attributes setAttribute: #firstTurtle of: aMessageNode to: p.
						attributes setAttribute: #firstNode of: aMessageNode to: n.
						^ self.
					].
				].
			].
		].
	].

	attributes setAttribute: #firstTurtle of: aMessageNode to: players first.
	attributes setAttribute: #requireVector of: aMessageNode to: false.
! !

!KedamaVectorizer methodsFor: 'player and selector look up' stamp: 'yo 9/24/2004 11:13'!
getAllPlayersIn: aMessageNode for: obj

	| aCollection |
	aCollection _ OrderedCollection new.
	self getPlayersMessage: aMessageNode for: obj into: aCollection.
	^ aCollection.
! !

!KedamaVectorizer methodsFor: 'player and selector look up' stamp: 'yo 10/8/2004 05:23'!
getAllPlayersInMethodNode: aMethodNode for: obj

	| aCollection |
	aCollection _ OrderedCollection new.
	aMethodNode block statements do: [:stmt |
		(stmt isMemberOf: MessageNode) ifTrue: [
			self getPlayersMessage: stmt for: obj into: aCollection.
		].
		(stmt isMemberOf: BlockNode) ifTrue: [
			self getPlayersBlock: stmt for: obj into: aCollection.
		].
	].
	^ aCollection.
! !

!KedamaVectorizer methodsFor: 'player and selector look up' stamp: 'yo 9/24/2004 11:08'!
getPlayersBlock: aBlockNode for: obj into: aCollection


	| statements |
	statements _ aBlockNode statements.
	statements do: [:stmt |
		(stmt isMemberOf: MessageNode) ifTrue: [
			self getPlayersMessage: stmt for: obj into: aCollection.
		].
		(stmt isMemberOf: BlockNode) ifTrue: [
			self getPlayersBlock: stmt for: obj into: aCollection..
		].
	].
! !

!KedamaVectorizer methodsFor: 'player and selector look up' stamp: 'yo 9/24/2004 12:02'!
getPlayersMessage: aMessageNode for: obj into: aCollection

	| thisPlayer |
	((Array with: aMessageNode receiver), aMessageNode arguments) do: [:stmt |
		(stmt isMemberOf: MessageNode) ifTrue: [
			self getPlayersMessage: stmt for: obj into: aCollection.
		].
		(stmt isMemberOf: BlockNode) ifTrue: [
			self getPlayersBlock: stmt for: obj into: aCollection..
		].
		(stmt isMemberOf: VariableNode) ifTrue: [
			thisPlayer _ Compiler evaluate: stmt name for: obj logged: false.
			(thisPlayer isKindOf: Player) ifTrue: [aCollection add: stmt].
		].
	].
! !

!KedamaVectorizer methodsFor: 'player and selector look up' stamp: 'yo 2/14/2005 15:22'!
getSelectorFor: receiver fromBlockNode: aBlockNode for: obj ifFoundDo: aBlock ignoreSelectors: ignoreSelectors

	root ifNotNil: [^ self].
	aBlockNode statements do: [:stmt |
		(stmt isMemberOf: MessageNode) ifTrue: [
			self getSelectorFor: receiver fromMessageNode: stmt for: obj ifFoundDo: aBlock ignoreSelectors: ignoreSelectors
		].
		(stmt isMemberOf: BlockNode) ifTrue: [
			self getSelectorFor: receiver fromBlockNode: stmt for: obj ifFoundDo: aBlock ignoreSelectors: ignoreSelectors
		].
	].
! !

!KedamaVectorizer methodsFor: 'player and selector look up' stamp: 'yo 2/14/2005 15:34'!
getSelectorFor: receiver fromMessageNode: aMessageNode for: obj ifFoundDo: aBlock ignoreSelectors: ignoreSelectors

	| thisPlayer key |
	root ifNotNil: [^ self].
	(Array with: aMessageNode receiver), aMessageNode arguments do: [:stmt |
		(stmt isMemberOf: VariableNode) ifTrue: [
			thisPlayer _ Compiler evaluate: stmt name for: obj logged: false.
			thisPlayer == receiver ifTrue: [
				key _  aMessageNode selector key.
				(ignoreSelectors includes: key) ifFalse: [aBlock value: key. ^ self]].
		].
		(stmt isMemberOf: MessageNode) ifTrue: [
			self getSelectorFor: receiver fromMessageNode: stmt for: obj ifFoundDo: aBlock ignoreSelectors: ignoreSelectors
		].
		(stmt isMemberOf: BlockNode) ifTrue: [
			self getSelectorFor: receiver fromBlockNode: stmt for: obj ifFoundDo: aBlock ignoreSelectors: ignoreSelectors
		].
	].
! !

!KedamaVectorizer methodsFor: 'player and selector look up' stamp: 'yo 2/14/2005 15:25'!
getSelectorRootFor: receiver fromMessageNode: aMessageNode for: obj ignoreSelectors: ignoreSelectors

	| val |
	root _ nil.
	self getSelectorFor: receiver fromMessageNode: aMessageNode for: obj ifFoundDo: [:sel | root _ sel] ignoreSelectors: ignoreSelectors.
	val _ root.
	root _ nil.
	^ val.
! !

!KedamaVectorizer methodsFor: 'entry point' stamp: 'yo 10/8/2004 05:33'!
includesTurtlePlayer: aMethodNode for: obj

	| players p |
	players _ self getAllPlayersInMethodNode: aMethodNode for: obj.
	players do: [:e |
		p _ Compiler evaluate: e name for: obj logged: false.
		(p isKindOf: KedamaExamplerPlayer) ifTrue: [^ true].
	].
	^ false.
! !

!KedamaVectorizer methodsFor: 'initialization' stamp: 'yo 8/28/2004 00:47'!
initialize

	attributes _ KedamaAttributeDictionary new initialize.
! !

!KedamaVectorizer methodsFor: 'insert block node' stamp: 'yo 9/18/2004 12:08'!
insertTurtleCommandAt: index in: statements variableNode: varNode from: encoder firstReceiverName: origName

	| messageNode blockNode newMessageNode selector |
	messageNode _ statements at: index.
	(attributes getAttribute: #requireVector of: messageNode) ifFalse: [^ self].
	blockNode _ BlockNode new arguments: (Array with: varNode) statements: (Array with: messageNode) returns: false from: encoder.
	selector _ (attributes getAttribute: #dieMessage of: messageNode) ifNotNil: [#doDieCommand:].
	selector ifNil: [
		selector _ (attributes getAttribute: #constant of: messageNode) ifTrue: [#doCommand:] ifFalse: [#doSequentialCommand:].
	].
		
	newMessageNode _ MessageNode new
				receiver: (encoder encodeVariable: origName)
				selector: selector
				arguments: (Array with: blockNode)
				precedence: 3
				from: encoder
				sourceRange: nil.

	statements at: index put: newMessageNode.
! !

!KedamaVectorizer methodsFor: 'private' stamp: 'yo 9/16/2004 10:20'!
isScalarizable: aPlayer andSelector: selector

	(aPlayer isKindOf: Player) ifFalse: [^ false].
	aPlayer isPrototypeTurtlePlayer ifFalse: [^ false].

	(#(getX getY getHeading getAngleTo: getDistanceTo: getUphillIn: getReplicated color) includes: selector) ifTrue: [^ true].

	^ ((aPlayer class organization listAtCategoryNamed: 'access') select: [:sel | sel beginsWith: 'get']) includes: selector.
! !

!KedamaVectorizer methodsFor: 'private' stamp: 'yo 9/29/2004 09:44'!
isVectorizationRequiredWithPlayer: aPlayer andSelector: selector

	(aPlayer isKindOf: Player) ifFalse: [^ false].
	aPlayer isPrototypeTurtlePlayer ifFalse: [^ false].

	^ (#(setTurtleCount: getTurtleCount setGrouped: getGrouped) includes: selector) not.
! !

!KedamaVectorizer methodsFor: 'replace receiver' stamp: 'yo 8/31/2004 18:34'!
replace: varName with: newNode inBlockNode: aBlockNode

	aBlockNode statements do: [:e |
		(e isMemberOf: MessageNode) ifTrue: [
			self replace: varName with: newNode inMessageNode: e.
		].
		(e isMemberOf: BlockNode) ifTrue: [
			self replace: varName with: newNode inBlockNode: e.
		].
	].
! !

!KedamaVectorizer methodsFor: 'replace receiver' stamp: 'yo 10/9/2004 15:56'!
replace: varName with: newNode inMessageNode: aMessageNode

	| receiver att |
	aMessageNode arguments withIndexDo: [:e :index |
		(e isMemberOf: MessageNode) ifTrue: [
			self replace: varName with: newNode inMessageNode: e.
		].
		(e isMemberOf: BlockNode) ifTrue: [
			self replace: varName with: newNode inBlockNode: e.
		].
		(e isMemberOf: VariableNode) ifTrue: [
			att _ attributes getAttribute: #isTurtle of: e.
			((att notNil & att) and: [e name = varName]) ifTrue: [aMessageNode arguments at: index put: newNode].
		].

	].

	receiver _ aMessageNode receiver.
	(receiver isMemberOf: MessageNode) ifTrue: [
		self replace: varName with: newNode inMessageNode: receiver.
	].

	(receiver isMemberOf: VariableNode) ifTrue: [
		att _ attributes getAttribute: #isTurtle of: receiver.
		((att notNil & att) and: [receiver name = varName]) ifTrue: [aMessageNode receiver: newNode].
	].
! !

!KedamaVectorizer methodsFor: 'replace receiver' stamp: 'yo 9/24/2004 09:30'!
replaceRoot: varName with: newNode inMessageNode: aMessageNode

	(attributes getAttribute: #requireVector of: aMessageNode) ifFalse: [^ self].

	(aMessageNode isMemberOf: MessageNode) ifTrue: [
		self replace: varName with: newNode inMessageNode: aMessageNode.
	].
	(aMessageNode isMemberOf: BlockNode) ifTrue: [
		self replace: varName with: newNode inBlockNode: aMessageNode.
	].
! !

!KedamaVectorizer methodsFor: 'vectorization attribute calculation' stamp: 'yo 9/17/2004 11:20'!
traverseBlock: aBlockNode in: obj firstPlayer: firstPlayer inCondition: inCondition

	| value statements |
	statements _ aBlockNode statements.
	statements do: [:stmt |
		(stmt isMemberOf: MessageNode)	ifTrue: [
			self traverseMessage: stmt in: obj firstPlayer: firstPlayer inCondition: inCondition.
		].
		(stmt isMemberOf: BlockNode) ifTrue: [
			self traverseBlock: stmt in: obj firstPlayer: firstPlayer inCondition: inCondition.
		].
		(stmt isMemberOf: VariableNode) ifTrue: [
			attributes setAttribute: #constant of: stmt to: false
		].
		(stmt isMemberOf: LiteralNode) ifTrue: [
			attributes setAttribute: #constant of: stmt to: true
		].
	].
	value _ statements collect: [:stmt | attributes getAttribute: #constant of: stmt].
	value _ value inject: true into: [:s :t | s & t].

	attributes setAttribute: #constant of: aBlockNode to: value.
! !

!KedamaVectorizer methodsFor: 'vectorization attribute calculation' stamp: 'yo 3/7/2005 08:36'!
traverseMessage: aMessageNode in: obj firstPlayer: firstPlayer inCondition: inCondition

	| receiver thisPlayer ret constant proto |

	aMessageNode arguments do: [:argument |
		(argument isMemberOf: MessageNode) ifTrue: [
			self traverseMessage: argument in: obj firstPlayer: firstPlayer inCondition: inCondition.
		].
		(argument isMemberOf: BlockNode) ifTrue: [
			self traverseBlock: argument in: obj firstPlayer: firstPlayer inCondition: inCondition.
		].
		(argument isMemberOf: LiteralNode) ifTrue: [
			attributes setAttribute: #constant of: argument to: true.
		].
		(argument isMemberOf: VariableNode) ifTrue: [
			thisPlayer _ Compiler evaluate: argument name for: obj logged: false.
			ret _ (thisPlayer isKindOf: Player) and: [thisPlayer costume renderedMorph isKindOf: KedamaPatchMorph].
			attributes setAttribute: #constant of: argument to: ret.
		].
	].

	receiver _ aMessageNode receiver.
	(receiver isMemberOf: MessageNode) ifTrue: [
		self traverseMessage: receiver in: obj firstPlayer: firstPlayer inCondition: inCondition.
	].
	(receiver isMemberOf: BlockNode) ifTrue: [
		self traverseBlock: receiver in: obj firstPlayer: firstPlayer inCondition: inCondition.
	].
	(receiver isMemberOf: LiteralNode) ifTrue: [
		attributes setAttribute: #constant of: receiver to: true.
	].
	(receiver isMemberOf: VariableNode) ifTrue: [
		thisPlayer _ Compiler evaluate: receiver name for: obj logged: false.
		ret _ thisPlayer == firstPlayer.
		attributes setAttribute: #constant of: receiver to: ret.
		proto _ (thisPlayer isKindOf: Player) and: [thisPlayer isPrototypeTurtlePlayer].
		attributes setAttribute: #isTurtle of: receiver to: proto.
		attributes setAttribute: #scalar of: aMessageNode selector to:
		(ret not and: [(proto and: [self isScalarizable: thisPlayer andSelector: aMessageNode selector key])]).
	].

	"special cases..."
	(#(atRandom die getReplicated bounceOn: bounceOn:color: bounceOnColor: ifTrue: ifFalse: ifTrue:ifFalse: itFalse:ifTrue:
	setPatchValueIn:to: getTurtleAt: getTurtleOf:) includes: aMessageNode selector key) ifTrue: [
		attributes setAttribute: #constant of: aMessageNode to: false.
		aMessageNode selector key = #die ifTrue: [
			attributes setAttribute: #dieMessage of: root to: true.
		].

	] ifFalse: [
		constant _ (aMessageNode arguments copyWith: receiver) inject: true into: [:s :t | s _ s and: [attributes getAttribute: #constant of: t]].
		attributes setAttribute: #constant of: aMessageNode to: constant.
	].

! !

!KedamaVectorizer methodsFor: 'vectorization attribute calculation' stamp: 'yo 9/24/2004 09:20'!
traverseMessageRoot: aMessageNode in: obj firstPlayer: firstPlayer

	| inCondition |
	inCondition _ #(ifTrue: ifFalse: ifTrue:ifFalse:) includes: aMessageNode selector key.

	root _ aMessageNode.

	self traverseMessage: aMessageNode in: obj firstPlayer: firstPlayer inCondition: inCondition.
! !

!KedamaVectorizer methodsFor: 'entry point' stamp: 'yo 2/14/2005 10:55'!
vectorize: aMethodNode object: obj

	aMethodNode block statements withIndexDo: [:stmt :index |
		(stmt isMemberOf: MessageNode) ifTrue: [
			self checkFirstPairFor: stmt in: obj.
			self traverseMessageRoot: stmt in: obj firstPlayer: (attributes getAttribute: #firstTurtle of: stmt).
		].

		(stmt isMemberOf: MessageNode) ifTrue: [
			self vectorizeAt: index inMethodNode: aMethodNode.
		].
	].

	^ aMethodNode.


! !

!KedamaVectorizer methodsFor: 'private' stamp: 'yo 9/24/2004 12:11'!
vectorizeAt: index inMethodNode: aMethodNode

	| v encoder origReceiverName origReceiver aMessageNode |
	aMessageNode _ aMethodNode block statements at: index.
	(attributes getAttribute: #requireVector of: aMessageNode) ifFalse: [^ self].

	encoder _ aMethodNode encoder.
	encoder autoBind: 'xxxObj'.
	v _ encoder encodeVariable: 'xxxObj' sourceRange: nil ifUnknown: [].
	origReceiver _ attributes getAttribute: #firstNode of: aMessageNode.
	(origReceiver isMemberOf: VariableNode) ifTrue: [
		origReceiverName _ origReceiver name.
	] ifFalse: [
		origReceiverName _ 'self'.
	].

	self replaceRoot: origReceiverName with: v inMessageNode: aMessageNode.
	"self changeSelectorAt: index inMethodNode: aMethodNode."
	self insertTurtleCommandAt: index in: aMethodNode block statements variableNode: v from: encoder firstReceiverName: origReceiverName.

! !

Object subclass: #Presenter
	instanceVariableNames: 'associatedMorph standardPlayer standardPlayfield standardPalette playerList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!Presenter commentStamp: '<historical>' prior: 0!
Optionally associated with a PasteUpMorph, provides a local scope for the running of scripts.

Once more valuable, may be again, but at present occupies primarily a historical niche.

Maintains a playerList cache.

Holds, optionally three 'standard items' -- standardPlayer standardPlayfield standardPalette -- originally providing idiomatic support of ongoing squeak-team internal work, but now extended to more general applicability.

   !

!Presenter methodsFor: 'scripting' stamp: 'ar 3/17/2001 20:14'!
adaptedToWorld: aWorld
	"If I refer to a world or a hand, return the corresponding items in the new world."
	^aWorld presenter! !

!Presenter methodsFor: 'button creation' stamp: 'sw 3/7/1999 00:52'!
addTrashCan
	| aPosition aCan |
	(aCan _ associatedMorph findA: TrashCanMorph) ifNotNil: [^ aCan].
	aCan _ TrashCanMorph newSticky.
	aPosition _ associatedMorph positionNear: (associatedMorph bottomRight - aCan extent) forExtent: aCan extent adjustmentSuggestion:  (-10 @ 0).
	aCan position: aPosition.
	associatedMorph addMorph: aCan.
	aCan startStepping.
	aCan setToAdhereToEdge: #bottomRight.
	^ aCan
! !

!Presenter methodsFor: 'playerList' stamp: 'sw 5/1/2005 00:22'!
allCurrentlyTickingScriptInstantiations
	"Answer a list of ScriptInstantiation objects representing all the scripts within the scope of the receiver which are currently ticking."

	^ Array streamContents:
		[:aStream | 
			self allExtantPlayers do:
				[:aPlayer | aPlayer instantiatedUserScriptsDo:
					[:aScriptInstantiation |
						aScriptInstantiation status == #ticking ifTrue:
							[aStream nextPut: aScriptInstantiation]]]]! !

!Presenter methodsFor: 'intialize' stamp: 'sw 12/13/2004 16:58'!
allExtantPlayers
	"The initial intent here was to produce a list of Player objects associated with any Morph in the tree beneath the receiver's associatedMorph.  whether it is the submorph tree or perhaps off on unseen bookPages.  We have for the moment moved away from that initial intent, and in the current version we only deliver up players associated with the submorph tree only.  <-- this note dates from 4/21/99

Call #flushPlayerListCache; to force recomputation."

	| fullList objectsReferredToByTiles |
	playerList ifNotNil:
		[^ playerList].

	fullList _ associatedMorph allMorphs select: 
		[:m | m player ~~ nil] thenCollect: [:m | m player].
	fullList copy do:
		[:aPlayer |
			aPlayer class scripts do:
				[:aScript |  aScript isTextuallyCoded ifFalse:
					[aScript currentScriptEditor ifNotNilDo: [:ed |
						objectsReferredToByTiles _ ed allMorphs
							select:
								[:aMorph | (aMorph isKindOf: TileMorph) and: [aMorph type == #objRef]]
							thenCollect:
								[:aMorph | aMorph actualObject].
						fullList addAll: objectsReferredToByTiles]]]].

	^ playerList _ (fullList asSet asSortedCollection:
			[:a :b | a externalName < b externalName]) asArray! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 14:17'!
allGoButtons
	"Answer a list of all script-controlling Go buttons within my scope"

	^ associatedMorph allMorphs select:
		[:aMorph | (aMorph isKindOf: ThreePhaseButtonMorph) and:
			[aMorph actionSelector == #goUp:with:]]

	"ActiveWorld presenter allGoButtons"! !

!Presenter methodsFor: 'playerList' stamp: 'yo 7/2/2004 19:45'!
allKnownScriptSelectors
	"Answer a list of all the selectors implemented by any user-scripted objected within the scope of the receiver"

	| aSet allUniclasses |
	aSet _ Set with: ('script' translated , '1') asSymbol.
	allUniclasses _ (self presenter allPlayersWithUniclasses collect:
		[:aPlayer | aPlayer class]) asSet.
	allUniclasses do:
		[:aUniclass | aSet addAll: aUniclass namedTileScriptSelectors].
	^ aSet asSortedArray

"ActiveWorld presenter allKnownScriptSelectors"
! !

!Presenter methodsFor: 'playerList' stamp: 'sw 12/19/2003 23:39'!
allKnownUnaryScriptSelectors
	"Answer a list of all the unary selectors implemented by any user-scripted objected within the scope of the receiver; include #emptyScript as a bail-out"

	| aSet allUniclasses |
	aSet _ Set with: #emptyScript.
	allUniclasses _ (self allPlayersWithUniclasses collect:
		[:aPlayer | aPlayer class]) asSet.
	allUniclasses do:
		[:aUniclass | aSet addAll: aUniclass namedUnaryTileScriptSelectors].
	^ aSet asSortedArray

"ActiveWorld presenter allKnownUnaryScriptSelectors"
! !

!Presenter methodsFor: 'playerList' stamp: 'sw 1/12/2001 16:58'!
allPlayersWithUniclasses
	"Answer a list of all players known to the receiver that have uniclasses"

	^ self allExtantPlayers select: [:p | p belongsToUniClass]! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 14:19'!
allStepButtons
	"Answer a list of all the script-controlling Step buttons within my scope"

	^ associatedMorph allMorphs select:
		[:aMorph | (aMorph isKindOf: ThreePhaseButtonMorph) and:
			[aMorph actionSelector == #stepStillDown:with:]]

	"ActiveWorld presenter allStepButtons"! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 14:18'!
allStopButtons
	"Answer a list of all script-controlling Stop buttons within my scope"

	^ associatedMorph allMorphs select:
		[:aMorph | (aMorph isKindOf: ThreePhaseButtonMorph) and:
			[aMorph actionSelector == #stopUp:with:]]

	"ActiveWorld presenter allStopButtons"! !

!Presenter methodsFor: 'access' stamp: 'sw 4/17/1998 20:07'!
associatedMorph
	^ associatedMorph! !

!Presenter methodsFor: 'access' stamp: 'sw 4/17/1998 20:07'!
associatedMorph: aMorph
	associatedMorph _ aMorph! !

!Presenter methodsFor: 'tile support' stamp: 'dgd 2/22/2003 19:08'!
booleanTiles
	"Answer some boolean-valued tiles.  This dates back to very early etoy work in 1997, and presently has no sent senders"

	| list rcvr op arg |
	list := #(#(0 #< 1) #(0 #<= 1) #(0 #= 1) #(0 #~= 1) #(0 #> 1) #(0 #>= 1)).
	list := list asOrderedCollection collect: 
					[:entry | 
					rcvr := entry first.
					op := (entry second) asSymbol.
					arg := entry last.
					self 
						phraseForReceiver: rcvr
						op: op
						arg: arg
						resultType: #Boolean].
	list add: (self 
				phraseForReceiver: Color red
				op: #=
				arg: Color red
				resultType: #Boolean).
	^list	"copyWith: CompoundTileMorph new"! !

!Presenter methodsFor: 'playerList' stamp: 'sw 8/2/2004 17:24'!
browseAllScriptsTextually
	"Open a method-list browser on all the scripts in the project"

	| aList aMethodList |
	(aList _ self uniclassesAndCounts) size == 0 ifTrue: [^ self inform: 'there are no scripted players'].
	aMethodList _ OrderedCollection new.
	aList do:
		[:aPair | aPair first addMethodReferencesTo: aMethodList].
	aMethodList size > 0 ifFalse: [^ self inform: 'there are no scripts in this project!!'].
	
	SystemNavigation new 
		browseMessageList: aMethodList 
		name: 'All scripts in this project' 
		autoSelect: nil

"
ActiveWorld presenter browseAllScriptsTextually
"! !

!Presenter methodsFor: 'viewer' stamp: 'sw 2/19/2001 15:41'!
cacheSpecs: aMorph
	"For SyntaxMorph's type checking, cache the list of all viewer command specifications."

	aMorph world ifNil: [^ true].
	Preferences universalTiles ifFalse: [^ true].
	Preferences eToyFriendly ifFalse: [^ true].	"not checking"
	(Project current projectParameterAt: #fullCheck ifAbsent: [false]) 
		ifFalse: [^ true].	"not checking"

	SyntaxMorph initialize.! !

!Presenter methodsFor: 'tile support' stamp: 'gm 2/22/2003 14:53'!
constantTile: anObject 
	"Answer a constant tile that represents the object"

	(anObject isColor) 
		ifTrue: 
			[^ColorTileMorph new typeColor: (ScriptingSystem colorForType: #Color)].
	^anObject newTileMorphRepresentative 
		typeColor: (ScriptingSystem colorForType: (self typeForConstant: anObject))! !

!Presenter methodsFor: 'standardPlayer etc' stamp: 'tk 7/28/2005 04:54'!
createStandardPlayer
	| aMorph |

	aMorph _ ImageMorph new image: (ScriptingSystem formAtKey: 'standardPlayer').
	associatedMorph addMorphFront: aMorph.
	(standardPlayer _ aMorph assuredPlayer) renameTo: 'dot' translated.
	aMorph setBalloonText: '...'.
	self positionStandardPlayer.
	^ standardPlayer! !

!Presenter methodsFor: 'misc' stamp: 'dgd 2/22/2003 19:08'!
currentlyViewing: aPlayer 
	"Only detects viewers in tabs"

	aPlayer ifNil: [^false].
	^aPlayer viewerFlapTab notNil! !

!Presenter methodsFor: 'misc' stamp: 'sw 8/28/2002 23:07'!
drawingJustCompleted: aSketchMorph
	"The user just finished drawing.  Now maybe put up a viewer"

	| aWorld |
	self flushPlayerListCache.  "Because a new drawing already created one, thus obviating #assuredPlayer kicking in with its invalidation"

	aWorld _ associatedMorph world.
	(aWorld hasProperty: #automaticFlapViewing)
		ifTrue:
			[^ aWorld presenter viewMorph: aSketchMorph].

	(aSketchMorph pasteUpMorph hasProperty: #automaticViewing)
		ifTrue:
			[self viewMorph: aSketchMorph]! !

!Presenter methodsFor: 'playerList' stamp: 'sw 6/29/1998 17:17'!
flushPlayerListCache
	playerList _ nil! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 14:08'!
goButtonState: newState
	"Get all go buttons in my scope to show the correct state"

	self allGoButtons do:
		[:aButton | aButton state: newState]! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 10/30/1998 15:36'!
goUp: evt with: aMorph
	self startRunningScripts! !

!Presenter methodsFor: 'playerList' stamp: 'sw 3/8/2004 22:09'!
hasAnyTextuallyCodedScripts
	"Answer whether any uniclasses in the receiver have any textually coded scripts"

	self uniclassesAndCounts do:
		[:classAndCount | 
			classAndCount first scripts do:
				[:aScript | aScript isTextuallyCoded ifTrue: [^ true]]].
	^ false

"
ActiveWorld presenter hasAnyTextuallyCodedScripts
"! !

!Presenter methodsFor: 'misc' stamp: 'sw 6/30/1999 20:31'!
morph: aMorph droppedIntoPasteUpMorph: aPasteUpMorph
	aPasteUpMorph automaticViewing ifTrue:
		[aMorph isCandidateForAutomaticViewing ifTrue:
			[self viewMorph: aMorph]]! !

!Presenter methodsFor: 'viewer' stamp: 'sw 1/17/1999 21:51'!
nascentPartsViewer
	^ StandardViewer new! !

!Presenter methodsFor: 'viewer' stamp: 'yo 3/30/2005 11:21'!
nascentPartsViewerFor: aViewee
	"Create a new, naked Viewer object for viewing aViewee.  Give it a vocabulary if either the viewee insists on one or if the project insists on one."

	| aViewer aVocab |
	(aViewee isKindOf: KedamaExamplerPlayer) ifTrue: [^ KedamaStandardViewer new].
	aViewer _ StandardViewer new.
	(aVocab _ aViewee vocabularyDemanded)
		ifNotNil:
			[aViewer useVocabulary: aVocab]
		ifNil:
			[(aVocab _ associatedMorph currentVocabularyFor: aViewee) ifNotNil:
				[aViewer useVocabulary: aVocab]].
	
	"If the viewee does not *demand* a special kind of Viewer, and if the project has not specified a preferred vocabulary, then the system defaults will kick in later"
	^ aViewer! !

!Presenter methodsFor: 'palette & parts bin' stamp: 'sw 4/21/1998 22:57'!
ownStandardPalette
	^ standardPalette! !

!Presenter methodsFor: 'tile support' stamp: 'dgd 2/21/2003 22:35'!
phraseForReceiver: rcvr op: op arg: arg resultType: resultType 
	"Answer a PhraseTileMorph affiliated with the given receiver, initialized to hold the given operator, argument, and result type"

	| m argTile rcvrTile |
	arg isNil 
		ifTrue: 
			[m := PhraseTileMorph new 
						setOperator: op
						type: resultType
						rcvrType: (self typeForConstant: rcvr)]
		ifFalse: 
			[m := PhraseTileMorph new 
						setOperator: op
						type: resultType
						rcvrType: (self typeForConstant: rcvr)
						argType: (self typeForConstant: arg).
			argTile := self constantTile: arg.
			argTile position: m lastSubmorph position.
			m lastSubmorph addMorph: argTile].
	rcvrTile := self constantTile: rcvr.
	"	TilePadMorph makeReceiverColorOfResultType ifTrue: [rcvrTile color: m color]."
	rcvrTile position: m firstSubmorph position.
	m firstSubmorph addMorph: rcvrTile.
	m vResizing: #shrinkWrap.
	^m! !

!Presenter methodsFor: 'standardPlayer etc' stamp: 'sw 1/20/2004 20:08'!
positionStandardPlayer
	"Put the standard player slightly off-screen"

	standardPlayer ifNotNil:
		[standardPlayer costume position: (associatedMorph topLeft - (13@0))]! !

!Presenter methodsFor: 'printing' stamp: 'sw 3/3/1999 01:15'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' (', self asOop printString, ')'! !

!Presenter methodsFor: 'playerList' stamp: 'sw 11/14/2001 00:31'!
reinvigorateAllScriptsTool: anAllScriptsTool 
	"Rebuild the contents of an All Scripts tool"

	| showingOnlyActiveScripts candidateList firstTwo oldList allExtantPlayers newList morphList |
	showingOnlyActiveScripts _ anAllScriptsTool showingOnlyActiveScripts.
	self flushPlayerListCache.
	"needed? Probably to pick up on programmatical script-status control only"

	firstTwo _ {anAllScriptsTool submorphs first.  anAllScriptsTool submorphs second}.
	oldList _ (anAllScriptsTool submorphs copyFrom: 3 to: anAllScriptsTool submorphs size) collect:
		[:aRow |
			(aRow findA: UpdatingSimpleButtonMorph) target].

	allExtantPlayers _ self allExtantPlayers.
	anAllScriptsTool showingAllInstances "take all instances of all classes"
		ifTrue:
			[candidateList _ allExtantPlayers]  

		ifFalse:  "include only one exemplar per uniclass.  Try to get one that has some qualifying scripts"
			[candidateList _ Set new.
			allExtantPlayers do:
				[:aPlayer |
					(candidateList detect: [:plyr | plyr isMemberOf:  aPlayer class] ifNone: [nil]) ifNil:
						[aPlayer instantiatedUserScriptsDo: [:aScriptInstantiation |
							(showingOnlyActiveScripts not or: [aScriptInstantiation pausedOrTicking]) 								ifTrue:
									[candidateList add: aPlayer]]]]].
	newList _ OrderedCollection new.
	candidateList do:
		[:aPlayer | aPlayer instantiatedUserScriptsDo:
			[:aScriptInstantiation |
				(showingOnlyActiveScripts not or: [aScriptInstantiation pausedOrTicking]) ifTrue:
					[newList add: aScriptInstantiation]]].

	oldList asSet = newList asSet
		ifFalse:
			[anAllScriptsTool removeAllMorphs; addAllMorphs: firstTwo.
			morphList _ newList collect:
				[:aScriptInstantiation |  aScriptInstantiation statusControlRowIn: anAllScriptsTool].
			anAllScriptsTool addAllMorphs: morphList.
			newList do:
				[:aScriptInstantiation | aScriptInstantiation updateAllStatusMorphs]]! !

!Presenter methodsFor: 'playerList' stamp: 'sw 7/28/2004 21:00'!
reinvigoratePlayersTool: aPlayersTool 
	"Rebuild the contents of the Players tool"

	| firstTwo oldList newList rowsForPlayers |
	firstTwo _ {aPlayersTool submorphs first.  aPlayersTool submorphs second}.
	oldList _ (aPlayersTool submorphs copyFrom: 3 to: aPlayersTool submorphs size) collect:
		[:aRow |
			aRow playerRepresented].
	self flushPlayerListCache.
	newList _ self allExtantPlayers.
	oldList asSet = newList asSet
		ifFalse:
			[aPlayersTool removeAllMorphs; addAllMorphs: firstTwo.
			rowsForPlayers _ newList collect:
				[:aPlayer |  aPlayer entryForPlayersTool: aPlayersTool].
			aPlayersTool addAllMorphs: rowsForPlayers ]! !

!Presenter methodsFor: 'playerList' stamp: 'yo 2/10/2005 17:07'!
reportPlayersAndScripts
	"Open a window which contains a report on players and their scripts"

	| aList aString |
	(aList _ self uniclassesAndCounts) ifEmpty:  [^ self inform: 'there are no scripted players' translated].
	aString _ String streamContents:
		[:aStream |
			aList do:
				[:aPair |
					aStream nextPutAll: aPair first name, ' -- ', aPair second printString.
					aStream nextPutAll: ' ', (aPair second > 1 ifTrue: ['instances'] ifFalse: ['instance']) translated, ', '.
					aStream nextPutAll: 'named' translated.
					aPair first allInstancesDo: [:inst | aStream space; nextPutAll: inst externalName].
					aStream cr].
			aStream cr.
			aList do:
				[:aPair |
					aStream cr.
					aStream nextPutAll: 
'--------------------------------------------------------------------------------------------'.
					aStream cr; nextPutAll: aPair first typicalInstanceName.
					aStream nextPutAll: '''s' translated.
					aStream nextPutAll: ' scripts:' translated.
					aPair first addDocumentationForScriptsTo: aStream]].

	(StringHolder new contents: aString)
		openLabel: 'All scripts in this project' translated

"self currentWorld presenter reportPlayersAndScripts"! !

!Presenter methodsFor: 'palette & parts bin' stamp: 'sw 6/16/1998 16:17'!
standardPalette: aPalette
	standardPalette _ aPalette! !

!Presenter methodsFor: 'standardPlayer etc' stamp: 'sw 6/17/1998 16:05'!
standardPlayer
	standardPlayer ifNil:
		[self createStandardPlayer].
	standardPlayer costume isInWorld ifFalse: [associatedMorph addMorphNearBack: standardPlayer costume].
	^ standardPlayer! !

!Presenter methodsFor: 'misc' stamp: 'sw 6/17/1998 10:16'!
standardPlayfield: aPlayfield
	standardPlayfield _ aPlayfield! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 18:43'!
startRunningScripts
	"Start running scripts; get stop-step-go buttons to show the right thing"

	self stopButtonState: #off.
	self stepButtonState: #off.
	self goButtonState: #on.
	associatedMorph startRunningAll.

	"associatedMorph borderColor: Preferences borderColorWhenRunning."

	ThumbnailMorph recursionReset.  "needs to be done once in a while (<- tk note from 1997)"! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 18:43'!
startRunningScriptsFrom: ignored
	"Start running all scripts.  Get all script-control buttons to show the right thing."

	self startRunningScripts! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 14:10'!
stepButtonState: newState
	"Get all step buttons in my scope to show the correct state"

	self allStepButtons do:
		[:aButton | aButton state: newState]! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 5/6/1998 10:20'!
stepDown: evt with: aMorph
	self stopRunningScripts! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 14:06'!
stepStillDown: dummy with: theButton
	"The step button is still down; get temporary button feedback right and step all and then get all button feedback right again"

	self stepButtonState: #pressed.
	self stopButtonState: #off.
	associatedMorph stepAll.
	associatedMorph world displayWorld.
	self stepButtonState: #off.
	self stopButtonState: #on
! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 14:06'!
stepUp: evt with: aMorph
	"The step button came up; get things right"

	self stepButtonState: #off! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 14:08'!
stopButtonState: newState
	"Get all stop buttons in my scope to show the correct state"

	self allStopButtons do:
		[:aButton | aButton state: newState]! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 18:42'!
stopRunningScripts
	"Put all ticking scripts within my scope into paused mode.  Get any scripting-control buttons to show the correct state"

	self stopButtonState: #on.
	self stepButtonState: #off.
	self goButtonState: #off.
	associatedMorph stopRunningAll.

	"associatedMorph borderColor: Preferences borderColorWhenStopped"! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 11/13/2001 18:42'!
stopRunningScriptsFrom: ignored
	"Stop running scripts; get all script-control buttons to reflect this"

	self stopRunningScripts! !

!Presenter methodsFor: 'stop-step-go buttons' stamp: 'sw 6/29/1998 17:23'!
stopUp: dummy with: theButton
	self flushPlayerListCache.  "catch guys not in cache but who're running"
	self stopRunningScripts! !

!Presenter methodsFor: 'palette & parts bin' stamp: 'sw 2/12/2001 22:02'!
systemQueryPhraseWithActionString: anActionString labelled: aLabel
	"Answer a SystemQueryPhrase with the given action string and label"

	| aTile aPhrase |
	
	aPhrase _ SystemQueryPhrase new.
	aTile _ BooleanTile new.
	aTile setExpression: anActionString label: aLabel.
	aPhrase addMorph: aTile.
	aPhrase enforceTileColorPolicy.
	^ aPhrase! !

!Presenter methodsFor: 'misc' stamp: 'sw 4/23/1998 18:47'!
tempCommand
	Transcript cr; show: '#tempCommand invoked for Presenter'! !

!Presenter methodsFor: 'playerList' stamp: 'sw 1/30/2001 23:07'!
toolToViewScriptInstantiations
	"Open a tool which shows, and allows the user to change the status of, all the instantiations of all the user-written scripts in the world"

	AllScriptsTool launchAllScriptsToolFor: self

	"self currentWorld presenter toolToViewScriptInstantiations"! !

!Presenter methodsFor: 'tile support' stamp: 'sw 4/25/1998 13:28'!
typeForConstant: anObject
	^ anObject basicType! !

!Presenter methodsFor: 'playerList' stamp: 'sw 1/12/2001 18:10'!
uniclassesAndCounts
	"Answer a list of all players known to the receiver that have uniclasses"

	^ (self allPlayersWithUniclasses collect: [:aPlayer | aPlayer class]) asSet asArray collect:
		[:aClass | Array
			with:	aClass
			with:	aClass instanceCount]


	"self currentWorld presenter uniclassesAndCounts"! !

!Presenter methodsFor: 'viewer' stamp: 'sw 12/28/1998 22:34'!
updateViewer: aViewer
	self updateViewer: aViewer forceToShow: nil! !

!Presenter methodsFor: 'viewer' stamp: 'yo 4/4/2005 18:22'!
updateViewer: aViewer forceToShow: aCategorySymbol
	"Update the given viewer to make sure it is in step with various possible changes in the outside world, and when reshowing it be sure it shows the given category"

	| aPlayer aPosition newViewer oldOwner wasSticky barHeight itsVocabulary aCategory categoryInfo restrictedIndex |
	aCategory _ aCategorySymbol ifNotNil: [aViewer currentVocabulary translatedWordingFor: aCategorySymbol].
	categoryInfo _ aViewer categoryMorphs  asOrderedCollection collect:
		[:aMorph | aMorph categoryRestorationInfo].

	itsVocabulary _ aViewer currentVocabulary.
	aCategory ifNotNil: [(categoryInfo includes: aCategorySymbol) ifFalse: [categoryInfo addFirst: aCategorySymbol]].
	aPlayer _ aViewer scriptedPlayer.
	aPosition _ aViewer position.
	wasSticky _ aViewer isSticky.
	newViewer _ aViewer species new visible: false.
	(aViewer isMemberOf: KedamaStandardViewer)
		ifTrue: [restrictedIndex _ aViewer restrictedIndex].
	barHeight _ aViewer submorphs first listDirection == #topToBottom
		ifTrue:
			[aViewer submorphs first submorphs first height]
		ifFalse:
			[0].
	Preferences viewersInFlaps ifTrue:
		[newViewer setProperty: #noInteriorThumbnail toValue: true].

	newViewer rawVocabulary: itsVocabulary.
	newViewer limitClass: aViewer limitClass.
	newViewer initializeFor: aPlayer barHeight: barHeight includeDismissButton: aViewer hasDismissButton showCategories: categoryInfo.
	(newViewer isMemberOf: KedamaStandardViewer)
		ifTrue: [
			newViewer providePossibleRestrictedView: 0.
			newViewer providePossibleRestrictedView: restrictedIndex].
	wasSticky ifTrue: [newViewer beSticky].
	oldOwner _ aViewer owner.
	oldOwner ifNotNil:
		[oldOwner replaceSubmorph: aViewer by: newViewer].
	
	"It has happened that old readouts are still on steplist.  We may see again!!"

	newViewer position: aPosition.
	newViewer enforceTileColorPolicy.
	newViewer visible: true.
	newViewer world ifNotNilDo: [:aWorld | aWorld startSteppingSubmorphsOf: newViewer].
	newViewer layoutChanged! !

!Presenter methodsFor: 'tile support' stamp: 'sw 9/27/2001 17:43'!
valueTiles
	"Answer some constant-valued tiles.  This dates back to very early etoy work in 1997, and presently has no senders"

	| tiles |
	tiles _ OrderedCollection new.
	tiles add: (5 newTileMorphRepresentative typeColor: (ScriptingSystem colorForType: #Number)).
	tiles add: (ColorTileMorph new typeColor: (ScriptingSystem colorForType: #Color)).
	tiles add: (TileMorph new typeColor: (ScriptingSystem colorForType: #Number);
			setExpression: '(180 atRandom)'
			label: 'random').
	tiles add: RandomNumberTile new.
	^ tiles! !

!Presenter methodsFor: 'viewer' stamp: 'nk 9/21/2003 12:53'!
viewMorph: aMorph 
	| aPlayer openViewers aViewer aPalette aRect aPoint nominalHeight aFlapTab topItem flapLoc |
	Sensor leftShiftDown 
		ifFalse: 
			[((aPalette := aMorph standardPalette) notNil and: [aPalette isInWorld]) 
				ifTrue: [^aPalette viewMorph: aMorph]].
	aPlayer := (topItem := aMorph topRendererOrSelf) assuredPlayer.
	openViewers _ aPlayer allOpenViewers.
	aViewer := openViewers isEmpty ifFalse: [ openViewers first ] ifTrue: [ self nascentPartsViewer ].
	self cacheSpecs: topItem.	"redo the spec cache once in a while"

	"19 sept 2000 - allow flaps in any paste up"
	flapLoc := associatedMorph.	"world"
	Preferences viewersInFlaps  ifTrue:  [
		aViewer owner ifNotNilDo: [ :f | ^f flapTab showFlap; yourself ].
		aViewer setProperty: #noInteriorThumbnail toValue: true.
			aViewer initializeFor: aPlayer barHeight: 0.
			aViewer enforceTileColorPolicy.
			aViewer fullBounds.	"force layout"
			"associatedMorph addMorph: aViewer."	"why???"
			flapLoc hideViewerFlapsOtherThanFor: aPlayer.
			aFlapTab := flapLoc viewerFlapTabFor: topItem.
			aFlapTab referent submorphs 
				do: [:m | (m isKindOf: Viewer) ifTrue: [m delete]].
			aViewer visible: true.
			aFlapTab applyThickness: aViewer width + 25.
			aFlapTab spanWorld.
			aFlapTab showFlap.
			aViewer position: aFlapTab referent position.
			aFlapTab referent addMorph: aViewer beSticky.	"moved"
			flapLoc startSteppingSubmorphsOf: aFlapTab.
			flapLoc startSteppingSubmorphsOf: aViewer.
			^aFlapTab].
	aViewer initializeFor: aPlayer barHeight: 6.
	aViewer enforceTileColorPolicy.
	aViewer fullBounds.	"force layout"
	Preferences automaticViewerPlacement 
		ifTrue: 
			[aPoint := aMorph bounds right 
						@ (aMorph center y - ((nominalHeight := aViewer initialHeightToAllow) // 2)).
			aRect := (aPoint extent: aViewer width @ nominalHeight) 
						translatedToBeWithin: flapLoc bounds.
			aViewer position: aRect topLeft.
			aViewer visible: true.
			associatedMorph addMorph: aViewer.
			flapLoc startSteppingSubmorphsOf: aViewer.
			"it's already in the world, somewhat coincidentally"
			^aViewer].
	aMorph primaryHand attachMorph: (aViewer visible: true).
	^aViewer! !

!Presenter methodsFor: 'viewer' stamp: 'sw 6/20/2001 12:37'!
viewObject: anObject
	"Open up and return a viewer on the given object.  If the object is a Morph, open a viewer on its associated Player"

	anObject isMorph
		ifTrue:
			[self viewMorph: anObject]  "historic morph/player implementation"
		ifFalse:
			[self viewObjectDirectly: anObject]! !

!Presenter methodsFor: 'viewer' stamp: 'sw 6/20/2001 13:12'!
viewObjectDirectly: anObject
	"Open up and return a viewer on the given object"

	|  aViewer aRect aPoint nominalHeight aFlapTab flapLoc |

	associatedMorph addMorph: (aViewer _ self nascentPartsViewerFor: anObject).
	flapLoc _ associatedMorph "world".
	Preferences viewersInFlaps ifTrue:
		[aViewer setProperty: #noInteriorThumbnail toValue: true.
		aViewer initializeFor: anObject barHeight: 0.
		aViewer enforceTileColorPolicy.
		flapLoc hideViewerFlapsOtherThanFor: anObject.
		aFlapTab _ flapLoc viewerFlapTabFor: anObject.
		aFlapTab referent submorphs do: 
			[:m | (m isKindOf: Viewer) ifTrue: [m delete]].
		aFlapTab referent addMorph: aViewer beSticky.
		aViewer visible: true.
		aFlapTab applyThickness: aViewer width + 25.
		aFlapTab spanWorld.
		aFlapTab showFlap. 
		aViewer position: aFlapTab referent position.
		flapLoc startSteppingSubmorphsOf: aFlapTab.
		flapLoc startSteppingSubmorphsOf: aViewer.
		^ aFlapTab].
	
	"Caution: the branch below is historical and has not been used for a long time, though if you set the #viewersInFlaps preference to false you'd hit it.  Not at all recently maintained."
	aViewer initializeFor: anObject barHeight: 6.
	aViewer enforceTileColorPolicy.
	Preferences automaticViewerPlacement ifTrue:
		[aPoint _ anObject bounds right @ 
			(anObject center y - ((nominalHeight _ aViewer initialHeightToAllow) // 2)).
		aRect _ (aPoint extent: (aViewer width @ nominalHeight)) translatedToBeWithin: flapLoc bounds.
		aViewer position: aRect topLeft.
		aViewer visible: true.
		flapLoc startSteppingSubmorphsOf: aViewer.
		"it's already in the world, somewhat coincidentally"
		^ aViewer].
	anObject primaryHand attachMorph: (aViewer visible: true).
	^aViewer! !

!Presenter methodsFor: 'access' stamp: 'sw 4/22/1998 20:05'!
world
	^ associatedMorph world! !

Object subclass: #ScriptInstantiation
	instanceVariableNames: 'player selector status frequency anonymous tickingRate lastTick'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!ScriptInstantiation commentStamp: '<historical>' prior: 0!
One of these is associated with each user-defined script for each Player.   Holds the state that defines when the script should be run automatically by the system.

	player				The player whose script this is.
	selector				The message to send my player to activate this script
	status				#ticking, #paused, #normal, #mouseDown, #mouseStillDown, #mouseUp,
							#mouseEnter, #mouseLeave, #keyStroke
	frequency			For ticking scripts, their frequency.  Place-holder: not implemented yet
	anonymous			If true, the script has is unnamed -- in this case, the selector is private to the implementation!

!ScriptInstantiation methodsFor: 'customevents-status control' stamp: 'nk 6/30/2004 19:00'!
addStatusChoices: choices toMenu: menu
	choices isEmpty ifFalse: [
		choices	 do: [ :choice || label sym |
			(choice isKindOf: Array) 
				ifTrue: [ label := choice first translated. sym := choice second ]
				ifFalse: [ label := choice translated. sym := choice ].
			menu add: label target: menu selector: #modalSelection: argument: sym ].
		menu addLine. ].
	^menu.
! !

!ScriptInstantiation methodsFor: 'customevents-status control' stamp: 'nk 7/17/2004 09:01'!
addStatusChoices: choices toSubMenu: submenu forMenu: menu
	choices isEmpty ifFalse: [
		choices	 do: [ :choice || label sym |
			(choice isKindOf: Array) 
				ifTrue: [ label := choice first translated. sym := choice second ]
				ifFalse: [ label := choice translated. sym := choice ].
			submenu add: label target: menu selector: #modalSelection: argument: sym ].
		menu addLine. ].
	^menu.
! !

!ScriptInstantiation methodsFor: 'misc' stamp: 'sw 10/24/2004 17:05'!
assignStatusToAllSiblings
	"Let all sibling instances of my player have the same status that I do.  The stati affected are both the event stati and the tickingStati"

	| aScriptInstantiation |
	(player class allInstances copyWithout: player) do:
		[:aPlayer |
			aScriptInstantiation _ aPlayer scriptInstantiationForSelector: selector.
			aScriptInstantiation status: status.
			aScriptInstantiation frequency: self frequency.
			aScriptInstantiation tickingRate: self tickingRate.
			aScriptInstantiation updateAllStatusMorphs]! !

!ScriptInstantiation methodsFor: 'misc' stamp: 'sw 11/14/2001 00:32'!
assignStatusToAllSiblingsIn: aStatusViewer
	"Let all sibling instances of my player have the same status that I do"

	self assignStatusToAllSiblings.
	self updateAllStatusMorphs.
	aStatusViewer presenter reinvigorateAllScriptsTool: aStatusViewer ! !

!ScriptInstantiation methodsFor: 'misc' stamp: 'sw 5/13/1998 13:06'!
assureEventHandlerRepresentsStatus
	self status: self status! !

!ScriptInstantiation methodsFor: 'player & selector access' stamp: 'sw 1/28/2001 16:06'!
changeSelectorTo: aNewSelector
	"Change the selector associated with the receiver to the new one provided"

	selector _ aNewSelector! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'sw 1/31/2001 19:16'!
chooseTriggerFrom: ignored
	"Backward compatibility for old scriptors"

	^ self presentScriptStatusPopUp! !

!ScriptInstantiation methodsFor: 'customevents-status control' stamp: 'nk 7/21/2003 20:07'!
defineNewEvent
	| newEventName newEventHelp |
	"Prompt the user for the name of a new event and install it into the custom event table"
	newEventName _ FillInTheBlankMorph request: 'What is the name of your new event?'.
	newEventName isEmpty ifTrue: [ ^self ].
	newEventName _ newEventName asSymbol.
	(ScriptingSystem customEventStati includes: newEventName) ifTrue: [
		self inform: 'That event is already defined.'. ^self ].
	newEventHelp _ FillInTheBlankMorph request: 'Please describe this event:'.
	ScriptingSystem addUserCustomEventNamed: newEventName help: newEventHelp.! !

!ScriptInstantiation methodsFor: 'customevents-status control' stamp: 'nk 7/21/2003 20:32'!
deleteCustomEvent
	| userEvents eventName |
	userEvents _  ScriptingSystem userCustomEventNames.
	eventName _ (SelectionMenu selections: userEvents) startUpWithCaption: 'Remove which event?' at: ActiveHand position allowKeyboard: true.
	eventName ifNotNil: [ ScriptingSystem removeUserCustomEventNamed: eventName ].
	self class allSubInstancesDo: [ :ea | ea status = eventName ifTrue: [ ea status: #normal ]]! !

!ScriptInstantiation methodsFor: 'customevents-status control' stamp: 'nk 11/1/2004 08:13'!
explainStatusAlternatives
	"Open a little window that explains the various status 
	alternatives "
	(StringHolder new contents: (ScriptingSystem statusHelpStringFor: player))
		openLabel: 'Script Status' translated! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'nk 8/21/2004 12:08'!
fireOnce
	"Run this script once"

	player triggerScript: selector! !

!ScriptInstantiation methodsFor: 'frequency' stamp: 'dgd 2/22/2003 14:50'!
frequency
	(frequency isNil or: [frequency = 0]) ifTrue: [frequency := 1].
	^frequency! !

!ScriptInstantiation methodsFor: 'frequency' stamp: 'sw 4/21/1999 09:13'!
frequency: aNumber
	frequency _ aNumber! !

!ScriptInstantiation methodsFor: 'anonymity' stamp: 'sw 10/30/2000 11:08'!
initializePermanentScriptFor: aPlayer selector: aSelector
	"Initialize the receiver on behalf of the player, setting its status to #normal and giving it the given selector"

	player _ aPlayer.
	status _ #normal.
	selector _ aSelector! !

!ScriptInstantiation methodsFor: 'misc' stamp: 'dgd 12/15/2003 10:11'!
offerMenuIn: aStatusViewer
	"Put up a menu."

	| aMenu |
	ActiveHand showTemporaryCursor: nil.
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu title: player knownName, ' ', selector.
	aMenu addStayUpItem.
	(player class instanceCount > 1) ifTrue:
		[aMenu add: 'propagate status to siblings' translated selector: #assignStatusToAllSiblingsIn: argument: aStatusViewer.
		aMenu balloonTextForLastItem: 'Make the status of this script in all of my sibling instances be the same as the status you see here' translated].

	aMenu add: 'reveal this object' translated target: player selector: #revealPlayerIn: argument: ActiveWorld.
	aMenu balloonTextForLastItem: 'Make certain this object is visible on the screen; flash its image for a little while, and give it the halo.' translated.
	aMenu add: 'open this script''s Scriptor' translated target: player selector: #grabScriptorForSelector:in: argumentList: {selector. aStatusViewer world}.
	aMenu balloonTextForLastItem: 'Open up the Scriptor for this script' translated.
	aMenu add: 'open this object''s Viewer' translated target: player selector: #beViewed.
	aMenu balloonTextForLastItem: 'Open up a Viewer for this object' translated.
	aMenu addLine.
	aMenu add: 'more...' translated target: self selector: #offerShiftedMenuIn: argument: aStatusViewer.
	aMenu balloonTextForLastItem: 'The "more..." branch offers you menu items that are less frequently used.' translated.
	aMenu popUpInWorld: ActiveWorld! !

!ScriptInstantiation methodsFor: 'misc' stamp: 'dgd 12/15/2003 10:12'!
offerShiftedMenuIn: aStatusViewer
	"Put up the shifted menu"

	| aMenu |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu title: player knownName, ' ', selector.
	aMenu add: 'grab this object' translated target: player selector: #grabPlayerIn: argument: self currentWorld.
	aMenu balloonTextForLastItem: 'Wherever this object currently is, the "grab" command will rip it out, and place it in your "hand".  This is a very drastic step, that can disassemble things that may be very hard to put back together!!' translated.
	aMenu add: 'destroy this script' translated target: player selector: #removeScriptWithSelector: argument: selector.
	aMenu balloonTextForLastItem: 'Caution!!  This is irreversibly destructive -- it removes the script from the system.' translated.
	aMenu addLine.
	aMenu add: 'inspect morph' translated target: player costume selector: #inspect.
	aMenu add: 'inspect player' translated target: player selector: #inspect.
	aMenu popUpInWorld: ActiveWorld! !

!ScriptInstantiation methodsFor: 'running' stamp: 'sw 11/13/2001 10:33'!
pausedOrTicking
	"Answer true if the receiver is either in paused or ticking status, false otherwise"

	^ #(paused ticking) includes: status! !

!ScriptInstantiation methodsFor: 'player & selector access' stamp: 'dgd 2/16/2003 21:42'!
player
	"answer the receiver's player"
	^ player! !

!ScriptInstantiation methodsFor: 'player & selector access' stamp: 'sw 5/13/1998 10:43'!
player: p
	"Needed for making duplicate players, otherwise private"
	player _ p! !

!ScriptInstantiation methodsFor: 'player & selector access' stamp: 'sw 5/1/1998 21:33'!
player: p selector: s status: st
	frequency _ 0.
	status _ st.
	player _ p.
	selector _ s! !

!ScriptInstantiation methodsFor: 'misc' stamp: 'sw 1/19/2001 17:45'!
playersExternalName
	"Answer the external name of my player"

	^ player externalName! !

!ScriptInstantiation methodsFor: 'misc' stamp: 'ar 2/12/2001 18:58'!
prepareToBeRunning
	lastTick _ nil.! !

!ScriptInstantiation methodsFor: 'customevents-status control' stamp: 'nk 11/1/2004 07:53'!
presentScriptStatusPopUp
	"Put up a menu of status alternatives and carry out the request"

	| reply  m menu submenu |

	menu _ MenuMorph new.
	self addStatusChoices: #( normal " -- run when called" ) toMenu: menu.
	self addStatusChoices: 
		#(	paused 		"ready to run all the time"
			ticking			"run all the time" )
		toMenu: menu.
	self addStatusChoices: (ScriptingSystem standardEventStati copyFrom: 1 to: 3) toMenu: menu.
	self addStatusChoices: (ScriptingSystem standardEventStati allButFirst: 3) toMenu: menu.
	self addStatusChoices: 
		#(opening			"when I am being opened"
			closing			"when I am being closed" )
		toMenu: menu.
	
	submenu _ MenuMorph new.
	self addStatusChoices: (ScriptingSystem globalCustomEventNamesFor: player) toSubMenu: submenu forMenu: menu.
	menu add: 'more... ' translated subMenu: submenu.

	(Preferences allowEtoyUserCustomEvents) ifTrue: [
		submenu addLine.
		self addStatusChoices: ScriptingSystem userCustomEventNames toSubMenu: submenu forMenu: menu.
		submenu addLine.
		self addStatusChoices:
			(Array streamContents: [ :s | s nextPut: { 'define a new custom event'. #defineNewEvent }.
			ScriptingSystem userCustomEventNames isEmpty
				ifFalse: [ s nextPut: { 'delete a custom event'. #deleteCustomEvent } ]])
			toSubMenu: submenu forMenu: menu ].
	
	menu addLine.

	self addStatusChoices: #(
		('what do these mean?'explainStatusAlternatives)
		('apply my status to all siblings' assignStatusToAllSiblings) ) toMenu: menu.

	menu addTitle: 'When should this script run?' translated.
	menu submorphs last delete.
	menu invokeModal.
	
	reply := menu modalSelection.

	reply == #explainStatusAlternatives ifTrue: [^ self explainStatusAlternatives].
	reply == #assignStatusToAllSiblings ifTrue: [^ self assignStatusToAllSiblings].
	reply == #defineNewEvent ifTrue: [ ^self defineNewEvent ].
	reply == #deleteCustomEvent ifTrue: [ ^self deleteCustomEvent ].

	reply ifNotNil: 
		[self status: reply.  "Gets event handlers fixed up"
		reply == #paused ifTrue:
			[m _ player costume.
			(m isKindOf: SpeakerMorph) ifTrue: [m stopSound]].
		self updateAllStatusMorphs]
! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'dgd 9/6/2003 18:15'!
presentTickingMenu
	"Put up a menu of status alternatives and carry out the request"
	| aMenu ticks item any |
	ticks _ self tickingRate.
	ticks = ticks asInteger ifTrue:[ticks _ ticks asInteger].
	aMenu _ MenuMorph new defaultTarget: self.
	any _ false.
	#(1 2 5 8 10 25 50 100) do:[:i | 
		item _ aMenu addUpdating: nil target: self selector: #tickingRate: argumentList: {i}.
		item contents:
			((ticks = i) ifTrue:[ any _ true. '<on>', i printString]
					ifFalse:['<off>', i printString])].
	item _ aMenu addUpdating: nil target: self selector: #typeInTickingRate argumentList: #().
	item contents: (any ifTrue:['<off>'] ifFalse:['<on>']), 'other...' translated.
	aMenu addTitle: ('Ticks (now: {1}/sec)' translated format:{ticks}).
	aMenu  popUpEvent: self currentEvent in: self currentWorld! !

!ScriptInstantiation methodsFor: 'printing' stamp: 'sw 5/24/2001 16:01'!
printOn: aStream
	"Print the receiver on aStream"

	super printOn: aStream.
	aStream nextPutAll: ' (', self oopString, ') '.
	aStream nextPutAll: ' for #', selector asString! !

!ScriptInstantiation methodsFor: 'customevents-status control' stamp: 'nk 9/25/2003 11:35'!
removeEventTriggersForMorph: actualMorph 
	"user custom events are triggered at the World, while system custom events are triggered on individual Morphs."

	actualMorph removeActionsSatisfying: 
			[:action | 
			action receiver == player and: 
					[(#(#doScript: #triggerScript:) includes: action selector) 
						and: [action arguments first == selector]]]
		forEvent: status.
	self currentWorld removeActionsSatisfying: 
			[:action | 
			action receiver == player and: 
					[(#(#doScript: #triggerScript:) includes: action selector) 
						and: [action arguments first == selector]]]
		forEvent: status! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'sw 6/13/2002 15:14'!
resetToNormalIfCurrently: aStatus
	"If my status *had been* aStatus, quietly reset it to normal, without tampering with event handlers.  But get the physical display of all affected status morphs right"

	status == aStatus ifTrue:
		[status _ #normal.
		self updateAllStatusMorphs]! !

!ScriptInstantiation methodsFor: 'running' stamp: 'nk 8/21/2004 12:08'!
runIfClosing
	| result |
	(result _ status == #closing) ifTrue:
		[player triggerScript: selector].
	^ result! !

!ScriptInstantiation methodsFor: 'running' stamp: 'nk 8/21/2004 12:08'!
runIfOpening
	| result |
	(result _ status == #opening) ifTrue:
		[player triggerScript: selector].
	^ result! !

!ScriptInstantiation methodsFor: 'running' stamp: 'sw 11/16/2004 13:55'!
runIfTicking: nowTick 
	"If the receiver is meant to be ticking, run it if it's time"

	| ticks rate |
	status == #ticking ifFalse: [^self].
	rate := self tickingRate.
	ticks := (lastTick isNil or: [nowTick < lastTick]) 
				ifTrue: 
					[lastTick := nowTick.
					1]
				ifFalse: [((nowTick - lastTick) * rate * 0.001) asInteger].
	ticks <= 0 ifTrue: [^self].

	"Scripts which have been out of the world and then return can have a huge number of ticks accumulated. A better fix would be to reset <lastTick> when a script leaves/enters the world. Also, if the system is falling behind, this attempt to catch up can result in falling further behind, leading to sluggish behavior and termination of ticking status. Whether the user really wants this catch up behavior needs to be determined (often she will not, I suspect) and better ways of doing it need to be found.  (This comment inserted by Bob Arning on 3/5/2001)"
	ticks := 1.
	1 to: ticks * self frequency do: [:i | player triggerScript: selector].
	lastTick := nowTick.
	ticks > 10 
		ifTrue: 
			["check if we're lagging behind"

			ticks <= ((Time millisecondClockValue - lastTick) * rate / 1000) asInteger 
				ifTrue: 
					["e.g., time to run script is higher than number of ticks"

					self status: #paused.
					self updateAllStatusMorphs]]! !

!ScriptInstantiation methodsFor: 'player & selector access' stamp: 'sw 4/30/1998 14:07'!
selector
	^ selector! !

!ScriptInstantiation methodsFor: 'player & selector access' stamp: 'sw 4/30/1998 14:07'!
selector: sel
	selector _ sel! !

!ScriptInstantiation methodsFor: 'running' stamp: 'ar 2/12/2001 17:00'!
startRunningIfPaused
	"If the receiver is paused, start it ticking"

	status == #paused ifTrue:
		[self status: #ticking.
		self updateAllStatusMorphs]! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'sw 4/30/1998 14:10'!
status
	status ifNil: [status _ #normal].
	^ status! !

!ScriptInstantiation methodsFor: 'customevents-status control' stamp: 'nk 9/25/2003 11:38'!
status: newStatus 
	"Set the receiver's status as indicated"

	| stati actualMorph |
	actualMorph := player costume renderedMorph.

	"standard (EventHandler) events"
	stati := ScriptingSystem standardEventStati.
	(stati includes: status) 
		ifTrue: 
			[actualMorph 
				on: status
				send: nil
				to: nil
			"remove old link in event handler"].
	(stati includes: newStatus) 
		ifTrue: 
			[actualMorph 
				on: newStatus
				send: selector
				to: player.
			"establish new link in evt handler"
			player assureNoScriptOtherThan: self hasStatus: newStatus].

	"user custom events are triggered at the World, while system custom events are triggered on individual Morphs."
	self removeEventTriggersForMorph: actualMorph.
	stati := ScriptingSystem customEventStati.
	(stati includes: newStatus) 
		ifTrue: 
			[(ScriptingSystem userCustomEventNames includes: newStatus) 
				ifTrue: 
					[self currentWorld 
						when: newStatus
						send: #triggerScript:
						to: player
						withArguments: { 
								selector}]
				ifFalse: 
					[actualMorph when: newStatus
						evaluate: (MessageSend 
								receiver: player
								selector: #triggerScript:
								arguments: { 
										selector})]].
	status := newStatus.
	self pausedOrTicking ifTrue: [lastTick := nil].
	self flag: #arNote.	"this from fall 2000"
	self flag: #workaround.	"Code below was in #chooseTriggerFrom: which did not reflect status changes from other places (e.g., the stepping/pause buttons). It is not clear why this is necessary though - theoretically, any morph should step when it has a player but alas!! something is broken and I have no idea why and where."

	"14 feb 2001 - bob - I reinstated this after alan noticed that a newly drawn car would not go until you picked it up and dropped it. The reason is that unscripted players have #wantSteps ^false. If a morph enters the world with an unscripted player and then acquires a scripted player, that would be a good time to change, but this will work too"
	status == #ticking 
		ifTrue: 
			[player costume isStepping ifFalse: [player costume arrangeToStartStepping]]! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'sw 9/21/2000 10:38'!
statusControlMorph
	"Answer a control that will serve to reflect (and allow the user to change) the status of the receiver"

	^ ScriptStatusControl new initializeFor: self
! !

!ScriptInstantiation methodsFor: 'misc' stamp: 'dgd 12/15/2003 10:09'!
statusControlRowIn: aStatusViewer
	"Answer an object that reports my status and lets the user change it"

	| aRow aMorph buttonWithPlayerName |
	aRow _ ScriptStatusLine newRow beTransparent.
	buttonWithPlayerName _ UpdatingSimpleButtonMorph new.
	buttonWithPlayerName
		on: #mouseEnter send: #menuButtonMouseEnter: to: buttonWithPlayerName;
		 on: #mouseLeave send: #menuButtonMouseLeave: to: buttonWithPlayerName.

	buttonWithPlayerName target: self; wordingSelector: #playersExternalName; actionSelector: #offerMenuIn:; arguments: {aStatusViewer}; beTransparent; actWhen: #buttonDown.
	buttonWithPlayerName setBalloonText: 'This is the name of the player to which this script belongs; if you click here, you will get a menu of interesting options pertaining to this player and script' translated.
	buttonWithPlayerName borderWidth: 1; borderColor: Color blue.
	aRow addMorphBack: buttonWithPlayerName.
	aRow addTransparentSpacerOfSize: 10@0.
	aRow addMorphBack: AlignmentMorph newVariableTransparentSpacer.

	aMorph _ UpdatingStringMorph on: self selector: #selector.
	aMorph color: Color brown lighter; useStringFormat.
	aMorph setBalloonText: 'This is the name of the script to which this entry pertains.' translated.
	aRow addMorphBack: aMorph.
	aRow addMorphBack: AlignmentMorph newVariableTransparentSpacer.
	aRow addTransparentSpacerOfSize: 10@0.

	aRow addMorphBack: self statusControlMorph.
	aRow submorphsDo: [:m | m wantsSteps ifTrue: [m step]].
	^ aRow! !

!ScriptInstantiation methodsFor: 'running' stamp: 'ar 2/12/2001 17:00'!
stopTicking
	"If I'm ticking stop, else do nothing"

	status == #ticking ifTrue:
		[self status: #paused.
		self updateAllStatusMorphs]! !

!ScriptInstantiation methodsFor: 'frequency' stamp: 'ar 2/12/2001 18:26'!
tickingRate
	"Return the number of ticks per second this script should get"
	^tickingRate ifNil:[8]! !

!ScriptInstantiation methodsFor: 'frequency' stamp: 'ar 2/12/2001 18:26'!
tickingRate: aNumber
	"See the comment in #tickingRate"
	tickingRate _ aNumber.
	self updateAllStatusMorphs.! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'dgd 9/21/2003 15:59'!
translatedStatus
	^ self status translated! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'em 3/24/2005 14:49'!
typeInTickingRate
	| reply aNumber |
	reply _ FillInTheBlank request: 'Number of ticks per second: ' translated initialAnswer: self tickingRate printString.

	reply ifNotNil:
		[aNumber _ reply asNumber.
		aNumber > 0 ifTrue:
			[self tickingRate: aNumber]]! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'sw 9/21/2000 10:37'!
updateAllStatusMorphs
	"Update all status morphs bound to the receiver.  Done with a sledge-hammer at present."

	(self currentWorld allMorphs select: [:m | (m isKindOf: ScriptStatusControl) and:
			[m scriptInstantiation == self]]) do:
		[:aStatusControl | self updateStatusMorph: aStatusControl]! !

!ScriptInstantiation methodsFor: 'status control' stamp: 'dgd 9/21/2003 16:04'!
updateStatusMorph: statusControlMorph
	"the status control may need to reflect an externally-induced change in status"

	| colorSelector statusReadoutButton |
	statusControlMorph ifNil: [^ self].

	self pausedOrTicking
		ifTrue:
			[statusControlMorph assurePauseTickControlsShow]
		ifFalse:
			[statusControlMorph maybeRemovePauseTickControls].
	statusReadoutButton _ statusControlMorph submorphs last.
	colorSelector _ ScriptingSystem statusColorSymbolFor: self status.
	statusReadoutButton color: (Color perform: colorSelector) muchLighter.
	statusReadoutButton label: self translatedStatus asString font: Preferences standardButtonFont! !

!ScriptInstantiation methodsFor: 'misc' stamp: 'sw 4/30/1998 13:47'!
userScriptObject
	"Answer the UserScript object in the receiver's class that holds on to the editing state for this script"

	^ player class userScriptForPlayer: player selector: selector! !

ScriptInstantiation subclass: #UserScript
	instanceVariableNames: 'currentScriptEditor formerScriptEditors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!UserScript commentStamp: '<historical>' prior: 0!
Holds the ScriptEditorMorph structures for the current version of a user-defined tile script, as well as previous versions thereof.
	currentScriptEditor	The current version of the ScriptEditorMorph for the script
	formerScriptEditors 	Earlier versions of the script, for recapturing via the Versions feature
							(a dictionary, <timeStamp> -> ScriptEditorMorph!

!UserScript methodsFor: 'script editor' stamp: 'sw 5/19/1998 12:18'!
allScriptVersionsDo: aBlock
	self isTextuallyCoded ifFalse: [aBlock value: currentScriptEditor].
	formerScriptEditors ifNotNil: [formerScriptEditors do:
		[:ed | aBlock value: ed]]! !

!UserScript methodsFor: 'textually coded' stamp: 'sw 5/19/1998 14:13'!
becomeTextuallyCoded
	currentScriptEditor _ #textuallyCoded! !

!UserScript methodsFor: 'script editor' stamp: 'sw 5/13/1998 15:38'!
bringUpToDate
	self allScriptVersionsDo: [:v | v bringUpToDate]! !

!UserScript methodsFor: 'script editor' stamp: 'sw 1/25/2001 10:32'!
currentScriptEditor
	"Answer the currentScriptEditor value"

	^ currentScriptEditor! !

!UserScript methodsFor: 'script editor' stamp: 'sw 1/29/98 22:55'!
currentScriptEditor: anEditor
	currentScriptEditor _ anEditor! !

!UserScript methodsFor: 'script editor' stamp: 'sw 1/25/2001 10:32'!
formerScriptEditors
	"Answer the formerScriptEditors value"

	^ formerScriptEditors! !

!UserScript methodsFor: 'script editor' stamp: 'sw 1/23/2001 11:28'!
instantiatedScriptEditor
	"Return the current script editor, creating it if necessary"

	self isTextuallyCoded ifTrue:
			[^ (player costume pasteUpMorph ifNil: [player costume "the world, backstop"]) scriptorForTextualScript: selector ofPlayer: player].

	currentScriptEditor ifNil:
		[currentScriptEditor _ (player class includesSelector: selector) 
			ifTrue: [ScriptEditorMorph new 
				fromExistingMethod: selector 
				forPlayer: player]
			ifFalse: [ScriptEditorMorph new
				setMorph: player costume
				scriptName: selector].
		status == #ticking ifTrue: [player costume arrangeToStartStepping]].
	
	^ currentScriptEditor! !

!UserScript methodsFor: 'textually coded' stamp: 'sw 5/19/1998 09:55'!
isTextuallyCoded
	^ currentScriptEditor == #textuallyCoded! !

!UserScript methodsFor: 'versions' stamp: 'di 2/19/2001 10:05'!
recreateScriptFrom: anEditor
	"Used to revert to old tiles"

	formerScriptEditors isEmptyOrNil ifTrue: [^ self].
	self revertScriptVersionFrom: anEditor installing: formerScriptEditors last! !

!UserScript methodsFor: 'script editor' stamp: 'tk 11/30/2000 16:32'!
releaseCachedState
	"release all non-showing scriptors.  What do we do about versions????"

	self isTextuallyCoded ifTrue: [formerScriptEditors _ OrderedCollection new].
		"to test new tiles.  We 'commit' to current script."
	currentScriptEditor ifNil: [^ self].
	true ifTrue: [^ self].	"<<< to test the reconstruction of scripts, change to false"
	currentScriptEditor world ifNil: ["not showing"
		currentScriptEditor _ nil].

! !

!UserScript methodsFor: 'versions' stamp: 'md 10/22/2003 16:07'!
revertScriptVersionFrom: anEditor 
	"Let user choose which prior tile version to revert to, and revert to it"

	| aMenu result |
	formerScriptEditors isEmptyOrNil ifTrue: [^Beeper beep].
	result := formerScriptEditors size == 1 
		ifTrue: [formerScriptEditors first]
		ifFalse: 
			[aMenu := SelectionMenu 
						labelList: (formerScriptEditors collect: [:e | e timeStamp])
						selections: formerScriptEditors.
			aMenu startUp].
	result 
		ifNotNil: [self revertScriptVersionFrom: anEditor installing: result]! !

!UserScript methodsFor: 'versions' stamp: 'sw 10/30/2000 08:54'!
revertScriptVersionFrom: anEditor installing: aSavedEditor
	"Replace anEditor with a brought-up-to-date version of aSavedEditor"

	| aPosition oldOwner |
	aPosition _ anEditor position.
	oldOwner _ anEditor topRendererOrSelf owner.
	anEditor delete.
	currentScriptEditor _ aSavedEditor bringUpToDate install.
	player costume viewAfreshIn: oldOwner showingScript: selector at: aPosition! !

!UserScript methodsFor: 'versions' stamp: 'sw 10/10/2000 11:07'!
revertToLastSavedTileVersionFor: anEditor
	"revert to the last saved tile version"

	formerScriptEditors isEmptyOrNil ifFalse:
		[self revertScriptVersionFrom: anEditor installing: formerScriptEditors last]! !

!UserScript methodsFor: 'versions' stamp: 'dgd 2/21/2003 22:39'!
saveScriptVersion: timeStampCurrentlyIgnored 
	self isTextuallyCoded 
		ifFalse: 
			[formerScriptEditors isNil 
				ifTrue: [formerScriptEditors := OrderedCollection new].
			currentScriptEditor 
				ifNotNil: [formerScriptEditors add: currentScriptEditor veryDeepCopy].
			formerScriptEditors size > 100 
				ifTrue: [^self error: 'apparent runaway versions']]! !

!UserScript methodsFor: 'versions' stamp: 'sw 10/30/2000 08:54'!
savedTileVersionsCount
	"Answer the number of saved tile versions of the script"

	^ formerScriptEditors ifNil: [0] ifNotNil: [formerScriptEditors size]! !

!UserScript methodsFor: 'updating' stamp: 'sw 1/22/2001 16:45'!
updateToPlayer: aPlayer
	"Set aPlayer as the current player referenced by the receiver and its script editor"

	(currentScriptEditor notNil and: [currentScriptEditor ~~ #textuallyCoded]) ifTrue:
		[currentScriptEditor updateToPlayer: aPlayer].
	player _ aPlayer! !

Object subclass: #SlotInformation
	instanceVariableNames: 'type documentation floatPrecision variableDock variableDefinition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!SlotInformation commentStamp: '<historical>' prior: 0!
Holds information about user-defined instance variables in Players.!

!SlotInformation methodsFor: 'access' stamp: 'yo 8/1/2004 02:05'!
documentation
	"Answer the receiver's documentation"

	documentation ifNil: [documentation _ 'This is a variable defined by you.  Please edit this into your own meaningful documentation.' translated].
	^ documentation! !

!SlotInformation methodsFor: 'access' stamp: 'sw 11/6/1998 10:33'!
documentation: d
	documentation _ d! !

!SlotInformation methodsFor: 'access' stamp: 'sw 7/27/2001 18:19'!
floatPrecision
	"Answer the floatPrecision for the slot:
		1.0 ->	show whole number
		0.1	->	show one digit of precision
		.01 ->	show two digits of precision
		etc.
	Initialize the precision to 1 if it is not present"

	^ floatPrecision isNumber ifTrue: [floatPrecision] ifFalse: [floatPrecision _ 1]! !

!SlotInformation methodsFor: 'access' stamp: 'sw 9/8/1999 18:31'!
floatPrecision: prec
	floatPrecision _ prec! !

!SlotInformation methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:44'!
initialize
	"Initialize the receiver's instance variables to default values"

	documentation _ 'as yet undocumented'.
	type _ #Number.
	floatPrecision _ 0.1.! !

!SlotInformation methodsFor: 'printing' stamp: 'sw 9/8/1999 22:11'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' precision: ', floatPrecision asString, ' ; type = ', type asString! !

!SlotInformation methodsFor: 'access' stamp: 'sw 4/22/2002 15:10'!
type
	"Answer the type of the receiver, initializing it to Number if it is nil"

	type isEmptyOrNil ifTrue: [^ type _ #Number].
	type first isUppercase ifFalse: [^ type _ type capitalized].
		"because of lingering, annoying issue of projects created in a plug-in image"
	^ type! !

!SlotInformation methodsFor: 'access' stamp: 'sw 11/6/1998 10:31'!
type: aType
	type _ aType! !

!SlotInformation methodsFor: 'access' stamp: 'sw 10/30/2000 10:15'!
variableDock
	"Answer the variable dock associated with the receiver, or nil if none"

	^ variableDock! !

!SlotInformation methodsFor: 'access' stamp: 'sw 10/30/2000 10:16'!
variableDock: vd
	"Set the receiver's variableDock as indicated"

	variableDock _ vd! !

Object subclass: #VariableDock
	instanceVariableNames: 'variableName type definingMorph morphGetSelector morphPutSelector playerGetSelector playerPutSelector defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Stacks'!

!VariableDock commentStamp: '<historical>' prior: 0!
Represents a variable held in a Player on behalf of a Morph.  When a new Player gets installed in the Morph, variables representing the old player need to be committed to the old player's storage, if not already done, and then new values for the variables need to be obtained from the new Player.  The VariableDock does the actual data transfer.

variableName 		A Symbol.  The name by which this variable known in the bearer,  a Card
type 				An object representing the variable's type.  Initially, we, like the rest
						use a Symbol to represent this.
						Presently #string #number #boolean #object #reference #sound etc.
definingMorph		The morph that requested storage of this variable
morphGetSelector	The message to be sent to the morph to obtain the variable's value
morphPutSelector	The message to be sent to the morph to put a new var value into it
owningClass			The Uniclass of which this is an instance variable
playerGetSelector 	The message to be sent to the Player to obtain its current stored value
playerPutSelector 	The message to be sent to the Player to set a new stored value
defaultValue		The value to set for the variable by default
floatPrecision		e.g. 0, 0.1, 0.001.  Only relevant for numeric-type variables
!

!VariableDock methodsFor: 'getters and setters' stamp: 'sw 10/30/2000 10:55'!
computePlayerGetterAndSetterSelectors
	"Compute and remember the getter and setter selectors for obtaining and setting values from the player instance"

	playerGetSelector _ Utilities getterSelectorFor: variableName.
	playerPutSelector _ Utilities setterSelectorFor: variableName! !

!VariableDock methodsFor: 'accessing' stamp: 'sw 11/2/2002 18:09'!
definingMorph
	"Answer the defining morph"

	^ definingMorph! !

!VariableDock methodsFor: 'initialization' stamp: 'sw 10/26/2000 13:33'!
dockMorphUpToInstance: anInstance
	"Dock my defining morph up to the given player instance.  NB: The odious #cardInstance mechanism used here was a last-minute stopgap for some demo, which surely should not be allowed to survive."

	definingMorph setProperty: #cardInstance toValue: anInstance.
	definingMorph perform: morphPutSelector with: (anInstance perform: playerGetSelector)! !

!VariableDock methodsFor: 'accessing' stamp: 'tk 5/25/2001 17:43'!
playerGetSelector
	^playerGetSelector! !

!VariableDock methodsFor: 'printing' stamp: 'sw 10/30/2000 08:50'!
printOn: aStream
	"Print a description of the receiver onto the given stream"

	super printOn: aStream.
	variableName ifNotNil: [aStream nextPutAll: (' (var name = ', variableName, ')')].
	type ifNotNil: [aStream nextPutAll: (' (type = ', type, ')')]! !

!VariableDock methodsFor: 'initialization' stamp: 'tk 5/30/2001 12:10'!
storeMorphDataInInstance: anInstance
	"Store the morph instance data represented by the receiver into the card instance provided.  This is done by retrieving the datum value from the morph that holds it on the card, and putting it into the card instance"

	anInstance perform: playerPutSelector with: (definingMorph perform: morphGetSelector) copy! !

!VariableDock methodsFor: 'accessing' stamp: 'tk 5/25/2001 17:43'!
type
	^type! !

!VariableDock methodsFor: 'name' stamp: 'sw 10/30/2000 08:52'!
variableName
	"Answer the variable name represented by the receiver"

	^ variableName! !

!VariableDock methodsFor: 'name' stamp: 'sw 10/30/2000 08:51'!
variableName: aSymbol
	"Set the receiver's variableName as indicated, and recompute corresponding getters and setters"

	variableName _ aSymbol asSymbol.
	self computePlayerGetterAndSetterSelectors! !

!VariableDock methodsFor: 'initialization' stamp: 'sw 10/30/2000 08:52'!
variableName: aSymbol type: aType definingMorph: aMorph morphGetSelector: getterSymbol morphPutSelector: putterSymbol
	"Initialize the receiver as indicated"

	variableName _ aSymbol asSymbol.
	type _ aType.
	definingMorph _ aMorph.
	morphGetSelector _ getterSymbol.
	morphPutSelector _ putterSymbol.
	self computePlayerGetterAndSetterSelectors! !

!VariableDock methodsFor: 'type and default value' stamp: 'sw 10/29/2000 17:40'!
variableType
	"Answer the data type of the receiver"

	^ type! !

TwoWayScrollPane subclass: #PluggableTileScriptorMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Tile Scriptors'!

!PluggableTileScriptorMorph methodsFor: 'event handling' stamp: 'tk 9/23/2001 02:28'!
keyStroke: evt
	"A keystroke was hit while the receiver had keyboard focus.  Pass the keystroke on to my syntaxMorph, and also, if I have an event handler, pass it on to that handler"


	| sm |
	(sm _ self syntaxMorph) ifNotNil: [sm keyStroke: evt].
	super keyStroke: evt! !

!PluggableTileScriptorMorph methodsFor: 'as yet unclassified' stamp: 'tk 9/23/2001 02:27'!
syntaxMorph
	"Return the SyntaxMorph(MethodNode) that is inside me."

	| tm |
	^ (tm _ self findA: TransformMorph) ifNotNil: [tm findA: SyntaxMorph]! !

!PluggableTileScriptorMorph methodsFor: 'updating' stamp: 'tk 9/14/2001 18:16'!
update: aSymbol
	"Update the receiver in the manner suggested by aSymbol"

	aSymbol == #flash ifTrue: [^ self flash].
	(aSymbol == #contents or: [aSymbol == #tiles])
		ifTrue: [^ self containingWindow model installTilesForSelection]! !

ThumbnailMorph subclass: #PlayerReferenceReadout
	instanceVariableNames: 'putSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting Support'!

!PlayerReferenceReadout commentStamp: '<historical>' prior: 0!
A thumbnail that serves as the value readout for a player-valued slot in a Viewer.  Clicking on it allows the user to select a new object for the slot to point to. !

!PlayerReferenceReadout methodsFor: 'event handling' stamp: 'sw 4/26/1998 02:02'!
handlesMouseDown: evt
	"Allow the user to respecify this by direct clicking"
	^ true! !

!PlayerReferenceReadout methodsFor: 'accessing' stamp: 'sw 1/6/2005 01:24'!
isEtoyReadout
	"Answer whether the receiver can serve as an etoy readout"

	^ true! !

!PlayerReferenceReadout methodsFor: 'event handling' stamp: 'dgd 2/21/2003 22:58'!
mouseDown: evt 
	"Allow the user to respecify this by direct clicking"

	| aMorph |
	(putSelector == #unused or: [putSelector isNil]) ifTrue: [^self].
	Sensor waitNoButton.
	aMorph := self world chooseClickTarget.
	aMorph ifNil: [^self].
	objectToView perform: putSelector with: aMorph assuredPlayer.
	self changed! !

!PlayerReferenceReadout methodsFor: 'initialization' stamp: 'sw 4/26/1998 02:07'!
objectToView: objectOrNil viewSelector: aSelector putSelector: aPutSelector
	self objectToView: objectOrNil viewSelector: aSelector.
	putSelector _ aPutSelector! !

!PlayerReferenceReadout methodsFor: 'initialization' stamp: 'sw 1/6/2005 17:12'!
putSelector
	"Answer the putSelector"

	^ putSelector! !

!PlayerReferenceReadout methodsFor: 'initialization' stamp: 'sw 1/6/2005 17:13'!
putSelector: aSel
	"Reset the putSelector"

	self objectToView: objectToView viewSelector: viewSelector putSelector: aSel! !

!PlayerReferenceReadout methodsFor: 'accessing' stamp: 'sw 1/6/2005 01:38'!
target
	"Answer the object on which I act"

	^ objectToView! !

ThumbnailMorph subclass: #ThumbnailForAllPlayersTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!ThumbnailForAllPlayersTool commentStamp: '<historical>' prior: 0!
A thumbnail for use in an All-Players tool!

!ThumbnailForAllPlayersTool methodsFor: 'stepping' stamp: 'sw 7/28/2004 22:23'!
step
	"periodic action"

	| aMorph |
	((aMorph _ objectToView costume) notNil and: [aMorph isInWorld]) ifTrue:
		[super step]  "don't bother changing my readout to blank when/if object disappears"! !

!ThumbnailForAllPlayersTool methodsFor: 'stepping' stamp: 'sw 7/28/2004 22:19'!
stepTime
	"Don't update too aggressively"

	^ 2500! !

!ThumbnailMorph methodsFor: '*eToys-scripting' stamp: 'nk 8/29/2004 17:18'!
tearOffTile
	(objectToView isPlayerLike) ifTrue: [^ objectToView tearOffTileForSelf].

	objectToView ifNil: [^ nil].
	^ objectToView isMorph
		ifTrue:
			[objectToView]
		ifFalse:
			[objectToView costume]
! !

!CascadeNode methodsFor: '*eToys-tiles' stamp: 'RAA 2/22/2001 13:56'!
asMorphicSyntaxIn: parent

	^parent
		cascadeNode: self 
		receiver: receiver 
		messages: messages
! !

!GraphMorph class methodsFor: '*eToys-scripting' stamp: 'sw 9/26/2001 04:19'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(

	(basic (
(slot cursor 	'The current cursor location, wrapped back to the beginning if appropriate' Number	 readWrite Player getCursor Player setCursorWrapped:)

(slot sampleAtCursor	'The sample value at the current cursor location' Number readWrite Player getSampleAtCursor Player setSampleAtCursor:)))

	(sampling (
(slot cursor 	'The current cursor location, wrapped back to the beginning if appropriate' Number	 readWrite Player getCursor Player setCursorWrapped:)
(slot sampleAtCursor	'The sample value at the current cursor location' Number readWrite Player getSampleAtCursor Player setSampleAtCursor:)
(slot lastValue 'The last value obtained' Number readWrite	Player getLastValue Player setLastValue:)
(command clear 'Clear the graph of current contents')
(command loadSineWave 'Load a sine wave as the current graph')
(command loadSound: 'Load the specified sound into the current graph' Sound)
(command reverse 'Reverse the graph')
(command play 'Play the current graph as a sound'))))! !

!HandleMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 6/30/1999 20:40'!
isCandidateForAutomaticViewing
	^ false! !

!MenuMorph methodsFor: '*eToys-control' stamp: 'sw 6/30/1999 20:30'!
isCandidateForAutomaticViewing
	^ false! !

!ReferenceMorph classSide methodsFor: '*eToys-scripting' stamp: 'sw 9/26/2001 04:04'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((paintbox ((command makeNewDrawingIn: 'make a new drawing in the specified playfield' Player))))



! !

!LassoPatchMorph methodsFor: '*eToys-misc' stamp: 'sw 7/5/2004 01:50'!
isCandidateForAutomaticViewing
	"Answer whether the receiver is a candidate for automatic viewing.  Only relevant if a now-seldom-used feature, automaticViewing, is in play"

	^ self isPartsDonor not! !

!ParseNode methodsFor: '*eToys-tiles' stamp: 'ab 7/13/2004 13:47'!
addCommentToMorph: aMorph
	| row |
	(self comment isNil or: [self comment isEmpty]) ifTrue: [^ self].
	row _ aMorph addTextRow:
		(String streamContents: [:strm | self printCommentOn: strm indent: 1]).
	row firstSubmorph color: (SyntaxMorph translateColor: #comment).
	row parseNode: (self as: CommentNode).
! !

!ParseNode methodsFor: '*eToys-tiles' stamp: 'RAA 8/24/1999 12:24'!
asMorphicSyntaxIn: parent

	| morph |
	"Default for missing implementations"

	morph _ parent addColumn: #error on: self.
	morph addTextRow: self class printString.
	^morph
	

! !

!ParseNode methodsFor: '*eToys-tiles' stamp: 'RAA 8/24/1999 13:06'!
currentValueIn: aContext

	^nil! !

!ParseNode methodsFor: '*eToys-tiles' stamp: 'RAA 8/24/1999 18:18'!
explanation

	^self class printString! !

!EventHandler methodsFor: '*eToys-initialization' stamp: 'jcg 9/21/2001 12:57'!
forgetDispatchesTo: aSelector
	"aSelector is no longer implemented by my corresponding Player, so don't call it any more"
	mouseDownSelector == aSelector
		ifTrue: [mouseDownRecipient _ mouseDownSelector _ nil].
	mouseMoveSelector == aSelector
		ifTrue: [mouseMoveRecipient _ mouseMoveSelector _ nil].
	mouseStillDownSelector == aSelector
		ifTrue: [mouseStillDownRecipient _ mouseStillDownSelector _ nil].
	mouseUpSelector == aSelector
		ifTrue: [mouseUpRecipient _ mouseUpSelector _ nil].
	mouseEnterSelector == aSelector
		ifTrue: [mouseEnterRecipient _ mouseEnterSelector _ nil].
	mouseLeaveSelector == aSelector
		ifTrue: [mouseLeaveRecipient _ mouseLeaveSelector _ nil].
	mouseEnterDraggingSelector == aSelector
		ifTrue: [mouseEnterDraggingRecipient _ mouseEnterDraggingSelector _ nil].
	mouseLeaveDraggingSelector == aSelector
		ifTrue: [mouseLeaveDraggingRecipient _ mouseLeaveDraggingSelector _ nil].
	clickSelector == aSelector
		ifTrue: [clickRecipient _ clickSelector _ nil].
	doubleClickSelector == aSelector
		ifTrue: [doubleClickRecipient _ doubleClickSelector _ nil].
	doubleClickTimeoutSelector == aSelector
		ifTrue: [doubleClickTimeoutRecipient _ doubleClickTimeoutSelector _ nil].
	keyStrokeSelector == aSelector
		ifTrue: [keyStrokeRecipient _ keyStrokeSelector _ nil].! !

!BlockNode methodsFor: '*eToys-tiles' stamp: 'RAA 2/27/2001 09:48'!
asMorphicCollectSyntaxIn: parent

	^parent 
		blockNodeCollect: self 
		arguments: arguments 
		statements: statements! !

!BlockNode methodsFor: '*eToys-tiles' stamp: 'RAA 2/16/2001 09:08'!
asMorphicSyntaxIn: parent

	^parent 
		blockNode: self 
		arguments: arguments 
		statements: statements! !

!Form methodsFor: '*eToys-testing' stamp: 'RAA 1/19/2001 15:04'!
appearsToBeSameCostumeAs: anotherForm

	(anotherForm isKindOf: self class) ifFalse: [^false].
	anotherForm depth = self depth ifFalse: [^false].
	^anotherForm bits = bits
! !

!CollapsedMorph class methodsFor: '*eToys-as yet unclassified' stamp: 'sw 4/9/2001 14:19'!
collapsedMorphOrNilFor: anActualMorph
	"If there is any instance of the receiver that represents anActualMorph, answer it, else answer nil"

	self allInstances do:
		[:cm | (cm isMyUncollapsedMorph: anActualMorph)
			ifTrue:	[^ cm]].
	^ nil! !

!CollapsedMorph methodsFor: '*eToys-queries' stamp: 'sw 4/9/2001 12:53'!
isMyUncollapsedMorph: aMorph
	"Answer whether my uncollapsed morph is aMorph"

	^ uncollapsedMorph == aMorph! !

!SoundType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:30'!
defaultArgumentTile
	"Answer a tile to represent the type"

	^ SoundTile new typeColor: self typeColor! !

!SoundType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:37'!
newReadoutTile
	"Answer a tile that can serve as a readout for data of this type"

	^ SoundReadoutTile new typeColor: Color lightGray lighter! !

!SoundType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:33'!
setFormatForDisplayer: aDisplayer
	"Set up the displayer to have the right format characteristics"

	aDisplayer useStringFormat
	! !

!SoundType methodsFor: '*eToys-color' stamp: 'sw 9/27/2001 17:21'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(1.0 0.06 0.84)	! !

!PasteUpMorph class methodsFor: '*eToys-scripting' stamp: 'nk 10/13/2004 11:39'!
additionsToViewerCategoryInput
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(input (
			(slot lastKeystroke 'The last unhandled keystroke' String readWrite Player getLastKeystroke Player setLastKeystroke:)
	))! !

!PasteUpMorph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/26/2003 23:24'!
addUserCustomEventNamed: aSymbol help: helpString
	self userCustomEventsRegistry at: aSymbol put: helpString.
! !

!PasteUpMorph methodsFor: '*eToys-e-toy support' stamp: 'nk 10/13/2004 11:26'!
lastKeystroke
	"Answer the last keystroke fielded by the receiver"

	^ self valueOfProperty: #lastKeystroke ifAbsent: ['']! !

!PasteUpMorph methodsFor: '*eToys-e-toy support' stamp: 'nk 10/13/2004 11:27'!
lastKeystroke: aString
	"Remember the last keystroke fielded by the receiver"

	^ self setProperty: #lastKeystroke toValue: aString! !

!PasteUpMorph methodsFor: '*eToys-world menu' stamp: 'nk 10/14/2004 07:07'!
makeAllScriptEditorsReferToMasters
	"Ensure that all script editors refer to the first (by alphabetical externalName) Player among the list of siblings"

	(self presenter allExtantPlayers groupBy: [ :p | p class ] having: [ :p | true ])
		do: [ :group | group first allScriptEditors ]! !

!PasteUpMorph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/26/2003 23:26'!
removeUserCustomEventNamed: aSymbol
	^self userCustomEventsRegistry removeKey: aSymbol ifAbsent: [].! !

!PasteUpMorph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/26/2003 23:20'!
userCustomEventNames
	| reg |
	reg _ self valueOfProperty: #userCustomEventsRegistry ifAbsent: [ ^#() ].
	^reg keys asArray sort! !

!PasteUpMorph methodsFor: '*eToys-customevents-scripting' stamp: 'nk 9/26/2003 23:18'!
userCustomEventsRegistry
	^self valueOfProperty: #userCustomEventsRegistry ifAbsentPut: [ IdentityDictionary new ].! !

!Color methodsFor: '*eToys-other' stamp: 'sw 6/10/1998 17:50'!
newTileMorphRepresentative
	^ ColorTileMorph new colorSwatchColor: self! !

!ServerDirectory class methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 16:10'!
eToyBaseFolderSpecForFileDirectory: aFileDirectory
	^self localEToyBaseFolderSpecs at: aFileDirectory ifAbsent:[nil]! !

!ServerDirectory class methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 16:30'!
eToyBaseFolderSpecForFileDirectory: aFileDirectory put: aString
	^self localEToyBaseFolderSpecs at: aFileDirectory put: aString! !

!ServerDirectory class methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 16:38'!
eToyUserListForFileDirectory: aFileDirectory
	| urlString |
	urlString _ self eToyUserListUrlForFileDirectory: aFileDirectory.
	urlString ifNil:[^nil].
	^self parseEToyUserListFrom: urlString! !

!ServerDirectory class methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 15:46'!
eToyUserListUrlForFileDirectory: aFileDirectory
	^self localEToyUserListUrls at: aFileDirectory ifAbsent:[nil]! !

!ServerDirectory class methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 15:48'!
eToyUserListUrlForFileDirectory: aFileDirectory put: urlString
	^self localEToyUserListUrls at: aFileDirectory put: urlString! !

!ServerDirectory class methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 16:11'!
localEToyBaseFolderSpecs
	^LocalEToyBaseFolderSpecs ifNil:[LocalEToyBaseFolderSpecs _ IdentityDictionary new]! !

!ServerDirectory class methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 15:47'!
localEToyUserListUrls
	^LocalEToyUserListUrls ifNil:[LocalEToyUserListUrls _ IdentityDictionary new].! !

!ServerDirectory class methodsFor: '*eToys-school support' stamp: 'fbs 2/2/2005 13:24'!
parseEToyUserListFrom: urlString
	| url userString userList |
	urlString ifNil:[^nil].
	url _ urlString asUrl.
	["Note: We need to prevent going through the plugin API 
	when retrieving a local (file) URL, since the plugin API
	(correctly) rejects file:// downloads."
		Cursor wait showWhile:[
			(url hasRemoteContents) ifTrue:[
				"Go through the browser (if present)"
				userString _ (HTTPClient httpGet: url asString) contents.
			] ifFalse:[
				"Go grab it directly"
				userString _ url retrieveContents contents.
			].
		].
	] on: Error do:[:ex| userString _ nil. ex return].
	userString ifNil:[^nil].
	"Get rid of any line ending problems"
	userString _ userString copyReplaceAll: String crlf with: String cr.
	userString _ userString copyReplaceAll: String lf with: String cr.
	userList _ (userString findTokens: Character cr) collect:[:each| each withBlanksTrimmed].
	userList _ userList reject:[:any| any isEmpty].
	(userList first = '##user list##') ifFalse:[^nil].
	userList _ userList copyFrom: 2 to: userList size.
	^userList! !

!ServerDirectory methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 15:45'!
eToyUserList
	"Return a list of all known users for eToy login support"
	| urlString |
	eToyUserList ifNotNil:[^eToyUserList].
	urlString _ self eToyUserListUrl.
	urlString ifNil:[^nil].
	eToyUserList _ self class parseEToyUserListFrom: urlString.
	^eToyUserList! !

!ServerDirectory methodsFor: '*eToys-school support' stamp: 'ar 8/24/2001 14:31'!
eToyUserList: aCollectionOrNil
	"Set a list of all known users for eToy login support"
	eToyUserList _ aCollectionOrNil.! !

!ServerDirectory methodsFor: '*eToys-school support' stamp: 'ar 8/24/2001 14:29'!
eToyUserListUrl
	^eToyUserListUrl! !

!ServerDirectory methodsFor: '*eToys-school support' stamp: 'ar 8/24/2001 14:31'!
eToyUserListUrl: aString
	eToyUserListUrl _ aString.
	eToyUserList _ nil.! !

!ServerDirectory methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 16:05'!
eToyUserName: aString
	"Ignored here"! !

!ServerDirectory methodsFor: '*eToys-school support' stamp: 'ar 9/5/2001 16:12'!
hasEToyUserList
	^eToyUserListUrl notNil! !

!SimpleSliderMorph class methodsFor: '*eToys-scripting' stamp: 'nk 6/9/2003 06:05'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(
(slider (

(slot numericValue 'A number representing the current position of the knob.' Number readWrite Player getNumericValue Player setNumericValue:)

(slot minVal	 'The number represented when the knob is at the left or top of the slider; the smallest value returned by the slider.' Number readWrite	Player getMinVal Player setMinVal:)

(slot maxVal 'The number represented when the knob is at the right or bottom of the slider; the largest value returned by the slider.' Number readWrite	Player getMaxVal Player setMaxVal:)

(slot descending 'Tells whether the smallest value is at the top/left (descending = false) or at the bottom/right (descending = true)' Boolean readWrite Player getDescending Player setDescending:)

(slot truncate 'If true, only whole numbers are used as values; if false, fractional values are allowed.' Boolean readWrite	Player getTruncate Player setTruncate:)

(slot color 'The color of the slider' Color readWrite Player getColor  Player  setColor:)

(slot knobColor 'The color of the slider' Color readWrite Player getKnobColor Player setKnobColor:)
(slot  width  'The width' Number readWrite Player getWidth  Player  setWidth:)
(slot  height  'The height' Number readWrite Player getHeight  Player  setHeight:)))

(basic	(
(slot numericValue 'A number representing the current position of the knob.' Number readWrite Player getNumericValue Player setNumericValue:))))

! !

!SimpleSliderMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 7/4/2004 01:53'!
arrowDeltaFor: aGetter
	"Here just for testing the arrowDelta feature.  To test, re-enable the code below by commenting out the first line, and you should see the minVal slot of the slider go up and down in increments of 5 as you work the arrows on its readout tile in a freshly-launched Viewer or detailed watcher"
	
	true ifTrue: [^ super arrowDeltaFor: aGetter]. 
	
	^ (aGetter == #getMinVal)
		ifTrue:
			[5]
		ifFalse:
			[1]! !

!SimpleSliderMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 9/1/2000 10:38'!
getNumericValue
	"Answer the numeric value of the receiver"

	^ self getScaledValue! !

!SimpleSliderMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 9/1/2000 10:38'!
setNumericValue: aValue
	"Set the numeric value of the receiver to be as indicated"

	^ self setScaledValue: aValue! !

!PaintBoxMorph methodsFor: '*eToys-e-toy support' stamp: 'sw 6/30/1999 20:33'!
isCandidateForAutomaticViewing
	^ false! !

!AssignmentNode methodsFor: '*eToys-tiles' stamp: 'RAA 2/26/2001 16:17'!
asMorphicSyntaxIn: parent

	^parent assignmentNode: self variable: variable value: value! !

!AssignmentNode methodsFor: '*eToys-tiles' stamp: 'RAA 8/15/1999 16:31'!
explanation

	^'The value of ',value explanation,' is being stored in ',variable explanation
! !

UpdatingSimpleButtonMorph subclass: #ScriptableButton
	instanceVariableNames: 'scriptSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EToys-Scripting'!

!ScriptableButton commentStamp: '<historical>' prior: 0!
A button intended for use with the card architecture and the user-scripting system.!

!ScriptableButton classSide methodsFor: 'scripting' stamp: 'sw 9/26/2001 04:26'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #((button (

			(slot label 'The wording on the button' String readWrite Player getLabel Player setLabel:)
			(slot color 'The color of the object' Color readWrite Player getColor  Player  setColor:)
			(slot height  'The height' Number readWrite Player getHeight  Player  setHeight:) 
			(slot borderColor 'The color of the object''s border' Color readWrite Player getBorderColor Player  setBorderColor:)
			(slot borderWidth 'The width of the object''s border' Number readWrite Player getBorderWidth Player setBorderWidth:)
			(slot  height  'The height' Number readWrite Player getHeight  Player  setHeight:)
			(slot roundedCorners 'Whether corners should be rounded' Boolean readWrite Player getRoundedCorners Player setRoundedCorners:) 
			(slot actWhen 'When the script should fire' ButtonPhase  readWrite Player getActWhen Player setActWhen: ))))! !

!ScriptableButton classSide methodsFor: 'authoring prototype' stamp: 'dgd 9/6/2003 17:38'!
authoringPrototype
	"Answer a scriptable button that can serve as a prototype for a parts bin"

	^ super authoringPrototype
		borderWidth: 1;
		borderColor: Color black;
		useRoundedCorners;
		color: Color yellow;
		label: 'Press me' translated;
		setNameTo: ('script{1}' translated format: {'1'});
		yourself

"ScriptableButton authoringPrototype openInHand"! !

!ScriptableButton classSide methodsFor: 'printing' stamp: 'sw 10/30/2000 10:29'!
defaultNameStemForInstances
	"Answer the default name stem to use for instances of the receiver"

	^ 'button'! !

!ScriptableButton classSide methodsFor: 'name' stamp: 'nk 8/23/2004 18:12'!
descriptionForPartsBin
	^ self partName:	'Button'
		categories:		#('Scripting' 'Basic')
		documentation:	'A button to use with tile scripting; its script will be a method of its containing playfield'! !

!ScriptableButton classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 10:56'!
initialize

	self registerInFlapsRegistry.	! !

!ScriptableButton classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 10:58'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: #(ScriptableButton		authoringPrototype	'Button' 		'A Scriptable button')
						forFlapNamed: 'PlugIn Supplies'.
						cl registerQuad: #(ScriptableButton		authoringPrototype	'Button' 		'A Scriptable button')
						forFlapNamed: 'Scripting'.
						cl registerQuad: #(ScriptableButton		authoringPrototype		'Scriptable Button'	'A button whose script will be a method of the background Player')
						forFlapNamed: 'Stack Tools'.
						cl registerQuad: #(ScriptableButton		authoringPrototype	'Button' 		'A Scriptable button')
						forFlapNamed: 'Supplies'.]! !

!ScriptableButton classSide methodsFor: 'class initialization' stamp: 'asm 4/11/2003 12:40'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!ScriptableButton methodsFor: 'thumbnail' stamp: 'sw 10/30/2000 16:07'!
demandsThumbnailing
	"Answer whether the receiver, if in a thumbnailable parts bin, wants to be thumbnailed whether or not size requires it.  This is set to true here because the recent event rework somehow made it possible for a scriptable button to be draggable from a parts bin otherwise, maddeningly"

	^ true! !

!ScriptableButton methodsFor: 'button' stamp: 'bf 9/20/2004 11:10'!
doButtonAction
	"The user has pressed the button.  Dispatch to the actual user script, if any."

	scriptSelector ifNil: [^ super doButtonAction].
	self pasteUpMorph player performScriptIfCan: scriptSelector! !

!ScriptableButton methodsFor: 'script' stamp: 'yo 2/10/2005 20:09'!
editButtonsScript
	"The user has touched my Scriptor halo-handle.  Bring up a Scriptor on the script of the button."

	| cardsPasteUp cardsPlayer anEditor |
	cardsPasteUp := self pasteUpMorph.
	(cardsPlayer := cardsPasteUp assuredPlayer) assureUniClass.
	anEditor := scriptSelector ifNil: 
					[scriptSelector := cardsPasteUp scriptSelectorToTriggerFor: self.
					cardsPlayer newTextualScriptorFor: scriptSelector.
					cardsPlayer scriptEditorFor: scriptSelector
					]
				ifNotNil: 
					[(cardsPlayer class selectors includes: scriptSelector) 
						ifTrue: [cardsPlayer scriptEditorFor: scriptSelector]
						ifFalse: 
							["Method somehow got removed; I guess we start afresh"

							scriptSelector := nil.
							^self editButtonsScript]].
	anEditor showingMethodPane ifTrue: [anEditor toggleWhetherShowingTiles].
	self currentHand attachMorph: anEditor! !

!ScriptableButton methodsFor: 'miscellaneous' stamp: 'dgd 9/6/2003 17:37'!
initializeToStandAlone
	super initializeToStandAlone.
	self borderWidth: 1;
		borderColor: Color black;
		useRoundedCorners;
		color: Color yellow;
		label: 'Press me' translated! !

!ScriptableButton methodsFor: 'script' stamp: 'yo 2/10/2005 20:04'!
isLikelyRecipientForMouseOverHalos

	self player ifNil: [^ false].
	self player getHeading = 0.0 ifTrue: [^ false].
	^ true.
! !

!ScriptableButton methodsFor: 'accessing' stamp: 'sw 10/30/2000 11:02'!
label
	"Answer a string representing the label of the receiver, returning an empty string if necessary"

	| aStringMorph |
	^ (aStringMorph _ self findA: StringMorph)
		ifNil:		['']
		ifNotNil:	[aStringMorph contents]! !

!ScriptableButton methodsFor: 'accessing' stamp: 'sw 10/30/2000 10:59'!
label: aString
	"Set the receiver's label as indicated"

	| aLabel |
	(aLabel _ self findA: StringMorph)
		ifNotNil:
			[aLabel contents: aString]
		ifNil:
			[aLabel _ StringMorph contents: aString font: TextStyle defaultFont.
			self addMorph: aLabel].

	self extent: aLabel extent + (borderWidth + 6).
	aLabel position: self center - (aLabel extent // 2).

	aLabel lock! !

!ScriptableButton methodsFor: 'label' stamp: 'yo 11/4/2002 22:13'!
label: aString font: aFontOrNil
	"Set the receiver's label and font as indicated"

	| oldLabel m aFont |
	(oldLabel _ self findA: StringMorph)
		ifNotNil: [oldLabel delete].
	aFont _ aFontOrNil ifNil: [TextStyle defaultFont].
	m _ StringMorph contents: aString font: aFont.
	self extent: (m width + 6) @ (m height + 6).
	m position: self center - (m extent // 2).
	self addMorph: m.
	m lock
! !

!ScriptableButton methodsFor: 'accessing' stamp: 'tk 9/28/2001 21:09'!
scriptSelector
	^ scriptSelector! !

!ScriptableButton methodsFor: 'accessing' stamp: 'tk 9/28/2001 21:09'!
scriptSelector: aSymbol
	scriptSelector _ aSymbol! !

!ScriptableButton methodsFor: 'menu' stamp: 'sw 10/30/2000 11:02'!
setLabel
	"Invoked from a menu, let the user change the label of the button"

	| newLabel |
	newLabel _ FillInTheBlank
		request:
'Enter a new label for this button'
		initialAnswer: self label.
	newLabel isEmpty ifFalse: [self label: newLabel font: nil].
! !

!ScriptableButton methodsFor: 'halos and balloon help' stamp: 'sw 10/10/2000 08:09'!
wantsScriptorHaloHandle
	"Answer whether the receiver would like to have a Scriptor halo handle put up on its behalf.  Initially, only the ScriptableButton says yes"

	^ true! !

!TempVariableNode methodsFor: '*eToys-tiles' stamp: 'ab 7/13/2004 13:54'!
asMorphicSyntaxIn: parent

	^ parent addToken: self name type: #tempVariable on: self! !

!TempVariableNode methodsFor: '*eToys-tiles' stamp: 'ab 7/13/2004 13:54'!
explanation

	^(self isArg ifTrue: ['Method argument'] ifFalse: ['Temporary variable']),' <',self name,'>'
! !

!ColorType methodsFor: '*eToys-tiles' stamp: 'sw 9/27/2001 17:30'!
defaultArgumentTile
	"Answer a tile to represent the type"

	^ Color blue newTileMorphRepresentative! !

!ColorType methodsFor: '*eToys-color' stamp: 'sw 9/27/2001 17:21'!
typeColor
	"Answer the color for tiles to be associated with objects of this type"

	^ self subduedColorFromTriplet: #(1.0  0 0.065)	! !
AllScriptsTool initialize!
KedamaMorph initialize!
KedamaPatchMorph initialize!
RandomNumberTile initialize!
ScriptableButton initialize!
StackMorph initialize!
TileMorph initialize!
UserCustomEventNameType initialize!
